

/************************************************************************
 * FILE NAME   : GpDefine.h                                             *
 *			                                                *
 * DESCRIPTION : This header file defines most constants and structures *
 *				 for the PCI library.			*
 *									*
 * (C) Copyright 2001 by GlobeSpan Inc.					*
 *                                                                      *
 * $Header: /usr/local/cvsroot/wanpipe_common/adsl/Epci/GpDefine.h,v 1.4 2003/10/29 18:22:58 sangoma Exp $ *
 *                                                                      *
 ************************************************************************/
#ifndef GP_DEFINE_H
#define GP_DEFINE_H

#include "GpDataTypes.h"

#define	GP_PCI_LIBRARY_VER				0.07E


/* Call back related macro definitions */
#define GpReceiveCells(a,b,c)	(pDslModem->ReceiveCells)(a,b,c)
#define GpDelay(a)       	(pDslModem->Delay)(a)
#define GpXmitDmaDone(a)	(pDslModem->XmitDmaDone)(a)

#define GpReadPciReg(a,b,c)	(pDslModem->ReadPciReg)(a,b,c)
#define GpReadPciRegB(a,b,c)	(pDslModem->ReadPciRegB)(a,b,c)
#define GpWritePciReg(a,b,c)	(pDslModem->WritePciReg)(a,b,c)
#define GpWritePciRegB(a,b,c)	(pDslModem->WritePciRegB)(a,b,c)

#define GSI_NUM_OF_XCVR		1
#define	EXECUTE_CMD		_GTI_XCVR_INFO
#define GTISysBlock		GTIXcvrBlock
#define GSI_XCVR_1		0

#define	GP_XCVR_VENDOR_ID	0x39
#define	GP_EOC_VENDOR_ID	0x39

#define	GP_TRUE			1
#define	GP_FALSE		0

#define GP_SUCCESS		1
#define GP_FAIL			0
#define GP_PENDING		2

/* Return status from GpMonitorTransceiver */
#define GP_NO_CHANGE		0		/* No change in status from previous time */
#define GP_LINK_DOWN		1		/* Link just went down (link state change from GP_TRAINED to GP_TRAINING) */
#define GP_LINK_UP		2		/* Link just went up (link state change from GP_TRAINING to GP_TRAINED)*/

#define GP_LINK_DISCONNECTING				3

/* GP_CELL_DEL_TIMEOUT how long (in msec.) the modem can be stuck
// in the pending cell deliniation state before a retain will be initiated.
*/
#define	GP_CELL_DEL_TIMEOUT				(10000)		/* Pending Timeout (10 sec) */

/* GP_TRAINING_TIMEOUT how long (in msec.) the modem can be stuck
// in the training state before a reset will be initiated.
*/
#define	GP_TRAINING_TIMEOUT				(150000)	/* Training timeout (150 sec. = 2.5 min) */

/* GP_TX_DMA_TIMEOUT how long (in msec.) the modem can be stuck
// in the transmit pending state before a TX DMA reset will be initiated.
*/
#define	GP_TX_DMA_TIMEOUT				(5000)		/* Transmit DMA timeout interval (5 sec.) */

typedef enum   {GP_NOT_TRAINED,
				GP_TRAINING,
				GP_TRAINED,
				GP_SPECTRUM,
				GP_BUSY,
				GP_PENDING_CELLDEL,
				GP_DGLOOPBACK,
				GP_RESTART,
				GP_NO_SIGNAL,
				GP_DISCONNECT,
				GP_LOCAL_LB} DSL_LINK_STATES;

/* Initialization Status - InitStatus */
#define GP_INIT_CLEAR				0x00000000
#define GP_INIT_HARDWARE			0x00000001
#define GP_INIT_SOFTSAR				0x00000002
#define GP_INIT_SOFTSAR_ALLOC		0x00000004
#define GP_INIT_TIMERS				0x00000008
#define GP_INIT_DYING_GASP			0x00000010
#define GP_INIT_PM_POWERDOWN		0x00000020
#define GP_INIT_ENABLED_INT			0x00000040
#define GP_INIT_COMPLETE			0x00008000
#define GP_INIT_SHUTDOWN			0x00010000
#define GP_INIT_HALT				0x00020000

/*
** Nenad added this option to kickstart the TxDMA
** by enabling Rx of idle cells
*/
#define GP_INIT_RX_IDLE				0x80000000

#define GP_NUM_DYING_GASP_MESSAGES		6
#define GP_DYING_GASP_EOC_MESSAGE		(0xE7);
#define GP_EOC_DYING_GASP_DELAY		(100*1000) /* 100msec = 100e+3usec */

/* EEPROM Signal Definitions
// EEPROM control register bits
*/
#define GP_EEDO                        0x0001  /* EEPROM Data out - Assume Output Pin 16 (1<<0) */
#define GP_EEDI                        0x0002  /* EEPROM Data in (set for writing data) - Assume Output Pin 17 (1<<1) */
#define GP_EECS                        0x0004  /* EEPROM chip select (1=low,1=high) - Assume Output Pin 18 (1<<2) */
#define GP_EESK                        0x0008  /* EEPROM shift clock (1=high,0=low) - Assume Output Pin 19 (1<<3) */

/* EEPROM opcodes */
#define GP_EEPROM_WRITE_OPCODE         02
#define GP_EEPROM_READ_OPCODE          03
#define GP_EEPROM_WREN_OPCODE          06

/*
// EEPROM data locations - As per EEPROM Map 10/13/00
*/
#define GP_EEPROM_FUNCTION_MODE_ADDRESS            0x01
#define GP_EEPROM_SUBSYS_ID_ADDR_HIGH_BYTE         0x08
#define GP_EEPROM_SUBSYS_ID_ADDR_LOW_BYTE          0x09
#define GP_EEPROM_SUBSYS_VENDOR_ID_ADDR_HIGH_BYTE  0x0a
#define GP_EEPROM_SUBSYS_VENDOR_ID_ADDR_LOW_BYTE   0x0b

/*
// MAC Address Information - As per EEPROM Map 10/13/00
*/
#define GP_EEPROM_MAC_ADDR_5                     	GSI_MAC_EEPROM_OFFSET
#define GP_EEPROM_MAC_ADDR_4                       (EEPROM_MAC_ADDR_5 + 1)
#define GP_EEPROM_MAC_ADDR_3                       (EEPROM_MAC_ADDR_4 + 1)
#define GP_EEPROM_MAC_ADDR_2                       (EEPROM_MAC_ADDR_3 + 1)
#define GP_EEPROM_MAC_ADDR_1                       (EEPROM_MAC_ADDR_2 + 1)
#define GP_EEPROM_MAC_ADDR_0                       (EEPROM_MAC_ADDR_1 + 1)

/* GPIO Select Configuration Definitions */
#define GPIO_SELECT_INPUT           0x00        /* 00b = Disable (GPIO is Input) */
#define GPIO_SELECT_OPEN_SOURCE     0x01        /* 01b = Open Source (Pull up is required)*/
#define GPIO_SELECT_OPEN_DRAIN      0x02        /* 10b = Open Drain (Pull down is required)*/
#define GPIO_SELECT_ALWAYS_ENABLE   0x03        /* 11b = Always Enable*/
/* [D7:D6] - Output (EESK)*/
/* [D5:D4] - Output (EECS) */
/* [D3:D2] - Input (EEDI) */
/* [D1:D0] - Output (EEDO) */
#define GPIO_EEPROM_SELECT_CFG          \
            ((GPIO_SELECT_ALWAYS_ENABLE << 6) |     \
             (GPIO_SELECT_ALWAYS_ENABLE << 4) |     \
             (GPIO_SELECT_INPUT << 2)         |     \
             (GPIO_SELECT_ALWAYS_ENABLE) )

/* GPIO Output Data Values in Normal Operating mode
// EESK = 0
// EECS = 1
// EEDI = 1
// EEDO = 1
*/
#define GPIO_EEPROM_OUTPUT_NORMAL		((0xfff0) | ((GP_EECS) | (GP_EEDI) | (GP_EEDO)))

/* EESK = 0
// EECS = 0 (HIGH to LOW)
// EEDI = 1
// EEDO = 1
*/
#define GPIO_EEPROM_OUTPUT_CS_H_TO_L	((0xfff0) | (GP_EEDI) | (GP_EEDO))

/* Global Definitions for Delay Functions - Input in microseconds */
#define GPIO_BASIC_DELAY        50          /* 50 microseconds */
#define GPIO_BASIC_US_PER_MS    1000
#define GPIO_DELAY_MS( ms )     (ms * GPIO_BASIC_US_PER_MS)
#define GPIO_DELAY_250us	(GPIO_BASIC_DELAY * 5)

/* Standard constants for ATM cells */
#define GP_ATM_PAYLOAD_SIZE		48

#define GP_ATM_HEADER_SIZE			4


#define GP_ATM_CELL_SIZE	(GP_ATM_PAYLOAD_SIZE + GP_ATM_HEADER_SIZE)  /* ATM cell size bytes (HEC byte excluded) */
#define GP_ATM_CELL_SIZE_DWORD	(GP_ATM_CELL_SIZE / sizeof(GP_DWORD))

/* Structure for an ATM cell in the host memory.
   (When a cell is received, the HEC byte is
   checked and stripped off by the PCU at the PHY layer. When transmitted,
   the PCU generates the HEC and inserts into the cell. So the HEC byte
   should not be considered here.) */

/* Defines used by GpCheckRetrain  - Re-train Criteria Spec AN-029 */
#define DEFAULT_MAX_CRC_ERRORS_PER_SEC_THRESHOLD                    25
#define DEFAULT_DOWNSTREAM_RATE_TO_MAX_CRC_ERRORS_PER_MIN_DIVIDER   32
#define SECS_TILL_RETRAIN_ON_EXCESS_CRC_ERRORS_PER_SEC              5
#define MINS_TILL_RETRAIN_ON_EXCESS_CRC_ERRORS_PER_MIN              5
#define DSP_STATUS_TIMER				    (pDslModem->GpMonitorInterval)
#define SECS_PER_DSP_STATUS_TICK                            (DSP_STATUS_TIMER / 1000)
#define DSP_STATUS_TICKS_PER_SEC                            (1000 / DSP_STATUS_TIMER)
#define DSP_STATUS_TICKS_PER_MIN                            ((60 * 1000) / DSP_STATUS_TIMER)
#define CRC_ERRORS_PER_DSP_STATUS_TICK_THRESHOLD            ((GP_WORD) ((DEFAULT_MAX_CRC_ERRORS_PER_SEC_THRESHOLD * DSP_STATUS_TIMER) / 1000))
#define TICKS_TILL_RETRAIN_ON_EXCESS_CRC_ERRORS_PER_TICK    ((GP_WORD) ((SECS_TILL_RETRAIN_ON_EXCESS_CRC_ERRORS_PER_SEC * 1000) / DSP_STATUS_TIMER))
#define TICKS_TILL_RETRAIN_ON_EXCESS_CRC_ERRORS_PER_MIN     ((GP_WORD) ((MINS_TILL_RETRAIN_ON_EXCESS_CRC_ERRORS_PER_MIN * 60 * 1000) / DSP_STATUS_TIMER))
#define ConvertPerSecondCountToPerTickCount(Count)          ((GP_WORD) ((Count * DSP_STATUS_TIMER) / 1000))


/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	Structure and Union declaration below here will have
	1-Byte packing alignment for structure and union.
	If you do not wish 1-Byte packing alignment,
	declare above this comment.
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/

#pragma pack(1)  /* 1-Byte packing alignment for structure and union */

/*

An ATM cell is laid out in 32-bit wide memory in the following way:

     bit 31                                    bit 0
       ---------------------------------------------
       | hbyte 3  | hbyte 2  | hbyte 1  | hbyte 0  |   Header
       ---------------------------------------------
       | ibyte 3  | ibyte 2  | ibyte 1  | ibyte 0  |   Info[3-0]
       ---------------------------------------------
       | ibyte 7  | ibyte 6  | ibyte 5  | ibyte 4  |   Info[7-4]
       ---------------------------------------------
       | ibyte 11 | ibyte 10 | ibyte 9  | ibyte 8  |   Info[11-8]
       ---------------------------------------------
       | ibyte 15 | ibyte 14 | ibyte 13 | ibyte 12 |
       ---------------------------------------------
       | ibyte 19 | ibyte 18 | ibyte 17 | ibyte 16 |
       ---------------------------------------------
       | ibyte 23 | ibyte 22 | ibyte 21 | ibyte 20 |
       ---------------------------------------------
       | ibyte 27 | ibyte 26 | ibyte 25 | ibyte 24 |
       ---------------------------------------------
       | ibyte 31 | ibyte 30 | ibyte 29 | ibyte 28 |
       ---------------------------------------------
       | ibyte 35 | ibyte 34 | ibyte 33 | ibyte 32 |
       ---------------------------------------------
       | ibyte 39 | ibyte 38 | ibyte 37 | ibyte 36 |
       ---------------------------------------------
       | ibyte 43 | ibyte 42 | ibyte 41 | ibyte 40 |
       ---------------------------------------------
       | ibyte 47 | ibyte 46 | ibyte 45 | ibyte 44 |   Info[47-44]
       ---------------------------------------------


The ATM header is packed as follows:


     bit 7              bit 0
       ---------------------
       | GFC     |  VPI(H) |     hdr[0]
       ---------------------
       | VPI(L)  |  VCI(H) |     hdr[1]
       ---------------------
       |        VCI        |     hdr[2]
       ---------------------
       |  VCI(L) | PTI |CLP|     hdr[3]
       ---------------------
            4       3    1
*/

typedef struct _GP_CELL
{
  GP_BYTE hdr[GP_ATM_HEADER_SIZE];      /* ATM header */
  GP_BYTE info[GP_ATM_PAYLOAD_SIZE];    /* ATM payload */
} GP_CELL;


/***************************************************************************************
	Struct _DSL_MODEM :
	This is the common data structure used by all high level PCI API routines.
	The variables in this section should be initialized by the customer's
	firmware prior to calling any PCI API routines.
	The user will allocate DSL_MODEM, zero the memory, then initialize it.
****************************************************************************************/
typedef struct _DSL_MODEM DSL_MODEM, *PDSL_MODEM;

struct _DSL_MODEM
{
	/* This section contains variables and pointers that must be initialized by the customer's
	 *  firmware prior to calling any embedded PCI routines
	 */
	void*			card;

	PPCI_REG		pciBase;	/* PCI memory mapped base address */
 
	GP_VOID			*pCustData;	/* User data context */

	GTI_XCVR_INFO	XcvrInfo;		/* Transceiver information structure */

	/* Items in this array can be set by both the customer's firmware and PCI API.*/
	struct PARAM_XCVR_ARRAY XcvrParam[GSI_NUM_OF_XCVR];

	GTIEOC_CMD 		EocCmd;	/* EOC command structure */
	GTIEOC_DATA_REG	EocDataReg;	/* EOC data register structure */

	/* The RX and TX DMA Memory must be allocated by the API User.
	// The memory must be Page-Locked and Contiguous.
	// The memory addresses must be linear and 32-bits.
	*/
	GP_DWORD		RxBuffStartV;	/* RX DMA Start Virtual(Linear) Address */
	GP_DWORD		RxBuffEndV;	/* RX DMA End Virtual(Linear) Address 	*/
	GP_DWORD		RxBuffStartP;	/* RX DMA Start Physical Address 	*/
	GP_DWORD		RxBuffEndP;	/* RX DMA End Physical Address		*/

	/* RX ATM Cell Count to cause an RX Interrupt */
	GP_WORD			RxCellCount;	/* 0-255 */

	GP_WORD			RetrainEnable;


	/* The timeout interval (msec) the User will call GpMonitorTransceiver()
	 * The API user must call GpMonitorTransceiver() at the milli-sec rate
	 * specified.  The user will set GpMonitorInterval in milli-sec.
	 */
	GP_DWORD		GpMonitorInterval;  /* milli-sec */

	/* --------------------------------------------------------------------------------
	 *	This section contains call back function pointers that must be initialized by the customer's
	 *  firmware prior to calling any embedded PCI routines
	 */

	GP_VOID			(*ReceiveCells)(PDSL_MODEM, GP_DWORD, GP_WORD);
	GP_VOID			(*Delay)(GP_DWORD);
	GP_VOID			(*XmitDmaDone)(PDSL_MODEM);

	GP_DWORD		(*ReadPciReg)(PDSL_MODEM, GP_WORD, GP_BYTE *);
	GP_BYTE			(*ReadPciRegB)(PDSL_MODEM,GP_WORD, GP_BYTE *);
	GP_VOID			(*WritePciReg)(PDSL_MODEM,GP_WORD, GP_DWORD);
	GP_VOID			(*WritePciRegB)(PDSL_MODEM, GP_WORD, GP_BYTE);

	/* --------------------------------------------------------------------------------
	 * Variables in the following section correspond to status which are set by
	 * PCI API routines and which may be needed by the customer's firmware.
	 */

	GP_DWORD		DownstreamRate;
	GP_DWORD		UpstreamRate;
	GP_WORD			DslLinkState;
	GP_DWORD		TotalNumTxCells;		/* Running count of transmitted cells*/
	GP_DWORD		TotalNumRxCells;		/* Running count of received cells*/
	GP_DWORD		RetrainCount;			/* Running count of retrains initiated*/
	GP_DWORD		RxDmaErrCnt;			/* Count of number of receive DMA errors detected*/
	GP_WORD			HardwareStatus;			/* GP_SUCCESS if hardware is ok, otherwise GP_FAIL.*/
	GP_WORD			StatusBuffer[512];		/* GpStatusXcvr(): Status buffer containing XCVR status*/
	GP_WORD			TestIODriverResult;		/* Stored return status from Gti_TestIODrivers, which is called by*/
								/* GpCheckHardware */

	GP_WORD			Counters[GTI_COUNTERS_LEN/sizeof(GP_WORD)];   /* Counters array read from transceiver status cmd.*/
	GP_WORD			AccCounters[GTI_COUNTERS_LEN/sizeof(GP_WORD)];/* Counters array read from transceiver and accumulated*/
	GP_WORD			Failures[GTI_FAILURES_LEN/sizeof(GP_WORD)];   /* Failures array read from transceiver status cmd.*/

	/* --------------------------------------------------------------------------------
	 * The variables in this section are used internally by PCI API routines.
	 */

	GP_DWORD		TrainingTimer;		/* Training timeout timer*/
	GP_DWORD		PendingCellDelTimer;	/* Pending cell delineation timeout timer*/

	GP_DWORD		TxDmaTimer;		/* Used to timeout the transmit DMA operation.*/
							/* If the transmit DMA remains in the pending state for too*/
							/* long, the TxDmaPending flag is cleared to allow further*/
							/* Tx DMA transfers to continue.*/

	GP_WORD			TxDmaPending;		/* Set when a Tx DMA operation is initiated.*/
							/* Cleared when the Tx DMA complete interrupt occurs.*/
	GP_WORD			InterruptStatus;	/* Interrupt status word - read from hardware*/
	GP_DWORD			InitStatus;		/* Initialization status - indicates how far initialization or shutdown*/
							/* has progressed.*/

	GP_DWORD		RxCellRdPtrSav;		/* Saves the last value of the read buffer pointer*/
							/* from the previous receive DMA interrupt.*/

	GP_DWORD		Disconnect_Timeout;

	/* SNR Margin status is read slowly in the background and stored.
	// When ReadStarWord or ReadStarByte
	// is called to read one of these locations, the value is read from
	// the local array rather than from the DSP directly. This prevents the
	// driver from taking too long (10s of msec) to read SNR margin.
	*/
	GP_WORD		SNRStatusRead;		/* 0 = SNR margin status has not yet been read since the last startup or retrain.*/
						/* Used to ensure that SNR margin status is read once directly from the DSP
						//     following retrain before background reading begins.
						// 1 = Status has been read once directly from DSP but background reading has not
						//     completed the first time.
						// 2 = Background reading of SNR status has completed at least once and the
						//     local array contains valid data.
						*/
	GP_WORD		LocalDspAddH;		/* Local image of DSP address pointer high byte. Used for reading from local array.*/
	GP_WORD		LocalDspAddL;		/* Local image of DSP address pointer low byte.*/

	GP_WORD		DspAddH;		/* DSP address pointer high byte. Used for reading from DSP and storing in local array.*/
	GP_WORD		DspAddL;		/* DSP address pointer low byte.*/

	/* The following are local copies of data required to compute SNR margin*/

	GP_WORD		DspRamD6_H[256];	/* Local image of DSP RAM block 0xD6 high byte (SQ per bin data)*/
	GP_WORD		DspRamD6_L[256];	/* Local image of DSP RAM block 0xD6 low byte*/

	GP_WORD		DspRamD7_H[256];	/* Local image of DSP RAM block 0xD7 high byte  (DS bit loading data)*/
	GP_WORD		DspRamD7_L[256];	/* Local image of DSP RAM block 0xD7 low byte*/
	GP_WORD		DspRamD7_E[256];	/* Local image of DSP RAM block 0xD7 extended byte*/

	GP_WORD		DspRamF4_H[256];	/* Local image of DSP RAM block 0xF4 high byte (US bit loading data)*/
	GP_WORD		DspRamF4_L[256];	/* Local image of DSP RAM block 0xF4 low byte*/
	GP_WORD		DspRamF4_E[256];	/* Local image of DSP RAM block 0xF4 extended byte*/

	/* Variables used by retrain algorithm */
	GP_WORD		DspCounters[GTI_COUNTERS_LEN/sizeof(GP_WORD)];
	GP_WORD		DspFailures[GTI_FAILURES_LEN/sizeof(GP_WORD)];

	GP_WORD		MaxCrcErrorsPerSecThreshold;
	GP_WORD		DownstreamRateToMaxCrcErrorsPerMinDivider;
	GP_WORD		CrcErrorsPerMinThreshold;
	GP_WORD		CrcErrorsPerMin;
	GP_WORD		ExcessiveConsecutiveCrcErrorsPerTickCount;
	GP_WORD		ExcessiveConsecutiveCrcErrorsPerMinCount;
	GP_WORD		TickCounter;
	GP_WORD		TrainingIsrCnt;
	GP_WORD		TrainingIsrCntOrig;

	int		GpioState; /* Image of the GPIO output register used for controlling LEDs.*/
	GP_WORD		ExcessiveConsecutiveOverallFailureCount;
} ;



/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	Structure and Union declaration above here will have
	1-Byte packing alignment for structure and union.
	If you do not wish 1-Byte packing alignment,
	declare above the #pragma pack(1).
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/

#pragma pack()  /* revert back packing to the value specified by the compiler */

#endif	/* GP_DEFINE_H */
