#ifndef GP_HARDWARE_H
#define GP_HARDWARE_H

/************************************************************************
 * FILE NAME   : GpHardware.h
 *
 * DESCRIPTION : This module defines constants for hardware specific
 *					addresses, registers, masks, etc..
 *
 * (C) Copyright 2001 by GlobeSpan Inc
 *
 * $Header: /usr/local/cvsroot/wanpipe_common/adsl/Epci/GpHardware.h,v 1.2 2003/01/10 07:56:07 alex Exp $ *
 *                                                                      *
 ************************************************************************/

/*
// PULSAR is the default.
// To build for PANDA, define GSI_PANDA
//
*/

#ifdef GSI_PANDA
#define GP_PCI_DEVICE_ID		(0xD001)	/* Panda ID*/
#define GP_DEFAULT_RX_CELL_CNT		(40)		/* RxCellCount Int 40 cells = 2 msec @ 8 Mbps */
#else
#define GP_PCI_DEVICE_ID		(0xD002)	/* Pulsar ID */
#define USE_PCU_RX_IDLE_CELLS
#undef RX_WD_TIMER_FIX
#define GP_DEFAULT_RX_CELL_CNT		(40)		/* RxCellCount Int 40 cells = 2 msec @ 8 Mbps */
#endif /* GSI_PANDA */

#define PCORE_VENDOR_ID			(0x14BC)   	/* Globespan PCI ADSL vendor id*/
#define PCORE_REVISION_NO		(0x01)

#define GP_ALL_ZEROS_MASK		(0x00000000)
#define GP_ALL_ONES_MASK		(0xffffffff)
#define PCORE_RESET_MASK		(0x00000033)    /* Resets the Panda chip*/
#define GP_NOVA_RESET_MASK		(0x00000002)    /* Resets the Nova chip only*/
#define RX_CELL_CNT_MASK		(0xFFFF0007)


#define PCU_ADDR_RX_IDLE_CELLS		(0xBE)		/* PCU addr - Send ATM Idle Cells (HIBYTE=Mode, LOBYTE=Count) */
#define PCU_RX_ALL_IDLE_CELLS		(1)		/* PCU mode - Pass all Idle Cells*/
#define PCU_RX_CNT_IDLE_CELLS		(0)		/* PCU mode - Pass only Count of Idle Cells*/


/* GPIO7[11], PHY_INT[8], RxWDTimer[6], RXCellCnt[5], TXEob[0] */
#ifdef RX_WD_TIMER_FIX
	#define PCORE_INT_ENABLE_MASK   (0x00000901)	/* enable GPIO7[11], PHY_INT[8], TXEob[0]*/
#else /* RX_WD_TIMER_FIX */
#ifdef USE_PCU_RX_IDLE_CELLS
	#define PCORE_INT_ENABLE_MASK   (0x00000121)	/* enable PHY_INT[8], RXCellCnt[5], TXEob[0]*/
#else /* USE_PCU_RX_IDLE_CELLS */
	#define PCORE_INT_ENABLE_MASK   (0x00000161)	/* enable PHY_INT[8], RxWDTimer[6], RXCellCnt[5], TXEob[0]*/
#endif /* USE_PCU_RX_IDLE_CELLS*/
#endif /* RX_WD_TIMER_FIX*/

#define PCORE_DSL_CHIPSELECT           (0x01)
/* For Pulsar, Glue Timing20 is the same as Panda Timing01, panda 4 range ---> pulsar 8 range */
#define PCORE_GLUETIMING01_MASK        (0x55550000)  /* Required for proper device operation */

#define PCORE_GPIOINPUTCONFG_MASK      (0x55555555)

#if defined(GSI_PANDA)&&defined(RX_WD_TIMER_FIX)
#define PCORE_GPIOOUTPUTCFG1_MASK	(0x0000c000)  /* GPIO7 for use as an interrupt - RX_WD_TIMER_FIX */
#else  /* PULSAR is default */
#define PCORE_GPIOOUTPUTCFG1_MASK	(0x000000FF)  /* (0x00000000) - Disable GPIO[0]-GPIO[15] */
#endif

#define PCORE_CLOCKCONFG_MASK           (0x00006000)  /* Required for proper device operation */

/* Panda/Pulsar address offset constants */
/* Range 0 - Pulsar/Panda DSL Data  0x0000 */

/* Write registers */
#define PCORE_ADDR_TXCFG            (0x00 / sizeof(GP_DWORD))
#define PCORE_ADDR_TXSTARTADD       (0x04 / sizeof(GP_DWORD))
#define PCORE_ADDR_TXENDADD         (0x08 / sizeof(GP_DWORD))
#define PCORE_ADDR_TXCOMMAND        (0x0C / sizeof(GP_DWORD))
#define PCORE_ADDR_RXCFG1           (0x10 / sizeof(GP_DWORD))
#define PCORE_ADDR_RXCFG2           (0x14 / sizeof(GP_DWORD))
#define PCORE_ADDR_RXSTARTADD       (0x18 / sizeof(GP_DWORD))
#define PCORE_ADDR_RXENDADD         (0x1C / sizeof(GP_DWORD))
#define PCORE_ADDR_RXCOMMAND        (0x20 / sizeof(GP_DWORD))

/* Read registers */
#define PCORE_ADDR_STATUS           (0x00 / sizeof(GP_DWORD))
#define PCORE_ADDR_RXCURRENTADD     (0x04 / sizeof(GP_DWORD))


/* Range 2 - Pulsar/Panda DSL Control (DSP Host interface)  0x4000 */

#define DSP_HOST			(0x4000 /sizeof(GP_DWORD))
#define DSP_CFG				(0x4024 /sizeof(GP_DWORD)) /* Host reg. 0x9 */ 
#define DSP_DATAE			(0x402C /sizeof(GP_DWORD)) /* Host reg. 0xb */
#define DSP_ADDL			(0x4030 /sizeof(GP_DWORD)) /* Host reg. 0xc */
#define DSP_ADDH			(0x4034 /sizeof(GP_DWORD)) /* Host reg. 0xd */
#define DSP_DATAL			(0x4038 /sizeof(GP_DWORD)) /* Host reg. 0xe */
#define DSP_DATAH			(0x403C /sizeof(GP_DWORD)) /* Host reg. 0xf */

/* Range 6 - Pulsar/Panda Glue starting 0xC000 */
/* Write Registers */
#define PCORE_ADDR_GLUECONFIG       	(0xC000 / sizeof(GP_DWORD))
#define PCORE_ADDR_GLUETIMING01     	(0xC004 / sizeof(GP_DWORD))
#define PCORE_ADDR_GLUETIMING23     	(0xC008 / sizeof(GP_DWORD))
#define PCORE_ADDR_GLUERESETIMAGE   	(0xC00C / sizeof(GP_DWORD))
#define PCORE_ADDR_CLOCKSCONFIG     	(0xC010 / sizeof(GP_DWORD))
#define PCORE_ADDR_CLOCKSRRM1       	(0xC014 / sizeof(GP_DWORD))
#define PCORE_ADDR_CLOCKSRRM2       	(0xC018 / sizeof(GP_DWORD))
#define PCORE_ADDR_DSLSUBSYSTEMID   	(0xC01C / sizeof(GP_DWORD))
#define PCORE_ADDR_GPIOINPUTCFG1    	(0xC024 / sizeof(GP_DWORD))
#define PCORE_ADDR_GPIOOUTPUTCFG1   	(0xC028 / sizeof(GP_DWORD))
#define PCORE_ADDR_GPIOOUTPUTSEL1   	(0xC02C / sizeof(GP_DWORD))
#define PCORE_ADDR_GPIOOUTPUT1      	(0xC030 / sizeof(GP_DWORD))
#define PCORE_ADDR_GPIOINPUTCFG2    	(0xC034 / sizeof(GP_DWORD))
#define PCORE_ADDR_GPIOOUTPUTCFG2   	(0xC038 / sizeof(GP_DWORD))
#define PCORE_ADDR_GPIOOUTPUTSEL2   	(0xC03C / sizeof(GP_DWORD))
#define PCORE_ADDR_GPIOOUTPUT2      	(0xC040 / sizeof(GP_DWORD))
#define PCORE_ADDR_DSLINTENABLE     	(0xC044 / sizeof(GP_DWORD))
#define PCORE_ADDR_DSLINTCLEAR      	(0xC048 / sizeof(GP_DWORD))
#define PCORE_ADDR_VMODEMINTENABLE  	(0xC04c / sizeof(GP_DWORD))
#define PCORE_ADDR_VMODEMINTCLEAR   	(0xC050 / sizeof(GP_DWORD))
#define PCORE_ADDR_POWMGMTINTENABLE 	(0xC054 / sizeof(GP_DWORD))
#define PCORE_ADDR_POWMGMTINTCLEAR  	(0xC058 / sizeof(GP_DWORD))

/* Read registers */
#define PCORE_ADDR_MISCREG          	(0xC000 / sizeof(GP_DWORD))
#define PCORE_ADDR_GPIOINPUTSTAT1   	(0xC004 / sizeof(GP_DWORD))
#define PCORE_ADDR_GPIOINPUTSTAT2   	(0xC008 / sizeof(GP_DWORD))


#ifdef GSI_PANDA
	#define PCORE_ADDR_DSLINTSTAT       (0xC00C / sizeof(GP_DWORD))
#else /* PULSAR is default */
	/* For Pulsar, Int Stat register is the same as Int. Clear Register 0x48 */
	#define PCORE_ADDR_DSLINTSTAT       (0xC048 / sizeof(GP_DWORD))
	#define PULSAR_UNCOMMITTED_REG_OUT  (0xC06C / sizeof(GP_DWORD))
	#define PULSAR_HOSTSEL_MASK         (0x00004020)
#endif



#define PCORE_ADDR_POWMGMTCTLSTAT   (0xC018 / sizeof(GP_DWORD))

#define PCORE_INITIAL_TXCFG         (0x000000d1)  /* single operation selected */

#define PCORE_INITIAL_RXCFG1	    (0x00000000)
#define PCORE_INITIAL_RXCFG2        (0x0000000D)	/* watchdog timer disabled */

#define PCORE_INITIAL_TXSTARTADD    (0)
#define PCORE_INITIAL_TXENDADD      (0)
#define PCORE_INITIAL_TXCOMMAND     (0x00034006)  /* TX DMA init/disabled, WordSize=13=1101 */

#define PCORE_INITIAL_RXSTARTADD    (0)
#define PCORE_INITIAL_RXENDADD      (0)
#define PCORE_INITIAL_RXCOMMAND     (0x00034000)  /* RX DMA init/disabled, WordSize=13=1101 */

#define GP_INT_CLEAR_MASK	PCORE_INT_ENABLE_MASK

/* Interrupt status bits - used to mask InterruptStatus read from the hardware */
#define    TX_EOB_INT		0x0001
#define    RESERVED1_INT	0x0002
#define    GPIO0_INT		0x0004
#define    GPIO1_INT		0x0008
#define    RX_EOB_INT		0x0010
#define    RX_CELL_CNT_INT  0x0020
#define    RX_WD_TIMER_INT  0x0040
#define    RESERVED2_INT	0x0080
#define    DSP_INT			0x0100
#define    PCI_BSY_INT		0x0200
#define    GPIO6_INT		0x0400
#define    GPIO7_INT		0x0800
#define    GPIO8_INT		0x1000
#define    GPIO16_INT		0x2000
#define    GPIO17_INT		0x4000
#define    GPIO18_INT		0x8000

/*
// Structures and data definition for Verifying the hardware
// is responding and functional
*/
typedef enum {GP_TEST_NONE=0, GP_DSP, GP_PIF} GP_TEST_TYPES;

typedef struct
{
   GP_DWORD region;
   GP_DWORD addr;
   GP_DWORD data;
} GP_IO;

typedef struct
{
   GP_DWORD	type;
   GP_IO	ioWrite;
   GP_IO	ioRead;
} GP_IO_CHECK;

typedef enum { DslDatReg,MacDmaReg,DslCtlReg,MacCtlReg,VmodemReg,HpnaReg,GeneralReg,ExtGlueReg} REG_BANK;


/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	Structure and Union declaration below here will have
	1-Byte packing alignment for structure and union.
	If you do not wish 1-Byte packing alignment,
	declare above this comment.
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/

#pragma pack(1)  /* 1-Byte packing alignment for structure and union */

/* DSL interrupt enable/clear has this format: */
/*
typedef union
{
   struct
   {
      GP_DWORD    txEob:1;
      GP_DWORD    reserved1:1;
      GP_DWORD    gpio0:1;
      GP_DWORD    gpio1:1;
      GP_DWORD    rxEob:1;
      GP_DWORD    rxCellCnt:1;
      GP_DWORD    rxWdTimer:1;
      GP_DWORD    reserved2:1;
      GP_DWORD    phyIntFlag:1;
      GP_DWORD    pciBsy:1;
      GP_DWORD    gpio6:1;
      GP_DWORD    gpio7:1;
      GP_DWORD    gpio8:1;
      GP_DWORD    gpio16:1;
      GP_DWORD    gpio17:1;
      GP_DWORD    gpio18:1;
      GP_DWORD    reserved3:16;
   };
   GP_DWORD   word;
} ISR_STAT_WORD;
*/


/* Panda occupies 16 kbytes of contiguous space in the PCI address */

/*typedef volatile struct _PCI_REG PCI_REG */
struct _PCI_REG						/* Device memory mappped I/O space */
{
	union
	{
		GP_DWORD	DslReg[0x4000];		/* 16K Dwords - used for long word access */
		GP_BYTE		DslRegB[0x10000];	/* 64K bytes  - used for byte access */
	} RegSpace;
};

typedef struct _PCI_REG PCI_REG, *PPCI_REG;

/*!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	Structure and Union declaration above here will have
	1-Byte packing alignment for structure and union.
	If you do not wish 1-Byte packing alignment,
	declare above the #pragma pack(1).
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*/

#pragma pack()  /* revert back packing to the value specified by the compiler */

#endif /* GP_HARDWARE_H */

