/************************************************************************
 * FILE NAME   : GpInitRegisters.c 
 *
 * DESCRIPTION : This module initializes registers in the PCI device.                                   
 *                                                                      
 * (C) Copyright 2001 by GlobeSpan Inc                   
 *                                                                      
 * $Header: /usr/local/cvsroot/wanpipe_common/adsl/Epci/GpInitRegisters.c,v 1.1.1.1 2002/11/25 09:21:53 ncorbic Exp $ *
 *                                                                      *
 ************************************************************************/

#include "GpInclude.h"

GP_VOID GpInitRegisters(PDSL_MODEM pDslModem)
{

	GpWritePciReg(pDslModem, PCORE_ADDR_GLUETIMING01,PCORE_GLUETIMING01_MASK );
	
	/* The following two lines do a reset on panda (which also includes
	// reseting NOVA). The PCORE_RESET_MASK gets written to the reset
	// image register in panda.
	*/
	GpWritePciReg(pDslModem,  PCORE_ADDR_GLUERESETIMAGE, PCORE_RESET_MASK);
	GpDelay(50);  /* wait 50 microseconds so that reset really takes effect. */

	GpWritePciReg(pDslModem, PCORE_ADDR_CLOCKSCONFIG, PCORE_CLOCKCONFG_MASK);

	/* Set gpio input pins to edge-sensitive rather than level-sensitive to avoid relentless 
	// interrupts (or they could kill you right at boot time; this is a friendly advice)
	*/
	GpWritePciReg(pDslModem, PCORE_ADDR_GPIOINPUTCFG1, PCORE_GPIOINPUTCONFG_MASK);
	GpWritePciReg(pDslModem, PCORE_ADDR_GPIOINPUTCFG2, PCORE_GPIOINPUTCONFG_MASK);


	/* Enable GPIOs based on PCORE_GPIOOUTPUTCFG1_MASK */
	GpWritePciReg(pDslModem,  PCORE_ADDR_GPIOOUTPUTCFG1, PCORE_GPIOOUTPUTCFG1_MASK); 

	/* Disable and clear DSL interrupt status bits */
	GpWritePciReg(pDslModem,  PCORE_ADDR_DSLINTENABLE, GP_ALL_ZEROS_MASK);
	GpWritePciReg(pDslModem,  PCORE_ADDR_DSLINTCLEAR, GP_ALL_ONES_MASK);

	#ifndef GSI_PANDA
	GpWritePciReg(pDslModem,  PULSAR_UNCOMMITTED_REG_OUT, PULSAR_HOSTSEL_MASK);
	#endif

	/* Disable and clear Vmodem interrupt status bits 
	// (this might be a bad thing for vmodem, we do it here to be safe for DSL driver, 
	// later on when the dust is settled and 
	// when we have voice modem onboard, we may consider removing this line.)
	*/
	GpWritePciReg(pDslModem,  PCORE_ADDR_VMODEMINTENABLE, GP_ALL_ZEROS_MASK);
	GpWritePciReg(pDslModem,  PCORE_ADDR_VMODEMINTCLEAR, GP_ALL_ONES_MASK);

	/* Disable and clear power management interrupt status bits */
	GpWritePciReg(pDslModem,  PCORE_ADDR_POWMGMTINTENABLE, GP_ALL_ZEROS_MASK);
	GpWritePciReg(pDslModem,  PCORE_ADDR_POWMGMTINTCLEAR, GP_ALL_ONES_MASK);

	/* set TX and RX DMA registers to initial values */
	GpWritePciReg(pDslModem,  PCORE_ADDR_TXCFG, PCORE_INITIAL_TXCFG);
	GpWritePciReg(pDslModem,  PCORE_ADDR_TXSTARTADD, PCORE_INITIAL_TXSTARTADD);
	GpWritePciReg(pDslModem,  PCORE_ADDR_TXENDADD, PCORE_INITIAL_TXENDADD);
	GpWritePciReg(pDslModem,  PCORE_ADDR_TXCOMMAND, PCORE_INITIAL_TXCOMMAND);
	GpWritePciReg(pDslModem,  PCORE_ADDR_RXCFG1, (PCORE_INITIAL_RXCFG1&RX_CELL_CNT_MASK)|(pDslModem->RxCellCount<<3));
	GpWritePciReg(pDslModem,  PCORE_ADDR_RXCFG2, PCORE_INITIAL_RXCFG2);
	GpWritePciReg(pDslModem,  PCORE_ADDR_RXSTARTADD, PCORE_INITIAL_RXSTARTADD);
	GpWritePciReg(pDslModem,  PCORE_ADDR_RXENDADD, PCORE_INITIAL_RXENDADD);
	GpWritePciReg(pDslModem,  PCORE_ADDR_RXCOMMAND,PCORE_INITIAL_RXCOMMAND);

}

