/************************************************************************
* File:		   : GpIo.c
*
* Description  : OS and hardware dependent register I/O call back routines.
*                These are intended to be customer supplied functions.
*		 This file is included as an example but will most likely
*	     	 require for the particular system being developed.
*
* $Header: /usr/local/cvsroot/wanpipe_common/adsl/Epci/GpIo.c,v 1.4 2003/10/08 20:54:19 sangoma Exp $ *
*                                                                      *
************************************************************************/

#define NEX_DBG 0
#define GpDbgPrint	DbgPrint

#if NEX_DBG > 1
#include <Gp.h>
#endif

#include "Gp.h"
#include "GpInclude.h"

/************************************************************************
* Function     : CustReadPciReg
*
* Description  : Read one 32 bit word from the specified PCI device register.
*
* Input        : Address - register Address offset relative to the specified register bank
*
* Output       : data - least sig. byte of data read.
*
* Return       : full 32 bit data value read. used when reading registers other than
*				 DSP host registers.
************************************************************************/

GP_DWORD CustReadPciReg(
	    PDSL_MODEM pDslModem,
	    GP_WORD Address,
	    GP_BYTE *pData
	    )
{
	GP_DWORD DataLong = 0;

	/* In general, need to delay 100 ns between consecutive reads (1 usec is over kill).
	// However, there is no other easy way to get machine independent timing.
	*/
	GpDelay(1);

	DataLong = wpabs_bus_read_4(pDslModem->card, Address << 2);

	*pData = (GP_BYTE) DataLong;

	GpDelay(1);
	
	return DataLong;
}
/************************************************************************
* Function     : CustReadPciRegB
*
* Description  : Read one byte word from the specified PCI device register.
*		 Used primarily to read from DSP host interface
*
* Input        : Address - register Address offset relative to the specified register bank
*
* Output       : data - data byte read.
*
* Return       : data byte read
************************************************************************/

GP_BYTE CustReadPciRegB(
	    PDSL_MODEM pDslModem,
	    GP_WORD Address,
	    GP_BYTE *pData
	    )
{
	GP_BYTE DataByte = 0;
	
	/* In general, need to delay 100 ns between consecutive reads (1 usec is over kill).
	// However, there is no other easy way to get machine independent timing.
	*/
	GpDelay(1);

	DataByte = wpabs_bus_read_1(pDslModem->card, Address << 2);

	*pData = DataByte;

	GpDelay(1);
	
	return DataByte;
}
/************************************************************************
* Function     : CustWritePciReg
*    :
* Description  : Write one 32 bit word to the specified PCI device register.
*
* Input        : Address - register Address offset relative to the specified register bank
*		 data - 32 bit value to be written.
*
* Output       : None
* Return       : None
************************************************************************/
GP_VOID CustWritePciReg(
	       PDSL_MODEM pDslModem,
	       GP_WORD Address,
	       GP_DWORD Data
	       )
{

	GpDelay(1);

	wpabs_bus_write_4(pDslModem->card, Address << 2, Data);

	GpDelay(1);
}

/************************************************************************
* Function     : CustWritePciRegB
*    :
* Description  : Write one byte to the specified PCI device register.
*
* Input        : Address - register Address offset relative to the specified register bank
*		 Data - byte value to be written.
*
* Output       : None
* Return       : None
************************************************************************/


GP_VOID CustWritePciRegB(
	       PDSL_MODEM pDslModem, 
	       GP_WORD Address, 
	       GP_BYTE Data
	       )
{
	GpDelay(1);

	wpabs_bus_write_1(pDslModem->card, Address << 2, Data);
		
	GpDelay(1);
}

