/************************************************************************
 * FILE NAME   : GpIsr.c
 *
 * DESCRIPTION : This file contains all interrupt related routines.
 *
 * (C) Copyright 2001 by GlobeSpan Inc
 *                                                                      *
 * $Header: /usr/local/cvsroot/wanpipe_common/adsl/Epci/GpIsr.c,v 1.9 2003/10/29 18:22:58 sangoma Exp $ *
 *                                                                      *
 ************************************************************************/

#include <Gp.h>
#include "GpInclude.h"

/************************************************************************
 * Function     : GpDisableInterrupt
 *
 * Description  :  This routine is called to disable interrupts
 *
 * Input        : pDslModem _ A pointer to the DslModem information structure.
 *
 * Output       : None
 * Return       : None
 ************************************************************************/
GP_VOID GpDisableInterrupt(
    PDSL_MODEM pDslModem
    )
{
	/* GP_BYTE dummy; */

    	GpWritePciReg(pDslModem, PCORE_ADDR_DSLINTENABLE, GP_ALL_ZEROS_MASK);
    	/* pDslModem->InterruptStatus.word |= GpReadPciReg(pDslModem, PCORE_ADDR_DSLINTSTAT, &dummy ); */
    	/* GpWritePciReg(pDslModem, PCORE_ADDR_DSLINTCLEAR, pDslModem->InterruptStatus.word ); */
	pDslModem->InitStatus &= (~GP_INIT_ENABLED_INT);

}

/************************************************************************
 * Function     : GpEnableInterrupt
 *
 * Description  : Called to enable interrupts
 *
 * Input        : pDslModem _ A pointer to the Dslmodem information structure.
 *
 * Output       : None
 * Return       : None
 ************************************************************************/
GP_VOID GpEnableInterrupt(
    PDSL_MODEM pDslModem
    )
{
	/*GP_BYTE	dummy; */

	pDslModem->InitStatus |= GP_INIT_ENABLED_INT;
    	GpWritePciReg(pDslModem, PCORE_ADDR_DSLINTENABLE, PCORE_INT_ENABLE_MASK );
    	/* pDslModem->InterruptStatus.word |= GpReadPciReg(pDslModem, PCORE_ADDR_DSLINTSTAT, &dummy ); */

}

/************************************************************************
 * Function     : GpCheckInterrupts
 * Description  :
 *
 *              : This routine should be called from the main ISR.
 *                The return status indicates whether the interrupt
 *				  was for this device.
 *
 * Output       : none
 *              :
 * Return       : GP_TRUE - if interrupt is for the DSL modem.
 *		  GP_FALSE - if interrupt is not for the DSL modem.
 ************************************************************************/
GP_WORD GpCheckInterrupts(
    PDSL_MODEM pDslModem
    )
{
	GP_BYTE	dummy;
	GP_DWORD LocalInterruptStatus;

    	/* Save the current interrupt status on the card.
    	** Get interrupt status word from PCI chip
	*/
	LocalInterruptStatus = GpReadPciReg(pDslModem, PCORE_ADDR_DSLINTSTAT, &dummy) & GP_INT_CLEAR_MASK;
	
	#if 0 /* GP_DBG */
	DbgPrint("   LocalInterruptStatus=%#x\n", LocalInterruptStatus);
	#endif

	if (LocalInterruptStatus){	/* Are any interupts set? */
		if(LocalInterruptStatus & DSP_INT){/* Is it an interrupt for the DSP? */
			Gti_ClearDSPInterrupt(&pDslModem->XcvrInfo, GTI_DSP_ALL);
		}
	
		/* Clear interrupts that were set*/
		GpWritePciReg(pDslModem, PCORE_ADDR_DSLINTCLEAR, LocalInterruptStatus);
	}
	pDslModem->InterruptStatus = (GP_WORD)LocalInterruptStatus; /*  Save Interrupt Status */

	return pDslModem->InterruptStatus;
}

/************************************************************************
 * Function     : GpInterruptHandler
 *
 * Description  : This routine does the actual processing for the DSL modem interrupts.
 *		  If the OS prefers to keep the bulk of the interrupt processing
 *		  at a prioroty level somewhat below that of the main ISR,
 *		  then this routine should be queued to run at the lower level.
 *
 * Input        :
 *
 * Output       : None
 * Return       : None
 ************************************************************************/
GP_VOID GpInterruptHandler(PDSL_MODEM pDslModem)
{
	
#if GP_DBG
	GP_BYTE index;
#endif
	GP_DWORD result;

    	GpDisableInterrupt(pDslModem);
	
	wpabs_debug_isr("ISR: GpInterruptHandler\n");

	/* Transceiver (DSP) Interrupt */
 	if(pDslModem->InterruptStatus & DSP_INT){

		wpabs_debug_isr("ISR: Transceiver Interrupt\n");
		++pDslModem->TrainingIsrCnt;
		
		/* Clear DSP interrupt status bit */
		pDslModem->InterruptStatus &= (~DSP_INT);

#if 0 /* GP_DBG */
		DbgPrint("PHY interrupt, pDslModem->DslLinkState: %#x\n", 
					pDslModem->DlsLinkState);
#endif

			
		if (pDslModem->DslLinkState == GP_TRAINED){			
			
			/* In data mode, interrupt corresponds to EOC (or fast retrain) */
			/* Perform EOC processing - interrupt occurs at most every 17 msec */
			GTIEocProc(&pDslModem->XcvrInfo, &pDslModem->EocCmd);
			
			/* Clear autonomous data length for next time. */
			pDslModem->EocCmd.tx.AutoLen = 0;				

		}else if (pDslModem->DslLinkState != GP_PENDING_CELLDEL){ 
		
			/* If state is not data mode or pending cell deliniation */
			/* Download the next code segment */
            		result = ExecuteXCVR(&pDslModem->XcvrInfo);

			wpabs_debug_isr("ExecuteXCVR from ISR handler %d\n",result);

			/* Work around for FW Prog POP1(output) into 
			 * PIP1(input) for DyingGasp */
			WriteDspAdd(&pDslModem->XcvrInfo, GTI_DSP_A, 0xA7, GTI_IREGA);
			WriteStarW(&pDslModem->XcvrInfo, GTI_DSP_A, 0x00, 0x00);

			/* If the return result is GTI_COMPLETE_PASS 
			 * then the XCVR is in DATA_MODE */
			
			if (result==GTI_COMPLETE_PASS){
				if (pDslModem->DslLinkState == GP_TRAINING){
					
					wpabs_debug_isr(" TX-ISR: Pending Cell Del\n");
					pDslModem->DslLinkState = GP_PENDING_CELLDEL;
					/* reset pending cell delineation timer */
					pDslModem->PendingCellDelTimer = 0;
					pDslModem->RetrainCount = 0;
					pDslModem->RxDmaErrCnt = 0;
				}
			}
		}

	}  /* End of transceiver (DSP) interrupt processing */

	
	/*
	 * Check for Panda DMA interrupts, but only
	 * if in data mode. DMA interrupts should be ignored
	 * when not in data mode.
	 * Note: this fixes a crash caused by the rcv. WD timer int.
	 */
  	
	if((pDslModem->DslLinkState == GP_TRAINED)||(pDslModem->DslLinkState == GP_LOCAL_LB)){

		if (pDslModem->InterruptStatus & TX_EOB_INT){  /* Tx End-of-DMA Block Interrupt */
	
#if 0 /* GP_DBG */
	        	GpDgbPrint("GpHandleInterrupt: Got a Tx End-of-Block interrupt\n");
#endif
			wpabs_debug_tx("(Debug): TX Interrupt\n");
			
			/* Clear Tx DMA interrupt status bit */
			pDslModem->InterruptStatus &= (~TX_EOB_INT);

			/* Reset Tx DMA timeout timer */
			pDslModem->TxDmaTimer = 0;

			/* Call customer call back function to
			// initiate the next Tx DMA transfer.
			*/
			GpXmitDmaDone(pDslModem);

		}	/* End of Tx DMA interrupt processing */


		/* Check RX cell count interrupt.
		// This occurs when the programmed no. of cells have been received.
		*/
		if(pDslModem->InterruptStatus & RX_CELL_CNT_INT){
 	
			wpabs_debug_rx("(Debug): RX Interrupt\n");

			/* Clear Rx DMA interrupt status bit	*/
       			pDslModem->InterruptStatus &= (~RX_CELL_CNT_INT);

			/* Check receive DMA buffer for new cells.
			// Cells will be passed to the customer's
			// firmware by calling the GpReceiveCells
			// call back function.
			*/
			GpReceiveDmaHandler(pDslModem);				
    		
			if (pDslModem->InitStatus & GP_INIT_RX_IDLE){
				wpabs_debug_tx("Turning off rx idle cells\n");
				GpSetRxIdleCellCount(pDslModem);
				pDslModem->InitStatus &= (~GP_INIT_RX_IDLE);
				
				/* NC: Added since this is caused by the
				** tx timeout
				*/
				gsi_dma_tx_start(pDslModem->pCustData);
			}
		}
  	
	}/* End of DMA interrupt processing */

  	GpEnableInterrupt(pDslModem);
}


/*************************************************************
* Function:   GpTransmitCells
* Input:
* Return:	   GP_SUCCESS if transmit DMA operation was initiated.
*			   GP_PENDING if previous Tx DMA is pending and a
*				new Tx DMA operation could not be started.
*			   GP_FAIL if buffer address is not valid
**************************************************************/
GP_WORD GpTransmitCells(PDSL_MODEM pDslModem, GP_DWORD TxBuffStartP, GP_DWORD TxBuffEndP)
{
	GP_DWORD NumTxCellsThisBuffer;

	/* Check if PHY is in DataMode */
	if (pDslModem->DslLinkState != GP_TRAINED){/*wally*/
		wpabs_debug_event("(Debug) !!!!!!!! GPISR: GpTransmitCells: GP NOT TRAINED!\n");
		return GP_FAIL;
	}
	
	/* Check for valid buffer address range */
	if(TxBuffEndP <= TxBuffStartP){
		wpabs_debug_event("(Debug): ERROR GpTransmitCells: TxBuffStartP Invalid!!!\n");
		return GP_FAIL;
	}

	/* Check if TX range is on a GP_ATM_CELL_SIZE Boundary */
	NumTxCellsThisBuffer = TxBuffEndP-TxBuffStartP;
	if(NumTxCellsThisBuffer%GP_ATM_CELL_SIZE){
		wpabs_debug_event("(Debug): ERROR GpTransmitCells: ATM CELL SIZE Boundary!!!\n");
		return GP_FAIL;
	}

	NumTxCellsThisBuffer /= GP_ATM_CELL_SIZE;	/*Determine no. of Tx cells in buffer */
	pDslModem->TotalNumTxCells += NumTxCellsThisBuffer; /* Update Tx cell count */

	if (pDslModem->DslLinkState == GP_TRAINED){/* wally */
		GpWritePciReg(pDslModem, PCORE_ADDR_TXSTARTADD, TxBuffStartP );
		GpWritePciReg(pDslModem, PCORE_ADDR_TXENDADD, TxBuffEndP );
		GpWritePciReg(pDslModem, PCORE_ADDR_TXCOMMAND, PCORE_INITIAL_TXCOMMAND | 0x00000001);
	}else{
		wpabs_debug_event("(Debug) !!!!!!!! CRITICAL GPISR: GP NOT TRAINED!\n");
		return GP_FAIL;
	}

	return GP_SUCCESS;
}

