
#include "Gp.h"
#include "GpInclude.h"

#define ATM_WATCHDOG_TIMEOUT 4

extern void adsl_arp_check (adapter_t *pAdapter);
extern int adsl_tx_oam_loopback_packet(adapter_t *pAdapter);

void adsl_tx_lockup_condition(adapter_t *pAdapter,PDSL_MODEM pDslModem);

void adsl_tx_lockup_condition(adapter_t *pAdapter,PDSL_MODEM pDslModem)
{
	PCI_PHY    *pPciPhy = &(pAdapter->pciPhy);
	PHY_DESCR *phy_descr;
	unsigned long flags;

	adsl_card_lock_irq(pDslModem->pCustData,&flags);
	
	wpabs_debug_tx("(DEBUG): adsl_lockup: Tx DMA = %d : IS EMPTY %d\n",
			pDslModem->TxDmaPending, 
			IS_LIST_EMPTY(&(pPciPhy->TxFreeList)));
	
	wpabs_clear_bit(0,&pDslModem->TxDmaPending);
	pDslModem->TxDmaTimer = 0;

	
	if ((phy_descr=pPciPhy->TxDescr) != NULL){

#if 1
		INSERT_HEAD_LIST(&(pPciPhy->TxPendingList),(PLIST_NODE)phy_descr);
#else
		((pkt_info_t*)phy_descr->Buffer->AllocatorContext)->length=0;
		((pkt_info_t*)phy_descr->Buffer->AllocatorContext)->chan=NULL;

		phy_descr->Buffer->Size=0;
		/* ALEX phy_descr->Buffer->DmaData.length=0;*/
		PHY_ADD_LIST(&(pPciPhy->TxFreeList), phy_descr);
		wpabs_debug_tx("Removing pending tx packet\n");
#endif
		pPciPhy->TxDescr = NULL;
	}

	GpDisableInterrupt(pDslModem);

	GpResetDma(pDslModem);

	pDslModem->InitStatus |= GP_INIT_RX_IDLE;
	wpabs_debug_tx("%s: Kickstarting TxDMA: rx idle cells\n",pAdapter->devname);
	WriteDspAdd(&pDslModem->XcvrInfo, GTI_STAR_A, 
			PCU_ADDR_RX_IDLE_CELLS, GTI_PSRAM0W);

	WriteStarW(&pDslModem->XcvrInfo, GTI_STAR_A, 
			(GP_BYTE)pDslModem->RxCellCount, 
			PCU_RX_ALL_IDLE_CELLS);
	
	GpEnableInterrupt(pDslModem);

	adsl_card_unlock_irq(pDslModem->pCustData,&flags);

	if (pAdapter->config.interface == LAN_INTERFACE){
		GpLanTxComplete(pAdapter, 0, -1);
	}
}

int adsl_disconnecting (adapter_t *pAdapter,PDSL_MODEM pDslModem)
{
	pAdapter->dspState = DspStateNotTrained;
	GpGetDataRate(pDslModem);
	GpResetTransceiver(pDslModem);
	
	pDslModem->Disconnect_Timeout=wpabs_get_systemticks();
	pDslModem->DslLinkState = GP_LINK_DISCONNECTING;

	return GP_LINK_DISCONNECTING;
}

int adsl_disconnected(adapter_t *pAdapter,PDSL_MODEM pDslModem)
{
	int err;
	pDslModem->TrainingIsrCnt = 0;
	pDslModem->TrainingIsrCntOrig = 0;
	pDslModem->TrainingTimer = 0;
	pDslModem->PendingCellDelTimer=0;
	pDslModem->TickCounter=0;
	pAdapter->atm_check=0;

	pDslModem->ExcessiveConsecutiveOverallFailureCount=0;
	pDslModem->CrcErrorsPerMin=0;

	pAdapter->dspState = DspStateNotTrained;
	GpGetDataRate(pDslModem);
	
	GpResetTransceiver(pDslModem);
	err=GpStartTraining(pDslModem);
	if (err == GP_FAIL){
		wpabs_debug_event("%s: Error: ADSL failed to start training 0x%X!\n",
				pAdapter->devname,err);
		return adsl_disconnecting(pAdapter,pDslModem);
	}else{
		wpabs_debug_event("%s: ADSL starting training 0x%X ...\n",
				pAdapter->devname,err);
	}


	return GP_LINK_DOWN;
}

/************************************************************************
* Function     : GpMonitorTransceiver
*
* Description  : This routine is called every .5 sec during training and in
*		 data mode. During trainig, a timeout is performed to reinitialize
*	     	 the transceiver if necessary.

	 In data mode:
	- Transceiver status if checked to see if a retrain is required.
	- SNR margin and line attenuation is updated periodically for use by the EOC.
	- The transmit DMA timer is updated and a timeout condition is checked.
*
* Input        :
*
* Output       : None
*
* Return       :GP_NO_CHANGE		// Link remains in previous state
		GP_LINK_DOWN		// Link just went down 
					   (link state change from GP_TRAINED to GP_TRAINING)
*		GP_LINK_UP		// Link just went up (link state change from 
*					   GP_TRAINING to GP_TRAINED)

*
* (C) Copyright 2001 by GlobeSpan Inc
*
* $Header: /usr/local/cvsroot/wanpipe_common/adsl/Epci/GpMonitorTransceiver.c,v 1.27 2004/01/15 16:44:07 sangoma Exp $ *
*                                                                      *
************************************************************************/
GP_WORD GpMonitorTransceiver( PDSL_MODEM pDslModem )
{
	GP_DWORD	j;
	GP_WORD		wValue;
	GP_BYTE		value;
	GP_WORD		returnValue;
	GP_WORD		ReturnStatus;
	unsigned long	flags;

	adapter_t *pAdapter=(adapter_t*)pDslModem->pCustData;

	ReturnStatus = GP_NO_CHANGE;

    	if(!pAdapter || pDslModem->InitStatus & GP_INIT_SHUTDOWN ){
		/* System is in the process of halting- return. */
        	return ReturnStatus;				
    	}

	if (pDslModem->DslLinkState == GP_LINK_DISCONNECTING){

		/* Turn Link Activity LED OFF */		
		GP_DISCONNECTING_LED;
		 
		/* Implement a 5 second delay before going into
		 * training. In the mean time, turn off TX and RX
		 * and make sure that the chip is ready for training. */
		if ((wpabs_get_systemticks() - pDslModem->Disconnect_Timeout) > (5*wpabs_get_hz())){	

			/* The retraing will be initiated by the above
			 * layer. Thus, once the timeout is reached,
			 * return LINK_DOWN. Once the above layer receives
			 * LINK_DOWN state, the training will comence*/

			pDslModem->RetrainCount++;	
			ReturnStatus = adsl_disconnected(pAdapter,pDslModem);
		}

		return ReturnStatus;
	}

   	if (pDslModem->DslLinkState == GP_TRAINED){

		/* Turn Link Activity LED ON */		
		GP_LINKACT_LED_ON;

		adsl_arp_check(pAdapter);

		if (pAdapter->atm_watchdog >= 0){

			adsl_tx_oam_loopback_packet(pAdapter);

			if (pAdapter->atm_check > ATM_WATCHDOG_TIMEOUT){
				if (pAdapter->atm_watchdog == 0){
					pAdapter->atm_watchdog = -1;
					wpabs_debug_event("%s: ADSL watchdog disabled: ATM OAM Not Supported!\n",
						pAdapter->devname);

				}else{
					wpabs_debug_event("%s: ADSL watchdog timed out! Retraining...\n",
						pAdapter->devname);
				
					ReturnStatus = adsl_disconnecting(pAdapter,pDslModem);
					return ReturnStatus;
				}
			}
		}
		
		/* check if ADSL data link is down or not */
		if (pAdapter->tx_collisions > MAX_TX_COLLISIONS){
			
			/* Turn off Rx and Tx */
			wpabs_debug_event("%s: Error: Maximum Tx collisions reached!\n",
					pAdapter->devname);

			adsl_tx_lockup_condition(pAdapter,pDslModem);

			/* Bug Fix: N.C. Aug 28 2002
			 * Reset the tx_collisions counter, since we are
			 * restarting again */
			pAdapter->tx_collisions=0;
		}

		
		adsl_card_lock_irq(pDslModem->pCustData,&flags);
  		returnValue = SStatusXCVR(&pDslModem->XcvrInfo, GTI_FAILURES,
       					  &pDslModem->Failures[0], 
					  sizeof(pDslModem->Failures), GTI_CH_A);
		
		
		if ((returnValue = SStatusXCVR(&pDslModem->XcvrInfo, GTI_COUNTERS, 
				 	       &pDslModem->Counters[0], 
					       sizeof(pDslModem->Counters), GTI_CH_A))){
			
			/* Accumulate each of the counters in the array */
			for(j=0;j<(sizeof(pDslModem->AccCounters)/sizeof(GP_WORD));j++)
			{
				pDslModem->AccCounters[j] += pDslModem->Counters[j];
			}
		
		}else{
			/* Clear counters */
			for(j=0;j<(sizeof(pDslModem->Counters)/sizeof(GP_WORD));j++){
				pDslModem->Counters[j] = 0;
			}
		}

		if (SStatusXCVR(&pDslModem->XcvrInfo, GTI_ATTENUATION_STATUS, 
				&wValue, GTI_ATTENUATION_STATUS_LEN, GTI_DSP_A) ){
			
				pDslModem->EocDataReg.lineAtten[0]  = (GTI_BYTE)(wValue & 0x00ff);
		}

		wpabs_debug_timer("Attenuation Status %d\n",
				pDslModem->EocDataReg.lineAtten[0]);

		
		if (pDslModem->SNRStatusRead > 0){		
		
			/* Check if SNR status has been read at least once since the
			// last startup. If so, call UpdateSNRStatus to read the SNR
			*/
			GpUpdateSNRStatus(pDslModem);	
			
			/* data in small blocks and store in a local array.
			// If not, call SStatusXCVR to read status
			// directly from the DSP.
			*/
			if ((pDslModem->DspAddH==0xD6) && (pDslModem->DspAddL==0)){
				
				/* Has the local status array been updated?
				// Yes - call SStatusXCVR to update the
				// SNR margin calculation.
				*/
				if( SStatusXCVR(&pDslModem->XcvrInfo, GTI_LCL_SNR_MARGIN, 
						&value, GTI_LCL_SNR_MARGIN_LEN, GTI_DSP_A)){
				
					pDslModem->EocDataReg.snrMargin[0]  = value;
				}
			}

		}else{
			if ( SStatusXCVR(&pDslModem->XcvrInfo, GTI_LCL_SNR_MARGIN, 
					 &value, GTI_LCL_SNR_MARGIN_LEN, GTI_DSP_A) ){
					
				pDslModem->EocDataReg.snrMargin[0]  = value;
			}
			pDslModem->SNRStatusRead = 1;	
			/* Indicate that SNR status has been completely 
			// read at least once
			*/
		}

		wpabs_debug_timer("SNR Margin %d\n",
				pDslModem->EocDataReg.snrMargin[0]);
		

		adsl_card_unlock_irq(pDslModem->pCustData,&flags);

		if(pDslModem->RetrainEnable == GP_TRUE){
		
			/* Check is retrain is enabled. */
			if (GpCheckRetrain(pDslModem) == GP_LINK_DISCONNECTING){	
				/* Check if a retrain is needed. */	 
				ReturnStatus = GP_LINK_DISCONNECTING;
				wpabs_debug_timer("Check Retraint Failed: Disconnecting\n");
				return ReturnStatus;
			}
		}

		if (wpabs_test_bit(0,&pDslModem->TxDmaPending)){
			
			if((wpabs_get_systemticks() - pDslModem->TxDmaTimer) > ((wpabs_get_hz()*2)/10)){
		
				wpabs_debug_tx("%s: TxDmaTimer timeout=%d, kick starting TX dma\n",
						pAdapter->devname,
						wpabs_get_systemticks() - pDslModem->TxDmaTimer);

				adsl_tx_lockup_condition(pAdapter,pDslModem);
			}
		}
	}

   	if(pDslModem->DslLinkState == GP_PENDING_CELLDEL){
	
		GP_CELL_DEL_LED;

		adsl_card_lock_irq(pDslModem->pCustData,&flags);
		SStatusXCVR(&pDslModem->XcvrInfo, GTI_COUNTERS, 
				 	       &pDslModem->Counters[0], 
					       sizeof(pDslModem->Counters), GTI_CH_A);		
		adsl_card_unlock_irq(pDslModem->pCustData,&flags);
	
		/* This link state is set by
		// GpTransceiverHandler (interrupt)
   		// once training is complete. The following
		// code waits for ATM cell deliniation.
		// If it does not occur within a timeout
		// interval, the transceiver is placed
		// back into retrain.
		// increment timer (in msec.) used to check for lock-up in PENDING_CELLDEL state
		*/	
		pDslModem->PendingCellDelTimer = pDslModem->PendingCellDelTimer + 
			                         pDslModem->GpMonitorInterval;

      		if (pDslModem->PendingCellDelTimer >= GP_CELL_DEL_TIMEOUT){

			wpabs_debug_event("%s: Error: Cell Delination timed out %d ncdi=%d ncdf%d!\n",
                                        pAdapter->devname,
					pDslModem->PendingCellDelTimer,
					pDslModem->Counters[GTI_LOCAL_NCDI_COUNTER],
					pDslModem->Counters[GTI_LOCAL_NCDF_COUNTER]);

	 		pDslModem->PendingCellDelTimer = 0;   /* reset timeout timer */

			pDslModem->Counters[GTI_LOCAL_NCDI_COUNTER]=0;
			pDslModem->Counters[GTI_LOCAL_NCDF_COUNTER]=0;

			ReturnStatus = adsl_disconnecting(pAdapter,pDslModem);
      		}else{
			/* Cell deliniation timeout has not occurred yet. 
			 * Check to see if we have acheived cell deliniation.
      			 */

			/* Has the framer now achieved cell deliniation? */
	               if ((pDslModem->Counters[GTI_LOCAL_NCDI_COUNTER]==0) && 
			   (pDslModem->Counters[GTI_LOCAL_NCDF_COUNTER]==0)){

				/* Yes - Initialize for data mode
				// UTOPIA CLAV FIX - CLAV pulse duration was decreased in PCU firmware
				// to a value that is incompatible with Panda & Pulsar. The following
				// 4 lines restore CLAV timing to the correct value.
				// Set IREG 0x82 = 0x30B4
				*/
				WriteStarB(&pDslModem->XcvrInfo, GTI_STAR_A, GTI_ADDL_DSP, 0x82); /* Reg. 0x82 */
	    			WriteStarB(&pDslModem->XcvrInfo, GTI_STAR_A, GTI_ADDH_DSP, 0x4);  /* IREG space */
				WriteStarB(&pDslModem->XcvrInfo, GTI_STAR_A, GTI_DATAL_DSP, 0xB4);/* Data = 0x30B4 */
				WriteStarB(&pDslModem->XcvrInfo, GTI_STAR_A, GTI_DATAH_DSP, 0x30);

				/* Turn off refout (Pulsar CAMHIT) to eliminate interfering 
				// tones in the receive band.
				// Specifically, clear bit 12 - refoe - DSP writes 0x52A4.
				*/	
				WriteStarB(&pDslModem->XcvrInfo, GTI_STAR_A, GTI_ADDL_DSP, 0x31); /* Reg. 0x31 */
	    			WriteStarB(&pDslModem->XcvrInfo, GTI_STAR_A, GTI_ADDH_DSP, 0x4);  /* IREG space */
				WriteStarB(&pDslModem->XcvrInfo, GTI_STAR_A, GTI_DATAL_DSP, 0xA4);/* Data = 0x42A4 */
				WriteStarB(&pDslModem->XcvrInfo, GTI_STAR_A, GTI_DATAH_DSP, 0x42);

				GpResetDma(pDslModem);		/* Init DMA */

				/* Tell the framer how many consecutive idle 
				// cells to send to receive DMA buffer.
				*/
				GpSetRxIdleCellCount(pDslModem);
				 
				/* Necessary in order to allow the
				// first Tx DMA transfer to begin.	
				*/
				wpabs_clear_bit(0,&pDslModem->TxDmaPending);
					
				/* This flag is used to ensure that SNR margin status is
				// read once directly from DSP following startup
				// before we start reading slowly in the background.
				*/
				pDslModem->SNRStatusRead = 0;	
				
				/* Init. DSP address for reading SNR status. */
				pDslModem->DspAddH = 0xD6;		
				
				pDslModem->DspAddL = 0;

				pDslModem->DslLinkState = GP_TRAINED;

				ReturnStatus = GP_LINK_UP;

				wpabs_debug_event("%s: Cell Delination successful\n",
                                        pAdapter->devname);

				/* Get the connected data rate from the DSP 
				// and store in DslModem structure.
				*/
				GpGetDataRate(pDslModem);	

                	} /* if PCU SYNC */
      		}
	}/* if PENDING_CELLDEL */

   
	if(pDslModem->DslLinkState == GP_TRAINING){
		
		/* Turn all LEDS in circles */		
		GP_TRAINING_LED;

		/* increment timer (in msec.) used to check for 
		// lock-up in PENDING_CELLDEL state
	   	*/	
		pDslModem->TrainingTimer = pDslModem->TrainingTimer + 
					   pDslModem->GpMonitorInterval;

		if (pDslModem->TrainingTimer == pDslModem->GpMonitorInterval){
			pDslModem->TrainingIsrCntOrig=pDslModem->TrainingIsrCnt=0;
		}else{
		
			if (((pDslModem->TrainingTimer/pDslModem->GpMonitorInterval) % 10) == 0){

				if (!(pDslModem->TrainingIsrCnt - pDslModem->TrainingIsrCntOrig) ||
				     pDslModem->TrainingTimer > GP_TRAINING_TIMEOUT){ 
					
					wpabs_debug_event("%s: Train timeout, Restarting training!\n",
							pAdapter->devname);


					{
						unsigned short opstate=0;

						adsl_card_lock_irq(pDslModem->pCustData,&flags);

						SStatusXCVR(&pDslModem->XcvrInfo, GTI_OP_STATE, 
								&opstate,sizeof(opstate), GTI_CH_A);

						adsl_card_unlock_irq(pDslModem->pCustData,&flags);

						
						wpabs_debug_event("%s: DEBUG: Opstate 0x%X Cnt %d Timer %d\n",
								pAdapter->devname,opstate,
								pDslModem->TrainingIsrCnt,
								pDslModem->TrainingTimer);
					}

					ReturnStatus = adsl_disconnecting(pAdapter,pDslModem);
				}

				pDslModem->TrainingIsrCntOrig=pDslModem->TrainingIsrCnt;
			}
		}
   	}

   	return ReturnStatus;

} 
/*-----------------------------End GpMonitorTransceiver--------------------------------*/

/************************************************************************
* Function     : GpCheckretrain
*
* Description  : Check to see if a retrain is indicated and initiate if required.


	 	There are currently three criteria defined for starting a retrain
	 	because of excessive CRC errors (AN-029; Rev 1.0; Sept 12, 2000):

		    1. Overall failures !=0

	 	2. "Short term" : 25 or more crc errors per second for 3 consecutive
				 					  seconds.
		 	3. "Long term": error rate per minute exceeeds the number of bytes per
				DMT symbol for 5 consecutive minutes.

				 	All calculations and thresholds in the code below are derived from
				 	defines in DSP.H and registry keys that can override them. They are
                    then adjusted to use the period of the DSP_STATUS_CHECK (a.k.a "tick"
				 	or "dsp_status_tick") as the time base.

				    Assume only one of Fast or Interleaved CRC errors can be nonzero at
                    one time.

                    NOTE: Roll over protection of the instance per minute crc error
                    count is not provided on purpose at this time. Being that it is
                    a 16 bit counter we don't expect this to happen without triggering
                    OVERALL_FAILURES or at least having excess per tick CRC errors.
                    Any of the latter two will cause the retrain anyway. Also, the
                    Counters of the DSP hw/fw itself are 16 bit with no rollover
                    protection.

*
* Input        :
*
* Output       : None
* Return       : None
*
* (C) Copyright 2001 by GlobeSpan Inc
*
************************************************************************/

GP_WORD GpCheckRetrain(PDSL_MODEM pDslModem)
{

	adapter_t *pAdapter=(adapter_t*)pDslModem->pCustData;
	GP_WORD	crcErrCount;
	GP_BYTE	ReTrainFlag = GP_FALSE;
	GP_WORD ReturnStatus = GP_NO_CHANGE;

	if (pDslModem->Failures[GTI_OVERALL_FAILURE] != 0){

		wpabs_debug_timer("%s: Overall failure = %d \n",
				pAdapter->devname,pDslModem->Failures[GTI_OVERALL_FAILURE]);

		if (++pDslModem->ExcessiveConsecutiveOverallFailureCount >= GSI_CONSEC_OVERALL_FAILURE){
		
			wpabs_debug_event("%s: Overall failure = %d, %d consec, time to retrain!\n",
					pAdapter->devname,
					pDslModem->Failures[GTI_OVERALL_FAILURE],
					GSI_CONSEC_OVERALL_FAILURE);

			ReTrainFlag = GP_TRUE;
		}else{
			return ReturnStatus;
		}
	
	}else{

		pDslModem->ExcessiveConsecutiveOverallFailureCount=0;
		
		crcErrCount = pDslModem->Counters[GTI_LOCAL_CRSI_COUNTER] + 
			      pDslModem->Counters[GTI_LOCAL_CRSF_COUNTER];
		
		pDslModem->CrcErrorsPerMin += crcErrCount;

		if(crcErrCount){
			wpabs_debug_timer("%s: TickCounter = %d, CrcErrorsThisTick = %d, CrcErrorsPerMin = %d\n", 
				pAdapter->devname,
				pDslModem->TickCounter, 
				crcErrCount, 
				pDslModem->CrcErrorsPerMin);
		}

		/* Counter of GSIDSL_CHECK_TRAIN_TIMEOUT ticks (0.5 sec) */
		pDslModem->TickCounter++;

		/* Check for excessive CRC errors per tick (closest to per sec) */
		if (crcErrCount >= ConvertPerSecondCountToPerTickCount(pDslModem->MaxCrcErrorsPerSecThreshold))
		{
			pDslModem->ExcessiveConsecutiveCrcErrorsPerTickCount++;

			wpabs_debug_timer("%s: Excessive CRC Errors per DSP Status Tick (%d) detected!\n", 
					pAdapter->devname,crcErrCount);

			wpabs_debug_timer("%s: Consecutive Times = %d\n",
					pAdapter->devname,
					pDslModem->ExcessiveConsecutiveCrcErrorsPerTickCount);

			/* Have we reached the consecutive times needed for a retrain already ? */
			if(pDslModem->ExcessiveConsecutiveCrcErrorsPerTickCount >= 
					TICKS_TILL_RETRAIN_ON_EXCESS_CRC_ERRORS_PER_TICK)
			{
				ReTrainFlag = GP_TRUE;

				wpabs_debug_event("%s: Excessive CRC errors per tick detected %d consec. ! Retraining...\n",
					pAdapter->devname,
					pDslModem->ExcessiveConsecutiveCrcErrorsPerTickCount);

			}
		}
		else /* if (crcErrCount >= CRC_ERRORS_PER_DSP_STATUS_TICK_THRESHOLD) */
		{
			/* Since this tick there weren't excessive errors, clear the consecutive count. */
			pDslModem->ExcessiveConsecutiveCrcErrorsPerTickCount = 0;
		} /* else (crcErrCount >= CRC_ERRORS_PER_DSP_STATUS_TICK_THRESHOLD) */

		/* Has 1 minute elapsed ? */
		if (pDslModem->TickCounter >= DSP_STATUS_TICKS_PER_MIN)
		{
			/* Calculate threshold for error rate per minute which is just equal to the number
			 * of bytes per DMT symbol (Default for DownstreamRateToMaxCrcErrorsPerMinDivider
			 * is 32 and provides this computation, but can be overwritten from the registry).
			 */
			pDslModem->CrcErrorsPerMinThreshold = 
				(GP_WORD) (pDslModem->DownstreamRate / 
					   pDslModem->DownstreamRateToMaxCrcErrorsPerMinDivider);

			wpabs_debug_timer("%s: DownstreamRate = %ld, CrcErrorsPerMinThreshold = %d\n",
				pAdapter->devname, pDslModem->DownstreamRate, pDslModem->CrcErrorsPerMinThreshold);


			/* Check for excessive CRC errors per min */
			if(pDslModem->CrcErrorsPerMin >= pDslModem->CrcErrorsPerMinThreshold)
			{
				pDslModem->ExcessiveConsecutiveCrcErrorsPerMinCount++;

				wpabs_debug_timer("%s: Excessive CRC Errors per Min (%d) detected!\n",
						pAdapter->devname, pDslModem->CrcErrorsPerMin);
				wpabs_debug_timer("%s: Consecutive Times = %d\n",
						pAdapter->devname,
						pDslModem->ExcessiveConsecutiveCrcErrorsPerMinCount);
				
				/* Have we reached the consecutive times needed for a retrain already ? */
				if(pDslModem->ExcessiveConsecutiveCrcErrorsPerMinCount >= \
					MINS_TILL_RETRAIN_ON_EXCESS_CRC_ERRORS_PER_MIN)
				{
					ReTrainFlag = GP_TRUE;

					wpabs_debug_event("%s: Excessive CRC errors per Min detected %d consec. ! Retraining...\n",
						  pAdapter->devname,
						  pDslModem->ExcessiveConsecutiveCrcErrorsPerMinCount);

				}
			}else{
				/* Since this minute there weren't excessive 
				 * errors, clear the consecutive count. */
				pDslModem->ExcessiveConsecutiveCrcErrorsPerMinCount = 0;
			}

			/* Reset Counters so that we start keeping track of the next minute. */
			pDslModem->CrcErrorsPerMin = pDslModem->TickCounter = 0;
		}
	}

	if(ReTrainFlag == GP_TRUE){
		/* Do not start retraining just yet!
		 * Just return the LINK DISCONNECTING Status and let
		 * the upper layer start retraining */
		/* Turn off Rx and Tx */
		ReturnStatus = adsl_disconnecting(pAdapter,pDslModem);
		ReTrainFlag = GP_FALSE;
	}

	return ReturnStatus;

}		/* End GpCheckRetrain */

