/************************************************************************
 * FILE NAME   : GpXcvr.c						*
 *									*
 * DESCRIPTION : This file contains routines used to control the	*
 *               transceiver.						*
 *                                                                      *
 *                                                                      *
 * (C) Copyright 2001 by GlobeSpan Inc					*
 *                                                                      *
 * $Header: /usr/local/cvsroot/wanpipe_common/adsl/Epci/GpXcvr.c,v 1.11 2003/10/30 15:46:21 sangoma Exp $ *
 *                                                                      *
 ************************************************************************/

#include <Gp.h>
#include "GpInclude.h"

             
/************************************************************************
* Function     : GpInitialize
* Description  : This is the main initialization routine. PCI registers,
*				 the transceiver, and certain variables are initialized.
*
* Input        : pDslModem - A pointer to the DslModem information structure.
* Output       : None
* Return       : GP_SUCCESS  - Initialization successful
*              : GP_FAIL - Initialization failed
************************************************************************/
GP_WORD GpInitialize( PDSL_MODEM pDslModem )
{
	GP_WORD result;

	if(pDslModem==GP_NULL)
	{
		#if GP_DBG
		GpDbgPrint ("\nGpInitialize: pDslModem=GP_NULL!\n" );
		#endif
		wpabs_debug_event("\nGpInitialize: pDslModem=GP_NULL!\n" );

		return GP_FAIL;
	}

	/* Store pointer to the modem info.structure
	// into the transceiver info. customer info.
	// structure. This is done so that the transceiver
	// I/O routines can get access to the device base
	// address which is stored in the modem info.
	// structure.
	*/
    	pDslModem->XcvrInfo.CustXcvrInfo.pDslModem = pDslModem;

	/* Initilize the InitStatus Flags */
	pDslModem->InitStatus = GP_INIT_CLEAR;

	/*
	// Check parameters set by customer's firmware
	*/
	if( pDslModem->RxBuffStartV >= pDslModem->RxBuffEndV)
	{
		wpabs_debug_event("\nGpInitialize: RxBuffStartV >= RxBuffEndV!\n" );
		return GP_FAIL;
	}
	if( pDslModem->RxBuffStartP >= pDslModem->RxBuffEndP)
	{
		wpabs_debug_event("\nGpInitialize: RxBuffStartP >= RxBuffEndP!\n" );
		return GP_FAIL;
	}
	if( pDslModem->RxBuffStartV == 0)
	{
		wpabs_debug_event("\nGpInitialize: RxBuffStartV = 0!\n" );
		return GP_FAIL;
	}
	if( pDslModem->RxBuffStartP == 0)
	{
		wpabs_debug_event("\nGpInitialize: RxBuffStartP = 0!\n" );
		return GP_FAIL;
	}
	if( pDslModem->RxBuffEndV == 0)
	{
		wpabs_debug_event("\nGpInitialize: RxBuffEndV = 0!\n" );
		return GP_FAIL;
	}
	if( pDslModem->RxBuffEndP == 0)
	{
		wpabs_debug_event("\nGpInitialize: RxBuffEndP = 0!\n" );
		return GP_FAIL;
	}
	if( pDslModem->GpMonitorInterval == 0)
	{
		wpabs_debug_event("\nGpInitialize: GpMonitorInterval = 0!\n" );
		return GP_FAIL;
	}

	/* If not defined, setup default values. */
	if( pDslModem->RxCellCount == 0)
	{
		pDslModem->RxCellCount = GP_DEFAULT_RX_CELL_CNT;
	}
	if( pDslModem->MaxCrcErrorsPerSecThreshold == 0)
	{
		pDslModem->MaxCrcErrorsPerSecThreshold = DEFAULT_MAX_CRC_ERRORS_PER_SEC_THRESHOLD;
	}
	if( pDslModem->DownstreamRateToMaxCrcErrorsPerMinDivider == 0)
	{
		pDslModem->DownstreamRateToMaxCrcErrorsPerMinDivider = DEFAULT_DOWNSTREAM_RATE_TO_MAX_CRC_ERRORS_PER_MIN_DIVIDER;
	}

	/* Check Hardware 
	// THere is no point of doing this here, only after XCVR !!!
	*/
	result = GpCheckHardware(pDslModem);
	if(result == GP_FAIL)
	{
		#if GP_DBG
		GpDbgPrint ("\nGpInitialize: GpCheckHardware Failed!\n" );
		#endif
		wpabs_debug_event("\nGpInitialize: GpCheckHardware Failed!\n" );

		return GP_FAIL;
	}

	/* NENAD RESET DSP */
	ResetDSP(&pDslModem->XcvrInfo,GTI_DSP_ALL);
	
	/* Initialize device registers */
	GpInitRegisters(pDslModem);

	/* Initialize the transceiver */
	InitXCVR_CS(&pDslModem->XcvrInfo, 0, GTI_DSP_A_MSK, 0, 0, 0, 0);
	InitXCVR(&pDslModem->XcvrInfo);

/*	printk(KERN_INFO "(Debug): Check Hardware Take 2\n");
//	GpCheckHardware(pDslModem);
*/
	
	InitSTARInterrupt(&pDslModem->XcvrInfo, GTI_TRUE);
	ClearSTARInterrupt(&pDslModem->XcvrInfo, GTI_DSP_ALL);

	GpInitParameters(pDslModem);		/* Initial the transceiver parameter array.*/
	GpInitEoc(pDslModem);			/* Initialize EOC data registers. */
	GpSetRxIdleCellCount(pDslModem);

	pDslModem->RxCellRdPtrSav = pDslModem->RxBuffStartV;	/* Initialize last RxCell processed */
	pDslModem->TxDmaPending = GP_FALSE;
	pDslModem->TotalNumRxCells = 0;				/* Clear status counters */
	pDslModem->TotalNumTxCells = 0;
	pDslModem->RxDmaErrCnt = 0;
	pDslModem->RetrainCount = 0;
	pDslModem->PendingCellDelTimer = 0;
	pDslModem->TrainingTimer = 0;
	pDslModem->TxDmaTimer = 0;

	pDslModem->SNRStatusRead = 0;
	pDslModem->DslLinkState = GP_NOT_TRAINED;

	pDslModem->InitStatus |= GP_INIT_HARDWARE;

	return GP_SUCCESS;
}


/************************************************************************
* Function     : GpInitEoc
*
* Description  : Initialiaze EOC Data Registers
*
* Input        : pDslModem _ A pointer to the DslModem information structure.
* Output       : None
* Return       : None
************************************************************************/

GP_VOID GpInitEoc(PDSL_MODEM pDslModem)
{
	GTI_DWORD j;

	/* Clear EocCmd */
	for(j=0;j<sizeof(pDslModem->EocCmd);j++)
	{
		*(((GTI_BYTE*)(&pDslModem->EocCmd))+j) = 0;
	}

	/* Clear EocDataReg */
	for(j=0;j<sizeof(pDslModem->EocDataReg);j++)
	{
		*(((GTI_BYTE*)(&pDslModem->EocDataReg))+j) = 0;
	}

    	pDslModem->EocDataReg.vendorId[0] = 0x00;
	pDslModem->EocDataReg.vendorId[1] = GP_EOC_VENDOR_ID;
	pDslModem->EocDataReg.vendorId[2] = 0x00;
	pDslModem->EocDataReg.vendorId[3] = 0x00;
	pDslModem->EocDataReg.vendorId[4] = 0x00;
	pDslModem->EocDataReg.vendorId[5] = 0x00;

	/* revision number init */
    	for(j = 0; j < GTI_EOC_REV_NUM_LEN; j++)
        	pDslModem->EocDataReg.revisionNum[ j ] = 0;

	/* serial number init */
    	for(j = 0; j < GTI_EOC_SERIAL_LEN; j++ )
        	pDslModem->EocDataReg.serialNum[j] = j % (GTI_EOC_SERIAL_LEN - 1);

    	pDslModem->EocDataReg.lineAtten[ 0 ] = 2;

	pDslModem->EocDataReg.snrMargin[ 0 ]  = 5;
}

/************************************************************************
* Function     : GpCheckHardware
*
* Description  : This routine verifies the ability to read-modify-write
*				 registers in the PCI device and the DSP host interface.
*
*
* Input        : pDslModem _ A pointer to the DslModem information structure.
* Output       : None
* Return       : GP_SUCCESS  - hardware check is successful
*              : GP_FAIL - hardware check failed
************************************************************************/
GP_WORD GpCheckHardware(PDSL_MODEM pDslModem)
{
	pDslModem->TestIODriverResult=Gti_TestIODrivers(&pDslModem->XcvrInfo); /* Test DSP Interface */
	#if GP_DBG
		GpDbgPrint("GpCheckHardware:  Gti_TestIODrivers, result = %x\n",pDslModem->TestIODriverResult);
	#endif
	/*pDslModem->TestIODriverResult=GTI_COMPLETE_PASS; // Test DSP Interface */

	/*NENAD Uncomment*/
	/*f( pDslModem->TestIODriverResult == GTI_COMPLETE_PASS){*/
		return GP_SUCCESS;
/*	}else{
//		printk(KERN_INFO "(Debug): pCheckHardware:  Gti_TestIODrivers, result = %x\n",
					pDslModem->TestIODriverResult);
//		return GP_FAIL;
//	}
*/
}

/************************************************************************
* Function     : GpResetTransceiver
* Description  : Reset the transceiver to prepare it for retraining
*
* Input        : pDslModem - A pointer to the DslModem information structure.
*
* Output       : None
* Return       : None
************************************************************************/
GP_VOID GpResetTransceiver(PDSL_MODEM pDslModem)
{
	GP_DWORD j;
	unsigned long flags;

	/* Clear DspCounters */
	for(j=0;j<(sizeof(pDslModem->DspCounters)/sizeof(GP_WORD));j++)
	{
		pDslModem->DspCounters[j] = 0;
	}

	pDslModem->DslLinkState = GP_NOT_TRAINED;
	pDslModem->TxDmaPending = GP_FALSE;

	/* reset timer accumulates */
	pDslModem->TxDmaTimer = 0;
	pDslModem->TrainingTimer = 0;
	pDslModem->PendingCellDelTimer = 0;
	pDslModem->RetrainCount = 0;

	/* Init - Re-train Criteria Spec AN-029 */
	pDslModem->TickCounter = 0;
	pDslModem->CrcErrorsPerMin = 0;
	pDslModem->ExcessiveConsecutiveCrcErrorsPerTickCount = 0;
	pDslModem->ExcessiveConsecutiveCrcErrorsPerMinCount = 0;

#if 0
	if (pDslModem->XcvrInfo.state == GTI_ACTIVE){
		GpCmdTransceiver(pDslModem, GTI_ABORT_REQ);
		GpDelay(50);
	}
#endif
	
	GpCmdTransceiver(pDslModem, GTI_ABORT_REQ);

	adsl_card_lock_irq(pDslModem->pCustData, &flags);
	
	GpDisableInterrupt(pDslModem);
	GpDelay(50);
	
/*	GpCmdTransceiver(pDslModem, GTI_DISCONNECT_REQ);*/
/*	GpDisableInterrupt(pDslModem);*/

	/* Turn off transmit and receive DMA */
	GpWritePciReg(pDslModem,PCORE_ADDR_GLUERESETIMAGE,1);

	/* Reset the hardware */
	GpWritePciReg(pDslModem, PCORE_ADDR_GLUERESETIMAGE, GP_NOVA_RESET_MASK );
	GpDelay(50);

	InitXCVR_CS(&pDslModem->XcvrInfo, 0, GTI_DSP_A_MSK, 0, 0, 0, 0);
	InitXCVR(&pDslModem->XcvrInfo);

	InitSTARInterrupt(&pDslModem->XcvrInfo, GTI_TRUE);

	ClearSTARInterrupt(&pDslModem->XcvrInfo, GTI_DSP_ALL);

	GpEnableInterrupt(pDslModem);

	adsl_card_unlock_irq(pDslModem->pCustData, &flags);

}


/************************************************************************
* Function     : GpGetDataRate
* Description  : Reads information from the transceiver to determine the
*		 data rate (not line rate). Should be called once following startup or retrain.
*
* Input        :
*
* Output       : None
* Return       : None
************************************************************************/
GP_DWORD GpGetDataRate( PDSL_MODEM pDslModem )
{

	GP_DWORD returnValue;
	unsigned long flags;

	if(pDslModem->DslLinkState == GP_TRAINED)
	{
		GP_WORD actualConfig[GTI_ACTUAL_CONFIGURATION_LEN/sizeof(GP_WORD)];

		/* Read Actual configuration info. from transceiver */
		adsl_card_lock_irq(pDslModem->pCustData,&flags);
		returnValue = SStatusXCVR(&pDslModem->XcvrInfo, GTI_ACTUAL_CONFIGURATION,
			&actualConfig[0],
			sizeof(actualConfig),
			GTI_CH_A);
		adsl_card_unlock_irq(pDslModem->pCustData,&flags);

		pDslModem->DownstreamRate = (GP_WORD)(actualConfig[2] * 32);
		pDslModem->UpstreamRate	  = (GP_WORD)(actualConfig[6] * 32);
	}
	else
	{
		pDslModem->DownstreamRate = 0;
		pDslModem->UpstreamRate = 0;
		returnValue = 0;
	}

	return returnValue;
}

/************************************************************************
* Function     : GpStartTraining
* Description  :
*
* Input        :
*
* Output       : None
* Return       : None
************************************************************************/
GP_DWORD GpStartTraining( PDSL_MODEM pDslModem )
{
	GP_DWORD result = GP_FAIL;

	if(pDslModem->DslLinkState == GP_NOT_TRAINED && 
	   !(pDslModem->InitStatus & GP_INIT_SHUTDOWN)){

		pDslModem->DslLinkState = GP_TRAINING;
		result = GpCmdTransceiver(pDslModem, GTI_STARTUP_REQ);
		pDslModem->TrainingTimer = 0;
	}

	return result;
}


/************************************************************************
* Function     : GpShutdown
* Description  : Called when system knows it is powering down.  Sends
*                dying gasp eoc messages, then disables interrupts and
*				 resets the transceiver.
*		   ;
* Input        : PDSL_MODEM pDslModem
* Output       : None
* Return       : None
************************************************************************/
GP_VOID GpShutdown(PDSL_MODEM pDslModem)
{
		
	if( (pDslModem->InitStatus&GP_INIT_HARDWARE)&&
	     (!(pDslModem->InitStatus&GP_INIT_DYING_GASP))){
		
		/* Enable Interrupt for Sending Dying Gasp */
		GpEnableInterrupt(pDslModem);

		/* Send Dying Gasp */
		GpSendDyingGasp(pDslModem);

		/* Allow EOC Interrupt to occur */
		GpDelay(GP_EOC_DYING_GASP_DELAY);
	}
	
	/* Place DSP in Idle Mode */
	GpResetTransceiver(pDslModem);

	/* Disable Interrupts */
	GpDisableInterrupt(pDslModem);

	pDslModem->InitStatus |= GP_INIT_SHUTDOWN;

}


/************************************************************************
* Function     : GpSendDyingGasp
* Description  : Called when system knows it is powering down.  Sends
*                6 consecutive dying gasp eoc messages.
*	       ; Timing depends on delay in calling routine before resetting the transceiver
* Input        : PDSL_MODEM pDslModem
* Output       : None
* Return       : None
************************************************************************/
GP_VOID GpSendDyingGasp(PDSL_MODEM pDslModem)
{
	GP_DWORD j;

#if defined(PRE_FW_P5)
	/* FW versions Earlier than P5 send EOC CMDS via GTIEOC_CMD.tx.AutoData */
	for (i=0; i < NUM_DYING_GASP_MESSAGES; i++)
		pDslModem->EocCmd.tx.AutoData[i] = DYING_GASP_EOC_MESSAGE;
	pDslModem->EocCmd.tx.AutoLen = NUM_DYING_GASP_MESSAGES;
#else
	/* FW versions P5 and beyond send EOC CMDS via GTIEOC_CMD.tx.EocCmd
	// Clear EocCmd
	*/
	for(j=0;j<sizeof(pDslModem->EocCmd);j++)
	{
		*(((GTI_BYTE*)(&pDslModem->EocCmd))+j)=0;
	}
	pDslModem->EocCmd.tx.EocCmd = GTI_EOC_CMD_DGASP;
#endif

	GTIEocProc(&pDslModem->XcvrInfo, &pDslModem->EocCmd);

	pDslModem->InitStatus |= GP_INIT_DYING_GASP;
}

/************************************************************************
* Function     : GpUpdateSNRStatus
*
* Description  : This function is called every 0.5 sec to read SNR margin data
*		 (SQ and bit loading) in small blocks (8 to 16 words) and to store
*		 the data in a local array. Once all of the data has been
*		 read, SStatusXCVR is called to calculate SNR margin using
*		 the stored data. This is done because reading all of the
*		 data directly from the DSP in one shot takes too long (10s of msec)
*		 and results in the ping delay problem.
*		 The blocks are split up so that each one takes no more than .5 msec
*		 on a 500 Mhz Pentium.
*
* Input        : pDslModem - A pointer to our adapter information structure.
*
*
* Output       : none
* Return       : GP_VOID
************************************************************************/

GP_VOID GpUpdateSNRStatus(PDSL_MODEM pDslModem )
{
	GP_DWORD i;
	GP_BYTE datal, datah, datae;

	/* DbgPrint("UpdateSNRStatus, AddH = %d, AddL = %d\n",pDslModem->DspAddH, pDslModem->DspAddL);*/

	switch (pDslModem->DspAddH)
	{
	case 0xD6:
		/* Read SQ data */
		GpWritePciReg(pDslModem, DSP_CFG, GTI_AP|GTI_BUS_MODE|GTI_INT_MODE);/* Set access mode for DSP read */
		GpWritePciReg(pDslModem, DSP_ADDL, pDslModem->DspAddL);	/* Set DSP RAM address */
		GpWritePciReg(pDslModem, DSP_ADDH, pDslModem->DspAddH);	/* for start of next block */
		for (i=0; i<16; i++)
		{

			GpDelay(15);	/* Delay 15 usec to avoid causing problems for the FDP */
			GpReadPciReg(pDslModem, DSP_DATAL, &datal);
			GpReadPciReg(pDslModem, DSP_DATAH, &datah);
			/* DbgPrint("               , DataH= %d, DataL= %d\n",datah, datal);*/

			pDslModem->DspRamD6_L[pDslModem->DspAddL] = datal;
			pDslModem->DspRamD6_H[pDslModem->DspAddL] = datah;
			pDslModem->DspAddL++;
		}
		if (pDslModem->DspAddL >= 255)
		{
			pDslModem->DspAddH = 0xD7;	/* Point to next block*/
			pDslModem->DspAddL = 0;
		}
		break;

	case 0xD7:
		/* Read DS bit loading data */
		GpWritePciReg(pDslModem, DSP_CFG, GTI_NO_AI|GTI_BUS_MODE|GTI_INT_MODE); /* Set RAM access mode */

		for (i=0; i<8; i++)
		{
			GpDelay(15);	/* Delay to avoid causing problems for the FDP */
			GpWritePciReg(pDslModem, DSP_ADDL, pDslModem->DspAddL);	/* Set DSP RAM address */
			GpWritePciReg(pDslModem, DSP_ADDH, pDslModem->DspAddH);	/* Note: autoinc. mode cannot be used for */
			/* bit loading RAM - it causes problems for 	*/
			/* the FDP.					*/
			GpReadPciReg(pDslModem, DSP_DATAE, &datae);
			GpReadPciReg(pDslModem, DSP_DATAL, &datal);
			GpReadPciReg(pDslModem, DSP_DATAH, &datah);
			/*DbgPrint("               , DataE = %d, DataH= %d, DataL= %d\n",datae, datah, datal); */

			pDslModem->DspRamD7_E[pDslModem->DspAddL] = datae;
			pDslModem->DspRamD7_L[pDslModem->DspAddL] = datal;
			pDslModem->DspRamD7_H[pDslModem->DspAddL] = datah;

			pDslModem->DspAddL++;
		}
		if (pDslModem->DspAddL >= 255)
		{
			pDslModem->DspAddH = 0xF4;
			pDslModem->DspAddL = 0xC0;
		}
		break;

	case 0xF4:
		/* Read US bit loading data */
		GpWritePciReg(pDslModem, DSP_CFG, GTI_NO_AI|GTI_BUS_MODE|GTI_INT_MODE);

		for (i=0; i<8; i++)
		{
			GpDelay(15);	/* Delay to avoid causing problems for the FDP */
			GpWritePciReg(pDslModem, DSP_ADDL, pDslModem->DspAddL);	/* Set DSP RAM address */
			GpWritePciReg(pDslModem, DSP_ADDH, pDslModem->DspAddH);

			GpReadPciReg(pDslModem, DSP_DATAE, &datae);
			GpReadPciReg(pDslModem, DSP_DATAL, &datal);
			GpReadPciReg(pDslModem, DSP_DATAH, &datah);
			/*DbgPrint("               , DataE = %d, DataH= %d, DataL= %d\n",datae, datah, datal); */

			pDslModem->DspRamF4_E[pDslModem->DspAddL] = datae;
			pDslModem->DspRamF4_L[pDslModem->DspAddL] = datal;
			pDslModem->DspRamF4_H[pDslModem->DspAddL] = datah;
			pDslModem->DspAddL++;
		}
		if (pDslModem->DspAddL >= 255)
		{
			pDslModem->DspAddH = 0xF5;	/* Note: this is a dummy value and is used as a way to allow */
			/* SStatusXcvr to run in it's own time slot. */
			pDslModem->SNRStatusRead = 2;	/* Indicate that the local SNR array has been filled at least once. */
		}
		break;

	case 0xF5:
		pDslModem->DspAddH = 0xD6;
		pDslModem->DspAddL = 0x00;	/* Do nothing - this .5 sec time slot is used to run SStatusXcvr */

	default:
		break;
	}
}

/************************************************************************
* Function     : GpSetIdleCellCount
*
* Description  : Set the number of continuous receive Idle cells that will be sent
*		 by the transceiver to the receive DMA buffer. Once this number has
*		 been exceeded, further idle cells will not be sent and receive
*		 DMA interrupts will cease until the next non-idle cell is received.
*		 This parameter is written to internal transceiver memory.
*		 IdleCellCount memory location. Max IdleCellCount is 255.
*		 LOBYTE = IdleCellCount (0=Disable, N=Idle Cells to send)
*		 HIBYTE = Modes (0=IdleCellCount, 1=Send All Idle Cells)
*
* Output       : GP_VOID
* Return       : GP_VOID
************************************************************************/
GP_VOID GpSetRxIdleCellCount(PDSL_MODEM pDslModem)
{
	WriteDspAdd(&pDslModem->XcvrInfo, GTI_STAR_A, PCU_ADDR_RX_IDLE_CELLS, GTI_PSRAM0W);
	WriteStarW(&pDslModem->XcvrInfo, GTI_STAR_A, (GP_BYTE)pDslModem->RxCellCount, PCU_RX_CNT_IDLE_CELLS);
}

/************************************************************************
* Function     : GpCmdTransceiver
* Description  : Calls SetParamXCVR and ExcuteXCVR with an action to be performed
*
* Input        : pDslModem: A pointer the modem information structure.
*              : action: Type of Parameter to set
*
* Output       : None
* Return       : return the result from the ExecuteXCVR()
************************************************************************/
GP_DWORD GpCmdTransceiver(
                   PDSL_MODEM pDslModem,
                   GP_BYTE action)
{
   GP_WORD result;
   unsigned long flags;

   pDslModem->XcvrParam[0].item[GTI_ACTION_ITEM] = action;

   wpabs_debug_event("\n");

   wpabs_debug_event("SetParamXCVR Action %d State %d\n",
		     action,pDslModem->XcvrInfo.state);
  
   adsl_card_lock_irq(pDslModem->pCustData, &flags);
   
   result = SetParamXCVR(&pDslModem->XcvrInfo, &pDslModem->XcvrParam[0]);

   wpabs_debug_event("SetParamXCVR Action %d State %d Rc=%d\n",
		     action,pDslModem->XcvrInfo.state,result);
   
   result = ExecuteXCVR(&pDslModem->XcvrInfo);

   adsl_card_unlock_irq(pDslModem->pCustData, &flags);

   wpabs_debug_event("ExecuteXCVR State %d Rc=%d\n",
		     pDslModem->XcvrInfo.state,result);
   
   /*result = GP_TRUE; */
   return result;
}


/************************************************************************
* Function     : GpGetStatusXcvr
* Description  : Call StatusXCVR and retrieve the actual information from XCVR
*
* Input        : pDslModem: A pointer to the DslModem information structure.
*                opCode: Type of status needed from the XCVR
* Output       : results are placed in pDslModem->StatusBuffer[] array
* Return       : WORD: no. of bytes returned (0 if failed)
************************************************************************/
GP_WORD GpGetStatusXcvr(
                       PDSL_MODEM pDslModem,
                       GP_BYTE opCode)
{
	GP_WORD result=GP_FAIL;
	GP_DWORD j;
	unsigned long flags;

	switch(opCode)
	{
	   case GTI_COUNTERS:
		   for(j=0;j<(sizeof(pDslModem->DspCounters)/sizeof(GP_WORD));j++)
		   {
			   pDslModem->StatusBuffer[j] = pDslModem->DspCounters[j];
			   pDslModem->DspCounters[j] = 0;
		   }
		   result=GP_SUCCESS;
		   break;

	   default:
			adsl_card_lock_irq(pDslModem->pCustData,&flags);
			result = SStatusXCVR(&pDslModem->XcvrInfo, opCode, &pDslModem->StatusBuffer[0], sizeof(pDslModem->StatusBuffer), 0);
			adsl_card_unlock_irq(pDslModem->pCustData,&flags);
		   break;
	}


	return result;
}

