/* 01/08/07 14:10:53 = RETRIEVAL
 * 01/07/27 11:54:21 = DELTA
 * @(#)gti.h	1.31.2.1
 */

/* (C) Copyright 1996-2001 by GlobeSpan Inc */

#ifndef  INCLUDE_FILE_GTI_H
#define  INCLUDE_FILE_GTI_H

#include "gti_flgs.h"
#include "gti_mem.h"
#include "gti_whip.h"

#ifdef __cplusplus
    #define GTI_EXTERN extern "C"
#else
    #define GTI_EXTERN extern
#endif

#define GTI_BYTE        unsigned char
#define GTI_WORD        unsigned short
#define GTI_DWORD       unsigned long
#define GTI_CS_ADDRESS  unsigned short          /* To store Chip Select Info */


/* Defines for Different DSP types     */
#if (defined(GTI_DSPQUAD)||defined(GTI_DSPPCI)||defined(GTI_DSPMULTI)||defined(GTI_DSPOCTANE)||defined(GTI_DSPVIKING))
#define GTI_DSP_FDP_ENG
#define GTI_DSP_PCU_ENG
#endif

#if (defined(GTI_DSPFRAMER) || defined(GTI_DSPFRAMER_B1) || defined(GTI_DSPFRAMER_C1) )
#define GTI_DSP_PCU_ENG
#endif

#if(!defined(GTI_USE_XDPXFR))
#define GTI_CRAM_16BIT
#endif

#if( defined(GTI_DSPOCTANE) || defined(GTI_DSPVIKING) )
#define GTI_DSP_SUPERNOVA_ENG
#endif

/* Only Nova does not use Special BSRAM access */
#if !defined(GTI_DSPMULTI)
#define GTI_DSP_SPECIAL_BSRAM
#endif



#ifdef GTI_DEBUG /* for internal use only */

#if defined(GTI_PROD_DMT)
#define GTI_USE_GLOBAL_BUFFER_FOR_LARGE_STATUS   /* Which memory to use for Per Bin status */
#endif

#ifdef GTI_MCM_12_BOARD
    #define GTI_MAX_CHIPS_PER_SYSTEM    3
    #define GTI_MAX_BUSES_PER_SYSTEM    1
#elif defined GTI_MCM_16_BOARD
    #define GTI_MAX_CHIPS_PER_SYSTEM    4
    #define GTI_MAX_BUSES_PER_SYSTEM    1
#elif defined GTI_MCM_24_BOARD
    #define GTI_MAX_CHIPS_PER_SYSTEM    6
    #define GTI_MAX_BUSES_PER_SYSTEM    1
#elif defined GTI_MCM_32_BOARD
    #define GTI_MAX_CHIPS_PER_SYSTEM    8
    #define GTI_MAX_BUSES_PER_SYSTEM    1
#elif defined GTI_MCM_48_BOARD
    #define GTI_MAX_CHIPS_PER_SYSTEM    12
    #define GTI_MAX_BUSES_PER_SYSTEM    2
#elif defined GTI_MCM_48_2_BOARD
    #define GTI_MAX_CHIPS_PER_SYSTEM    12
    #define GTI_MAX_BUSES_PER_SYSTEM    3
#elif defined GTI_MCM_72_BOARD
    #define GTI_MAX_CHIPS_PER_SYSTEM    18
    #define GTI_MAX_BUSES_PER_SYSTEM    3
#elif defined GTI_MCM_96_BOARD
    #define GTI_MAX_CHIPS_PER_SYSTEM    32
    #define GTI_MAX_BUSES_PER_SYSTEM    4
#elif defined GTI_MCM_BOARD
    #define GTI_MAX_CHIPS_PER_SYSTEM    4
    #define GTI_MAX_BUSES_PER_SYSTEM    1
#elif defined GTI_MYDRA_BOARD
    #define GTI_MAX_CHIPS_PER_SYSTEM    2
    #define GTI_MAX_BUSES_PER_SYSTEM    1
#else
    #define GTI_MAX_CHIPS_PER_SYSTEM    1
    #define GTI_MAX_BUSES_PER_SYSTEM    1
#endif

#endif

#include "gtiusdef.h"

#ifdef GTI_DSPOCTANE
    #define GTI_CORES_PER_CHIP  8

#elif (defined(GTI_DSPQUAD))
    #define GTI_CORES_PER_CHIP  4

#elif ( defined(GTI_DSPFRAMER_B1) || defined(GTI_DSPFRAMER_C1) )
    #define GTI_CORES_PER_CHIP  2

#else
    #define GTI_CORES_PER_CHIP  1
#endif

#define GTI_MAX_XCVRS_PER_CHIP (GTI_CORES_PER_CHIP/GTI_NUM_CORES_PER_XCVR)

#define GTI_TOTAL_NUM_OF_XCVRS  (GTI_MAX_CHIPS_PER_SYSTEM*GTI_MAX_XCVRS_PER_CHIP)
#define GTI_NUM_XCVRS GTI_TOTAL_NUM_OF_XCVRS

#include "gtistat.h"

#define GTI_TRUE   1
#define GTI_FALSE  0

#define GTI_INIT_EXP_STANDARD_MASK 0x80
#define GTI_INIT_AFE_CALIBRATION_MASK 0x80

#ifdef GTI_DSPQUAD
#define GTI_LNDR_OFF_MASK  1
#define GTI_AFE_OFF_MASK   2
#define GTI_TRINITY31_MASK 3
#define GTI_HDCLK_OFF_MASK 4
#define GTI_SFCLK_OFF_MASK 8
#endif

#define HIBYTE(w)   ((GTI_BYTE) (((GTI_WORD)(w)>>0x08)&0xFF))
#define LOBYTE(w)   ((GTI_BYTE)((GTI_BYTE) (w)&0xFF))
#define MAKEWORD(a, b) ((GTI_WORD)((GTI_WORD)(a&0xFF)|((GTI_WORD)((GTI_BYTE)(b&0xFF))<<8)))

#define GTI_SIZE_OF_TRACE   300             /* Size of Trace Buffer          */
                                            /* Also used to control the max  */
                                            /* number of loop iterations     */
                                            /* GTI_MAX_SIZE_OF_XCVR_LOOP     */

#define GTI_MAX_HIP_LENGTH  20              /* Size of HIP message buffer    */

/* RADSL Only - MAX TRIES                                                    */
/* This is used only for Warm Startup                                        */
/* This number is related to the access speed of the Host processor          */
/* The slower the access speed the smaller the number.                       */
/* The number should never be less then 2                                    */
/* If access time = ~10us        = 10                                        */
/*                = ~50us        = 2                                         */

#define GTI_MAX_TRIES_GET_DATA 10

/*******************************************************************************/
/* Defines for Execute Structure.                                              */
/* Return Values for ExecuteXCVR  & ParamXCVR                                  */
/*******************************************************************************/

#define GTI_COMPLETE_PASS           0    /* Command Completed Successfully     */
#define GTI_COMPLETE_FAIL           1    /* Command Completed Failed           */
#define GTI_ACTIVE                  2    /* Command Not Completed              */
#define GTI_OK                      3    /* Parameters Ok. Command Exists      */
#define GTI_CONFLICT                4    /* ExecuteXCVR is ACTIVE              */
#define GTI_FAIL                    5    /* Fail Request Decoding              */
#define GTI_NOT_SUPPORTED           6    /* Command Does not exist             */
#define GTI_ILLEGAL_SAS             7    /* Illegal values read from SAS       */
#define GTI_ILLEGAL_PARAM           8    /* Command Parameters illegal         */
#define GTI_ILLEGAL_YAT             9    /* YAT information incorrect          */
#define GTI_COMPLETE_STOP           10   /* From HIP Stoped SAS                */
#define GTI_INCOMPATIABLE_VERSION   11   /* Number of Command Parameters       */
#define GTI_CANCELLED_SAS           15   /* From WHIP Cancell SAS              */
#define GTI_ILLEGAL_STACK           16   /* SAS Stack Overflowed -Internal Usage */
#define GTI_SAS_LOOP_ILLEGAL        17   /* #of SAS lines executed in 1 loop >max*/

/**/

/*******************************************************************************/
/* Defines for LB Load Status.                                                  */
/*******************************************************************************/
#define GTI_LB_MIN_LOAD_TABLE       48

#define GTI_LB_LOAD_START           0
#define GTI_LB_LOAD_CONTINUE        1
#define GTI_LB_LOAD_CANCEL          2

#define GTI_LB_STATUS_IDLE                 0
#define GTI_LB_STATUS_LOAD_INITIATED       1
#define GTI_LB_STATUS_LENGTH_NOT_ENOUGH    2
#define GTI_LB_STATUS_ILLEGAL_PROJ_ID      3
#define GTI_LB_STATUS_LENGTH_TOO_BIG       4
#define GTI_LB_STATUS_WRONG_MODE           5
#define GTI_LB_STATUS_DMA_NOT_READY        6
#define GTI_LB_STATUS_INTERNAL_ERROR       7
#define GTI_LB_STATUS_FILE_CHECKSUM_ERROR  8
#define GTI_LB_STATUS_NULL_BUFFER          9
#define GTI_LB_STATUS_LENTGH_EERROR        10
#define GTI_LB_STATUS_LOAD_CANCELLED       11
#define GTI_LB_STATUS_LOAD_COMPLETE_OK     12

/*****************************************************************************/
/*     Defines DSP messaging system (OCTANE and Plus)                        */
/*****************************************************************************/

/* Messages from 0x00 to 0xff are reserved for HOST - DSP COMMAND operations */
/*****************************************************************************/
#define GTI_DSP_MSG_INIT_CORE       0x100
#define GTI_DSP_MSG_SET_PARAM       0x101

/*****************************************************************************/
/*     Defines for Table Pages                                               */
/*****************************************************************************/

#define GTI_PAGE_A          0
#define GTI_PAGE_B          1
#define GTI_PAGE_C          2
#define GTI_PAGE_D          3
#define GTI_PAGE_E          4
#define GTI_PAGE_F          5
#define GTI_PAGE_G          6

#define GTI_PAGE_AA         10
#define GTI_PAGE_BB         11
#define GTI_PAGE_CC         12
#define GTI_PAGE_DD         13
#define GTI_PAGE_EE         14
#define GTI_PAGE_FF         15

#define GTI_PAGE_NONE       0xff

/***********************************************/
/*      Project Types                          */
/***********************************************/
#define GTI_PROJECT_TYPE_NULL   GTI_PAGE_NONE
#define GTI_PROJECT_TYPE_0      GTI_PAGE_A /* DMT Annex A */
#define GTI_PROJECT_TYPE_1      GTI_PAGE_B /* DMT Annex C */
#define GTI_PROJECT_TYPE_2      GTI_PAGE_C /* DMT Common  */
#define GTI_PROJECT_TYPE_3      GTI_PAGE_D /* DMT Annex B */
#define GTI_PROJECT_TYPE_4      GTI_PAGE_E /* DMT Annex H */
#define GTI_PROJECT_TYPE_5      GTI_PAGE_F /* DMT High Spped */
#define GTI_PROJECT_TYPE_6      GTI_PAGE_G

/*****************************************************************************/
/*     Defines for Command Table                                             */
/*****************************************************************************/
#define GTI_NOP             0   /* 0x00 */
#define GTI_END_OK_CMD      1   /* 0x01 */
#define GTI_END_FAIL_CMD    2   /* 0x02 */
#define GTI_REG_READ        3   /* 0x03 */
#define GTI_REG_LOAD        4   /* 0x04 */
#define GTI_MEM_READ        5   /* 0x05 */
#define GTI_MEM_LOAD        6   /* 0x06 */
#define GTI_FUNCTION        7   /* 0x07 */
#define GTI_TIMEOUT         8   /* 0x08 */
#define GTI_DELAY_CMD       9   /* 0x09 */
#define GTI_FILL            10  /* 0x0a */
#define GTI_HOST_LOAD       11  /* 0x0b */
#define GTI_IMM_LOAD        12  /* 0x0c */
#define GTI_IMM_READ        13  /* 0x0d */
#define GTI_INDIRECT_REG    14  /* 0x0e */
#define GTI_IND_LOAD        15  /* 0x0f */
#define GTI_IF_FUNCTION     16  /* 0x10 */
#define GTI_END_ILLEGAL_CMD 17  /* 0x11 */
#define GTI_CONTEX_CMD      18  /* 0x12 */
#define GTI_CACHE_LOAD      19  /* 0x13 */

                                /* Start Debugging type trace codes */
#define GTI_STRTDOWNLOAD    192 /* 0xc0 */  /* For debugging - Marker in TRACE */
#define GTI_INT_TYPE_DBG    193 /* 0xc1 */  /* When get an interrupt what type */
#define GTI_INT_ISW_DBG     194 /* 0xc2 */  /* What is the ISW =               */
#define GTI_API_TXCLAV      195 /* 0xc3 */  /* TxCLAV API                      */
#define GTI_API_PIN         196 /* 0xc4 */  /* PIN    API                      */
#define GTI_API_NTR         197 /* 0xc5 */  /* SelectNTR    API                */
#define GTI_API_BIASMODE    198 /* 0xc6 */  /* MercBiasMode     API            */
#define GTI_API_PSMODE      199 /* 0xc7 */  /* PSMode           API            */
#define GTI_API_STATUS      200 /* 0xc8 */  /* SStatuxCVR       API            */
#define GTI_API_INITXCVR    201 /* 0xc9 */  /* InitXCVR         API            */

/* Next                                 */
/* FFF0->FFFF = Special meaning GOTO's  */
/* F000->FFEF = Indirect Addressing     */
/*              using HOST RAM          */
/*              0->FEF(0-4079) limit    */
/* 0000->EFFF = Normal GOTO Label #     */

#define GTI_NEXT            0xffff
#define GTI_WAIT_IPU_A      0xfffe
#define GTI_GOTO_IPU_A      0xfffd
#define GTI_WAIT_IPU_B      0xfffc
#define GTI_GOTO_IPU_B      0xfffb
#define GTI_WAIT_IPU_X      0xfffa
#define GTI_GOTO_IPU_X      0xfff9
#define GTI_WAIT_IPU_Y      0xfff8
#define GTI_GOTO_IPU_Y      0xfff7
#define GTI_WAIT_IPU_Z      0xfff6
#define GTI_GOTO_IPU_Z      0xfff5
#define GTI_WAIT_IPU_AB     0xfff4
#define GTI_GOTO_IPU_AB     0xfff3
#define GTI_CONTEX_A        0xfff2
#define GTI_CONTEX_B        0xfff1
#define GTI_STACK_OVERFLOW  0xefff


#define GTI_LBUS_MAX_PROFILE_PER_CORE   0x2000

/*****************************************************************************/
/*************** Defines for the Base Address of the Local Memory ************/

/* Local Bus configuration ANNEX type (must be a bitmap) */
#define GTI_LB_CONFIG_ANNEX_A       0x01
#define GTI_LB_CONFIG_ANNEX_B       0x02
#define GTI_LB_CONFIG_ANNEX_C       0x04
#define GTI_LB_CONFIG_ANNEX_H       0x08
#define GTI_LB_CONFIG_COMMON_DMT    0x10
#define GTI_LB_CONFIG_HIGH_SPEED    0x20

#define GTI_LB_CONFIG_ANNEX_AUTO    0x80

/*****************************************************************************/

/*****************************************************************************/
/*     Defines for IPU <-> HOST                                              */
/*****************************************************************************/
#define GTI_I2H_ready       122 /* 0x7a */
#define GTI_I2H_next        121 /* 0x79 */
#define GTI_I2H_cmd         118 /* 0x76 */
#define GTI_LocalMemBAPR    120 /* 0x78 */

#define GTI_H2I_ready       127 /* 0x7f */
#define GTI_DMA_window      126 /* 0x7e */
#define GTI_H2I_cmd         123 /* 0x7b */

#define GTI_XSRAM_LBUS_RAM_OFFS  0x01
#define GTI_XSRAM_LBUS_RAM_PAGE  0x02

/*****************************************************************************/
/*     ISRAM Location of Status                                              */
/*****************************************************************************/
#define GTI_OP_CH_LOCATION              0x6a
#define GTI_OP_CH_SLAVE_LOCATION        0x6b
#define GTI_WATCHDOG_LOCATION           0x6c
#define GTI_PROGRESS_LOCATION           0x6d
#define GTI_XMIT_LOCATION               0x6e
#define GTI_SQ_LOCATION                 0x6f
#define GTI_SWAP_LOCATION               0x70
#define GTI_ANALOG_REC_LOCATION         0x71
#define GTI_COARSE_LOCATION             0x72
#define GTI_FINE_LOCATION               0x73
#define GTI_SFINE_LOCATION              0x74
#define GTI_DAGC_LOCATION               0x75
#define GTI_RMT_REQ_DWN_CONST_LOCATION  0x60
#define GTI_RMT_REQ_UP_CONST_LOCATION   0x61
#define GTI_ACTUAL_DWN_CONST_LOCATION   0x62
#define GTI_ACTUAL_UP_CONST_LOCATION    0x63
#define GTI_BERT_SYNC_LOCATION          0x08
#define GTI_BERT_ERROR_LOCATION         0x09
#define GTI_TXRX_DATA_LOCATION          0x2f
#define GTI_TX_SELECT_LOCATION          0x2e
#define GTI_TX_START_LOCATION           0x29
#define GTI_TXRX_COUNT_LOCATION         0x34

/*****************************************************************************/
/* Location in IREG for Customer/XCVR                                        */
/* Clock Rate selection                                                      */
/* Interface Control                                                         */
/*****************************************************************************/
#define GTI_RRM3REG_27      0x27
#define GTI_RRM4REG_28      0x28
#define GTI_RRMSDIV_29      0x29
#define GTI_DATACNTL_10     0x10

/*****************************************************************************/
/* Defines for FRAMER                                                        */
/*****************************************************************************/
/* Defines for different                */
/* Framer Configurations                */
/* Call ParamFramer with value          */
/* after startup for framer to          */
/* be configured correctly              */
/* Set to ...NOT_INSTALLED to           */
/* Skip                                 */
/*****************************************************************************/

/* Product = ADSL7M */
#define GTI_FRAMER_SUPERCOM   0x01  /* AS0=4096,AS1=2048,AS3=512,LS0=64      */
#define GTI_FRAMER_N_1_1      0x02  /* Down=AS0=6312,BiDirect=LS0=676        */
#define GTI_FRAMER_N_1_2      0x03  /* Down=AS0=4096,AS1=2048,BiDirec=LS0    */
#define GTI_FRAMER_N_1_3      0x04  /* Down=AS0=AS1=AS2=2048,BiDirec=LS0=676 */
#define GTI_FRAMER_I_1        0x05  /* Down=6312=AS0,Up=676=LS0              */
#define GTI_FRAMER_I_2        0x06  /* Down=6952=AS0,Up=676=LS0              */
#define GTI_FRAMER_P_3        0x07  /* Down=6968=AS0,Bi=64k,Up=588           */
#define GTI_FRAMER_N_1_5      0x08  /* Down=6608=AS0,Dwn LS0=600,Up LS0=694  */
#define GTI_FRAMER_N_1_6      0x09  /* */

/* Product = SDSL E1 */
#define GTI_FRAMER_E1_NS      0x01  /* LS1=2048,Dch=0, NonSlotted */

#define GTI_FRAMER_INSTALLED  GTI_FRAMER_SUPERCOM      /* default */
#define GTI_FRAMER_NOT_INSTALLED GTI_FALSE

/*****************************************************************************/
/* Defines for Memory Load Table                                             */
/*****************************************************************************/
/* Source TYPE */
#define GTI_HOST_ROM            0
#define GTI_HOST_RAM            1

/*****************************************************************************/
/* Defines for Command Pointer Table                                         */
/*****************************************************************************/
#define GTI_EMPTY               0xffff
#define GTI_EOT                 0xff

/*****************************************************************************/
/* Defines for IF Table                                                      */
/*****************************************************************************/
/* Type of compare */
#define GTI_TYPE_EQUAL          0x00
#define GTI_TYPE_NEQUAL         0x01
#define GTI_TYPE_LESS           0x02
#define GTI_TYPE_GREATER        0x03
#define GTI_TYPE_GREATOREQUAL   0x04
#define GTI_TYPE_LESSOREQUAL    0x05
#define GTI_TYPE_INDIRECT       0x80
/* Is value2 Immediate # or HOST RAM offset */

/*****************************************************************************/
/* Defines for IMM copy to Host                                              */
/*****************************************************************************/
#define GTI_TYPE_COPY           0x00
#define GTI_TYPE_OREQUAL        0x01
#define GTI_TYPE_ANDEQUAL       0x02
#define GTI_TYPE_EXOREQUAL      0x03
#define GTI_TYPE_PLUSEQUAL      0x04
#define GTI_TYPE_MINUSEQUAL     0x05
#define GTI_TYPE_RIGHTSHIFT     0x06
#define GTI_TYPE_LEFTSHIFT      0x07
#define GTI_TYPE_1COMPLEMENT    0x08

/*****************************************************************************/
/* MAX SAS stack Size                                                        */
/*****************************************************************************/
#define GTI_MAX_SAS_STACK_SIZE  0x05


/*****************************************************************************/
/* Defines for Write Driver                                                  */
/*****************************************************************************/
#define GTI_STAR_A      0
#define GTI_DSP_A       0

#define GTI_STAR_B      1
#define GTI_DSP_B       1

#define GTI_DSP_TOP     2

#define GTI_STAR_AB     3
#define GTI_DSP_AB      3
#define GTI_ALL_XCVR_CORES 3    /* ALL Cores in a XCVR!! */

#define GTI_STAR_ALL    4       /* All Cores in a CHIP!! */
#define GTI_DSP_ALL     4
#define GTI_ALL_CHIP_CORES 4

#define GTI_DSP_DMA     5
#define GTI_GEMINI_DMA  5       /* Temp  must = to DSP_DMA */

#define GTI_DSP_CORE_A  6
#define GTI_DSP_CORE_0  6

#define GTI_DSP_CORE_B  7
#define GTI_DSP_CORE_1  7

#define GTI_STAR_X      8       /* Used only in SAS table. Not passed.       */
#define GTI_DSP_X       8       /* Used only in SAS table. Not passed.       */
#define GTI_STAR_Y      9       /* Used only in SAS table. Not passed.       */
#define GTI_DSP_Y       9       /* Used only in SAS table. Not passed.       */
#define GTI_STAR_Z      10      /* Used only in SAS table. Not passed.       */
#define GTI_DSP_Z       10      /* Used only in SAS table. Not passed.       */
#define GTI_FRAMER      11      /* Used only in SAS table. Not passed ADSL   */
#define GTI_FRAMER_H    12      /* Used only in SAS table. Not passed HDSL   */
#define GTI_BROOK_RAML  13      /* Used only in SAS table. Not passed Control*/
#define GTI_BROOK_RAMH  14      /* Used only in SAS table. Not passed RAM    */

#define GTI_DSP_C       15
#define GTI_DSP_D       16

#define GTI_STAR_NONE   0xff    /* Used only in SAS table. Not passed        */
#define GTI_DSP_NONE    0xff    /* Used only in SAS table. Not passed        */
                                /* == GTI_EOT  */

/*****************************************************************************/
/* Defines for Memory READ Table                                             */
/*****************************************************************************/
/* Source DUMP                                                               */
#define GTI_DISR            0
#define GTI_NOND            1
#define GTI_SYMBOL          2
#define GTI_HOST2HOST       3

/*****************************************************************************/
/* Defines for ADSL FRAMER REG                                               */
/*****************************************************************************/
#define GTI_ADSL_STATUS     0x1e
#define GTI_SYNC_MASK       0x07

/*****************************************************************************/
/* Defines for HIP routines                                                 */
/*****************************************************************************/
#define GTI_XCVR_RUN        0
#define GTI_XCVR_INACTIVE   1
#define GTI_FINISHED        2


#define GTI_CODE_IPU        0
#define GTI_CODE_AVM        1
#define GTI_CODE_BE         2
#define GTI_CODE_TR         3
#define GTI_CODE_COEF       4
#define GTI_CODE_PCU        5

/*****************************************************************************/
/*  Defines EOC States for DMT                                               */
/*****************************************************************************/
typedef enum
{
    GTI_EOC_STATE_IDLE,         /* Eoc State Idle               */
    GTI_EOC_STATE_PREWRITE,     /* Eoc State Pre-Write          */
    GTI_EOC_STATE_WRITE,        /* Eoc State Write Data         */
    GTI_EOC_STATE_PREREAD,      /* Eoc State Pre-Read           */
    GTI_EOC_STATE_READ,         /* Eoc State Read Data          */
    GTI_EOC_STATE_UTC,          /* Eoc State Unable To Comply   */
    GTI_EOC_STATE_LPR           /* Eoc State LPR, dying gasp    */
}   GTIEOC_STATE;

/* Command Vars */

typedef struct
{
    GTI_BYTE  eocDisabled;
    GTI_BYTE  frameType3Found;
    GTI_BYTE  txEocOffset;

/* SPECIAL ACTION */
    GTI_BYTE    specialAction;
    GTI_BYTE    sendCompressedData;
    GTI_BYTE    ignoreReceivedData;

/* LAYER 2 FUNCTION */
    GTI_WORD  txEocMsg;         /* i */
    GTI_WORD  lastRxEocMsg;     /* Save the previous message  received   */
    GTI_BYTE  nIdentEocMsg;     /* Number of Identical messages received */

/* LAYER 3 FUNCTION */
    GTI_BYTE  *regData;
    GTI_BYTE  regLength;
    GTI_BYTE  regIndex;
    GTI_WORD  saveMsg;          /* CO */
    GTIEOC_STATE  state;        /* State of the EOC */
    GTI_BYTE  latch;
    GTI_BYTE  rmtLatch;         /* CO */

    GTI_BYTE  regId;            /* CPE */
    GTI_BYTE  hostReqCmd;       /* CO */    /* Eoc Request command buffer */
    GTI_WORD  rxErrCnt;         /* Counter for number of Errors received */
    GTI_WORD  tick;             /* CO */
    GTI_BYTE  status;           /* Status of the EOC CMD */
    GTI_BYTE  waitForData;
    GTI_BYTE  data[32];
}   GTIEOC_INFO;

struct BASIC_CMD
{
    GTI_BYTE type;
    GTI_WORD offset;
    GTI_WORD next;
};

typedef struct _GTI_XCVR_EXEC_FLAGS
{
    GTI_WORD active;
}   GTI_XCVR_EXEC_FLAGS;


/**************** Global System Structure Declarations ***********************/

struct _GTI_CHIP_INFO;
struct _GTI_SYST_INFO;
struct _GTI_XCVR_INFO;

#if defined(GTI_USE_DMAXFR) || defined(GTI_USE_XDPXFR)

struct _GTI_LBUS_INFO;

typedef struct _GTI_DSP_TBL_ELEMENT
{
    GTI_CONST_BYTE_ORDER GTI_ADDRESS_QUAL *pTblPtr;
    GTI_DWORD wTblLength;
}   GTI_DSP_TBL_ELEMENT;


typedef struct _GTI_REF_TBL_ELEMENT
{
    GTI_CONST_BYTE_ORDER GTI_ADDRESS_QUAL *pTblPtr;
    GTI_DWORD dwTblLength;
    GTI_DWORD dwTblAddress;
}   GTI_REF_TBL_ELEMENT;

#if defined(GTI_USE_DMAXFR)
#define GTI_MAX_SUPPORTED_XDMA_PAGES 6
#endif

#define GTI_MAX_BIN_XDMA_TABLES 10

typedef struct _GTI_LBUS_INFO
{
    GTI_WORD  wStatus;
    GTI_DWORD dwCurrentAdr;
    GTI_DWORD dwCurrentLbAdr;
    GTI_WORD  wBootId;
    GTI_WORD  wBootChksum;
    GTI_WORD  wBootLength;
    GTI_BYTE  bMode;
    GTI_BYTE  bReloadBoot;
    GTI_BYTE  bBootVer[8];
    GTI_WORD  wProjChksum;
    GTI_DWORD dwProjLength;
    GTI_DWORD dwProjAddr;
    GTI_DWORD dwDescAddr;
    GTI_WORD  wProjId;
    GTI_BYTE  bProjVer[8];
    GTI_DWORD wProfStartPage;
    GTI_WORD  wLoadedAnnexBmp;
    GTI_WORD  bLbConfiguration;
    GTI_WORD  wTotalChkSum;
    GTI_WORD  wExpectedChkSum;
    GTI_WORD  wFileChkSum;
    GTI_DSP_TBL_ELEMENT pFwImageDb[GTI_MAX_BIN_XDMA_TABLES];
    GTI_WORD  wFwImageCurTbl;
    GTI_DWORD dwBufferIdx;
    GTI_DWORD dwTotalLoadedSize;
    GTI_DWORD dwMaxSize;
#if defined(GTI_USE_DMAXFR)
    GTI_BYTE bStartUpTable[GTI_MAX_SUPPORTED_XDMA_PAGES][GTI_DXLCMDTABLE_ENTRIES * 2];
#endif
}   GTI_LBUS_INFO;

#endif

typedef struct _GTI_XCVR_INFO
{
    GTI_BYTE cmd_start;
    GTI_WORD table_pnt;
    GTI_BYTE state;
    GTI_BYTE which_ipu;
    GTI_BYTE test_status;
    GTI_BYTE bExpStandard;
    GTI_BYTE bAfeCalibration;
    GTI_WORD wAnnexSlctd;
    GTI_CS_ADDRESS dsp_A_cs;
    GTI_CS_ADDRESS dsp_B_cs;
    GTI_CS_ADDRESS framer_cs;

    GTI_CUST_XCVR_INFO  CustXcvrInfo;   /* customer-supplied structure */

    GTI_WORD xcvrId;

#if !defined(GTI_USE_DMAXFR) && !defined(GTI_USE_XDPXFR)

    GTI_BYTE paramsDB[GTI_RAMSize];

    /* Pointers to Tables */
    struct CMDPOINT GTI_ADDRESS_QUAL *PntYAT;
#ifdef GTI_DSPOCTANE
    GTI_DWORD GTI_ADDRESS_QUAL *PntCmdPnt;
#else
    struct CMDPOINT GTI_ADDRESS_QUAL *PntCmdPnt;
#endif
    struct BASIC_CMD GTI_ADDRESS_QUAL *PntCmd;
    struct TIMEOUT_STRUCT GTI_ADDRESS_QUAL *PntTime;
    struct FILL_STRUCT GTI_ADDRESS_QUAL *PntFill;
    struct DELAY_STRUCT GTI_ADDRESS_QUAL *PntDelay;
    struct FUN_STRUCT GTI_ADDRESS_QUAL *PntFun;
    struct MEM_LOAD_STRUCT GTI_ADDRESS_QUAL *PntMEML;
    struct MEM_READ_STRUCT GTI_ADDRESS_QUAL *PntMEMR;
    struct REG_LOAD_STRUCT GTI_ADDRESS_QUAL *PntREGL;
    struct REG_READ_STRUCT GTI_ADDRESS_QUAL *PntREGR;
    struct HOST_2_LOAD_STRUCT GTI_ADDRESS_QUAL *PntHOSTL;
    struct IMM_1_LOAD_STRUCT GTI_ADDRESS_QUAL *PntIMML;
    struct IMM_2_READ_STRUCT GTI_ADDRESS_QUAL *PntIMMR;
    struct REG_LOADIND_STRUCT GTI_ADDRESS_QUAL *PntREGInd;
    struct IND_1_LOAD_STRUCT GTI_ADDRESS_QUAL *PntINDL;
    struct IF_STRUCT GTI_ADDRESS_QUAL *PntIF;

    GTI_BYTE GTI_ADDRESS_QUAL *PntIPU;
    GTI_BYTE GTI_ADDRESS_QUAL *PntAVM;
    GTI_BYTE GTI_ADDRESS_QUAL *PntBE;
    GTI_BYTE GTI_ADDRESS_QUAL *PntTR;
    GTI_BYTE GTI_ADDRESS_QUAL *PntCOEF;
    GTI_BYTE GTI_ADDRESS_QUAL *PntPCU;
    GTI_BYTE GTI_ADDRESS_QUAL *PntFDP;
    GTI_BYTE GTI_ADDRESS_QUAL *PntXDP;

    struct BASIC_CMD CurrentCmd;

    GTI_WORD ReturnLabel;                         /* SAS stack pointer */
    GTI_WORD ReturnStack[GTI_MAX_SAS_STACK_SIZE]; /* SAS stack storage */
    GTI_WORD enableTRACE;
#ifdef GTI_DEBUG
    GTI_WORD WhichMEM;
#endif

#else

    GTI_WORD wStartBapr;
    GTI_WORD paramsBA;
#endif

#ifdef GTI_ESC
    GTI_PM_DATA_STATUS_TYPE PM;
#endif

    GTIEOC_INFO eocInfo;

    GTI_XCVR_EXEC_FLAGS ExecFlags;
    GTI_BYTE IrEnabled;
    GTI_BYTE bActiveProjectId;
    GTI_BYTE CurrentMode;

    struct _GTI_CHIP_INFO *pChipInfo;
}   GTI_XCVR_INFO;

/*****************************************************************************/


typedef struct _GTI_CHIP_INFO
{
    struct _GTI_SYST_INFO *pSystInfo;
    GTI_CUST_INFO *pCustInfo;
#if defined(GTI_USE_DMAXFR) || defined(GTI_USE_XDPXFR)
    GTI_WORD wLbusID;
    GTI_WORD wChipIdOnLbus;
#endif
    GTI_BYTE bChipVersion;
    GTI_BYTE bAfeVersion;
    GTI_BYTE lBusVersion[8];
    struct _GTI_XCVR_INFO *XCVR_db[GTI_MAX_XCVRS_PER_CHIP];
}   GTI_CHIP_INFO;

typedef struct _GTI_SYST_INFO
{
    struct _GTI_CHIP_INFO *Chip_db[GTI_MAX_CHIPS_PER_SYSTEM];
#if defined(GTI_USE_DMAXFR) || defined(GTI_USE_XDPXFR)
    struct _GTI_LBUS_INFO Lbus_db[GTI_MAX_BUSES_PER_SYSTEM];
#endif
}   GTI_SYST_INFO;


#include "gtiparam.h"

#ifdef GTI_MULTI_PROJ
#define GTI_MAX_SIZE_OF_XCVR_LOOP   (GTI_SIZE_OF_TRACE*2)
#else
#define GTI_MAX_SIZE_OF_XCVR_LOOP   GTI_MAX_SAS_TABLE
#endif

struct IMM_2_READ_STRUCT
{
    GTI_WORD address;
    GTI_WORD value;
    GTI_BYTE type;
};

struct IMM_LOAD_STRUCT
{
    GTI_BYTE id;
    GTI_BYTE type;
    GTI_BYTE address;
    GTI_BYTE value[6];
};

struct IMM_1_LOAD_STRUCT
{
    GTI_BYTE id;
    GTI_BYTE type;
    GTI_BYTE address;
    GTI_BYTE value[6];
};

struct HOST_2_LOAD_STRUCT
{
    GTI_BYTE id;
    GTI_BYTE address;
    GTI_BYTE value;
};

struct MEM_LOAD_STRUCT
{
    GTI_BYTE id;
    GTI_BYTE dtype;
    GTI_BYTE stype;
    GTI_BYTE STARPnt;
    GTI_WORD StarSrc;
    GTI_WORD EndSrc;
};

struct TIMEOUT_STRUCT
{
   GTI_WORD time_ms;
   GTI_WORD time_off;
};

struct FILL_STRUCT
{
    GTI_BYTE id;
    GTI_BYTE type;
    GTI_BYTE STARPnt;
    GTI_WORD STARLen;
    GTI_BYTE dat;
};

struct DELAY_STRUCT
{
    GTI_WORD time_ms;
};

struct FUN_STRUCT
{
    /*GTI_BYTE (*GTIFunc)();*/
    GTI_WORD tmp;
    GTI_WORD goto0;
    GTI_WORD goto1;
    GTI_WORD goto2;
};

struct REG_LOAD_STRUCT
{
    GTI_BYTE id;
    GTI_BYTE reg;
    GTI_WORD dat;
};

struct IND_1_LOAD_STRUCT
{
    GTI_BYTE id;
    GTI_BYTE type;
    GTI_BYTE STARPnt;
    GTI_WORD offset;
};

struct REG_LOADIND_STRUCT
{
    GTI_BYTE id;
    GTI_BYTE reg;
    GTI_WORD offset;
    GTI_BYTE or_valuel;
    GTI_BYTE or_valueh;
};

struct REG_READ_STRUCT
{
    GTI_BYTE id;
    GTI_BYTE reg;
    GTI_WORD offset;
};

struct MEM_READ_STRUCT
{
    GTI_BYTE id;
    GTI_BYTE type;
    GTI_BYTE dump;
    GTI_WORD src_st;
    GTI_WORD src_sp;
    GTI_WORD dest;
};

struct IF_STRUCT
{
    GTI_WORD value1;
    GTI_WORD value2;
    GTI_BYTE type;
};

struct COEF_STORAGE
{
    GTI_BYTE address;
    GTI_BYTE length;
};

typedef struct _WHIP_BRKPNT_DATA
{
    GTI_BYTE bState; /* On/Off  */
    GTI_BYTE bPage;  /* Page Number     */
    GTI_WORD wAddr;  /* Code location   */
    GTI_WORD wOffs;  /* Offset number   */
    GTI_WORD wType;  /* Type  */
    GTI_BYTE aCode[8];
}   WHIP_BRKPNT_DATA;

#endif /* INCLUDE_FILE_GTI_H */
