/*
  @(#)gtioeoc.c	1.14
*/

/************************************************************************
* FILE NAME   : gtioeoc.c                                              *
* DESCRIPTION : ATU-C *
*                                                                      *
* COMMENT     : *
*                                                                      *
* (C) Copyright 1999 by GlobeSpan Semiconductor Inc                    *
*                                                                      *
************************************************************************/

/************************************************************************
*                             HEADER FILES
************************************************************************/
#include "gti.h"
#include "gtidsp.h"
#include "gtistat.h"
#include "gti_eoca.h"
#include "gtiextn.h"

/************************************************************************
*                          DATA DECLARATIONS
************************************************************************/
extern GTIEOC_DATA_REG  eocDataReg;

GTIEOC_REG_TABLE GTIEocRegInfo[]=
{
{GTI_EOC_CMD_READREG0,  (&(eocDataReg.vendorId[0])),    GTI_EOC_VENDOR_LEN},
{GTI_EOC_CMD_READREG1,  (&(eocDataReg.revisionNum[0])), GTI_EOC_REV_NUM_LEN},
{GTI_EOC_CMD_READREG2,  (&(eocDataReg.serialNum[0])),   GTI_EOC_SERIAL_LEN},
{GTI_EOC_CMD_READREG3,  (&(eocDataReg.selfTest[0])),    GTI_EOC_SELFTEST_LEN},
{GTI_EOC_CMD_READREG4,  (&(eocDataReg.vendorDisc1[0])), GTI_EOC_VEN_DISC1_LEN},
{GTI_EOC_CMD_READREG5,  (&(eocDataReg.vendorDisc2[0])), GTI_EOC_VEN_DISC2_LEN},
{GTI_EOC_CMD_READREG6,  (&(eocDataReg.lineAtten[0])),   GTI_EOC_ATTENU_LEN},
{GTI_EOC_CMD_READREG7,  (&(eocDataReg.snrMargin[0])),   GTI_EOC_SNRMARGN_LEN},
{GTI_EOC_CMD_READREG8,  (&(eocDataReg.aturConfig[0])),  GTI_EOC_CONFIG_LEN},
{GTI_EOC_CMD_READREGA,  (&(eocDataReg.registryA[0])),   GTI_EOC_REGA_LEN},
{GTI_EOC_CMD_WRITEREG0, (&(eocDataReg.vendorId[0])),    GTI_EOC_VENDOR_LEN},
{GTI_EOC_CMD_WRITEREG1, (&(eocDataReg.revisionNum[0])), GTI_EOC_REV_NUM_LEN},
{GTI_EOC_CMD_WRITEREG2, (&(eocDataReg.serialNum[0])),   GTI_EOC_SERIAL_LEN},
{GTI_EOC_CMD_WRITEREG3, (&(eocDataReg.selfTest[0])),    GTI_EOC_SELFTEST_LEN},
{GTI_EOC_CMD_WRITEREG4, (&(eocDataReg.vendorDisc1[0])), GTI_EOC_VEN_DISC1_LEN},
{GTI_EOC_CMD_WRITEREG5, (&(eocDataReg.vendorDisc2[0])), GTI_EOC_VEN_DISC2_LEN},
{GTI_EOC_CMD_WRITEREG6, (&(eocDataReg.lineAtten[0])),   GTI_EOC_ATTENU_LEN},
{GTI_EOC_CMD_WRITEREG7, (&(eocDataReg.snrMargin[0])),   GTI_EOC_SNRMARGN_LEN},
{GTI_EOC_CMD_WRITEREG8, (&(eocDataReg.aturConfig[0])),  GTI_EOC_CONFIG_LEN},
{GTI_EOC_CMD_WRITEREGA,  (&(eocDataReg.registryA[0])),  GTI_EOC_REGA_LEN}
};

/************************************************************************
*                             DATA TYPES
************************************************************************/

/************************************************************************
*                        FUNCTION DECLARATIONS
************************************************************************/

/************************************************************************
* GTIEocInit
************************************************************************/
void GTIEocInit(GTIEOC_INFO *eInfoPtr)
{
    eInfoPtr->txEocMsg = 0x0;
    eInfoPtr->lastRxEocMsg = 0;
    eInfoPtr->nIdentEocMsg = 0;
    eInfoPtr->rxErrCnt = 0;
    eInfoPtr->latch = GTI_FALSE; /* FALSE */
    eInfoPtr->rmtLatch = GTI_FALSE; /* FALSE */
    eInfoPtr->state = GTI_EOC_STATE_IDLE;
    eInfoPtr->saveMsg = 0;
    eInfoPtr->status = GTI_EOC_COMPLETE;

    eInfoPtr->hostReqCmd = 0;
    eInfoPtr->tick = 0;

    eInfoPtr->regLength = 0;
    eInfoPtr->regIndex = 0;

    eInfoPtr->eocDisabled = GTI_TRUE;
    eInfoPtr->frameType3Found = GTI_FALSE;
    eInfoPtr->txEocOffset = 9;

#if 0
    /* Initialize the values in the PSRAM for the restransmission */
    WRITE_DSP_ADDR(GTISysPtr, GTI_DSP_A, GTI_EOC_RETX_LIMIT_OFFSET, GTI_PSRAM0W);
    WRITE_DSP_WORD(GTISysPtr, GTI_DSP_A, (GTI_BYTE)(GTI_EOC_RETX_SET_COUNT & 0xFF),
        (GTI_BYTE)((GTI_EOC_RETX_SET_COUNT >> 8) & 0xFF) );
#endif

    /* DSP Does the initialization of the flags */
}

/************************************************************************
* GTIEocProc
************************************************************************/

void GTIEocProc(GTI_XCVR_INFO *GTISysPtr, GTIEOC_CMD *hostPtr)
{
#ifdef GTI_EOC_PCU
    Gti_EocProcPcu(GTISysPtr, hostPtr);
#else
    Gti_EocProcHost(GTISysPtr, hostPtr);
#endif
}



#ifndef GTI_EOC_PCU
/************************************************************************
* GTIEocProc in host
************************************************************************/
void Gti_EocProcHost(GTI_XCVR_INFO *GTISysPtr, GTIEOC_CMD *hostPtr)
{
    /* Buffer that is stored in the PCU for each Superframe */
    GTIEOC_INFO *eocInfoPtr;
    GTI_WORD    rxBuff[GTI_EOC_MAX_NUMOF_MSG];
    GTI_WORD    txBuff[GTI_EOC_MAX_NUMOF_MSG];
    GTI_BYTE    txBuffIndex=0;
    GTI_BYTE    i;
    GTI_WORD    newMsg;
    GTI_WORD    tempMsg=0;
    GTI_BYTE    txAutoIndex=0;
    GTI_WORD    dspState=0;

    for(i=0; i < GTI_EOC_MAX_NUMOF_MSG; i++)
        txBuff[i] = 0;

    /* Get the saved structure for EOC */
    eocInfoPtr = (GTIEOC_INFO *)&GTISysPtr->eocInfo;
    hostPtr->rx.AutoLen = 0;
    hostPtr->rx.EocCmd = 0;
    hostPtr->rx.EocLen = 0;
    hostPtr->rx.Status = GTI_EOC_COMPLETE;

    if (eocInfoPtr->eocDisabled)
    {
        /* Check to see the if we are in DATA_MODE */
        if(!Gti_SStatusXCVR(GTISysPtr, GTI_OP_STATE, &dspState, sizeof(dspState), 0))
            return;

        if (dspState == GTI_DATA_OP)
        {
            eocInfoPtr->eocDisabled = GTI_FALSE;

            /* Read the Framing Structure Type from the DSP-PCU */
            WRITE_DSP_ADDR(GTISysPtr, GTI_DSP_A, 0x6, (GTI_BYTE)(GTI_READ_DSP | GTI_PSRAM0W) );
            if (((READ_DSP_WORD(GTISysPtr, GTI_DSP_A)) & 0x600) == 0x600)
            {
                eocInfoPtr->frameType3Found = GTI_TRUE;
                eocInfoPtr->txEocOffset = 5;
            }
        }
        else
            return;
    }

    /* ***** RX ***** */
    /* Read the RX Flag */
    WRITE_DSP_ADDR(GTISysPtr, GTI_DSP_A, GTI_EOC_RXBUFF_FLAG_OFFSET, (GTI_READ_DSP | GTI_ISRAM) );
    if (READ_DSP_WORD(GTISysPtr, GTI_DSP_A) == GTI_EOC_RXBUFF_FULL)
    {
        eocInfoPtr->tick++;

        /* Turn ON the Auto Increment */
        CONFIGURE_DSP(GTISysPtr, GTI_DSP_A, GTI_AI);

        /* Read the Data from the DSP */
        WRITE_DSP_ADDR(GTISysPtr, GTI_DSP_A, GTI_EOC_RXBUFF_OFFSET, (GTI_READ_DSP | GTI_ISRAM) );

        for(i=0; i<GTI_EOC_MAX_NUMOF_MSG; i++)
            rxBuff[i] = READ_DSP_WORD(GTISysPtr, GTI_DSP_A);

        /* Turn OFF the Auto Increment */
        CONFIGURE_DSP(GTISysPtr, GTI_DSP_A, GTI_NO_AI);

        /* Reset the RX Flag to empty Buffer */
        WRITE_DSP_ADDR(GTISysPtr, GTI_DSP_A, GTI_EOC_RXBUFF_FLAG_OFFSET, GTI_ISRAM );
        WRITE_DSP_WORD(GTISysPtr, GTI_DSP_A, 0, GTI_EOC_RXBUFF_EMPTY);

        /* ***** */

        /* Process the RX Information */
        for (i=0; i<GTI_EOC_MAX_NUMOF_MSG; i++)
        {
            /* NOT (crc, ib[0-2], or Synch_Byte) */
            if( !((i == 0) || (i == GTI_EOC_FRAME_34) ||
                (rxBuff[i] == GTI_EOC_SYNCH_CONTROL) ||
                ((eocInfoPtr->frameType3Found) && (i>GTI_EOC_FRAME_34))) )
            {
                /* Only EOC Message to be processed */
                newMsg = rxBuff[i];

                /* Check, if the overhead of the eoc is there */
                if((newMsg & GTI_EOC_OVERHEAD_MASK) != GTI_EOC_OVERHEAD_MASK)
                {
                    eocInfoPtr->rxErrCnt++;
                }
                else if( !(((newMsg & GTI_EOC_ADDRESS_MASK)==GTI_EOC_ATUC_ADDRESS) ||
                    ((newMsg & GTI_EOC_ADDRESS_MASK)==GTI_EOC_ATUR_ADDRESS)) )
                {
                    eocInfoPtr->rxErrCnt++;
                }
                /* Check if this is an autonomous message */
                else if(!(newMsg & GTI_EOC_AUTONOMOUS_MASK))
                {
                    /* Store the AUTONOMOUS MESSAGE Into the Data Structure */
                    hostPtr->rx.AutoData[hostPtr->rx.AutoLen++]=GTI_EOC_M_INFO_FIELD(newMsg);
                }
                /* Compare the new message(2Byte) with lastRxEocMsg */
                else if (newMsg == eocInfoPtr->lastRxEocMsg)
                {
                    /* Increment the number of identical message received */
                    eocInfoPtr->nIdentEocMsg++;
                    if (eocInfoPtr->nIdentEocMsg == GTI_EOC_MAGIC_NUM)
                    {
                        tempMsg = GTIEocExecCmd(GTISysPtr, hostPtr, newMsg);
                        if (tempMsg)
                        {
                            /* Three times */
                            eocInfoPtr->txEocMsg = tempMsg;
                            txBuff[ (txBuffIndex++) ] = eocInfoPtr->txEocMsg;
                            txBuff[ (txBuffIndex++) ] = eocInfoPtr->txEocMsg;
                            txBuff[ (txBuffIndex++) ] = eocInfoPtr->txEocMsg;
                        }
                    }
                    else
                        if ((eocInfoPtr->nIdentEocMsg < GTI_EOC_MAGIC_NUM) &&
                            (eocInfoPtr->status == GTI_EOC_BUSY))
                            txBuff[ (txBuffIndex++) ] = eocInfoPtr->txEocMsg;
                }
                else if (eocInfoPtr->status == GTI_EOC_BUSY)
                {
                    eocInfoPtr->lastRxEocMsg = newMsg;
                    eocInfoPtr->nIdentEocMsg=1;
                    txBuff[ (txBuffIndex++) ] = eocInfoPtr->txEocMsg;
                }
            }
        }
    }

    /* ***** reTX ***** */
    hostPtr->tx.Status = GTI_EOC_COMPLETE;

    /* Is this a new command? */
    if (hostPtr->tx.EocCmd)
    {
        tempMsg = GTIEocHostCmd(GTISysPtr, hostPtr);
        if (tempMsg)
        {
            /* Three times */
            eocInfoPtr->txEocMsg = tempMsg;
            txBuff[ (txBuffIndex++) ] = eocInfoPtr->txEocMsg;
            txBuff[ (txBuffIndex++) ] = eocInfoPtr->txEocMsg;
            txBuff[ (txBuffIndex++) ] = eocInfoPtr->txEocMsg;
        }
    }

    /* Return the correct status */
    hostPtr->rx.Status = eocInfoPtr->status;

    /* Is there something to transmit? */
    if ( !((txBuffIndex) || (hostPtr->tx.AutoLen)) )
        return;

    if (hostPtr->tx.AutoLen > GTI_EOC_MAX_AUTONOMOUS_LEN)
    {
        hostPtr->tx.Status = (GTI_WORD)((hostPtr->tx.Status) ? GTI_EOC_ERROR : GTI_EOC_INVALID_LEN);
        txAutoIndex = hostPtr->tx.AutoLen;   /* Do not transmit */
    }

    /* ***** TX ***** */
    WRITE_DSP_ADDR(GTISysPtr, GTI_DSP_A, GTI_EOC_TXBUFF_FLAG_OFFSET, (GTI_READ_DSP | GTI_ISRAM) );
    if (READ_DSP_WORD(GTISysPtr, GTI_DSP_A) == GTI_EOC_TXBUFF_EMPTY)
    {
        /* Turn ON the Auto Increment */
        CONFIGURE_DSP(GTISysPtr, GTI_DSP_A, GTI_AI);

        /* Write the Data to the DSP */
        WRITE_DSP_ADDR(GTISysPtr, GTI_DSP_A, GTI_EOC_TXBUFF_OFFSET, GTI_ISRAM);
        for (i=0; i<GTI_EOC_MAX_NUMOF_MSG; i++)
        {
        /* Note: Frame 0 (CRC), Frame 1 (IB7:0), Frame 34 (IB15:8), and
            Frame 35 (IB23:16) will be over written by the PCU */
            if ( (i == 0) || (i == GTI_EOC_FRAME_34) )
            {
                WRITE_DSP_WORD(GTISysPtr, GTI_DSP_A, 0xFF, 0xFF);
            }
            /* TEMP FIX */
            else if (i < eocInfoPtr->txEocOffset)
            {
                /* Write Control Synch. Byte */
                WRITE_DSP_WORD(GTISysPtr, GTI_DSP_A,
                    (GTI_BYTE)(GTI_EOC_SYNCH_CONTROL & 0xFF),
                    (GTI_BYTE)((GTI_EOC_SYNCH_CONTROL >> 8) & 0xFF) );
            }
            /* TEMP FIX */
            else if((eocInfoPtr->frameType3Found) && (i>GTI_EOC_FRAME_34))
            {
                ; /* Do not Write Anything */
            }
            else
            {
                if ( txBuffIndex )
                {
                /*  NEED TO be careful here because we know that we will not be tx-ing
                    more than 3 command at a time */
                    /*  Write EOC command to the DSP */

                    WRITE_DSP_WORD(GTISysPtr, GTI_DSP_A,
                        /* TEMP FIX */
                        (GTI_BYTE)( txBuff[i - (eocInfoPtr->txEocOffset)] & 0xFF),
                        (GTI_BYTE)((txBuff[i - (eocInfoPtr->txEocOffset)] >> 8) & 0xFF));
                    /* TEMP FIX */
                    txBuffIndex--;
                }
                else if (txAutoIndex < hostPtr->tx.AutoLen)
                {
                    /* Write the AUTONOMOUS Message to the PCU */
                    WRITE_DSP_WORD(GTISysPtr, GTI_DSP_A,
                        (GTI_BYTE)(GTI_EOC_AUTONOMOUS_MSG_CO | ((hostPtr->tx.AutoData[txAutoIndex]&0x01)<<7)),
                        (GTI_BYTE)(hostPtr->tx.AutoData[txAutoIndex] | GTI_EOC_OVERHEAD_B1));
                    txAutoIndex++;
                }
                else
                {
                    /* Write Control Synch. Byte */
                    WRITE_DSP_WORD(GTISysPtr, GTI_DSP_A,
                        (GTI_BYTE)(GTI_EOC_SYNCH_CONTROL & 0xFF),
                        (GTI_BYTE)((GTI_EOC_SYNCH_CONTROL >> 8) & 0xFF) );
                }
            }
        }
        /* ***** */
        /* Turn OFF the Auto Increment */
        CONFIGURE_DSP(GTISysPtr, GTI_DSP_A, GTI_NO_AI);
        /* Set the Flag to say GO */
        WRITE_DSP_ADDR(GTISysPtr, GTI_DSP_A, GTI_EOC_TXBUFF_FLAG_OFFSET, GTI_ISRAM );
        WRITE_DSP_WORD(GTISysPtr, GTI_DSP_A, 0, GTI_EOC_TXBUFF_FULL);
    }
    else
    {
        /* save information to transmit????? */
        hostPtr->tx.Status = GTI_EOC_ERROR;
    }
}

/************************************************************************
* FUNCTION     : Gti_EocStateIdle()
* DESCRIPTION  :
*              :
* INPUT        :
* OUTPUT       : None
* RETURN       : None
************************************************************************/
GTI_WORD Gti_EocStateIdle(GTI_XCVR_INFO *GTISysPtr,
                          GTIEOC_CMD *hostPtr,
                          GTIEOC_FIELDS *fields)
{
    /* setup pointer for register information */
    GTI_WORD retEocMsg=0;
    GTIEOC_INFO *infoPtr;
    GTI_BYTE i;

    /* Look at only OPCODE */
    if ( ! fields->isOpcode )
        return retEocMsg;

    infoPtr = (GTIEOC_INFO *)&GTISysPtr->eocInfo;

    /* Should be able to Rx any Messages */
    switch( fields->info )
    {
    case GTI_EOC_CMD_UTC:        /* 0x04 u Unable To Comply */
        /* Received UTC from ATU-R */
        infoPtr->status = GTI_EOC_UTC;
        hostPtr->rx.EocLen = 0;
        hostPtr->rx.EocCmd = infoPtr->hostReqCmd;
        break;

    case GTI_EOC_CMD_RTN:        /* 0xF0 d<->u Return To Normal */
        /* This is an Echo!!! */
        infoPtr->status = GTI_EOC_COMPLETE;
        hostPtr->rx.EocCmd = infoPtr->hostReqCmd;
        hostPtr->rx.EocLen = infoPtr->regIndex;
        for(i=0; i<infoPtr->regIndex; i++)
            hostPtr->rx.EocData[i] = infoPtr->regData[i];
        if (infoPtr->rmtLatch == GTI_TRUE)
            infoPtr->rmtLatch= GTI_FALSE;
        if (infoPtr->latch == GTI_TRUE)
        {
            WRITE_DSP_ADDR(GTISysPtr,GTI_DSP_A,GTI_EOC_REQCOR_OFFSET,GTI_PSRAM0W);
            WRITE_DSP_WORD(GTISysPtr, GTI_DSP_A, GTI_EOC_REQCOR_DISABLE, 0);
            infoPtr->latch = GTI_FALSE; /* FALSE */
        }
        break;

    case GTI_EOC_CMD_HOLD:       /* 0x01 d<->u Hold State */
    case GTI_EOC_CMD_EOD:
        /* This is an Echo!!! */
        infoPtr->status = GTI_EOC_COMPLETE;
        hostPtr->rx.EocCmd = infoPtr->hostReqCmd;
        hostPtr->rx.EocLen = infoPtr->regIndex;
        for(i=0; i<infoPtr->regIndex; i++)
            hostPtr->rx.EocData[i] = infoPtr->regData[i];
        break;

    case GTI_EOC_CMD_NOTCOR:     /* 0x0B d<->u Notify Corrupt CRC */
        WRITE_DSP_ADDR(GTISysPtr, GTI_DSP_A, GTI_EOC_REQCOR_OFFSET, GTI_PSRAM0W);
        WRITE_DSP_WORD(GTISysPtr, GTI_DSP_A, GTI_EOC_REQCOR_ENABLE, 0);
        infoPtr->latch = GTI_TRUE; /* TRUE */

        /* Tell the far-end to hold this state */
        retEocMsg = GTI_EOC_M_MAKE_MSG( GTI_EOC_ATUR_ADDRESS,
            GTI_EOC_OPCODE_FIELD, GTI_EOC_ODD_PARITY,
            GTI_EOC_CMD_HOLD );
        infoPtr->saveMsg = retEocMsg;
        infoPtr->regIndex = 0;
        break;

    case GTI_EOC_CMD_REQCOR:     /* 0x07 d<->u Request Corrupt CRC */
        infoPtr->rmtLatch = GTI_TRUE;

        /* Tell the far-end to hold this state */
        retEocMsg = GTI_EOC_M_MAKE_MSG( GTI_EOC_ATUR_ADDRESS,
            GTI_EOC_OPCODE_FIELD, GTI_EOC_ODD_PARITY,
            GTI_EOC_CMD_HOLD );
        infoPtr->saveMsg = retEocMsg;
        infoPtr->regIndex = 0;
        break;

    case GTI_EOC_CMD_NOTEND:     /* 0x0D d<->u Notify End of Corrupt CRC */
        WRITE_DSP_ADDR(GTISysPtr, GTI_DSP_A, GTI_EOC_REQCOR_OFFSET, GTI_PSRAM0W);
        WRITE_DSP_WORD(GTISysPtr, GTI_DSP_A, GTI_EOC_REQCOR_DISABLE, 0);
        infoPtr->latch = GTI_FALSE; /* FALSE */

        infoPtr->status = GTI_EOC_COMPLETE;
        hostPtr->rx.EocCmd = infoPtr->hostReqCmd;
        hostPtr->rx.EocLen = 0;
        break;

    case GTI_EOC_CMD_REQEND:     /* 0x08 d<->u Request End of Corrupt CRC */
        /* This is an Echo!!! */
        infoPtr->rmtLatch= GTI_FALSE;
        infoPtr->status = GTI_EOC_COMPLETE;
        hostPtr->rx.EocCmd = infoPtr->hostReqCmd;
        hostPtr->rx.EocLen = 0;
        break;

    case GTI_EOC_CMD_SLFTEST:    /* 0x02 d<->u Self-Test */
    case GTI_EOC_CMD_REQTPU:     /* 0x13 d<->u Request Test Parameter Update */
    case GTI_EOC_CMD_VPP0:       /* 0x19 d<->u Vendor Protrietary Protocols */
    case GTI_EOC_CMD_VPP1:       /* 0x1A d<->u Vendor Protrietary Protocols */
    case GTI_EOC_CMD_VPP2:       /* 0x1C d<->u Vendor Protrietary Protocols */
    case GTI_EOC_CMD_VPP3:       /* 0x1F d<->u Vendor Protrietary Protocols */
        /* This is an Echo!!! */
        infoPtr->status = GTI_EOC_COMPLETE;
        hostPtr->rx.EocCmd = infoPtr->hostReqCmd;
        hostPtr->rx.EocLen = 0;
        break;

    default:
        /* Ignore the message */
        break;
    }
    return retEocMsg;
}

/************************************************************************
* FUNCTION     : Gti_EocStatePreWrite()
* DESCRIPTION  :
*              :
* INPUT        :
* OUTPUT       : None
* RETURN       : None
************************************************************************/
GTI_WORD Gti_EocStatePreWrite(GTIEOC_INFO *infoPtr,
                              GTIEOC_CMD *hostPtr,
                              GTIEOC_FIELDS *fields)
{
    GTI_WORD retEocMsg = 0;

    /* Is this an Echo? */
    if (infoPtr->saveMsg == fields->msg)
    {
        /* Send Next DATA Byte */
        retEocMsg = GTI_EOC_M_MAKE_MSG( GTI_EOC_ATUR_ADDRESS,
            GTI_EOC_DATA_FIELD, GTI_EOC_ODD_PARITY,
            infoPtr->regData[ infoPtr->regIndex ] );
        infoPtr->regIndex++;
        infoPtr->saveMsg = retEocMsg;

        /* Change the state to write */
        infoPtr->state = GTI_EOC_STATE_WRITE;
    }
    else if ((fields->isOpcode) && (fields->info == GTI_EOC_CMD_UTC))
    {
        /* received UTC */
        infoPtr->status = GTI_EOC_UTC;
        hostPtr->rx.EocCmd = infoPtr->hostReqCmd;
        infoPtr->state = GTI_EOC_STATE_IDLE;
    }
    return (retEocMsg);
}

/************************************************************************
* FUNCTION     : Gti_EocStateWrite()
* DESCRIPTION  :
*              :
* INPUT        :
* OUTPUT       : None
* RETURN       : None
************************************************************************/
GTI_WORD Gti_EocStateWrite(GTIEOC_INFO *infoPtr,
                           GTIEOC_CMD *hostPtr,
                           GTIEOC_FIELDS *fields)
{
    GTI_WORD retEocMsg = 0;
    GTI_BYTE bNext;

    /* Is this an ECHO? */
    if (infoPtr->saveMsg == fields->msg)
    {
        if (infoPtr->regIndex < infoPtr->regLength)
        {
            /* Send Next DATA Byte */
            retEocMsg = GTI_EOC_M_MAKE_MSG( GTI_EOC_ATUR_ADDRESS, GTI_EOC_DATA_FIELD,
                ((GTI_EOC_M_PARITY_FIELD(infoPtr->saveMsg)) ?
                  GTI_EOC_EVEN_PARITY : GTI_EOC_ODD_PARITY),
                  infoPtr->regData[infoPtr->regIndex] );

            infoPtr->regIndex++;
            infoPtr->saveMsg = retEocMsg;
        }
        else
        {
            bNext = GTI_EOC_CMD_EOD;
            retEocMsg = GTI_EOC_M_MAKE_MSG( GTI_EOC_ATUR_ADDRESS,
                GTI_EOC_OPCODE_FIELD, GTI_EOC_ODD_PARITY,
                bNext );

            infoPtr->saveMsg = retEocMsg;
            infoPtr->regIndex = 0;
            infoPtr->state = GTI_EOC_STATE_IDLE;
        }
    }
    else if (fields->isOpcode)
    {
        /* Rx only Data and EOD Message */
        if ( fields->info == GTI_EOC_CMD_EOD )
        {
            /* ATU-R said, WE can not write data anymore */
            infoPtr->status = GTI_EOC_COMPLETE;
            hostPtr->rx.EocCmd = infoPtr->hostReqCmd;
            infoPtr->state = GTI_EOC_STATE_IDLE;
        }
        else if( fields->info == GTI_EOC_CMD_UTC )
        {
            /* Received UTC */
            infoPtr->status = GTI_EOC_UTC;
            hostPtr->rx.EocCmd = infoPtr->hostReqCmd;
            infoPtr->state = GTI_EOC_STATE_IDLE;
        }
    }
    return retEocMsg;
}

/************************************************************************
* FUNCTION     : Gti_EocStatePreRead()
* DESCRIPTION  :
*              :
* INPUT        :
* OUTPUT       : None
* RETURN       : None
************************************************************************/
GTI_WORD Gti_EocStatePreRead(GTIEOC_INFO *infoPtr,
                             GTIEOC_CMD *hostPtr,
                             GTIEOC_FIELDS *fields)
{
    GTI_WORD retEocMsg=0;
    GTI_BYTE bNext;

    /* Is this an Echo? */
    if (infoPtr->saveMsg == fields->msg)
    {
        /* Send Next Command NEXT_BYTE */
        bNext = GTI_EOC_CMD_NEXT;
        retEocMsg = GTI_EOC_M_MAKE_MSG(GTI_EOC_ATUR_ADDRESS,
            GTI_EOC_OPCODE_FIELD, GTI_EOC_ODD_PARITY,
            bNext);
        infoPtr->saveMsg = retEocMsg;

        /* Change the state to read */
        infoPtr->state = GTI_EOC_STATE_READ;
    }
    /* Is it Opcode/Command? */
    else if ((fields->isOpcode) && (fields->info == GTI_EOC_CMD_UTC))
    {
        infoPtr->status = GTI_EOC_UTC;
        hostPtr->rx.EocCmd = infoPtr->hostReqCmd;
        infoPtr->state = GTI_EOC_STATE_IDLE;
    }
    return retEocMsg;
}

/************************************************************************
* FUNCTION     : Gti_EocStateRead()
* DESCRIPTION  :
*              :
* INPUT        :
* OUTPUT       : None
* RETURN       : None
************************************************************************/
GTI_WORD Gti_EocStateRead(GTIEOC_INFO *infoPtr,
                          GTIEOC_CMD *hostPtr,
                          GTIEOC_FIELDS *fields)
{
    GTI_WORD retEocMsg = 0;
    GTI_BYTE bNext;

    /* Is it Opcode/Command? */
    if (fields->isOpcode)
    {
        if (fields->info == GTI_EOC_CMD_EOD)
        {
            /* Send RTN or HOLD */
            /* Is there anything that is latched on ATU-R? */
            retEocMsg = GTI_EOC_M_MAKE_MSG( GTI_EOC_ATUR_ADDRESS,
                GTI_EOC_OPCODE_FIELD, GTI_EOC_ODD_PARITY,
                ((infoPtr->rmtLatch) ?
                GTI_EOC_CMD_HOLD : GTI_EOC_CMD_RTN));

            infoPtr->saveMsg = retEocMsg;
            infoPtr->state = GTI_EOC_STATE_IDLE;
        }
        else if (fields->info == GTI_EOC_CMD_UTC)
        {
            /* Rx-ed UTC */
            infoPtr->status = GTI_EOC_UTC;
            hostPtr->rx.EocCmd = infoPtr->hostReqCmd;
            infoPtr->state = GTI_EOC_STATE_IDLE;
        }
    }
    else if (infoPtr->regIndex < infoPtr->regLength)
    {
        /* This is DATA....READ ONLY */
        infoPtr->regData[infoPtr->regIndex] = fields->info;
        infoPtr->regIndex++;

        /* Send Next Command NEXT_BYTE */
        bNext = GTI_EOC_CMD_NEXT;

        retEocMsg = GTI_EOC_M_MAKE_MSG(
            GTI_EOC_ATUR_ADDRESS, GTI_EOC_OPCODE_FIELD,
            ((GTI_EOC_M_PARITY_FIELD(infoPtr->saveMsg)) ?
              GTI_EOC_EVEN_PARITY : GTI_EOC_ODD_PARITY),
              bNext );
        infoPtr->saveMsg = retEocMsg;
    }
    else
    {
        /* Send RTN or HOLD */
        /* Is there anything that is latched on ATU-R? */
        retEocMsg = GTI_EOC_M_MAKE_MSG( GTI_EOC_ATUR_ADDRESS,
            GTI_EOC_OPCODE_FIELD,GTI_EOC_ODD_PARITY,
            ( (infoPtr->latch) ? GTI_EOC_CMD_HOLD : GTI_EOC_CMD_RTN) );

        infoPtr->saveMsg = retEocMsg;
        infoPtr->state = GTI_EOC_STATE_IDLE;
    }

    return retEocMsg;
}

/************************************************************************
* FUNCTION     : GTIEocHostCmd()
* DESCRIPTION  :
*              :
* INPUT        :
* OUTPUT       : None
* RETURN       : None
************************************************************************/
GTI_WORD GTIEocHostCmd( GTI_XCVR_INFO *GTISysPtr, GTIEOC_CMD *hostPtr )
{
    /* setup pointer for register information */
    GTIEOC_REG_TABLE *reg = (GTIEOC_REG_TABLE *)GTIEocRegInfo;
    GTIEOC_INFO *infoPtr;
    GTI_WORD retEocMsg=0;
    GTI_BYTE i;
    GTI_BYTE rStandard;

    infoPtr = (GTIEOC_INFO *)&GTISysPtr->eocInfo;

    /* Reset the EOC Module (GSI) */
    if (hostPtr->tx.EocCmd == GTI_EOC_CMD_RESET)
    {
        /* Initialize the EOC Module */
        GTIEocInit (infoPtr);
        return retEocMsg;
    }

    /* We can not accept anything during BUSY state */
    /*if (infoPtr->status != GTI_EOC_COMPLETE) */
    if (infoPtr->status == GTI_EOC_BUSY)
    {
        hostPtr->tx.Status = GTI_EOC_INVALID_CMD;
        return retEocMsg;
    }

    infoPtr->regIndex = 0;
    infoPtr->regLength = 0;
    infoPtr->saveMsg = 0;
    infoPtr->lastRxEocMsg = 0;
    infoPtr->nIdentEocMsg = 0;

    /* Should be able to Rx any Messages */
    switch( hostPtr->tx.EocCmd )
    {
    case GTI_EOC_CMD_SLFTEST:    /* 0x02 d<->u Self-Test */
    case GTI_EOC_CMD_REQTPU:     /* 0x13 d<->u Req Test Parameter Update */
    case GTI_EOC_CMD_VPP0:       /* 0x19 d<->u Vendor Protrietary Protocols */
    case GTI_EOC_CMD_VPP1:       /* 0x1A d<->u Vendor Protrietary Protocols */
    case GTI_EOC_CMD_VPP2:       /* 0x1C d<->u Vendor Protrietary Protocols */
    case GTI_EOC_CMD_VPP3:       /* 0x1F d<->u Vendor Protrietary Protocols */
    case GTI_EOC_CMD_RTN:        /* 0xF0 d<->u Return To Normal */
    case GTI_EOC_CMD_NOTCOR:     /* 0x0B d<->u Notify Corrupt CRC */
    case GTI_EOC_CMD_REQCOR:     /* 0x07 d<->u Request Corrupt CRC */
        /* Message will be build last */
        break;

    case GTI_EOC_CMD_NOTEND:     /* 0x0D d<->u Notify End of Corrupt CRC */
        if (infoPtr->latch == GTI_FALSE)
        {
            hostPtr->tx.Status = GTI_EOC_INVALID_CMD;
            return (0);
        }
        /* Message will be build last */
        break;

    case GTI_EOC_CMD_REQEND:     /* 0x08 d<->u Request End of Corrupt CRC */
        if (infoPtr->rmtLatch == GTI_FALSE)
        {
            hostPtr->tx.Status = GTI_EOC_INVALID_CMD;
            return (0);
        }
        /* Message will be build last */
        break;

    case GTI_EOC_CMD_WRITEREG4:  /* 0x29 d<->u Write Vendor Discretionary */
    case GTI_EOC_CMD_WRITEREG5:  /* 0x2A d<->u Write Vendor Discretionary */
    case GTI_EOC_CMD_WRITEREG9:
    case GTI_EOC_CMD_WRITEREGA:
    case GTI_EOC_CMD_WRITEREGB:
    case GTI_EOC_CMD_WRITEREGC:
    case GTI_EOC_CMD_WRITEREGD:
    case GTI_EOC_CMD_WRITEREGE:
    case GTI_EOC_CMD_WRITEREGF:
        if (hostPtr->tx.EocLen > GTI_EOC_MAX_DATA_LEN)
        {
            hostPtr->tx.Status = GTI_EOC_INVALID_LEN;
            return 0;
        }
        infoPtr->regIndex = 0;  /* Which byte of register to send */
        infoPtr->regLength = hostPtr->tx.EocLen;
        infoPtr->regData = (GTI_BYTE *) &(infoPtr->data[0]);

        for(i=0; i<hostPtr->tx.EocLen; i++)
            infoPtr->data[i] = hostPtr->tx.EocData[i];

        infoPtr->state = GTI_EOC_STATE_PREWRITE;

        /* Message will be build last */
        break;

    case GTI_EOC_CMD_READREG0:   /* 0x40 d<->u Read VendorID */
        for (; (hostPtr->tx.EocCmd != reg->regID); reg++)
            ;
        rStandard = GET_HOST_DB_B(GTISysPtr, GTI_SELCTD_STANDARD);
        if((rStandard == GTI_G_LITE_STANDARD) || (rStandard == GTI_G_DMT_STANDARD))
        {
            infoPtr->regLength = GTI_EOC_VENDOR_LEN;
        }
        else
        {
            infoPtr->regLength = 2;
        }
        infoPtr->regData = (GTI_BYTE *) &(infoPtr->data[0]);
        infoPtr->state = GTI_EOC_STATE_PREREAD;
        /* Message will be build last */
        break;

    case GTI_EOC_CMD_READREG1:   /* 0x43 d<->u Read Revision Number */
    case GTI_EOC_CMD_READREG2:   /* 0x45 d<->u Read Serial Number */
    case GTI_EOC_CMD_READREG3:   /* 0x46 d<->u Read Self-Test Result */
    case GTI_EOC_CMD_READREG4:   /* 0x49 d<->u Read Vendor Discretionary */
    case GTI_EOC_CMD_READREG5:   /* 0x4A d<->u Read Vendor Discretionary */
    case GTI_EOC_CMD_READREG6:   /* 0x4C d<->u Read Line Attenuation */
    case GTI_EOC_CMD_READREG7:   /* 0x4F d<->u Read SNR Margin */
    case GTI_EOC_CMD_READREG8:   /* 0x51 d<->u Read ATU-R Configuration */
        for (; (hostPtr->tx.EocCmd != reg->regID); reg++)
            ;
        infoPtr->regLength = reg->regLen;
        infoPtr->regData = (GTI_BYTE *) &(infoPtr->data[0]);
        infoPtr->state = GTI_EOC_STATE_PREREAD;
        /* Message will be build last */
        break;

    case GTI_EOC_CMD_READREG9:
    case GTI_EOC_CMD_READREGA:
    case GTI_EOC_CMD_READREGB:
    case GTI_EOC_CMD_READREGC:
    case GTI_EOC_CMD_READREGD:
    case GTI_EOC_CMD_READREGE:
    case GTI_EOC_CMD_READREGF:
        /* Since we don't know the length of the register, pick 32 bytes */
        infoPtr->regLength = GTI_EOC_MAX_DATA_LEN;
        infoPtr->regData = (GTI_BYTE *) &(infoPtr->data[0]);
        infoPtr->state = GTI_EOC_STATE_PREREAD;
        /* Message will be build last */
        break;

    default:
        hostPtr->tx.Status = GTI_EOC_INVALID_CMD;
        return (0);
    }

    /* Build a message here instead of within each case statement */
    /* This will save some code space for the customer */
    retEocMsg = GTI_EOC_M_MAKE_MSG( GTI_EOC_ATUR_ADDRESS,
        GTI_EOC_OPCODE_FIELD, GTI_EOC_ODD_PARITY,
        hostPtr->tx.EocCmd );
    infoPtr->saveMsg = retEocMsg;
    infoPtr->hostReqCmd = hostPtr->tx.EocCmd;
    infoPtr->status = GTI_EOC_BUSY;

    return retEocMsg;
}

/************************************************************************
* FUNCTION     : GTIEocExecCmd()
* DESCRIPTION  :
*              :
* INPUT        :
* OUTPUT       : None
* RETURN       : None
************************************************************************/
GTI_WORD GTIEocExecCmd(GTI_XCVR_INFO *GTISysPtr,
                       GTIEOC_CMD *hostPtr,
                       GTI_WORD eocMsg)
{
    GTIEOC_FIELDS  eocFields;

    /* Get EOC Message Fields */
    eocFields.addr = GTI_EOC_M_ADDR_FIELD(eocMsg);
    eocFields.isOpcode = GTI_EOC_M_OPCODE_FIELD(eocMsg);
    eocFields.parity = GTI_EOC_M_PARITY_FIELD(eocMsg);
    eocFields.isMsg = GTI_EOC_M_MSG_FIELD(eocMsg);
    eocFields.info = GTI_EOC_M_INFO_FIELD(eocMsg);
    eocFields.msg = eocMsg;

    switch( GTISysPtr->eocInfo.state )
    {
    case GTI_EOC_STATE_IDLE: /* 0 */
        return ( Gti_EocStateIdle(GTISysPtr, hostPtr, &eocFields) );

    case GTI_EOC_STATE_PREWRITE:   /* 1 */
        return ( Gti_EocStatePreWrite(&(GTISysPtr->eocInfo), hostPtr, &eocFields) );

    case GTI_EOC_STATE_WRITE:      /* 2 */
        return ( Gti_EocStateWrite(&(GTISysPtr->eocInfo), hostPtr, &eocFields) );

    case GTI_EOC_STATE_PREREAD:    /* 3 */
        return ( Gti_EocStatePreRead(&(GTISysPtr->eocInfo), hostPtr, &eocFields) );

    case GTI_EOC_STATE_READ:       /* 4 */
        return ( Gti_EocStateRead(&(GTISysPtr->eocInfo), hostPtr, &eocFields) );

    case GTI_EOC_STATE_UTC:
    case GTI_EOC_STATE_LPR:
    default:
        break;
    }

    return (0);
}
/* ************* */

#else
/************************************************************************
* FUNCTION     : GTIEocProcPcu()
* DESCRIPTION  :
*              :
* INPUT        :
* OUTPUT       : None
* RETURN       : None
************************************************************************/
void Gti_EocProcPcu(GTI_XCVR_INFO *GTISysPtr, GTIEOC_CMD *hostPtr)
{
    GTIEOC_INFO *eocInfo;
    GTI_EOC_STATUS eocStatus;

    /* Get the saved structure for EOC */
    eocInfo = (GTIEOC_INFO *)&GTISysPtr->eocInfo;
    hostPtr->rx.AutoLen = 0;
    hostPtr->rx.EocCmd = 0;
    hostPtr->rx.EocLen = 0;

    Gti_EocPcuReadStatus(GTISysPtr, &eocStatus);
    /*dying gasp */
    if (eocStatus.lprFlag)
    {
        eocInfo->status = GTI_EOC_LPR;
        hostPtr->rx.Status = GTI_EOC_LPR;
        hostPtr->rx.EocCmd = GTI_EOC_CMD_DGASP;
        WRITE_DSP_ADDR(GTISysPtr, GTI_DSP_A, GTI_ABUF_POINTER, GTI_PSRAM0W);
        WRITE_DSP_WORD(GTISysPtr, GTI_DSP_A, eocStatus.adataFlag, 0x00);
        return;
    }
    /*GTI reset*/
    if (hostPtr->tx.EocCmd == GTI_EOC_CMD_RESET)
    {
        Gti_EocPcuResetEoc(GTISysPtr, hostPtr, eocInfo);
        return;
    }

    /*UTC from ATU-R*/
    if (eocStatus.utcFlag)
    {
        hostPtr->rx.EocCmd = eocInfo->hostReqCmd;
        eocInfo->status = GTI_EOC_UTC;
        hostPtr->rx.Status = GTI_EOC_UTC;
        WRITE_DSP_ADDR(GTISysPtr, GTI_DSP_A, GTI_UTC_FLAG, GTI_PSRAM0W);
        WRITE_DSP_WORD(GTISysPtr, GTI_DSP_A, 0x00, eocStatus.commDone);
        return;
    }
    /*comm done*/
    if (eocStatus.commDone)
    {
        /* reset flag */
        WRITE_DSP_ADDR(GTISysPtr, GTI_DSP_A, GTI_UTC_FLAG, GTI_PSRAM0W);
        WRITE_DSP_WORD(GTISysPtr, GTI_DSP_A, eocStatus.utcFlag, 0x00);
        /*read command */
        if (eocInfo->hostReqCmd >= GTI_EOC_CMD_READREG0 &&
            eocInfo->hostReqCmd <= GTI_EOC_CMD_READREG8)
            Gti_EocPcuReadReg(GTISysPtr, hostPtr, &eocStatus);
        hostPtr->rx.EocCmd = eocInfo->hostReqCmd;
        hostPtr->rx.Status = GTI_EOC_COMPLETE;
        eocInfo->status = GTI_EOC_COMPLETE;
        eocInfo->hostReqCmd = 0;
        return;
    }
    /*send opcode*/
    Gti_EocPcuSendOpcode(GTISysPtr, hostPtr, eocInfo);
    /*send auto data*/
    Gti_EocPcuSendAuto(hostPtr);
    /*receive auto data */
    Gti_EocPcuAutoData(GTISysPtr, hostPtr, eocInfo, &eocStatus);

}

/************************************************************************
* FUNCTION     : Gti_EocPcuReadReg()
* DESCRIPTION  : Read value of reg from pcu data buf
*              :
* INPUT        :
* OUTPUT       : None
* RETURN       : None
************************************************************************/
void Gti_EocPcuReadReg(GTI_XCVR_INFO *GTISysPtr, GTIEOC_CMD *hostPtr,
                       GTI_EOC_STATUS *eocStatus)
{
    int iLoop;
    GTI_WORD wTemp;

    WRITE_DSP_ADDR(GTISysPtr, GTI_DSP_A, GTI_DATA_BUF, (GTI_READ_DSP |
        GTI_PSRAM0W));

    for(iLoop=0; iLoop<DIVBYTWO(eocStatus->dataLen+1); iLoop++)
    {
        wTemp = READ_DSP_WORD(GTISysPtr, GTI_DSP_A);
        hostPtr->rx.EocData[iLoop*2] =  LOBYTE(wTemp);
        hostPtr->rx.EocData[iLoop*2+1] = HIBYTE(wTemp);
    }
    hostPtr->rx.EocLen = eocStatus->dataLen+1;
}

/************************************************************************
* FUNCTION     : Gti_EocPcuResetReg()
* DESCRIPTION  : Reset EOC processing in both CO and CPE
*              :
* INPUT        :
* OUTPUT       : None
* RETURN       : None
************************************************************************/
void Gti_EocPcuResetEoc(GTI_XCVR_INFO *GTISysPtr, GTIEOC_CMD *hostPtr,
                        GTIEOC_INFO *eocInfo)
{
    /*reset co; reset current state*/
    WRITE_DSP_ADDR(GTISysPtr, GTI_DSP_A, GTI_CUR_STATE, GTI_PSRAM0W);
    WRITE_DSP_WORD(GTISysPtr, GTI_DSP_A, 0x00, 0x00 );

    /*reset cpe; send RTN opcode*/
    WRITE_DSP_ADDR(GTISysPtr, GTI_DSP_A, GTI_SEND_OPCODE, GTI_PSRAM0W);
    WRITE_DSP_WORD(GTISysPtr, GTI_DSP_A, GTI_EOC_CMD_RTN, 0x01 );
    hostPtr->rx.EocCmd = GTI_EOC_CMD_RESET;
    hostPtr->rx.Status = GTI_EOC_COMPLETE;
    hostPtr->tx.EocCmd = 0;
    hostPtr->tx.EocLen = 0;
    eocInfo->hostReqCmd = 0;
    eocInfo->status = GTI_EOC_COMPLETE;
}

/************************************************************************
* FUNCTION     : Gti_EocCmdLen()
* DESCRIPTION  : Send eoc opcode
*              :
* INPUT        :
* OUTPUT       : None
* RETURN       : GTI_BYTE
************************************************************************/

GTI_BYTE Gti_ValidEocStatCmdLen(GTIEOC_CMD *hostPtr, GTIEOC_INFO *eocInfo)
{
    GTI_BYTE gbFlag = GTI_TRUE;

    if (eocInfo->status != GTI_EOC_BUSY)
    {
        switch(hostPtr->tx.EocCmd)
        {
        case GTI_EOC_CMD_HOLD:       /* 0x01 d<->u Hold State */
        case GTI_EOC_CMD_EOD:
        case GTI_EOC_CMD_RESET:
        case GTI_EOC_CMD_SLFTEST:    /* 0x02 d<->u Self-Test */
        case GTI_EOC_CMD_REQTPU:     /* 0x13 d<->u Req Test Parameter Update */
        case GTI_EOC_CMD_VPP0:       /* 0x19 d<->u Vendor Protrietary Protocols */
        case GTI_EOC_CMD_VPP1:       /* 0x1A d<->u Vendor Protrietary Protocols */
        case GTI_EOC_CMD_VPP2:       /* 0x1C d<->u Vendor Protrietary Protocols */
        case GTI_EOC_CMD_VPP3:       /* 0x1F d<->u Vendor Protrietary Protocols */
        case GTI_EOC_CMD_RTN:        /* 0xF0 d<->u Return To Normal */
        case GTI_EOC_CMD_READREG0:   /* 0x40 d<->u Read VendorID */
        case GTI_EOC_CMD_READREG1:   /* 0x43 d<->u Read Revision Number */
        case GTI_EOC_CMD_READREG2:   /* 0x45 d<->u Read Serial Number */
        case GTI_EOC_CMD_READREG3:   /* 0x46 d<->u Read Self-Test Result */
        case GTI_EOC_CMD_READREG4:   /* 0x49 d<->u Read Vendor Discretionary */
        case GTI_EOC_CMD_READREG5:   /* 0x4A d<->u Read Vendor Discretionary */
        case GTI_EOC_CMD_READREG6:   /* 0x4C d<->u Read Line Attenuation */
        case GTI_EOC_CMD_READREG7:   /* 0x4F d<->u Read SNR Margin */
        case GTI_EOC_CMD_READREG8:   /* 0x51 d<->u Read ATU-R Configuration */
        case GTI_EOC_CMD_READREG9:
        case GTI_EOC_CMD_READREGA:
        case GTI_EOC_CMD_READREGB:
        case GTI_EOC_CMD_READREGC:
        case GTI_EOC_CMD_READREGD:
        case GTI_EOC_CMD_READREGE:
        case GTI_EOC_CMD_READREGF:
            break;

        case GTI_EOC_CMD_NOTCOR:     /* 0x0B d<->u Notify Corrupt CRC */
        case GTI_EOC_CMD_REQCOR:     /* 0x07 d<->u Request Corrupt CRC */
            eocInfo->rmtLatch = GTI_TRUE;
            break;

        case GTI_EOC_CMD_WRITEREG4:  /* 0x29 d<->u Write Vendor Discretionary */
        case GTI_EOC_CMD_WRITEREG5:  /* 0x2A d<->u Write Vendor Discretionary */
        case GTI_EOC_CMD_WRITEREG9:
        case GTI_EOC_CMD_WRITEREGA:
        case GTI_EOC_CMD_WRITEREGB:
        case GTI_EOC_CMD_WRITEREGC:
        case GTI_EOC_CMD_WRITEREGD:
        case GTI_EOC_CMD_WRITEREGE:
        case GTI_EOC_CMD_WRITEREGF:
            if (hostPtr->tx.EocLen > GTI_EOC_MAX_DATA_LEN)
            {
                gbFlag = GTI_FALSE;
                hostPtr->tx.Status = GTI_EOC_INVALID_LEN;
            }
            break;

        case GTI_EOC_CMD_NOTEND:     /* 0x0D d<->u Notify End of Corrupt CRC */
        case GTI_EOC_CMD_REQEND:     /* 0x08 d<->u Request End of Corrupt CRC */
            if (eocInfo->rmtLatch == GTI_FALSE)
            {
                hostPtr->tx.Status = GTI_EOC_INVALID_CMD;
                gbFlag = GTI_FALSE;
            }
            else
                eocInfo->rmtLatch = GTI_FALSE;
            break;

        default:
            gbFlag = GTI_FALSE;
        };
    }
    else
    {
        hostPtr->tx.Status = GTI_EOC_BUSY;
        gbFlag = GTI_FALSE;
    }
    return gbFlag;
}

/************************************************************************
* FUNCTION     : Gti_EocSendOpcode()
* DESCRIPTION  : Send eoc opcode
*              :
* INPUT        :
* OUTPUT       : None
* RETURN       : None
************************************************************************/

void Gti_EocPcuSendOpcode(GTI_XCVR_INFO *GTISysPtr, GTIEOC_CMD *hostPtr,
                          GTIEOC_INFO *eocInfo)
{
    int iLoop;

    if (hostPtr->tx.EocCmd)
    {
        if (Gti_ValidEocStatCmdLen(hostPtr, eocInfo))
        {
            /*Write command*/
            if ((hostPtr->tx.EocCmd > 0x1F) && (hostPtr->tx.EocCmd < 0x3F ))
            {
                /*write buf len in pcu*/
                WRITE_DSP_ADDR(GTISysPtr, GTI_DSP_A, GTI_MAX_BUF_LEN, GTI_PSRAM0W);
                WRITE_DSP_WORD(GTISysPtr, GTI_DSP_A, 0x00, hostPtr->tx.EocLen);

                /*write data in pcu buf*/
                WRITE_DSP_ADDR(GTISysPtr, GTI_DSP_A, GTI_DATA_BUF, GTI_PSRAM0W);

                for(iLoop=0; iLoop<DIVBYTWO(hostPtr->tx.EocLen) ; iLoop++)
                    WRITE_DSP_WORD(GTISysPtr, GTI_DSP_A, hostPtr->tx.EocData[iLoop*2],
                    hostPtr->tx.EocData[iLoop*2+1]);
            }
            /*write opcode in pcu buf & send */
            WRITE_DSP_ADDR(GTISysPtr, GTI_DSP_A, GTI_SEND_OPCODE, GTI_PSRAM0W);
            WRITE_DSP_WORD(GTISysPtr, GTI_DSP_A, hostPtr->tx.EocCmd, 0x01 );

            eocInfo->hostReqCmd = hostPtr->tx.EocCmd;
            eocInfo->status = GTI_EOC_BUSY;
            hostPtr->rx.Status = GTI_EOC_BUSY;
            hostPtr->tx.Status = GTI_EOC_COMPLETE;
        }
        hostPtr->tx.EocCmd = 0;
        hostPtr->tx.EocLen = 0;
    }
}

/************************************************************************
* FUNCTION     : Gti_EocSendAuto()
* DESCRIPTION  : Send auto data
*              :
* INPUT        :
* OUTPUT       : None
* RETURN       : None
************************************************************************/

void Gti_EocPcuSendAuto(GTIEOC_CMD *hostPtr)
{
    if (hostPtr->tx.AutoLen > 0)
    {
        hostPtr->tx.Status = GTI_EOC_INVALID_CMD;
    }
}

/************************************************************************
* FUNCTION     : Gti_EocPcuAutoData()
* DESCRIPTION  : Receive Auto data
*              :
* INPUT        :
* OUTPUT       : None
* RETURN       : None
************************************************************************/

void Gti_EocPcuAutoData(GTI_XCVR_INFO *GTISysPtr, GTIEOC_CMD *hostPtr,
                        GTIEOC_INFO *eocInfo, GTI_EOC_STATUS *eocStatus)
{
    int iLoop;
    GTI_WORD wTemp;

    if (eocStatus->adataFlag > 0 && eocInfo->status != GTI_EOC_BUSY)
    {
        if (eocInfo->tick >= GTI_MAX_AUTO_WAIT)
        {
            /*Reset flag */
            WRITE_DSP_ADDR(GTISysPtr, GTI_DSP_A, GTI_ABUF_POINTER, GTI_PSRAM0W);
            WRITE_DSP_WORD(GTISysPtr, GTI_DSP_A, 0x00, eocStatus->lprFlag);

            WRITE_DSP_ADDR(GTISysPtr, GTI_DSP_A, GTI_AUTO_BUF, (GTI_READ_DSP |
                GTI_PSRAM0W));

            for(iLoop = 0; iLoop<DIVBYTWO(eocStatus->adataFlag); iLoop++)
            {
                wTemp = READ_DSP_WORD(GTISysPtr, GTI_DSP_A);
                hostPtr->rx.AutoData[iLoop*2] =  LOBYTE(wTemp);
                hostPtr->rx.AutoData[iLoop*2+1] = HIBYTE(wTemp);
            }
            hostPtr->rx.AutoLen = eocStatus->adataFlag;
            hostPtr->rx.Status = GTI_EOC_COMPLETE;
        }
        else
            eocInfo->tick++;    /*inc wait counter*/
    }
    else
        eocInfo->tick = 0;
}

/************************************************************************
* FUNCTION     : Gti_EocPcuReadStatus()
* DESCRIPTION  : Read status from pcu
*              :
* INPUT        :
* OUTPUT       : None
* RETURN       : None
************************************************************************/
void Gti_EocPcuReadStatus(GTI_XCVR_INFO *GTISysPtr, GTI_EOC_STATUS *eocStatus)
{
    GTI_WORD wTemp;

    /* Turn ON Auto Increment */
    CONFIGURE_DSP(GTISysPtr, GTI_DSP_A, GTI_AI);
    WRITE_DSP_ADDR(GTISysPtr, GTI_DSP_A, GTI_ABUF_POINTER, (GTI_READ_DSP
        | GTI_PSRAM0W));
    wTemp = READ_DSP_WORD(GTISysPtr, GTI_DSP_A);
    eocStatus->adataFlag = LOBYTE(wTemp);
    eocStatus->lprFlag = HIBYTE(wTemp);
    wTemp = READ_DSP_WORD(GTISysPtr, GTI_DSP_A);
    eocStatus->utcFlag = LOBYTE(wTemp);
    eocStatus->commDone = HIBYTE(wTemp);
    WRITE_DSP_ADDR(GTISysPtr, GTI_DSP_A, GTI_CUR_STATE, (GTI_READ_DSP
        | GTI_PSRAM0W));
    wTemp = READ_DSP_WORD(GTISysPtr, GTI_DSP_A);
    eocStatus->curState = LOBYTE(wTemp);
    eocStatus->dataLen = HIBYTE(wTemp);
}

#endif  /*ifdef GTI_EOC_PCU*/










