/*
  @(#)gtipeoc.c	1.11.4.1
*/

/************************************************************************
* FILE NAME   : gtipeoc.c                                              *
* DESCRIPTION :  ATU-R *
*                                                                      *
* COMMENT     : *
*                                                                      *
* (C) Copyright 1999 by GlobeSpan Semiconductor Inc                    *
*                                                                      *
************************************************************************/

/************************************************************************
*                             HEADER FILES
************************************************************************/
#include "gti.h"
#include "gti_eoca.h"
#include "gtidsp.h"
#include "gtistat.h"
#include "gtiextn.h"

/************************************************************************
*                             DATA TYPES
************************************************************************/
#define NOT_USED(v)                    ((void) v)
#define MAX_RMT_VENDOR_ID_STATUS_LEN   GTI_RMT_VENDOR_ID_G_DMT_STATUS_LEN
#define GTI_EOC_NUM_OF_DGASP_MSG       8
#define GTI_EOC_DYING_GASP_MSG         0xE7BF

/************************************************************************
*                          DATA DECLARATIONS
************************************************************************/

extern GTIEOC_DATA_REG  eocDataReg;

GTIEOC_REG_TABLE GTIEocRegInfo[]=
{
    {GTI_EOC_CMD_READREG0,  (&(eocDataReg.vendorId[0])),    GTI_EOC_VENDOR_LEN},
    {GTI_EOC_CMD_READREG1,  (&(eocDataReg.revisionNum[0])), GTI_EOC_REV_NUM_LEN},
    {GTI_EOC_CMD_READREG2,  (&(eocDataReg.serialNum[0])),   GTI_EOC_SERIAL_LEN},
    {GTI_EOC_CMD_READREG3,  (&(eocDataReg.selfTest[0])),    GTI_EOC_SELFTEST_LEN},
    {GTI_EOC_CMD_READREG4,  (&(eocDataReg.vendorDisc1[0])), GTI_EOC_VEN_DISC1_LEN},
    {GTI_EOC_CMD_READREG5,  (&(eocDataReg.vendorDisc2[0])), GTI_EOC_VEN_DISC2_LEN},
    {GTI_EOC_CMD_READREG6,  (&(eocDataReg.lineAtten[0])),   GTI_EOC_ATTENU_LEN},
    {GTI_EOC_CMD_READREG7,  (&(eocDataReg.snrMargin[0])),   GTI_EOC_SNRMARGN_LEN},
    {GTI_EOC_CMD_READREG8,  (&(eocDataReg.aturConfig[0])),  GTI_EOC_CONFIG_LEN},
    {GTI_EOC_CMD_READREGA,  (&(eocDataReg.registryA[0])),   GTI_EOC_REGA_LEN},
    {GTI_EOC_CMD_WRITEREG0, (&(eocDataReg.vendorId[0])),    GTI_EOC_VENDOR_LEN},
    {GTI_EOC_CMD_WRITEREG1, (&(eocDataReg.revisionNum[0])), GTI_EOC_REV_NUM_LEN},
    {GTI_EOC_CMD_WRITEREG2, (&(eocDataReg.serialNum[0])),   GTI_EOC_SERIAL_LEN},
    {GTI_EOC_CMD_WRITEREG3, (&(eocDataReg.selfTest[0])),    GTI_EOC_SELFTEST_LEN},
    {GTI_EOC_CMD_WRITEREG4, (&(eocDataReg.vendorDisc1[0])), GTI_EOC_VEN_DISC1_LEN},
    {GTI_EOC_CMD_WRITEREG5, (&(eocDataReg.vendorDisc2[0])), GTI_EOC_VEN_DISC2_LEN},
    {GTI_EOC_CMD_WRITEREG6, (&(eocDataReg.lineAtten[0])),   GTI_EOC_ATTENU_LEN},
    {GTI_EOC_CMD_WRITEREG7, (&(eocDataReg.snrMargin[0])),   GTI_EOC_SNRMARGN_LEN},
    {GTI_EOC_CMD_WRITEREG8, (&(eocDataReg.aturConfig[0])),  GTI_EOC_CONFIG_LEN},
    {GTI_EOC_CMD_WRITEREGA,  (&(eocDataReg.registryA[0])),  GTI_EOC_REGA_LEN}
};

/************************************************************************
*                        FUNCTION DECLARATIONS
************************************************************************/

/************************************************************************
*                          FUNCTION CALLS
************************************************************************/

/************************************************************************
* FUNCTION     : GTIEocInit()
* DESCRIPTION  :
*              :
* INPUT        :
* OUTPUT       : None
* RETURN       : None
************************************************************************/
void GTIEocInit(GTIEOC_INFO *eInfoPtr)
{
    eInfoPtr->txEocMsg = 0x0;
    eInfoPtr->lastRxEocMsg = 0;
    eInfoPtr->nIdentEocMsg = 0;
    eInfoPtr->rxErrCnt = 0;
    eInfoPtr->latch = GTI_FALSE;
    eInfoPtr->state = GTI_EOC_STATE_IDLE;
    eInfoPtr->saveMsg = 0;
    eInfoPtr->status = GTI_EOC_COMPLETE;

    eInfoPtr->eocDisabled = GTI_TRUE;
    eInfoPtr->frameType3Found = GTI_FALSE;
    eInfoPtr->txEocOffset = 9;

   eInfoPtr->specialAction = GTI_FALSE;
   eInfoPtr->sendCompressedData = GTI_FALSE;
   eInfoPtr->ignoreReceivedData = GTI_FALSE;
}

/************************************************************************
* FUNCTION     : GTIEocDataReg()
* DESCRIPTION  :
*              :
* INPUT        :
* OUTPUT       : None
* RETURN       : None
************************************************************************/
void GTIEocDataReg(GTIEOC_INFO *infoPtr, GTIEOC_CMD *hostPtr)
{
    GTIEOC_REG_TABLE *reg = (GTIEOC_REG_TABLE *) GTIEocRegInfo;
    GTI_BYTE i;

    switch (hostPtr->tx.EocCmd)
    {

    case GTI_EOC_CMD_WRITEREG0:
    case GTI_EOC_CMD_WRITEREG1:
    case GTI_EOC_CMD_WRITEREG2:
    case GTI_EOC_CMD_WRITEREG3:
    case GTI_EOC_CMD_WRITEREG4:
    case GTI_EOC_CMD_WRITEREG5:
    case GTI_EOC_CMD_WRITEREG6:
    case GTI_EOC_CMD_WRITEREG7:
    case GTI_EOC_CMD_WRITEREG8:
        if (hostPtr->tx.EocLen > 0)
        {
            for ( ; (hostPtr->tx.EocCmd != reg->regID); reg++)
                ;
            for(i=0; i<(hostPtr->tx.EocLen < reg->regLen ?
		hostPtr->tx.EocLen: reg->regLen); i++)
                reg->regData[i] = hostPtr->tx.EocData[i];
            for( ; i<reg->regLen; i++)/*rest fill with zero*/
                reg->regData[i] = 0; 
        }
        else
            hostPtr->tx.Status = GTI_EOC_INVALID_LEN;
        break;

    case GTI_EOC_CMD_RESET:
        GTIEocInit(infoPtr);
        break;

    default:
        hostPtr->tx.Status = GTI_EOC_INVALID_CMD;
        break;
    }
}

/************************************************************************
* FUNCTION     : GTIEocSpecialAction()
* DESCRIPTION  : Take special action in certain cases
*              : 
* INPUT        : 
* OUTPUT       : None
* RETURN       : None
************************************************************************/
GTI_BYTE GTIEocSpecialAction(GTI_XCVR_INFO *GTISysPtr)
{
GTI_WORD   rmtVendorIdStatus[MAX_RMT_VENDOR_ID_STATUS_LEN];
GTI_WORD   rmtVendorRevStatus[GTI_RMT_VENDOR_REV_STATUS_LEN];
GTI_WORD   statusLen=0;

   /* Find remote information - VendorID and VendorRev */
   statusLen = Gti_SStatusXCVR(GTISysPtr, GTI_RMT_VENDOR_REV_STATUS, &(rmtVendorRevStatus[0]),
				  	  GTI_RMT_VENDOR_REV_STATUS_LEN, GTI_DSP_A);
   if (!statusLen)
       return GTI_FALSE;

   /* Possibly newer version of the CO code */
   if (rmtVendorRevStatus[0] != 0x0000)
	   return GTI_FALSE;

   statusLen = Gti_SStatusXCVR(GTISysPtr, GTI_RMT_VENDOR_ID_STATUS, &(rmtVendorIdStatus[0]),
				  	  MAX_RMT_VENDOR_ID_STATUS_LEN, GTI_DSP_A);
   switch (statusLen)
   {
        case MAX_RMT_VENDOR_ID_STATUS_LEN:
        case GTI_RMT_VENDOR_ID_STATUS_LEN:
            /* Is it GSPN VendorID? */
            if (rmtVendorIdStatus[0] == GTI_ID_GLOBESPAN)
                return GTI_TRUE;
              
            /* If the connection was made for G.DMT, check different VendorID */
            if (rmtVendorIdStatus[0] == 0x4350)
            {
                if (rmtVendorIdStatus[1] == 0x5347)
                {
                    if (rmtVendorIdStatus[2] == 0xB500)
                        return GTI_TRUE;
                }
            }
            break;
            
        default:
            break;
   }

   return GTI_FALSE;
}

/************************************************************************
* FUNCTION     : GTIEocProc()
* DESCRIPTION  :
*              :
* INPUT        :
* OUTPUT       : None
* RETURN       : None
************************************************************************/
void GTIEocProc(GTI_XCVR_INFO *GTISysPtr, GTIEOC_CMD *hostPtr)
{
    /* Buffer that is stored in the DSP for each Superframe */
    GTI_WORD    rxBuff[GTI_EOC_MAX_NUMOF_MSG];
    GTI_WORD    txBuff[GTI_EOC_MAX_NUMOF_MSG];
    GTI_BYTE    txBuffIndex=0;
    GTI_WORD    i;
    GTIEOC_INFO *eocInfoPtr;
    GTI_WORD    newMsg;
    GTI_WORD    tempMsg=0;
    GTI_BYTE    txAutoIndex=0;
    GTI_WORD    dspState=0;

    /* Get the saved structure for EOC */
    eocInfoPtr = (GTIEOC_INFO *)&GTISysPtr->eocInfo;

    /* Initialize the output parameters */
    hostPtr->rx.AutoLen = 0;
    hostPtr->rx.EocCmd = 0;
    hostPtr->rx.EocLen = 0;
    hostPtr->rx.Status = GTI_EOC_COMPLETE;
    hostPtr->tx.Status = GTI_EOC_COMPLETE;  /* OK */

    for (i=0; i<GTI_EOC_MAX_NUMOF_MSG; i++)
        txBuff[i] = 0;

    if (eocInfoPtr->eocDisabled)
    {
        /* Check to see the if we are in DATA_MODE */
        (void)Gti_SStatusXCVR(GTISysPtr, GTI_OP_STATE, &dspState, sizeof(dspState), 0);
        if (dspState != GTI_DATA_OP)
            return;

        eocInfoPtr->eocDisabled = GTI_FALSE;

        /* Read the Framing Structure Type from the DSP-PCU */
        WRITE_DSP_ADDR(GTISysPtr, GTI_DSP_A, 0x6, (GTI_READ_DSP | GTI_PSRAM0W) );
        if (((READ_DSP_WORD(GTISysPtr, GTI_DSP_A)) & 0x600) == 0x600)
        {
            eocInfoPtr->frameType3Found = GTI_TRUE;
            eocInfoPtr->txEocOffset = 5;
        }
        /* Does it require special action from the CPE side? */
        eocInfoPtr->sendCompressedData = GTI_FALSE;
        eocInfoPtr->ignoreReceivedData = GTI_FALSE;
        eocInfoPtr->specialAction = (GTIEocSpecialAction(GTISysPtr)) ? GTI_TRUE : GTI_FALSE;
    }

    /* **** RX **** */
    /* Read the RX Flag */
    WRITE_DSP_ADDR(GTISysPtr, GTI_DSP_A, GTI_EOC_RXBUFF_FLAG_OFFSET, (GTI_READ_DSP | GTI_ISRAM) );
    if (READ_DSP_WORD(GTISysPtr, GTI_DSP_A) == GTI_EOC_RXBUFF_FULL)
    {

	/* check if it's okay to tx before processing */
        WRITE_DSP_ADDR(GTISysPtr, GTI_DSP_A, GTI_EOC_TXBUFF_FLAG_OFFSET, (GTI_READ_DSP | GTI_ISRAM) );
        if (READ_DSP_WORD(GTISysPtr, GTI_DSP_A) != GTI_EOC_TXBUFF_EMPTY)
	{
            return; /* not okay */
	}

        /* Turn ON the Auto Increment */
        CONFIGURE_DSP(GTISysPtr, GTI_DSP_A, GTI_AI);

        /* Read the Data from the DSP */
        WRITE_DSP_ADDR(GTISysPtr, GTI_DSP_A, GTI_EOC_RXBUFF_OFFSET, (GTI_READ_DSP | GTI_ISRAM) );
        for(i=0; i<GTI_EOC_MAX_NUMOF_MSG; i++)
            rxBuff[i] = READ_DSP_WORD(GTISysPtr, GTI_DSP_A);

        /* Turn OFF the Auto Increment */
        CONFIGURE_DSP(GTISysPtr, GTI_DSP_A, GTI_NO_AI);
        /* Reset the RX Flag to empty Buffer */
        WRITE_DSP_ADDR(GTISysPtr, GTI_DSP_A, GTI_EOC_RXBUFF_FLAG_OFFSET, GTI_ISRAM );
        WRITE_DSP_WORD(GTISysPtr, GTI_DSP_A, 0, GTI_EOC_RXBUFF_EMPTY);

        /* ***** */
        /* Process the RX Information */
        for (i=0; i<GTI_EOC_MAX_NUMOF_MSG; i++)
        {
            /* NOT (crc, ib[0-2], or Synch_Byte) */
            if( !((i == 0) || (i == GTI_EOC_FRAME_34) ||
                (rxBuff[i] == GTI_EOC_SYNCH_CONTROL) ||
                ((eocInfoPtr->frameType3Found) && (i>GTI_EOC_FRAME_34))) )
            {
                /* Only EOC Message to be processed */
                newMsg = rxBuff[i];

                /* Check, if the overhead of the eoc is there */
                if ( (newMsg & GTI_EOC_OVERHEAD_MASK) != GTI_EOC_OVERHEAD_MASK )
                {
                    eocInfoPtr->rxErrCnt++;
                }
                else if( !((newMsg & GTI_EOC_ADDRESS_MASK)==GTI_EOC_ATUR_ADDRESS) )
                {
                    eocInfoPtr->rxErrCnt++;
                }
                /* Check if this is an autonomous message */
                else if ( !(newMsg & GTI_EOC_AUTONOMOUS_MASK) )
                {
                    /* Store the AUTONOMOUS MESSAGE Into the Data Structure */
                    hostPtr->rx.AutoData[hostPtr->rx.AutoLen++]=GTI_EOC_M_INFO_FIELD(newMsg);
                }
                /* Compare the new message(2Byte) with lastRxEocMsg */
                else if (newMsg == eocInfoPtr->lastRxEocMsg)
                {
                    eocInfoPtr->nIdentEocMsg++;
                    if (eocInfoPtr->nIdentEocMsg == GTI_EOC_MAGIC_NUM)
                    {
                        /* Execute the incoming message */
                        tempMsg = GTIEocExecCmd(GTISysPtr, hostPtr, newMsg);
                        if (tempMsg)
                            eocInfoPtr->txEocMsg = tempMsg;
                    }
                    txBuff[ (txBuffIndex++) ] = eocInfoPtr->txEocMsg;
                }
                else
                {
                    eocInfoPtr->nIdentEocMsg=1;
                    eocInfoPtr->lastRxEocMsg = eocInfoPtr->txEocMsg = newMsg;
                    txBuff[ (txBuffIndex++) ] = eocInfoPtr->txEocMsg;  /* Echo */
                }
            } /* if */
        } /* for */
    }

    /* take any specialAction */
    if (eocInfoPtr->specialAction)
    {
        if (eocInfoPtr->sendCompressedData)
        {
            tempMsg = GTI_EOC_M_MAKE_MSG( GTI_EOC_ATUR_ADDRESS,
                    GTI_EOC_DATA_FIELD, GTI_EOC_ODD_PARITY,
                    (eocInfoPtr->regData[eocInfoPtr->regIndex]) );
            eocInfoPtr->regIndex++;
            txBuff[ (txBuffIndex++) ] = tempMsg;
            txBuff[ (txBuffIndex++) ] = tempMsg;
            txBuff[ (txBuffIndex++) ] = tempMsg;

            tempMsg = GTI_EOC_M_MAKE_MSG(GTI_EOC_ATUR_ADDRESS,
                    GTI_EOC_OPCODE_FIELD, GTI_EOC_ODD_PARITY,
                    GTI_EOC_CMD_EOD);
            txBuff[ (txBuffIndex++) ] = tempMsg;
            txBuff[ (txBuffIndex++) ] = tempMsg;
            txBuff[ (txBuffIndex++) ] = tempMsg;

            tempMsg = GTI_EOC_M_MAKE_MSG( GTI_EOC_ATUR_ADDRESS,
                    GTI_EOC_OPCODE_FIELD, GTI_EOC_ODD_PARITY,
                    GTI_EOC_CMD_RTN);
            txBuff[ (txBuffIndex++) ] = tempMsg;
            txBuff[ (txBuffIndex++) ] = tempMsg;
            txBuff[ (txBuffIndex++) ] = tempMsg;

            eocInfoPtr->state = GTI_EOC_STATE_IDLE;
            /* Turn OFF one flag and turn another one ON */
            eocInfoPtr->sendCompressedData = GTI_FALSE;
            eocInfoPtr->ignoreReceivedData = GTI_TRUE;
        }
        else if (eocInfoPtr->ignoreReceivedData)
            txBuffIndex=0; /* Ignore any received data and therefore do not transmit */
    }

    /* ???(MAY BE) DEBUG ONLY */
    if (hostPtr->tx.EocCmd)
        GTIEocDataReg(eocInfoPtr, hostPtr);
    /* ???(MAY BE) DEBUG ONLY */
        
    /* Send-out dying gasp. Note: user needs to abort the DSL-connection after
    ** Dying Gasp message is sent out on the line (Message will be send out within 50msec.)
    */
    if (hostPtr->tx.EocCmd == GTI_EOC_CMD_DGASP)
    {
       txBuffIndex = 0;
       for(i=0; i < GTI_EOC_NUM_OF_DGASP_MSG; i++, txBuffIndex++)
        txBuff[i] = GTI_EOC_DYING_GASP_MSG;
    }

    /* ***** SANITY CHECK ***** */
    /* Is there something to transmit? */
    if (!( txBuffIndex || hostPtr->tx.AutoLen ))
        return;
    if (hostPtr->tx.AutoLen > GTI_EOC_MAX_AUTONOMOUS_LEN)
    {
        hostPtr->tx.Status = (GTI_WORD)((hostPtr->tx.Status) ? GTI_EOC_ERROR : GTI_EOC_INVALID_LEN);
        txAutoIndex = hostPtr->tx.AutoLen;   /* Do not transmit */
    }

    /* ***** TX ***** */
    WRITE_DSP_ADDR(GTISysPtr, GTI_DSP_A, GTI_EOC_TXBUFF_FLAG_OFFSET, (GTI_READ_DSP | GTI_ISRAM) );
    if (READ_DSP_WORD(GTISysPtr, GTI_DSP_A) == GTI_EOC_TXBUFF_EMPTY)
    {
        /* Turn ON the Auto Increment */
        CONFIGURE_DSP(GTISysPtr, GTI_DSP_A, GTI_AI);
        /* Write the Data to the DSP */
        WRITE_DSP_ADDR(GTISysPtr, GTI_DSP_A, GTI_EOC_TXBUFF_OFFSET, GTI_ISRAM );
        for (i=0; i<GTI_EOC_MAX_NUMOF_MSG; i++)
        {
        /* Note: Frame 0 (CRC), Frame 1 (IB7:0), Frame 34 (IB15:8), and
            Frame 35 (IB23:16) will be over written by the PCU */
            if ( (i == 0) || (i == GTI_EOC_FRAME_34) )
                WRITE_DSP_WORD(GTISysPtr, GTI_DSP_A, 0xFF, 0xFF);
            /* TEMP FIX */
            else if (i < eocInfoPtr->txEocOffset)
            {
                /* Write Control Synch. Byte */
                WRITE_DSP_WORD(GTISysPtr, GTI_DSP_A,
                    (GTI_EOC_SYNCH_CONTROL & 0xFF),
                    ((GTI_EOC_SYNCH_CONTROL >> 8) & 0xFF) );
            }
            /* TEMP FIX */
            else if((eocInfoPtr->frameType3Found) && (i>GTI_EOC_FRAME_34))
            {
                ; /* Do not Write Anything */
            }
            else
            {
                /* Write EOC command to the DSP */
                if ( txBuffIndex )
                {
                /* NOTE: NEED TO be careful here because we know that we will not be tx-ing
                    more than 3 command at a time */
                    WRITE_DSP_WORD(GTISysPtr, GTI_DSP_A,
                        /* TEMP FIX */
                        (GTI_BYTE)(txBuff [i- eocInfoPtr->txEocOffset] & 0xFF),
                        (GTI_BYTE)((txBuff[i- eocInfoPtr->txEocOffset] >> 8) & 0xFF) );
                    /* TEMP FIX */
                    txBuffIndex--;
                }
                /* Write the AUTONOMOUS Message to the DSP */
                else if (txAutoIndex < hostPtr->tx.AutoLen)
                {
                    WRITE_DSP_WORD(GTISysPtr, GTI_DSP_A,
                        (GTI_BYTE)(GTI_EOC_AUTONOMOUS_MSG_CP | ((hostPtr->tx.AutoData[txAutoIndex]&0x01)<<7)),
                        (GTI_BYTE)(hostPtr->tx.AutoData[txAutoIndex] | GTI_EOC_OVERHEAD_B1));
                    txAutoIndex++;
                }
                /* Write Control Synch. Byte */
                else
                {
                    WRITE_DSP_WORD(GTISysPtr, GTI_DSP_A,
                        (GTI_EOC_SYNCH_CONTROL & 0xFF),
                        ((GTI_EOC_SYNCH_CONTROL >> 8) & 0xFF) );
                }
            }
        }

      UPDATE_DSP(GTISysPtr);
      
        /* Turn OFF the Auto Increment */
        CONFIGURE_DSP(GTISysPtr, GTI_DSP_A, GTI_NO_AI);
        /* Set the Flag to say GO */
        WRITE_DSP_ADDR(GTISysPtr, GTI_DSP_A, GTI_EOC_TXBUFF_FLAG_OFFSET, GTI_ISRAM );
        WRITE_DSP_WORD(GTISysPtr, GTI_DSP_A, 0, GTI_EOC_TXBUFF_FULL);
    }
}

/************************************************************************
* GTIEocStateIdle
************************************************************************/
GTI_WORD GTIEocStateIdle(GTI_XCVR_INFO *GTISysPtr,
                         GTIEOC_CMD *hostPtr,
                         GTIEOC_FIELDS *fields)
{
    GTIEOC_INFO *infoPtr;
    GTIEOC_REG_TABLE *reg = (GTIEOC_REG_TABLE *) GTIEocRegInfo;
    GTI_WORD retEocMsg=0;
    GTI_BYTE bTemp;

    GTI_BYTE rStandard;
#ifdef GTI_USB        
    GTI_WORD wValue;
    GTI_BYTE value;
#endif /* GTI_USB */
    /* Get the saved structure for EOC */
    infoPtr = (GTIEOC_INFO *)&GTISysPtr->eocInfo;

    if (infoPtr->specialAction)
    {
        /* We only expect SNR margin read */
        if (fields->info != GTI_EOC_CMD_READREG7) /* 0x4F d<->u Read SNR Margin */
            return 0;

        infoPtr->sendCompressedData = GTI_TRUE;
        infoPtr->ignoreReceivedData = GTI_FALSE;
    }

    /* Should be able to Rx any Messages */
    switch( fields->info )
    {
    case GTI_EOC_CMD_HOLD:       /* 0x01 d<->u Hold State */
        /* Do Nothing */
        break;

    case GTI_EOC_CMD_RTN:        /* 0xF0 d<->u Return To Normal */
        if (infoPtr->latch)
        {
            WRITE_DSP_ADDR(GTISysPtr, GTI_DSP_A, GTI_EOC_REQCOR_OFFSET, GTI_PSRAM0W );
            WRITE_DSP_WORD(GTISysPtr, GTI_DSP_A, GTI_EOC_REQCOR_DISABLE, 0);
            UPDATE_DSP(GTISysPtr);

            infoPtr->latch = GTI_FALSE;
        }
        hostPtr->rx.EocCmd = fields->info;
        break;

    case GTI_EOC_CMD_REQCOR:     /* 0x07 d<->u Request Corrupt CRC */
        hostPtr->rx.EocCmd = fields->info;
        WRITE_DSP_ADDR(GTISysPtr, GTI_DSP_A, GTI_EOC_REQCOR_OFFSET, GTI_PSRAM0W );
        WRITE_DSP_WORD(GTISysPtr, GTI_DSP_A, GTI_EOC_REQCOR_ENABLE, 0);
        UPDATE_DSP(GTISysPtr);

        infoPtr->latch = GTI_TRUE;
        break;

    case GTI_EOC_CMD_REQEND:     /* 0x08 d<->u Request End of Corrupt CRC */
        hostPtr->rx.EocCmd = fields->info;
        WRITE_DSP_ADDR(GTISysPtr, GTI_DSP_A, GTI_EOC_REQCOR_OFFSET, GTI_PSRAM0W );
        WRITE_DSP_WORD(GTISysPtr, GTI_DSP_A, GTI_EOC_REQCOR_DISABLE, 0);
        UPDATE_DSP(GTISysPtr);

        infoPtr->latch = GTI_FALSE;
        break;

    case GTI_EOC_CMD_NOTCOR:     /* 0x0B d<->u Notify Corrupt CRC */
    case GTI_EOC_CMD_NOTEND:     /* 0x0D d<->u Notify End of Corrupt CRC */


        hostPtr->rx.EocCmd = fields->info;
        break;
      
    case GTI_EOC_CMD_GNTPDN:

        hostPtr->rx.Status = GTI_EOC_STATUS_GNTPDN;
        hostPtr->rx.EocCmd = fields->info;
        break;

    case GTI_EOC_CMD_REJPDN:
   	    hostPtr->rx.Status = GTI_EOC_STATUS_REJPDN;
        hostPtr->rx.EocCmd = fields->info;
        break;

    case GTI_EOC_CMD_SLFTEST:    /* 0x02 d<->u Self-Test */
    case GTI_EOC_CMD_REQTPU:     /* 0x13 d<->u Req Test Parameter Update */
        hostPtr->rx.EocCmd = fields->info;
        break;

    case GTI_EOC_CMD_WRITEREG4:  /* 0x29 d<->u Write Vendor Discretionary */
        infoPtr->regId = fields->info;
        infoPtr->regData = (GTI_BYTE *)&(eocDataReg.vendorDisc1[0]);
        infoPtr->regIndex = 0;
        infoPtr->regLength = GTI_EOC_VEN_DISC1_LEN;
        infoPtr->state = GTI_EOC_STATE_WRITE;
        hostPtr->rx.EocCmd = fields->info;
        break;

    case GTI_EOC_CMD_WRITEREG5:  /* 0x2A d<->u Write Vendor Discretionary */
        infoPtr->regId = fields->info;
        infoPtr->regData = (GTI_BYTE *)&(eocDataReg.vendorDisc2[0]);
        infoPtr->regIndex = 0;
        infoPtr->regLength = GTI_EOC_VEN_DISC2_LEN;
        infoPtr->state = GTI_EOC_STATE_WRITE;
        hostPtr->rx.EocCmd = fields->info;
        break;

    case GTI_EOC_CMD_WRITEREGA:
        infoPtr->regId = fields->info;
        infoPtr->regData = (GTI_BYTE *)&(eocDataReg.registryA[0]);
        infoPtr->regIndex = 0;
        infoPtr->regLength = GTI_EOC_REGA_LEN;
        infoPtr->state = GTI_EOC_STATE_WRITE;
        hostPtr->rx.EocCmd = fields->info;
        break;

    case GTI_EOC_CMD_READREG0:   /* 0x40 d<->u Read VendorID */
        hostPtr->rx.EocCmd = fields->info;
        infoPtr->regId = fields->info;
        for (; (fields->info != reg->regID); reg++)
            ;
        infoPtr->regData = (GTI_BYTE *)reg->regData;

        rStandard = GET_HOST_DB_B(GTISysPtr, GTI_SELCTD_STANDARD);
        infoPtr->regLength =(GTI_BYTE)(((rStandard == GTI_G_LITE_STANDARD) || (rStandard == GTI_G_DMT_STANDARD)) ? GTI_EOC_VENDOR_LEN : 2);
        infoPtr->regIndex = 0;  /* Which byte of register to send */
        infoPtr->state = GTI_EOC_STATE_READ;
        break;

    case GTI_EOC_CMD_READREG6:   /* 0x4C d<->u Read Line Attenuation */
#ifdef GTI_USB    
        if(SStatusXCVR(GTISysPtr, GTI_ATTENUATION_STATUS, &wValue, GTI_ATTENUATION_STATUS_LEN, GTI_DSP_A) )
            eocDataReg.lineAtten[ 0 ]  = (GTI_BYTE)(wValue & 0x00ff);
        else
            eocDataReg.lineAtten[ 0 ]  = 0xFF;
#endif /* GTI_USB */
        hostPtr->rx.EocCmd = fields->info;
        infoPtr->regId = fields->info;
        for (; (fields->info != reg->regID); reg++)
            ;
        infoPtr->regData = (GTI_BYTE *)reg->regData;
        infoPtr->regLength = reg->regLen;
        infoPtr->regIndex = 0;  /* Which byte of register to send */
        infoPtr->state = GTI_EOC_STATE_READ;
        break;

    case GTI_EOC_CMD_READREG7:   /* 0x4F d<->u Read SNR Margin */
#ifdef GTI_USB    
        if(SStatusXCVR(GTISysPtr, GTI_LCL_SNR_MARGIN, &value, GTI_LCL_SNR_MARGIN_LEN, GTI_DSP_A) )
            eocDataReg.snrMargin[ 0 ]  = value;
#endif /* GTI_USB */
        hostPtr->rx.EocCmd = fields->info;
        infoPtr->regId = fields->info;
        for (; (fields->info != reg->regID); reg++)
            ;
        infoPtr->regData = (GTI_BYTE *)reg->regData;
        infoPtr->regLength = reg->regLen;
        infoPtr->regIndex = 0;  /* Which byte of register to send */
        infoPtr->state = GTI_EOC_STATE_READ;
        break;

    case GTI_EOC_CMD_READREG1:   /* 0x43 d<->u Read Revision Number */
    case GTI_EOC_CMD_READREG2:   /* 0x45 d<->u Read Serial Number */
    case GTI_EOC_CMD_READREG3:   /* 0x46 d<->u Read Self-Test Result */
    case GTI_EOC_CMD_READREG4:   /* 0x49 d<->u Read Vendor Discretionary */
    case GTI_EOC_CMD_READREG5:   /* 0x4A d<->u Read Vendor Discretionary */
    case GTI_EOC_CMD_READREG8:   /* 0x51 d<->u Read ATU-R Configuration */
    case GTI_EOC_CMD_READREGA:   /* 0x54 d<->u Read Registry A Power Management*/

        hostPtr->rx.EocCmd = fields->info;
        infoPtr->regId = fields->info;
        for (; (fields->info != reg->regID); reg++)
            ;
        infoPtr->regData = (GTI_BYTE *)reg->regData;
        infoPtr->regLength = reg->regLen;
        infoPtr->regIndex = 0;  /* Which byte of register to send */
        infoPtr->state = GTI_EOC_STATE_READ;
        break;

    default:
        /* Send UTC */
        bTemp = GTI_EOC_CMD_UTC;
        retEocMsg = GTI_EOC_M_MAKE_MSG(GTI_EOC_ATUR_ADDRESS,
            GTI_EOC_OPCODE_FIELD, GTI_EOC_ODD_PARITY,
            bTemp);
        /* NOTE: This should be GTIEOC_STATE_UTC */
        infoPtr->state = GTI_EOC_STATE_IDLE;
        break;
    }

    return retEocMsg;
}

/************************************************************************
* GTIEocStateWrite
************************************************************************/
GTI_WORD GTIEocStateWrite(GTI_XCVR_INFO *GTISysPtr,
                          GTIEOC_CMD *hostPtr,
                          GTIEOC_FIELDS *fields)
{
    GTIEOC_INFO *infoPtr;
    GTI_WORD retEocMsg=0;
    GTI_BYTE i, bTemp;

    /* Get the saved structure for EOC */
    infoPtr = (GTIEOC_INFO *)&GTISysPtr->eocInfo;

    if (fields->isOpcode)
    {
        /* Rx only Data and EOD Message */
        if ( fields->info == GTI_EOC_CMD_EOD )
        {
            /* Write Data to the register specified */
            for(i=0; i<infoPtr->regLength; i++)
                hostPtr->rx.EocData[i] = infoPtr->regData[i];
            hostPtr->rx.EocCmd = infoPtr->regId;
            hostPtr->rx.EocLen = infoPtr->regLength;
            infoPtr->state = GTI_EOC_STATE_IDLE;
        }
        else
        {
            /* Send UTC */;
            bTemp = GTI_EOC_CMD_UTC;
            retEocMsg = GTI_EOC_M_MAKE_MSG(GTI_EOC_ATUR_ADDRESS,
                GTI_EOC_OPCODE_FIELD, GTI_EOC_ODD_PARITY,
                bTemp);
            /* NOTE: This should be GTI_EOC_STATE_UTC */
            infoPtr->state = GTI_EOC_STATE_IDLE;
        }
    }
    else
    {
        /* This is DATA....WRITE ONLY */
        if (infoPtr->regIndex < infoPtr->regLength)
        {
            infoPtr->regData[infoPtr->regIndex] = fields->info;
            infoPtr->regIndex++;
        }
        else
        {
            /* Send EOD */;
            bTemp = GTI_EOC_CMD_EOD;
            retEocMsg = GTI_EOC_M_MAKE_MSG(GTI_EOC_ATUR_ADDRESS,
                GTI_EOC_OPCODE_FIELD, GTI_EOC_ODD_PARITY,
                bTemp);

            for(i=0; i<infoPtr->regLength; i++)
                hostPtr->rx.EocData[i] = infoPtr->regData[i];
            hostPtr->rx.EocCmd = infoPtr->regId;
            hostPtr->rx.EocLen = infoPtr->regLength;
            infoPtr->state = GTI_EOC_STATE_IDLE;
        }
    }
    return retEocMsg;
}

/************************************************************************
* GTIEocStatePreRead
************************************************************************/
GTI_WORD GTIEocStatePreRead(GTI_XCVR_INFO *GTISysPtr,
                            GTIEOC_CMD *hostPtr,
                            GTIEOC_FIELDS *fields)
{
    GTIEOC_INFO *infoPtr;
    GTI_WORD retEocMsg=0;
    GTI_BYTE bTemp;

    NOT_USED(hostPtr);

    /* Get the saved structure for EOC */
    infoPtr = (GTIEOC_INFO *)&GTISysPtr->eocInfo;

    if ( (fields->info == GTI_EOC_CMD_NEXT) && (fields->isOpcode) )
    {
        /* DEBUG ONLY */
        infoPtr->state = GTI_EOC_STATE_READ;
        retEocMsg = 0xFFFF;
        /* DEBUG ONLY */
    }
    else
    {
        /* Send UTC */;
        bTemp = GTI_EOC_CMD_UTC;
        retEocMsg = GTI_EOC_M_MAKE_MSG(GTI_EOC_ATUR_ADDRESS,
            GTI_EOC_OPCODE_FIELD, GTI_EOC_ODD_PARITY,
            bTemp);
        /* NOTE: This should be GTI_EOC_STATE_UTC */
        infoPtr->state = GTI_EOC_STATE_IDLE;
    }

    return retEocMsg;
}

/************************************************************************
* GTIEocStateRead
************************************************************************/
GTI_WORD GTIEocStateRead(GTI_XCVR_INFO *GTISysPtr,
                         GTIEOC_CMD *hostPtr,
                         GTIEOC_FIELDS *fields)
{
    GTIEOC_INFO *infoPtr;
    GTI_WORD retEocMsg=0;
    GTI_BYTE bTemp;

    NOT_USED(hostPtr);

    /* Get the saved structure for EOC */
    infoPtr = (GTIEOC_INFO *)&GTISysPtr->eocInfo;

    /* Is it OPCODE? */
    if (fields->isOpcode)
    {
        switch (fields->info)
        {
        case GTI_EOC_CMD_NEXT:
            /* Send Data */
            if (infoPtr->regIndex < infoPtr->regLength)
            {
                retEocMsg = GTI_EOC_M_MAKE_MSG( GTI_EOC_ATUR_ADDRESS,
                    GTI_EOC_DATA_FIELD, (fields->parity<<5),
                    (infoPtr->regData[infoPtr->regIndex]) );
                infoPtr->regIndex++;
            }
            else
            {
                /* Send End of Data */
                bTemp = GTI_EOC_CMD_EOD;
                retEocMsg = GTI_EOC_M_MAKE_MSG(GTI_EOC_ATUR_ADDRESS,
                    GTI_EOC_OPCODE_FIELD, GTI_EOC_ODD_PARITY,
                    bTemp);
                /* Do not change the state */
            }
            break;

        case GTI_EOC_CMD_RTN:
            /* Remove any latches */
            if (infoPtr->latch)
            {
                WRITE_DSP_ADDR(GTISysPtr, GTI_DSP_A, GTI_EOC_REQCOR_OFFSET, GTI_PSRAM0W );
                WRITE_DSP_WORD(GTISysPtr, GTI_DSP_A, GTI_EOC_REQCOR_DISABLE, 0);
               UPDATE_DSP(GTISysPtr);

                infoPtr->latch = GTI_FALSE; /* FLASE Remove latch */
            }
            infoPtr->state = GTI_EOC_STATE_IDLE;
            break;

        case GTI_EOC_CMD_HOLD:
            /* Save any Latches */
            infoPtr->state = GTI_EOC_STATE_IDLE;
            break;

        default:
            /* Send UTC */;
            bTemp = GTI_EOC_CMD_UTC;
            retEocMsg = GTI_EOC_M_MAKE_MSG(GTI_EOC_ATUR_ADDRESS,
                GTI_EOC_OPCODE_FIELD, GTI_EOC_ODD_PARITY,
                bTemp);
            /* NOTE: This should be GTI_EOC_STATE_UTC */
            infoPtr->state = GTI_EOC_STATE_IDLE;
            break;
        }
    }
    else
    {
        /* Send UTC */;
        bTemp = GTI_EOC_CMD_UTC;
        retEocMsg = GTI_EOC_M_MAKE_MSG(GTI_EOC_ATUR_ADDRESS,
            GTI_EOC_OPCODE_FIELD, GTI_EOC_ODD_PARITY,
            bTemp);
        /* NOTE: This should be GTI_EOC_STATE_UTC */
        infoPtr->state = GTI_EOC_STATE_IDLE;
    }

    return retEocMsg;
}

/************************************************************************
* GTIEocExecCmd
************************************************************************/
GTI_WORD GTIEocExecCmd
(
                       GTI_XCVR_INFO *GTISysPtr,
                       GTIEOC_CMD *hostPtr,
                       GTI_WORD eocMsg)
{
    GTIEOC_FIELDS  eocFields;

    /* Get EOC Message Fields */
    eocFields.addr = GTI_EOC_M_ADDR_FIELD(eocMsg);
    eocFields.isOpcode = GTI_EOC_M_OPCODE_FIELD(eocMsg);
    eocFields.parity = GTI_EOC_M_PARITY_FIELD(eocMsg);
    eocFields.isMsg = GTI_EOC_M_MSG_FIELD(eocMsg);
    eocFields.info = GTI_EOC_M_INFO_FIELD(eocMsg);

    switch( GTISysPtr->eocInfo.state )
    {
    case GTI_EOC_STATE_IDLE: /* 0 */
        return ( GTIEocStateIdle(GTISysPtr, hostPtr, &eocFields) );

    case GTI_EOC_STATE_WRITE:      /* 2 */
        return ( GTIEocStateWrite(GTISysPtr, hostPtr, &eocFields) );

    case GTI_EOC_STATE_PREREAD:    /* 3 */
        return ( GTIEocStatePreRead(GTISysPtr, hostPtr, &eocFields) );

    case GTI_EOC_STATE_READ:       /* 4 */
        return ( GTIEocStateRead(GTISysPtr, hostPtr, &eocFields) );

    case GTI_EOC_STATE_UTC:
    case GTI_EOC_STATE_LPR:
    case GTI_EOC_STATE_PREWRITE:
    default:
        return ( 0 );
    }
}

