/*
 * @(#)gtixcvr.c	1.3.2.1
 */

/************************************************************************/
/* TSM         Application Code                                         */
/* (C) Copyright 1996-2001 by GlobeSpan Technologies Inc                */
/************************************************************************/

#include "gti.h"
#include "gtiextn.h"
#include "gtidsp.h"

#ifdef GTI_DEBUG
#include "p3hw.h"
#include "extern.h"
#include "global.h"
#endif

/**/
/**********************************************************/
/*     ALL ROUTINES CALLED BY CUSTOMERS HAVE A REQUIRED   */
/*     INPUT WHICH IS THE ADDRESS of the GTI_XCVR_INFO    */
/*     STRUCTURE BEING USED                               */
/**********************************************************/

GTI_WORD
Gti_ExecuteXCVR(GTI_XCVR_INFO *pXcvrInfo)
{

#if !defined(GTI_USE_DMAXFR) && !defined(GTI_USE_XDPXFR)

    GTI_WORD GTI_LOCAL_QUAL result;

    result = GTI_COMPLETE_PASS;


    /* Information about commands/state are is struct GTIXcvr */
    switch(pXcvrInfo->state)
    {
    case GTI_ACTIVE:
        /* See if IPU is Ready? */
        Gti_ConfigReadRdy(pXcvrInfo);

        if(Gti_IsIPUready(pXcvrInfo) == GTI_FALSE)
        {
            result = GTI_ACTIVE;
            break;
        }

        /* IPU is ready to continue Get Next State */
        pXcvrInfo->state = GTI_COMPLETE_PASS;
        pXcvrInfo->table_pnt = Gti_GetNextState(pXcvrInfo);

#ifdef GTI_DEBUG
        /************FOR GTI DEBUGGING ONLY*************/
        CheckDelay();
#endif
        result = Gti_StartXcvrCmd(pXcvrInfo);
        break;

    case GTI_COMPLETE_PASS:
    case GTI_COMPLETE_FAIL:
    case GTI_ILLEGAL_SAS:
    case GTI_ILLEGAL_PARAM:
    case GTI_ILLEGAL_YAT:
    case GTI_COMPLETE_STOP:
    case GTI_CANCELLED_SAS:
    case GTI_ILLEGAL_STACK:
        result = Gti_StartXcvrCmd(pXcvrInfo);
        break;

    default:
        result = GTI_COMPLETE_FAIL;
        break;
    }

    pXcvrInfo->state = (GTI_BYTE)result;

#ifndef GTI_DEMO
#ifdef GTI_DEBUG
    pokeb(LED_BASE, LED_D8, LED_OFF);
#endif
#endif

    return (result);
#else

    CONFIGURE_DSP(pXcvrInfo, GTI_DSP_A, GTI_NO_AI);
    WRITE_DSP_ADDR(pXcvrInfo, GTI_DSP_A, GTI_DMA_window, GTI_ISRAM|GTI_READ_DSP);
    return READ_DSP_WORD(pXcvrInfo, GTI_DSP_A);

#endif /* GTI_USE_DMAXFR */
}

#if !defined(GTI_USE_DMAXFR) && !defined(GTI_USE_XDPXFR)
/**/
GTI_BYTE
Gti_StartXcvrCmd(GTI_XCVR_INFO *pXcvrInfo)
{
    struct BASIC_CMD GTI_ADDRESS_QUAL *Pcmd;
    GTI_WORD loopCnt;
#ifdef GTI_PROD_DMT
#ifdef GTI_TRACE
    GTI_WORD wTempHistIdx = 0;
#endif
#endif

#ifdef GTI_MULTI_PROJ
       GTI_WORD tresult;
#endif

    /* Is this command supported by the loaded table? */
    if(pXcvrInfo->table_pnt == GTI_EMPTY)
    {
        return (GTI_ILLEGAL_SAS);
    }

#ifndef GTI_DEMO
#ifdef GTI_DEBUG
    pokeb(LED_BASE,LED_D8,LED_ON);
#endif
#endif

    /* Read and execute next SAS line till finished */
    for(loopCnt=0; loopCnt < GTI_MAX_SIZE_OF_XCVR_LOOP; loopCnt++)
    {
        Pcmd = pXcvrInfo->PntCmd+pXcvrInfo->table_pnt;

        pXcvrInfo->CurrentCmd.type  = Pcmd->type;
        pXcvrInfo->CurrentCmd.offset = Pcmd->offset;
        pXcvrInfo->CurrentCmd.next =  Pcmd->next;

#ifdef GTI_TRACE
        /*********************************************************/
        /* WARNING: THIS IS FOR DEBUGGING ONLY                   */
        /*********************************************************/

        RESET_TRACE_TIMER();

        if( (HistPnt < GTI_SIZE_OF_TRACE) && (pXcvrInfo->enableTRACE==GTI_TRUE) )
        {
            History[0][HistPnt]=pXcvrInfo->table_pnt;
            History[1][HistPnt]=pXcvrInfo->CurrentCmd.next;
            History[2][HistPnt]=pXcvrInfo->CurrentCmd.type;
#ifdef GTI_PROD_DMT
            wTempHistIdx = HistPnt;
            HistTimeStamp[wTempHistIdx] = 0xffff;
#endif
            ++HistPnt;
        }
#endif
        /*********************************************************/

#ifdef GTI_MULTI_PROJ
        /* Check that SAS is not lost */
        if(pXcvrInfo->table_pnt > 0x1500)
        {
            return (GTI_ILLEGAL_SAS);
        }
#else
        /* Check that SAS is not lost */
        if(pXcvrInfo->table_pnt > GTI_MAX_SAS_TABLE)
        {
            return (GTI_ILLEGAL_SAS);
        }
#endif

        switch(pXcvrInfo->CurrentCmd.type)
        {
        case GTI_NOP:
            /*SerialTXString("DEBUG:nop");*/
            break;

        case GTI_END_OK_CMD:
            /*SerialTXString("\r\nDEBUG:End OK");*/

            /* Are we returning from a "call"               */
            /* if ReturnLabel == GTI_EMPTY - stack is empty */
            /* else POP the stack                           */

            if( pXcvrInfo->ReturnLabel ==  GTI_EMPTY ){
                   return (GTI_COMPLETE_PASS);
            }else{
                   pXcvrInfo->CurrentCmd.next = pXcvrInfo->ReturnStack[pXcvrInfo->ReturnLabel];
                   if( pXcvrInfo->ReturnLabel == 0x00 ){
                       /* Now stack is empty */
                       pXcvrInfo->ReturnLabel = GTI_EMPTY;
                   }else{
                       --pXcvrInfo->ReturnLabel;
                   }
            }
            break;

        case GTI_END_FAIL_CMD:
            /*SerialTXString("DEBUG:End Fail");*/
            return (GTI_COMPLETE_FAIL);

        case GTI_END_ILLEGAL_CMD:
            /*SerialTXString("DEBUG:End Illegal");*/
            return (GTI_ILLEGAL_PARAM);

#ifdef GTI_MULTI_PROJ
        case GTI_CONTEX_CMD:
            /*SerialTXString("DEBUG:Contex Switch");*/
            tresult = Gti_ParseCONTEX(pXcvrInfo);
            if(tresult == GTI_FAIL )
            {
                 return(GTI_ILLEGAL_YAT);
            }
            break;
#endif

        case GTI_REG_READ:
            /*SerialTXString("DEBUG:Reg. Read");*/
            Gti_ParseREGR(pXcvrInfo);
            break;

        case GTI_REG_LOAD:
            /*SerialTXString("DEBUG:Reg. Load");*/
            Gti_ParseREGL(pXcvrInfo);
            break;

        case GTI_MEM_READ:
            /*SerialTXString("DEBUG:Mem Read");*/
            Gti_ParseMEMR(pXcvrInfo);
            break;

        case GTI_MEM_LOAD:
            /*SerialTXString("DEBUG:Mem Load");*/
            Gti_ParseMEMLoad(pXcvrInfo);
            break;

        case GTI_FUNCTION:
            /*SerialTXString("DEBUG:Function");*/
            Gti_ParseFUN(pXcvrInfo);
            break;

#ifdef GTI_FUTURE
        case GTI_TIMEOUT:
            /*SerialTXString("DEBUG:Timeout");*/
            Gti_ParseTIME(pXcvrInfo);
            break;

        case GTI_DELAY_CMD:
            /*SerialTXString("DEBUG:Delay");*/
            Gti_ParseDELAY(pXcvrInfo);
            break;
#endif

        case GTI_FILL:
            /*SerialTXString("DEBUG:Fill");*/
            Gti_ParseFILL(pXcvrInfo);
            break;

        case GTI_HOST_LOAD:
            /*SerialTXString("DEBUG:Host Load");*/
            Gti_ParseHOSTL(pXcvrInfo);
            break;

        case GTI_IMM_LOAD:
            /*SerialTXString("DEBUG:Immediate Load");*/
            Gti_ParseIMML(pXcvrInfo);
            break;

        case GTI_IMM_READ:
            /*SerialTXString("DEBUG:Immediate Read");*/
            Gti_ParseIMMR(pXcvrInfo);
            break;

        case GTI_INDIRECT_REG:
            /*SerialTXString("DEBUG:Indirect REG");*/
            Gti_ParseREGInd(pXcvrInfo);
            break;

        case GTI_IND_LOAD:
            /*SerialTXString("DEBUG:Indirect ISRAM");*/
            Gti_ParseINDL(pXcvrInfo);
            break;

        case GTI_IF_FUNCTION:
            /*SerialTXString("DEBUG:IF Function");*/
            Gti_ParseIF(pXcvrInfo);
            break;

        case GTI_CACHE_LOAD:
            /*SerialTXString("DEBUG:CACHE Load");*/
            Gti_ParseCACHELoad(pXcvrInfo);
            break;

        default:
            /* Illegal Command Type */
            /* If reach this point  */
            /* code execution  is   */
            /* not working correctly*/
            return(GTI_ILLEGAL_SAS);
        }

#ifdef GTI_TRACE
        /*********************************************************/
        /* Before seeing what to do next see if HIP requested a  */
        /* Stop                                                  */
        /*********************************************************/
        if( (pXcvrInfo->table_pnt == SASStop_line ) &&
            (SASStop_baud == SASStop_hip) && (SASStop_enable))
            return(GTI_COMPLETE_STOP);
#endif

        /*********************************************************/
        /* See if Indirect of Direct Load of NEXT instruction    */
        /* if number is F000->FFEF = Indirect Addressing using   */
        /*                           HOST RAM  0->4079 limit     */
        /*********************************************************/
        if( (pXcvrInfo->CurrentCmd.next >= 0xf000) &&
            (pXcvrInfo->CurrentCmd.next < 0xfff0) )
        {
            pXcvrInfo->CurrentCmd.next = (GTI_WORD)
                (GET_HOST_DB_B(pXcvrInfo, pXcvrInfo->CurrentCmd.next - 0xf000) +
                (GET_HOST_DB_B(pXcvrInfo, (GTI_WORD)((pXcvrInfo->CurrentCmd.next - 0xf000) + 1)) << 8));
        }

        switch(pXcvrInfo->CurrentCmd.next)
        {
        case GTI_NEXT:
            ++pXcvrInfo->table_pnt;
            break;

        case GTI_WAIT_IPU_A:
            /*SerialTXString(" State=Wait-A");*/
            pXcvrInfo->which_ipu = GTI_DSP_A;
            Gti_WaitIPU(pXcvrInfo);
            break;

        case GTI_WAIT_IPU_B:
            /*SerialTXString(" State=Wait-B");*/
            pXcvrInfo->which_ipu = GTI_DSP_B;
            Gti_WaitIPU(pXcvrInfo);
            break;

        case GTI_WAIT_IPU_X:
            /*SerialTXString(" State=Wait-X");*/
            pXcvrInfo->which_ipu = GET_HOST_DB_B(pXcvrInfo, GTI_WHICH_STAR_LOCATION_X);
            Gti_WaitIPU(pXcvrInfo);
            break;

        case GTI_WAIT_IPU_Y:
            /*SerialTXString(" State=Wait-Y");*/
            pXcvrInfo->which_ipu = GET_HOST_DB_B(pXcvrInfo, GTI_WHICH_STAR_LOCATION_Y);
            Gti_WaitIPU(pXcvrInfo);
            break;

        case GTI_WAIT_IPU_Z:
            /*SerialTXString(" State=Wait-Z");*/
            pXcvrInfo->which_ipu = GET_HOST_DB_B(pXcvrInfo, GTI_WHICH_STAR_LOCATION_Z);
            Gti_WaitIPU(pXcvrInfo);
            break;

        case GTI_WAIT_IPU_AB:
            /*SerialTXString(" State=Wait-AB");*/
            pXcvrInfo->which_ipu = GTI_DSP_AB;
            Gti_WaitIPU(pXcvrInfo);
            break;

        case GTI_GOTO_IPU_A:
            /*SerialTXString(" State=Goto-A");*/
            pXcvrInfo->which_ipu = GTI_DSP_A;
            return (GTI_ACTIVE);

        case GTI_GOTO_IPU_B:
            /*SerialTXString(" State=Goto-B");*/
            pXcvrInfo->which_ipu = GTI_DSP_B;
            return (GTI_ACTIVE);

        case GTI_GOTO_IPU_X:
            /*SerialTXString(" State=Goto-B");*/
            pXcvrInfo->which_ipu = GET_HOST_DB_B(pXcvrInfo, GTI_WHICH_STAR_LOCATION_X);
            return (GTI_ACTIVE);

        case GTI_GOTO_IPU_Y:
            /*SerialTXString(" State=Goto-B");*/
            pXcvrInfo->which_ipu = GET_HOST_DB_B(pXcvrInfo, GTI_WHICH_STAR_LOCATION_Y);
            return (GTI_ACTIVE);

        case GTI_GOTO_IPU_Z:
            /*SerialTXString(" State=Goto-B");*/
            pXcvrInfo->which_ipu = GET_HOST_DB_B(pXcvrInfo, GTI_WHICH_STAR_LOCATION_Z);
            return (GTI_ACTIVE);

        case GTI_GOTO_IPU_AB:
            /* SerialTXString(" State=Goto-AB");*/
            pXcvrInfo->which_ipu = GTI_DSP_AB;
            return (GTI_ACTIVE);

#ifdef GTI_MULTI_PROJ
        case GTI_CONTEX_A:
            /* SerialTXString(" State=Contex-A");*/
            pXcvrInfo->which_ipu = GTI_DSP_A;
            return (GTI_ACTIVE);

        case GTI_CONTEX_B:
            /* SerialTXString(" State=Goto-B");*/
            pXcvrInfo->which_ipu = GTI_DSP_B;
            return (GTI_ACTIVE);
#endif
        case GTI_STACK_OVERFLOW:
            return (GTI_ILLEGAL_STACK);

        default:
            /* It is Goto Line Number */
            /* SerialTXString(" State=Goto-Line num");*/
            pXcvrInfo->table_pnt = pXcvrInfo->CurrentCmd.next;
            break;
        }

        /* Add Mode Unselect ALL */
        /* SAP pointer to empty  */

#ifdef GTI_PROD_DMT
#ifdef GTI_TRACE
        HistTimeStamp[wTempHistIdx] = READ_TRACE_TIMER();
#endif
#endif
    }
    return (GTI_SAS_LOOP_ILLEGAL);

    /* return(GTI_COMPLETE_PASS); */
}

/**/
void
Gti_ParseMEMLoad(GTI_XCVR_INFO *pXcvrInfo)
{
    GTI_BYTE GTI_LOCAL_QUAL size;
    GTI_BYTE GTI_LOCAL_QUAL id;
    struct MEM_LOAD_STRUCT GTI_ADDRESS_QUAL *Pmeml;

    /* Point to Memory Load command */
    Pmeml = pXcvrInfo->PntMEML + pXcvrInfo->CurrentCmd.offset;

    /* Using Command Table Information Read Starlet Memory Load */
    /* Paramaters                                               */

    /* Set ID for Current Command                               */
    id = Pmeml->id;

    if( (Pmeml->id) == GTI_DSP_X )
        id = GET_HOST_DB_B(pXcvrInfo, GTI_WHICH_STAR_LOCATION_X);
    else
    if( (Pmeml->id) == GTI_DSP_Y )
        id = GET_HOST_DB_B(pXcvrInfo, GTI_WHICH_STAR_LOCATION_Y);
    else
    if( (Pmeml->id) == GTI_DSP_Z )
        id = GET_HOST_DB_B(pXcvrInfo, GTI_WHICH_STAR_LOCATION_Z);

    if(id == GTI_DSP_NONE )
        return;     /* Skip this Operation */

    size = Gti_GetWordSize(Pmeml->dtype);

    /* Configure for Write to RAM */

#if( defined(GTI_DSP_SPECIAL_BSRAM) )
    if( Pmeml->dtype == GTI_BSRAM )
    {
        /* BSRAM because of Data organization can not */
        /* be loaded in DMA-BLOCK mode                */
        CONFIGURE_DSP(pXcvrInfo, id, GTI_AP);
    }
    else
    {
        CONFIGURE_DSP(pXcvrInfo, id, GTI_DMA);
    }
#else
   CONFIGURE_DSP(pXcvrInfo, id, GTI_DMA);
#endif

#if( defined(GTI_CRAM_16BIT) )
    /* Set up RAM Type Address */
    /* Look for 16bit CRAM */
    if( size == GTI_SPECIAL_CRAM )
    {
        /*
        WRITE_DSP_ADDR(pXcvrInfo, id, Pmeml->STARPnt, (GTI_BYTE)(Pmeml->dtype-0x20) );
        WRITE_DSP_BYTE(pXcvrInfo, id, GTI_EXT_DATA_DSP, 0x00);
        */
        WRITE_DSP_SPECIAL(pXcvrInfo, id, Pmeml->STARPnt, (GTI_BYTE)(Pmeml->dtype-GTI_CRAM16_ADJUST), 0x00);
    }
    else
#endif
    {
        WRITE_DSP_ADDR(pXcvrInfo, id, Pmeml->STARPnt, Pmeml->dtype);
    }

    /* Write Data from Start to End */
    if(Pmeml->stype == GTI_HOST_ROM)
    {
        switch(Pmeml->dtype)
        {
#if ( defined(GTI_DSP_FDP_ENG) )
        case GTI_FDPRAM0:
        case GTI_FDPRAM1:
        case GTI_FDPRAM2:
        case GTI_FDPRAM3:
#if (defined(GTI_DSP_SUPERNOVA_ENG))
        case GTI_FDPRAM4:
        case GTI_FDPRAM5:
#endif
            Gti_SendCodeROM(pXcvrInfo,id,size,pXcvrInfo->PntFDP);
            break;
        case GTI_FDPSRAM0:
        case GTI_FDPSRAM1:
        case GTI_FDPSRAM2:
        case GTI_FDPSRAM3:
        case GTI_FDPSRAM4:
        case GTI_FDPSRAM5:
        case GTI_FDPSRAM6:
        case GTI_FDPSRAM7:
        case GTI_FDPSRAM8:
        case GTI_FDPSRAM9:
        case GTI_FDPSRAMA:
        case GTI_FDPSRAMB:
        case GTI_FDPSRAMC:
        case GTI_FDPSRAMD:
        case GTI_FDPSRAME:
        case GTI_FDPSRAMF:
#if (defined(GTI_DSP_SUPERNOVA_ENG))
        case GTI_FDPSRAM10:
        case GTI_FDPSRAM11:
        case GTI_FDPSRAM12:
        case GTI_FDPSRAM13:
#endif
            Gti_SendCodeROM(pXcvrInfo,id,size,pXcvrInfo->PntCOEF);
            break;
#endif /* FDP Engine */

#ifdef GTI_DSP_PCU_ENG
        case GTI_PCURAM0:
        case GTI_PCURAM1:

#if (defined (GTI_DSPMULTI ) || defined (GTI_DSPFRAMER_B1) || defined(GTI_DSPQUAD) || defined(GTI_DSPPCI) || defined (GTI_DSPFRAMER_C1)  || defined(GTI_DSPOCTANE) || defined(GTI_DSPVIKING) )
        case GTI_PCURAM2:
        case GTI_PCURAM3:
#if (defined (GTI_DSPQUAD ) || defined (GTI_DSPPCI) || defined (GTI_DSPFRAMER_C1)  || defined(GTI_DSPOCTANE) || defined(GTI_DSP_VIKING) )
        case GTI_PCURAM4:
        case GTI_PCURAM5:
#endif
#endif
            Gti_SendCodeROM(pXcvrInfo,id,size,pXcvrInfo->PntPCU);
            break;

        case GTI_PSRAM0W:
#if (defined (GTI_DSPMULTI ) || defined (GTI_DSPFRAMER_B1) || defined(GTI_DSPQUAD) || defined(GTI_DSPPCI) || defined (GTI_DSPFRAMER_C1)  || defined(GTI_DSPOCTANE) || defined(GTI_DSPVIKING) )
        case GTI_PSRAM1W:
#endif

            Gti_SendCodeROM(pXcvrInfo,id,size,pXcvrInfo->PntCOEF);
            break;
#endif  /* PCU Engine */

        case GTI_IRAM0:
        case GTI_IRAM1:
#if (defined(GTI_DSP_SUPERNOVA_ENG))
        case GTI_IRAM2:
        case GTI_IRAM3:
        case GTI_IRAM4:
        case GTI_IRAM5:
        case GTI_IRAM6:
        case GTI_IRAM7:
        case GTI_IRAM8:
        case GTI_IRAM9:
        case GTI_IRAMA:
        case GTI_IRAMB:
        case GTI_IRAMC:
        case GTI_IRAMD:
        case GTI_IRAME:
        case GTI_IRAMF:
#endif  /* Octane, Viking */
        case GTI_RSRAM0:
        case GTI_RSRAM1:
        case GTI_RSRAM2:
        case GTI_RSRAM3:
        case GTI_RSRAM4:
        case GTI_RSRAM5:
        case GTI_RSRAM6:
        case GTI_RSRAM7:
#if (defined (GTI_DSPMULTI ) || defined (GTI_DSPFRAMER_B1) || defined(GTI_DSPQUAD) || defined(GTI_DSPPCI) || defined(GTI_DSPFRAMER_C1)  || defined(GTI_DSPOCTANE) || defined(GTI_DSPVIKING) )
        case GTI_RSRAM8:
        case GTI_RSRAM9:
        case GTI_RSRAMA:
        case GTI_RSRAMB:
        case GTI_RSRAMC:
        case GTI_RSRAMD:
#endif
            Gti_SendCodeROM(pXcvrInfo,id,size,pXcvrInfo->PntIPU);
            break;
        case GTI_ISRAM:
            Gti_SendCodeROM(pXcvrInfo,id,size,pXcvrInfo->PntCOEF);
            break;
        case GTI_FRAM:
            Gti_SendCodeROM(pXcvrInfo,id,size,pXcvrInfo->PntAVM);
            break;
        case GTI_CRAM0:
        case GTI_CRAM1:
        case GTI_CRAM2:
        case GTI_CRAM3:
        case GTI_CRAM4:
        case GTI_CRAM5:
        case GTI_CBROAD:

#if ( defined(GTI_CRAM_16BIT) )
        case GTI_CRAM016:
        case GTI_CRAM116:
        case GTI_CRAM216:
        case GTI_CRAM316:
        case GTI_CRAM416:
        case GTI_CRAM516:
        case GTI_CBROAD16:
#endif

        case GTI_DRAM0:
        case GTI_DRAM2:
        case GTI_DRAM4:
        case GTI_DBROAD:
            Gti_SendCodeROM(pXcvrInfo,id,size,pXcvrInfo->PntCOEF);
            break;
        case GTI_TRAM:
            Gti_SendCodeROM(pXcvrInfo,id,size,pXcvrInfo->PntTR);
            break;
        case GTI_BRAM:
            Gti_SendCodeROM(pXcvrInfo,id,size,pXcvrInfo->PntBE);
            break;
        case GTI_BSRAM:
#if( defined(GTI_DSP_SPECIAL_BSRAM) )
            Gti_SendSpecialROM(pXcvrInfo,id,pXcvrInfo->PntCOEF);
#else
            Gti_SendCodeROM(pXcvrInfo,id,size,pXcvrInfo->PntCOEF);
#endif
            break;

        default:
            break;
        }
    }
    else
        Gti_SendWordRAM(pXcvrInfo, id, size);

    UPDATE_DSP(pXcvrInfo);
}

/**/
void
Gti_ParseCACHELoad(GTI_XCVR_INFO *pXcvrInfo)
{
    GTI_BYTE GTI_LOCAL_QUAL size;
    GTI_BYTE GTI_LOCAL_QUAL id;
    struct MEM_LOAD_STRUCT GTI_ADDRESS_QUAL *Pmeml;

    /* Point to Memory Load command */
    Pmeml = pXcvrInfo->PntMEML + pXcvrInfo->CurrentCmd.offset;

    /* Using Command Table Information Read Starlet Memory Load */
    /* Paramaters                                               */

    /* Set ID for Current Command                               */
    id = Pmeml->id;
    if((Pmeml->id) == GTI_DSP_X)
        id = GET_HOST_DB_B(pXcvrInfo, GTI_WHICH_STAR_LOCATION_X);

    if((Pmeml->id) == GTI_DSP_Y)
        id = GET_HOST_DB_B(pXcvrInfo, GTI_WHICH_STAR_LOCATION_Y);

    if((Pmeml->id) == GTI_DSP_Z)
        id = GET_HOST_DB_B(pXcvrInfo, GTI_WHICH_STAR_LOCATION_Z);;

    if(id == GTI_DSP_NONE)
        return;     /* Skip this Operation */

    size = Gti_GetWordSize(Pmeml->stype); /* Get Word Size of Source */

    /* Configure for Write to RAM */

#if( defined(GTI_DSP_SPECIAL_BSRAM) )
    if( Pmeml->dtype == GTI_BSRAM )
    {
        /* BSRAM because of Data organization can not */
        /* be loaded in DMA-BLOCK mode                */
        CONFIGURE_DSP(pXcvrInfo, id, GTI_AP);
    }
    else
    {
        CONFIGURE_DSP(pXcvrInfo, id, GTI_DMA);
    }
#else

    /* NOVA don't need SPECIAL BSRAM access */
    CONFIGURE_DSP(pXcvrInfo, id, GTI_DMA);
#endif

#if ( defined(GTI_CRAM_16BIT) )
    /* Set up RAM Type Address */
    /* Look for 16bit CRAM */
    if(size == GTI_SPECIAL_CRAM)
    {
        WRITE_DSP_SPECIAL(pXcvrInfo, id, Pmeml->STARPnt, (GTI_BYTE)(Pmeml->dtype-GTI_CRAM16_ADJUST), 0x00);
    }
    else
#endif
    {
        WRITE_DSP_ADDR(pXcvrInfo, id, Pmeml->STARPnt, Pmeml->dtype);
    }

    /* Write Data from Start to End */
    switch( Pmeml->stype )
    {
#if (defined(GTI_DSP_FDP_ENG))
    case GTI_FDPRAM0:
    case GTI_FDPRAM1:
    case GTI_FDPRAM2:
    case GTI_FDPRAM3:
#if (defined(GTI_DSP_SUPERNOVA_ENG))
    case GTI_FDPRAM4:
    case GTI_FDPRAM5:
#endif
        Gti_SendCodeROM(pXcvrInfo,id,size,pXcvrInfo->PntFDP);
        break;
    case GTI_FDPSRAM0:
    case GTI_FDPSRAM1:
    case GTI_FDPSRAM2:
    case GTI_FDPSRAM3:
    case GTI_FDPSRAM4:
    case GTI_FDPSRAM5:
    case GTI_FDPSRAM6:
    case GTI_FDPSRAM7:
    case GTI_FDPSRAM8:
    case GTI_FDPSRAM9:
    case GTI_FDPSRAMA:
    case GTI_FDPSRAMB:
    case GTI_FDPSRAMC:
    case GTI_FDPSRAMD:
    case GTI_FDPSRAME:
    case GTI_FDPSRAMF:
#if (defined(GTI_DSP_SUPERNOVA_ENG))
    case GTI_FDPSRAM10:
    case GTI_FDPSRAM11:
    case GTI_FDPSRAM12:
    case GTI_FDPSRAM13:
#endif
        Gti_SendCodeROM(pXcvrInfo,id,size,pXcvrInfo->PntCOEF);
        break;
#endif /* FDP Eng */

#if (defined(GTI_DSP_PCU_ENG))
    case GTI_PCURAM0:
    case GTI_PCURAM1:
    case GTI_PSRAM0W:
#if (defined (GTI_DSPMULTI ) || defined (GTI_DSPFRAMER_B1) || defined (GTI_DSPQUAD) || defined(GTI_DSPPCI)|| defined(GTI_DSPFRAMER_C1)  || defined(GTI_DSPOCTANE) || defined(GTI_DSPVIKING) )
    case GTI_PCURAM2:
    case GTI_PCURAM3:
    case GTI_PSRAM1W:
#if ( defined (GTI_DSPQUAD) || defined (GTI_DSPPCI) || defined(GTI_DSPFRAMER_C1)  || defined(GTI_DSPOCTANE) || defined(GTI_DSPVIKING) )
    case GTI_PCURAM4:
    case GTI_PCURAM5:
#endif
#endif
        Gti_SendCodeROM(pXcvrInfo,id,size,pXcvrInfo->PntPCU);
        break;
#endif /* PCU Eng */

    case GTI_IRAM0:
    case GTI_IRAM1:
#if (defined(GTI_DSP_SUPERNOVA_ENG))
    case GTI_IRAM2:
    case GTI_IRAM3:
    case GTI_IRAM4:
    case GTI_IRAM5:
    case GTI_IRAM6:
    case GTI_IRAM7:
    case GTI_IRAM8:
    case GTI_IRAM9:
    case GTI_IRAMA:
    case GTI_IRAMB:
    case GTI_IRAMC:
    case GTI_IRAMD:
    case GTI_IRAME:
    case GTI_IRAMF:
#endif
    case GTI_RSRAM0:
    case GTI_RSRAM1:
    case GTI_RSRAM2:
    case GTI_RSRAM3:
    case GTI_RSRAM4:
    case GTI_RSRAM5:
    case GTI_RSRAM6:
    case GTI_RSRAM7:
#if (defined (GTI_DSPMULTI ) || defined (GTI_DSPFRAMER_B1) || defined(GTI_DSPQUAD) || defined(GTI_DSPPCI) || defined (GTI_DSPFRAMER_C1)  || defined(GTI_DSPOCTANE) || defined(GTI_DSPVIKING) )
    case GTI_RSRAM8:
    case GTI_RSRAM9:
    case GTI_RSRAMA:
    case GTI_RSRAMB:
    case GTI_RSRAMC:
    case GTI_RSRAMD:
#endif
        Gti_SendCodeROM(pXcvrInfo,id,size,pXcvrInfo->PntIPU);
        break;
    case GTI_ISRAM:
        Gti_SendCodeROM(pXcvrInfo,id,size,pXcvrInfo->PntCOEF);
        break;
    case GTI_FRAM:
        Gti_SendCodeROM(pXcvrInfo,id,size,pXcvrInfo->PntAVM);
        break;
    case GTI_CRAM0:
    case GTI_CRAM1:
    case GTI_CRAM2:
    case GTI_CRAM3:
    case GTI_CRAM4:
    case GTI_CRAM5:
    case GTI_CBROAD:

#if (defined(GTI_CRAM_16BIT))
    case GTI_CRAM016:
    case GTI_CRAM116:
    case GTI_CRAM216:
    case GTI_CRAM316:
    case GTI_CRAM416:
    case GTI_CRAM516:
    case GTI_CBROAD16:
#endif

    case GTI_DRAM0:
    case GTI_DRAM2:
    case GTI_DRAM4:
    case GTI_DBROAD:
        Gti_SendCodeROM(pXcvrInfo,id,size,pXcvrInfo->PntCOEF);
        break;
    case GTI_TRAM:
        Gti_SendCodeROM(pXcvrInfo,id,size,pXcvrInfo->PntTR);
        break;
    case GTI_BRAM:
        Gti_SendCodeROM(pXcvrInfo,id,size,pXcvrInfo->PntBE);
        break;
    case GTI_BSRAM:
#if( defined(GTI_DSP_SPECIAL_BSRAM) )
        Gti_SendSpecialROM(pXcvrInfo,id,pXcvrInfo->PntCOEF);
#else
        /* NOVA don't need SPECIAL BSRAM access */
        Gti_SendCodeROM(pXcvrInfo,id,size,pXcvrInfo->PntCOEF);
#endif
        break;

    default:
        break;
    }

    UPDATE_DSP(pXcvrInfo);
}

void
Gti_SendSpecialROM(GTI_XCVR_INFO *pXcvrInfo,
                   GTI_BYTE id,
                   GTI_CONST_BYTE_ORDER GTI_ADDRESS_QUAL *code_pnt)
{
    /* Copy code from HOST ROM to STARLET Internal RAM */
    /* This is Special for BSRAM bit order             */
    /* The Bit order in ROM does not match STARLET     */
    /*                 1,0 ,x,x,x,x,      19,18        */
    /* LowOut =  LowIn[5,4],x,x,x,x,HighIn[7,6]        */
    /*                 9,8,7,6,5,4,       3,2          */
    /* MedOut =  MedIn[5,4,3,2,1,0],LowIn[7,6]         */
    /*                 17,16,15,14,13,12,11,10         */
    /* HighOut = HighIn[5,4,3,2,1,0],MedIn[7,6]        */

    GTI_WORD GTI_LOCAL_QUAL off;
    GTI_BYTE GTI_LOCAL_QUAL lowin,medin, highin;
    GTI_BYTE GTI_LOCAL_QUAL lowout, medout, highout;
    struct MEM_LOAD_STRUCT GTI_ADDRESS_QUAL *Pmeml;

    Pmeml = pXcvrInfo->PntMEML + pXcvrInfo->CurrentCmd.offset;

    off=Pmeml->StarSrc;

    do
    {
        lowin =*(code_pnt+off);++off;
        medin =*(code_pnt+off);++off;
        highin =*(code_pnt+off);++off;

        /* Reorder BITS for STARLET   */
        lowout = (GTI_BYTE)(((highin>>6) & 0x03) | ((lowin<<2) & 0xc0));
        medout = (GTI_BYTE)(((medin<<2) & 0xfc) |  ((lowin>>6) & 0x03));
        highout= (GTI_BYTE)(((highin<<2) & 0xfc) | ((medin>>6) & 0x03));

        WRITE_DSP_BYTE(pXcvrInfo,id,GTI_EXT_DATA_DSP,lowout);
        WRITE_DSP_WORD(pXcvrInfo,id,medout,highout);

    } while(off <= Pmeml->EndSrc);
}

void
Gti_SendCodeROM(GTI_XCVR_INFO *pXcvrInfo,
            GTI_BYTE id,GTI_BYTE bSize,
            GTI_CONST_BYTE_ORDER GTI_ADDRESS_QUAL *code_pnt)
{
    /* Copy code from HOST ROM to STARLET Internal RAM */

    GTI_WORD GTI_LOCAL_QUAL length;
    struct MEM_LOAD_STRUCT GTI_ADDRESS_QUAL *Pmeml;

    Pmeml = pXcvrInfo->PntMEML + pXcvrInfo->CurrentCmd.offset;

    length = (GTI_WORD)(Pmeml->EndSrc - Pmeml->StarSrc);

#ifdef GTI_CRAM_16BIT
    if(bSize == GTI_SPECIAL_CRAM)
    {
        length += 2;
    }
    else
#endif
        length = (GTI_WORD)(length + bSize);

    switch(bSize)
    {
    case 2:
    case 4:
    case 6:
#ifdef GTI_CRAM_16BIT
    case GTI_SPECIAL_CRAM:  /* CRAM 16 bit into 24 bit */
#endif

        WRITE_DSP_BLOCK(pXcvrInfo,id, GTI_DATAL_DSP, length,
            (code_pnt+(Pmeml->StarSrc)));

        break;

    case 3:
        WRITE_DSP_BLOCK(pXcvrInfo,id, GTI_EXT_DATA_DSP, length,
            (code_pnt+(Pmeml->StarSrc)));
        break;
    default:
        /* Should never get here */
        break;
    }
}

void
Gti_SendWordRAM(GTI_XCVR_INFO *pXcvrInfo, GTI_BYTE id, GTI_BYTE bSize)
{
    /* Copy  code from HOST RAM to STARLET Internal RAM */

    GTI_WORD wLength;

    struct MEM_LOAD_STRUCT GTI_ADDRESS_QUAL *Pmeml;

    Pmeml = pXcvrInfo->PntMEML + pXcvrInfo->CurrentCmd.offset;

    /*SerialTXString("DEBUG:In Send Word RAM\n");*/
    wLength =  (GTI_WORD)(Pmeml->EndSrc - Pmeml->StarSrc + bSize);

    if(bSize == 3)
        Gti_CopyHostDBToDsp(pXcvrInfo, id, Pmeml->StarSrc, GTI_EXT_DATA_DSP, wLength);
    else
        Gti_CopyHostDBToDsp(pXcvrInfo, id, Pmeml->StarSrc, GTI_DATAL_DSP, wLength);
}

/**/
void
Gti_ParseFUN(GTI_XCVR_INFO *pXcvrInfo)
{
    /* Store the current line number+1 in GTI_XCVR_INFO structure */
    /* Adjust Stack pointer before coping return address        */

    if( pXcvrInfo->ReturnLabel == GTI_EMPTY ){
        pXcvrInfo->ReturnLabel =  0x00;
    }
    else
    {
        ++pXcvrInfo->ReturnLabel;
    }

    if( pXcvrInfo->ReturnLabel == GTI_MAX_SAS_STACK_SIZE){
        /* ERROR condition - stack is overflow  */
        /* Fill ReturnLabel stack with 0xefff   */
        /* To trigger an ILLEGAL_SAS message    */
        pXcvrInfo->ReturnStack[--pXcvrInfo->ReturnLabel] = GTI_STACK_OVERFLOW;
        pXcvrInfo->CurrentCmd.next = GTI_STACK_OVERFLOW;
    }
    else
    {
        pXcvrInfo->ReturnStack[pXcvrInfo->ReturnLabel] = (GTI_WORD)(pXcvrInfo->table_pnt + 1);
        /*pXcvrInfo->ReturnLabel =  pXcvrInfo->table_pnt+1;*/
    }
}

#ifdef GTI_FUTURE
/**/
void
Gti_ParseDELAY( GTI_XCVR_INFO *pXcvrInfo )
{
    struct DELAY_STRUCT GTI_ADDRESS_QUAL  *Pdly;

    Pdly = pXcvrInfo->PntDelay+pXcvrInfo->CurrentCmd.offset;

    /* Using Command Table Information Read Delay time  */
    /* Wait the time specified before continuing        */

    WaitXms(Pdly->time_ms);
    /*SerialTXString("DEBUG:Time =");*/
    /*itohex(pXcvrInfo->time_ms,3,SERIAL_INTF,4);*/
    /*SerialTXString(&buffer[0]);*/
    /*SerialTXString(" Label =");*/
    /*itohex(pXcvrInfo->time_off,3,SERIAL_INTF,4);*/
    /*SerialTXString(&buffer[0]);*/
    /*SerialTXString("\n");*/
}

/**/
void
Gti_ParseTIME(GTI_XCVR_INFO *pXcvrInfo)
{
    struct TIMEOUT_STRUCT GTI_ADDRESS_QUAL *Ptime;

    Ptime = pXcvrInfo->PntTime+pXcvrInfo->CurrentCmd.offset;

    /* Using Command Table Information Read TimeOut time,offset */
    /* Save Information into Global State Machine               */

    pXcvrInfo->time_ms = Ptime->time_ms;
    pXcvrInfo->time_off =Ptime->time_off;
    /*SerialTXString("DEBUG:Time =");*/
    /*itohex(pXcvrInfo->time_ms,3,SERIAL_INTF,4);*/
    /*SerialTXString(&buffer[0]);*/
    /*SerialTXString(" Label =");*/
    /*itohex(pXcvrInfo->time_off,3,SERIAL_INTF,4);*/
    /*SerialTXString(&buffer[0]);*/
    /*SerialTXString("\n");*/
}

/**/
void
Gti_ParseFUN(GTI_XCVR_INFO *pXcvrInfo)
{
    GTI_BYTE GTI_LOCAL_QUAL result;
    struct FUN_STRUCT GTI_ADDRESS_QUAL *Pfun;

    Pfun = pXcvrInfo->PntFun + pXcvrInfo->CurrentCmd.offset;

    /* Using Command Table Information Read Function */
    /* and Goto Information                          */

    /* Call Function */
    result = (*(Pfun->GTIFunc))();

    switch(result)
    {
    case 0:
        pXcvrInfo->CurrentCmd.next=Pfun->goto0;
        break;
    case 1:
        pXcvrInfo->CurrentCmd.next=Pfun->goto1;
        break;
    case 2:
        pXcvrInfo->CurrentCmd.next=Pfun->goto2;
        break;
    default:
        pXcvrInfo->CurrentCmd.next++;
        break;
    }
}

#endif
/**/
void
Gti_ParseREGL(GTI_XCVR_INFO *pXcvrInfo)
{
    GTI_WORD GTI_LOCAL_QUAL offset;
    GTI_BYTE GTI_LOCAL_QUAL datal,datah;
    GTI_BYTE GTI_LOCAL_QUAL id;

    offset = pXcvrInfo->CurrentCmd.offset;

    id = (pXcvrInfo->PntREGL+offset)->id;
    if(id == GTI_DSP_X)
        id = GET_HOST_DB_B(pXcvrInfo, GTI_WHICH_STAR_LOCATION_X);

    if(id == GTI_DSP_Y)
        id = GET_HOST_DB_B(pXcvrInfo, GTI_WHICH_STAR_LOCATION_Y);

    if(id == GTI_DSP_Z)
        id = GET_HOST_DB_B(pXcvrInfo, GTI_WHICH_STAR_LOCATION_Z);;

    if(id == GTI_DSP_NONE)
        return;     /* Skip this Operation */

    /* Using Command Table Information Read REGs */
    /* Write Information                         */

    CONFIGURE_DSP(pXcvrInfo, id, GTI_NO_AI);

    while(id != GTI_EOT)
    {
        datal = (GTI_BYTE) ((pXcvrInfo->PntREGL+offset)->dat& 0x00ff);
        datah = (GTI_BYTE) (((pXcvrInfo->PntREGL+offset)->dat >> 8) & 0x00ff);

        /* Is this a write to FRAMER or STARLET Registers */
#if ( !defined(GTI_DSPMULTI) && !defined(GTI_DSPQUAD) && !defined(GTI_DSPPCI)  && !defined(GTI_DSPOCTANE) && !defined(GTI_DSPVIKING) )
        switch(id)
        {
#ifdef GTI_DEBUG
        case GTI_FRAMER:
            WriteADSLFramerW(pXcvrInfo,(pXcvrInfo->PntREGL + offset)->reg,
                (pXcvrInfo->PntREGL + offset)->dat);
            break;

        case GTI_FRAMER_H:
            WriteHDSLFramerB(pXcvrInfo,(pXcvrInfo->PntREGL + offset)->reg, datal);
            break;

        case GTI_BROOK_RAML:
            WriteRAMLE1T1(pXcvrInfo, (pXcvrInfo->PntREGL + offset)->reg, datal);
            break;

        case GTI_BROOK_RAMH:
            WriteRAMHE1T1(pXcvrInfo,(pXcvrInfo->PntREGL + offset)->reg, datal);
            break;
#else /* Debug */
        case GTI_FRAMER_H:
        case GTI_BROOK_RAML:
        case GTI_BROOK_RAMH:
            break;     /* Not supported */
#endif /* !debug */

        default:
            /* Select IREG RAM */
            WRITE_DSP_ADDR(pXcvrInfo, id, (pXcvrInfo->PntREGL + offset)->reg, GTI_IREGA);
            WRITE_DSP_WORD(pXcvrInfo, id, datal, datah);
            break;
        }
#else /* !GTI_DSPMULTI && !GTI_DSPQUAD && !GTI_DSPPCI */
            /* Configure for Write to RAM-REG */

        /* Select IREG RAM */
        WRITE_DSP_ADDR(pXcvrInfo, id, (pXcvrInfo->PntREGL + offset)->reg, GTI_IREGA);
        WRITE_DSP_WORD(pXcvrInfo, id, datal, datah);

#endif /* !GTI_DSPMULTI && !GTI_DSPQUAD && !GTI_DSPPCI */

        ++offset;
        id = (pXcvrInfo->PntREGL+offset)->id;
        if(id == GTI_DSP_X)
            id = GET_HOST_DB_B(pXcvrInfo, GTI_WHICH_STAR_LOCATION_X);
        if(id == GTI_DSP_Y)
            id = GET_HOST_DB_B(pXcvrInfo, GTI_WHICH_STAR_LOCATION_Y);
        if(id == GTI_DSP_Z)
            id = GET_HOST_DB_B(pXcvrInfo, GTI_WHICH_STAR_LOCATION_Z);;
    }

    UPDATE_DSP(pXcvrInfo);
}

/**/
void
Gti_ParseREGR(GTI_XCVR_INFO *pXcvrInfo)
{
    GTI_WORD GTI_LOCAL_QUAL offset,result;
    GTI_BYTE GTI_LOCAL_QUAL datal,datah;
    GTI_BYTE GTI_LOCAL_QUAL id;

    offset = pXcvrInfo->CurrentCmd.offset;
    id = (pXcvrInfo->PntREGR+offset)->id;
    datal = datah = 0;

    if(id == GTI_DSP_X)
        id = GET_HOST_DB_B(pXcvrInfo, GTI_WHICH_STAR_LOCATION_X);

    if(id == GTI_DSP_Y)
        id = GET_HOST_DB_B(pXcvrInfo, GTI_WHICH_STAR_LOCATION_Y);

    if(id == GTI_DSP_Z)
        id = GET_HOST_DB_B(pXcvrInfo, GTI_WHICH_STAR_LOCATION_Z);;

    if(id == GTI_DSP_NONE)
        return;     /* Skip this Operation */

    /* Using Command Table Information Read REGs */
    /* Read Information and store in HOST RAM    */

    CONFIGURE_DSP(pXcvrInfo, id, GTI_NO_AI);

    while(id != GTI_EOT)
    {
#if (!defined(GTI_DSPMULTI) && !defined(GTI_DSPQUAD) && !defined(GTI_DSPPCI) && !defined(GTI_DSPOCTANE) && !defined(GTI_DSPVIKING) )
        switch(id)
        {
#ifdef GTI_DEBUG
        case GTI_FRAMER:
        case GTI_FRAMER_H:
            datal = ReadFramer(pXcvrInfo,(pXcvrInfo->PntREGR + offset)->reg);
            datah = 0;
            break;

        case GTI_BROOK_RAML:
            datal = ReadRAMLE1T1(pXcvrInfo,(pXcvrInfo->PntREGR + offset)->reg);
            datah = 0;
            break;

        case GTI_BROOK_RAMH:
            datal = ReadRAMHE1T1(pXcvrInfo,(pXcvrInfo->PntREGR + offset)->reg);
            datah = 0;
            break;
#else  /* Debug */
        case GTI_BROOK_RAML:
        case GTI_BROOK_RAMH:
            break;
#endif /* Debug */
        default:
            /* Configure for Read to RAM-REG */

            WRITE_DSP_ADDR(pXcvrInfo, id,
                (pXcvrInfo->PntREGR + offset)->reg, (GTI_BYTE)(GTI_IREGA|GTI_READ_DSP));

            result = READ_DSP_WORD(pXcvrInfo,id);
            datal = (GTI_BYTE) (result & 0x00ff);
            datah = (GTI_BYTE) ((result >> 8) & 0x00ff);
            break;

        }
#else  /* !GTI_DSPMULTI && !GTI_DSPQUAD && !GTI_DSPPCI && !GTI_DSPOCTANE && !GTI_DSPVIKING */

        WRITE_DSP_ADDR(pXcvrInfo, id,
            (pXcvrInfo->PntREGR + offset)->reg, (GTI_BYTE)(GTI_IREGA|GTI_READ_DSP));

        result = READ_DSP_WORD(pXcvrInfo,id);
        datal = (GTI_BYTE) (result & 0x00ff);
        datah = (GTI_BYTE) ((result >> 8) & 0x00ff);
#endif

        SET_HOST_DB_B(pXcvrInfo, (pXcvrInfo->PntREGR + offset)->offset, datal);
        SET_HOST_DB_B(pXcvrInfo, (GTI_WORD)(((pXcvrInfo->PntREGR + offset)->offset) + 1), datah);

        /*SerialTXString("DEBUG:RAM Add=");*/
        /*itohex((pXcvrInfo->PntREGR+offset)->offset,3,SERIAL_INTF,4);*/
        /*SerialTXString(&buffer[0]);*/
        /*SerialTXString("\n");*/

        ++offset;
        id = (pXcvrInfo->PntREGR+offset)->id;
        if(id == GTI_DSP_X)
            id = GET_HOST_DB_B(pXcvrInfo, GTI_WHICH_STAR_LOCATION_X);

        if(id == GTI_DSP_Y)
            id = GET_HOST_DB_B(pXcvrInfo, GTI_WHICH_STAR_LOCATION_Y);

        if(id == GTI_DSP_Z)
            id = GET_HOST_DB_B(pXcvrInfo, GTI_WHICH_STAR_LOCATION_Z);
    }
}

/**/
void
Gti_ParseMEMR(GTI_XCVR_INFO *pXcvrInfo)
{
    GTI_BYTE GTI_LOCAL_QUAL size, *pHostDb;
    GTI_WORD GTI_LOCAL_QUAL HostPnt, wWordsToRead;
    GTI_BYTE GTI_LOCAL_QUAL id;
    GTI_WORD GTI_LOCAL_QUAL itmp1, itmp2;
    struct MEM_READ_STRUCT GTI_ADDRESS_QUAL *Pmemr;

    Pmemr = pXcvrInfo->PntMEMR + pXcvrInfo->CurrentCmd.offset;

    pHostDb = GET_HOST_DB_PTR(pXcvrInfo);

    /* Using Command Table Information Read Starlet Memory Read */
    /* Paramaters                                               */

    HostPnt = Pmemr->dest;
    id = Pmemr->id;

    /* Check for Host2Host Copy */
    if( Pmemr->dump == GTI_HOST2HOST )
    {

        /* Check for Special TYPE                          */
        /* If its not a copy then handle as single 16 bit #*/
        switch(Pmemr->type)
        {
           case GTI_TYPE_OREQUAL:
                 SET_HOST_DB_B(pXcvrInfo, Pmemr->dest,
                     ( GET_HOST_DB_B(pXcvrInfo, Pmemr->src_st) |
                     GET_HOST_DB_B(pXcvrInfo, Pmemr->dest)) );

                 SET_HOST_DB_B(pXcvrInfo, (Pmemr->dest) + 1,
                     ( GET_HOST_DB_B(pXcvrInfo, (Pmemr->src_st) +1 ) |
                     GET_HOST_DB_B(pXcvrInfo, (Pmemr->dest) + 1)) );
                 break;

           case GTI_TYPE_ANDEQUAL:
                 SET_HOST_DB_B(pXcvrInfo, Pmemr->dest,
                     ( GET_HOST_DB_B(pXcvrInfo, Pmemr->src_st) &
                     GET_HOST_DB_B(pXcvrInfo,Pmemr->dest)) );

                 SET_HOST_DB_B(pXcvrInfo, (Pmemr->dest) + 1,
                     ( GET_HOST_DB_B(pXcvrInfo, (Pmemr->src_st) +1 ) &
                     GET_HOST_DB_B(pXcvrInfo, (Pmemr->dest) + 1)) );
                 break;

           case GTI_TYPE_EXOREQUAL:
                 SET_HOST_DB_B(pXcvrInfo, Pmemr->dest,
                     ( GET_HOST_DB_B(pXcvrInfo, Pmemr->src_st) ^
                     GET_HOST_DB_B(pXcvrInfo,Pmemr->dest)) );

                 SET_HOST_DB_B(pXcvrInfo, (Pmemr->dest) + 1,
                     ( GET_HOST_DB_B(pXcvrInfo, (Pmemr->src_st) +1 ) ^
                     GET_HOST_DB_B(pXcvrInfo, (Pmemr->dest) + 1)) );
                 break;

           case GTI_TYPE_PLUSEQUAL:
                 itmp1 = GET_HOST_DB_W(pXcvrInfo, Pmemr->dest);
                 itmp2 = GET_HOST_DB_W(pXcvrInfo, Pmemr->src_st);

                 itmp1 = (GTI_WORD)( itmp1 + itmp2);

                 SET_HOST_DB_W(pXcvrInfo, Pmemr->dest, itmp1);

                 break;

           case GTI_TYPE_MINUSEQUAL:
                 itmp1 = GET_HOST_DB_W(pXcvrInfo, Pmemr->dest);
                 itmp2 = GET_HOST_DB_W(pXcvrInfo, Pmemr->src_st);

                 itmp1 = (GTI_WORD)( itmp1 - itmp2);

                 SET_HOST_DB_W(pXcvrInfo, Pmemr->dest, itmp1);

                 break;

           case GTI_TYPE_RIGHTSHIFT:
                 itmp1 = GET_HOST_DB_W(pXcvrInfo, Pmemr->dest);
                 itmp2 = GET_HOST_DB_W(pXcvrInfo, Pmemr->src_st);

                 itmp1 >>= itmp2;

                 SET_HOST_DB_W(pXcvrInfo, Pmemr->dest, itmp1);

                 break;

           case GTI_TYPE_LEFTSHIFT:
                 itmp1 = GET_HOST_DB_W(pXcvrInfo, Pmemr->dest);
                 itmp2 = GET_HOST_DB_W(pXcvrInfo, Pmemr->src_st);

                 itmp1 <<= itmp2;

                 SET_HOST_DB_W(pXcvrInfo, Pmemr->dest, itmp1);

                 break;

           case GTI_TYPE_COPY:
           default:
                Gti_CopyHostDBToHostDB( pXcvrInfo, (GTI_WORD)Pmemr->src_st,
                          Pmemr->src_sp, HostPnt );
                break;
        }
        return;
    }




    size = Gti_GetWordSize(Pmemr->type);

    if(id == GTI_DSP_X)
        id = GET_HOST_DB_B(pXcvrInfo, GTI_WHICH_STAR_LOCATION_X);

    if(id == GTI_DSP_Y)
        id = GET_HOST_DB_B(pXcvrInfo, GTI_WHICH_STAR_LOCATION_Y);

    if(id == GTI_DSP_Z)
        id = GET_HOST_DB_B(pXcvrInfo, GTI_WHICH_STAR_LOCATION_Z);;

    if(id == GTI_DSP_NONE)
        return;     /* Skip this Operation */


    /* Configure for Read from STARLET RAM */
    CONFIGURE_DSP(pXcvrInfo, id, GTI_READ_BLOCK_MODE);
    WRITE_DSP_ADDR(pXcvrInfo, id, Pmemr->src_st, Pmemr->type|GTI_READ_DSP);

    wWordsToRead = (GTI_WORD)(Pmemr->src_sp - Pmemr->src_st + 1);
    switch(size)
    {
        case 2:
        case 6:
            READ_DSP_BLOCK(pXcvrInfo, id, GTI_DATAL_DSP, wWordsToRead * 2, &pHostDb[HostPnt]);
            break;
        case 3:
            READ_DSP_EXT_BLOCK(pXcvrInfo, id, GTI_EXT_DATA_DSP, wWordsToRead * 3, &pHostDb[HostPnt]);
            break;
        default:
            break;
    }
}

/**/
void
Gti_ParseFILL(GTI_XCVR_INFO *pXcvrInfo)
{
    GTI_WORD GTI_LOCAL_QUAL length;
    GTI_BYTE GTI_LOCAL_QUAL bSize;
    GTI_BYTE GTI_LOCAL_QUAL id;
    struct FILL_STRUCT GTI_ADDRESS_QUAL *PFill;

    PFill = pXcvrInfo->PntFill + pXcvrInfo->CurrentCmd.offset;

    /* Using Command Table Information Read Fill info           */
    /* Fill RAM with specified data                             */

    bSize = Gti_GetWordSize(PFill->type);
    id = PFill->id;

    if(id == GTI_DSP_X)
        id = GET_HOST_DB_B(pXcvrInfo, GTI_WHICH_STAR_LOCATION_X);

    if(id == GTI_DSP_Y)
        id = GET_HOST_DB_B(pXcvrInfo, GTI_WHICH_STAR_LOCATION_Y);

    if(id == GTI_DSP_Z)
        id = GET_HOST_DB_B(pXcvrInfo, GTI_WHICH_STAR_LOCATION_Z);;

    if(id == GTI_DSP_NONE)
        return;     /* Skip this Operation */

    /* Configure for Write to RAM */
    CONFIGURE_DSP(pXcvrInfo, id, GTI_AP);

#ifdef GTI_CRAM_16BIT
    /* Look for 16bit CRAM */
    if(bSize == GTI_SPECIAL_CRAM)
    {
        WRITE_DSP_SPECIAL(pXcvrInfo, id, PFill->STARPnt,
              (GTI_BYTE)(PFill->type-0x20), PFill->dat);
    }
    else
#endif
    {
        WRITE_DSP_ADDR(pXcvrInfo, id, PFill->STARPnt, PFill->type);
    }

    /* Write Data from Start to End */
    length=0;

    do
    {
        if(bSize == 3)
        {
            WRITE_DSP_BYTE(pXcvrInfo, id, GTI_EXT_DATA_DSP, PFill->dat);
        }

        WRITE_DSP_WORD(pXcvrInfo, id, PFill->dat, PFill->dat);

        if(bSize == 4)
        {
            WRITE_DSP_WORD(pXcvrInfo, id, PFill->dat, PFill->dat);
        }
        if(bSize == 6)
        {
            WRITE_DSP_WORD(pXcvrInfo,id, PFill->dat, PFill->dat);
            WRITE_DSP_WORD(pXcvrInfo,id, PFill->dat, PFill->dat);
        }

        ++length;

    } while (length < PFill->STARLen);

    UPDATE_DSP(pXcvrInfo);
}

/**/
void
Gti_ParseHOSTL(GTI_XCVR_INFO *pXcvrInfo)
{
    GTI_WORD GTI_LOCAL_QUAL offset;
    GTI_BYTE GTI_LOCAL_QUAL id;

    offset = pXcvrInfo->CurrentCmd.offset;
    id = (pXcvrInfo->PntHOSTL+offset)->id;

    if(id == GTI_DSP_X)
        id = GET_HOST_DB_B(pXcvrInfo, GTI_WHICH_STAR_LOCATION_X);

    if(id == GTI_DSP_Y)
        id = GET_HOST_DB_B(pXcvrInfo, GTI_WHICH_STAR_LOCATION_Y);

    if(id == GTI_DSP_Z)
        id = GET_HOST_DB_B(pXcvrInfo, GTI_WHICH_STAR_LOCATION_Z);

    if(id == GTI_DSP_NONE)
        return;     /* Skip this Operation */

    /* Changing the way this table is handled */

    /* Using Command Table Information Write Host STARLET Registers */

    if( id == GTI_DSP_DMA )
    {
        WRITE_DSP_REG(pXcvrInfo, (pXcvrInfo->PntHOSTL + offset)->address,
            (pXcvrInfo->PntHOSTL+offset)->value );
    }
    else
    {
        WRITE_DSP_BYTE(pXcvrInfo, id, (pXcvrInfo->PntHOSTL + offset)->address,
            (pXcvrInfo->PntHOSTL+offset)->value );
    }

    UPDATE_DSP(pXcvrInfo);
}

/**/
void
Gti_ParseIMML(GTI_XCVR_INFO *pXcvrInfo)
{
    struct IMM_1_LOAD_STRUCT GTI_ADDRESS_QUAL *Pimml;
    GTI_BYTE GTI_LOCAL_QUAL size;
    GTI_BYTE GTI_LOCAL_QUAL id;

#if( defined(GTI_DSP_SPECIAL_BSRAM) )
    GTI_BYTE GTI_LOCAL_QUAL lowin, medin, highin;
    GTI_BYTE GTI_LOCAL_QUAL lowout, medout, highout;
#endif

    Pimml = pXcvrInfo->PntIMML + pXcvrInfo->CurrentCmd.offset;

    /* Using Command Table Information Write Starlet Memory */

    id = Pimml->id;
    if(id == GTI_DSP_X)
        id = GET_HOST_DB_B(pXcvrInfo, GTI_WHICH_STAR_LOCATION_X);

    if(id == GTI_DSP_Y)
        id = GET_HOST_DB_B(pXcvrInfo, GTI_WHICH_STAR_LOCATION_Y);

    if(id == GTI_DSP_Z)
        id = GET_HOST_DB_B(pXcvrInfo, GTI_WHICH_STAR_LOCATION_Z);;

    if(id == GTI_DSP_NONE)
        return;     /* Skip this Operation */

    /* Configure for Write to RAM */

    /*while( id != GTI_EOT ){   Comment out when change SAS */
    size = Gti_GetWordSize(Pimml->type);

    /* Configure for Write to RAM-REG */
    CONFIGURE_DSP(pXcvrInfo, id, GTI_AP);

#ifdef GTI_CRAM_16BIT
    /* Select RAM */
    /* Look for 16bit CRAM */
    if(size == GTI_SPECIAL_CRAM)
    {
        WRITE_DSP_SPECIAL(pXcvrInfo, id, Pimml->address, (GTI_BYTE)(Pimml->type-GTI_CRAM16_ADJUST), 0x00);
    }
    else
#endif
    {
        WRITE_DSP_ADDR(pXcvrInfo, id, Pimml->address, Pimml->type);
    }

    switch(size)
    {
#ifdef GTI_CRAM_16BIT
    case GTI_SPECIAL_CRAM: /* Special CRAM 16 bit access */
        WRITE_DSP_WORD(pXcvrInfo, id, Pimml->value[0], Pimml->value[1]);
        break;
#endif
    case 2:
        WRITE_DSP_WORD(pXcvrInfo, id, Pimml->value[0], Pimml->value[1]);
        break;

    case 3:
#if( defined(GTI_DSP_SPECIAL_BSRAM) )
        if(Pimml->type == GTI_BSRAM)
        {
            /* This is Special for BSRAM bit order             */
            /* Low Bit justified - Different from Block write  */
            /* which is hight nibble justified                 */
            /* The Bit order in ROM does not match STARLET     */
            /*                 1,0 ,x,x,x,x,      19,18        */
            /* LowOut =  LowIn[1,0],x,x,x,x,HighIn[3,2]        */
            /*                 9,8,7,6,5,4,       3,2          */
            /* MedOut =  MedIn[1,0],LowIn[7,6,5,4,3,2]         */
            /*                 17,16,15,14,13,12,11,10         */
            /* HighOut = HighIn[1,0],MedIn[7,6,5,4,3,2]        */

            lowin = Pimml->value[0];
            medin = Pimml->value[1];
            highin =Pimml->value[2];

            /* Reorder BITS for STARLET   */
            lowout = (GTI_BYTE)(((highin >> 2) & 0x03) | ((lowin<<6) & 0xc0));
            medout = (GTI_BYTE)(((medin << 6) & 0xc0)  | ((lowin>>2) & 0x3f));
            highout= (GTI_BYTE)(((highin << 6) & 0xc0) | ((medin>>2) & 0x3f));

            WRITE_DSP_BYTE(pXcvrInfo,id, GTI_EXT_DATA_DSP,lowout);
            WRITE_DSP_WORD(pXcvrInfo,id,medout,highout);
        }
        else
        {
            WRITE_DSP_BYTE(pXcvrInfo, id, GTI_EXT_DATA_DSP, Pimml->value[0]);
            WRITE_DSP_WORD(pXcvrInfo, id, Pimml->value[1], Pimml->value[2]);
        }
#else
        WRITE_DSP_BYTE(pXcvrInfo, id, GTI_EXT_DATA_DSP, Pimml->value[0]);
        WRITE_DSP_WORD(pXcvrInfo, id, Pimml->value[1], Pimml->value[2]);
#endif
        break;

    case 4:
        WRITE_DSP_WORD( pXcvrInfo, id, Pimml->value[0], Pimml->value[1]);
        WRITE_DSP_WORD( pXcvrInfo, id, Pimml->value[2], Pimml->value[3]);
        break;

    case 6:
        WRITE_DSP_WORD(pXcvrInfo, id, Pimml->value[0], Pimml->value[1]);
        WRITE_DSP_WORD(pXcvrInfo, id, Pimml->value[2], Pimml->value[3]);
        WRITE_DSP_WORD(pXcvrInfo, id, Pimml->value[4], Pimml->value[5]);
        break;
    default:
        /* Should not get here */
        break;
    }

    /*++pXcvrInfo->CurrentCmd.offset;   comment out section when change SAS */
    /*
    id=Pimml->id;
    if(id == GTI_DSP_X ) id = GET_HOST_DB_B(pXcvrInfo, GTI_WHICH_STAR_LOCATION_X);
    if(id == GTI_DSP_Y ) id = GET_HOST_DB_B(pXcvrInfo, GTI_WHICH_STAR_LOCATION_Y);
    if(id == GTI_DSP_Z ) id = GET_HOST_DB_B(pXcvrInfo, GTI_WHICH_STAR_LOCATION_Z);;
    */
 /*}*/

    UPDATE_DSP(pXcvrInfo);
}

/**/
void
Gti_ParseIMMR(GTI_XCVR_INFO *pXcvrInfo)
{
    struct IMM_2_READ_STRUCT GTI_ADDRESS_QUAL *Pimmr;
    GTI_WORD itmp;

    Pimmr = pXcvrInfo->PntIMMR + pXcvrInfo->CurrentCmd.offset;

    /* Using Command Table Information Write Host Memory */
    switch(Pimmr->type)
    {
    case GTI_TYPE_COPY:
        SET_HOST_DB_B(pXcvrInfo, Pimmr->address,
            (GTI_BYTE)((Pimmr->value) & 0x00ff));

        SET_HOST_DB_B(pXcvrInfo, Pimmr->address + 1,
            (GTI_BYTE)(((Pimmr->value) >> 0x8) & 0x00ff));
        break;

    case GTI_TYPE_OREQUAL:
        SET_HOST_DB_B(pXcvrInfo, Pimmr->address,
            (GTI_BYTE)(((Pimmr->value) & 0x00ff) |
            GET_HOST_DB_B(pXcvrInfo,Pimmr->address)));

        SET_HOST_DB_B(pXcvrInfo, (Pimmr->address) + 1,
            (GTI_BYTE)((((Pimmr->value) >> 0x8) & 0x00ff) |
            GET_HOST_DB_B(pXcvrInfo, (Pimmr->address) + 1)));
        break;

    case GTI_TYPE_ANDEQUAL:
        SET_HOST_DB_B(pXcvrInfo, Pimmr->address, (GTI_BYTE)
            (((Pimmr->value) & 0x00ff) &
            GET_HOST_DB_B(pXcvrInfo, Pimmr->address)));

        SET_HOST_DB_B(pXcvrInfo, (Pimmr->address) + 1, (GTI_BYTE)
            ((((Pimmr->value) >> 0x8) & 0x00ff) &
            GET_HOST_DB_B(pXcvrInfo, (Pimmr->address) + 1)));
        break;

    case GTI_TYPE_EXOREQUAL:
        SET_HOST_DB_B(pXcvrInfo, Pimmr->address, (GTI_BYTE)
            (((Pimmr->value) & 0x00ff) ^
            GET_HOST_DB_B(pXcvrInfo, Pimmr->address)));

        SET_HOST_DB_B(pXcvrInfo, (Pimmr->address) + 1, (GTI_BYTE)
            ((((Pimmr->value) >> 0x8) & 0x00ff) ^
            GET_HOST_DB_B(pXcvrInfo, (Pimmr->address) + 1)));
        break;

    case GTI_TYPE_PLUSEQUAL:
        itmp = (GTI_WORD) (GET_HOST_DB_B(pXcvrInfo, Pimmr->address) |
            (GET_HOST_DB_B(pXcvrInfo, (Pimmr->address) + 1) << 0x8));

        itmp = (GTI_WORD)(itmp + Pimmr->value);

        SET_HOST_DB_B(pXcvrInfo, Pimmr->address, (GTI_BYTE)itmp);
        SET_HOST_DB_B(pXcvrInfo, (Pimmr->address) + 1, (GTI_BYTE)(itmp >> 0x8));
        break;

    case GTI_TYPE_MINUSEQUAL:
        itmp = (GTI_WORD)(GET_HOST_DB_B(pXcvrInfo, Pimmr->address) |
            (GTI_WORD)(GET_HOST_DB_B(pXcvrInfo, (Pimmr->address) + 1) << 0x8));
        itmp = (GTI_WORD)(itmp - Pimmr->value);

        SET_HOST_DB_B(pXcvrInfo, Pimmr->address, (GTI_BYTE)itmp);
        SET_HOST_DB_B(pXcvrInfo, (Pimmr->address) + 1, (GTI_BYTE)(itmp >> 0x8));
        break;

    case GTI_TYPE_RIGHTSHIFT:
        itmp = (GTI_WORD)(GET_HOST_DB_B(pXcvrInfo, Pimmr->address) |
            (GTI_WORD)(GET_HOST_DB_B(pXcvrInfo, (Pimmr->address) + 1) << 0x8));

        itmp = (GTI_WORD)(itmp >> Pimmr->value);

        SET_HOST_DB_B(pXcvrInfo, Pimmr->address, (GTI_BYTE)itmp);
        SET_HOST_DB_B(pXcvrInfo, (Pimmr->address) + 1, (GTI_BYTE)(itmp >> 0x8));
        break;

    case GTI_TYPE_LEFTSHIFT:
        itmp = (GTI_WORD) (GET_HOST_DB_B(pXcvrInfo, Pimmr->address) |
            (GTI_WORD)(GET_HOST_DB_B(pXcvrInfo, (Pimmr->address) + 1) << 0x8));

        itmp = (GTI_WORD)(itmp << Pimmr->value);

        SET_HOST_DB_B(pXcvrInfo, Pimmr->address, (GTI_BYTE)itmp);
        SET_HOST_DB_B(pXcvrInfo, (Pimmr->address) + 1, (GTI_BYTE)(itmp >> 0x8));
        break;

    default:
        /* Should not get here */
        break;
    }
}

/**/
void
Gti_ParseREGInd(GTI_XCVR_INFO *pXcvrInfo)
{
    struct REG_LOADIND_STRUCT GTI_ADDRESS_QUAL *Pregi;
    GTI_BYTE GTI_LOCAL_QUAL datal,datah;
    GTI_BYTE GTI_LOCAL_QUAL id;

    Pregi = pXcvrInfo->PntREGInd + pXcvrInfo->CurrentCmd.offset;

    /* Using Command Table Information Copy Host RAM space to IREG */

    id = Pregi->id;

    if(id == GTI_DSP_X)
        id = GET_HOST_DB_B(pXcvrInfo, GTI_WHICH_STAR_LOCATION_X);

    if(id == GTI_DSP_Y)
        id = GET_HOST_DB_B(pXcvrInfo, GTI_WHICH_STAR_LOCATION_Y);

    if(id == GTI_DSP_Z)
        id = GET_HOST_DB_B(pXcvrInfo, GTI_WHICH_STAR_LOCATION_Z);;

    if(id == GTI_DSP_NONE)
        return;     /* Skip this Operation */

    datal = GET_HOST_DB_B(pXcvrInfo, Pregi->offset);
    datah = GET_HOST_DB_B(pXcvrInfo, (GTI_WORD)(Pregi->offset + 1) );

    datal |= Pregi->or_valuel;
    datah |= Pregi->or_valueh;

    /* Now Write to IREG space in starlet                       */
    /* Select IREG RAM */
    CONFIGURE_DSP(pXcvrInfo, id, GTI_NO_AI);
    WRITE_DSP_ADDR(pXcvrInfo, id, Pregi->reg, GTI_IREGA);
    WRITE_DSP_WORD(pXcvrInfo, id, datal,datah);

    UPDATE_DSP(pXcvrInfo);
}

/**/
void
Gti_ParseINDL(GTI_XCVR_INFO *pXcvrInfo)
{
    struct IND_1_LOAD_STRUCT GTI_ADDRESS_QUAL *Pindl;
    GTI_BYTE GTI_LOCAL_QUAL datal,datah;
    GTI_BYTE GTI_LOCAL_QUAL id;


    Pindl = pXcvrInfo->PntINDL + pXcvrInfo->CurrentCmd.offset;

    /* Using Command Table Information Copy Host RAM space to RAM-16Bit */

    id = Pindl->id;
    if(id == GTI_DSP_X)
        id = GET_HOST_DB_B(pXcvrInfo, GTI_WHICH_STAR_LOCATION_X);

    if(id == GTI_DSP_Y)
        id = GET_HOST_DB_B(pXcvrInfo, GTI_WHICH_STAR_LOCATION_Y);

    if(id == GTI_DSP_Z)
        id = GET_HOST_DB_B(pXcvrInfo, GTI_WHICH_STAR_LOCATION_Z);;

    if(id == GTI_DSP_NONE)
        return;   /* Skip this Operation */

    datal = GET_HOST_DB_B(pXcvrInfo, Pindl->offset);
    datah = GET_HOST_DB_B(pXcvrInfo, (GTI_WORD)(Pindl->offset + 1) );

    /* Now Write to IPU space in starlet                       */
    CONFIGURE_DSP(pXcvrInfo, id, GTI_NO_AI);

    /* Select RAM */
    WRITE_DSP_ADDR(pXcvrInfo, id, Pindl->STARPnt, Pindl->type);
    WRITE_DSP_WORD(pXcvrInfo, id, datal,datah);

    UPDATE_DSP(pXcvrInfo);
}

/**/
void
Gti_ParseIF(GTI_XCVR_INFO *pXcvrInfo)
{
    GTI_WORD GTI_LOCAL_QUAL data1,data2;
    GTI_BYTE GTI_LOCAL_QUAL result;
    struct IF_STRUCT GTI_ADDRESS_QUAL *Pif;

    Pif = pXcvrInfo->PntIF + pXcvrInfo->CurrentCmd.offset;

    /* Using Command Table Information Compare two Values*/
    /* if pass comparision then execute GOTO else NEXT   */
    data1 = (GTI_WORD)(GET_HOST_DB_B(pXcvrInfo, Pif->value1) +
        (GET_HOST_DB_B(pXcvrInfo, (Pif->value1) + 1) << 0x08));

    data2 = Pif->value2;

    /* Is "data2" an Immediate value or an RAM Offset? */
    if( ((Pif->type) & GTI_TYPE_INDIRECT) == GTI_TYPE_INDIRECT )
    {
        /* Get Host RAM Value to Compare */
        data2 = (GTI_WORD)(GET_HOST_DB_B(pXcvrInfo, data2) +
            (GET_HOST_DB_B(pXcvrInfo, data2 + 1) << 0x08));
    }

    result=GTI_FALSE;

    switch((Pif->type) & 0x7f)
    {
    case GTI_TYPE_EQUAL:
        if(data1 == data2)
            result = GTI_TRUE;
        break;

    case GTI_TYPE_NEQUAL:
        if(data1 != data2)
            result = GTI_TRUE;
        break;

    case GTI_TYPE_LESS:
        if(data1 < data2)
            result = GTI_TRUE;
        break;

    case GTI_TYPE_GREATER:
        if(data1 > data2)
            result = GTI_TRUE;
        break;

    case GTI_TYPE_GREATOREQUAL:
        if(data1 >= data2)
            result = GTI_TRUE;
        break;

    case GTI_TYPE_LESSOREQUAL:
        if(data1 <= data2)
            result = GTI_TRUE;
        break;

    default:
        /* Should not get here */
        break;
    }

    if(result == GTI_FALSE)
        pXcvrInfo->CurrentCmd.next = GTI_NEXT;
}


#ifdef GTI_MULTI_PROJ
/*^L*/
GTI_WORD
Gti_ParseCONTEX(GTI_XCVR_INFO *pXcvrInfo)
{
    GTI_WORD GTI_LOCAL_QUAL new_line;
    GTI_WORD data1;

    /* Using Command Table Information Switch to next table */
    /* pXcvrInfo->CurrentCmd.offset =  Which  YAT Table     */
    /* pXcvrInfo->CurrentCmd.next;  =  Offset into YetAtable*/

    data1 = pXcvrInfo->CurrentCmd.next;   /* Offset into YetAtable */

    if( data1  >= GTI_MAX_YAT_TABLE_SIZE )
    {
       return(GTI_FAIL);
    }

    switch( pXcvrInfo->CurrentCmd.offset ) 
    {
#ifdef GTI_PAGE_A_BUILT
       case GTI_PAGE_A:
            pXcvrInfo->PntYAT = (struct CMDPOINT GTI_ADDRESS_QUAL *)&YAT_a_tbl[0];
            break;
#endif
#ifdef GTI_PAGE_B_BUILT
       case GTI_PAGE_B:
            pXcvrInfo->PntYAT = (struct CMDPOINT GTI_ADDRESS_QUAL *)&YAT_b_tbl[0];
            break;
#endif
#ifdef GTI_PAGE_C_BUILT
       case GTI_PAGE_C:
            pXcvrInfo->PntYAT = (struct CMDPOINT GTI_ADDRESS_QUAL *)&YAT_c_tbl[0];
            break;
#endif
#ifdef GTI_PAGE_D_BUILT
       case GTI_PAGE_D:
            pXcvrInfo->PntYAT = (struct CMDPOINT GTI_ADDRESS_QUAL *)&YAT_d_tbl[0];
            break;
#endif
#ifdef GTI_PAGE_G_BUILT
       case GTI_PAGE_G:
            pXcvrInfo->PntYAT = (struct CMDPOINT GTI_ADDRESS_QUAL *)&YAT_g_tbl[0];
            break;
#endif
       default:
              return(GTI_FAIL);
    }

    new_line = (pXcvrInfo->PntYAT + data1)->start;
    if( new_line == GTI_EMPTY )
    {
       return(GTI_FAIL);
    }

#ifdef GTI_TRACE
    if( HistPnt <  GTI_SIZE_OF_TRACE )
    {
        History[0][HistPnt] = data1;  /* Yat table line # */
        History[2][HistPnt] = pXcvrInfo->CurrentCmd.offset; /* Which YAT table */
        History[1][HistPnt] = new_line;  /* information read from YAT table */
        ++HistPnt;
    }
#endif
    /* Init Table Pointers */
#ifdef GTI_DEBUG
    if(pXcvrInfo->WhichMEM == GTI_TRUE)
    {
         ReadRAMPointers(pXcvrInfo, (GTI_BYTE)(pXcvrInfo->CurrentCmd.offset));
    }
    else
        Gti_InitPointers(pXcvrInfo, (GTI_BYTE)(pXcvrInfo->CurrentCmd.offset));
#else
    Gti_InitPointers(pXcvrInfo, (GTI_BYTE)(pXcvrInfo->CurrentCmd.offset) );
#endif
    /* Set new Line number */
    pXcvrInfo->CurrentCmd.next = new_line;
    return(GTI_OK);
}
#endif /* GTI_MULTI_PROJ */

/**/
/***********************************************/
/* These are the routines that to the IPU wait */
/***********************************************/
void
Gti_ConfigReadRdy(GTI_XCVR_INFO *pXcvrInfo)
{
    CONFIGURE_DSP(pXcvrInfo, pXcvrInfo->which_ipu, GTI_NO_AI);
    WRITE_DSP_ADDR(pXcvrInfo, pXcvrInfo->which_ipu, GTI_I2H_ready,
        (GTI_BYTE)(GTI_ISRAM|GTI_READ_DSP) );
}

GTI_BYTE
Gti_IsIPUready(GTI_XCVR_INFO *pXcvrInfo)
{
    GTI_WORD GTI_LOCAL_QUAL result;

    if(pXcvrInfo->which_ipu == GTI_DSP_AB)
    {
        result = READ_DSP_WORD(pXcvrInfo, GTI_DSP_A);

        if(result == 1)
        {
            pXcvrInfo->which_ipu = GTI_DSP_A; /* Force to A Chip */

#if(defined(GTI_PROD_H2B1Q))
            /*pXcvrInfo->HostRAM[GTI_WHICH_STAR_LOCATION_Z]=pXcvrInfo->which_ipu;*/
            SET_HOST_DB_B(pXcvrInfo, GTI_WHICH_STAR_LOCATION_Z, pXcvrInfo->which_ipu);
#endif
        }
        else
        {
            result = READ_DSP_WORD(pXcvrInfo, GTI_DSP_B);
            if( result == 1 )
            {
                pXcvrInfo->which_ipu = GTI_DSP_B; /* Force to B Chip */
#if(defined(GTI_PROD_H2B1Q) )
                /*pXcvrInfo->HostRAM[GTI_WHICH_STAR_LOCATION_Z] = pXcvrInfo->which_ipu;*/
                SET_HOST_DB_B(pXcvrInfo, GTI_WHICH_STAR_LOCATION_Z, pXcvrInfo->which_ipu);
#endif
            }
        }
    }
    else
    {
        result = READ_DSP_WORD(pXcvrInfo, pXcvrInfo->which_ipu);
    }

    if(result == 1)
        return (GTI_TRUE);

    return(GTI_FALSE);
}

GTI_WORD
Gti_GetNextState(GTI_XCVR_INFO *pXcvrInfo)
{
    GTI_WORD GTI_LOCAL_QUAL result;
#ifdef GTI_MULTI_PROJ
    GTI_BYTE GTI_LOCAL_QUAL new_table;
#endif

    /* Get The Next State from the IPU ISRAM Space */
    CONFIGURE_DSP(pXcvrInfo, pXcvrInfo->which_ipu, GTI_NO_AI);

    WRITE_DSP_ADDR(pXcvrInfo, pXcvrInfo->which_ipu, GTI_I2H_next,
        (GTI_BYTE)(GTI_ISRAM|GTI_READ_DSP) );

    result = READ_DSP_WORD(pXcvrInfo, pXcvrInfo->which_ipu);

    /* Clear I2H Ready Flag */
    WRITE_DSP_ADDR(pXcvrInfo, pXcvrInfo->which_ipu, GTI_I2H_ready, GTI_ISRAM);
    WRITE_DSP_WORD(pXcvrInfo, pXcvrInfo->which_ipu, 0x00,0x00);

    /* Clear Possible Interupt that triggered Gti_StartXcvrCmd */
    /* Customer should Clear Interrupt of STARLET that generated interrupt */
#ifdef GTI_DEBUG
    ClearSTARInterrupt(pXcvrInfo, pXcvrInfo->which_ipu);
#endif

#ifdef GTI_MULTI_PROJ
    /* If Contex SW then get Line number from YAT table */
    if( (pXcvrInfo->CurrentCmd.next == GTI_CONTEX_A) ||
        (pXcvrInfo->CurrentCmd.next == GTI_CONTEX_B) )
    {
#ifdef GTI_TRACE
        if(HistPnt <  GTI_SIZE_OF_TRACE)
        {
            History[0][HistPnt] = result;
        }
#endif
        new_table = (pXcvrInfo->PntYAT+result)->table_sw;
        result = (pXcvrInfo->PntYAT+result)->start;

        /*result=start;*/

#ifdef GTI_TRACE
        if(HistPnt <  GTI_SIZE_OF_TRACE)
        {
            History[2][HistPnt] = new_table;
            History[1][HistPnt] = result;
            ++HistPnt;
        }
#endif
        /* Init Table Pointers */
#ifdef GTI_DEBUG
        if(pXcvrInfo->WhichMEM == RAM_MEM)
        {
              ReadRAMPointers(pXcvrInfo, new_table);
        }
        else
             Gti_InitPointers(pXcvrInfo, new_table);
#else
        Gti_InitPointers(pXcvrInfo, new_table);
#endif
     }
#endif

    return (result);
}

void
Gti_WaitIPU(GTI_XCVR_INFO *pXcvrInfo)
{
    /* Stay Here till IPU is ready continue */
    /* IPU will indicate NEXT table state   */
    Gti_ConfigReadRdy(pXcvrInfo);

    while(Gti_IsIPUready(pXcvrInfo)==GTI_FALSE) {};

    /* IPU is ready to continue Get Next State */
    pXcvrInfo->table_pnt = Gti_GetNextState(pXcvrInfo);
}
/*^L*/
/******************************************************/
/* ROUTINES CALLED BY COMMAND TABLE GO HERE           */
/******************************************************/
GTI_BYTE
Gti_GenFun1(void)
{
    /*SerialTXString("DEBUG:Call Gen Function 1\n");*/
    return(0);
}

GTI_BYTE
Gti_GenFun2(void)
{
    /*SerialTXString("DEBUG:Call Gen Function 2\n");*/
    return(0);
}

/****************************************************/
/* Name:CopyHostDBToDsp                             */
/* Input:                                           */
/* pXcvrInfo: System pointer                          */
/* wChnl  : Dsp channel                             */
/* id     : Host Ram location                       */
/* bReg   : Extention register                      */
/* wLength: Block length in bytes                   */
/****************************************************/
void
Gti_CopyHostDBToDsp(GTI_XCVR_INFO* pXcvrInfo, GTI_BYTE wChnl,
                GTI_WORD wId, GTI_BYTE bReg, GTI_WORD wLength)
{
    WRITE_DSP_BLOCK(pXcvrInfo, wChnl, bReg, wLength,
        &(pXcvrInfo->paramsDB[wId]));
}

/****************************************************/
/* Name:CopyHostDBToHostDB                          */
/* Input:                                           */
/* pXcvrInfo: System pointer                          */
/* startSrc     : Host Ram location                 */
/* endSrc     : Host Ram location                   */
/* dest       : Host Ram location                   */
/****************************************************/
void
Gti_CopyHostDBToHostDB(GTI_XCVR_INFO* pXcvrInfo,
                   GTI_WORD startSrc,
                   GTI_WORD endSrc,
                   GTI_WORD dest )
{
    GTI_WORD istmp, idtmp;

    for( istmp = startSrc, idtmp = dest; istmp < endSrc; ++istmp,++idtmp )
    {
        pXcvrInfo->paramsDB[idtmp] = pXcvrInfo->paramsDB[istmp];
    }
}

#endif

GTI_BYTE
Gti_GetWordSize(GTI_BYTE ram_type)
{
    GTI_BYTE GTI_LOCAL_QUAL size;
    size = 2;

    switch(ram_type)
    {
    case GTI_PCURAM0:
    case GTI_PCURAM1:
#if (defined (GTI_DSPMULTI ) || defined (GTI_DSPFRAMER_B1) || defined (GTI_DSPQUAD) || defined (GTI_DSPPCI) || defined(GTI_DSPFRAMER_C1) || defined(GTI_DSPOCTANE) || defined(GTI_DSPVIKING))
    case GTI_PCURAM2:
    case GTI_PCURAM3:
    case GTI_PSRAM1W:
#if (defined (GTI_DSPQUAD) || defined (GTI_DSPPCI) || defined(GTI_DSPFRAMER_C1) || defined(GTI_DSPOCTANE) || defined(GTI_DSPVIKING) )
    case GTI_PCURAM4:
    case GTI_PCURAM5:
#endif
#endif
#if (defined (GTI_DSPMULTI ) || defined (GTI_DSPFRAMER_B1) || defined(GTI_DSPQUAD) || defined(GTI_DSPPCI) || defined(GTI_DSPFRAMER_C1) || defined(GTI_DSPOCTANE) || defined(GTI_DSPVIKING) )
    case GTI_RSRAM8:
    case GTI_RSRAM9:
    case GTI_RSRAMA:
    case GTI_RSRAMB:
    case GTI_RSRAMC:
    case GTI_RSRAMD:
#endif
    case GTI_PSRAM0W:
    case GTI_VRAM:
    case GTI_LRAM:
    case GTI_ISRAM:
    case GTI_IRAM0:
    case GTI_IRAM1:
#if (defined(GTI_DSP_SUPERNOVA_ENG))
    case GTI_IRAM2:
    case GTI_IRAM3:
    case GTI_IRAM4:
    case GTI_IRAM5:
    case GTI_IRAM6:
    case GTI_IRAM7:
    case GTI_IRAM8:
    case GTI_IRAM9:
    case GTI_IRAMA:
    case GTI_IRAMB:
    case GTI_IRAMC:
    case GTI_IRAMD:
    case GTI_IRAME:
    case GTI_IRAMF:
#endif
    case GTI_RSRAM0:
    case GTI_RSRAM1:
    case GTI_RSRAM2:
    case GTI_RSRAM3:
    case GTI_RSRAM4:
    case GTI_RSRAM5:
    case GTI_RSRAM6:
    case GTI_RSRAM7:
    case GTI_DRAM0:
    case GTI_DRAM2:
    case GTI_DRAM4:
    case GTI_DBROAD:
    case GTI_TRAM:
#if( (!defined (GTI_DSPMULTI )) && (!defined (GTI_DSPFRAMER_B1)) && (!defined (GTI_DSPQUAD)) && (!defined(GTI_DSPOCTANE)) && (!defined (GTI_DSPPCI)) && (!defined (GTI_DSPFRAMER_C1)) && (!defined(GTI_DSPVIKING))  )
    case GTI_PSRAMB:
    case GTI_BSRAM16:
#endif
        break;

#if (defined(GTI_CRAM_16BIT))
    case GTI_CRAM016:
    case GTI_CRAM116:
    case GTI_CRAM216:
    case GTI_CRAM316:
    case GTI_CRAM416:
    case GTI_CRAM516:
    case GTI_CBROAD16:
        size = GTI_SPECIAL_CRAM;  /* Special 16 bit mode access */
        break;
#endif

    case GTI_CRAM0:
    case GTI_CRAM1:
    case GTI_CRAM2:
    case GTI_CRAM3:
    case GTI_CRAM4:
    case GTI_CRAM5:
    case GTI_CBROAD:
    case GTI_BSRAM:
#if (defined (GTI_DSP_FDP_ENG) )
    case GTI_FDPSRAM0:
    case GTI_FDPSRAM1:
    case GTI_FDPSRAM2:
    case GTI_FDPSRAM3:
    case GTI_FDPSRAM4:
    case GTI_FDPSRAM5:
    case GTI_FDPSRAM6:
    case GTI_FDPSRAM7:
    case GTI_FDPSRAM8:
    case GTI_FDPSRAM9:
    case GTI_FDPSRAMA:
    case GTI_FDPSRAMB:
    case GTI_FDPSRAMC:
    case GTI_FDPSRAMD:
    case GTI_FDPSRAME:
    case GTI_FDPSRAMF:
#if (defined(GTI_DSP_SUPERNOVA_ENG))
    case GTI_FDPSRAM10:
    case GTI_FDPSRAM11:
    case GTI_FDPSRAM12:
    case GTI_FDPSRAM13:
#endif
#endif
        size = 3;
        break;
#if (defined (GTI_DSP_FDP_ENG))
    case GTI_FDPRAM0:
    case GTI_FDPRAM1:
    case GTI_FDPRAM2:
    case GTI_FDPRAM3:
#if (defined(GTI_DSP_SUPERNOVA_ENG))
    case GTI_FDPRAM4:
    case GTI_FDPRAM5:
#endif
        size = 4;
        break;
#endif /* FDP Eng */
    case GTI_FRAM:
    case GTI_BRAM:
        size = 6;
        break;

    default:
        /* Should never get here */
        break;
     }
     return(size);
}

GTI_BYTE
Gti_GetWordBitSize(GTI_BYTE ram_type)
{
    GTI_BYTE GTI_LOCAL_QUAL size;

    switch(ram_type)
    {
    case GTI_PCURAM0:
    case GTI_PCURAM1:
    case GTI_PSRAM0W:
#if (defined (GTI_DSPMULTI ) || defined (GTI_DSPFRAMER_B1) || defined (GTI_DSPQUAD) || defined(GTI_DSPPCI) || defined(GTI_DSPFRAMER_C1) || defined(GTI_DSPOCTANE) || defined(GTI_DSPVIKING))
    case GTI_PCURAM2:
    case GTI_PCURAM3:
    case GTI_PSRAM1W:
#if (defined  (GTI_DSPQUAD) || defined (GTI_DSPPCI) || defined(GTI_DSPFRAMER_C1) || defined(GTI_DSPOCTANE) || defined(GTI_DSPVIKING))
    case GTI_PCURAM4:
    case GTI_PCURAM5:
#endif
#endif
#if (defined (GTI_DSPMULTI ) || defined (GTI_DSPFRAMER_B1) || defined(GTI_DSPQUAD) || defined(GTI_DSPPCI) || defined(GTI_DSPFRAMER_C1) || defined(GTI_DSPOCTANE) || defined(GTI_DSPVIKING) )
    case GTI_RSRAM8:
    case GTI_RSRAM9:
    case GTI_RSRAMA:
    case GTI_RSRAMB:
    case GTI_RSRAMC:
    case GTI_RSRAMD:
#endif
    case GTI_VRAM:
    case GTI_LRAM:
    case GTI_ISRAM:
    case GTI_IRAM0:
    case GTI_IRAM1:
#if (defined(GTI_DSP_SUPERNOVA_ENG))
    case GTI_IRAM2:
    case GTI_IRAM3:
    case GTI_IRAM4:
    case GTI_IRAM5:
    case GTI_IRAM6:
    case GTI_IRAM7:
    case GTI_IRAM8:
    case GTI_IRAM9:
    case GTI_IRAMA:
    case GTI_IRAMB:
    case GTI_IRAMC:
    case GTI_IRAMD:
    case GTI_IRAME:
    case GTI_IRAMF:
#endif
    case GTI_RSRAM0:
    case GTI_RSRAM1:
    case GTI_RSRAM2:
    case GTI_RSRAM3:
    case GTI_RSRAM4:
    case GTI_RSRAM5:
    case GTI_RSRAM6:
    case GTI_RSRAM7:
    case GTI_TRAM:
#if( (!defined (GTI_DSPMULTI )) && (!defined (GTI_DSPFRAMER_B1)) && (!defined (GTI_DSPQUAD)) && !defined(GTI_DSPOCTANE) && (!defined (GTI_DSPPCI)) && (!defined(GTI_DSPFRAMER_C1)) && (!defined(GTI_DSPVIKING)) )
    case GTI_PSRAMB:
    case GTI_BSRAM16:
    case GTI_IREGA:
#endif
        size = 16;
        break;

    case GTI_DRAM0:
    case GTI_DRAM2:
    case GTI_DRAM4:
    case GTI_DBROAD:
        size = 12;
        break;

#if (defined(GTI_CRAM_16BIT))
    case GTI_CRAM016:
    case GTI_CRAM116:
    case GTI_CRAM216:
    case GTI_CRAM316:
    case GTI_CRAM416:
    case GTI_CRAM516:
    case GTI_CBROAD16:
        size = 16;  /* Special 16 bit mode access */
        break;
#endif

    case GTI_CRAM0:
    case GTI_CRAM1:
    case GTI_CRAM2:
    case GTI_CRAM3:
    case GTI_CRAM4:
    case GTI_CRAM5:
    case GTI_CBROAD:
        size = 24;
        break;

    case GTI_BSRAM:
#if ( defined(GTI_DSPMULTI) || defined(GTI_DSPQUAD) || defined(GTI_DSPPCI) || defined(GTI_DSPOCTANE) || defined(GTI_DSPVIKING) )
    case GTI_FDPSRAM0:
    case GTI_FDPSRAM1:
    case GTI_FDPSRAM2:
    case GTI_FDPSRAM3:
    case GTI_FDPSRAM4:
    case GTI_FDPSRAM5:
    case GTI_FDPSRAM6:
    case GTI_FDPSRAM7:
    case GTI_FDPSRAM8:
    case GTI_FDPSRAM9:
    case GTI_FDPSRAMA:
    case GTI_FDPSRAMB:
    case GTI_FDPSRAMC:
    case GTI_FDPSRAMD:
    case GTI_FDPSRAME:
    case GTI_FDPSRAMF:
#if (defined(GTI_DSP_SUPERNOVA_ENG))
    case GTI_FDPSRAM10:
    case GTI_FDPSRAM11:
    case GTI_FDPSRAM12:
    case GTI_FDPSRAM13:
#endif
#endif
        size = 20;
        break;
#if ( defined(GTI_DSPMULTI) || defined(GTI_DSPQUAD) || defined(GTI_DSPPCI) || defined(GTI_DSPOCTANE) || defined(GTI_DSPVIKING) )
    case GTI_FDPRAM0:
    case GTI_FDPRAM1:
    case GTI_FDPRAM2:
    case GTI_FDPRAM3:
#if (defined(GTI_DSP_SUPERNOVA_ENG))
    case GTI_FDPRAM4:
    case GTI_FDPRAM5:
#endif
        size = 32;
        break;
#endif
    case GTI_FRAM:
    case GTI_BRAM:
        size = 48;
        break;
    default:
        size = 16;
        break;
     }

     return(size);
}

