/****************************************************************************
 * GlobeSpan definitions
 *
 * Copyright (c) 2000 GlobeSpan Semiconductor Inc. -- http://www.globespan.net
 * All rights reserved.
 * History:
 *   2/5/01 PG: Added support for EOC
 *   7/18/01 PG Bumped the Driver Rev 
 *-I*************************************************************************/
#ifndef GSI_H_
#define GSI_H_

#define GSI_DRIVER_VERSION  "v2.04.0006E"
#define GSI_DRIVER_DATE     "11/28/2001"


#undef ASSERT

#include <drvh/common.h>
#include <drvh/GpSnull.h>
#include <drvh/atmphy.h>
#include <gti.h>
#include <drvh/pciphy.h>
#include <drvh/gsi_ioctl.h>
#include "drvh/GpUtil.h"
/*#include "drvh/GpDebug.h"*/

#include <Epci/GpInclude.h>

#include "wanpipe_abstr.h"
#include "wanpipe_cfg.h"

#if defined(__NetBSD__)
# include <sys/errno.h>
#else
# include "errno.h"
#endif

#define __ADSL_IFACE
#include "sdla_adsl_iface.h"

#define IFNAMSIZ 32
#define ETHER_ADDR_LEN 6

#define _NELEM(arr)     (sizeof(arr) / sizeof(arr[0]))
#define _OFFSETOF(t,m)  ((u_int32_t) &((t *)0)->m)
#define _MALIGN(l)      (((l) + 4 - 1) & ~(4 - 1))

#define MTAG    "wanpipe"
#define DTYPE    "Pulsar-Atlas"

#ifndef MAX_UDELAY_MS
#define MAX_UDELAY_MS 5
#endif 

/* #define EXECUTE_CMD	_GTI_XCVR_INFO */
/* #define GtiSysBlock	GtiXcvrBlock */


typedef struct _adapter_t           ATM_USB_INSTANCE, *PATM_USB_INSTANCE;

/*******************************************************************/
/*		ATM UNI DECLARATIONS				   */ 	
/*******************************************************************/


typedef void * ATM_UNI_HANDLE;
typedef void * ATM_UNI_USER_HANDLE;
BOOLEAN
GpatmisAtmUniInitialize( void);
/*------------------------------------------------------------------------------
//
// ATM_UNI_USER_LINK_STATUS_CHANGE
//		Defines the prototype for the user link status change function.  The
//		user link status change function is called whenever the status of
//		the physical link has changed.
//
//		Note:  This routine is optional and specified when the ATM UNI is 
//			   created.
//
//------------------------------------------------------------------------------
*/
typedef void (*ATM_UNI_USER_LINK_STATUS_CHANGE) 
                 ( ATM_UNI_USER_HANDLE,ATM_PHY_LINK_STATE,ULONG,ULONG);

ATM_UNI_HANDLE
GpatmisAtmUniCreate( 
	ATM_UNI_USER_HANDLE				UserHandle,
	ATM_UNI_USER_LINK_STATUS_CHANGE	UserLinkStatusChange,
	BOOLEAN							IndicatePackets,
	ULONG							MediaSpecificInformationSize,
	USHORT							MaxSduSize, 
	USHORT							RxBufferCount, 
	USHORT							TxBufferCount, 
	NDIS_HANDLE						MiniportAdapterHandle,
	ATM_PHY_BIND 					PhyBind,
	ATM_PHY_DEVICE_HANDLE			PhyDeviceHandle,
	PUCHAR							PhyUniqueId
);


void
GpatmisAtmUniDestroy( ATM_UNI_HANDLE AtmUniHandle);


/***********************************************************************************/
/* 					RFC1577 DECLARATIONS			   */ 	
/***********************************************************************************/



BOOLEAN GpatmisRfc1577Initialize(PATM_USB_INSTANCE Instance);

NDIS_STATUS GpatmisRfc1577Send (PATM_USB_INSTANCE pInstance, 
                                void* rx_skb, 
                             	unsigned int Flag);
/*
NDIS_STATUS GpatmisRfc1577Send (PATM_USB_INSTANCE pInstance, 
                             	unsigned char* tx_data, 
				int tx_len,
                             	unsigned int Flag);
*/


/***************************************************************************************/
/* 			RFC2364 DECLARATIONS					   */ 	
/***************************************************************************************/


#define RFC2364_MAX_SDU_SIZE        2048

BOOLEAN GpatmisRfc2364Initialize(
    PATM_USB_INSTANCE       Instance,
    ATM_PHY_BIND            PhyBind,
    ATM_PHY_DEVICE_HANDLE   PhyDeviceHandle
    );

void GpatmisRfc2364Shutdown(
    PATM_USB_INSTANCE       Instance
    );

NDIS_STATUS GpatmisRfc2364DeleteVc(
    PATM_USB_INSTANCE       Instance,
    UINT                    VPI,
    UINT                    VCI
    );

NDIS_STATUS GpatmisRfc2364CreateVc(
    PATM_USB_INSTANCE       Instance,
    UINT                    VPI,
    UINT                    VCI,
    NDIS_HANDLE		    NdisLinkContext
    );

NDIS_STATUS GpatmisRfc2364Send (PATM_USB_INSTANCE pInstance, 
                                void* rx_skb, 
                             	unsigned int Flag);

/***************************************************************************************/
/* 				RFC1483 DECLARATIONS				   */ 	
/***************************************************************************************/


#define ETHERNET_HEADER_SIZE            14
#define ETHERNET_MAX_PACKET_SIZE        1500
#define ETHERNET_LAN_FCS_SIZE           4

#define RFC1483_MAX_ENCAPSULATION_SIZE      10

#define RFC1483_MAX_SDU_SIZE            ETHERNET_MAX_PACKET_SIZE       + \
                                        ETHERNET_HEADER_SIZE           + \
                                        RFC1483_MAX_ENCAPSULATION_SIZE + \
                                        ETHERNET_LAN_FCS_SIZE


/*------------------------------------------------------------------------------
//
// Function Declarations
//
//------------------------------------------------------------------------------
*/
BOOLEAN
GpatmisRfc1483Initialize(
    PATM_USB_INSTANCE       Instance,
    ATM_PHY_BIND            PhyBind,
    ATM_PHY_DEVICE_HANDLE   PhyDeviceHandle

);

void
GpatmisRfc1483Shutdown( PATM_USB_INSTANCE Instance);



#define DBG_PCI_SYS         0x00010000L
#define DBG_DSP_SYS         0x00020000L
#define DBG_EEPROM_SYS      0x00040000L
#define DBG_NDIS_SYS        0x00080000L
#define DBG_PHY_SYS         0x00100000L
#define DBG_LAN_SYS         0x00200000L
#define DBG_WAN_SYS         0x00400000L
#define DBG_ALL_ON          0xFFFFFFFFL
#define DBG_SYS_ALL         0xFFFFFFFFL

#define GSI_RX_BUFFER_COUNT         50
#define GSI_TX_BUFFER_COUNT         50

#define GSI_RETRAIN_LIMIT           20

#define GSI_XCVR_1      0
#define GSI_NUM_XCVR    1

#define GSI_CHECK_TRAIN_TIMEOUT         2  /*(adsl_get_hz())*/
#define GSI_AUTO_MODE_TIMEOUT           10  /*(10 * adsl_get_hz())*/
#define GSI_AUTO_MODE_MONITOR_TIMEOUT   120 /*(120 * adsl_get_hz())*/
#define GSI_RETRAIN_TIMEOUT             60  /*((60 * adsl_get_hz())*/

#define GSI_CONSEC_OVERALL_FAILURE	4	

/*PG 2/9/01*/
/* Dying Gasp*/
#define NUM_DYING_GASP_MESSAGES 6
#define DYING_GASP_EOC_MESSAGE (0xe7);
#define EOC_DYING_GASP_DELAY 1000000 


#define GSI_MAC_ADDRESS_LENGTH      6


#define RFC1483_DEFAULT_VCI         100
#define RFC1483_DEFAULT_VPI         0

#define RFC1483_DEFAULT_MODE        RFC1483_MODE_BRIDGED_ETH_LLC


#define RFC1483_MAX_MULTICAST_ADDRESSES 32

#define E164_ADDR                   0x8
#define NSAP_ADDR                   0x3
#define MAX_ATMADDR_LEN             20   /* Maximum ATM address len */

#define ATM_ADDRESS_13_BYTES_STRING         "1234567890123"
#define ATM_ADDRESS_13_BYTES_STRING_SIZE    13

#define RFC1577_SPIN_LOCK_COUNT     8
#define RFC1577_TIMER_COUNT         16
#define RFC1577_HANDLE              ULONG

typedef struct _RFC1577_TIMER {
    BOOLEAN                 busy;
    /*NDIS_MINIPORT_TIMER     NdisTimer;*/
    PVOID     		    NdisTimer;
    PVOID		    TimerFunction;
} RFC1577_TIMER, *PRFC1577_TIMER;

typedef NDIS_STATUS(*RFC1577RFC1483SEND)(
    PATM_USB_INSTANCE, /*NC PNDIS_PACKET*/ PVOID, ATM_UNI_HANDLE, BOOLEAN);

typedef void (*RFC1577RFC1483RECEIVECOMPLETE)(PATM_USB_INSTANCE,
    PXFER_DATA_BUFFER );

typedef void (*RFC1483RFC1577SENDCOMPLETE)(PATM_USB_INSTANCE,
    ATM_UNI_USER_HANDLE, /*PNDIS_PACKET*/PVOID, BOOLEAN);

typedef void (*RFC1483RFC1577RECEIVE)(PVOID, ATM_UNI_USER_HANDLE,
    PXFER_DATA_BUFFER, PUCHAR, ULONG, BOOLEAN);

typedef int (*RFC1577CLOSE)(PVOID);

#define RFC2364_DEFAULT_VCI         100
#define RFC2364_DEFAULT_VPI         100

#define RFC2364_DEFAULT_MODE        RFC2364_MODE_VC
#define RFC2364_LLC_ENCAP_SIZE      4
#define RFC2364_VC_ENCAP_SIZE       0

#define GSI_WAN_TTY_NAME            "ttyWP"
#define GSI_WAN_CU_NAME             "cug"

typedef struct CONFIG {
    char ATMAddress[MAX_ATMADDR_LEN];
    char ATMAddrType;
    char ATMSubAddress[MAX_ATMADDR_LEN];
    char ATMSubAddrType;

    unsigned char   DefaultVPI;
    unsigned short  DefaultVCI;
    RFC_MODE        DefaultEncap;
    unsigned int    DefaultIPAddr;
    ATM_UNI_HANDLE  DefaultHandle;

    RFC1577RFC1483SEND            Rfc1577Rfc1483Send; 		/* gs */
    RFC1577RFC1483RECEIVECOMPLETE Rfc1577Rfc1483ReceiveComplete;/* gs */
    RFC1483RFC1577SENDCOMPLETE    Rfc1483Rfc1577SendComplete;   /* fa */
    RFC1483RFC1577RECEIVE         Rfc1483Rfc1577Receive;        /* fa */
    RFC1577CLOSE                  GpatmisRfc1577Close;          /* fa */

     /*
     * Following will be filled by Rfc1577 and returned
     */
    void           *Rfc1577Context;	/* FA */
    BOOLEAN         Rfc1577Enable;   	/* gs */
    ULONG           Rfc1577MTUSize;

} Rfc1577Config_t;

typedef struct  Rfc1577Status {
    unsigned int    IntfAddr;
    unsigned int    TgtIPAddr;
    UCHAR           VPI;
    USHORT          VCI;
    int             State;
    int             NoOfTries;
    int             TTL;
} Rfc1577Status_t;

#define GSI_LAN_BUFFER_TAG          0xBABE0001
#define GSI_LAN_NUM_NDIS_PACKETS    8
#define GSI_LAN_NDIS_BUFFER_SIZE    1536

#define _TX_FULL        (0x01 << 0)
#define _NO_CARRIER     (0x01 << 1)

typedef struct {
    void*			dev;
    /*void*		        stats;*/
} gsi_lan_t;

#define GSI_WAN_BUFFER_TAG          0xBABE0002
#define GSI_WAN_NUM_NDIS_PACKETS    8
#define GSI_WAN_NDIS_BUFFER_SIZE    1536
#define GSI_WAN_MAX_CHANNELS        1
#define GSI_WAN_DEF_CHANNEL         0

typedef struct {
    struct _adapter_t  *pAdapter;
    int                 count;
    unsigned long       flags;
    void * 	        slock;
    void *	        tqueue;
    void *              normal_termios;
    void * 		callout_termios;

    void *		tty;
    unsigned long       event;

    u_int8_t            msr;
    u_int8_t            mcr;
    u_int8_t            lsr;

    int                 txAvailable;
    u_int8_t            txHead;
    u_int32_t           txBytes;
    PVOID	        txPacket;
    u_int32_t           txPacketLen;

    u_int8_t            rxData[GSI_WAN_NDIS_BUFFER_SIZE];
    u_int8_t            rxFlags[GSI_WAN_NDIS_BUFFER_SIZE];
} channel_t;

#define _TTY_ALLOC      (0x01 << 0)
#define _CU_ALLOC       (0x01 << 1)
#define _TX_HALTED      (0x01 << 2)
#define _RX_HALTED      (0x01 << 3)

typedef struct {
    u_int8_t            flags;
    int                 refcount;
    void *		cu;
    void *	        table[GSI_WAN_MAX_CHANNELS]; 
    void *	        termios[GSI_WAN_MAX_CHANNELS];	    
    void *              termios_locked[GSI_WAN_MAX_CHANNELS];
    unsigned char 	tty_minor;
    channel_t           channel[GSI_WAN_MAX_CHANNELS];
} gsi_wan_t;

typedef enum {
    DspStateNotTrained,
    DspStateTraining,
    DspStateTrained,
    DspStatePendingCellDel,
    DspStateDisconnecting
} dsp_state_t;

#define _MEM_RESERVE        (0x01 << 0)
#define _MEM_ALLOC          (0x01 << 1)
#define _IRQ_ALLOC          (0x01 << 2)
#define _LAN_ALLOC          (0x01 << 3)
#define _WAN_ALLOC          (0x01 << 4)
#define _ATMUNI_ALLOC       (0x01 << 5)
#define _RFC1483_ALLOC      (0x01 << 6)
#define _RFC1577_ALLOC      (0x01 << 7)
#define _RFC2364_ALLOC      (0x01 << 8)
#define _RFC2364_VC_ALLOC   (0x01 << 9)
#define _ADAPTER_RESTART    (0x01 << 10)

#define MAX_TX_COLLISIONS   5

typedef struct _gsi_struct{
	PVOID check_train;
}gsi_struct;

#define ADSL_AUTO_VCI_VPI_KNOWN		0x01
#define ADSL_AUTO_VCI_VPI_SEARCH	0x02
#define ADSL_AUTO_VCI_VPI_STOP		0x03

typedef struct _adapter_t {
    void*		custstruct;
    char*		devname;
    u_int8_t            id;
    u_int16_t           flags;

    void*	        virtAddr;
    u_int32_t		interruptStatus;
    dsp_state_t         dspState;

/*    NDIS_MINIPORT_TIMER check_train; */

    u_int8_t            macAddr[GSI_MAC_ADDRESS_LENGTH];
    u_int8_t            remoteMacAddr[GSI_MAC_ADDRESS_LENGTH];

    PCI_PHY             pciPhy;

    NDIS_HANDLE         MiniportAdapterHandle;
    ATM_UNI_HANDLE      AtmUniHandle;
    UCHAR               UniqueId[UNIQUE_ID_SIZE];

    ATM_PHY_LINK_STATE  LinkState;

    ULONG               UpstreamRate;
    ULONG               DownstreamRate;
    USHORT              TxBufferCount;
    USHORT              RxBufferCount;

    RFC_MODE        	EncapMode;
    UCHAR               EncapsulationSize;
    UCHAR               Vpi;
    USHORT              Vci;
    ATM_UNI_HANDLE      VcHandle;
    ULONG               PacketFilter;
    UCHAR               MulticastFilter[
                            RFC1483_MAX_MULTICAST_ADDRESSES *
                            ETH_LENGTH_OF_ADDRESS];
    USHORT              MulticastFilterSize;

    gsi_lan_t           lan;

    PUCHAR              Encapsulation;

    gsi_wan_t           wan;

    gsi_version_t       version;
    gsi_config_t        config; 
    PGSIDSL_PANDA_EEPROM  pEeprom;

    UINT                EocStatusTimer;
    int			LinkLedCounter;	/* Used for controlling the Link Status LED*/
    int			SDLedCounter;
    int			RDLedCounter;
    int			LedTestCounter;

 /*   DBG_FIELDS*/

    u_int32_t           retrainCount;
    struct _adapter_t  	*next;

    /*------- NENADS STUFF-----------*/
    unsigned char	atm_autocfg;
    unsigned char	atm_autocfg_running;
    void*	        auto_vcivpi;
    UCHAR               auto_vpi;
    USHORT              auto_vci;
    unsigned int	auto_vcivpi_status;
    unsigned int	auto_vcivpi_index;
    void*		tty_skb;
    void*		rx_skb;
    void* 		card;
    void*	        check_train;
    void*		tq_atm_task;
    void*		trace_info;
    unsigned char 	tq_atm_task_running;
    unsigned int	atm_pkt_state;
    unsigned int 	max_sdu_size;
    unsigned char	tx_collisions;

    unsigned char 	atm_header[4];
    unsigned char 	prot_header[10];
    unsigned char 	prot_header_size;
    unsigned char 	trailer[8];
    unsigned char 	pad_len_buf[51];

    unsigned char	atm_addr[20];
    atm_stats_t		atm_stats;
    unsigned long	con_timeout;
    unsigned char	arp_timeout;

    unsigned char	atm_timeout;
    unsigned char	atm_check;
    char       		atm_watchdog;
} adapter_t;

#if DBG
extern const char *DbgIrqStatus(u_int32_t irqStatus);
extern const char *DbgHwAddr(u_int8_t *hwAddr);

#define DBG_TX(A,S)     {if (_DBG_SYS(A,DBG_TRACE_ON)){DBG_PRINT(A,S);}}
#define DBG_RX(A,S)     {if (_DBG_SYS(A,DBG_TRACE_ON)){DBG_PRINT(A,S);}}
#define DBG_PCI(A,S)    {if (_DBG_SYS(A,DBG_TRACE_ON)){DBG_PRINT(A,S);}}
#define DBG_DSP(A,S)    {if (_DBG_SYS(A,DBG_TRACE_ON)){DBG_PRINT(A,S);}}
#define DBG_IOCTL(A,S)  {if (_DBG_SYS(A,DBG_TRACE_ON)){DBG_PRINT(A,S);}}
#else /* DBG */
#define DBG_TX(A,S)
#define DBG_RX(A,S)
#define DBG_PCI(A,S)
#define DBG_DSP(A,S)
#define DBG_IOCTL(A,S)
#endif /* DBG */



#if (defined(GSI_LAN) || defined(GSI_WAN))
void GpLinkStatusChange(PATM_USB_INSTANCE instance,
    ATM_PHY_LINK_STATE linkState, ULONG upstreamRate, ULONG downstreamRate);
#endif /* (GSI_LAN && GSI_WAN) */

#if defined(GSI_LAN)
int GpLanAlloc(adapter_t *pAdapter);
int GpLanRegister(adapter_t *pAdapter);
void GpLanUnregister(adapter_t *pAdapter);
void GpLanFree(adapter_t *pAdapter);
void GpLanTxComplete(adapter_t *pAdapter, int length, int status);
void GpLanMediaChange(adapter_t *pAdapter);
#endif

#if defined(GSI_WAN)
int GpWanAlloc(adapter_t *pAdapter);
int GpWanRegister(adapter_t *pAdapter);
void GpWanUnregister(adapter_t *pAdapter);
void GpWanFree(adapter_t *pAdapter);
void GpWanRx(adapter_t *pAdapter, channel_t *pChan,
    void *pPacket, u_int32_t packetSize);
void GpWanTxComplete(adapter_t *pAdapter, channel_t *pChan, int status);
void GpWanMediaChange(adapter_t *pAdapter);
#endif

#include "drvh/gsi_pci.h"
/* DBG_STATIC u_int16_t GpInterruptStatus(adapter_t  *pAdapter); */

GP_VOID CustReceiveCells(PDSL_MODEM pDSLModem, GP_DWORD CellPtr, GP_WORD NumRxCells);
GP_VOID CustDelay(GP_DWORD delay );
GP_VOID CustXmitDmaDone(PDSL_MODEM pDSLModem );

GP_DWORD CustReadPciReg(
	    PDSL_MODEM pDslModem,
	    GP_WORD Address,
	    GP_BYTE *pData
	    );

GP_VOID CustWritePciReg(
	       PDSL_MODEM pDslModem,
	       GP_WORD Address,
	       GP_DWORD Data
	       );

GP_BYTE CustReadPciRegB(
	    PDSL_MODEM pDslModem,
	    GP_WORD Address,
	    GP_BYTE *pData
	    );

GP_VOID CustWritePciRegB(
	       PDSL_MODEM pDslModem, 
	       GP_WORD Address, 
	       GP_BYTE Data
	       );

typedef struct pkt_info
{
	unsigned int length;
	void *chan;

}pkt_info_t;

void GpatmisStateDown(adapter_t *pAdapter);


typedef struct atmarp {
	unsigned short arphrd 	; /* 16 bits  Hardware type */
        unsigned short arppro 	; /* 16 bits  Protocol type */
        unsigned char  arpshtl 	; /* 8 bits  Type & length of source ATM number (q) */
        unsigned char  arpsstl 	; /* 8 bits  Type & length of source ATM subaddress (r) */
        unsigned short arpop   	; /* 16 bits  Operation code (request, reply, or NAK) */
        unsigned char  arpspln 	; /* 8 bits  Length of source protocol address (s=4) */
        unsigned char  arpthtl 	; /* 8 bits  Type & length of target ATM number (x) */
        unsigned char  arptstl 	; /* 8 bits  Type & length of target ATM subaddress (y) */
        unsigned char  arptpln 	; /* 8 bits  Length of target protocol address (z=4) */
}atmarp_t;

typedef	struct arphdr {
	unsigned short	ar_hrd;		/* format of hardware address	*/
	unsigned short	ar_pro;		/* format of protocol address	*/
	unsigned char	ar_hln;		/* length of hardware address	*/
	unsigned char	ar_pln;		/* length of protocol address	*/
	unsigned short	ar_op;		/* ARP opcode (command)		*/
}arphdr_t;

#define ATMARP_HEADER  0x1300
#define ATMARP_PROT_IP 0x0008

#define ARP_REQUEST   0x0100
#define ARP_REPLY     0x0200
#define InARP_REQUEST 0x0800
#define InARP_REPLY   0x0900
#define ARP_NAK       0x1000


/*
********************************************************************************
** 				Sangoma Macro Definitions			    	
********************************************************************************
*/
#define GP_NONE_LED	0x00000000
#define GP_POWER_LED	0x00000001	/* top light */
#define GP_RXACT_LED	0x00000002
#define GP_TXACT_LED	0x00000004
#define GP_LINKACT_LED	0x00000008	/* bottom light */

#define GP_SET_LED_STATUS	GpWritePciReg(pDslModem, PCORE_ADDR_GPIOOUTPUT1, pDslModem->GpioState)
#define GP_TRAINING_LED		pDslModem->GpioState = pDslModem->GpioState << 0x01;  pDslModem->GpioState = (pDslModem->GpioState > GP_LINKACT_LED)?GP_POWER_LED:pDslModem->GpioState; GP_SET_LED_STATUS
#define GP_POWER_LED_ON		pDslModem->GpioState = GP_POWER_LED;  GP_SET_LED_STATUS
#define GP_POWER_LED_OFF	pDslModem->GpioState = GP_NONE_LED;    GP_SET_LED_STATUS
#define GP_SET_RXACT_LED	pDslModem->GpioState=(pDslModem->GpioState&GP_RXACT_LED)?pDslModem->GpioState&~GP_RXACT_LED:pDslModem->GpioState|GP_RXACT_LED; GP_SET_LED_STATUS	
#define GP_SET_TXACT_LED	pDslModem->GpioState=(pDslModem->GpioState&GP_TXACT_LED)?pDslModem->GpioState&~GP_TXACT_LED:pDslModem->GpioState|GP_TXACT_LED; GP_SET_LED_STATUS	
#define GP_LINKACT_LED_ON	pDslModem->GpioState = (GP_LINKACT_LED|GP_POWER_LED);  GP_SET_LED_STATUS
#define GP_LINKACT_LED_OFF	pDslModem->GpioState &= ~GP_LINKACT_LED; GP_SET_LED_STATUS

#define GP_DISCONNECTING_LED    pDslModem->GpioState = pDslModem->GpioState==GP_POWER_LED; GP_SET_LED_STATUS
#define GP_CELL_DEL_LED         pDslModem->GpioState = pDslModem->GpioState==GP_LINKACT_LED;  GP_SET_LED_STATUS


#define ADSL_ASSERT(val) if (val){								\
			wpabs_debug_event("**************** ASSERT FAILED ****************\n");	\
			wpabs_debug_event("%s:%d - Critical error\n",__FILE__,__LINE__);		\
			return -EINVAL;								\
			}

#define ADSL_ASSERT1(val) if (val){								\
			wpabs_debug_event("**************** ASSERT FAILED ****************\n");	\
			wpabs_debug_event("%s:%d - Critical error\n",__FILE__,__LINE__);		\
			return;									\
			}

#define ADSL_ASSERT2(val, ret) if (val){								\
			wpabs_debug_event("**************** ASSERT FAILED ****************\n");	\
			wpabs_debug_event("%s:%d - Critical error\n",__FILE__,__LINE__);		\
			return ret;								\
			}

#define ADSL_MEM_ASSERT(str) {if (str){ \
                          	wpabs_debug_event("%s: Error: No memory in %s:%i\n", \
						str,__FILE__,__LINE__); \
			  }else{ \
				wpabs_debug_event("wanpipe: Error: No memory in %s:%i\n", \
						__FILE__,__LINE__); \
			  }

#if 0
#define ADSL_FUNC_DEBUG() wpabs_debug_event("%s: %s: %d\n", \
						__FILE__,__FUNCTION__,__LINE__)
#else
#define ADSL_FUNC_DEBUG()
#endif
	
extern void adsl_auto_vci_vpi(unsigned long);
#if defined(__OpenBSD__)
extern int		ticks;
#endif


static __inline void adsl_card_lock_irq(adapter_t *pAdapter, unsigned long *flags)
{
	wpabs_card_lock_irq(pAdapter->card,flags);
}

static __inline void adsl_card_unlock_irq(adapter_t *pAdapter, unsigned long *flags)
{
	wpabs_card_unlock_irq(pAdapter->card,flags);
}


#define DBG_FUNCX(a,b) 
#define DBG_FUNC(a)
#define DBG_ENTER(a)
#define DBG_PARAMF(a,b,c)
#define DBG_PRINT(a,b)
#define DBG_RETURNF(a,b)
#define DBG_LEAVE(a)
#define DBG_STATIC 
#define DBG_ASSERT(a)
#define DBG_WARNING(a,b)
#define DBG_ERROR(a,b)

#endif /* GSI_H_ */

