/*+M*************************************************************************
 * TriplePoint NDIS Abstraction Layer
 *
 * Copyright (c) 1997-2000 TriplePoint, Inc. -- http://www.TriplePoint.com
 * All rights reserved.
 *-M*************************************************************************/

#include "Gp.h"	

#include "drvh/GpSnull.h"

void HexDump1(unsigned char *pBuffer, unsigned long nSize, int pkt_cnt);
#if defined(linux)
void GpTimerFunc(unsigned long arg);
#else	/* FreeBSD or OpenBSD */
void GpTimerFunc(void* arg);
#endif

void HexDump1(unsigned char *pBuffer, unsigned long nSize, int pkt_cnt)
{
    unsigned int nAddr = (unsigned int)pBuffer;
    const int nMaxBPR = 16;                     /* Max byes per row of dump */
    int nBPR;
    unsigned long nCount = 0;

    if (pBuffer[0] == 0x00 &&
        pBuffer[1] == 0x00 && 
        pBuffer[2] == 0x00 &&
	pBuffer[3] == 0x01){
    
    	return;
    }
    wpabs_debug_rx(" %d -------------------------\n",pkt_cnt);
    while (nCount < nSize)
    {
        nBPR = 0;
        wpabs_debug_rx("%x: ", nAddr);
        while ((nCount < nSize) && (nBPR < nMaxBPR))
        {
            wpabs_debug_rx("%02x ",(unsigned char)*(pBuffer+nCount));
            nCount++;
            nBPR++;
        }
        nAddr += nMaxBPR;
        wpabs_debug_rx("\n");
    }
    return ;
}

#define RUN_AT(x) (jiffies + (x))

GP_VOID CustDelay(GP_DWORD delay )
{
	NdisStallExecution(delay);
	return;
}	       


VOID NdisStallExecution(IN ULONG Microsecs)
{
	wpabs_udelay(Microsecs);
}


GP_VOID CustXmitDmaDone(PDSL_MODEM pDSLModem) 
{ 
	adapter_t* pAdapter;

	pAdapter = (adapter_t*)pDSLModem->pCustData;

        gsi_dma_tx_complete(pAdapter);

        gsi_phy_send_complete(&(pAdapter->pciPhy));
	return;
}

GP_VOID CustReceiveCells(PDSL_MODEM pDSLModem, GP_DWORD CellPtr,GP_WORD NumRxCells ) 
{ 
	adapter_t* pAdapter;
	PCI_PHY    *pPciPhy;
	PPHY_DESCR  rxDescr;
	int i;

	pAdapter = (adapter_t*)pDSLModem->pCustData;
	pPciPhy = &(pAdapter->pciPhy);

	wpabs_debug_rx("************* CellPtr = %lx ***********\n",CellPtr);
	wpabs_debug_rx("********** NumRxCells = %d ***********\n",NumRxCells);
	wpabs_debug_rx("\n");
	wpabs_debug_rx("*********************************************\n");
	wpabs_debug_rx("RX: CELLS %d\n",NumRxCells);
	for (i=0;i<NumRxCells;i++){
   		HexDump1((unsigned char*)CellPtr+(52*i), 52,(i+1));
	}
	wpabs_debug_rx("\n");
	wpabs_debug_rx("*********************************************\n");

	rxDescr = (PPHY_DESCR) PHY_REMOVE_LIST(&(pPciPhy->RxFreeList));

        if (rxDescr != NULL){
		
        	rxDescr->Buffer->Data = (void *)CellPtr; /*bus_to_virt(pPciPhy->RxDataHead);*/
               	rxDescr->Buffer->Size = NumRxCells*52;
#if 0
ALEX
        	rxDescr->Buffer->DmaData.vAddr = (void *)CellPtr; /*bus_to_virt(pPciPhy->RxDataHead);*/
               	rxDescr->Buffer->DmaData.length = NumRxCells*52;
#endif

               	PHY_ADD_LIST(&(pPciPhy->RxCompleteList), rxDescr);
        }

        gsi_phy_receive_complete(&(pAdapter->pciPhy));
	return;
}




/*+F*************************************************************************
 * Function:
 *   NdisMIndicateStatus
 *
 * Description:
 *   Notify the LAN/WAN subsystems that the link state has changed.
 *-F*************************************************************************/
void
NdisMIndicateStatus(
    NDIS_HANDLE     ndisHandle,
    NDIS_STATUS     linkStatus,
    PVOID           statusBuffer,
    UINT            statusBufferSize
    )
{
    DBG_FUNCX("NdisMIndicateStatus", DBG_NDIS_SYS)
    adapter_t  *pAdapter = (adapter_t *) ndisHandle;
    
    DBG_ENTER(DbgInfo);

    switch (linkStatus)
    {
    case NDIS_STATUS_MEDIA_DISCONNECT:
        DBG_PRINT(DbgInfo, ("**************************\n"));
        DBG_PRINT(DbgInfo, ("**** Media Disconnect ****\n"));
        DBG_PRINT(DbgInfo, ("**************************\n"));
        wpabs_debug_event("%s: ADSL Link disconnected\n", pAdapter->devname);
        break;

    case NDIS_STATUS_MEDIA_CONNECT:
        DBG_PRINT(DbgInfo, ("*************************************\n"));
        DBG_PRINT(DbgInfo, ("**** Media Connect\n"));
        DBG_PRINT(DbgInfo, ("****   UpRate:%ld  DownRate:%ld\n",
            pAdapter->UpstreamRate, pAdapter->DownstreamRate));
        DBG_PRINT(DbgInfo, ("*************************************\n"));
        wpabs_debug_event("%s: ADSL Link connected (Down %ld kbps, Up %ld kbps)\n",
            pAdapter->devname, pAdapter->DownstreamRate, pAdapter->UpstreamRate);
        break;

    default:
        DBG_ERROR(DbgInfo, ("Unhandled indicate status 0x%lx\n", linkStatus));
        break;
    }


    if (pAdapter != NULL && pAdapter->config.interface == LAN_INTERFACE){
	ADSL_FUNC_DEBUG();
        GpLanMediaChange(pAdapter);
    }

#ifdef GSI_WAN
    if (pAdapter != NULL && pAdapter->config.interface == WAN_INTERFACE){
	ADSL_FUNC_DEBUG();
        GpWanMediaChange(pAdapter);
    }
#endif

    DBG_LEAVE(DbgInfo);
}

/*+F*************************************************************************
 * Function:
 *   NdisMSendComplete
 *
 * Description:
 *   The transmit is complete. Notify the LAN/WAN subsystems so the buffers
 *   can be freed.
 *-F*************************************************************************/
void
NdisMSendComplete(
    NDIS_HANDLE     ndisHandle,
    PVOID           pktinfo,
    NDIS_STATUS     status
    )
{
    	DBG_FUNCX("NdisMSendComplete", DBG_TX_SYS)
    	adapter_t  *pAdapter = (adapter_t *) ndisHandle;
    	pkt_info_t *pkt_info = (pkt_info_t*)pktinfo;

	ADSL_FUNC_DEBUG();

    	if (!pkt_info){
    		return;
	}
    
    	GpLanTxComplete(pAdapter, pkt_info->length, (status == NDIS_STATUS_SUCCESS) ? 0 : -1);

    	DBG_LEAVE(DbgInfo);
}

#ifdef GSI_LAN
/*+F*************************************************************************
 * Function:
 *   NdisMEthIndicateReceive
 *
 * Description:
 *   Notify that LAN subsystem that a new receive buffer is available.
 *-F*************************************************************************/
void
NdisMEthIndicateReceive(
    NDIS_HANDLE     ndisHandle,
    NDIS_HANDLE     ndisReceiveContext,
    PVOID           pHeader,
    UINT            headerSize,
    PVOID           pLookAhead,
    UINT            lookAheadSize,
    UINT            packetSize
    )
{
	DBG_FUNCX("NdisMEthIndicateReceive", DBG_RX_SYS)
    	adapter_t*	pAdapter = (adapter_t *) ndisHandle;
	gsi_lan_t*	pLan = (gsi_lan_t*)&(pAdapter->lan);

	DBG_ENTER(DbgInfo);
    	DBG_ASSERT(packetSize == lookAheadSize);

	ADSL_ASSERT1((!pLan->dev));

    	adsl_lan_rx(pLan->dev, pHeader, headerSize, pLookAhead, lookAheadSize);

    	DBG_LEAVE(DbgInfo);
}
#endif /* GSI_LAN */

#ifdef GSI_WAN
/*+F*************************************************************************
 * Function:
 *LANNdisMWanIndicateReceive
 *
 * Description:
 *   Notify that WAN subsystem that a new receive buffer is available.
 *-F*************************************************************************/
void
NdisMWanIndicateReceive(
    PNDIS_STATUS    pStatus,
    NDIS_HANDLE     ndisHandle,
    NDIS_HANDLE     ndisLinkContext,
    PUCHAR          pPacket,
    ULONG           packetSize
    )
{
    DBG_FUNCX("NdisMWanIndicateReceive", DBG_RX_SYS)
    adapter_t  *pAdapter = (adapter_t *) ndisHandle;
    channel_t  *pChan = (channel_t *) ndisLinkContext;

    DBG_ENTER(DbgInfo);

    DBG_PRINT(DbgInfo, ("Apapter Handle %4x\n", pAdapter));
    DBG_PRINT(DbgInfo, ("NdisLink Context %4x\n", pChan));


    GpWanRx(pAdapter, pChan, pPacket, packetSize);

    DBG_LEAVE(DbgInfo);
}
#endif /* GSI_WAN */

