/*+M*************************************************************************
 * GlobeSpan ADSL LAN Interface
 *
 * Copyright (c) 2000 GlobeSpan Semiconductor Inc. -- http://www.globespan.net
 * All rights reserved.
 *
 *-M*************************************************************************/
#include "Gp.h"
#include <drvh/gsi_pci.h>
#include <drvh/gsi_ioctl.h>

#define IFP2AC(IFP) ((struct arpcom *)IFP)

extern int TMP_GLOBAL_VAR;

void GpLanMediaInit(adapter_t*);
/*
void adsl_lan_multicast(void* adapter, short if_flags);
extern void adsl_tx_complete(void* dev, int length, int txStatus);
*/
#if 0
static unsigned char tmp_frame[]= 
{0xff,0xff,0xff,0xff,0xff,0xff,0x00,0x00,0x00,0x44,
0x56,0x24,0x08,0x06,0x00,0x01,0x08,0x00,0x06,0x04,
0x00,0x01,0x00,0x00,0x00,0x44,0x56,0x24,0xc9,0x01,
0x01,0x03,0x00,0x00,0x00,0x00,0x00,0x00,0xc9,0x01,
0x01,0x01};
#endif

/*+F*************************************************************************
 * Function:
 *   GpLanTxComplete
 *
 * Description:
 *   The driver/adapter have finished transmitting the packet. Update the
 *   LAN statistics, free the buffers, and tell the operating system that
 *   we can handle another buffer.
 *-F*************************************************************************/
void
GpLanTxComplete(adapter_t *pAdapter, int length, int txStatus)
{
    	DBG_FUNCX("GpLanTxComplete", DBG_LAN_SYS)
    	gsi_lan_t      *pLan = &(pAdapter->lan);

	ADSL_FUNC_DEBUG();

	if (!pLan->dev){
		wpabs_debug_event("%s: GpLanTxComplete no dev!\n",
				pAdapter->devname);
		return;
	}
#if 0
NOTE: Alex May 5, 2004
Check not needed. If_send would fail!
	if (!wpabs_netif_dev_up(pLan->dev)){
		wpabs_debug_event("%s: Network device is not UP!\n",
				pAdapter->devname);
		return;
	}
#endif	
    	DBG_ENTER(DbgInfo);
    	DBG_PARAMF(DbgInfo, "pAdapter", ("%d (0x%p)", DBG_ID(pAdapter), pAdapter));
    	DBG_PARAMF(DbgInfo, "txStatus", ("%d", txStatus));

	adsl_tx_complete(pLan->dev, length, txStatus);

       	/* Tell the upper layers that we are ready for another packet.*/
       	if (wpabs_netif_queue_stopped(pLan->dev)){

		ADSL_FUNC_DEBUG();
		
		wpabs_debug_tx("%s: TxLan Complete, waking dev...\n",
				pAdapter->devname);
       		wpabs_netif_wake_queue(pLan->dev);
		pAdapter->tx_collisions=0;
	}
    	DBG_LEAVE(DbgInfo);
}

void GpLanMediaInit(adapter_t *pAdapter)
{
	return;
}


/*+F*************************************************************************
 * Function:
 *   GpLanMediaChange
 *
 * Description:
 *   The adapter is notifying the LAN subsystem that the link state has
 *   changed. What we care about here is if the link is now available, and
 *   there are buffers ready, we need to tell the operating system that we
 *   are ready to handle more transmit requests.
 *-F*************************************************************************/
void
GpLanMediaChange(adapter_t *pAdapter)
{
    	gsi_lan_t *pLan = &(pAdapter->lan);

	ADSL_FUNC_DEBUG();

    	if (!pLan || !pLan->dev){
		wpabs_debug_event("%s: GpLanMediaChange no dev!\n",
				pAdapter->devname);
		return;
	}
    	if (!wpabs_netif_dev_up(pLan->dev)){
		wpabs_debug_event("%s: Network device is not UP!\n",
				pAdapter->devname);
		return;
	}

	if (pAdapter->dspState != DspStateTrained){
		ADSL_FUNC_DEBUG();
		GpatmisStateDown(pAdapter);	
	}
	
        if (wpabs_netif_queue_stopped(pLan->dev) && 
	    (pAdapter->dspState == DspStateTrained)){

		ADSL_FUNC_DEBUG();

	    	wpabs_debug_tx("%s: LanMediaChange : Waking Dev...\n",
			    		pAdapter->devname);
	    	
		wpabs_netif_wake_queue(pLan->dev);
    	}
}

/*+F*************************************************************************
 * Function:
 *   GpLanIoctl
 *
 * Description:
 *   LAN interface IOCTL entry point.
 *-F*************************************************************************/
#if 0
FIXME_ADSL
int GpLanIoctl(void* dev, struct ifreq* rq, int cmd)
{
    DBG_FUNC("GpLanIoctl")
    adapter_t  *pAdapter;
    int         status = 0;
    dsl_private_area_t *dsl = dev->priv;

    if (!chan){
	return -EOPNOTSUPP;
    }
    
    DBG_ENTER(DbgInfo);
    DBG_PARAMF(DbgInfo, "dev", ("%s (0x%p)", dev->name, dev));
    DBG_PARAMF(DbgInfo, "rq", ("(0x%p)", rq));
    DBG_PARAMF(DbgInfo, "cmd", ("%s (0x%04x)", DbgIoctlCmd(cmd), cmd));

    if ((pAdapter = (adapter_t *) chan->pAdapter) != NULL)
    {
        struct gsi_lan_req *pLanReq = (struct gsi_lan_req *) rq;

        if (!GpatmisIoctl(pAdapter, cmd, &(pLanReq->request), &status))
        {
            switch (cmd)
            {
            default:
                DBG_WARNING(DbgInfo, ("Unhandled IOCTL command: %s (0x%x)\n",
                    DbgIoctlCmd(cmd), cmd));
                status = -EOPNOTSUPP;
                break;
            }
        }
    }
    else
    {
        DBG_WARNING(DbgInfo, ("No adapter for IOCTL command!\n"));
        status = -ENODEV;
    }

    DBG_RETURNF(DbgInfo, ("status %d", status));
    return status;
}
#endif

#if 0
/*+F*************************************************************************
 * Function:
 *   GpLanMulticast
 *
 * Description:
 *   Handle the LAN subsystem multicast, broadcast, and promiscous updates.
 *   These values can be modified by using the ifconfig command or other
 *   applications like network sniffers.
 *-F*************************************************************************/
void adsl_lan_multicast(void* adapter, short if_flags, char* mcaddrs, int mcount)
{
    	DBG_FUNCX("GpLanMulticast", DBG_LAN_SYS)
    	adapter_t  *pAdapter;

   	DBG_ENTER(DbgInfo);

	if ((pAdapter = (adapter_t *) adapter) != NULL){
        	pAdapter->Rfc1483PacketFilter |= NDIS_PACKET_TYPE_DIRECTED;

		if (if_flags & IFF_BROADCAST){
            		pAdapter->Rfc1483PacketFilter |= NDIS_PACKET_TYPE_BROADCAST;
        	}else{
            		pAdapter->Rfc1483PacketFilter &= ~NDIS_PACKET_TYPE_BROADCAST;
        	}

        	if (if_flags & IFF_PROMISC){
            		pAdapter->Rfc1483PacketFilter |= NDIS_PACKET_TYPE_PROMISCUOUS;
        	}else{
            		pAdapter->Rfc1483PacketFilter &= ~NDIS_PACKET_TYPE_PROMISCUOUS;
        	}

        	if (if_flags & IFF_ALLMULTI){
            		pAdapter->Rfc1483PacketFilter |= NDIS_PACKET_TYPE_ALL_MULTICAST;
        	}else{
            		pAdapter->Rfc1483PacketFilter &= ~NDIS_PACKET_TYPE_ALL_MULTICAST;
        	}

        	if ((if_flags & IFF_MULTICAST) && (mcount != 0)){
            		int     x;

            		pAdapter->Rfc1483PacketFilter |= NDIS_PACKET_TYPE_MULTICAST;
            		if (mcount > RFC1483_MAX_MULTICAST_ADDRESSES){
                		pAdapter->Rfc1483PacketFilter |= NDIS_PACKET_TYPE_ALL_MULTICAST;
                		mcount = 0;
            		}

            		pAdapter->Rfc1483MulticastFilterSize = 0;
            		for (x = 0; x < mcount;x++){
                		wpabs_memcpy(
                    			&(pAdapter->Rfc1483MulticastFilter[x * ETH_LENGTH_OF_ADDRESS]),
                    			&mcaddrs[x * ETH_LENGTH_OF_ADDRESS], 
					ETH_LENGTH_OF_ADDRESS);
                		pAdapter->Rfc1483MulticastFilterSize += ETH_LENGTH_OF_ADDRESS;
            		}
        	}else{
            		pAdapter->Rfc1483PacketFilter &= ~NDIS_PACKET_TYPE_MULTICAST;
        	}
    	}

    	DBG_LEAVE(DbgInfo);
}

#endif
