/*+M*************************************************************************
 * GlobeSpan ADSL PCI Access Routines
 *
 * Copyright (c) 2000 GlobeSpan Semiconductor Inc. -- http://www.globespan.net
 * All rights reserved.
 *
 *-M*************************************************************************/
#undef  GTI_VERBOSE

#include "Gp.h"	

#include <drvh/gsi_pci.h>

#if !(defined(GSI_LAN) && defined(GSI_WAN))
# ifdef GSI_LAN
/*#include <rfc1483.h>*/
# endif /* GSI_LAN*/
# ifdef GSI_WAN
/*#include <rfc2364.h>*/
# endif /* GSI_WAN*/
#endif	/* (GSI_LAN && GSI_WAN)*/

#include <Epci/GpInclude.h>

#define EXPORT_SYMTAB

#define RUN_AT(x)       (jiffies + (x))

static unsigned char sppp_term[] = {0xC0,0x21,0x05,0x98,0x00,0x07};

void gsi_dsp_datarate(adapter_t *pAdapter, ATM_PHY_LINK_STATE linkState);
void gsi_dsp_reset(adapter_t*);
void gsi_dsp_train(adapter_t*);
void gsi_check_train_handler(unsigned long);
static void GpWpLinkStatusChange(PATM_USB_INSTANCE   pAdapter);
static void adsl_ppp_term_request (adapter_t *pAdapter);


/*+F*************************************************************************
 * Function:
 *   gsi_pci_read
 *
 * Description:
 *   Read an adapter register value. All reads are 32-bit accesses.
 *-F*************************************************************************/
u_int32_t
gsi_pci_read(
    adapter_t  *pAdapter,
    u_int16_t         offset
    )
{
    DBG_FUNCX("gsi_pci_read", DBG_PCI_SYS)
    u_int32_t     data;

    DBG_ENTER(DbgInfo);
    DBG_PARAMF(DbgInfo, "pAdapter", ("%d (0x%p)", DBG_ID(pAdapter), pAdapter));
    DBG_PARAMF(DbgInfo, "offset", ("0x%04x", offset));

    data = wpabs_bus_read_4(pAdapter->card, offset);
/*   udelay(1);*/
     NdisStallExecution(1);

    DBG_RETURNF(DbgInfo, ("data 0x%08x", data));
    return data;
}

/*+F*************************************************************************
 * Function:
 *   gsi_pci_write
 *
 * Description:
 *   Write a 32-bit register value on the adapter.
 *-F*************************************************************************/
void
gsi_pci_write(
    adapter_t  *pAdapter,
    u_int16_t         offset,
    u_int32_t         data
    )
{
    DBG_FUNCX("gsi_pci_write", DBG_PCI_SYS)

    DBG_ENTER(DbgInfo);
    DBG_PARAMF(DbgInfo, "pAdapter", ("%d (0x%p)", DBG_ID(pAdapter), pAdapter));
    DBG_PARAMF(DbgInfo, "offset", ("0x%04x", offset));
    DBG_PARAMF(DbgInfo, "data", ("0x%08x", data));

    wpabs_bus_write_4(pAdapter->card, offset, data);
    NdisStallExecution(1);

    DBG_LEAVE(DbgInfo);
}


#if DBG
/*+F*************************************************************************
 * Function:
 *   DbgIrqStatus
 *
 * Description:
 *   Convert the IRQ status flags to a character string
 *-F*************************************************************************/
const char *
DbgIrqStatus(
    u_int32_t         irqStatus
    )
{
    static char irqBuffer[DBG_BUFFER_LEN];

    struct {
        u_int16_t     mask;
        char   *name;
    } dslIrq[] = {
        { DSLINT_TX_EOB,     "TxEob"      },
        { DSLINT_RESERVED_0, "RESERVED_0" },
        { DSLINT_GPIO_0,     "Gpio0"      },
        { DSLINT_GPIO_1,     "Gpio1"      },
        { DSLINT_RX_EOB,     "RxEob"      },
        { DSLINT_RX_CELLCNT, "RxCellCnt"  },
        { DSLINT_RX_WDTIMER, "RxWdTimer"  },
        { DSLINT_RESERVED_7, "RESERVED_7" },
        { DSLINT_PHY,        "Phy"        },
        { DSLINT_PCI_BUSY,   "PciBusy"    },
        { DSLINT_GPIO_6,     "Gpio6"      },
        { DSLINT_GPIO_7,     "Gpio7"      },
        { DSLINT_GPIO_8,     "Gpio8"      },
        { DSLINT_GPIO_16,    "Gpio16"     },
        { DSLINT_GPIO_17,    "Gpio17"     },
        { DSLINT_GPIO_18,    "Gpio18"     },
    };
    int x;

    *irqBuffer = '\0';
    for (x = 0; x < _NELEM(dslIrq); x++)
    {
        if (irqStatus & dslIrq[x].mask)
        {
            strcat(irqBuffer, dslIrq[x].name);
            strcat(irqBuffer, " ");
        }
    }
    if (strlen(irqBuffer) != 0)
    {
        irqBuffer[strlen(irqBuffer) - 1] = '\0';
    }

    return irqBuffer;
}
#endif /* DBG */

#if DBG
/*+F*************************************************************************
 * Function:
 *   DbgHwAddr
 *
 * Description:
 *   Convert a hardware ethernet address to a character string
 *-F*************************************************************************/
const char *
DbgHwAddr(
    u_int8_t     *hwAddr
    )
{
    static char     buffer[18];

    tpi_sprintf(buffer, "%02X:%02X:%02X:%02X:%02X:%02X",
        hwAddr[0], hwAddr[1], hwAddr[2], hwAddr[3], hwAddr[4], hwAddr[5]);

    return buffer;
}
#endif

/*+F*************************************************************************
 * Function:
 *   gsi_dsp_reset
 *
 * Description:
 *   Perform a software reset of the DSP.
 *-F*************************************************************************/
void
gsi_dsp_reset(adapter_t  *pAdapter)
{
	
	DSL_MODEM	*pDslModem;
	DBG_FUNCX("gsi_dsp_reset", DBG_DSP_SYS)
	unsigned long flags;

	DBG_ENTER(DbgInfo);
	DBG_PARAMF(DbgInfo, "pAdapter", ("%d (0x%p)", DBG_ID(pAdapter), pAdapter));

	pDslModem = pAdapter->custstruct;
	if (!pDslModem || pDslModem->InitStatus & GP_INIT_SHUTDOWN){
		return;
	}

	pAdapter->dspState = DspStateNotTrained;

	{/*wally*/
		PCI_PHY *pPciPhy = &(pAdapter->pciPhy);

	   /* Turn off PANDA transmit and receive*/
	   /* Clear all transmit and receive interrupts*/
		gsi_pci_write(pAdapter, GSI_GLUE_DSLINTENABLE, ALL_ZEROS_MASK );
		gsi_pci_write(pAdapter, GSI_GLUE_DSLINTCLEAR, ALL_ONES_MASK );
		gsi_pci_write(pAdapter, GSI_GLUE_RESETIMAGE, 1 );
		/*udelay(50);*/
		gsi_pci_write(pAdapter, GSI_GLUE_RESETIMAGE, 2 );
		/*udelay(50);*/

		adsl_card_lock_irq(pAdapter,&flags);

		if (pPciPhy && (pPciPhy->TxDescr != NULL)){
			PHY_ADD_LIST(&(pPciPhy->TxFreeList), pPciPhy->TxDescr);
			pPciPhy->TxDescr = NULL;
		}
		adsl_card_unlock_irq(pAdapter,&flags);


	}/*wally*/
       
	/* Update the data rate*/
	gsi_dsp_datarate(pAdapter, ATM_PHY_LINK_STATE_DISCONNECTED);

	/* CardResetStarlet();*/
	/* Cancel the timer that checks the line quality*/
	wpabs_del_timer(pAdapter->check_train);

	GpResetTransceiver(pDslModem);

	DBG_LEAVE(DbgInfo);
}

/*+F*************************************************************************
 * Function:
 *   gsi_dsp_train
 *
 * Description:
 *   Start the DSP training.
 *-F*************************************************************************/
void
gsi_dsp_train(
    adapter_t  *pAdapter
    )
{
	DSL_MODEM	*pDslModem;

    	pDslModem = pAdapter->custstruct;
    	if (!pDslModem || pDslModem->InitStatus & GP_INIT_SHUTDOWN){
		return;
	}

    	if (pAdapter->dspState == DspStateNotTrained){
        
		pAdapter->dspState = DspStateTraining;
		pDslModem->DslLinkState = GP_TRAINING;
		GpCmdTransceiver(pDslModem, GTI_STARTUP_REQ);
    	}

    	DBG_LEAVE(DbgInfo);
}

/*+F*************************************************************************
 * Function:
 *   GpLinkStatusChange
 *
 * Description:
 *   Adapter link status change routine. This routine is called when the
 *   low level DSP detects that the link status has changed.
 *-F*************************************************************************/
void GpLinkStatusChange(PATM_USB_INSTANCE   instance,
    			ATM_PHY_LINK_STATE  linkState,
    			ULONG               upstreamRate,
    			ULONG               downstreamRate)
{
#define pAdapter ((adapter_t*)instance)
    	DBG_FUNC("gsi_link_status_change")
    	NDIS_STATUS ndisStatus;

    	instance->UpstreamRate   = upstreamRate;
    	instance->DownstreamRate = downstreamRate;

	/* If the link state has changed, indicate the new link state to NDIS*/
    	if (linkState != instance->LinkState){
        
		instance->LinkState = linkState;

        	if (linkState == ATM_PHY_LINK_STATE_CONNECTED){
            		ndisStatus = NDIS_STATUS_MEDIA_CONNECT;
		}else{
            		ndisStatus = NDIS_STATUS_MEDIA_DISCONNECT;
			wpabs_set_state(pAdapter->card, WAN_DISCONNECTED);
			wpabs_set_baud(pAdapter->card, 0);

        	}

        	NdisMIndicateStatus(
                           instance->MiniportAdapterHandle,
                           ndisStatus,
                           NULL,
                           0);

		NdisMIndicateStatusComplete(instance->MiniportAdapterHandle);
    	}

    	DBG_LEAVE(DbgInfo);
#undef pAdapter
}

/*+F*************************************************************************
 * Function:
 *   GpWpLinkStatusChange
 *
 * Description:
 *-F*************************************************************************/
static void GpWpLinkStatusChange(PATM_USB_INSTANCE  pAdapter)
{
	unsigned long timeout=0;

        if (pAdapter->dspState == DspStateTrained &&
            wpabs_get_state(pAdapter->card) != WAN_CONNECTED){

		wpabs_set_state(pAdapter->card, WAN_CONNECTED);
		wpabs_set_baud(pAdapter->card,pAdapter->DownstreamRate);

		/* Initialize timers*/
		if (pAdapter->atm_autocfg != 0){
			wpabs_debug_event("%s: Start auto VCI/VPI detection...\n",
					pAdapter->devname);
			wpabs_set_bit(0, &pAdapter->atm_autocfg_running);
			
			
			/* For PPPoE wait longer after connect so that 
			 * the remote PPPoE server starts. Otherwise for
			 * all other protocols, start auto detection ASAP */
			if (pAdapter->EncapMode == RFC_MODE_BRIDGED_ETH_LLC ||
		    	    pAdapter->EncapMode == RFC_MODE_BRIDGED_ETH_LLC){
				timeout=12*wpabs_get_hz();
				wpabs_add_timer(pAdapter->auto_vcivpi, timeout);
			}else{
				timeout=5*wpabs_get_hz();
				wpabs_add_timer(pAdapter->auto_vcivpi, timeout);
			}
		}
        }
} 

/*+F*************************************************************************
 * Function:
 *   gsi_dsp_datarate
 *
 * Description:
 *   Update the DSP link state. This routine also calculates both the
 *   upstream and downstream data rates.
 *-F*************************************************************************/
void
gsi_dsp_datarate(adapter_t *pAdapter, ATM_PHY_LINK_STATE  linkState)
{
	DSL_MODEM *pDslModem;

    	DBG_FUNCX("gsi_dsp_datarate", DBG_DSP_SYS)
	
	if (!pAdapter)
		return;
		
    	pDslModem = pAdapter->custstruct;

    	if (!pDslModem || pDslModem->InitStatus & GP_INIT_SHUTDOWN){
		return;
	}

    	GpGetDataRate(pDslModem);
    	GpLinkStatusChange(pAdapter, linkState, pDslModem->UpstreamRate,pDslModem->DownstreamRate);
}

/*+F*************************************************************************
 * Function:
 *   gsi_dma_tx_start
 *
 * Description:
 *   Give the next outgoing transmit to the hardware.
 *-F*************************************************************************/
void
gsi_dma_tx_start(adapter_t *pAdapter)
{
	DSL_MODEM   *pDslModem;
	PPHY_DESCR  txDescr;

    	DBG_FUNCX("gsi_dma_tx_start", DBG_TX_SYS)
   	PCI_PHY    *pPciPhy = &(pAdapter->pciPhy);

	pDslModem = pAdapter->custstruct;
	if (!pDslModem || pDslModem->InitStatus & GP_INIT_SHUTDOWN){
		return;
	}

	wpabs_debug_tx("(Debug): gsi_dma_tx_start\n");

	DBG_ENTER(DbgInfo);
	DBG_PARAMF(DbgInfo, "pAdapter", ("%d (0x%p)", DBG_ID(pAdapter), pAdapter));

	if (pAdapter->dspState != DspStateTrained || pDslModem->DslLinkState != GP_TRAINED){ 
		/* wally */
		wpabs_debug_event("%s: dma_tx_start: DmaTx: Not trained!\n",pAdapter->devname);
		return;
	}

	if (wpabs_test_and_set_bit(0,&pDslModem->TxDmaPending)){
    	    	wpabs_debug_tx("%s: ERROR gsi_dma_tx_start: Tx DMA Busy!!!\n",pAdapter->devname);
	    	return;
    	}

    	if (pPciPhy->State != PCI_PHY_STATE_OPERATIONAL){
	    	wpabs_debug_event("%s: gsi_dma_tx_start: PciPhy not operational!\n",pAdapter->devname);
		wpabs_clear_bit(0,&pDslModem->TxDmaPending);
	    	return;
    	}

	if ((txDescr=pPciPhy->TxDescr) != NULL){
		wpabs_debug_event("%s: gsi_dma_tx_start: PciPhy not empty but TxDMA is off!\n",
				pAdapter->devname);
		
	    	pPciPhy->TxDescr = NULL;
	    	((pkt_info_t*)txDescr->Buffer->AllocatorContext)->length=0;
	    	((pkt_info_t*)txDescr->Buffer->AllocatorContext)->chan=NULL;
                txDescr->Buffer->Size = 0;
                /* ALEX txDescr->Buffer->DmaData.length = 0; */

	    	PHY_ADD_LIST(&(pPciPhy->TxFreeList), txDescr);
		
		wpabs_clear_bit(0,&pDslModem->TxDmaPending);
		return;
    	}		

	txDescr = (PPHY_DESCR) PHY_REMOVE_LIST(&(pPciPhy->TxPendingList));
        if (txDescr != NULL){
				
		unsigned long    pAddr;
            	u32             length;

            	pPciPhy->TxDescr = txDescr;

            	pAddr = wpabs_dma_get_paddr(pAdapter->card, txDescr->Buffer->Dma);
            	length = txDescr->Buffer->Size;
#if 0
ALEX
            	pAddr = adsl_virt_to_bus((volatile void*)txDescr->Buffer->DmaData.vAddr);
            	length = txDescr->Buffer->DmaData.length;
#endif

            	/*  Enforce hardware constraint */
            	DBG_ASSERT(!(pAddr & 0x03));
            	DBG_ASSERT(!((pAddr + length) & 0x03));

            	DBG_TX(DbgInfo, ("######     TxATM queue %d bytes...\n", length));

	    	/* wally */
	    	if(pAdapter->dspState == DspStateTrained && 
		   pDslModem->DslLinkState == GP_TRAINED){ /* wally */
			
		    	int err=GpTransmitCells(pDslModem,pAddr , pAddr+length);
		    	if (err != GP_SUCCESS){
		    		wpabs_debug_event("%s: gsi_dma_tx_start --> GpTransmitCells: Failed !!!\n",
						pAdapter->devname);
				wpabs_clear_bit(0,&pDslModem->TxDmaPending);
				goto gp_dma_tx_fail;
		    	}
			
			pDslModem->TxDmaTimer=wpabs_get_systemticks();
			
	    	}else{
		    	wpabs_debug_event("%s: CRITICAL !!!! DSP NOT TRAINED DMA TX!",pAdapter->devname);
gp_dma_tx_fail:
		    	pPciPhy->TxDescr = NULL;
		    	((pkt_info_t*)txDescr->Buffer->AllocatorContext)->length=0;
		    	((pkt_info_t*)txDescr->Buffer->AllocatorContext)->chan=NULL;
                    	txDescr->Buffer->Size = 0;
                    	/* ALEX txDescr->Buffer->DmaData.length = 0; */

		    	PHY_ADD_LIST(&(pPciPhy->TxFreeList), txDescr);
			wpabs_clear_bit(0,&pDslModem->TxDmaPending);
	    	}
	}else{
		wpabs_clear_bit(0,&pDslModem->TxDmaPending);
	}

    	DBG_LEAVE(DbgInfo);
}

/*+F*************************************************************************
 * Function:
 *   gsi_dma_tx_complete
 *
 * Description:
 *   The hardware has completed the transmit request. Queue the request to
 *   the complete list, and start the next request.
 *-F*************************************************************************/
void
gsi_dma_tx_complete(
    adapter_t  *pAdapter
    )
{
    DBG_FUNCX("gsi_dma_tx_complete", DBG_TX_SYS)
    PCI_PHY    *pPciPhy = &(pAdapter->pciPhy);
    PPHY_DESCR  txDescr;
    DSL_MODEM  *pDslModem = pAdapter->custstruct;

    ADSL_ASSERT1((!pDslModem));

    DBG_ENTER(DbgInfo);
    DBG_PARAMF(DbgInfo, "pAdapter", ("%d (0x%p)", DBG_ID(pAdapter), pAdapter));

    wpabs_debug_tx("(Debug): gsi_dma_tx_complete\n");
    

    if (((txDescr = pPciPhy->TxDescr) != NULL) &&
        (pPciPhy->State == PCI_PHY_STATE_OPERATIONAL))
    {
        DBG_TX(DbgInfo, ("######     TxATM complete.\n"));

        pPciPhy->TxDescr = NULL;
        PHY_ADD_LIST(&(pPciPhy->TxCompleteList), txDescr);
    }


    /* Indicate that Tx DMA has completed */
    wpabs_clear_bit(0,&pDslModem->TxDmaPending);	

    gsi_dma_tx_start(pAdapter);

    DBG_LEAVE(DbgInfo);
}

/*+F*************************************************************************
 * Function:
 *   gsi_dma_rx_start
 *
 * Description:
 *   Setup the circular receive buffer and start it running.
 *-F*************************************************************************/
void
gsi_dma_rx_start(
    adapter_t  *pAdapter
    )
{
    DBG_FUNCX("gsi_dma_rx_start", DBG_RX_SYS)
    PCI_PHY        *pPciPhy = &(pAdapter->pciPhy);
    unsigned long     pAddr;
    u_int32_t             length;

    DBG_ENTER(DbgInfo);
    DBG_PARAMF(DbgInfo, "pAdapter", ("%d (0x%p)", DBG_ID(pAdapter), pAdapter));


    if (pPciPhy->State == PCI_PHY_STATE_OPERATIONAL)
    {
        pAddr = pPciPhy->RxBuffer.pAddr;
        length = pPciPhy->RxBuffer.length;

        pPciPhy->RxDataHead = pAddr;

        /* Enforce hardware constraint */
        DBG_ASSERT(!(pAddr & 0x03));
        DBG_ASSERT(!((pAddr + length) & 0x03));

        DBG_RX(DbgInfo, ("######     RxATM ready %d bytes...\n", length));

        gsi_pci_write(pAdapter, GSI_DSLDATA_RXSTARTADD, pAddr);
        gsi_pci_write(pAdapter, GSI_DSLDATA_RXENDADD, pAddr + length);
        gsi_pci_write(pAdapter, GSI_DSLDATA_RXCOMMAND, PANDA_INITIAL_RXCOMMAND | 0x01);

        gsi_pci_write(pAdapter, GSI_DSLDATA_RXCFG1, PANDA_INITIAL_RXCFG1);
        gsi_pci_write(pAdapter, GSI_DSLDATA_RXCFG2, PANDA_INITIAL_RXCFG2);

	wpabs_debug_event("(DEbug): STARTING DMA RX  OK \n");
    }


    DBG_LEAVE(DbgInfo);
}

/*+F*************************************************************************
 * Function:
 *   gsi_dma_rx_complete
 *
 * Description:
 *   Check if receive data is available and needs to be passed up.
 *-F*************************************************************************/
void
gsi_dma_rx_complete(
    adapter_t  *pAdapter
    )
{
    DBG_FUNCX("gsi_dma_rx_complete", DBG_RX_SYS)
    PCI_PHY    *pPciPhy = &(pAdapter->pciPhy);
    PPHY_DESCR  rxDescr;
    u_int32_t         rxDataTail;

    /*DBG_VERBOSE    DBG_ENTER(DbgInfo);*/
    /*DBG_VERBOSE    DBG_PARAMF(DbgInfo, "pAdapter", ("%d (0x%p)", DBG_ID(pAdapter), pAdapter));*/


    if (pPciPhy->State == PCI_PHY_STATE_OPERATIONAL)
    {
        rxDataTail = gsi_pci_read(pAdapter, GSI_DSLDATA_RXCURRENTADD);

        /* Check if the current address is out of the buffer! If so, the */
        /* Rx DMA needs to be re-started. */
        if (rxDataTail > (pPciPhy->RxBuffer.pAddr + pPciPhy->RxBuffer.length))
        {
            DBG_WARNING(DbgInfo, ("Rx current address is outside of buffer!\n"));
            gsi_dma_rx_start(pAdapter);
        }

        /* Make sure we actually have data in the buffer. */
        if (rxDataTail != pPciPhy->RxDataHead)
        {
            u_int32_t     length;

            /* This is the case where the buffer has wrapped. The tail is
            // before the head of where the data starts. This is pretty simple.
            // Just give it up as 2 buffers. The buffer starting from the
            // head to the end of the circular buffer, and another buffer
            // from the beginning of the circular buffer to the tail.
            // This code block takes care of the head to end-of-buffer case.
	    */
            if (rxDataTail < pPciPhy->RxDataHead)
            {
                length = (pPciPhy->RxBuffer.pAddr + pPciPhy->RxBuffer.length) -
                    pPciPhy->RxDataHead;

                rxDescr = (PPHY_DESCR) PHY_REMOVE_LIST(&(pPciPhy->RxFreeList));
                if (rxDescr != NULL)
                {
                    rxDescr->Buffer->Data  = (unsigned long*)wpabs_bus2virt(pPciPhy->RxDataHead);
                    rxDescr->Buffer->Size = length;

                    PHY_ADD_LIST(&(pPciPhy->RxCompleteList), rxDescr);
                }
                else
                {
                    DBG_WARNING(DbgInfo, ("Unable to find a free RxDescr!\n"));
                }

                pPciPhy->RxDataHead += length;
                if (pPciPhy->RxDataHead >=
                    (pPciPhy->RxBuffer.pAddr + pPciPhy->RxBuffer.length))
                {
                    pPciPhy->RxDataHead = pPciPhy->RxBuffer.pAddr;
                }
                DBG_RX(DbgInfo, ("######     RxATM RxWrap %d bytes...\n", length));
            }

            /* Simple case where the data hasn't wrapped, and we have one nice
             * linear buffer. Note, this may be a continuation of the wrap case
             * as described above.
	     */
            if (rxDataTail > pPciPhy->RxDataHead)
            {
                length = rxDataTail - pPciPhy->RxDataHead;

                rxDescr = (PPHY_DESCR) PHY_REMOVE_LIST(&(pPciPhy->RxFreeList));
                if (rxDescr != NULL)
                {
                    rxDescr->Buffer->Data = (unsigned long*)wpabs_bus2virt(pPciPhy->RxDataHead);
                    rxDescr->Buffer->Size = length;

                    PHY_ADD_LIST(&(pPciPhy->RxCompleteList), rxDescr);
                }
                else
                {
                    DBG_WARNING(DbgInfo, ("Unable to find a free RxDescr!\n"));
                }

                pPciPhy->RxDataHead += length;
                DBG_RX(DbgInfo, ("######     RxATM Rx %d bytes...\n", length));
            }
        }

        /* Restart the timer interrupt to fix the wandering watchdog timer. DWG*/
        gsi_pci_write(pAdapter, GSI_DSLDATA_RXCOMMAND,
            PANDA_INITIAL_RXCOMMAND | 0x01);
    }


/*DBG_VERBOSE    DBG_LEAVE(DbgInfo);*/
}

/*+F*************************************************************************
 * Function:
 *   gsi_check_train_handler
 *
 * Description:
 *   Timer routine to check to DSP counters for loss of synchronization. If
 *   the link is lost, the DSP needs to be reset, and the retrain routine
 *   needs to be started. If the link is truely down, then the auto mode
 *   detect routine is restarted to try to re-establish the connection with
 *   the CO.
 *-F*************************************************************************/
VOID
gsi_check_train_handler(unsigned long functionContext)
{

       	adapter_t  *pAdapter = (adapter_t *) functionContext;
	DSL_MODEM  *pDslModem;
	unsigned long ReturnValue;
	unsigned long timeout;
	
	pDslModem = pAdapter->custstruct;

	if (!pDslModem && pAdapter->config.interface == WAN_INTERFACE){
		goto gsi_check_train_handler_exit;
	}

	if (!pDslModem || pDslModem->InitStatus & GP_INIT_SHUTDOWN){
		return;
	}
	
	ReturnValue = GpMonitorTransceiver(pDslModem);
				    
	switch(ReturnValue){
	case GP_LINK_DOWN:

		wpabs_debug_event("%s: GP_LINK_DOWN, Training State\n",
				pAdapter->devname);

		if(pAdapter->dspState == DspStateDisconnecting){ 
			wpabs_debug_event("%s: ADSL Link connecting...\n",
					pAdapter->devname);
		}
		
		pAdapter->dspState = DspStateTraining;
		pAdapter->tx_collisions=0;

	        /* Update the data rate*/
 	        gsi_dsp_datarate(pAdapter, ATM_PHY_LINK_STATE_DISCONNECTED);
		break;

	case GP_LINK_UP:
		wpabs_debug_event("%s: GP_LINK_UP, State Trained\n",
					pAdapter->devname);

                /* BugFix: NC Aug 28 2002
                 * New variable con_timeout is used to indicate
		 * when the dsl link became active. We will use
                 * this value to wait N number of seconds before
                 * we declare driver state as connected.
                 * Refer to explanation below */

                if (pAdapter->dspState != DspStateTrained){
                        pAdapter->dspState	= DspStateTrained;
                        pAdapter->con_timeout	= wpabs_get_systemticks();
                }

                /* Update the data rate */
                gsi_dsp_datarate(pAdapter, ATM_PHY_LINK_STATE_CONNECTED); 
		break;	

	case GP_LINK_DISCONNECTING:

		wpabs_debug_event("%s: GP_LINK_DISCONNECTED, State Waiting\n",
					pAdapter->devname);
		
		if (pAdapter->dspState == DspStateTrained){
			adsl_ppp_term_request (pAdapter);
			wpabs_debug_event("%s: ADSL Link disconnecting...\n",
						pAdapter->devname);
		}

		pAdapter->dspState = DspStateDisconnecting;
		gsi_dsp_datarate(pAdapter, ATM_PHY_LINK_STATE_DISCONNECTED);
	
		break;
		
	case GP_NO_CHANGE:
	        /* BugFix: NC Aug 28 2002
                 *
                 * The card->wandev.state is used by LanTx
                 * to start transmitting. However, after the
                 * dsl link comes up, we must wait N number
                 * of seconds before we declare link up to
                 * the IP stack.  The Tx and Rx DMA buffer
                 * are written to after the DSP link comes
                 * up. Thus, delay chaning the link state
                 * for at least 7 seconds in order to give
                 * dsp chip time to get its DMS's in order */

                if (pAdapter->dspState == DspStateTrained &&
                    wpabs_get_state(pAdapter->card) != WAN_CONNECTED){
                        if ((wpabs_get_systemticks() - pAdapter->con_timeout) > 7*wpabs_get_hz()){
                                GpWpLinkStatusChange(pAdapter);
                        }
                }                          
		break;
	}

gsi_check_train_handler_exit:
	wpabs_del_timer(pAdapter->check_train);
	timeout=GSI_CHECK_TRAIN_TIMEOUT*wpabs_get_hz();
    	wpabs_add_timer(pAdapter->check_train, timeout);
}

/*+F*************************************************************************
 * Function:
 *   gsi_init
 *
 * Description:
 *   Initialize the GSI subsystem. This initializes the adapter timer routines,
 *   and initializes the DSP.
 *-F*************************************************************************/
void
gsi_init(
    adapter_t  *pAdapter
    )
{
    DBG_FUNC("gsi_init")
		    

    DBG_ENTER(DbgInfo);
    DBG_PARAMF(DbgInfo, "pAdapter", ("%d (0x%p)", DBG_ID(pAdapter), pAdapter));

    /* Initialize timers*/
    wpabs_init_timer(pAdapter->check_train,
		    &gsi_check_train_handler, (unsigned long)pAdapter);
    wpabs_init_timer(pAdapter->auto_vcivpi,
		    &adsl_auto_vci_vpi, 
		    (unsigned long)pAdapter);

    DBG_LEAVE(DbgInfo);
}

/*+F*************************************************************************
 * Function:
 *   gsi_disconnect
 *
 * Description:
 *   Disconnect from the DSL network.
 *-F*************************************************************************/
void
gsi_disconnect(
    adapter_t  *pAdapter
    )
{
    DBG_FUNC("gsi_disconnect")

    DBG_ENTER(DbgInfo);
    DBG_PARAMF(DbgInfo, "pAdapter", ("%d (0x%p)", DBG_ID(pAdapter), pAdapter));

    /* Reset the DSP*/
    gsi_dsp_reset(pAdapter);

    DBG_LEAVE(DbgInfo);
}

/*+F*************************************************************************
 * Function:
 *   gsi_restart
 *
 * Description:
 *   Reset and start the DSP training process.
 *-F*************************************************************************/
void gsi_restart(adapter_t  *pAdapter)
{
    DBG_FUNC("gsi_restart")
    gsi_config_t   *pCfg;
    DSL_MODEM	*pDslModem = pAdapter->custstruct;
    struct PARAM_XCVR_ARRAY    *pParam;
    unsigned long timeout;

    if (!pDslModem || pDslModem->InitStatus & GP_INIT_SHUTDOWN){
	return;
    }

    /* Reload the GTI parameters from the adapter configuration space.*/
    pCfg = &(pAdapter->config);

    pParam = &(pDslModem->XcvrParam[GSI_XCVR_1]);

    DBG_PRINT(DbgInfo,("In gsi_restart\n"))

    pParam->item[GTI_STANDARD_ITEM]             = pCfg->gtiStandard;
    pParam->item[GTI_TRELLIS_ITEM]              = pCfg->gtiTrellis;
    pParam->item[GTI_TX_POWER_ATTENUATION_ITEM] = pCfg->gtiTxPowerAtten;
    pParam->item[GTI_CODING_GAIN_ITEM]          = pCfg->gtiCodingGain;
    pParam->item[GTI_MAX_BITS_PER_BIN_ITEM]     = pCfg->gtiMaxBitsPerBin;
    pParam->item[GTI_TX_START_BIN_ITEM]         = pCfg->gtiTxStartBin;
    pParam->item[GTI_TX_END_BIN_ITEM]           = pCfg->gtiTxEndBin; 	/*0x1f*/
    pParam->item[GTI_RX_START_BIN_ITEM]         = pCfg->gtiRxStartBin; 	/*6*/
    pParam->item[GTI_RX_END_BIN_ITEM]           = pCfg->gtiRxEndBin; 	/*0xff;*/
    pParam->item[GTI_RX_BIN_ADJUST_ITEM]        = pCfg->gtiRxBinAdjust;
    pParam->item[GTI_FRAMING_STRUCTURE_ITEM]    = pCfg->gtiFramingStruct;
    pParam->item[GTI_EXPANDED_EXCHANGE_ITEM]    = pCfg->gtiExpandedExchange;
    pParam->item[GTI_CLOCK_TYPE_ITEM]           = pCfg->gtiClockType; 	/*0*/

    /* The MAX DOWN RATE Value is not actual
     * baud rate:  1=32kbps.  Therefore,
     * the user selected baud rate must be
     * devided by 32 */
    pParam->item[GTI_MAX_DOWN_RATE_ITEM]        = pCfg->gtiMaxDownRate/32; 

#if 0
   DBG_PRINT(DbgInfo,("GTI_MAX_DOWN_RATE_ITEM = %x\n", pParam->item[GTI_MAX_DOWN_RATE_ITEM]))
   DBG_PRINT(DbgInfo,("GTI_CLOCK_TYPE_ITEM = %x\n", pParam->item[GTI_CLOCK_TYPE_ITEM]))
   DBG_PRINT(DbgInfo,("GTI_EXPANDED_EXCHANGE_ITEM = %x\n", pParam->item[GTI_EXPANDED_EXCHANGE_ITEM]))
   DBG_PRINT(DbgInfo,("GTI_FRAMING_STRUCTURE_ITEM = %x\n", pParam->item[GTI_FRAMING_STRUCTURE_ITEM]))
   DBG_PRINT(DbgInfo,("GTI_RX_BIN_ADJUST_ITEM = %x\n", pParam->item[GTI_RX_BIN_ADJUST_ITEM]))
   DBG_PRINT(DbgInfo,("GTI_RX_END_BIN_ITEM = %x\n", pParam->item[GTI_RX_END_BIN_ITEM]))
   DBG_PRINT(DbgInfo,("GTI_RX_START_BIN_ITEM = %x\n", pParam->item[GTI_RX_START_BIN_ITEM]))
   DBG_PRINT(DbgInfo,("GTI_TX_END_BIN_ITEM = %x\n", pParam->item[GTI_TX_END_BIN_ITEM]))
   DBG_PRINT(DbgInfo,("GTI_TX_START_BIN_ITEM = %x\n", pParam->item[GTI_TX_START_BIN_ITEM]))
   DBG_PRINT(DbgInfo,("GTI_MAX_BITS_PER_BIN_ITEM = %x\n", pParam->item[GTI_MAX_BITS_PER_BIN_ITEM]))
   DBG_PRINT(DbgInfo,("GTI_CODING_GAIN_ITEM = %x\n", pParam->item[GTI_CODING_GAIN_ITEM]))
   DBG_PRINT(DbgInfo,("GTI_TX_POWER_ATTENUATION_ITEM = %x\n", pParam->item[GTI_TX_POWER_ATTENUATION_ITEM]))
   DBG_PRINT(DbgInfo,("GTI_TRELLIS_ITEM = %x\n", pParam->item[GTI_TRELLIS_ITEM]))
   DBG_PRINT(DbgInfo,("GTiStandard = %x\n", pParam->item[GTI_STANDARD_ITEM]))

#endif

    /* Reset the DSP*/
    gsi_dsp_reset(pAdapter);

    /* Start the DSP training*/
    gsi_dsp_train(pAdapter);

    wpabs_del_timer(pAdapter->check_train);

    timeout=GSI_CHECK_TRAIN_TIMEOUT*wpabs_get_hz();
    wpabs_add_timer(pAdapter->check_train, timeout);

    DBG_LEAVE(DbgInfo);
}


/*+F*************************************************************************
 * Function:
 *   gsi_dma_init
 *
 * Description:
 *   Initialize the adapter DMA subsystem.
 *-F*************************************************************************/
void
gsi_dma_init(
    adapter_t  *pAdapter
    )
{
    DBG_FUNCX("gsi_dma_init", DBG_ALL_SYS)
    PCI_PHY    *pPciPhy = &(pAdapter->pciPhy);
    unsigned long flags;

    DBG_ENTER(DbgInfo);
    DBG_PARAMF(DbgInfo, "pAdapter", ("%d (0x%p)", DBG_ID(pAdapter), pAdapter));

    adsl_card_lock_irq(pAdapter,&flags);
    
    /* Halt PANDA transmit and receive*/
    gsi_pci_write(pAdapter, GSI_GLUE_RESETIMAGE, 1);

    /* Make sure the receive handler is in sync with this update.*/
    pPciPhy->RxDataHead = PANDA_INITIAL_RXSTARTADD;

    gsi_pci_write(pAdapter, GSI_DSLDATA_TXCFG, PANDA_INITIAL_TXCFG);
    gsi_pci_write(pAdapter, GSI_DSLDATA_TXSTARTADD, PANDA_INITIAL_TXSTARTADD);
    gsi_pci_write(pAdapter, GSI_DSLDATA_TXENDADD, PANDA_INITIAL_TXENDADD);
    gsi_pci_write(pAdapter, GSI_DSLDATA_TXCOMMAND, PANDA_INITIAL_TXCOMMAND);
    gsi_pci_write(pAdapter, GSI_DSLDATA_RXCFG1, PANDA_INITIAL_RXCFG1);
    gsi_pci_write(pAdapter, GSI_DSLDATA_RXCFG2, PANDA_INITIAL_RXCFG2);
    gsi_pci_write(pAdapter, GSI_DSLDATA_RXSTARTADD, PANDA_INITIAL_RXSTARTADD);
    gsi_pci_write(pAdapter, GSI_DSLDATA_RXENDADD, PANDA_INITIAL_RXENDADD);
    gsi_pci_write(pAdapter, GSI_DSLDATA_RXCOMMAND, PANDA_INITIAL_RXCOMMAND);


    adsl_card_unlock_irq(pAdapter,&flags);

    DBG_LEAVE(DbgInfo);
}

static void adsl_ppp_term_request (adapter_t *pAdapter)
{
#define pLan ((gsi_lan_t*)&(pAdapter->lan))
	
	if (pAdapter->EncapMode == RFC_MODE_PPP_LLC ||
            pAdapter->EncapMode == RFC_MODE_PPP_VC){
		adsl_lan_rx(pLan->dev,
			    NULL,	/*rx_skb->data,	//Header*/
			    0,      /*1,*//*Header length*/
			    &sppp_term[0],
			    sizeof(sppp_term));
	}

#undef pLan
}
