
#include "Gp.h"	

#define MAX(a,b) (((a)>(b))?(a):(b))
#define IFP2AC(IFP) ((struct arpcom *)IFP)
#define ATM_OVERHEAD  1.1

extern wan_adsl_vcivpi_t	adsl_vcivpi_list[];
extern int			adsl_vcivpi_num;

static int 	GpAdapterInit(adapter_t* pAdapter);
static void 	GpQueryConfig(adapter_t  *pAdapter);
static void 	adsl_halt_adapter(adapter_t  *pAdapter);
static void 	adsl_release_adapter(adapter_t *pAdapter);

#if 0
static void 	readEEPROM(adapter_t *pAdapter);
static void 	writeEEPROM(adapter_t *pAdapter);
#endif

/*+F*************************************************************************
 * Function:
 *   adsl_global_init
 *
 * Description:
 *-F*************************************************************************/
void* adsl_create(void* dsl_cfg, void* card_id, char* devname)
{
 	DSL_MODEM*	pDslModem = NULL;
	adapter_t*	pAdapter = NULL;
	wan_adsl_conf_t*	conf = (wan_adsl_conf_t*)dsl_cfg;
	int success = TRUE, x = 0;

  	pAdapter = (adapter_t *)wpabs_malloc(sizeof(*pAdapter));
    	if (pAdapter == NULL){
       		wpabs_debug_event("%s: Unable to allocate adapter data!\n",	
					devname);
        	success = FALSE;
        	goto attach_adapter_exit;
    	}

    	wpabs_memset(pAdapter, 0, sizeof(*pAdapter));
	pAdapter->devname = devname;

	pAdapter->tq_atm_task = wpabs_tasklet_alloc();
	if (pAdapter->tq_atm_task==NULL){
       		wpabs_debug_event("%s: Unable to allocate tasklet queue!\n",	
					devname);
		success = FALSE;
		goto attach_adapter_exit;
	}
	
	pAdapter->trace_info = adsl_trace_info_alloc();
	if (pAdapter->trace_info==NULL){
       		wpabs_debug_event("%s: Unable to allocate trace info!\n",	
					devname);
		success = FALSE;
		goto attach_adapter_exit;
	}
	adsl_trace_info_init(pAdapter->trace_info);
	
	pAdapter->card = (void*)card_id;
	/* Copy ADSL configuration to pAdapter struct */
    	wpabs_memset(&pAdapter->config, 0, sizeof(gsi_config_t));

	pAdapter->config.EncapMode = conf->EncapMode;	
	pAdapter->config.Vci	= conf->Vci;
	pAdapter->config.Vpi	= conf->Vpi;
	pAdapter->max_sdu_size  = conf->mtu*ATM_OVERHEAD;
	if (pAdapter->max_sdu_size%sizeof(GP_CELL)){
		pAdapter->max_sdu_size-=pAdapter->max_sdu_size%sizeof(GP_CELL);
		pAdapter->max_sdu_size+=sizeof(GP_CELL);
	}

	wpabs_debug_init("%s: ADSL Interface: EncapMode=%d Vci=%d Vpi=%d MTU(+ATM)=%d\n",
					pAdapter->devname,
					pAdapter->config.EncapMode,
					pAdapter->config.Vci,
					pAdapter->config.Vpi,
					pAdapter->max_sdu_size);

	pAdapter->config.interface 	= LAN_INTERFACE;
	
#ifdef GSIWAN 
	if (pAdapter->config.EncapMode == RFC_MODE_PPP_LLC || 
	    pAdapter->config.EncapMode == RFC_MODE_PPP_VC){
		
		wpabs_debug_event("%s: Initializing WAN Interface\n",
					pAdapter->devname);
		pAdapter->config.interface = WAN_INTERFACE;

    	}else
#endif	
	
	{
		wpabs_debug_event("%s: Initializing LAN Interface\n",
					pAdapter->devname);
	}
	
        pAdapter->config.rxBufferCount		= conf->RxBufferCount;
        pAdapter->config.txBufferCount		= conf->TxBufferCount;

        pAdapter->config.gtiStandard		= conf->Standard;
        pAdapter->config.gtiTrellis		= conf->Trellis;
        pAdapter->config.gtiTxPowerAtten	= conf->TxPowerAtten;
        pAdapter->config.gtiCodingGain		= conf->CodingGain;
        pAdapter->config.gtiMaxBitsPerBin	= conf->MaxBitsPerBin;
        pAdapter->config.gtiTxStartBin		= conf->TxStartBin;
        pAdapter->config.gtiTxEndBin		= conf->TxEndBin;
        pAdapter->config.gtiRxStartBin		= conf->RxStartBin;
        pAdapter->config.gtiRxEndBin		= conf->RxEndBin;
        pAdapter->config.gtiRxBinAdjust		= conf->RxBinAdjust;
        pAdapter->config.gtiFramingStruct	= conf->FramingStruct;	
        pAdapter->config.gtiExpandedExchange	= conf->ExpandedExchange;
        pAdapter->config.gtiClockType		= conf->ClockType;
        pAdapter->config.gtiMaxDownRate		= conf->MaxDownRate;

	if (!conf->atm_watchdog){	
		pAdapter->atm_watchdog = -1;
		wpabs_debug_event("%s: Disabling ADSL (ATM OAM) Watchdog\n",pAdapter->devname);
	}else{
		wpabs_debug_event("%s: Enabling ADSL (ATM OAM) Watchdog\n",pAdapter->devname);
		pAdapter->atm_watchdog = 0;
	}
	
	pAdapter->atm_autocfg			= conf->atm_autocfg;
	/* If interface configured to automatic VCI/VPI detection, start searching
	 * from known VCI/VPI list.
	 */
	pAdapter->auto_vcivpi_status		= ADSL_AUTO_VCI_VPI_KNOWN;
	pAdapter->auto_vcivpi_index		= 0;
	pAdapter->auto_vci			= 0;
	pAdapter->auto_vpi			= 0;

	for(x = 0; x < conf->vcivpi_num; x++){
		if (adsl_vcivpi_num){
			int y = 0;
			for(y = 0; y < adsl_vcivpi_num; y++){
				if ((adsl_vcivpi_list[y].vci == conf->vcivpi_list[x].vci) &&
				    (adsl_vcivpi_list[y].vpi == conf->vcivpi_list[x].vpi)){
					continue;
				}
			}
		}
		adsl_vcivpi_list[adsl_vcivpi_num + x].vci = conf->vcivpi_list[x].vci;
		adsl_vcivpi_list[adsl_vcivpi_num + x].vpi = conf->vcivpi_list[x].vpi;
	}
	adsl_vcivpi_num += conf->vcivpi_num;

	pAdapter->auto_vcivpi = wpabs_timer_alloc();
	if (pAdapter->auto_vcivpi == NULL){
       		wpabs_debug_event("%s: Unable to allocate auto_vcivpi timer!\n",	
					pAdapter->devname);
        	success = FALSE;
        	goto attach_adapter_exit;
    	}

	
    	pAdapter->check_train = wpabs_timer_alloc();
    	if (pAdapter->check_train == NULL){
       		wpabs_debug_event("%s: Unable to allocate check_train timer!\n",	
					pAdapter->devname);
        	success = FALSE;
        	goto attach_adapter_exit;
    	}

    	pDslModem = (DSL_MODEM *) wpabs_malloc(sizeof(*pDslModem));
    	if (pDslModem == NULL){
        	wpabs_debug_event("%s: Unable to allocate DSL_MODEM data!\n",
					pAdapter->devname);
        	success = FALSE;
        	goto attach_adapter_exit;
    	}
    	wpabs_memset(pDslModem, 0, sizeof(*pDslModem));

	wpabs_debug_cfg("%s: DslModem structure = %p\n",
				pAdapter->devname, pDslModem);
        wpabs_debug_cfg("%s: Size of DslModem structure = %d\n",
				pAdapter->devname, sizeof(*pDslModem));
	wpabs_debug_cfg("%s: Size of Adapter structure = %d\n",
				pAdapter->devname, sizeof(*pAdapter));
	wpabs_debug_cfg("%s: Size of pciPhy structure = %d\n",
				pAdapter->devname, sizeof(pAdapter->pciPhy));
	wpabs_debug_cfg("%s: Size of lan structure = %d\n",
				pAdapter->devname, sizeof(pAdapter->lan));
	wpabs_debug_cfg("%s: Size of wan structure = %d\n",
				pAdapter->devname, sizeof(pAdapter->wan));

	/* GPIO 6 & 7 are used for other things (EEPROM?). Make sure they are always on. */
	pAdapter->interruptStatus	= (DSLINT_GPIO_6 | DSLINT_GPIO_7);
	pAdapter->dspState 		= DspStateNotTrained;
	pDslModem->DslLinkState 	= GP_NOT_TRAINED;
	pAdapter->arp_timeout		= 10;

	/* Note the HWABST layer makes these two
	 * variables obselete!  Thus they are not
	 * being used any more :) */
	pDslModem->pciBase = pAdapter->virtAddr = NULL;
	
	pDslModem->card = card_id;
        pAdapter->custstruct = (void*)pDslModem;

	pDslModem->ReceiveCells		= &CustReceiveCells;
	pDslModem->Delay		= &CustDelay;      
	pDslModem->XmitDmaDone		= &CustXmitDmaDone ;
	pDslModem->ReadPciReg		= &CustReadPciReg;
	pDslModem->ReadPciRegB		= &CustReadPciRegB;   
	pDslModem->WritePciReg		= &CustWritePciReg;	      
	pDslModem->WritePciRegB		= &CustWritePciRegB;

    	GpDisableInterrupt(pDslModem);

#if GSI_WAN
	if (pAdapter->config.interface == WAN_INTERFACE){
		wpabs_debug_cfg("%s: Initializing WAN tty device, minor=%d\n",
				pAdapter->devname, conf->tty_minor);

    		if (GpWanAlloc(pAdapter)){
        		wpabs_debug_event("%s: Unable to allocate adapter (wan)\n",
            					pAdapter->devname);
        		success = FALSE;
        		goto attach_adapter_exit;
    		}
		pAdapter->flags |= _WAN_ALLOC;

		pAdapter->wan.tty_minor = conf->tty_minor;

    		if (GpWanRegister(pAdapter)){
        		wpabs_debug_event("%s: Unable to register adapter (tty device)\n",
            						pAdapter->devname);
        		success = FALSE;
        		goto attach_adapter_exit;
    		}
	}
#endif
	
       	for (x = 0; x < GSI_MAC_ADDRESS_LENGTH; x++){
       		pAdapter->macAddr[x] =
			GpEepromRead(pDslModem, GSI_MAC_EEPROM_OFFSET + x);
       	}

    	wpabs_debug_event("%s: ADSL HW Addr: %02x:%02x:%02x:%02x:%02x:%02x\n",
        		pAdapter->devname, 
			pAdapter->macAddr[0], pAdapter->macAddr[1], 
			pAdapter->macAddr[2], pAdapter->macAddr[3],
			pAdapter->macAddr[4], pAdapter->macAddr[5]);

attach_adapter_exit:
    	if (!success){
		
   		wpabs_debug_event("%s: Adapter Config Failed, Release\n", 
					pAdapter->devname);
 
        	if (pAdapter != NULL){
            	
			wpabs_debug_cfg("%s:  Memory Release for DSL_MODEM structure\n",
						pAdapter->devname);
			pAdapter->card=NULL;
			if (pDslModem){
				wpabs_free(pDslModem);	
				pAdapter->custstruct=NULL;
			}

			if (pAdapter->auto_vcivpi){
				wpabs_free(pAdapter->auto_vcivpi);	
				pAdapter->auto_vcivpi=NULL;
			}
	
			if (pAdapter->check_train){
				wpabs_free(pAdapter->check_train);	
				pAdapter->check_train=NULL;
			}

			wpabs_debug_cfg("%s: Memory Release for pAdapter structure\n",
						pAdapter->devname);

			if (pAdapter->tq_atm_task){
				wpabs_free(pAdapter->tq_atm_task);
				pAdapter->tq_atm_task=NULL;
			}

			if (pAdapter->trace_info){
				wpabs_free(pAdapter->trace_info);
				pAdapter->trace_info=NULL;
			}

			if (pAdapter){
				wpabs_free(pAdapter);
			}
        	}
		return NULL;
    	}

	return (void*)pAdapter;
}

/*+F*************************************************************************
 * Function:
 *   adsl_wan_init
 *
 * Description:
 *-F*************************************************************************/
int adsl_wan_init(void *adapter)
{
	adapter_t *pAdapter=(adapter_t*)adapter;
	DSL_MODEM*	pDslModem = NULL;	
	
	wpabs_debug_event("%s: Initialize WAN adapter...\n", 
						pAdapter->devname);
	if (GpAdapterInit(pAdapter)){
       		wpabs_debug_event("%s: Unable to initialize adapter (wan)\n",
           					pAdapter->devname);
       		return -EINVAL;
	}
	
	pDslModem = pAdapter->custstruct;
	GP_POWER_LED_ON;

	return 0;
}



/*+F*************************************************************************
 * Function:
 *   adsl_new_if
 *
 * Description:
 *-F*************************************************************************/
void* adsl_new_if(void* adapter, unsigned char* macAddr, void* if_dev)
{
	adapter_t*	pAdapter = (adapter_t *)adapter;
	DSL_MODEM*	pDslModem = NULL;
	gsi_lan_t*	pLan = NULL;
	int		x = 0;

	ADSL_ASSERT2(pAdapter == NULL, NULL);
	ADSL_ASSERT2(pAdapter->custstruct == NULL, NULL);

	if (pAdapter->config.interface == WAN_INTERFACE){
		wpabs_debug_event("%s: Error, Wan interface doesn't support network interaces\n",
				pAdapter->devname);
		return NULL;
	}

	switch (pAdapter->config.EncapMode){
	
		case RFC_MODE_BRIDGED_ETH_LLC:	
		case RFC_MODE_BRIDGED_ETH_VC:	
			wpabs_memcpy(macAddr, pAdapter->macAddr, 6);
			break;

		case RFC_MODE_ROUTED_IP_LLC:
		case RFC_MODE_ROUTED_IP_VC:	
		case RFC_MODE_PPP_LLC:
		case RFC_MODE_PPP_VC:
			break;
			
		default:
			wpabs_debug_event("%s: Error: Invalid 1483 Mode %d\n",
					pAdapter->devname,pAdapter->config.EncapMode);
			return NULL;
	}

    	pLan = &(pAdapter->lan);

	for (x = 0; x<20; x++){
		if (x<=8){
			pAdapter->atm_addr[x]=0x31+x;
		}else if (x<=12){
			pAdapter->atm_addr[x]=0x30+x-9;
		}else if (x<=18){
			pAdapter->atm_addr[x]=0x0; /* FIXME_ADSL dev->dev_addr[x-13]; */
		}else{
			pAdapter->atm_addr[x]=0x0f;
		}
	}

	if (GpAdapterInit(pAdapter)){
		wpabs_debug_event("%s: Unable to initialize adapter \n",
						pAdapter->devname);
		return NULL;
	}
	
	pAdapter->lan.dev = if_dev;

	/* Turn ON Power LED */
	pDslModem = pAdapter->custstruct;
	GP_POWER_LED_ON;

	return (void*)pAdapter;
}


/*+F*************************************************************************
 * Function:
 *   adsl_if_init
 *
 * Description:
 *-F*************************************************************************/
int adsl_del_if(void* adapter)
{
	adapter_t*	pAdapter = NULL;
	DSL_MODEM*	pDslModem = NULL;

	ADSL_ASSERT(adapter == NULL);

	pAdapter = (adapter_t *)adapter;
	pDslModem = pAdapter->custstruct;

	/* Turn OFF all LEDs */
	GP_POWER_LED_OFF;

	GpDisableInterrupt(pDslModem);

	if (!(pDslModem->InitStatus & GP_INIT_SHUTDOWN)){
		/*NC: We should not reset the chip now,
		 *    because we are shutting down. No
		 *    point of triggering interrupts here!
		 *
		wpabs_debug_event("Del IF GSI Disconnect!\n");
		gsi_disconnect(pAdapter);
		*/
		pDslModem->InitStatus |= GP_INIT_SHUTDOWN;
	}
		
	if (pAdapter->flags & _ADAPTER_RESTART){
		adsl_halt_adapter(pAdapter);
		pAdapter->flags &= ~_ADAPTER_RESTART;
	}

    	if (pAdapter->flags & _ATMUNI_ALLOC){
		wpabs_debug_cfg("%s: AtmUni Shutdown\n", pAdapter->devname);
        	GpatmisAtmUniDestroy(pAdapter->AtmUniHandle);
        	pAdapter->flags &= ~_ATMUNI_ALLOC;
    	}

	pAdapter->lan.dev = NULL;
	
	return 0;
}

/*+F*************************************************************************
 * Function:
 *   adsl_can_tx
 *
 * Description:
 *-F*************************************************************************/
int adsl_can_tx(void* adapter)
{
    	adapter_t*		pAdapter = NULL;
	gsi_lan_t*		pLan = NULL;
	
	ADSL_ASSERT(adapter == NULL);
    	pAdapter = (adapter_t *)adapter;
	pLan = &(pAdapter->lan);

	ADSL_ASSERT2((pLan->dev==NULL),-EFAULT);
	
	if (!wpabs_netif_dev_up(pLan->dev)){
		return -EINVAL;
	}
	
	wpabs_debug_tx("%s: TxLan State=%s\n",
				pAdapter->devname, 
				(pAdapter->dspState == DspStateTrained) ? 
						"Trained" : "Not Trained");
       	if (pAdapter->dspState != DspStateTrained){
		wpabs_debug_tx("%s: TxLan carrier failed, dropping tx pkt!\n",
				pAdapter->devname);
		return -EINVAL;
	}

	if (wpabs_get_state(pAdapter->card) != WAN_CONNECTED){
		wpabs_debug_tx("%s: TxLan interface not connected, dropping tx pkt!\n",
				pAdapter->devname);
		return -EINVAL;
	}

	return 0; 
}


/*+F*************************************************************************
 * Function:
 *   adsl_send
 *
 * Description:
 *-F*************************************************************************/
int adsl_send(void* adapter_id, void* tx_skb, unsigned int Flag)
{
	adapter_t*	pAdapter = (adapter_t *)adapter_id;
	DSL_MODEM*	pDslModem = NULL;
	int 		status = 0x0;

	ADSL_ASSERT(pAdapter == NULL);
	ADSL_ASSERT(pAdapter->custstruct == NULL);

	pDslModem = (DSL_MODEM*)pAdapter->custstruct;

	/* During atm auto configuration, if non discovery
	 * packet is transmitted drop it */
	/*if (!Flag && wpabs_test_bit(0, &pAdapter->atm_autocfg_running)){
	 *	return 2;
	 *}
	 */

       	status = GpatmisRfc1577Send((PATM_USB_INSTANCE)pAdapter, 
				    tx_skb,
				    Flag);
       	switch (status){
	case NDIS_STATUS_SUCCESS:
		wpabs_debug_tx("%s: TxLan Complete\n", pAdapter->devname);
               	pAdapter->tx_collisions=0;
		break;

	case NDIS_STATUS_PENDING:
		wpabs_debug_tx("%s: TxLan Pending\n", pAdapter->devname);
		return 1;

	default:
		wpabs_debug_tx("%s: TxLan Default Failed\n", pAdapter->devname);
		return 2;
	}
	/* Turn ON/OFF TX Activity LED while sending packets */
	GP_SET_TXACT_LED;

	return 0;
}

/*+F*************************************************************************
 * Function:
 *   adsl_rx_complete
 *
 * Description:
 *-F*************************************************************************/
void adsl_rx_complete(void* adapter_id)
{
	adapter_t*	pAdapter = (adapter_t *)adapter_id;
	DSL_MODEM*	pDslModem = NULL;
	
	ADSL_ASSERT1(pAdapter == NULL);
	ADSL_ASSERT1(pAdapter->custstruct == NULL);

	pDslModem = (DSL_MODEM*)pAdapter->custstruct;
	wpabs_debug_rx("%s: RxLan completed!\n", pAdapter->devname); 

	GP_SET_RXACT_LED;

	return;
}


/*+F*************************************************************************
 * Function:
 *   adsl_timeout
 *
 * Description:
 *-F*************************************************************************/
void adsl_timeout (void* dsl_adapter)
{
	adapter_t*		pAdapter = (adapter_t*)dsl_adapter;

	ADSL_ASSERT1(pAdapter == NULL);
	pAdapter->tx_collisions++;
	return;
}

/*+F*************************************************************************
 * Function:
 *   adsl_disable_comm
 *
 * Description:
 *-F*************************************************************************/
void adsl_disable_comm (void* adapter)
{
	adapter_t*	pAdapter = (adapter_t *)adapter;
	DSL_MODEM*	pDslModem = NULL;

	ADSL_ASSERT1(pAdapter == NULL);
	ADSL_ASSERT1(pAdapter->custstruct == NULL);

	pDslModem = pAdapter->custstruct;

	adsl_release_adapter(pAdapter);
	adsl_trace_purge(pAdapter->trace_info);
	
	wpabs_debug_cfg("%s: Memory Release for DSL_MODEM structure\n",
					pAdapter->devname);    
	if (pDslModem){
    		wpabs_free(pDslModem);	
	}
	pAdapter->custstruct=NULL;
	
	wpabs_debug_cfg("%s: Memory Release for pAdapter->check_train structure\n",
					pAdapter->devname);

	if (pAdapter->auto_vcivpi){
		wpabs_free(pAdapter->auto_vcivpi);	
		pAdapter->auto_vcivpi=NULL;
	}

    	if (pAdapter->check_train){
		wpabs_free(pAdapter->check_train);	
		pAdapter->check_train=NULL;
	}
	
	wpabs_debug_cfg("%s: Memory Release for pAdapter structure\n",
					pAdapter->devname);    
	pAdapter->card=NULL;

	if (pAdapter->tq_atm_task){
		wpabs_free(pAdapter->tq_atm_task);
		pAdapter->tq_atm_task=NULL;
	}

	if (pAdapter->trace_info){
		wpabs_free(pAdapter->trace_info);
		pAdapter->trace_info=NULL;
	}

    	wpabs_free(pAdapter);
	return;
}


/*+F*************************************************************************
 * Function:
 *   adsl_isr
 *
 * Description:
 *   Handle adapter interrupt requests.
 *-F*************************************************************************/
void adsl_isr(void* adapter)
{
	adapter_t*	pAdapter = (adapter_t *)adapter;
	DSL_MODEM*	pDslModem;
	GP_WORD		CurInterruptStatus = 0;

	ADSL_ASSERT1(pAdapter->custstruct == NULL);

	pDslModem = pAdapter->custstruct;

	if (!(pDslModem->InitStatus & GP_INIT_HARDWARE)){
		wpabs_debug_isr("%s: Hardware is not ready yet (0x%08lx)!\n",
				pAdapter->devname, pDslModem->InitStatus);
		return;
	}

	if ((CurInterruptStatus = GpCheckInterrupts(pDslModem)) != 0){
		wpabs_debug_isr("%s: ISR Status = 0x%02x!\n",
				pAdapter->devname, pDslModem->InterruptStatus);

		GpInterruptHandler(pDslModem);
		if (CurInterruptStatus & DSP_INT){
			Gti_ClearDSPInterrupt(&pDslModem->XcvrInfo, GTI_DSP_ALL);
		}
		GpWritePciReg(pDslModem, PCORE_ADDR_DSLINTCLEAR, CurInterruptStatus);
	}

	return;
}

#if 0
static void readEEPROM(adapter_t *pAdapter)
{
	DSL_MODEM *pDslModem = pAdapter->custstruct;

	pAdapter->pEeprom.macAddr[0] = 
			GpEepromRead( pDslModem,GSI_MAC_EEPROM_OFFSET);
	pAdapter->pEeprom.macAddr[1] = 
			GpEepromRead( pDslModem,GSI_MAC_EEPROM_OFFSET+1);
	pAdapter->pEeprom.macAddr[2] = 
			GpEepromRead( pDslModem,GSI_MAC_EEPROM_OFFSET + 2);
	pAdapter->pEeprom.macAddr[3] = 
			GpEepromRead( pDslModem,GSI_MAC_EEPROM_OFFSET + 3);
	pAdapter->pEeprom.macAddr[4] = 
			GpEepromRead( pDslModem,GSI_MAC_EEPROM_OFFSET + 4);
	pAdapter->pEeprom.macAddr[5] = 
			GpEepromRead( pDslModem,GSI_MAC_EEPROM_OFFSET + 5);
}


static void writeEEPROM(adapter_t *pAdapter)
{
	DSL_MODEM *pDslModem = pAdapter->custstruct;
	
	/*
    	// Write MAC Address.
    	*/
    	GpEepromWrite(pDslModem, 
		      GSI_MAC_EEPROM_OFFSET, 
		      (pAdapter->pEeprom.macAddr[0]));
    	GpEepromWrite(pDslModem,
		      GSI_MAC_EEPROM_OFFSET+1, 
		      (pAdapter->pEeprom.macAddr[1]));
    	GpEepromWrite(pDslModem,
		      GSI_MAC_EEPROM_OFFSET+2,
		      (pAdapter->pEeprom.macAddr[2]));
    	GpEepromWrite(pDslModem,
		      GSI_MAC_EEPROM_OFFSET+3,
		      (pAdapter->pEeprom.macAddr[3]));
    	GpEepromWrite(pDslModem,
		      GSI_MAC_EEPROM_OFFSET+4,
		      (pAdapter->pEeprom.macAddr[4]));
    	GpEepromWrite(pDslModem,
		      GSI_MAC_EEPROM_OFFSET+5,
		      (pAdapter->pEeprom.macAddr[5]));
}
#endif



/*+F*************************************************************************
 * Function:
 *   GpQueryConfig
 *
 * Description:
 *   Query the current configuration for reporting through the IOCTL
 *   inteface.
 *-F*************************************************************************/
static void GpQueryConfig(adapter_t  *pAdapter)
{
	
	gsi_config_t   *pCfg;
	DSL_MODEM *pDslModem = pAdapter->custstruct;
	unsigned long flags;

	wpabs_memset(&(pAdapter->version), 0, sizeof(pAdapter->version));

	pAdapter->version.protocol.major = GSI_PROTOCOL_VERSION_MAJOR;
	pAdapter->version.protocol.minor = GSI_PROTOCOL_VERSION_MINOR;

	wpabs_memcpy(pAdapter->version.driver, 
	       GSI_DRIVER_VERSION,
	       sizeof(GSI_DRIVER_VERSION));

	adsl_card_lock_irq(pAdapter,&flags);

    	SStatusXCVR(&pDslModem->XcvrInfo, GTI_FW_REL,
        	pAdapter->version.firmware, sizeof(pAdapter->version.firmware),
        	GTI_DSP_A);

	adsl_card_unlock_irq(pAdapter,&flags);

    	pCfg = &(pAdapter->config);

	wpabs_debug_cfg("%s: Standard = %x\n", pAdapter->devname,pCfg->gtiStandard);
	wpabs_debug_cfg("%s: Trellis = %x\n", pAdapter->devname,pCfg->gtiTrellis);
	wpabs_debug_cfg("%s: TxPowerAtten = %x\n", pAdapter->devname,pCfg->gtiTxPowerAtten);
	wpabs_debug_cfg("%s: CodingGain = %x\n", pAdapter->devname,pCfg->gtiCodingGain);
	wpabs_debug_cfg("%s: MaxBitsPerBin = %x\n", pAdapter->devname,pCfg->gtiMaxBitsPerBin);
	wpabs_debug_cfg("%s: TxStartBin = %x\n", pAdapter->devname,pCfg->gtiTxStartBin);
	wpabs_debug_cfg("%s: TxEndBin = %x\n", pAdapter->devname,pCfg->gtiTxEndBin);
	wpabs_debug_cfg("%s: RxStartBin = %x\n", pAdapter->devname,pCfg->gtiRxStartBin);
	wpabs_debug_cfg("%s: RxEndBin = %x\n", pAdapter->devname,pCfg->gtiRxEndBin);
	wpabs_debug_cfg("%s: RxBinAdjust = %x\n", pAdapter->devname,pCfg->gtiRxBinAdjust);
	wpabs_debug_cfg("%s: FramingStruct = %x\n", pAdapter->devname,pCfg->gtiFramingStruct);
	wpabs_debug_cfg("%s: ExpandedExchange = %x\n", pAdapter->devname,pCfg->gtiExpandedExchange);
	wpabs_debug_cfg("%s: ClockType = %x\n", pAdapter->devname,pCfg->gtiClockType);
	wpabs_debug_cfg("%s: MaxDownRate = %x\n", pAdapter->devname,pCfg->gtiMaxDownRate);

}

/*+F*************************************************************************
 * Function:
 *   GpAdapterInit
 *
 * Description:
 *   Initialize the adapter operating system and software interfaces.
 *-F*************************************************************************/
static int GpAdapterInit(adapter_t* pAdapter)
{
    	gsi_config_t   *pCfg;
    	int     status = 0;
    	int     x;
    	PCI_PHY        *pPciPhy;
	DSL_MODEM *pDslModem = pAdapter->custstruct;

	
	pDslModem->pCustData = (void*)pAdapter;
	pPciPhy = &(pAdapter->pciPhy);

	/* Dummy values to pass GpInitialize */
	pDslModem->RxBuffStartV = 1;
	pDslModem->RxBuffStartP = 1; 
	pDslModem->RxBuffEndV = 2;
	pDslModem->RxBuffEndP = 2;

	/* CHECK TRAIN TIMEOUT is in seconds, the monitor
	 * interface value is in ms */
	pDslModem->GpMonitorInterval = GSI_CHECK_TRAIN_TIMEOUT*1000; 

#ifdef GSI_PULSAR 
	pDslModem->RxCellCount=40;
#else
	pDslModem->RxCellCount=1;
#endif	
	pDslModem->RetrainEnable=TRUE;

	wpabs_debug_cfg("%s: Initializing GP Layer\n", pAdapter->devname);
    	
	/* Initialize the adapter hardware */
    	if(GpInitialize(pDslModem) == GP_FAIL){
        	status = -EINVAL;
		goto gsi_adapter_init_exit;
	}

	wpabs_debug_cfg("%s: Initializing GSI Layer\n", pAdapter->devname);

	/* Initialize the Linux Specific stuff like timer */
	gsi_init(pAdapter);

	wpabs_debug_cfg("%s: Initializing ATM Stack\n", pAdapter->devname);

	if (!GpatmisAtmUniInitialize()){
        	wpabs_debug_event("%s: AtmUniInitialize failed!\n", 
					pAdapter->devname);
        	status = -ENODEV;
        	goto gsi_adapter_init_exit;
    	}


	/* Query the driver configuration */
	GpQueryConfig(pAdapter);
	pCfg = &(pAdapter->config);

	pAdapter->MiniportAdapterHandle = pAdapter;
	pAdapter->EncapMode		= pCfg->EncapMode;
	pAdapter->Vci			= pCfg->Vci;
	pAdapter->Vpi			= pCfg->Vpi;
	pAdapter->RxBufferCount		= pCfg->rxBufferCount;
	pAdapter->TxBufferCount		= pCfg->txBufferCount;

	wpabs_debug_init("\n");
	wpabs_debug_init("%s: GSI Parameters:\n", pAdapter->devname);
	wpabs_debug_init("\n");
	wpabs_debug_init("%s: Encapsulation Mode:  %d\n",
					pAdapter->devname,pCfg->EncapMode);
	wpabs_debug_init("%s: VCI:                 %d\n",
					pAdapter->devname,pCfg->Vci);
	wpabs_debug_init("%s: VPI:                 %d\n",
					pAdapter->devname,pCfg->Vpi);
	wpabs_debug_init("%s: Rx Buffer Count:     %d\n",
					pAdapter->devname,pCfg->rxBufferCount);
	wpabs_debug_init("%s: Tx Buffer Count:     %d\n",
					pAdapter->devname,pCfg->txBufferCount);
	wpabs_debug_init("\n");
	wpabs_debug_init("%s: Standard:         0x%04x\n",
					pAdapter->devname,pCfg->gtiStandard);
	wpabs_debug_init("%s: Trellis:          0x%04x\n",
					pAdapter->devname,pCfg->gtiTrellis);
	wpabs_debug_init("%s: TxPowerAtten:     0x%04x\n",
					pAdapter->devname,pCfg->gtiTxPowerAtten);
	wpabs_debug_init("%s: CodingGain:       0x%04x\n",
					pAdapter->devname,pCfg->gtiCodingGain);
	wpabs_debug_init("%s: MaxBitsPerBin:    0x%04x\n",
					pAdapter->devname,pCfg->gtiMaxBitsPerBin);
	wpabs_debug_init("%s: TxStartBin:       0x%04x\n",
					pAdapter->devname,pCfg->gtiTxStartBin);
	wpabs_debug_init("%s: TxEndBin:         0x%04x\n",
					pAdapter->devname,pCfg->gtiTxEndBin);
	wpabs_debug_init("%s: RxStartBin:       0x%04x\n",
					pAdapter->devname,pCfg->gtiRxStartBin);
	wpabs_debug_init("%s: RxEndBin:         0x%04x\n",
					pAdapter->devname,pCfg->gtiRxEndBin);
	wpabs_debug_init("%s: RxBinAdjust:      0x%04x\n",
					pAdapter->devname,pCfg->gtiRxBinAdjust);
	wpabs_debug_init("%s: FramingStruct:    0x%04x\n",
					pAdapter->devname,pCfg->gtiFramingStruct);
	wpabs_debug_init("%s: ExpandedExchange: 0x%04x\n",
					pAdapter->devname,pCfg->gtiExpandedExchange);
	wpabs_debug_init("%s: ClockType:        0x%04x\n",
					pAdapter->devname,pCfg->gtiClockType);
	wpabs_debug_init("%s: MaxDownRate:      %d kbps\n",
					pAdapter->devname,pCfg->gtiMaxDownRate);
	wpabs_debug_init("\n");

    	for (x = 0; x < GSI_MAC_ADDRESS_LENGTH; x++){
        	pAdapter->UniqueId[x] = pAdapter->macAddr[x];
    	}


      	wpabs_debug_cfg("%s: GSI ATM UNI Create\n", pAdapter->devname);

#if GSI_WAN	
	if (pAdapter->config.interface == WAN_INTERFACE){

		pAdapter->AtmUniHandle = 
			GpatmisAtmUniCreate(pAdapter,
        			    (ATM_UNI_USER_LINK_STATUS_CHANGE) GpLinkStatusChange,
        			    FALSE,
        			    0,
				    pAdapter->max_sdu_size,
				    pAdapter->RxBufferCount,
				    pAdapter->TxBufferCount,
				    pAdapter->MiniportAdapterHandle,
				    PciPhyBind,
				    pAdapter,
				    pAdapter->UniqueId);
    
    		if (pAdapter->AtmUniHandle == NULL){
        		status = -ENODEV;
        		goto gsi_adapter_init_exit;
    		}
    
		pAdapter->flags |= _ATMUNI_ALLOC;
		

		if (!GpatmisRfc2364Initialize(pAdapter, PciPhyBind, pAdapter)){
			status = -ENODEV;
			goto gsi_adapter_init_exit;
		}

		wpabs_debug_cfg("%s: GSI ATM RC2364 Create VC\n", pAdapter->devname);

	}else
#endif
	{
		pAdapter->AtmUniHandle = 
			GpatmisAtmUniCreate(pAdapter,
				    (ATM_UNI_USER_LINK_STATUS_CHANGE) GpLinkStatusChange,
				    FALSE,
				    0,
				    pAdapter->max_sdu_size,
				    pAdapter->RxBufferCount,
				    pAdapter->TxBufferCount,
				    pAdapter->MiniportAdapterHandle,
				    PciPhyBind,
				    pAdapter,
				    pAdapter->UniqueId);

		if (pAdapter->AtmUniHandle == NULL){
			wpabs_debug_event("%s: AtmUniCreate failed!\n",
						pAdapter->devname);
			status = -ENODEV;
			goto gsi_adapter_init_exit;
		}

		pAdapter->flags |= _ATMUNI_ALLOC;

	
	}


	wpabs_debug_cfg("%s: GSI ATM Initialize RFC\n", pAdapter->devname);


	pDslModem->RxBuffStartV = (GP_DWORD)pPciPhy->RxBuffer.vAddr;
	pDslModem->RxBuffStartP = pPciPhy->RxBuffer.pAddr; 

	pDslModem->RxBuffEndV = (GP_DWORD)pPciPhy->RxBuffer.vAddr + pPciPhy->RxBuffer.length;
	pDslModem->RxBuffEndP = pPciPhy->RxBuffer.pAddr + pPciPhy->RxBuffer.length;

	wpabs_debug_cfg("%s: GSI Restart\n", pAdapter->devname);

    	/* Startup the adapter */
        gsi_restart(pAdapter);

	wpabs_debug_event("\n");
	wpabs_debug_event("%s: ADSL Link connecting...\n", pAdapter->devname);
	
	pAdapter->flags |= _ADAPTER_RESTART;

gsi_adapter_init_exit:
    	return status;
}


/*+F*************************************************************************
 * Function:
 *   adsl_halt_adapter
 *
 * Description:
 *   Halt the adapter.
 *-F*************************************************************************/
static void adsl_halt_adapter(adapter_t  *pAdapter)
{
	DSL_MODEM *pDslModem = pAdapter->custstruct;
	
	wpabs_del_timer(pAdapter->check_train);
	wpabs_del_timer(pAdapter->auto_vcivpi);

	GpShutdown(pDslModem);

	gsi_pci_write(pAdapter, GSI_GLUE_RESETIMAGE, PANDA_RESET_MASK);
	gsi_pci_write(pAdapter, GSI_GLUE_DSLINTENABLE, ALL_ZEROS_MASK);
	gsi_pci_write(pAdapter, GSI_GLUE_DSLINTCLEAR, ALL_ONES_MASK);

	InitSTARInterrupt(&pDslModem->XcvrInfo, GTI_FALSE);
	ClearSTARInterrupt(&pDslModem->XcvrInfo, GTI_DSP_ALL);

	pAdapter->dspState = DspStateNotTrained;
	pDslModem->DslLinkState = GP_NOT_TRAINED;

}


/*+F*************************************************************************
 * Function:
 *   adsl_release_adapter
 *
 * Description:
 *   This routine releases all the allocated adapter resources.
 *-F*************************************************************************/
static void adsl_release_adapter(adapter_t *pAdapter)
{
	DSL_MODEM*	pDslModem = NULL;

	ADSL_ASSERT1(pAdapter == NULL);
	ADSL_ASSERT1(pAdapter->custstruct == NULL);
	wpabs_debug_cfg("%s: Release Adapter\n", pAdapter->devname);
	
	pDslModem = pAdapter->custstruct;

	if (!(pDslModem->InitStatus & GP_INIT_SHUTDOWN)){
		wpabs_debug_cfg("%s: ADSL Disconnect\n", 
				pAdapter->devname);
		
		if (pAdapter->flags & _ADAPTER_RESTART){
			wpabs_debug_event("%s: ADSL Disconnecting ...\n",
					pAdapter->devname);
			gsi_disconnect(pAdapter);
		}
		
		pDslModem->InitStatus |= GP_INIT_SHUTDOWN;
	}
	
#ifdef GSI_WAN
	if (pAdapter->config.interface == WAN_INTERFACE){
                channel_t *pChan = &pAdapter->wan.channel[0];
                if (pChan && pChan->tty && pChan->count){
#if defined(linux)
                        adsl_tty_hangup(pChan->tty);
#endif
                }
        }
#endif
	
	if (pAdapter->flags & _ADAPTER_RESTART){
		wpabs_debug_cfg("%s: ADSL halt adapter\n", 
				pAdapter->devname);
		adsl_halt_adapter(pAdapter);
		pAdapter->flags &= ~_ADAPTER_RESTART;
	}
	
	/* We just set the SHUTDOWN flag, schedule this
	 * thread in the background, and make sure that 
	 * any running threads are shutdown before
	 * we proceed */
	
	/* FIXME_ADSL schedule(); */
#ifdef GSI_WAN	
    	if (pAdapter->config.interface == WAN_INTERFACE){
	    
    		if (pAdapter->flags & _WAN_ALLOC){
			wpabs_debug_cfg("%s: WAN Shutdown\n", pAdapter->devname);
        		GpWanUnregister(pAdapter);
			wpabs_debug_cfg("%s: WAN Dealloc\n", pAdapter->devname);
        		GpWanFree(pAdapter);
        		pAdapter->flags &= ~_WAN_ALLOC;
    		}

		if (pAdapter->flags & _ATMUNI_ALLOC){
			wpabs_debug_cfg("%s: AtmUni Shutdown\n", pAdapter->devname);
        		GpatmisAtmUniDestroy(pAdapter->AtmUniHandle);
        		pAdapter->flags &= ~_ATMUNI_ALLOC;
    		}
    	}
#endif

}


/*+F*************************************************************************
 * Function:
 *   adsl_udp_cmd
 *
 * Description:
 *-F*************************************************************************/
void adsl_udp_cmd(void* adapter, unsigned char command, unsigned char* udp_data, unsigned short* udp_len)
{
	adapter_t*	pAdapter = (adapter_t*)adapter;
	DSL_MODEM*	pDslModem = NULL;
	unsigned long 	flags;

	ADSL_ASSERT1(pAdapter == NULL);
	ADSL_ASSERT1(pAdapter->custstruct == NULL);
	pDslModem = pAdapter->custstruct;

	adsl_card_lock_irq(pAdapter,&flags);
	
	switch(command){

	case ADSL_OP_STATE:
	{
		unsigned short opstate=0;

		SStatusXCVR(&pDslModem->XcvrInfo, GTI_OP_STATE, 
		    	        &opstate,sizeof(opstate), GTI_CH_A);

		wpabs_memcpy(&udp_data[0], &opstate, sizeof(opstate));
		*udp_len = GTI_OP_STATE_LEN;
		
		break;
	}
	
	case ADSL_COUNTERS:
	{
		int j;

		{
			adsl_failures_t *adsl_failures=
				(adsl_failures_t*)&udp_data[*udp_len];
			
			adsl_failures->CrcErrorsPerMin= 
					pDslModem->CrcErrorsPerMin;
			
			adsl_failures->ExcessiveConsecutiveCrcErrorsPerTickCount=
					pDslModem->ExcessiveConsecutiveCrcErrorsPerTickCount;
			
			adsl_failures->ExcessiveConsecutiveCrcErrorsPerMinCount=
					pDslModem->ExcessiveConsecutiveCrcErrorsPerMinCount;
			
			adsl_failures->ExcessiveConsecutiveOverallFailureCount=
					pDslModem->ExcessiveConsecutiveOverallFailureCount;

			*udp_len=sizeof(adsl_failures_t);
		}

		
		for(j=0;j<(sizeof(pDslModem->AccCounters)/sizeof(GP_WORD));j++){
			udp_data[j+*udp_len]=pDslModem->AccCounters[j];
		}
		*udp_len += (sizeof(pDslModem->AccCounters)/sizeof(GP_WORD));
		break;
	}

	case ADSL_LAST_FAILED_STATUS:
	{
		unsigned short status=0;

		SStatusXCVR(&pDslModem->XcvrInfo, GTI_LAST_FAILED_STATUS, 
			    	        &status,sizeof(status), GTI_CH_A);

		*((unsigned short *)&udp_data[0]) = status;
		*udp_len = sizeof(status);
		
		break;
	}

	case ADSL_LCL_SNR_MARGIN:
	{
		unsigned char status=0;

		SStatusXCVR(&pDslModem->XcvrInfo, GTI_LCL_SNR_MARGIN, 
			    	        &status,sizeof(status), GTI_CH_A);

		status=status/2;
		udp_data[0] = status;
		*udp_len = sizeof(status);
		
		break;
	}

	case ADSL_UPSTREAM_MARGIN_STATUS:
	{
		unsigned char status;

		SStatusXCVR(&pDslModem->XcvrInfo, GTI_UPSTREAM_MARGIN_STATUS, 
			    	        &status,sizeof(status), GTI_CH_A);

		status=status/2;
		udp_data[0] = status;
		*udp_len = sizeof(status);

		break;
	}
			
	case ADSL_FAILURES:
	{
		if (SStatusXCVR(&pDslModem->XcvrInfo, GTI_FAILURES, 
					&udp_data[*udp_len],
			    	        sizeof(pDslModem->Failures), GTI_CH_A)){
			*udp_len = sizeof(pDslModem->Failures);
		}else{
			*udp_len = 0;
		}

		break;
	}

	case ADSL_ATTENUATION_STATUS:
	{
		unsigned short status=0;	

		SStatusXCVR(&pDslModem->XcvrInfo, GTI_ATTENUATION_STATUS, 
				&status, GTI_ATTENUATION_STATUS_LEN, GTI_DSP_A);
			
		udp_data[0] = (GTI_BYTE)((status & 0x00ff)/2);
		*udp_len = 1;
			
		break;
	}	

	case ADSL_BAUD_RATE:
	{
		unsigned long *data_ptr = (unsigned long*)&udp_data[0];
		data_ptr[0]=pAdapter->DownstreamRate;
		data_ptr[1]=pAdapter->UpstreamRate;
		*udp_len = 8;
		break;
	}

	case ADSL_ATM_CONF:
	{
		unsigned long *data_ptr = (unsigned long*)&udp_data[0];
		data_ptr[0] = pAdapter->Vpi;
		data_ptr[1] = pAdapter->Vci;
		data_ptr[2] = pAdapter->EncapMode;
		*udp_len = 12;
		break;
	}

	case ADSL_RMT_VENDOR_ID_STATUS:
		*udp_len = SStatusXCVR(&pDslModem->XcvrInfo, 
				       GTI_RMT_VENDOR_ID_STATUS, 
				       &udp_data[0], 
				       10, 
				       GTI_CH_A);
			
		break;

	case ADSL_GET_MODULATION:
		udp_data[0] = (GTI_BYTE)(GET_HOST_DB_B(&pDslModem->XcvrInfo,
						       GTI_SELCTD_STANDARD));
		*udp_len = 1;
		break;

	case ADSL_START_PROGRESS:

		SStatusXCVR(&pDslModem->XcvrInfo, 
				GTI_START_PROGRESS, 
				&udp_data[0], 
				GTI_START_PROGRESS_LEN, 
				GTI_CH_A);
		*udp_len = GTI_START_PROGRESS_LEN;
		
		break;

	case ADSL_LCL_XMIT_POWER_STATUS:


		SStatusXCVR(&pDslModem->XcvrInfo, 
				GTI_LCL_XMIT_POWER_STATUS, 
				&udp_data[0],
				GTI_LCL_XMIT_POWER_STATUS_LEN, 
				GTI_CH_A);

		*udp_len = GTI_LCL_XMIT_POWER_STATUS_LEN;
			
		break;

	case ADSL_RMT_TX_POWER_STATUS:

		SStatusXCVR(&pDslModem->XcvrInfo, 
				GTI_RMT_TX_POWER_STATUS, 
				&udp_data[0],
				GTI_RMT_TX_POWER_STATUS_LEN, 
				GTI_CH_A);

		*udp_len=GTI_RMT_TX_POWER_STATUS_LEN;
		
		break;
	
	case ADSL_ACTUAL_CONFIGURATION:

		wpabs_memset(udp_data,0,GTI_ACTUAL_CONFIGURATION_LEN);

		/* Read Actual configuration info. from transceiver */
		SStatusXCVR(&pDslModem->XcvrInfo,
				GTI_ACTUAL_CONFIGURATION,
				&udp_data[0],
				GTI_ACTUAL_CONFIGURATION_LEN,
				GTI_CH_A);
		*udp_len=GTI_ACTUAL_CONFIGURATION_LEN;
		break;

	case ADSL_ACTUAL_INTERLEAVE_STATUS:

		SStatusXCVR(&pDslModem->XcvrInfo,
				GTI_ACTUAL_INTERLEAVE_STATUS,
				&udp_data[0],
				GTI_ACTUAL_INTERLEAVE_STATUS_LEN,
				GTI_CH_A);
			
		*udp_len = GTI_ACTUAL_INTERLEAVE_STATUS_LEN;
		
		break;

	case ADSL_ATM_CELL_COUNTER:

		{
			unsigned short cell_data[2]={0,0};
			
			if (pAdapter->dspState == DspStateTrained){
				SStatusXCVR(&pDslModem->XcvrInfo,
						GTI_ATM_CELL_COUNTER,
						&cell_data[0],
						GTI_ATM_CELL_COUNTER_LEN,
						GTI_CH_A);
			}

			pAdapter->atm_stats.rx_chip += cell_data[0];
			pAdapter->atm_stats.tx_chip += cell_data[1];
		}

		wpabs_memcpy(&udp_data[0],
		       &pAdapter->atm_stats,
		       sizeof(atm_stats_t));

		*udp_len = sizeof(atm_stats_t);
		break;
	}

	adsl_card_unlock_irq(pAdapter,&flags);

	return;
}

void* adsl_get_trace_ptr(void *pAdapter_ptr)
{	
	adapter_t *pAdapter = (adapter_t *)pAdapter_ptr;
	return pAdapter->trace_info;
}

int adsl_wan_interface_type(void *pAdapter_ptr)
{	
	adapter_t *pAdapter = (adapter_t *)pAdapter_ptr;
	if (pAdapter->config.interface == WAN_INTERFACE){
		return 1;
	}else{
		return 0;
	}
}

