/*+I*************************************************************************
 * TriplePoint NDIS Abstraction Layer Definitions
 *
 * Copyright (c) 1997-2000 TriplePoint, Inc. -- http://www.TriplePoint.com
 * All rights reserved.
 *-I*************************************************************************/
#ifndef _GPSNULL_H_
#define _GPSNULL_H_

#include "common.h"

typedef PVOID       NDIS_HANDLE, *PNDIS_HANDLE;
typedef ULONG       NDIS_STATUS, *PNDIS_STATUS;
typedef ULONG       NDIS_PHYSICAL_ADDRESS, *PNDIS_PHYSICAL_ADDRESS;
#define OPTIONAL

#define NDIS_STATUS_SUCCESS                     0x00000000
#define NDIS_STATUS_PENDING                     0x00000001
#define NDIS_STATUS_FAILURE                     0xFFFFFFFF
#define NDIS_STATUS_MEDIA_CONNECT               ((NDIS_STATUS)0x4001000BL)
#define NDIS_STATUS_MEDIA_DISCONNECT            ((NDIS_STATUS)0x4001000CL)

#define NDIS_PACKET_TYPE_DIRECTED               0x00000001
#define NDIS_PACKET_TYPE_MULTICAST              0x00000002
#define NDIS_PACKET_TYPE_ALL_MULTICAST          0x00000004
#define NDIS_PACKET_TYPE_BROADCAST              0x00000008
#define NDIS_PACKET_TYPE_PROMISCUOUS            0x00000020

/*
// Determine if an argument is present by testing the value of the pointer
// to the argument value.
*/

#define ARGUMENT_PRESENT(ArgumentPointer)    (\
    (PVOID)(ArgumentPointer) != (PVOID)(NULL) )

/* begin_winnt begin_ntminiport
//
// Calculate the byte offset of a field in a structure of type type.
*/

#define FIELD_OFFSET(type, field)    ((ULONG)(PULONG)&(((type *)0)->field))


/*
// Calculate the address of the base of the structure given its type, and an
// address of a field within the structure.
*/

#define CONTAINING_RECORD(address, type, field) ((type *)( \
                                                  (PCHAR)(address) - \
                                                  (PCHAR)(&((type *)0)->field)))

/*
// Check whether an address is multicast.
*/
#define ETH_IS_MULTICAST(Address) \
    (BOOLEAN)(((PUCHAR)(Address))[0] & ((UCHAR)0x01))

/*
// Check whether an address is broadcast.
*/
#define ETH_IS_BROADCAST(Address) \
    ((((PUCHAR)(Address))[0] == ((UCHAR)0xff)) && \
     (((PUCHAR)(Address))[1] == ((UCHAR)0xff)))

#ifndef ETH_LENGTH_OF_ADDRESS
#   define ETH_LENGTH_OF_ADDRESS    6
#endif

/*
// NDIS Buffers
*/
/* #define NDIS_BUFFER_TO_SPAN_PAGES(_Buffer)      1 */

/* typedef struct _NDIS_BUFFER *PNDIS_BUFFER; */

/*
// NDIS Packets
*/
/* typedef struct _NDIS_PACKET *PNDIS_PACKET; */
/*
// NDIS Work Items
*/
typedef struct _NDIS_WORK_ITEM *PNDIS_WORK_ITEM;
typedef VOID    (*NDIS_PROC)(PNDIS_WORK_ITEM, PVOID);

typedef struct _NDIS_WORK_ITEM
{
    PVOID           Context;
    NDIS_PROC       Routine;
} NDIS_WORK_ITEM;

VOID NdisInitializeWorkItem(
    IN  PNDIS_WORK_ITEM         WorkItem,
    IN  NDIS_PROC               Routine,
    IN  PVOID                   Context
    );

NDIS_STATUS NdisScheduleWorkItem(
    IN  PNDIS_WORK_ITEM         WorkItem
    );

/*
// NDIS Memory
*/
#define NDIS_MEMORY_TAG 0xBEEF0006

NDIS_STATUS NdisAllocateMemoryWithTag(
    OUT PVOID *                 VirtualAddress,
    IN  UINT                    Length,
    IN  ULONG                   Tag
    );

VOID NdisFreeMemory(
    IN  PVOID                   VirtualAddress,
    IN  UINT                    Length,
    IN  UINT                    MemoryFlags
    );

/*
// NDIS Events
*/
typedef ULONG NDIS_EVENT, *PNDIS_EVENT;

VOID NdisInitializeEvent(
    IN  PNDIS_EVENT             Event
    );

VOID NdisResetEvent(
    IN  PNDIS_EVENT             Event
    );

BOOLEAN NdisWaitEvent(
    IN  PNDIS_EVENT             Event,
    IN  UINT                    msToWait
    );

/*
// NDIS Strings
*/
typedef struct NDIS_STRING
{
    USHORT                      MaximumLength;
    USHORT                      Length;
    PUCHAR                      Buffer;
} NDIS_STRING, *PNDIS_STRING;

#define NdisFreeString(NdisString) if (NdisString.Buffer) \
                                    {NdisFreeMemory(NdisString.Buffer, \
                                                    NdisString.MaximumLength);\
                                    NdisString.Buffer = NULL;\
                                    NdisString.MaximumLength = 0;\
                                    NdisString.Length = 0;}

/*
// NDIS Files
*/
VOID NdisOpenFile(
    OUT PNDIS_STATUS            Status,
    OUT PNDIS_HANDLE            FileHandle,
    OUT PUINT                   FileLength,
    IN  PNDIS_STRING            FileName,
    IN  NDIS_PHYSICAL_ADDRESS   HighestAcceptableAddress
    );

VOID NdisCloseFile(
    IN  NDIS_HANDLE             FileHandle
    );

VOID NdisMapFile(
    OUT PNDIS_STATUS            Status,
    OUT PVOID *                 MappedBuffer,
    IN  NDIS_HANDLE             FileHandle
    );

VOID NdisUnmapFile(
    IN  NDIS_HANDLE             FileHandle
    );

/*
// Ndis Indications
*/
void NdisMSendComplete(
    IN  NDIS_HANDLE             MiniportAdapterHandle,
    IN  PVOID 			Packet,
    IN  NDIS_STATUS             Status
    );

void NdisMEthIndicateReceive(
    IN  NDIS_HANDLE             MiniportAdapterHandle,
    IN  NDIS_HANDLE             MiniportReceiveContext,
    IN  PVOID                   HeaderBuffer,
    IN  UINT                    HeaderBufferSize,
    IN  PVOID                   LookaheadBuffer,
    IN  UINT                    LookaheadBufferSize,
    IN  UINT                    PacketSize
    );

#define NdisMEthIndicateReceiveComplete(MiniportAdapterHandle)

VOID NdisMIndicateStatus(
    IN  NDIS_HANDLE             MiniportAdapterHandle,
    IN  NDIS_STATUS             GeneralStatus,
    IN  PVOID                   StatusBuffer,
    IN  UINT                    StatusBufferSize
    );

#define NdisMIndicateStatusComplete(MiniportAdapterHandle)

#define NdisMWanSendComplete    NdisMSendComplete

VOID NdisMWanIndicateReceive(
    OUT PNDIS_STATUS            Status,
    IN NDIS_HANDLE              MiniportAdapterHandle,
    IN NDIS_HANDLE              NdisLinkContext,
    IN PUCHAR                   Packet,
    IN ULONG                    PacketSize
    );               

VOID NdisStallExecution(IN ULONG Microsecs);

#define NdisMWanIndicateReceiveComplete(MiniportAdapterHandle, NdisLinkContext)

/*
// Misc.
*/
LONG InterlockedIncrement(
    IN OUT PLONG                Addend
    );

LONG InterlockedDecrement(
    IN OUT PLONG                Addend
    );

#endif /* _GPSNULL_H_ */

