
/*//////////////////////////////////////////////////////////////////////////////
//
// INTEL CORPORATION PROPRIETARY INFORMATION
// This software is supplied under the terms of a license agreement
// or non-disclosure agreement with Intel Corporation and may not be
// copied nor disclosed except in accordance with the terms of that
// agreement.
// Copyright  1998 Intel Corporation. All Rights Reserved.
//
// PVCS Information:
//     $WorkFile$
//     $Revision: 1.2 $
//     $Modtime: 2/14/01 10:14p $
//     $Author: alex $
//
// ABSTRACT:
//      Contains the types, macros, and function definitions that define the
//      interface with the ATM physical layer
//
//
////////////////////////////////////////////////////////////////////////////////
*/
#ifndef ATMPHY_H_
#define ATMPHY_H_

#include "common.h"

typedef void * ATM_PHY_HANDLE;
typedef void * ATM_PHY_USER_HANDLE;
typedef void * ATM_PHY_DEVICE_HANDLE;

#define    MAX_RX_RESOURCE_COUNT           20

/*------------------------------------------------------------------------------
//
// ATM_PHY_STATUS
//      Defines the status values returned by ATM PHY functions
//
//------------------------------------------------------------------------------
*/
typedef enum {
   ATM_PHY_STATUS_SUCCESS,
   ATM_PHY_STATUS_FAILURE,
   ATM_PHY_STATUS_PENDING,
   ATM_PHY_STATUS_RESOURCES
} ATM_PHY_STATUS;

/*------------------------------------------------------------------------------
//
// ATM_PHY_LINK_STATE
//      Defines the values that the link state can assume
//
//------------------------------------------------------------------------------
*/
typedef enum {
    ATM_PHY_LINK_STATE_DISCONNECTED,
    ATM_PHY_LINK_STATE_CONNECTED
} ATM_PHY_LINK_STATE;

#ifdef GSI_USB
   void UsbPhyDspDownLoadFunction(ATM_PHY_HANDLE phyHandle);
#endif /* GSI_USB */

typedef struct tATM_PHY_BIND_INPUT  *PATM_PHY_BIND_INPUT;
typedef struct tATM_PHY_BIND_OUTPUT *PATM_PHY_BIND_OUTPUT;

/*------------------------------------------------------------------------------
//
// UCHAR
// ATM_PHY_USER_RECEIVE(UserHandle, Buffer)
//
//      Called to indicate a data buffer to the user of the PHY
//
// PARAMETERS:
//
// UserHandle
//      The PHY user's handle (provided during binding)
// Buffer
//      The indicated data
//
//------------------------------------------------------------------------------
*/
typedef UCHAR (* ATM_PHY_USER_RECEIVE)(ATM_PHY_USER_HANDLE,PXFER_DATA_BUFFER);

#ifdef GSI_USB
/*------------------------------------------------------------------------------
//
// void
// ATM_PHY_USER_CPL(UserHandle, *ULONG, *ULONG)
//
//      Called to by the CPL routine to get ATM Stats
//
// PARAMETERS:
//
// UserHandle
//      The PHY user's handle (provided during binding)
// Buffer
//      The indicated data
//
//------------------------------------------------------------------------------
*/
typedef void (* ATM_PHY_USER_CPL)
                    (ATM_PHY_USER_HANDLE, PULONG, PULONG
#if ATMSTATISTICS
                    ,
                    ATM_PHY_USER_HANDLE//PATM_STATISTICS
#endif //ATMSTATISTICS
                    );

#ifdef ATMSTATISTICS
typedef void (* ATM_PHY_USER_CPL2)
                    (ATM_PHY_USER_HANDLE);
#endif //ATMSTATISTICS
#endif/* GSI_USB */

/*------------------------------------------------------------------------------
//
// void
// ATM_PHY_USER_SEND_COMPLETE(UserHandle,Buffer,UserSendContext,SendStatus)
//
//      Called to indicate that a send request previously submitted to the
//      PHY layer has been completed
//
// PARAMETERS:
//
// UserHandle
//      The PHY user's handle (provided during binding)
// Buffer
//      The buffer being completed
// UserSendContext
//      User provided context for the send request being completed (provided
//      with the send request)
// SendStatus
//      The status of the send
//
//------------------------------------------------------------------------------
*/
typedef void (* ATM_PHY_USER_SEND_COMPLETE)
                    (ATM_PHY_USER_HANDLE,PXFER_DATA_BUFFER,ATM_PHY_USER_HANDLE,
                      ATM_PHY_STATUS);

/*------------------------------------------------------------------------------
//
// void
// ATM_PHY_USER_LINK_STATUS_CHANGE(UserHandle,LinkState,
//                                  UpstreamRate,DownstreamRate)
//
//      Called to indicate that the PHY link status has changed (and to
//      report the new link status)
//
// PARAMETERS:
//
// UserHandle
//      The PHY user's handle (provided during binding)
// LinkState
//      The new link state
// UpstreamRate
//      The upstream rate (in bits per second)
// DownstreamRate
//      The downstream rate (in bits per second)
//
//------------------------------------------------------------------------------
*/
typedef void (* ATM_PHY_USER_LINK_STATUS_CHANGE)
                    (ATM_PHY_USER_HANDLE,ATM_PHY_LINK_STATE,ULONG,ULONG);

/*------------------------------------------------------------------------------
//
// ATM_PHY_STATUS
// ATM_PHY_SEND(PhyHandle,Buffer,UserContext)
//
//      Called to request that a data buffer be sent by the phy layer
//
// PARAMETERS:
//
// PhyHandle
//      The PHY handle (provided during binding)
// Buffer
//      The buffer to send
// UserContext
//      The user's context for the send (returned to the user in send complete)
//
//
//--------------------
*/
typedef ATM_PHY_STATUS (* ATM_PHY_SEND)
                            (ATM_PHY_HANDLE,PXFER_DATA_BUFFER,ATM_PHY_USER_HANDLE);

#ifdef GSI_USB
//------------------------------------------------------------------------------
// ULONG
// ATM_PHY_LINEQ_TIMER(PhyHandle)
//
// PARAMETERS:
//--------------------
//typedef ULONG (* ATM_PHY_LINEQ_TIMER) (ATM_PHY_HANDLE);
#endif /* GSI_USB */

/*------------------------------------------------------------------------------
//
// ATM_PHY_STATUS
// ATM_PHY_BIND(BindInput,BindOutput)
//
//      Called to bind the phy layer with the phy user
//
// PARAMETERS:
//
// BindInput
//      The bind input parameters (provided by the PHY user)
// BindOutput
//      The bind output parameters (provided by the PHY on successful bind)
//
//------------------------------------------------------------------------------
*/
typedef ATM_PHY_STATUS (* ATM_PHY_BIND)
                            (PATM_PHY_BIND_INPUT,PATM_PHY_BIND_OUTPUT);

/*------------------------------------------------------------------------------
//
// void
// ATM_PHY_UNBIND(PhyHandle)
//
//      Called to release a previously established binding between the phy
//      layer and a phy user
//
// PARAMETERS:
//
// PhyHandle
//      The PHY handle (provided during binding)
//
//------------------------------------------------------------------------------
*/
/*typedef void (* ATM_PHY_UNBIND)(ATM_PHY_HANDLE);*/

/*typedef ATM_PHY_STATUS (* ATM_PHY_EOC_PARSE)(ATM_PHY_HANDLE, PUCHAR);*/

/*------------------------------------------------------------------------------
//
// ATM_PHY_BIND_INPUT
//      Defines the input parameters provided by the phy user when binding
//      with the phy layer
//
//------------------------------------------------------------------------------
*/
/*
typedef struct tATM_PHY_BIND_INPUT {
    ATM_PHY_USER_HANDLE             UserHandle;
    ATM_PHY_USER_RECEIVE            UserReceive;
    ATM_PHY_USER_SEND_COMPLETE      UserSendComplete;
    ATM_PHY_USER_LINK_STATUS_CHANGE UserLinkStatusChange;
    ATM_PHY_DEVICE_HANDLE           PhyDeviceHandle;
    USHORT                          PhyTxResourceCount;
    USHORT                          PhyRxResourceCount;
    USHORT                          PhyRxBufferSize;
} ATM_PHY_BIND_INPUT;
*/

/*------------------------------------------------------------------------------
//
// ATM_PHY_BIND_OUTPUT
//      Defines the output parameters returned by the phy layer to the phy
//      user after a successful bind request
//
//------------------------------------------------------------------------------
*/
/*typedef struct tATM_PHY_BIND_OUTPUT {
    ATM_PHY_HANDLE          PhyHandle;
    ATM_PHY_SEND            PhySend;
    ATM_PHY_UNBIND          PhyUnbind;
    ATM_PHY_EOC_PARSE       PhyEocParse;
    UCHAR                   PhyUniqueId[UNIQUE_ID_SIZE];
} ATM_PHY_BIND_OUTPUT;
*/
#endif /* ATMPHY_H_ */


