#ifndef COMMON_H
#define COMMON_H

#if defined(MAC_OS) || defined(linux) || defined(__NetBSD__) || defined (__FreeBSD__) || defined (__OpenBSD__)
#define _NDIS_
#ifndef NULL
#define NULL    0
#endif
#ifndef FALSE
#define FALSE   0
#define TRUE    !FALSE
#endif
#ifndef IN
#define IN
#define OUT
#endif

typedef unsigned short  WCHAR,   *PWCHAR;
typedef unsigned char   UCHAR,   *PUCHAR;
typedef unsigned short  USHORT,  *PUSHORT;
typedef unsigned short  SHORT,   *PSHORT;
typedef unsigned long   ULONG,   *PULONG;
typedef long            LONG,    *PLONG;
typedef unsigned int    UINT,    *PUINT;
typedef void            VOID,    *PVOID;
typedef int             BOOLEAN, *PBOOLEAN;
typedef long            LARGE_INTEGER, *PLARGE_INTEGER;
typedef ULONG           NTSTATUS,*PNTSTATUS;
typedef unsigned int	u_int32_t, u32;
typedef unsigned short	u_int16_t, u16;
typedef unsigned char	u_int8_t, u8;

#define STATUS_SUCCESS                  ((NTSTATUS)0x00000000L)
#define STATUS_UNSUCCESSFUL             ((NTSTATUS)0xC0000001L)
#define STATUS_MORE_PROCESSING_REQUIRED ((NTSTATUS)0xC0000016L)
#define STATUS_TIMEOUT                  ((NTSTATUS)0x00000102L)
#define STATUS_PENDING                  ((NTSTATUS)0x00000103L)
#define NT_SUCCESS(Status)              ((NTSTATUS)(Status) == 0)
#define RtlConvertLongToLargeInteger(_long) (LARGE_INTEGER)(_long)

typedef unsigned long*	virt_addr_t;
typedef unsigned long	phys_addr_t;

#ifdef linux
# ifndef PACKED
#  define PACKED __attribute__((packed))
# endif
#else
# ifndef PACKED
#  define PACKED
# endif
#if 0
extern void *memset (void *s, int c, unsigned long n);
extern void *memccpy(void *s1, const void *s2, int c, unsigned int n);
#endif
#endif

#define NdisZeroMemory(Destination, Length)         memset(Destination, 0, Length)
#define NdisMoveMemory(Destination, Source, Length) memcpy(Destination, Source, Length)

#endif /* MAC_OS || linux */

/*------------------------------------------------------------------------------
//
//  Constant definitions
//
//------------------------------------------------------------------------------
*/
#define UNIQUE_ID_SIZE              7

/*
// The memory allocation routines come in two flavors; if the compilation
// unit is built with ndis.h, the NDIS version of the allocation routines
// is used.  Otherwise, the WDM version of the allocation routines is
// used.
*/
#ifdef _NDIS_

#ifdef linux
# define NDIS_TAG                   0xDEADBEEF
#else
# define NDIS_TAG                   'NPSG'      // GSPN in reverse (to display properly)
#endif

/*------------------------------------------------------------------------------
//
// ALLOCATE_MEMORY
//      Allocates and zeros a block of nonpaged pool memory of the length
//      specified.  If the allocation attempt fails, the provided pointer
//      is set to NULL.
//
//------------------------------------------------------------------------------
*/
#define ALLOCATE_MEMORY( pptr, len)                                         \
{                                                                           \
    NdisAllocateMemoryWithTag( (pptr), (len), NDIS_TAG);                    \
    if (*(pptr) != NULL)                                                    \
    {                                                                       \
        NdisZeroMemory (*(pptr), (len));                                    \
    }                                                                       \
}

/*------------------------------------------------------------------------------
//
// FREE_MEMORY
//      Frees a previously allocated block of nonpaged pool memory of the
//      specified length.
//
//------------------------------------------------------------------------------
*/
#define FREE_MEMORY( ptr, len)                                              \
{                                                                           \
    NdisFreeMemory ((ptr), (len), 0);                                       \
    (ptr) = NULL;                                                           \
}

/*------------------------------------------------------------------------------
//
// COMPARE_MEMORY
//      Compares two blocks of memory of the specified length and returns
//      the following:
//
//      < 0
//       buf1 less than buf2
//
//      0
//       buf1 identical to buf2
//
//      > 0
//       buf1 greater than buf2
//
//------------------------------------------------------------------------------
*/
#define COMPARE_MEMORY( buf1, buf2, len)   memcmp( (buf1), (buf2), (len))

/*------------------------------------------------------------------------------
//
// COPY_MEMORY
//      Copies len bytes from the src memory block to the dst memory block.
//
//------------------------------------------------------------------------------
*/
#define COPY_MEMORY( dst, src, len)      NdisMoveMemory( (dst), (src), (len))

/*------------------------------------------------------------------------------
//
// ZERO_MEMORY
//      Zeros out the given block of memory
//
//------------------------------------------------------------------------------
*/
#define ZERO_MEMORY( dst, len)      NdisZeroMemory( (dst),(len))


#else

/*------------------------------------------------------------------------------
//
// ALLOCATE_MEMORY
//      Allocates and zeros a block of nonpaged pool memory of the length
//      specified.  If the allocation attempt fails, the provided pointer
//      is set to NULL.
//
//------------------------------------------------------------------------------
*/
#define ALLOCATE_MEMORY( pptr, len)                                         \
{                                                                           \
    *(pptr) = ExAllocatePool( NonPagedPool,(len));                          \
    if (*(pptr) != NULL)                                                    \
    {                                                                       \
        RtlZeroMemory( *(pptr),(len));                                      \
    }                                                                       \
}

/*------------------------------------------------------------------------------
//
// FREE_MEMORY
//      Frees a previously allocated block of nonpaged pool memory of the
//      specified length.
//
//------------------------------------------------------------------------------
*/
#define FREE_MEMORY( ptr, len)                                              \
{                                                                           \
    RtlZeroMemory( (ptr), (len));                                           \
    ExFreePool( (ptr));                                                     \
}

/*------------------------------------------------------------------------------
//
// COMPARE_MEMORY
//      Compares two blocks of memory of the specified length and returns
//      the following:
//
//      < 0
//       buf1 less than buf2
//
//      0
//       buf1 identical to buf2
//
//      > 0
//       buf1 greater than buf2
//
//------------------------------------------------------------------------------
*/
#define COMPARE_MEMORY( src, dst, len)  memcmp( (src), (dst), (len))

/*------------------------------------------------------------------------------
//
// COPY_MEMORY
//      Copies len bytes from the src memory block to the dst memory block.
//
//------------------------------------------------------------------------------
*/
#define COPY_MEMORY( dst, src, len)     RtlMoveMemory( (dst), (src), (len))

/*------------------------------------------------------------------------------
//
// ZERO_MEMORY
//      Zeros out the given block of memory
//
//------------------------------------------------------------------------------
*/
#define ZERO_MEMORY( dst, len)      RtlZeroMemory( (dst),(len))

#endif /* _NDIS_ */

/*------------------------------------------------------------------------------
//
// FIELD_SIZE
//      Returns the size of the given field within the given structure type
//
//------------------------------------------------------------------------------
*/
#define FIELD_SIZE(type, field)     (sizeof (((type*) 0)->field))

/*------------------------------------------------------------------------------
//
// LIST_NODE
//      Data structure used to maintain doubly linked lists throughout the
//      driver
//
//------------------------------------------------------------------------------
*/
typedef struct  tLIST_NODE {
   struct tLIST_NODE * volatile Flink;
   struct tLIST_NODE * volatile Blink;
   void * lock;
} LIST_NODE, *PLIST_NODE;

/*------------------------------------------------------------------------------
//
// INITIALIZE_LIST_HEAD
//      Initializes the LIST_NODE structure at the head of a linked list.
//      Must be called prior to using any of the other list manipulation
//      macros.
//
//------------------------------------------------------------------------------
*/
#define INITIALIZE_LIST_HEAD(ListHead) (\
    (ListHead)->Flink = (ListHead)->Blink = (ListHead))

/*------------------------------------------------------------------------------
//
// IS_LIST_EMPTY
//      Returns TRUE if the given list is empty
//
//------------------------------------------------------------------------------
*/
#define IS_LIST_EMPTY(ListHead) \
    ((ListHead)->Flink == (ListHead))

/*------------------------------------------------------------------------------
//
// REMOVE_HEAD_LIST
//      Returns the entry at the head of the list.
//      Must NEVER be called with an empty list.  Use IS_LIST_EMPTY to determine
//      if the list is empty prior to calling this function.
//
//------------------------------------------------------------------------------
*/
#define REMOVE_HEAD_LIST(ListHead) \
    (ListHead)->Flink;\
    {REMOVE_ENTRY_LIST((ListHead)->Flink)}

/*------------------------------------------------------------------------------
//
// REMOVE_TAIL_LIST
//      Returns the entry at the tail of the list.
//      Must NEVER be called with an empty list.  Use IS_LIST_EMPTY to determine
//      if the list is empty prior to calling this function.
//
//------------------------------------------------------------------------------
*/
#define REMOVE_TAIL_LIST(ListHead) \
    (ListHead)->Blink;\
    {REMOVE_ENTRY_LIST((ListHead)->Blink)}

/*------------------------------------------------------------------------------
//
// REMOVE_ENTRY_LIST
//      Removes the given entry from the list
//
//------------------------------------------------------------------------------
*/
#define REMOVE_ENTRY_LIST(Entry) {\
    PLIST_NODE _EX_Blink;\
    PLIST_NODE _EX_Flink;\
    _EX_Flink = (Entry)->Flink;\
    _EX_Blink = (Entry)->Blink;\
    _EX_Blink->Flink = _EX_Flink;\
    _EX_Flink->Blink = _EX_Blink;\
    }

/*------------------------------------------------------------------------------
//
// INSERT_TAIL_LIST
//      Inserts the given entry at the tail of the list specified by ListHead
//
//------------------------------------------------------------------------------
*/
#define INSERT_TAIL_LIST(ListHead,Entry) {\
    PLIST_NODE _EX_Blink;\
    PLIST_NODE _EX_ListHead;\
    _EX_ListHead = (ListHead);\
    _EX_Blink = _EX_ListHead->Blink;\
    (Entry)->Flink = _EX_ListHead;\
    (Entry)->Blink = _EX_Blink;\
    _EX_Blink->Flink = (Entry);\
    _EX_ListHead->Blink = (Entry);\
    }

/*------------------------------------------------------------------------------
//
// INSERT_HEAD_LIST
//      Inserts the given entry at the head of the list specified by ListHead
//
//------------------------------------------------------------------------------
*/
#define INSERT_HEAD_LIST(ListHead,Entry) {\
    PLIST_NODE _EX_Flink;\
    PLIST_NODE _EX_ListHead;\
    _EX_ListHead = (ListHead);\
    _EX_Flink = _EX_ListHead->Flink;\
    (Entry)->Flink = _EX_Flink;\
    (Entry)->Blink = _EX_ListHead;\
    _EX_Flink->Blink = (Entry);\
    _EX_ListHead->Flink = (Entry);\
    }

/*------------------------------------------------------------------------------
//
// GET_FIRST_ENTRY_FROM_LIST
//      Returns the first entry in the list without removing the entry from
//      the list (if the list is empty, NULL is returned)
//
//------------------------------------------------------------------------------
*/
#define GET_FIRST_ENTRY_FROM_LIST( ListHead) \
    ((IS_LIST_EMPTY(ListHead)) ? NULL : ((ListHead)->Flink))

/*------------------------------------------------------------------------------
//
// GET_NEXT_ENTRY_FROM_LIST
//      Returns the entry in the list following the given entry.  If the given
//      entry is the last in the list, NULL is returned.
//
//------------------------------------------------------------------------------
*/
#define GET_NEXT_ENTRY_FROM_LIST( ListHead,Entry)      \
    (((((PLIST_NODE)(Entry))->Flink) == (ListHead)) ?    \
         NULL: (((PLIST_NODE)(Entry))->Flink))

/*------------------------------------------------------------------------------
//
// SINGLE_LIST_NODE
//      Data structure used to maintain singly linked lists throughout the
//      driver
//
//------------------------------------------------------------------------------
*/
typedef struct tSINGLE_LIST_NODE {
    struct tSINGLE_LIST_NODE *Next;
} SINGLE_LIST_NODE, *PSINGLE_LIST_NODE;

/*------------------------------------------------------------------------------
//
// POP_ENTRY_LIST
//      Pops the an entry from the head of a singly linked list
//
//------------------------------------------------------------------------------
*/
#define POP_ENTRY_LIST(ListHead) \
    (ListHead)->Next;\
    {\
        PSINGLE_LIST_NODE FirstEntry;\
        FirstEntry = (ListHead)->Next;\
        if (FirstEntry != NULL) {     \
            (ListHead)->Next = FirstEntry->Next;\
        }                             \
    }

/*------------------------------------------------------------------------------
//
// PUSH_ENTRY_LIST
//      Pushes an entry onto the head of a singly linked list
//
//------------------------------------------------------------------------------
*/
#define PUSH_ENTRY_LIST(ListHead,Entry) \
    (Entry)->Next = (ListHead)->Next; \
    (ListHead)->Next = (Entry)

#ifdef GTI_LITTLE_ENDIAN

/*------------------------------------------------------------------------------
//
// HTONS
//      Converts a short integer (two bytes) from host to network byte order
//
//------------------------------------------------------------------------------
*/
#ifndef HTONS
#define HTONS( _short ) \
    ( (((_short) & 0x00FF)<<8) | (((_short) & 0xFF00)>>8))
#endif

/*------------------------------------------------------------------------------
//
// HTONL
//      Converts a long integer (four bytes) from host to network byte order
//
//------------------------------------------------------------------------------
*/
#ifndef HTONL
#define HTONL( _long ) \
(((_long)&0x000000FF)<<24) | (((_long)&0x0000FF00)<<8) | \
(((_long)&0x00FF0000)>>8)  | (((_long)&0xFF000000)>>24)
#endif

/*------------------------------------------------------------------------------
//
// NTOHS
//      Converts a short integer (four bytes) from network to host byte order
//
//------------------------------------------------------------------------------
*/
#ifndef NTOHS
#define NTOHS( _short ) \
   ( (((_short) & 0x00FF)<<8) | (((_short) & 0xFF00)>>8))
#endif

/*------------------------------------------------------------------------------
//
// NTOHL
//      Converts a long integer (four bytes) from network to host byte order
//
//------------------------------------------------------------------------------
*/
#ifndef NTOHL
#define NTOHL( _long ) \
(((_long)&0x000000FF)<<24) | (((_long)&0x0000FF00)<<8) | \
(((_long)&0x00FF0000)>>8)  | (((_long)&0xFF000000)>>24)
#endif

#endif /* GTI_LITTLE_ENDIAN */

#ifdef GTI_BIG_ENDIAN

#define HTONS( _short ) (_short)
#define HTONL( _long ) (_long)
#define NTOHS( _short ) (_short)
#define NTOHL( _long ) (_long)

#endif /* GTI_BIG_ENDIAN */

#define WTOHL( _long )   ((_long)&0xFFFFFFFF)





/*------------------------------------------------------------------------------
//
// XFER_DATA_BUFFER
//      Describes the data buffer structure used to exchange data between
//      modules within the driver.  The AllocatorContext parameter is for the
//      exclusive use of the allocator of the data buffer and must not be
//      modified by any other entity.
//
//------------------------------------------------------------------------------
*/
typedef struct tDATA_BUFFER {
   SINGLE_LIST_NODE 	Link;
#if 0
   PUCHAR           	Data;
   ULONG            	Size;
#endif
   unsigned long*      	Data;
   unsigned long       	Size;
   PVOID            	AllocatorContext;
   BOOLEAN          	ArpFlag;
#if 0
//   DMA_DESCRIPTION	DmaData;
#endif
   void*		Dma;
} XFER_DATA_BUFFER, *PXFER_DATA_BUFFER;

/*------------------------------------------------------------------------------
//
// ANSI_STRING_CONST
//      Initializes an ANSI string constant
//
//------------------------------------------------------------------------------
*/
#define ANSI_STRING_CONST(x)    {sizeof(x)-1, sizeof(x), x}

/*------------------------------------------------------------------------------
//
// PARAM_NOT_USED
//      Used to mark a parameter as not used to avoid compiler warnings
//
//------------------------------------------------------------------------------
*/
#define PARAM_NOT_USED( x) (x) = (x);

typedef PUCHAR * PPUCHAR;

#if defined(MAC_OS) || defined(linux) || defined(__NetBSD__) || defined(__FreeBSD__) || defined(__OpenBSD__)

/*#include "drvh/GpDebug.h"*/
/*#include "debug.h"*/

#endif /* MAC_OS || linux */

#endif /* COMMON_H */
