/*+I*************************************************************************
 * GlobeSpan IOCTL Definitions
 *
 * Copyright (c) 2001 GlobeSpan Semiconductor Inc. -- http://www.globespan.net
 * All rights reserved.
 *
 * Developed for GlobeSpan Semiconductor Inc. by TriplePoint, Inc.
 *   http://www.triplepoint.com
 * PG added the support for read and write EEPROM 03/05/01
 * 
 *-I*************************************************************************/
#ifndef GSI_IOCTL_H_
#define GSI_IOCTL_H_


#define GSI_PROTOCOL_VERSION_MAJOR  1
#define GSI_PROTOCOL_VERSION_MINOR  0

#define GSI_IOCTL               SIOCDEVPRIVATE
#define   GSI_IOCTL_GET_VERSION   0x01
#define   GSI_IOCTL_GET_CONFIG    0x02
#define   GSI_IOCTL_SET_CONFIG    0x03
#define   GSI_IOCTL_DISCONNECT    0x04
#define   GSI_IOCTL_RETRAIN       0x05

/* added for reading and writing to EEPROM PG 03/05/01*/
#define	  GSI_IOCTL_READ_EEPROM   0x06
#define	  GSI_IOCTL_WRITE_EEPROM  0x07	

#define   GSI_IOCTL_SET_FILECONFIG       0x08 

#define GSI_IOCTL_SUCCESS   1
#define GSI_IOCTL_ERR_LEN   2
#define GSI_IOCTL_FAILURE   ~0

#ifndef PACKED
# define PACKED __attribute__((packed))
#endif /* PACKED */

/* IOCTL request structure */
struct gsi_req {
    unsigned char     command;    /* Command */
    unsigned char     result;     /* Command status */
    unsigned short    len;        /* Length in bytes of the buffer pointed to by data */
    void       *data;       /* Data buffer for additional information */
} PACKED;

/* LAN inteface IOCTL request structure */
struct gsi_lan_req {
    union {
        char            ifrn_name[32];
    }               ifr_ifrn;

    struct gsi_req  request;
} PACKED;

typedef struct {
    struct {
        unsigned char     major;
        unsigned char     minor;
    }           protocol;
    char        driver[16];
    char        firmware[8];
} PACKED gsi_version_t;


typedef struct {
  unsigned char     macAddr[6];
} PACKED PGSIDSL_PANDA_EEPROM;


typedef struct {
    unsigned char     major;
    unsigned char     minor;
} PACKED gsi_stats_t;

#define GSI_INTERFACE_LAN    (0x01 << 0)
#define GSI_INTERFACE_WAN    (0x01 << 1)


typedef struct {

    unsigned char     interface;
#if 1 
    unsigned char     EncapMode;
    unsigned short    Vci;
    unsigned short    Vpi;
#else
    unsigned char     rfc1483Mode;
    unsigned short    rfc1483Vci;
    unsigned short    rfc1483Vpi;
    unsigned char     rfc2364Mode;
    unsigned short    rfc2364Vci;
    unsigned short    rfc2364Vpi;
#endif    
    
    unsigned short    rxBufferCount;
    unsigned short    txBufferCount;

    unsigned short    gtiStandard;
    unsigned short    gtiTrellis;
    unsigned short    gtiTxPowerAtten;
    unsigned short    gtiCodingGain;
    unsigned short    gtiMaxBitsPerBin;
    unsigned short    gtiTxStartBin;
    unsigned short    gtiTxEndBin;
    unsigned short    gtiRxStartBin;
    unsigned short    gtiRxEndBin;
    unsigned short    gtiRxBinAdjust;
    unsigned short    gtiFramingStruct;
    unsigned short    gtiExpandedExchange;
    unsigned short    gtiClockType;
    unsigned short    gtiMaxDownRate;
} PACKED gsi_config_t;


#if defined (__KERNEL__) || defined (WAN_KERNEL)
#if DBG
extern const char *DbgIoctlCmd(unsigned int cmd);
extern const char *DbgGsiReq(struct gsi_req *pReq);
#endif /* DBG */
#endif /* __KERNEL__ */

#endif /* GSI_IOCTL_H_ */











