/*+I*************************************************************************
 * GlobeSpan PCI Definitions
 *
 * Copyright (c) 2000 GlobeSpan Semiconductor Inc. -- http://www.globespan.net
 * All rights reserved.
 *
 * Developed for GlobeSpan Semiconductor Inc. by TriplePoint, Inc.
 *   http://www.triplepoint.com
 *
 * History:
 *   2/9/01 PG: Added ENABLE READ and ENABLE_WRITE
 *-I*************************************************************************/
#ifndef GSI_PCI_H_
#define GSI_PCI_H_

#define GSI_PCI_MEMORY_SIZE         (8 * (4 * 2 * 1024))

#define GSI_MAC_EEPROM_OFFSET   0x10

#ifndef PCI_VENDOR_ID_GSI
#define PCI_VENDOR_ID_GSI           0x14BC
#endif /* PCI_VENDOR_ID_GSI */

#ifndef PCI_DEVICE_ID_GSI_PULSAR
#define PCI_DEVICE_ID_GSI_PULSAR    0xD002
#endif /* PCI_DEVICE_ID_GSI_PULSAR */

#ifndef PCI_DEVICE_ID_GSI_PANDA
#define PCI_DEVICE_ID_GSI_PANDA     0xD001
#endif /* PCI_DEVICE_ID_GSI_PANDA */

#ifdef GSI_PANDA
#define PCI_DEVICE_ID_GSI_ADSL      PCI_DEVICE_ID_GSI_PANDA
#else
#define PCI_DEVICE_ID_GSI_ADSL      PCI_DEVICE_ID_GSI_PULSAR
#endif

#define ALL_ZEROS_MASK              0x00000000
#define ALL_ONES_MASK               0xffffffff
#define PANDA_RESET_MASK            0x00000033   /* Resets the Panda chip*/
#define NOVA_RESET_MASK             0x00000002   /* Resets the Nova chip only*/

#define PCU_ADDR_RX_IDLE_CELLS		(0xBE)	/* PCU addr - Send ATM Idle Cells (HIBYTE=Mode, LOBYTE=Count)*/
#define PCU_RX_ALL_IDLE_CELLS		(1)	/* PCU mode - Pass all Idle Cells*/
#define PCU_RX_CNT_IDLE_CELLS		(0)	/* PCU mode - Pass only Count of Idle Cells*/

#define PANDA_INT_ENABLE_MASK       (DSLINT_PHY | \
                                     DSLINT_RX_CELLCNT | \
                                     DSLINT_TX_EOB)


/* For Pulsar, Glue Timing20 is the same as Panda Timing01, panda 4 range ---> pulsar 8 range*/
#define PANDA_GLUETIMING01_MASK     0x55550000   /* a magic number from the designers of Panda chip*/
#define PANDA_GPIOINPUTCONFG_MASK   0x55555555
#define PANDA_CLOCKCONFG_MASK       0x00006000   /* another important constant courtesy of Panda designers*/

#define PANDA_INITIAL_TXCFG         0x000000d1  /* single operation selected*/

/*#define PANDA_INITIAL_RXCFG1        0x00000320      // multi-operation selected, cellCount Interrupt enabled 100 cells */
#define PANDA_INITIAL_RXCFG1          0x00000140      /* multi-operation selected, cellCount Interrupt enabled 100 cells*/
#define PANDA_INITIAL_RXCFG2          0x00000A1D      /* watchdog timer 5ms*/
/*#define PANDA_INITIAL_RXCFG1        0x00000640      // multi-operation selected, cellCount Interrupt enabled 200 cells*/
/*#define PANDA_INITIAL_RXCFG2        0x0000142D      // watchdog timer 10ms*/
/*#define PANDA_INITIAL_RXCFG2        0x00000E2D      // watchdog timer 7ms*/
/*#define PANDA_INITIAL_RXCFG2        0x0000040D      // watchdog timer 2ms*/

#define PANDA_INITIAL_TXSTARTADD    0
#define PANDA_INITIAL_TXENDADD      0
#define PANDA_INITIAL_TXCOMMAND     0x00034006  /* TX DMA init/disabled, WordSize=13=1101*/

#define PANDA_INITIAL_RXSTARTADD    0
#define PANDA_INITIAL_RXENDADD      0
#define PANDA_INITIAL_RXCOMMAND     0x00034000  /* RX DMA init/disabled, WordSize=13=1101*/

#define PCU_CELLDEL_SYNC            0x1

typedef enum {
    SubSysDsl,
    SubSysVmodem,
    SubSysHpna
} subSys_t;


/*PG Enables READs and WRITEs for RAM 2/8/01*/
#define ENABLE_READ		0x80
#define ENABLE_WRTE		0x0

/* Make all the data access on a 32 bit boundary from the base offset */
#define _LOFF(b,o)  (b + (o << 2))

/****************************************************************************
 * Panda/Pulsar Registers
 ****************************************************************************/
/* Range 0 - Pulsar/Panda DSL Data (32bit) */
#define GSI_DSLDATA_OFFSET      0x0000
#define GSI_DSLDATA_TXCFG           _LOFF(GSI_DSLDATA_OFFSET, 0)
#define GSI_DSLDATA_TXSTARTADD      _LOFF(GSI_DSLDATA_OFFSET, 1)
#define GSI_DSLDATA_TXENDADD        _LOFF(GSI_DSLDATA_OFFSET, 2)
#define GSI_DSLDATA_TXCOMMAND       _LOFF(GSI_DSLDATA_OFFSET, 3)
#define GSI_DSLDATA_RXCFG1          _LOFF(GSI_DSLDATA_OFFSET, 4)
#define GSI_DSLDATA_RXCFG2          _LOFF(GSI_DSLDATA_OFFSET, 5)
#define GSI_DSLDATA_RXSTARTADD      _LOFF(GSI_DSLDATA_OFFSET, 6)
#define GSI_DSLDATA_RXENDADD        _LOFF(GSI_DSLDATA_OFFSET, 7)
#define GSI_DSLDATA_RXCOMMAND       _LOFF(GSI_DSLDATA_OFFSET, 8)

/* Range 1 - Pulsar Packet DMA (32bit) */
#define GSI_MACDMA_OFFSET       0x2000
#define GSI_MACDMA_TXCFGSTAT        _LOFF(GSI_MACDMA_OFFSET, 0)
#define GSI_MACDMA_TXSTARTADDR      _LOFF(GSI_MACDMA_OFFSET, 1)
#define GSI_MACDMA_TXPKTLEN         _LOFF(GSI_MACDMA_OFFSET, 2)
#define GSI_MACDMA_TXCMD            _LOFF(GSI_MACDMA_OFFSET, 3)
#define GSI_MACDMA_ACTIONS          _LOFF(GSI_MACDMA_OFFSET, 4)

#define GSI_MACDMA_RXCFGSTAT        _LOFF(GSI_MACDMA_OFFSET, 5)
#define GSI_MACDMA_RXSTARTADDR      _LOFF(GSI_MACDMA_OFFSET, 6)
#define GSI_MACDMA_RXMAXPKTLEN      _LOFF(GSI_MACDMA_OFFSET, 7)
#define GSI_MACDMA_RXCMD            _LOFF(GSI_MACDMA_OFFSET, 8)

#define GSI_MACDMA_COUNTER0         _LOFF(GSI_MACDMA_OFFSET, 10)
#define GSI_MACDMA_COUNTER1         _LOFF(GSI_MACDMA_OFFSET, 11)
#define GSI_MACDMA_COUNTERCFG       _LOFF(GSI_MACDMA_OFFSET, 12)

/* Range 2 - Pulsar/Panda DSL Control (8bit) */
#define GSI_DSLCTRL_OFFSET      0x4000

/* Range 3 - Pulsar MAC Control (8bit) */
#define GSI_MACCTRL_OFFSET      0x6000
#define GSI_MACCTRL_IND_CTL         _LOFF(GSI_MACCTRL_OFFSET, 0)
#define GSI_MACCTRL_BYTE_ENABLE     _LOFF(GSI_MACCTRL_OFFSET, 1)

/* Range 4 - Pulsar/Panda V.90 */
#define GSI_VMODEM_OFFSET       0x8000

/* Range 5 - Pulsar HomePNA */
#define GSI_HPNA_OFFSET         0xA000

/* Range 6 - Pulsar/Panda Glue (32bit) */
#define GSI_GLUE_OFFSET         0xC000
#define GSI_GLUE_CONFIG             _LOFF(GSI_GLUE_OFFSET, 0)
#define GSI_GLUE_TIMING01           _LOFF(GSI_GLUE_OFFSET, 1)
#define GSI_GLUE_TIMING23           _LOFF(GSI_GLUE_OFFSET, 2)
#define GSI_GLUE_RESETIMAGE         _LOFF(GSI_GLUE_OFFSET, 3)
#define GSI_GLUE_CLOCKSCONFIG       _LOFF(GSI_GLUE_OFFSET, 4)
#define GSI_GLUE_CLOCKSRRM1         _LOFF(GSI_GLUE_OFFSET, 5)
#define GSI_GLUE_CLOCKSRRM2         _LOFF(GSI_GLUE_OFFSET, 6)
#define GSI_GLUE_DSLSUBSYSTEMID     _LOFF(GSI_GLUE_OFFSET, 7)
#define GSI_GLUE_GPIOINPUTCFG1      _LOFF(GSI_GLUE_OFFSET, 9)
#define GSI_GLUE_GPIOOUTPUTCFG1     _LOFF(GSI_GLUE_OFFSET, 10)
#define GSI_GLUE_GPIOOUTPUTSEL1     _LOFF(GSI_GLUE_OFFSET, 11)
#define GSI_GLUE_GPIOOUTPUT1        _LOFF(GSI_GLUE_OFFSET, 12)
#define GSI_GLUE_GPIOINPUTCFG2      _LOFF(GSI_GLUE_OFFSET, 13)
#define GSI_GLUE_GPIOOUTPUTCFG2     _LOFF(GSI_GLUE_OFFSET, 14)
#define GSI_GLUE_GPIOOUTPUTSEL2     _LOFF(GSI_GLUE_OFFSET, 15)
#define GSI_GLUE_GPIOOUTPUT2        _LOFF(GSI_GLUE_OFFSET, 16)
#define GSI_GLUE_DSLINTENABLE       _LOFF(GSI_GLUE_OFFSET, 17)
#define GSI_GLUE_DSLINTCLEAR        _LOFF(GSI_GLUE_OFFSET, 18)
#define GSI_GLUE_VMODEMINTENABLE    _LOFF(GSI_GLUE_OFFSET, 19)
#define GSI_GLUE_VMODEMINTCLEAR     _LOFF(GSI_GLUE_OFFSET, 20)
#define GSI_GLUE_POWMGMTINTENABLE   _LOFF(GSI_GLUE_OFFSET, 21)
#define GSI_GLUE_POWMGMTINTCLEAR    _LOFF(GSI_GLUE_OFFSET, 22)
#define GSI_GLUE_PKTCLASSCODE       _LOFF(GSI_GLUE_OFFSET, 8)
#define GSI_GLUE_TIMING013          _LOFF(GSI_GLUE_OFFSET, 25)
#define GSI_GLUE_TIMING057          _LOFF(GSI_GLUE_OFFSET, 26)
#define GSI_GLUE_ARBITRPRI          _LOFF(GSI_GLUE_OFFSET, 28)

/* Read Registers */
#define GSI_DSLDATA_STATUS          GSI_DSLDATA_TXCFG
#define GSI_DSLDATA_RXCURRENTADD    GSI_DSLDATA_TXSTARTADD
#define GSI_GLUE_MISCREG            GSI_GLUE_CONFIG
#define GSI_GLUE_GPIOINPUTSTAT1     GSI_GLUE_TIMING01
#define GSI_GLUE_GPIOINPUTSTAT2     GSI_GLUE_TIMING23

/* For Pulsar, Int Stat register is the same as Int. Clear Register 0x48 */
#ifdef GSI_PULSAR
#  define GSI_GLUE_DSLINTSTAT       GSI_GLUE_DSLINTCLEAR
#  define GSI_GLUE_UNCOMMITTED_REG_OUT  _LOFF(GSI_GLUE_OFFSET, 27)
/*#  define PULSAR_HOSTSEL_MASK       0x00004000*/
#else /* PANDA */
#  define GSI_GLUE_DSLINTSTAT       GSI_GLUE_RESETIMAGE
#endif

/* Range 7 - Pulsar Extend Glue (32bit) */
#define GSI_EXTGLUE_OFFSET      0xE000
#define GSI_EXTGLUE_VCO_CONF_0      _LOFF(GSI_EXTGLUE_OFFSET, 0)
#define GSI_EXTGLUE_VCO_CONF_1      _LOFF(GSI_EXTGLUE_OFFSET, 1)
#define GSI_EXTGLUE_VCO_CONF_2      _LOFF(GSI_EXTGLUE_OFFSET, 2)
#define GSI_EXTGLUE_DMA_MUX_CONF    _LOFF(GSI_EXTGLUE_OFFSET, 3)


#define PANDA_ADDR_POWMGMTCTLSTAT   6

/* GSI_GLUE_DSLINTSTAT/GSI_GLUE_DSLINTCLEAR bit definitions */
#define DSLINT_TX_EOB       (0x01 << 0)
#define DSLINT_RESERVED_0   (0x01 << 1)
#define DSLINT_GPIO_0       (0x01 << 2)
#define DSLINT_GPIO_1       (0x01 << 3)
#define DSLINT_RX_EOB       (0x01 << 4)
#define DSLINT_RX_CELLCNT   (0x01 << 5)
#define DSLINT_RX_WDTIMER   (0x01 << 6)
#define DSLINT_RESERVED_7   (0x01 << 7)
#define DSLINT_PHY          (0x01 << 8)
#define DSLINT_PCI_BUSY     (0x01 << 9)
#define DSLINT_GPIO_6       (0x01 << 10)
#define DSLINT_GPIO_7       (0x01 << 11)
#define DSLINT_GPIO_8       (0x01 << 12)
#define DSLINT_GPIO_16      (0x01 << 13)
#define DSLINT_GPIO_17      (0x01 << 14)
#define DSLINT_GPIO_18      (0x01 << 15)

#define INT_CLEAR_MASK \
    (DSLINT_RX_EOB | DSLINT_RX_CELLCNT | DSLINT_GPIO_6 | \
     DSLINT_GPIO_7 | DSLINT_GPIO_17 | DSLINT_GPIO_18)

#ifdef NOT_USED
/****************************************************************************
 * Configuration Register
 ****************************************************************************/
#define CFG_PHY_INT_ENA         (0x01 << 0)   /* phy interrupt enable*/
#define CFG_TX_FULL_INT_ENA     (0x01 << 1)   /* transmit status full interrupt enable*/
#define CFG_UTOPIA_MODE         (0x01 << 2)   /* utopia mode select*/
#define CFG_TX_UNDER_INT_ENA    (0x01 << 3)   /* transmit under-run interrupt enable*/
#define CFG_TX_INT_ENA          (0x01 << 4)   /* transmit interrupt enable*/
#define CFG_TX_ENA              (0x01 << 5)   /* transmit operation enable*/
#define CFG_TIMER_ROLL_INT_ENA  (0x01 << 7)   /* timer roll over interrupt enable*/
#define CFG_LOOPBACK            (0x01 << 8)   /* loop back enable*/
#define CFG_RX_RM               (0x01 << 9)   /* receive RM cells*/
#define CFG_Q_FULL_INT_ENA      (0x01 << 10)  /* receive queue almost full interrrupt e*/
#define CFG_RAW_Q_INT_ENA       (0x01 << 11)  /* raw cell queue interrupt enable*/
#define CFG_RX_END_PDU_INT_ENA  (0x07 << 12)  /* end of PDU handling*/
#define CFG_VPI_VCI_ERR_ACCEPT  (0x01 << 15)  /* vpi/vci error cell accept*/
#define CFG_RX_CONNECT_TABLE_SZ (0x03 << 16)  /* receive connection table size (00-4k,*/
#define CFG_VPI_VCI_SELECT      (0x03 << 18)  /* vpi/vci select based on the rxcns size*/
#define CFG_IGNORE_GFC          (0x01 << 20)  /* ignore GFC*/
#define CFG_INVALID_CELL_ACCEPT (0x01 << 21)  /* invalid cell accept*/
#define CFG_RX_Q_SIZE           (0x03 << 22)  /* receive status queue size;*/
#define CFG_EMPTY_FBQ_INT_ENA   (0x01 << 24)  /* empty free buffer queue interrupt enab*/
#define CFG_LARGE_BUF_SIZE      (0x03 << 25)  /* large buf size*/
#define CFG_SMALL_BUF_SIZE      (0x03 << 27)  /* small buf size*/
#define CFG_RX_ENA              (0x01 << 29)  /* receive path enable*/
#define CFG_SW_RESET            (0x01 << 31)  /* software reset*/

/****************************************************************************
 * Status Register
 ****************************************************************************/
#define STAT_RESERVED_0         (0x01 << 0)
#define STAT_RX_STAT_Q_AFULL    (0x01 << 1)   /* receive status queue almost full*/
#define STAT_LARGE_BUF_Q_EMPTY  (0x01 << 2)   /* large buffer queue empty*/
#define STAT_SMALL_BUF_Q_EMPTY  (0x01 << 3)   /* small buffer queue empty*/
#define STAT_RAW_CELL_FLAG      (0x01 << 4)   /* raw cell flag*/
#define STAT_END_PDU_FLAG       (0x01 << 5)   /* end of PDU flag*/
#define STAT_RX_STAT_Q_FULL     (0x01 << 6)   /* receive status queue full*/
#define STAT_LARGE_BUF_Q_FULL   (0x01 << 7)   /* large buffer queue full*/
#define STAT_SMALL_BUF_Q_FULL   (0x01 << 8)   /* small buffer queue full*/
#define STAT_COMMAND_BUSY_FLAG  (0x01 << 9)   /* command busy flag*/
#define STAT_PHY_INT_FLAG       (0x01 << 10)  /* PHY interrupt flag*/
#define STAT_TIMER_OVERFLOW     (0x01 << 11)  /* timer overflow*/
#define STAT_TX_STAT_Q_FULL     (0x01 << 12)  /* transmit status queue full*/
#define STAT_RESERVED_13        (0x01 << 13)
#define STAT_TX_INCOMP_PDU      (0x01 << 14)  /* transmit incomplete PDU*/
#define STAT_TX_STATUS          (0x01 << 15)  /* transmit status indicator*/
#define STAT_LARGE_BUF_COUNT    (0xFF << 16)  /* large buffer count*/
#define STAT_SMALL_BUF_COUNT    (0xFF << 24)  /* small buffer count*/
#endif /* NOT_USED */

extern void GpDspClearInterrupts(adapter_t *pAdapter, GTI_BYTE id);

extern void gsi_init(adapter_t *pAdapter);
extern void gsi_disconnect(adapter_t *pAdapter);
extern void gsi_restart(adapter_t *pAdapter);
extern void gsi_shutdown(adapter_t *pAdapter);
extern BOOLEAN GpatmisIoctl(adapter_t *pAdapter, int cmd,
    struct gsi_req *arg, int *pResult);

extern void gsi_pci_write(adapter_t *pAdapter, u_int16_t offset, u_int32_t data);
extern u_int32_t gsi_pci_read(adapter_t *pAdapter, u_int16_t offset);
extern u_int8_t gsi_phy_read(adapter_t *pAdapter, subSys_t subSys, u_int8_t offset);
extern void gsi_phy_write(adapter_t *pAdapter, subSys_t subSys, u_int8_t offset, u_int8_t data);

/*extern void GsiUpdateSNRStatus(adapter_t  *pAdapter); //PG 2/9/01*/

extern void gsi_dma_init(adapter_t *pAdapter);
extern void gsi_dma_tx_start(adapter_t *pAdapter);
extern void gsi_dma_tx_complete(adapter_t *pAdapter);
extern void gsi_dma_rx_start(adapter_t *pAdapter);
extern void gsi_dma_rx_complete(adapter_t *pAdapter);


#endif /* GSI_PCI_H_ */

