/*+I*************************************************************************
 * GlobeSpan PCI PHY Interface
 *
 * Copyright (c) 2000 GlobeSpan Semiconductor Inc. -- http://www.globespan.net
 * All rights reserved.
 *
 * Developed for GlobeSpan Semiconductor Inc. by TriplePoint, Inc.
 *   http://www.triplepoint.com
 *-I*************************************************************************/
#ifndef PCI_PHY_H_
#define PCI_PHY_H_

#include "drvh/GpSnull.h"
#include "atmphy.h"

#define PHY_REMOVE_LIST(ListHead) \
    ((IS_LIST_EMPTY(ListHead)) ? NULL : ((ListHead)->Flink)); \
    if (!IS_LIST_EMPTY(ListHead)) REMOVE_ENTRY_LIST((ListHead)->Flink)

#define PHY_ADD_LIST(ListHead, Entry) \
    INSERT_TAIL_LIST(ListHead, (PLIST_NODE) Entry)

typedef struct {
    LIST_NODE           Link;
    PXFER_DATA_BUFFER   Buffer;
} PHY_DESCR, *PPHY_DESCR;

/*----------------------------------------------------------------------------
// PCI_PHY_STATE
//   Defines the possible states that a PCI Phy can take on
//----------------------------------------------------------------------------
*/
typedef enum {
    PCI_PHY_STATE_HALTED,
    PCI_PHY_STATE_OPERATIONAL
} PCI_PHY_STATE;

/*----------------------------------------------------------------------------
// PCI_PHY
//   Describes an instance of the PCI Phy
//----------------------------------------------------------------------------
*/
typedef struct tPCI_PHY {
    PVOID                       AdapterHandle;

    PCI_PHY_STATE               State;
    ATM_PHY_USER_HANDLE         UserHandle;
    ATM_PHY_USER_RECEIVE        UserReceive;
    ATM_PHY_USER_SEND_COMPLETE  UserSendComplete;
    ATM_PHY_USER_LINK_STATUS_CHANGE UserLinkStatusChange;

    USHORT                      TxDescrCount;
    USHORT                      RxDescrCount;
    USHORT                      RxBufferSize;

/*    PVOID	                TxSLock; */
    PPHY_DESCR                  TxDescr;
    
    LIST_NODE                   TxFreeList;
    LIST_NODE                   TxPendingList;
    LIST_NODE                   TxCompleteList;

/*    PVOID	              	RxSLock; */
    PVOID			Dma;
    struct {
 	unsigned long*		vAddr;
        unsigned long		pAddr;
        unsigned long		length;
    }                           RxBuffer;
    unsigned long               RxDataHead;
    LIST_NODE                   RxFreeList;
    LIST_NODE                   RxCompleteList;
} PCI_PHY, *PPCI_PHY;

/*----------------------------------------------------------------------------
// Function Declarations
//----------------------------------------------------------------------------
*/
extern ATM_PHY_STATUS PciPhyBind(PATM_PHY_BIND_INPUT pBindInput,
    PATM_PHY_BIND_OUTPUT pBindOutput);

extern void gsi_phy_send_complete(PCI_PHY *pPciPhy);
extern void gsi_phy_receive_complete(PCI_PHY *pPciPhy);

#endif /* PCI_PHY_H_ */

