/*****************************************************************************
* wanpipe_atm.h	WANPIPE(tm) ATM ALL5 SAR Header
* 		
* Authors: 	Nenad Corbic <ncorbic@sangoma.com>
*
* Copyright:	(c) 2003 Sangoma Technologies Inc.
*
*		This program is free software; you can redistribute it and/or
*		modify it under the terms of the GNU General Public License
*		as published by the Free Software Foundation; either version
*		2 of the License, or (at your option) any later version.
* ============================================================================
* Jan 07, 2002	Nenad Corbic	Based on ADSL ATM SAR
*****************************************************************************/

#ifndef __WANPIPE_ATM_H_
#define __WANPIPE_ATM_H_

#ifndef NULL
#define NULL ((void *)0)
#endif

#define WPVPI(cell)	(   (((unsigned char*)(cell))[0]<<4)| \
			    ((((unsigned char*)(cell))[1]&0xF0)>>4) ) 

#define WPVCI(cell)       ( ((((unsigned char*)(cell))[1]&0x0F)<<12) |\
	                    (((unsigned char*)(cell))[2]<<4) | \
	  		    ((((unsigned char*)(cell))[3]&0xF0)>>4) )

#define WPATMHASH_SZ 		(4096 >> 2)
#define vpivci_hashfn(x)	((((x) >> 8) ^ (x)) & (WPATMHASH_SZ - 1))
#define ATM_HEADER_MASK  	0x0F000000
	

#define ATM_CELL_SIZE		53

#define ATM_HEADER_SZ 		5
#define ATM_PAYLOAD_SZ		48
#define ATM_PROT_HEADER_SZ	15


#define INITIAL_FCS_VALUE 	0xFFFFFFFF
#define EIGHT_BITS 	  	8

#define FLIP_CRC(a,b)  { b=0; \
			 b |= (a&0x000000FF)<<24 ; \
			 b |= (a&0x0000FF00)<<8; \
			 b |= (a&0x00FF0000)>>8; \
			 b |= (a&0xFF000000)>>24; }

#define CLP_BIT		0
#define PT_SDU1		1
#define PT_CONGESTION 	2			 
#define PT_ADMIN	3

#define PAD_CHAR	0x6A
#define ATM_OVERHEAD    1.1

#define WPATM_MEM_ASSERT()  wpabs_debug_event("%s:%d: No Memory!\n",__FUNCTION__,__LINE__)

#define MAX_TX_PROT_LIST 100
#define MAX_RX_DATA_LIST 100
			 
typedef struct wp_atm_reg
{
	void 		*card;
	void 		*atm_task;	
	unsigned char	*devname;
	unsigned char	atm_task_running;
	unsigned char	init;

	void		*rx_data_list;
	void		*tx_prot_list;

	void		*trace_info;

	struct wp_atm_reg	*next;
	void 		*atm_hash[WPATMHASH_SZ];

}wp_atm_reg_t;


#define	MAX_ATM_ADDR	20
#define ATM_AIS_DATA_SZ	17
typedef struct wp_sar_dev{
	
	unsigned char  	init;
	void*		priv;
	void*		dev;
	void* 		tx_cells_skb;
	void* 		rx_pkt_skb;
	void*		tx_arp_skb;
	char* 		devname;

	wp_atm_reg_t*	atmdevice;

	unsigned short vpi; 
	unsigned short vci;
	unsigned char  pt;
	unsigned char  hec;
	unsigned char  hec_eof;

	unsigned char  encap_mode;
#define EncapMode      encap_mode

	unsigned char  atm_header[ATM_HEADER_SZ];
	unsigned char  prot_header[ATM_PROT_HEADER_SZ];
	unsigned char  prot_header_size;
	unsigned char  trailer[8];
        unsigned char  pad_len_buf[51];

        unsigned char  atm_addr[MAX_ATM_ADDR];
        atm_stats_t    atm_stats;
    	unsigned long  timeout;

	unsigned short max_sdu_size;

	/* Hash pointers used to implement
	 * VPI/VCI to Device mapping */
	struct wp_sar_dev **hash_pprev;
	struct wp_sar_dev *hash_next;

	wan_atm_conf_if_t cfg;

	unsigned char	state;
	unsigned long	ais_timeout;
	unsigned char	ais_data[ATM_AIS_DATA_SZ];
}wp_sar_dev_t;


#define atm_vpi(hdr)	((hdr).vpi_hi<<4|(hdr).vpi_lo)
#define atm_vci(hdr)	(((hdr).vpi_hi<<12)|((hdr).vci_mid<<4)|(hdr).vci_lo)

#define atm_crc10(data)		(((data).u.mgmt.crc_hi<<8)|(data).u.mgmt.crc_lo)

#define set_atm_crc10(data,val)	(data).u.mgmt.crc_hi=((val>>8)&0x3);\
		                (data).u.mgmt.crc_lo=(val&0xFF); \
				(data).u.mgmt.reserved=0;


#pragma pack(1)
typedef struct wp_atm_header
{
	unsigned char vpi_hi;
	unsigned char vpi_lo:4;
	unsigned char vci_hi:4;
	unsigned char vci_mid;
	unsigned char clp:1;
	unsigned char pt:3;
	unsigned char vci_lo:4;
	unsigned char hec;
}wp_atm_header_t;

typedef struct wp_atm_payload
{
	union {
		struct {
			unsigned char  info[40];
			unsigned short reserved;
			unsigned short len;
			unsigned int   crc;
		}tail;
		struct {
			unsigned char  info[46];
			unsigned char  crc_hi:2;
			unsigned char  reserved:6;
			unsigned char  crc_lo;
		}mgmt;
		unsigned char info[ATM_PAYLOAD_SZ];
			
	}u;
}wp_atm_payload_t;


typedef struct wp_atm_cell_1
{
	wp_atm_header_t 	hdr;
	wp_atm_payload_t 	data;
}wp_atm_cell_1_t;

typedef struct wp_atm_cell
{
  unsigned char hdr[ATM_HEADER_SZ];      /* ATM header */
  unsigned char info[ATM_PAYLOAD_SZ];    /* ATM payload */
} wp_atm_cell_t;

#pragma pack()


typedef struct atmarp {
	unsigned short arphrd 	; /* 16 bits  Hardware type */
        unsigned short arppro 	; /* 16 bits  Protocol type */
        unsigned char  arpshtl 	; /* 8 bits  Type & length of source ATM number (q) */
        unsigned char  arpsstl 	; /* 8 bits  Type & length of source ATM subaddress (r) */
        unsigned short arpop   	; /* 16 bits  Operation code (request, reply, or NAK) */
        unsigned char  arpspln 	; /* 8 bits  Length of source protocol address (s=4) */
        unsigned char  arpthtl 	; /* 8 bits  Type & length of target ATM number (x) */
        unsigned char  arptstl 	; /* 8 bits  Type & length of target ATM subaddress (y) */
        unsigned char  arptpln 	; /* 8 bits  Length of target protocol address (z=4) */
}atmarp_t;

typedef	struct arphdr {
	unsigned short	ar_hrd;		/* format of hardware address	*/
	unsigned short	ar_pro;		/* format of protocol address	*/
	unsigned char	ar_hln;		/* length of hardware address	*/
	unsigned char	ar_pln;		/* length of protocol address	*/
	unsigned short	ar_op;		/* ARP opcode (command)		*/
}arphdr_t;

#define ATMARP_HEADER  0x1300
#define ATMARP_PROT_IP 0x0008

#define ARP_REQUEST   0x0100
#define ARP_REPLY     0x0200
#define InARP_REQUEST 0x0800
#define InARP_REPLY   0x0900
#define ARP_NAK       0x1000

#define MAX_ARP_SIZE  500

enum {
	INIT,
	SAR_TX,
	ATM_ARP
};


#define ATM_ASSERT(val) if (val){								\
			wpabs_debug_event("**************** ASSERT FAILED ****************\n");	\
			wpabs_debug_event("%s:%d - Critical error\n",__FILE__,__LINE__);		\
			return -EINVAL;								\
			}

#define ATM_ASSERT1(val) if (val){								\
			wpabs_debug_event("**************** ASSERT FAILED ****************\n");	\
			wpabs_debug_event("%s:%d - Critical error\n",__FILE__,__LINE__);		\
			return;									\
			}

#define ATM_ASSERT2(val, ret) if (val){								\
			wpabs_debug_event("**************** ASSERT FAILED ****************\n");	\
			wpabs_debug_event("%s:%d - Critical error\n",__FILE__,__LINE__);		\
			return ret;								\
			}


#endif
