

%{

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>

#include "aft_main.h"
#include "aft_log.h"
#include "aft_cmd.h"

#define END(v)		(v-1+sizeof(v)/sizeof(v[0]))
#define YY_NO_UNPUT
#undef YY_INPUT
#define YY_INPUT(buf,result,max_size)			\
	{						\
		char c;				\
		if (targv < arglim){			\
			c = targv[0][offset++];		\
			if (c  != '\0'){		\
				buf[0] = c;		\
			}else{				\
				targv++;		\
				offset = 0;		\
				buf[0] = ' ';		\
			}				\
			result = 1;			\
		}else{					\
			result = 0;			\
		}					\
	}

static char** targv;
static char**arglim;
static unsigned offset = 0;

static int aft_cmd_read_word(char* str);
static int aft_cmd_read_num(void);
static int aft_card_type(void);

static sdlaprod_t*	g_sdlacfg;

%}



%%

-type		{ aft_card_type(); }

-card		{ g_sdlacfg->cards_no = aft_cmd_read_num(); }

-txsize		{ g_sdlacfg->txsize = aft_cmd_read_num(); }

-txcnt		{ g_sdlacfg->txcnt = aft_cmd_read_num(); }

-get		{ g_sdlacfg->get_flash_code = 1; }

-boot		{ g_sdlacfg->prg_boot_sector = 1; }

-update		{ 
			if (g_sdlacfg->no_update){
				aft_log(AFT_LOG_INFO, "WARNING: Can't use both UPDATE and NOUPDATE command argument!\n");
			}
			g_sdlacfg->no_update = 0; 
			g_sdlacfg->update = 1; 
		}

-noupdate	{
			if (g_sdlacfg->update){
				aft_log(AFT_LOG_INFO, "WARNING: Can't use both UPDATE and NOTUPDATE command argument!\n");
			}
			g_sdlacfg->update = 0; 
			g_sdlacfg->no_update = 1;
		}

-verify		{ g_sdlacfg->only_verify = 1; }

-v		{ aft_version(); return 1; }

-h		{ aft_cmd_help(); return 1; }

%%

int yywrap(void)
{
	return 1;
}

int aft_cmd_read_word(char* str)
{
	int i = 0;

	while((str[i] = input()) == ' ');
	while(str[i] != EOF && str[i] != ' ' && str[i] != '\n' && str[i] != '\0'){
		str[++i] = input();
	}
	if (i == 0){
		return 0;
	}
	str[i] = '\0';
	return i;
}

int aft_cmd_read_num(void)
{
	char str[100];
	int i = 0;

	while((str[i] = input()) == ' ');
	while(str[i] != EOF && str[i] != ' ' && str[i] != '\n' && str[i] != '\0'){
		str[++i] = input();
	}
	if (i == 0){
		return 0;
	}
	str[i] = '\0';
	return strtoul(str, NULL, 10);
}

static int aft_card_type()
{
	char str[100];

	if (!aft_cmd_read_word(str)){
		return -EINVAL;
	}
	g_sdlacfg->card_type = aft_get_type(str);
	g_sdlacfg->te_lines_no = 
		(g_sdlacfg->card_type == A101_ADPTR_1TE1) ? 1 :
		(g_sdlacfg->card_type == A101_ADPTR_2TE1) ? 2 : 0;
	return 0;
}

int aft_cmd_parse(int argc, char* argv[], sdlaprod_t* sdlacfg)
{
	targv = argv+1;
	arglim = argv+argc;
	g_sdlacfg = sdlacfg;
	yylex();
}

int aft_cmd_help(void)
{
	aft_version();
	printf("\nUsage: aft_prod [-h] [-v ] [-card <num> -type [a101|a102]] [options]\n");
	printf("\t-v\t\t\tPrint program version\n");
	printf("\t-h\t\t\tPrint this help\n");
	printf("\t-card <num>\t\tTotal number of Sangoma card\n");
	printf("\t-type [a101|a102]\tSangoma card type\n");
	printf("\nwhere options are:\n");
	printf("\t-get\t\t\tGet Flash code from the server\n");
	printf("\t-boot\t\t\tUpdate Default sector in Xilinx Flash\n");
	printf("\t-update\t\t\tForce to Update Xilinx Flash\n");
	printf("\t-noupdate\t\tDo not Update Xilinx Flash\n");
	printf("\t-txsize <num>\t\tPacket size for production test\n");
	printf("\t-txcnt <num>\t\tNumber of packet for production test\n");
	printf("\t-verify\t\t\tRun Xilinx Flash verifycation only\n");
	printf("\t\t\t\t(if Xilinx Flash has old version, program will\n");
	printf("\t\t\t\tre-program Xilinx Flash and then verify it)\n");
	printf("\n");
	return 0;
}
