#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
#include <unistd.h>
#include <sched.h>
#include <sys/ipc.h>
#include <sys/sem.h>
#include <sys/time.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <string.h>
#include <errno.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <signal.h>
#include <linux/if_wanpipe.h>
#include "aft_log.h"
#include "aft_main.h"
#include "aft_ioctl.h"
#include "aft_flash.h"

/*
******************************************************************************
**			DEFINES/MACROS				
******************************************************************************
*/
#define JP8_VALUE               0x02
#define JP7_VALUE               0x01
#define SW0_VALUE               0x04
#define SW1_VALUE               0x08   

#define WRITE_DEF_SECTOR_DSBL   0x01
#define FRONT_END_TYPE_MASK     0x38

#define BIT_DEV_ADDR_CLEAR	0x600
#define BIT_DEV_ADDR_CPLD	0x200

#define MEMORY_TYPE_SRAM	0x00
#define MEMORY_TYPE_FLASH	0x01
#define MASK_MEMORY_TYPE_SRAM	0x10
#define MASK_MEMORY_TYPE_FLASH	0x20

#define MASK_DEF_SECTOR_FLASH	0x00
#define MASK_USER_SECTOR_FLASH	0x04

#define BIT_A18_SECTOR_SA4_SA7	0x20
#define USER_SECTOR_START_ADDR	0x40000

#define FLASH_VER_ADDR		0x3FFFF

#define SECTOR_FLASH_DECODE(sector_type)				\
		(sector_type == USER_SECTOR_FLASH) ? "USER" :		\
		(sector_type == DEF_SECTOR_FLASH) ? "BOOT" : "UNKNOWN"
		
/*
******************************************************************************
**			TYPEDEF/STRUCTURES				
******************************************************************************
*/


/*
******************************************************************************
**			GLOBAL VARIABLES				
******************************************************************************
*/
unsigned int	tmp_data[MAX_DATA_SIZE], tmp_data1[MAX_DATA_SIZE];
extern int verbose;
/*
******************************************************************************
**			FUNCTION PROTOTYPES				
******************************************************************************
*/

/*
******************************************************************************
**			FUNCTION DEFINITIONS				
******************************************************************************
*/


static unsigned long filesize(FILE* f)
{
	unsigned long size = 0;
	unsigned long cur_pos = 0;

	cur_pos = ftell(f);
	if ((cur_pos != -1l) && !fseek(f, 0, SEEK_END)){
		size = ftell(f);
		fseek(f, cur_pos, SEEK_SET);
	}
	return size;
}

static long read_data_file(char* filename, unsigned char** data)
{
	FILE* f = NULL;
	unsigned char* buf = NULL;
	unsigned long fsize = 0;

	f = fopen(filename, "rb");
	if (f == NULL){
		aft_log(AFT_LOG_ERR, "read_data_file: Can't open data file %s!\n",
					filename);
		return -EINVAL;
	}
	fsize = filesize(f);
	if (!fsize){
		aft_log(AFT_LOG_ERR, "read_data_file: Data file %s is empty!\n",
					filename);
		return -EINVAL;
	}
	buf = malloc(fsize);
	if (buf == NULL){
		aft_log(AFT_LOG_ERR, "read_data_file: Can't allocate memory for data!\n");
		return -ENOMEM;
	}
	if (fread(buf, 1, fsize, f) < fsize){
		aft_log(AFT_LOG_ERR, "read_data_file: Can't copy all data to local buffer!\n");
		free(buf);
		return -EINVAL;
	}
	*data = buf;
	fclose(f);
	return fsize;
	
}

unsigned int write_cpld(netif_t* netif, unsigned short cpld_off,unsigned short cpld_data)
{
	cpld_off &= ~BIT_DEV_ADDR_CLEAR; 
	cpld_off |= BIT_DEV_ADDR_CPLD; 
	exec_write_cmd(netif, 0x46, 2, cpld_off);
	exec_write_cmd(netif, 0x44, 2, cpld_data);
	return 0;
}

unsigned int read_cpld(netif_t* netif, unsigned short cpld_off, unsigned int* cpld_data)
{
	cpld_off &= ~BIT_DEV_ADDR_CLEAR; 
	cpld_off |= BIT_DEV_ADDR_CPLD; 
	exec_write_cmd(netif, 0x46, 2, cpld_off);
	if (exec_read_cmd(netif, 0x44, 4, cpld_data)){
		return -EINVAL;
	}
	return 0;
}

unsigned int write_flash(netif_t* netif, int sector_type, int memory_type, unsigned long flash_off,unsigned char flash_data)
{
	unsigned char	offset;

	//Writing flash address to cpld
	offset = flash_off & 0xFF;
	write_cpld(netif, 0x05, offset);
	offset = (flash_off >> 8) & 0xFF;
	write_cpld(netif, 0x06, offset);
	offset = (flash_off >> 16) & 0x3;
	if (memory_type == MEMORY_TYPE_SRAM){
		offset |= MASK_MEMORY_TYPE_SRAM;
	}else if (memory_type == MEMORY_TYPE_FLASH){
		offset |= MASK_MEMORY_TYPE_FLASH;
	}else{
		return -EINVAL;
	}
	if (sector_type == USER_SECTOR_FLASH){
		offset |= MASK_USER_SECTOR_FLASH;
	}else if (sector_type == DEF_SECTOR_FLASH){
		;
	}else{
		return -EINVAL;
	}
	write_cpld(netif, 0x07, offset);
	write_cpld(netif, 0x04, flash_data);
        write_cpld(netif, 0x07, 0x00);  // disable CS signal for the Boot FLASH/SRAM

	return 0;
}

int read_flash(netif_t* netif, int sector_type, int memory_type, unsigned long flash_off, unsigned int* flash_data)
//unsigned char read_flash(netif_t* netif, int sector_type, int memory_type, unsigned long flash_off, unsigned char* data)
{
	unsigned char	offset;
	int		err;

	//Writing flash address to cpld
	offset = flash_off & 0xFF;
	write_cpld(netif, 0x05, offset);
	offset = (flash_off >> 8) & 0xFF;
	write_cpld(netif, 0x06, offset);
	offset = (flash_off >> 16) & 0x3;
        offset |= MASK_MEMORY_TYPE_FLASH;
//           
//	if (memory_type == MEMORY_TYPE_SRAM){
//		offset |= MASK_MEMORY_TYPE_SRAM;
//	}else if (memory_type == MEMORY_TYPE_FLASH){
//		offset |= MASK_MEMORY_TYPE_FLASH;
//	}else{
//		return -EINVAL;
//	}
	if (sector_type == USER_SECTOR_FLASH){
		offset |= MASK_USER_SECTOR_FLASH;
	}else if (sector_type == DEF_SECTOR_FLASH){
		;
	}else{
		return -EINVAL;
	}

	write_cpld(netif, 0x07, offset);
	if ((err = read_cpld(netif, 0x04, flash_data))){
		aft_log(AFT_LOG_ERR, "Failed to read flash (offset=0x%X)\n",
				offset);
		return err;
	}

        write_cpld(netif, 0x07, 0x00); // Disable CS for the Boot FLASH/SRAM
	return 0;
}

#if 0
static int verify_write_sector_flash(netif_t* netif, int sector_type)
{
	unsigned char	val;
	int err;

        if(sector_type == DEF_SECTOR_FLASH){
		// Checking write enable to the Default Boot Flash Sector 
		err = read_cpld(netif, 0x09, (unsigned int*)&val);
		if (val & WRITE_DEF_SECTOR_DSBL){
			aft_log(AFT_LOG_ERR, "erase_sector: Default sector protected!\n");
			return -EINVAL;
		}
	}    

	return 0;
}
#endif

static int erase_sector_flash(netif_t* netif, int cardnum, int sector_type)
{
	unsigned long	offset = 0x00;
	int		sector_no = 0;
	int		cnt=0, ready_cnt = 0, err = 0, i = 0;

#if 0
	if (verify_write_sector_flash(netif, sector_type)){
		aft_log(AFT_LOG_ERR, "erase_sector: Default sector protected!\n");
		return -EINVAL;
	}    
#endif
	if (sector_type == USER_SECTOR_FLASH){
                offset  = USER_SECTOR_START_ADDR;
       	}
	for(sector_no = 0; sector_no < 4; sector_no++){
                offset += (sector_no << 16); 
		write_flash(netif, sector_type, MEMORY_TYPE_FLASH, 0x5555, 0xAA);
		write_flash(netif, sector_type, MEMORY_TYPE_FLASH, 0x2AAA, 0x55);
		write_flash(netif, sector_type, MEMORY_TYPE_FLASH, 0x5555, 0x80);
		write_flash(netif, sector_type, MEMORY_TYPE_FLASH, 0x5555, 0xAA);
		write_flash(netif, sector_type, MEMORY_TYPE_FLASH, 0x2AAA, 0x55);
		write_flash(netif, sector_type, MEMORY_TYPE_FLASH, offset, 0x30);
		do{
//MF			for(i=0;i<100000;i++);
			ready_cnt = 0;
			err = read_flash(netif, sector_type, MEMORY_TYPE_FLASH, offset, tmp_data);
			if (err){
				return -EINVAL;
			}
			for(i = 0; i < cardnum; i++){
				if (tmp_data[i] & 0x80){
					ready_cnt++;
				}else if (tmp_data[i] & 0x20){
					err = read_flash(netif, 
							  sector_type, 
							  MEMORY_TYPE_FLASH, 
							  offset,
							  tmp_data1);
					if (err){
						return -EINVAL;
					}
					if (tmp_data1[i] & 0x80){
						ready_cnt++;
					}else{
						aft_log(AFT_LOG_ERR, "erase_sector_flash: Failed!\n");
						aft_log(AFT_LOG_ERR, "erase_sector_flash: Sector=%d!\n",
							   (sector_type == USER_SECTOR_FLASH) ? 
										sector_no+4:sector_no);
						return -EINVAL;
					}
				}
			}
			if (ready_cnt == cardnum){
				/* All cards are ready!! */
				break;
			}
		} while(1);

		aft_flash_log(AFT_LOG_INFO, "\nerase_sector_flash: Sector %d PASSED!", 
			(sector_type == USER_SECTOR_FLASH) ? 
					sector_no+4:sector_no);
	}
	goto erase_sector_flash_done;
	
	// Verify that flash is 0xFF
	offset = 0x00;
	if (sector_type == USER_SECTOR_FLASH){
		offset = USER_SECTOR_START_ADDR;
	}
//        sector_type = DEF_SECTOR_FLASH;    // M.F debug
//        offset = 0;  

	aft_flash_log(AFT_LOG_INFO, "\nerase_sector_flash: Verification ");
	fflush(stdout);
	for(cnt = 0; cnt < 0x40000; cnt++){
//MF		for(i=0;i<10000;i++);
		err = read_flash(netif, sector_type, MEMORY_TYPE_FLASH, offset+cnt, tmp_data);
		if (err){
			aft_flash_log(AFT_LOG_INFO, "FAILED.\n");
			return -EINVAL;
		}
		for(i = 0; i < cardnum; i++){
			if (tmp_data[i] != 0xFF){
				aft_log(AFT_LOG_ERR, " Failed to compare! %05lx -> %02x \n",
						offset+cnt, tmp_data[i]);
				return -EINVAL;
			}
		}
		if ((cnt & 0x1FFF) == 0x1000){
			aft_log(AFT_LOG_INFO, ".");
			fflush(stdout);
		}
	}
	aft_flash_log(AFT_LOG_INFO, "PASSED.\n");

erase_sector_flash_done:
	return 0;
}

static int prg_flash_byte(netif_t* netif, int cardnum, int sector_type, unsigned long off32, unsigned char byte)
{
	int		err, ready_cnt, i;

#if 0
	if (verify_write_sector_flash(netif, sector_type)){
		aft_log(AFT_LOG_ERR, "erase_sector: Default sector protected!\n");
		return -EINVAL;
	}    
#endif

	write_flash(netif, sector_type, MEMORY_TYPE_FLASH, 0x5555, 0xAA);
	write_flash(netif, sector_type, MEMORY_TYPE_FLASH, 0x2AAA, 0x55);
	write_flash(netif, sector_type, MEMORY_TYPE_FLASH, 0x5555, 0xA0);
	write_flash(netif, sector_type, MEMORY_TYPE_FLASH, off32, byte);

	do{
//MF		for(i=0;i<1000;i++);
		ready_cnt = 0;
		err = read_flash(netif, sector_type, MEMORY_TYPE_FLASH, off32, tmp_data);
		if (err){
			return -EINVAL;
		}
		for(i = 0; i < cardnum; i++){
			if ((tmp_data[i] & 0x80) == (byte & 0x80)){
				ready_cnt++;
			}else if (tmp_data[i] & 0x20){
				err = read_flash(netif,
					       	   sector_type, 
						   MEMORY_TYPE_FLASH, 
						   off32, tmp_data1);
				if (err){
					return -EINVAL;
				}
				if ((tmp_data1[i] & 0x80) == (byte & 0x80)){
					ready_cnt++;
				}else{
					aft_log(AFT_LOG_ERR, "Failed to program flash byte!\n");
					return -EINVAL;
				}
			}
		}
		if (ready_cnt == cardnum){
			break;
		}
	} while(1);
	return 0;
}

static int reset_flash(netif_t* netif)
{
	write_flash(netif, DEF_SECTOR_FLASH, MEMORY_TYPE_FLASH, 0x00, 0xF0);
	return 0;
}

int aft_check_security_id(netif_t* netif, int cardnum, unsigned int *security)
{
	unsigned int	adptr_security = 0;
	int		i, err;

	err = read_cpld(netif,SECURITY_CPLD_REG, tmp_data);
	if (err){
		aft_log(AFT_LOG_ERR, "Failed to read Adapter Security!\n");
		return -EINVAL;
	}
	for(i = 0; i < cardnum; i++){
		if (tmp_data[i] == 0xFFFF){
			continue;
		}
		tmp_data[i] = tmp_data[i] >> SECURITY_CPLD_SHIFT;
		tmp_data[i] = tmp_data[i] & SECURITY_CPLD_MASK;
		if (!i || !adptr_security){
			adptr_security = tmp_data[i];
		}else if (adptr_security != tmp_data[i]){
			aft_log(AFT_LOG_ERR, "Adapter Security is not the same for all Sangoma AFT cards (0x%02X:0x%02X(%d))!\n",
				adptr_security, tmp_data[i],i);
			aft_log(AFT_LOG_ERR, "Please, check that all cards has the same Security ID!\n");
			return -EINVAL;
		}
	}
	switch(adptr_security){
	case SECURITY_1LINE_UNCH:
		aft_log(AFT_LOG_INFO, "\nAll cards has security 1 T1/E1 line Unchannelized!\n");
		break;
	case SECURITY_2LINE_UNCH:
		aft_log(AFT_LOG_INFO, "\nAll cards has security 2 T1/E1 lines Unchannelized!\n");
		break;
	case SECURITY_1LINE_CH:
		aft_log(AFT_LOG_INFO, "\nAll cards has security 1 T1/E1 line Channelized!\n");
		break;
	case SECURITY_2LINE_CH:
		aft_log(AFT_LOG_INFO, "\nAll cards has security 2 T1/E1 lines Channelized!\n");
		break;
	default:
		aft_log(AFT_LOG_ERR, "Invalid Adapter Security ID (0x%04X)!\n", security);
		return -EINVAL;
	}
	if (security){
		*security = adptr_security;
	}
	return 0;
}

int aft_get_flash_ver(netif_t* netif, int cardnum, unsigned char* flash_ver)
{
	int		sector_type = USER_SECTOR_FLASH; 
	unsigned long	offset = 0;
	unsigned int	ver = 0;
	int		i;
	
	if (sector_type == USER_SECTOR_FLASH){
		offset = USER_SECTOR_START_ADDR;
	}
	if (read_flash(netif, USER_SECTOR_FLASH, MEMORY_TYPE_FLASH, offset + FLASH_VER_ADDR, tmp_data)){
		aft_log(AFT_LOG_INFO, "Failed to read Xilinx Flash version!\n");
		return -EINVAL;
	}
	for(i = 0; i < cardnum; i++){
		if ((unsigned char)tmp_data[i] == 0xFF){
			continue; // CPU B
		}
		if (ver == 0){
			ver = tmp_data[i];
		}else if (ver != tmp_data[i]){
			aft_log(AFT_LOG_INFO, "WARNING: Xilinx Flash Ver is not the same for all cards (0x%02X:0x%02X(%d))!\n",
					(unsigned char)ver,
				       	(unsigned char)tmp_data[i],
					i);
			if (flash_ver) *flash_ver = 0;
			return -EINVAL;
		}
	}
	if (flash_ver){
		*flash_ver = (unsigned char)ver;
	}
	return 0;	
}

int aft_check_flash_id(netif_t* netif, int cardnum, int sector_type, int *man_id, int* dev_id)
{
	int	memory_type = MEMORY_TYPE_FLASH;
	int	i;

	reset_flash(netif);
	write_flash(netif, sector_type, memory_type, 0x5555, 0xAA);
	write_flash(netif, sector_type, memory_type, 0x2AAA, 0x55);
	write_flash(netif, sector_type, memory_type, 0x5555, 0x90);

	if (read_flash(netif, sector_type, memory_type, 0x00, tmp_data)){
		aft_log(AFT_LOG_INFO, "Failed to read Manufacture ID!\n");
		return -EINVAL;
	}
	for(i = 0; i < cardnum; i++){
		if ((unsigned char)tmp_data[i] == (unsigned char)0xFF){
			aft_log(AFT_LOG_ERR, "Failed to read Manufacture ID (0x%02X)!\n",
					(unsigned char)tmp_data[i]);
			return -EINVAL;
		}
		if (tmp_data[0] != tmp_data[i]){
			aft_log(AFT_LOG_ERR, "Manufacture ID is not the same for all cards (0x%02X:0x%02X)!\n",
					(unsigned char)tmp_data[0],
			       		(unsigned char)tmp_data[i]);
			return -EINVAL;
		}
	}
	aft_log(AFT_LOG_INFO, "Manufacture ID (0x01): %02X\n",
		       	(unsigned char)tmp_data[0]);
	if (man_id) *man_id = (unsigned char)tmp_data[0];

	if (read_flash(netif, sector_type, memory_type, 0x01, tmp_data)){
		aft_log(AFT_LOG_INFO, "Failed to read Device ID!\n");
		return -EINVAL;
	}
	for(i = 0; i < cardnum; i++){
		if ((unsigned char)tmp_data[i] == (unsigned char)0xFF){
			aft_log(AFT_LOG_ERR, "Failed to read Device ID (0x%02X)!\n",
						(unsigned char)tmp_data[i]);
			return -EINVAL;
		}
		if (tmp_data[0] != tmp_data[i]){
			aft_log(AFT_LOG_ERR, "Device ID is not the same for all cards (0x%02X:0x%02X)!\n",
						(unsigned char)tmp_data[0], (unsigned char)tmp_data[i]);
			return -EINVAL;
		}
	}
	aft_log(AFT_LOG_INFO, "Device ID (0x4F)     : %02X\n", (unsigned char)tmp_data[0]);
	if (dev_id) *dev_id = (unsigned char)tmp_data[0];
	reset_flash(netif);
	return 0;
}

int aft_update_flash(netif_t* netif, int cardnum, int sector_type, char* filename, unsigned char flash_ver, int only_verify)
{
	unsigned char*	data = NULL;
	long		fsize = 0;
	unsigned long	offset = 0, findex = 0;
	int		err = 0, i;
        
	// Checking write enable to the Default Boot Flash Sector 
	fsize = read_data_file(filename, &data);
	if (fsize < 0){
		return -EINVAL;
	}
	if (fsize > 0x40000){
		aft_log(AFT_LOG_ERR, "update: Data file is too big!\n");
		err = -EINVAL;
		goto aft_update_flash_done;
	}

	if (only_verify){
		goto aft_verify_flash;
	}
	reset_flash(netif);
	erase_sector_flash(netif, cardnum, sector_type);
	if (sector_type == USER_SECTOR_FLASH){
		offset = USER_SECTOR_START_ADDR;
	}

	aft_log(AFT_LOG_INFO, "\n");
	fflush(stdout);
	while(findex < fsize){
		err = prg_flash_byte(netif, cardnum, sector_type, offset+findex, data[findex]);
		if (err){
			goto aft_update_flash_done;
		}
		findex++;
		if ((findex & 0x1FFF) == 0x1000){
			aft_log(AFT_LOG_INFO, "\rUpdating %s Xilinx Flash code ...\t%d (%d)",
					SECTOR_FLASH_DECODE(sector_type),
					findex, fsize);
			fflush(stdout);
		}
	}
	aft_log(AFT_LOG_INFO, "\rUpdating %s Xilinx Flash code ...\tPASSED (total %d bytes)\n",
					SECTOR_FLASH_DECODE(sector_type), fsize);
	aft_log(AFT_LOG_FILE, "Updating %s Xilinx Flash code ...\tPASSED (total %d bytes)\n",
					SECTOR_FLASH_DECODE(sector_type), fsize);

aft_verify_flash:
	reset_flash(netif);
	findex = 0;
	if (sector_type == USER_SECTOR_FLASH){
		offset = USER_SECTOR_START_ADDR;
	}
	fflush(stdout);
	while(findex < fsize){
		err = read_flash(netif, sector_type, MEMORY_TYPE_FLASH, offset+findex, tmp_data);
		if (err){
			aft_flash_log(AFT_LOG_INFO, "FAILED.\n");
			return -EINVAL;
		}
		for(i = 0; i < cardnum; i++){
			if ((unsigned char)tmp_data[i] != data[findex]){
				aft_log(AFT_LOG_INFO, "FAILED (offset=0x%lX, %d card)!\n",
								offset + findex, i+1);
				aft_log(AFT_LOG_ERR, "Failed to compare data at offset 0x%lX for %d card(0x%02X:0x%02X)!\n",
						offset + findex, i+1, 
						(unsigned char)tmp_data[i], data[findex]);
				return -EINVAL;
			}
		}
		if ((findex & 0x1FFF) == 0x1000){
			aft_log(AFT_LOG_INFO, "\rVerification %s Xilinx Flash code ...\t%d (%d)",
						SECTOR_FLASH_DECODE(sector_type),
						findex, fsize);
			fflush(stdout);
		}
		findex++;
	}
	aft_log(AFT_LOG_INFO, "\rVerification %s Xilinx Flash code ...\tPASSED (total %d bytes)\n",
						SECTOR_FLASH_DECODE(sector_type), fsize);
	aft_log(AFT_LOG_FILE, "Verification %s Xilinx Flash code ...\tPASSED (total %d bytes)\n",
						SECTOR_FLASH_DECODE(sector_type), fsize);
	if (!only_verify && sector_type == USER_SECTOR_FLASH){
		aft_log(AFT_LOG_INFO, "Updating Xilinx Flash version to 0x%02X\n",
						flash_ver);
		err = prg_flash_byte(netif, cardnum, USER_SECTOR_FLASH, offset + FLASH_VER_ADDR, flash_ver);
	}else{
		aft_log(AFT_LOG_INFO, "\n");
	}
aft_update_flash_done:
	fflush(stdout);
	if (data) free(data);
	return err;
}


int aft_reload_code(netif_t* netif, int sector_type)
{
	if (sector_type == USER_SECTOR_FLASH){
		aft_log(AFT_LOG_INFO, "Reloading Xilinx code from Flash User sector ...\n");
                /* Reload new code in to Xilinx from
		 * Flash User sector */  
		write_cpld(netif, 0x07,0xC0); 
	}else{
		aft_log(AFT_LOG_INFO, "Reloading Xilinx code from Flash Default sector ...\n");
        	/* Reload new code in to Xilinx from
		 * Flash Default sector */   
		write_cpld(netif, 0x07,0x80);   
	}
	return 0;
}
