#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
#include <unistd.h>
#include <sched.h>
#include <string.h>
#include <errno.h>
#include <fcntl.h>
#include <netinet/in.h>
#include <signal.h>
#include <sys/ipc.h>
#include <sys/sem.h>
#include <sys/time.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <sys/socket.h>
#include <linux/if.h>
#include <linux/if_packet.h>
#include <linux/sockios.h>

#include "aft_log.h"
#include "aft_main.h"
#include "aft_ioctl.h"

/*
******************************************************************************
**			DEFINES/MACROS				
******************************************************************************
*/

/*
******************************************************************************
**			TYPEDEF/STRUCTURES				
******************************************************************************
*/


/*
******************************************************************************
**			GLOBAL VARIABLES				
******************************************************************************
*/
extern int	verbose;
/*
******************************************************************************
**			FUNCTION PROTOTYPES				
******************************************************************************
*/

/*
******************************************************************************
**			FUNCTION DEFINITIONS				
******************************************************************************
*/
int MakeConnection(netif_t* netif, int flag) 
{
	struct sockaddr_ll	sa;
	struct ifreq		ifr;

	memset(&sa,0,sizeof(struct sockaddr_ll));
	if (netif->sock > 0){
		CloseConnection(netif);
	}
	aft_socket_log(AFT_LOG_INFO, "Open Socket to %s\n", netif->ifname); 
	if (flag){
	   	netif->sock = socket(AF_PACKET, SOCK_DGRAM, 0);
	}else{
	   	netif->sock = socket(AF_WANPIPE, SOCK_RAW, 0);
	}
   	if (netif->sock < 0){
      		perror("Socket");
      		return -EIO;
   	} /* if */

	if (flag){	
		strcpy(ifr.ifr_name, netif->ifname);
		if (ioctl(netif->sock, SIOCGIFINDEX, &ifr)){
			perror("Get index: ");
			return -EIO;
		}

		sa.sll_protocol = htons(PVC_PROT);
		sa.sll_family=AF_PACKET;
		sa.sll_ifindex = ifr.ifr_ifindex;

        	if (bind(netif->sock, (struct sockaddr *)&sa, sizeof(struct sockaddr_ll)) < 0){
                	perror("bind");
			printf("Failed to bind a socket to %s interface\n", netif->ifname);
			return -EIO;
	        }
	}else{
		struct wan_sockaddr_ll 	sa;

		strcpy(sa.sll_device, netif->ifname);
		strcpy(sa.sll_card, "wanpipe1");
		sa.sll_protocol = htons(PVC_PROT);
		sa.sll_family = AF_WANPIPE;
	
	        if(bind(netif->sock, (struct sockaddr *)&sa, sizeof(struct wan_sockaddr_ll)) < 0){
        	        perror("bind");
			printf("%s: Failed to bind a socket to %s interface\n",
					"wanpipe1", 
					netif->ifname);
	                return -EIO;
        	}
	}
	aft_socket_log(AFT_LOG_INFO, "Socket bound to %s (%d)\n\n", 
				netif->ifname, netif->sock);
	return 0;
}

int CloseConnection(netif_t* netif) 
{
	aft_socket_log(AFT_LOG_INFO, "Close Socket to %s\n", netif->ifname); 
	if (netif->sock > 0){
		close(netif->sock);
		netif->sock = 0;
	}
	return 0;
}

int exec_read_cmd(netif_t* netif, unsigned int off, unsigned int len, unsigned int *data)
{
	struct ifreq		ifr;
	struct sdla_hdlc_api	api_cmd;
	int			err = 0, cnt, i;
	
	memset(&api_cmd, 0, sizeof(struct sdla_hdlc_api));
	strcpy(ifr.ifr_name, netif->ifname);
	ifr.ifr_data = (char*)&api_cmd;

	api_cmd.cmd	= SDLA_HDLC_READ_REG;
	api_cmd.bar	= 0;
	api_cmd.len	= len;
	api_cmd.offset	= off;

	err = ioctl(netif->sock, api_cmd.cmd, &ifr);
	if (err){
		perror("Read Cmd Exec: ");
	}

	cnt = api_cmd.len / len;
	for(i = 0; i < cnt; i++){
		if (len==1){
			(unsigned char*)data[i] = 
					((unsigned char*)api_cmd.data)[i];
		}else if (len==2){
			(unsigned short*)data[i] = ((unsigned short*)api_cmd.data)[i];
		}else{
			(unsigned int*)data[i] = ((unsigned int*)api_cmd.data)[i];
		}	
	}
//	memcpy(data, api_cmd.data, api_cmd.len);
#if 0
	if (len==1){
		*(unsigned char*)data = *(unsigned char*)api_cmd.data;
	}else if (len==2){
		*(unsigned short*)data = *(unsigned short*)api_cmd.data;
	}else{
		*(unsigned int*)data = *(unsigned int*)api_cmd.data;
	}
#endif
	return err;
}

int exec_write_cmd(netif_t* netif, unsigned int off, unsigned int len, unsigned int data)
{
	struct ifreq		ifr;
	struct sdla_hdlc_api	api_cmd;
	int			err = 0;

	memset(&api_cmd, 0, sizeof(struct sdla_hdlc_api));
	strcpy(ifr.ifr_name, netif->ifname);
	ifr.ifr_data	= (char*)&api_cmd;

	api_cmd.cmd	= SDLA_HDLC_WRITE_REG;
	api_cmd.bar	= 0;
	api_cmd.len	= len;
	api_cmd.offset	= off;

	if (len==1){
		*(unsigned char*)api_cmd.data=(unsigned char)data;
	}else if (len==2){
		*(unsigned short*)api_cmd.data=(unsigned short)data;
	}else{
		*(unsigned int*)api_cmd.data=(unsigned int)data;
	}

	err = ioctl(netif->sock, api_cmd.cmd, &ifr);
	if (err){
		perror("Write Cmd Exec: ");
	}

	return err;
}

int exec_bios_cmd(netif_t* netif)
{
	struct ifreq		ifr;
	struct sdla_hdlc_api	api_cmd;
	int			err = 0;

	strcpy(ifr.ifr_name, netif->ifname);
	ifr.ifr_data	= (char*)&api_cmd;

	api_cmd.cmd	= SDLA_HDLC_SET_PCI_BIOS;
	api_cmd.bar	= 0;
	api_cmd.len	= 0;
	api_cmd.offset	= 0;

	err = ioctl(netif->sock, api_cmd.cmd, &ifr);
	if (err){
		perror("Bios Cmd Exec: ");
	}

	return err;
}
