/*****************************************************************************
* sdla_log.c	SDLA Hardware Test program
*
* Author(s):	Alex Feldman <al.feldman@sangoma.com>
*
* Copyright:	(c) 2000-2003 Sangoma Technologies Inc.
*
*		This program is free software; you can redistribute it and/or
*		modify it under the terms of the GNU General Public License
*		as published by the Free Software Foundation; either version
*		2 of the License, or (at your option) any later version.
******************************************************************************
*/

/*
******************************************************************************
**			INCLUDES	FILES				
******************************************************************************
*/
#include <stdlib.h>
#include <stdio.h>
#include <stdarg.h>
#include <ctype.h>
#include <unistd.h>
#include <time.h>
#include <sys/time.h>
#include <string.h>
#include <errno.h>
#include "aft_main.h"
#include "aft_log.h"


/*
******************************************************************************
**			DEFINES/MACROS				
******************************************************************************
*/
#define MAX_TIME_LEN	40
#define MAX_LOG_LEN	150

#define AFT_LOG_FILENAME	"aft_msg.log"
/*
******************************************************************************
**			TYPEDEF/STRUCTURES				
******************************************************************************
*/


/*
******************************************************************************
**			GLOBAL VARIABLES				
******************************************************************************
*/
static char 	log_name[MAX_FILE_NAME];

/*
******************************************************************************
**			FUNCTION PROTOTYPES				
******************************************************************************
*/
/*
******************************************************************************
**			FUNCTION DEFINITIONS				
******************************************************************************
*/

int aft_log_init(char* name)
{
	FILE*	log_file;
	
	strcpy(log_name, (name == NULL) ? AFT_LOG_FILENAME : name);
	log_file = fopen(log_name, "w");
	if (log_file == NULL){
		printf("Failed to open test log file %s\n",
				log_name);
		return -EINVAL;
	}
	fclose(log_file);
	return 0;
}

static void aft_log_time(int log_type, int only)
{
	FILE*		log_file;
	char		time_str[MAX_TIME_LEN];
	struct timeval	tv;
	struct timezone tz;
	time_t		now;
	int		len;

	gettimeofday(&tv, &tz);
	now = tv.tv_sec;
	if (only){
		sprintf(time_str, "\n%s", ctime(&now));
	}else{
		sprintf(time_str, "%s", ctime(&now));
	}
	len = strlen(time_str);
	time_str[len-1] = ':';
	time_str[len] = ' ';
	time_str[len+1] = '\0';

	if ((log_file = fopen(log_name, "a")) != NULL){
		fputs(time_str, log_file);
		fclose(log_file);
	}
	return;
}

void aft_log(int type, const char * fmt, ...)
{
	FILE*		log_file;
	va_list		args;
	char		buf[MAX_LOG_LEN], *ptr;
	int		len, screen_only=0;

	memset(buf, '\0', MAX_LOG_LEN);
	va_start(args, fmt);
	vsnprintf(buf, sizeof(buf), fmt, args);
	va_end(args);
	ptr = &buf[0];

	len = strlen(buf);
	if (buf[0] == '\r' || buf[len-1] != '\n') screen_only = 1;
	buf[len] = '\0';
	if (buf[0] == '\n' && len != 1){
		aft_log_time(AFT_LOG_INFO, 1);
		if (--len == 0){
			buf[0] = '\n';
			buf[1] = '\0';
			len = 1;
		}else{
			printf("\n");
			ptr++;	
		}
	}else{
		if (!screen_only){
			aft_log_time(AFT_LOG_INFO, 0);
		}
	}

	log_file = fopen(log_name, "a");
	switch(type){
	case AFT_LOG_INFO:
		printf(ptr);
		if (log_file && !screen_only) fputs(ptr, log_file);
		break;
	case AFT_LOG_ERR:
		printf("ERROR: %s", ptr);
		if (log_file){
			fputs("ERROR: ", log_file);
			fputs(ptr, log_file);
		}
		break;
	case AFT_LOG_FILE:
		if (log_file) fputs(ptr, log_file);
		break;
	}	
	if (log_file) fclose(log_file);
	fflush(stdout);
	return;
}


