#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
#include <unistd.h>
#include <sched.h>
#include <sys/ipc.h>
#include <sys/sem.h>
#include <sys/time.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <string.h>
#include <errno.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <signal.h>

#include "aft_log.h"
#include "aft_main.h"
#include "aft_flash.h"
#include "aft_prg.h"
#include "aft_cmd.h"
#include "aft_test.h"

/*
******************************************************************************
**			DEFINES/MACROS				
******************************************************************************
*/
#define AFT_PROD_VERSION	"1.1.0"

#define SDLA_CPU_A	1
#define SDLA_CPU_B	2

#define SDLA_CPU2CHAR(cpu_no)		(cpu_no == SDLA_CPU_A) ? 'A' : 'B'
#define SDLA_COMMPORT2STR(comm_port)	(comm_port == WANOPT_PRI) ? "PRI" : "SEC"

/*
******************************************************************************
**			TYPEDEF/STRUCTURES				
******************************************************************************
*/


/*
******************************************************************************
**			GLOBAL VARIABLES				
******************************************************************************
*/
char	script_sim[]="./script_sim.sh";
char	lsmod_out[]="lsmod.out";
char	hwprobe_out[]="hwprobe.out";
int	verbose = AFT_VERBOSE_NONE;

sdlaprod_t*	sdlacfg = NULL;

extern FILE*	yyin;

/*
******************************************************************************
**			FUNCTION PROTOTYPES				
******************************************************************************
*/

/*
******************************************************************************
**			FUNCTION DEFINITIONS				
******************************************************************************
*/
void hit_any_key(void)
{
	printf("Hit any key to continue... \n");
	getchar();
	getchar();
}


void aft_version(void)
{
	printf("\nSangoma AFT Production Program (version %s)!\n", 
				AFT_PROD_VERSION);
	return;
}

int wanrouter_run(sdlacard_t* card, int flag)
{
	pid_t		pid_child;
	int 		status = 0, err = 0;

	aft_script_log(AFT_LOG_INFO, "wanrouter_run...Prepare to run chilc process\n"); 
	if ((pid_child = fork()) == 0){
		char	command[MAX_FILE_NAME];
		int	ret;

		aft_script_log(AFT_LOG_INFO,"wanrouter_run...CHILD (pid=%d)\n", 
				getpid()); 
		/* Child process */
		switch(flag){
		case WANROUTER_CHECK:
			sprintf(command, "%s lsmod wanpipe %s",
				script_sim, lsmod_out);
			break;

		case WANROUTER_LOAD:
			sprintf(command, "%s modprobe wanpipe",
				script_sim);
			break;

		case WANROUTER_UNLOAD:
			sprintf(command, "%s modprobe -r wanpipe",
				script_sim);
			break;

		case WANROUTER_HWPROBE:
			sprintf(command, "%s /usr/sbin/wanrouter hwprobe %s",
					script_sim,
					hwprobe_out);
			break;

		case (WANROUTER_DEVICE|WANROUTER_START):
		case (WANROUTER_DEVICE|WANROUTER_STOP):
			sprintf(command, "%s /usr/sbin/wanrouter %s %s",
					script_sim,
					(flag & WANROUTER_START) ? "start" : "stop",
					card->devname);
			break;
			
#if 0
		case (WANROUTER_INTERFACE|WANROUTER_START):
		case (WANROUTER_INTERFACE|WANROUTER_STOP):
			sprintf(command, "%s /usr/sbin/wanrouter %s %s %s",
					script_sim,
					(flag & WANROUTER_START) ? "start" : "stop",
					card->card_name,
					wan_api->if_name);
			break;
#endif			
		default:
			aft_log(AFT_LOG_ERR, "Undefined flag value!\n");
			exit(1);
		}
		ret=system(command);
		exit(WEXITSTATUS(ret));
	}else if (pid_child < 0){
		perror("fork");
		aft_script_log(AFT_LOG_INFO,"wanrouter_run...fork is failed\n"); 
	}
	
	aft_script_log(AFT_LOG_INFO, "wanrouter_run...Wait\n");	
	waitpid(pid_child, &status, 0);
	if (WEXITSTATUS(status) != 0){
		err = -EINVAL;
	}

	aft_script_log(AFT_LOG_INFO, "wanrouter_run...Done\n"); 
	return err;
}

static int aft_check_card_type(sdlaprod_t* sdlacfg)
{
	sdlacard_t*	card = sdlacfg->head;
	int 		type = card->type;

	for(card = sdlacfg->head; card; card = card->next){
		if (card->type != type){
			aft_log(AFT_LOG_ERR, "%s: This card has different type (0x%04X) from other cards (0x%04X)!\n",
						card->devname, card->type, type);
			aft_log(AFT_LOG_ERR, "Sangoma AFT Production test can apply only to the same type of card!\n");
			return -EINVAL;
		}	
	}
	return 0;
}

int aft_get_type(char* line)
{
	if (strncmp(line, "AFT-A101", 8) == 0 || strncasecmp(line, "A101", 4) == 0){
		return A101_ADPTR_1TE1;
	}else if (strncmp(line, "AFT-A102", 8) == 0 || strncasecmp(line, "A102", 4) == 0){
		return A101_ADPTR_2TE1;
	}else{
		aft_log(AFT_LOG_ERR, "Unsupported card -> %s\n", line);
		aft_exit(-EINVAL);
	}
	return -EINVAL;
}
/*
** Return 1 if this device will stay not configured. 
*/
static int parse_hwprobe(char* hwprobe_out, sdlaprod_t* sdlacfg)
{
	FILE*		f = fopen(hwprobe_out, "r");
	sdlacard_t*	card, *tmpcard;
	char		line[100];
	char*		tmp;
	int		i, devnum = 0, cardnum = 0;

	while(fgets(line, sizeof(line), f) != NULL){
		card = 	(sdlacard_t*)calloc(1, sizeof(sdlacard_t));
		if (card == NULL){
			aft_log(AFT_LOG_ERR, "Failed allocate card memory (%d)!\n",
					devnum + 1);
			return -ENOMEM;
		}

		devnum++;
		sprintf(card->devname, "wanpipe%d", devnum);
		sprintf(card->netif.ifname, "wp%daft1", devnum);
		card->num = devnum;
		card->netif.test = SDLATST_FAILED;
		tmp = strtok(line, " ");
		/* Card type */
		tmp = strtok(NULL, ".");
		while(isspace(*tmp)) tmp++;
		card->type = aft_get_type(tmp);
		tmp = strtok(tmp, ":");
		while((tmp = strtok(NULL, ":")) != NULL){
			i = 0;
			while(isspace(tmp[i])) i++;
			if (strncmp(&tmp[i], "SLOT", 4) == 0){
				while(!isdigit(tmp[i])) i++;
				card->slot_no = atoi(&tmp[i]);
			}else if (strncmp(&tmp[i], "BUS",3) == 0){
				while(!isdigit(tmp[i])) i++;
				card->bus_no = atoi(&tmp[i]);
			}else if (strncmp(&tmp[i], "IRQ", 3) == 0){
				while(!isdigit(tmp[i])) i++;
				card->irq_no = atoi(&tmp[i]);
			}else if (strncmp(&tmp[i], "CPU", 3) == 0){
				while(tmp[i++] != '=');
				if (tmp[i] == 'A'){
					card->cpu_no = SDLA_CPU_A;
				}else if (tmp[i] == 'B'){
					card->cpu_no = SDLA_CPU_B;
				}else{
					printf("Error: Wrong CPU number (%c)!\n",
							tmp[i]);
					continue;
				}
			}else if (strncmp(&tmp[i], "PORT", 4) == 0){
				while(tmp[i++] != '=');
				if (strncmp(&tmp[i], "PRI", 3) == 0){
					card->comm_port = WANOPT_PRI;
				}else if (strncmp(&tmp[i], "SEC", 3) == 0){
					card->comm_port = WANOPT_SEC;
				}else{
					printf("Error: Wrong Comm Port!\n");
					continue;
				}
			}
		}
		aft_hwprobe_log(AFT_LOG_INFO, "%s configuration: Slot %d Bus %d IRQ %d CPU %c Port %s\n",
						card->devname, 
						card->slot_no, 
						card->bus_no, 
						card->irq_no, 
						SDLA_CPU2CHAR(card->cpu_no), 
						SDLA_COMMPORT2STR(card->comm_port));
		if (card->cpu_no == SDLA_CPU_A){
			cardnum++;
		}
		/* Link the main cards */
		if (sdlacfg->head == NULL){
			sdlacfg->head = card;
		}else{
			for(tmpcard = sdlacfg->head; tmpcard->next; tmpcard = tmpcard->next);
			if (tmpcard != card){
				tmpcard->next = card;
			}
		}
	}
	fclose(f);
	unlink(hwprobe_out);
	if ((cardnum != sdlacfg->cards_no) || (devnum != (sdlacfg->cards_no * sdlacfg->te_lines_no))){
		aft_log(AFT_LOG_ERR, "Mistmatch in number of cards set for this production test!\n");
		aft_log(AFT_LOG_ERR, "Found %d Sangoma PCI cards.\n", cardnum);
		if (cardnum != sdlacfg->cards_no){
			aft_log(AFT_LOG_ERR, "Command line defines %d Sangoma cards!\n",
						sdlacfg->cards_no);
			aft_log(AFT_LOG_ERR, "Please, use the correct number at command line!\n");
		}
		if (cardnum == sdlacfg->cards_no && devnum != (sdlacfg->cards_no * sdlacfg->te_lines_no)){
			aft_log(AFT_LOG_ERR, "Not all Sangoma cards has the same type!!\n");
			aft_log(AFT_LOG_ERR, "Please, check that you are testing exactly the same type of Sangoma Cards!\n");
		}
		aft_log(AFT_LOG_ERR, "\n\n");
		return -EINVAL;
	}
	sdlacfg->cardnum = cardnum;
	sdlacfg->devnum = devnum;
	return 0;
}

static int aft_get_core_type(void)
{
	int val;

	system("clear");
	printf("\n 
	Sangoma AFT card core types: 
	-----------------------

1. 1/2 T1/E1 line(s) HDLC core

2. 1/2 T1/E1 line(s) ATM core

3. 1/2 T1/E1 line(s) SS7 core

q. Quit

Please Select (<opt>|q): ");

	scanf("%d",&val);
	if (val==-1){
		getchar();
	}

	switch (val){
	case 1:
		return AFT_TE1_HDLC_CORE_ID;
	case 2:
		return AFT_TE1_ATM_CORE_ID;
	case 3:
		return AFT_TE1_SS7_CORE_ID;
	}
	return -EINVAL;

}

static int aft_create_conf_file(sdlacard_t* card, int security_id)
{
	FILE*		f;
	netif_t*	netif;
	char		line[100];
	
	switch(security_id){
	case SECURITY_1LINE_UNCH:
	case SECURITY_2LINE_UNCH:
		card->iface_no = 1;
		break;
	case SECURITY_1LINE_CH:
	case SECURITY_2LINE_CH:
		card->iface_no = 2;
		card->netif.next = malloc(sizeof(netif_t));
		if (card->netif.next == NULL){
			printf("ERROR: Failed allocate memory for configuration structure!\n");
			return -EINVAL;
		}
		sprintf(card->netif.next->ifname, "wp%daft2", card->num);
		break;
	}

	sprintf(line, "/etc/wanpipe/%s.conf", 
					card->devname);
	f = fopen(line, "w");
	fputs("[devices]\n", f);
	sprintf(line, "%s = WAN_AFT, Comment\n",
		       card->devname);	
	fputs(line, f);
	fputs("\n", f);
	fputs("[interfaces]\n", f);
	for(netif = &card->netif; netif; netif = netif->next){
		sprintf(line, "%s = %s, , API, Comment\n", 
					netif->ifname, card->devname);
		fputs(line, f);
	}
	fputs("\n", f);
	sprintf(line, "[%s]\n", card->devname);
	fputs(line, f);
	fputs("CARD_TYPE = AFT\n", f);
	sprintf(line,"S514CPU = %c\n", SDLA_CPU2CHAR(card->cpu_no));
	fputs(line, f);
	fputs("AUTO_PCISLOT = NO\n", f);
	sprintf(line,"PCISLOT = %d\n", card->slot_no);
	fputs(line, f);
	sprintf(line,"PCIBUS = %d\n", card->bus_no);
	fputs(line, f);
	if (card->media == WANOPT_MEDIA_T1){	
		fputs("MEDIA = T1\n", f);
		fputs("LCODE = B8ZS\n", f);
		fputs("FRAME = ESF\n", f);
	}else{
		fputs("MEDIA = E1\n", f);
		fputs("LCODE = HDB3\n", f);
		fputs("FRAME = NCRC4\n", f);
	}
	fputs("TE_CLOCK = NORMAL\n", f);
	fputs("ACTIVE_CH = ALL\n", f);
	if (card->media == WANOPT_MEDIA_T1){	
		fputs("LBO = 0DB\n", f);
	}
	fputs("Firmware	= /etc/wanpipe/firmware/cdual514.sfm\n", f);
	sprintf(line, "CommPort = %s\n", SDLA_COMMPORT2STR(card->comm_port));
	fputs(line, f);
	fputs("Receive_Only	= NO\n", f);
	fputs("Connection	= Permanent\n", f);
	fputs("LineCoding	= NRZ\n", f);
	fputs("LineIdle		= Flag\n", f);
	if (card->media == WANOPT_MEDIA_T1){	
		fputs("Interface 	= V35\n", f);
	}else{
		fputs("Interface 	= RS232\n", f);
	}
	fputs("Clocking 	= External\n", f);
	fputs("BaudRate 	= 1540000\n", f);
	fputs("MTU 		= 1500\n", f);
	fputs("UDPPORT 	= 9000\n", f);
	fputs("TTL 		= 255\n", f);
	fputs("IGNORE_FRONT_END  = NO\n", f);

	fputs("\n", f);

	for(netif = &card->netif; netif; netif = netif->next){
		sprintf(line, "[%s]\n", netif->ifname);
		fputs(line, f);
		fputs("HDLC_STREAMING = YES\n", f);
		/* FIXME */
		if (card->iface_no == 1){
			fputs("ACTIVE_CH = ALL", f);
		}else{
			if (netif->next){
				fputs("ACTIVE_CH = 1-12\n", f);	
			}else{
				fputs("ACTIVE_CH = 13-24", f);	
			}
		}
	}
	fclose(f);
	return 0;


}


static int create_conf_files(sdlaprod_t* sdlacfg)
{
	sdlacard_t*	card;
	int		media;

	switch (sdlacfg->core_type){
	case AFT_TE1_HDLC_CORE_ID:
		media = WANOPT_MEDIA_T1;
		break;
	case AFT_TE1_ATM_CORE_ID:
	case AFT_TE1_SS7_CORE_ID:
		aft_log(AFT_LOG_ERR, "AFT production test doesn't support these core types (%d)!\n",
				sdlacfg->core_type);
		return -EINVAL;
	default:
		aft_log(AFT_LOG_ERR, "Unknown new core type (%d)!\n",
				sdlacfg->core_type);
		return -EINVAL;
	}
		
	for(card = sdlacfg->head; card; card = card->next){
		card->media = media;
		card->iface_no = 1;
		if (aft_create_conf_file(card, sdlacfg->security_id)){
			aft_log(AFT_LOG_ERR, "Failed to create %s configuration file!\n",
					card->devname);
			return -EINVAL;
		}
	}

	return 0;
}

#define AFT_RESULT_FORMAT	"* %-12s | %-9s | %-9s | %-11s | %-23s *\n"
#define AFT_RESULT_FORMAT	"* %-12s | %-9s | %-9s | %-11s | %-23s *\n"
static int aft_print_result(sdlaprod_t* sdlacfg)
{
	sdlacard_t*	card;
	netif_t*	netif;
	int		tst_failed = 0, prg_failed = 0;
	
	aft_log(AFT_LOG_INFO, "\n\n");
	aft_log(AFT_LOG_INFO, "********************************************************************************\n");
	aft_log(AFT_LOG_INFO, "*    S A N G O M A    A F T    P R O D U C T I O N    T E S T    R E P O R T   *\n");
	aft_log(AFT_LOG_INFO, "********************************************************************************\n");
	aft_log(AFT_LOG_INFO, AFT_RESULT_FORMAT, 
				"Device name", "Interface", "Flash Prg", "Test Result", "Comments");
	aft_log(AFT_LOG_INFO, "********************************************************************************\n");
	for(card = sdlacfg->head; card; card = card->next){
		for(netif = &card->netif; netif; netif = netif->next){
			if (sdlacfg->flash == AFTPRG_USER_FAILED || 
			    sdlacfg->flash == AFTPRG_DEF_FAILED){
				prg_failed++;
				printf("%c[5;31m", 0x1B);
			}
			if (netif->test == SDLATST_FAILED){
				tst_failed++;
				printf("%c[5;31m", 0x1B);
			}
			if (sdlacfg->flash == AFTPRG_NONE && !sdlacfg->no_update){
				printf("%c[1;31m", 0x1B);
			}
			aft_log(AFT_LOG_INFO, AFT_RESULT_FORMAT,
					card->devname, 
					netif->ifname,
					(sdlacfg->flash == AFTPRG_OK) ?
								 "PASSED" : 
					(sdlacfg->flash == AFTPRG_NONE) ?
								 "NOT PROGR" :
					(sdlacfg->flash == AFTPRG_USER_FAILED) ?
								 "U FAILED" :
								 "B FAILED",
					(netif->test == SDLATST_OK) ? "PASSED" : "FAILED",
					"None");
			printf("%c[0;m", 0x1B);
		}
	}
	aft_log(AFT_LOG_INFO, "********************************************************************************\n");
	aft_log(AFT_LOG_INFO, "\n");
	aft_log(AFT_LOG_INFO, "RESULT:\n");
	aft_log(AFT_LOG_INFO, "\t* AFT Production Application tested %d card(s) (%d device(s))!\n",
		       		sdlacfg->cardnum, sdlacfg->devnum);
	if (prg_failed){
		aft_log(AFT_LOG_INFO, "\t* Xilinx Flash Update failed for %d devices!\n", 
				prg_failed);
	}
	if (tst_failed){
		aft_log(AFT_LOG_INFO, "\t* Xilinx Production Test failed for %d devices!\n", 
				tst_failed);
	}
	aft_log(AFT_LOG_INFO, "\n");
	return 0;
}

int main(int argc, char* argv[])
{
	int		err;

	sdlacfg = malloc(sizeof(sdlaprod_t));
	if (sdlacfg == NULL){
		printf("ERROR: Failed allocate memory for configuration structure!\n");
		return -EINVAL;
	}
	memset(sdlacfg, 0, sizeof(sdlaprod_t));
	sdlacfg->flash = AFTPRG_NONE;
	sdlacfg->txsize = 1300;
	sdlacfg->txcnt = 1000;
	if (argc > 1){
		/* Parse command line */
		if (aft_cmd_parse(argc, argv, sdlacfg)){
			free(sdlacfg);
			/* Any command line errors or help message */
			return 0;
		}
	}else{
		aft_cmd_help();
		free(sdlacfg);
		return -EINVAL;
	}

	if (sdlacfg->cards_no == 0 || sdlacfg->te_lines_no == 0){
		aft_log(AFT_LOG_ERR, "Invalid command line argumnets (cards number=%d, card type=%s)\n",
						sdlacfg->cards_no, SDLA_ADPTR_DECODE(sdlacfg->card_type));
		aft_log(AFT_LOG_ERR, "Please, provide the follow information to Sangoma AFT Production test:\n");
		aft_log(AFT_LOG_ERR, "\t\t1. Total number of Sangoma card ready for the test!\n");
		aft_log(AFT_LOG_ERR, "\t\t2. Sangoma card type (For example, A101 or A102)\n");
		aft_cmd_help();
		free(sdlacfg);
		return -EINVAL;
	}
	if (aft_log_init(NULL)){
		aft_log(AFT_LOG_ERR, "Failed to initialize log file!\n");
		free(sdlacfg);
		return 1;
	}
	aft_log(AFT_LOG_INFO, "\n");
	aft_log(AFT_LOG_INFO, "Running Sangoma AFT Production Test (v.%s)\n\n", 
					AFT_PROD_VERSION);
	err = wanrouter_run(NULL, WANROUTER_HWPROBE);
	if (err){
		aft_log(AFT_LOG_ERR,
				"Hardware probe is FAILED!\n");
		free(sdlacfg);
		return 1;
	}
	wanrouter_run(NULL, WANROUTER_UNLOAD);

	err = parse_hwprobe(hwprobe_out, sdlacfg);
	if (err){
		goto main_exit;
	}

	if (aft_check_card_type(sdlacfg)){
		goto main_exit;
	}
	
	if ((sdlacfg->core_type = aft_get_core_type()) <= 0){
		goto main_exit;
	}
	
	if (create_conf_files(sdlacfg)){
		aft_log(AFT_LOG_ERR, "Failed to create all configuration files!\n");
		err = -EINVAL;
		goto main_exit;
	}

	if (aft_program_flash(sdlacfg)){
		aft_log(AFT_LOG_ERR, "Failed to program flash!\n");
		err = -EINVAL;
		goto main_done;
	}

	/* If security id is Channelized, create configuration files again */
	if (sdlacfg->security_id == SECURITY_1LINE_CH || sdlacfg->security_id == SECURITY_2LINE_CH){
		if (create_conf_files(sdlacfg)){
			aft_log(AFT_LOG_ERR, "Failed to create all configuration files!\n");
			err = -EINVAL;
			goto main_exit;
		}
	}

	if (aft_test(sdlacfg) != SDLATST_OK){
		aft_log(AFT_LOG_ERR, "AFT test failed!\n");
		err = -EINVAL;
		goto main_done;
	}
	
main_done:
	/* Printf result */
	aft_print_result(sdlacfg);

main_exit:
	while(sdlacfg->head){
		sdlacard_t*	tmp;
		tmp = sdlacfg->head;
		sdlacfg->head = sdlacfg->head->next;
		if (tmp->netif.next){
			netif_t*	tmp1;
			for(tmp1 = tmp->netif.next; tmp1; tmp1 = tmp1->next){
				free(tmp1);
			}
		}
		free(tmp);
	}
	free(sdlacfg);
	return err;
}

void aft_exit(int err)
{
	printf("\n\nCritical Error....Bye..Bye!\n");
	hit_any_key();
	aft_clean(0);
	exit(err);
}

void aft_clean(int wanrouter_stop)
{
	return;
}	
