
#ifndef __AFT_MAIN_H
# define __AFT_MAIN_H

/*
******************************************************************************
**			INCLUDES	FILES				
******************************************************************************
*/
//#include <linux/if.h>
#include <linux/if_wanpipe.h>
#include <linux/wanpipe.h>
#include <linux/sdla_xilinx.h>


/*
******************************************************************************
**			DEFINES/MACROS				
******************************************************************************
*/
#define MAX_CHAN_LEN		100
#define MAX_DESCR_LEN		256
#define MAX_LABEL_LEN		100
#define MAX_FILE_NAME		100

#define VERBOSE_NONE		0x00
#define VERBOSE_CFG		0x01
#define VERBOSE_MSG		0x02
#define VERBOSE_DATA		0x04
#define VERBOSE_PROCESS		0x08
#define VERBOSE_SIM		0x10
#define VERBOSE_TEST		0x20
#define VERBOSE_ERR_IGNR	0x40

#define WANROUTER_START		0x01
#define WANROUTER_STOP		0x02
#define WANROUTER_INTERFACE	0x04
#define WANROUTER_DEVICE	0x08
#define WANROUTER_HWPROBE	0x10
#define WANROUTER_CHECK		0x20
#define WANROUTER_LOAD		0x40
#define WANROUTER_UNLOAD	0x80

#define AFT_TE1_HDLC_CORE_ID	0x01
#define AFT_TE1_ATM_CORE_ID	0x02
#define AFT_TE1_SS7_CORE_ID	0x03

/*
******************************************************************************
**			TYPEDEF/STRUCTURE
******************************************************************************
*/
typedef struct netif_
{
	char	ifname[WAN_IFNAME_SZ];
	int 	hwmode;
	int	sock;
	int	test;		/* test result */

	/* Test variables */
	int		pid;
	int		rx_ind;
	int 		tx_ind;
	int		txcnt;
	unsigned char	tx_byte0;
	int 		write_enable;
	int		read_enable;
	int		txsize;
	unsigned char	txdata[2000];
	unsigned char	rxdata[2000];
	struct netif_*	next;
} netif_t;

typedef struct sdlacard
{
	char	devname[WAN_IFNAME_SZ];
	int	num;
	pid_t	pid;
	/* PCI configuration */
	int	type;
	int 	media;
	int	slot_no;
	int	bus_no;
	int	irq_no;
	int	cpu_no;
	int	comm_port;
	int	iface_no;
	netif_t	netif;
	
	struct sdlacard*	next;
	struct sdlacard*	next_main;
} sdlacard_t; 

typedef struct sdlaprod
{
	int	cardnum;
	int	devnum;
	int	card_type;
	int	core_type;
	int	security_id;
	int	cards_no;
	int	te_lines_no;
	int	prg_boot_sector;
	int	get_flash_code;
	int	update;
	int	no_update;
	int	only_verify;
	int	txsize, txcnt;
	int	flash;		/* program flash result */
	
	sdlacard_t*	head;
} sdlaprod_t;

/*
******************************************************************************
**			EXTERNAL FUNCTIONs
******************************************************************************
*/
void hit_any_key(void);
extern int wanrouter_run(sdlacard_t*, int);
int aft_get_type(char* line);

extern void aft_exit(int err);
extern void aft_clean(int);

#endif

