#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
#include <unistd.h>
#include <sched.h>
#include <sys/param.h>
#include <sys/ipc.h>
#include <sys/sem.h>
#include <sys/time.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <string.h>
#include <errno.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <signal.h>
#include <ftplib.h>

#include "aft_main.h"
#include "aft_log.h"
#include "aft_flash.h"
#include "aft_ioctl.h"
#include "aft_prg.h"

/*
******************************************************************************
**			DEFINES/MACROS				
******************************************************************************
*/
#define CORE_FILE_IN_LOCAL_DIR
#define MAIN_HOST_IP		"192.168.1.100"
#define MAIN_HOST_USER		"alex"
#define MAIN_HOST_PSWRD		"sangoma"
#define AFT_CORE_DIR		"Engineering/Hardware/AFT_Series_MF/HW_programming"
#define AFT_CORE_TEMPL		"A10?_V??.BIN"
#define AFT_CORE_FILE_DEF	"A101_V17.BIN"
#define AFT_CORE_FILE_DEF_VER	0x17

#define COREFILELIST		"corefile_list.output"

#define MAX_LINE_SIZE		200

/*
******************************************************************************
**			TYPEDEF/STRUCTURES				
******************************************************************************
*/
typedef struct {
	int		type;
	char*		coref_prefix;
	char*		coref_template;
	unsigned char	version;
	char*		descr;
//#if defined(CORE_FILE_IN_LOCAL_DIR)
//	char*		filename;
//#else
	char		filename[MAXPATHLEN];
//#endif
} aft_core_info_t;

/*
******************************************************************************
**			GLOBAL VARIABLES				
******************************************************************************
*/
#if 0
aft_core_info_t aft_core_table[] = {
	{ AFT_TE1_HDLC_CORE_ID,	
	  "A101_V", "A101_V*.BIN", 0x17,
	  "1/2 T1/E1 line(s) HDLC core",
	  "A101_V17.BIN" },
	{ AFT_TE1_ATM_CORE_ID,	
	  NULL, NULL, 0x00,
	  "1/2 T1/E1 line(s) ATM core",
	  NULL },
	{ AFT_TE1_SS7_CORE_ID,
	  NULL, NULL, 0x00,
	  "1/2 T1/E1 line(s) SS7 core",
	  NULL },
	{ 0x0, NULL, NULL, 0x00, NULL, NULL }
};
#endif
aft_core_info_t aft_core_table[] = {
	{ AFT_TE1_HDLC_CORE_ID,	
	  "A101_V", "A101_V*.BIN", 0x0,
	  "1/2 T1/E1 line(s) HDLC core" },
	{ AFT_TE1_ATM_CORE_ID,	
	  NULL, NULL, 0x00,
	  "1/2 T1/E1 line(s) ATM core" },
	{ AFT_TE1_SS7_CORE_ID,
	  NULL, NULL, 0x00,
	  "1/2 T1/E1 line(s) SS7 core" },
	{ 0x0, NULL, NULL, 0x00, NULL }
};
//#endif
extern char	script_sim[];

/*
******************************************************************************
**			FUNCTION PROTOTYPES				
******************************************************************************
*/

/*
******************************************************************************
**			FUNCTION DEFINITIONS				
******************************************************************************
*/
static int aft_search_core_file(sdlaprod_t* sdlacfg, int core_ind, aft_core_info_t* core_info)
{
	char	command[MAX_FILE_NAME];
	char	*str, *pver, line[MAX_LINE_SIZE];
	FILE	*f;
	int	err = 0, ver = 0, new_ver = 0, i;
		
	if (sdlacfg->get_flash_code){
		netbuf	*nbuf;
			
		FtpInit();
		if (!FtpConnect(MAIN_HOST_IP, &nbuf)){
			aft_log(AFT_LOG_ERR, "Failed to connect to server %s!\n",
					MAIN_HOST_IP);
			err = -EINVAL;
			goto aft_get_core_info_done;
		}	
		if (!FtpLogin(MAIN_HOST_USER, MAIN_HOST_PSWRD, nbuf)){
			aft_log(AFT_LOG_ERR, "Failed to login as %s! Check your password!\n",
					MAIN_HOST_USER);
			err = -EINVAL;
			goto aft_get_core_info_done;
		}	
		if (!FtpChdir(AFT_CORE_DIR, nbuf)){
			aft_log(AFT_LOG_ERR, "Failed to change remote directory to %s!\n",
					AFT_CORE_DIR);
			err = -EINVAL;
			goto aft_get_core_info_done;
		}	
		if (!FtpDir(COREFILELIST, aft_core_table[core_ind].coref_template, nbuf)){
			aft_log(AFT_LOG_ERR, "Failed to get list of files from remote computer!\n");
			err = -EINVAL;
			goto aft_get_core_info_done;
		}
		if ((f = fopen(COREFILELIST, "r")) == NULL){
			aft_log(AFT_LOG_ERR, "Failed to open %s file!\n", COREFILELIST);
			err = -EINVAL;
			goto aft_get_core_info_done;
		}
		while(fgets(line, MAX_LINE_SIZE, f)){
			str = strstr(line, aft_core_table[core_ind].coref_prefix);
			pver = str + strlen(aft_core_table[core_ind].coref_prefix);
			new_ver = strtoul(pver, NULL, 16);
			if (new_ver > ver){
				i = 0;
				memset(core_info->filename, '\0', MAXPATHLEN);
				while(str[i] != ' ' && str[i] != '\n' && str[i] != '\0') i++;
				memcpy(core_info->filename, str, i);
				core_info->filename[i] = '\0';
				core_info->version = new_ver;
			}
		}
		fclose(f);
		unlink(COREFILELIST);
	
		if (!FtpGet(core_info->filename, core_info->filename, FTPLIB_IMAGE, nbuf)){
			aft_log(AFT_LOG_ERR, "Failed to get file %s from remote computer!\n",
					core_info->filename);
			err = -EINVAL;
			goto aft_get_core_info_done;
		}	
		core_info->version = AFT_CORE_FILE_DEF_VER;
				
aft_get_core_info_done:
		FtpQuit(nbuf);
		FtpClose(nbuf);

	}else{
		sprintf(command, "%s ls %s %s",
				script_sim, 
				aft_core_table[core_ind].coref_template,
				COREFILELIST);
		system(command);
		if ((f = fopen(COREFILELIST, "r")) == NULL){
			aft_log(AFT_LOG_ERR, "Failed to open %s file!\n",
					COREFILELIST);
			return -EINVAL;
		}
		while(fgets(line, MAX_LINE_SIZE, f)){
			str = strstr(line, aft_core_table[core_ind].coref_prefix);
			pver = str + strlen(aft_core_table[core_ind].coref_prefix);
			new_ver = strtoul(pver, NULL, 16);
			if (new_ver > ver){
				i = 0;
				memset(core_info->filename, '\0', MAXPATHLEN);
				while(str[i] != ' ' && str[i] != '\n' && str[i] != '\0') i++;
				memcpy(core_info->filename, str, i);
				core_info->filename[i] = '\0';
				core_info->version = new_ver;
			}
		}
		fclose(f);
		unlink(COREFILELIST);
		if (!new_ver){
			aft_log(AFT_LOG_ERR, "Flash code file doesn't find!\n");
			return -EINVAL;
		}	
	}
	return err;
}

static int aft_get_core_info(sdlaprod_t* sdlacfg, aft_core_info_t* core_info)
{
	int i = 0;
	
	for(i = 0; aft_core_table[i].type; i++){
		if (aft_core_table[i].type == sdlacfg->core_type){
			core_info->type = sdlacfg->core_type;
			return aft_search_core_file(sdlacfg, i, core_info);
		}
	}
	return -EINVAL;
}

int aft_program_flash(sdlaprod_t* sdlacfg)
{
	sdlacard_t*	card = sdlacfg->head;
	aft_core_info_t	core_info;
	int		man_id, dev_id, err = 0;
	unsigned char	flash_ver;
	
	if (sdlacfg->no_update){
		aft_log(AFT_LOG_INFO, "\nNOTE: NO Xilinx flash update selected!\n");
		return 0;
	}
	if (aft_get_core_info(sdlacfg, &core_info)){
		aft_log(AFT_LOG_ERR, "Failed to find core file for adapter type %d\n",
				sdlacfg->core_type);
		return -EINVAL;
	}
	if (system("./load.sh")){ //if (system("wan_aftload.sh")){
		aft_log(AFT_LOG_ERR, "Failed to load main device\n");
		return -EINVAL;
	}
	if (MakeConnection(&card->netif, 1)){
		err = -EINVAL;
		goto aft_program_flash_done;
	}

	if (aft_check_security_id(&card->netif, sdlacfg->cardnum, &sdlacfg->security_id)){
		err = -EINVAL;
		goto aft_program_flash_done;
	}

	aft_get_flash_ver(&card->netif, sdlacfg->cardnum, &flash_ver);
	if (!sdlacfg->update && flash_ver >= core_info.version){
		aft_log(AFT_LOG_INFO, "\nNOTE: All AFT cards have the latest Xilinx Flash Core ver(0x%02X)!\n",
				flash_ver);
		aft_log(AFT_LOG_INFO, "      Sangoma AFT Production app will not re-program Xilinx Flash!\n");
		if (!sdlacfg->only_verify){
			sdlacfg->flash = AFTPRG_OK;
			goto aft_program_flash_done;
		}
	}else{
		/* We need re-program Xilinx Flash */
		sdlacfg->only_verify = 0;
	}

	if (!sdlacfg->only_verify){
		aft_log(AFT_LOG_INFO, "\n\nStart Xilinx Flash memory programing!\n"); 
	}else{
		aft_log(AFT_LOG_INFO, "\n\nStart Xilinx Flash memory verifycation!\n"); 
	}
	aft_log(AFT_LOG_INFO, "(Descr=%s, Filename=%s, Ver=0x%02X)\n", 
					(core_info.descr) ? core_info.descr : "None",
					core_info.filename,
					core_info.version);
	err = aft_check_flash_id(&card->netif, sdlacfg->cardnum, USER_SECTOR_FLASH, &man_id, &dev_id);
	if (err){
		sdlacfg->flash = AFTPRG_DEF_FAILED;
		err = -EINVAL;
		goto aft_program_flash_done;
	}
	if (sdlacfg->prg_boot_sector){
		err = aft_update_flash(&card->netif, sdlacfg->cardnum, DEF_SECTOR_FLASH,
					core_info.filename,
					core_info.version,
					sdlacfg->only_verify);
		if (err){
			sdlacfg->flash = AFTPRG_DEF_FAILED;
			err = -EINVAL;
			goto aft_program_flash_done;
		}
	}

	err = aft_update_flash(&card->netif, sdlacfg->cardnum, USER_SECTOR_FLASH,
				core_info.filename,
				core_info.version,
				sdlacfg->only_verify);
	if (err){
		sdlacfg->flash = AFTPRG_USER_FAILED;
		err = -EINVAL;
		goto aft_program_flash_done;
	}
	sdlacfg->flash = AFTPRG_OK;
	
	if (sdlacfg->only_verify){
		/* Verification is DONE! */
		goto aft_program_flash_done;
	}
	/* If Boot sector was update, boot first from the boot and 
	 * check flash id's */
	aft_reload_code(&card->netif, 
			(sdlacfg->prg_boot_sector) ? 
					DEF_SECTOR_FLASH : USER_SECTOR_FLASH);
	usleep(1000000);
	exec_bios_cmd(&card->netif);

	if (sdlacfg->prg_boot_sector){
		int def_man_id, def_dev_id;
		
		CloseConnection(&card->netif);
		system("./unload.sh");
		system("./load.sh");

		if (MakeConnection(&card->netif, 1)){
			sdlacfg->flash = AFTPRG_USER_FAILED;
			err = -EINVAL;
			goto aft_program_flash_done;
		}
		err = aft_check_flash_id(&card->netif, 
					sdlacfg->cardnum, 
					DEF_SECTOR_FLASH, 
					&def_man_id, 
					&def_dev_id);
		if (err){
			sdlacfg->flash = AFTPRG_USER_FAILED;
			err = -EINVAL;
			goto aft_program_flash_done;
		}
		if (def_man_id != man_id || def_dev_id != dev_id){
			aft_log(AFT_LOG_ERR, "Default Flash Sector is not properly updated (0x%02X, 0x%02X)!\n",
					def_man_id, def_man_id);
			sdlacfg->flash = AFTPRG_USER_FAILED;
			err = -EINVAL;
			goto aft_program_flash_done;
		}
		aft_reload_code(&card->netif, USER_SECTOR_FLASH);
		usleep(1000000);
		exec_bios_cmd(&card->netif);
	}
	 
aft_program_flash_done:
	CloseConnection(&card->netif);
	system("./unload.sh");
	return err;
}
