#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
#include <unistd.h>
#include <sched.h>
#include <sys/ipc.h>
#include <sys/sem.h>
#include <sys/time.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <string.h>
#include <errno.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <signal.h>

#include "aft_log.h"
#include "aft_main.h"
#include "aft_ioctl.h"
#include "aft_test.h"

/*
******************************************************************************
**			DEFINES/MACROS				
******************************************************************************
*/
#define SDLATST_TIMEOUT			1	
#define SDLATST_OOB_MSG			2
#define SDLATST_ERR_SEND		3
#define SDLATST_ERR_RCV			4
#define SDLATST_ERR_FILE		5
#define SDLATST_ERR_LOG			6
#define SDLATST_SEND_DONE		7
#define SDLATST_ERR_WANROUTER		8
#define SDLATST_ERR_RCV_CORRUPTION	9
#define SDLATST_ERR_RCV_FIFO		10
#define SDLATST_ERR_RCV_CRC		11
#define SDLATST_ERR_RCV_ABORT		12
#define SDLATST_ERR_RCV_LEN		13
#define SDLATST_ERR_RCV_SHORT_LEN	14
#define SDLATST_ERR_RCV_TIMEOUT		15

#define MAX_DATA_LEN	2000
/*
******************************************************************************
**			TYPEDEF/STRUCTURES				
******************************************************************************
*/

/*
******************************************************************************
**			GLOBAL VARIABLES				
******************************************************************************
*/
extern int verbose;
/*
******************************************************************************
**			FUNCTION PROTOTYPES				
******************************************************************************
*/

/*
******************************************************************************
**			FUNCTION DEFINITIONS				
******************************************************************************
*/

static int get_tx_pkt(netif_t* netif)
{
	api_tx_element_t*	api_tx_el;
	unsigned char		byte;
	int i;

	memset(&netif->txdata[0], 0, netif->txsize + sizeof(api_tx_hdr_t));
	api_tx_el = (api_tx_element_t*)&netif->txdata[0];
	byte = (unsigned char)netif->tx_ind;
	for (i=0; i < netif->txsize; i++){
		api_tx_el->data[i] = (unsigned char)byte++;
	}
	if (!netif->read_enable){
		netif->tx_byte0 = api_tx_el->data[0];
	}
	return 0;
}

static int socket_error(int error_flag, netif_t* netif, int offset, unsigned char byte1, unsigned char org_byte, unsigned char* rx_data, int rx_len)
{

	aft_log(AFT_LOG_FILE, 
			"ERROR: %s %d H %d %d DI %d %d\n",
			netif->ifname,
			netif->sock,
			netif->txsize,
			netif->txcnt,
			netif->read_enable,
			netif->write_enable);
	aft_log(AFT_LOG_FILE, 
		 "ERROR: %s (%d): Rx: Last test: txsize=%d, txcnt=%d (rx_ind=%d, tx_ind=%d).\n",
		 netif->ifname,
		 netif->sock,
		 netif->txsize,
		 netif->txcnt,
		 netif->rx_ind,
		 netif->tx_ind);

	switch(error_flag){
	case SDLATST_ERR_RCV:
		aft_log(AFT_LOG_FILE,
			 "ERROR: %s (%d): Rx: Failed to receive data!\n",
			 netif->ifname,
			 netif->sock);
		break;

	case SDLATST_ERR_RCV_CORRUPTION:
		aft_log(AFT_LOG_FILE, 
			 "ERROR: %s (%d): Rx: Packet corruption on offset %d, rx_byte=%02X:%02X!\n",
			 netif->ifname,
			 netif->sock,
			 offset,
			 byte1, 
			 org_byte);
		break;

	case SDLATST_ERR_RCV_FIFO:
		aft_log(AFT_LOG_FILE, 
			 "ERROR: %s (%d): Rx: Packet with fifo overflow err=%d length %d!\n",
			 netif->ifname, 
			 netif->sock,
			 byte1,
			 offset);
		break;

	case SDLATST_ERR_RCV_CRC:
		aft_log(AFT_LOG_FILE, 
			 "ERROR: %s (%d): Rx: Packet with CRC overflow err=%d length %d!\n",
			 netif->ifname, 
			 netif->sock,
			 byte1,
			 offset);
		break;

	case SDLATST_ERR_RCV_ABORT:
		aft_log(AFT_LOG_FILE, 
			 "ERROR: %s (%d): Rx: Packet with fifo overflow err=%d length %d!\n",
			 netif->ifname, 
			 netif->sock,
			 byte1,
			 offset);
		break;

	case SDLATST_ERR_RCV_LEN:
		aft_log(AFT_LOG_FILE, 
			 "ERROR: %s (%d): Rx: Wrong length %d should be %d!\n",
			 netif->ifname, 
			 netif->sock,
			 offset,
			 netif->txsize);
		break;

	case SDLATST_ERR_RCV_SHORT_LEN:
               	aft_log(AFT_LOG_FILE, 
			 "ERROR: %s (%d): Rx: Frame is too short %d!\n",
			 netif->ifname,
			 netif->sock,
                         offset);
		break;
	case SDLATST_ERR_RCV_TIMEOUT:
		aft_log(AFT_LOG_INFO, 
			"\nERROR: %s (%d): Timeout expired on SELECT (rx_ind=%d,tx_ind=%d)!\n",
			netif->ifname,
			netif->sock,
			netif->rx_ind,
			netif->tx_ind);
		aft_log(AFT_LOG_FILE, 
			"ERROR: %s (%d): Timeout expired on SELECT (rx_ind=%d,tx_ind=%d)!\n",
			netif->ifname,
			netif->sock,
			netif->rx_ind,
			netif->tx_ind);
		aft_log(AFT_LOG_INFO, 
			"\nERROR: %s (%d): Timeout expired on SELECT (rx_ind=%d,tx_ind=%d)!\n",
			netif->ifname,
			netif->sock,
			netif->rx_ind,
			netif->tx_ind);
		goto socket_error_done;
		break;
	}

socket_error_done:
	return SDLATST_ERR_RCV;
}

static int socket_oob(netif_t* netif, int ret)
{

	if(ret < 0 ) {
		aft_log(AFT_LOG_FILE, 
			"ERROR: %s (%d): Rx: Failed to receive OOB %d (rx_ind=%d,tx_ind=%d)\n", 
			netif->ifname,
			netif->sock,
		       	ret,
			netif->rx_ind,
			netif->tx_ind);
		ret = ioctl(netif->sock,SIOC_WANPIPE_SOCK_STATE,0);
		aft_log(AFT_LOG_FILE,
			"%s: Sock state is %s (%d)\n",
			netif->ifname, 
			(ret == 0) ? "CONNECTED" : 
				(ret == 1) ? "DISCONNECTED" :
				 "CONNECTING", ret);
	}else{
		aft_log(AFT_LOG_FILE, "%s: GOT OOB EXCEPTION CMD Exiting\n", 
				netif->ifname);
	}
	return SDLATST_OOB_MSG;
}


static int compare_rx_data(netif_t* netif, unsigned char* rxdata)
{
	unsigned char		byte = 0;
	int			i = 0;

	byte = netif->tx_byte0;
	for (i=0; i < netif->txsize; i++){
		if (rxdata[i] != (unsigned char)byte++){
			/* Return sample byte to original
			 * value.
			 */
			byte--;
			socket_error(SDLATST_ERR_RCV_CORRUPTION,
					netif,
					i,
					(unsigned char)rxdata[i],
					(unsigned char) byte,
					rxdata,
					netif->txsize);
			return SDLATST_ERR_RCV;
		}
	}
	byte = (char)netif->rx_ind;
	netif->tx_byte0 = (char)byte;
	return SDLATST_OK;
}


static int socket_send(netif_t* netif, int ret)
{
	api_tx_element_t*	api_tx_el;
	int			Tx_lgth;
	
	if (ret <= 0){
		if (errno == EBUSY){
			aft_socket_log(AFT_LOG_INFO,
					"%s: Sock busy try again!\n",
					netif->ifname);
			/* Socket busy try sending again !*/
		}else{
			perror("Send: ");
	 		/* Check socket state */
			ret = ioctl(netif->sock,SIOC_WANPIPE_SOCK_STATE,0);
			aft_log(AFT_LOG_FILE,
					"ERROR: %s: Sock state is %s\n",
					netif->ifname,
					(ret == 0) ? "CONNECTED" : 
						(ret == 1) ? "DISCONNECTED" :
						 "CONNECTING");

			aft_log(AFT_LOG_FILE, 
					"ERROR: %s: Faild to send %d",
					netif->ifname, 
					errno);
			return SDLATST_ERR_SEND;
		}
	}else{

               	Tx_lgth = ret - sizeof(api_tx_hdr_t);
		api_tx_el = (api_tx_element_t*)&netif->txdata[0];
				
		if (ret != netif->txsize+sizeof(api_tx_hdr_t)){
			aft_log(AFT_LOG_FILE,
					"ERROR: %s: Failed to send %d(%d) bytes\n",
					netif->ifname,
					ret,
					netif->txsize+sizeof(api_tx_hdr_t));
			return SDLATST_ERR_SEND;
		}

		netif->tx_ind++;
		aft_test_log(AFT_LOG_INFO, 
				"%s: Packet sent: Sent %d : %d\n",
				netif->ifname,
				ret - sizeof(api_tx_hdr_t),
				netif->tx_ind);
		get_tx_pkt(netif);
	}

	return SDLATST_OK;
}

static int socket_rcv_hdlc(netif_t* netif, int ret)
{
	api_rx_element_t*	api_rx_el;
	unsigned short		Rx_lgth;
	int 			err = SDLATST_OK;

	/* err indicates bytes received */
	if(ret > 0) {
		api_rx_el = (api_rx_element_t*)&netif->rxdata[0];
               	Rx_lgth = ret - sizeof(api_rx_hdr_t);

		if (api_rx_el->api_rx_hdr.error_flag & (1<<WP_FIFO_ERROR_BIT)){
			socket_error(SDLATST_ERR_RCV_FIFO, 
					netif, 
					Rx_lgth, 
					api_rx_el->api_rx_hdr.error_flag, 
					0x00,
					api_rx_el->data,
					0);
//			return SDLATST_ERR_RCV;

		}
		if (api_rx_el->api_rx_hdr.error_flag & (1<<WP_CRC_ERROR_BIT)){
			socket_error(SDLATST_ERR_RCV_CRC, 
					netif,
					Rx_lgth, 
					api_rx_el->api_rx_hdr.error_flag, 
					0x00,
					NULL,
					0);
//			return SDLATST_ERR_RCV;

		}
		if (api_rx_el->api_rx_hdr.error_flag & (1<<WP_ABORT_ERROR_BIT)){
			socket_error(SDLATST_ERR_RCV_ABORT, 
					netif,
					Rx_lgth, 
					api_rx_el->api_rx_hdr.error_flag, 
					0x00,
					NULL,
					0);
//			return SDLATST_ERR_RCV;

		}
		if (Rx_lgth != netif->txsize){
			/* Rx pkt length not the same */
			socket_error(SDLATST_ERR_RCV_LEN, 
					netif,
					Rx_lgth, 
					0x00, 
					0x00,
					api_rx_el->data,
					Rx_lgth);
			return SDLATST_ERR_RCV;
		}

		/* Check the packet length */
               	if (Rx_lgth<=0){
			socket_error(SDLATST_ERR_RCV_SHORT_LEN, 
					netif,
					Rx_lgth, 
					0x00, 
					0x00,
					api_rx_el->data,
					Rx_lgth);
                       	return SDLATST_ERR_RCV;
               	}

		netif->rx_ind++;

		aft_test_log(AFT_LOG_INFO,
				"%s: Received %d Olen=%d Length = %d\n", 
				netif->ifname,
				netif->rx_ind, 
				ret,
				Rx_lgth);

		/* Compare the rx/tx packets */
		err = compare_rx_data(netif, (char*)&api_rx_el->data[0]); 
	
	}else{
		perror("recv");
		socket_error(SDLATST_ERR_RCV, 
				netif, 
				0, 
				0x00, 
				0x00,
				NULL,
				0);
		return SDLATST_ERR_RCV;
	}
		
	return err;
}



/***************************************************
* HANDLE SOCKET 
*
*   o Read a socket 
*   o Cast data received to api_rx_element_t data type 
*   o The received packet contains 16 bytes header 
*
*	------------------------------------------
*      |  16 bytes      |        X bytes        ...
*	------------------------------------------
* 	   Header              Data
*
*   o Data structures:
*
*	typedef struct {
*        	unsigned char   SIO      	PACKED;
*        	unsigned short  time_stamp      PACKED;
*        	unsigned char   reserved[13]    PACKED;
*	} api_rx_hdr_t; 
*
*	typedef struct {
*        	api_rx_hdr_t    api_rx_hdr      PACKED;
*        	unsigned char   data[1]         PACKED;
*	} api_rx_element_t;
*
*
*/

int handle_socket(netif_t* netif) 
{
	api_tx_element_t*	api_tx_el;
	fd_set			ready, write, oob;
	int			serr= 0,err = 0, ret = 0;
	int			tlen, rlen_max;
	struct timeval		timeout;

        netif->rx_ind = 0;
	netif->tx_ind = 0;
	netif->write_enable = 1;
	netif->read_enable = 0;
	tlen = netif->txsize + sizeof(api_tx_hdr_t);
	rlen_max = MAX_DATA_LEN;
		
	aft_test_log(AFT_LOG_INFO, "%s %d H %d %d DI %d %d\n",
				netif->ifname,
				netif->sock,
				netif->txsize,
				netif->txcnt,
				netif->read_enable,
				netif->write_enable);

	/* If running HDLC_STREAMING then the received CRC bytes
         * will be passed to the application as part of the 
         * received data.  
	 */
	memset(&netif->txdata[0], 0, tlen);
	api_tx_el = (api_tx_element_t*)&netif->txdata[0];
	get_tx_pkt(netif);

	for(;;) {	
		FD_ZERO(&ready);
		FD_ZERO(&write);
		FD_ZERO(&oob);
		FD_SET(netif->sock,&oob);
		if (netif->read_enable){
			FD_SET(netif->sock,&ready);
		}
		if (netif->write_enable){
			FD_SET(netif->sock,&write);
		}

		timeout.tv_sec = 10;
		timeout.tv_usec = 0;
		serr=select(netif->sock + 1,&ready, &write, &oob, &timeout);

		if (serr < 0){
			printf("SELECT RETURNED %i\n",serr);
			continue;
		}

  		if(serr > 0){

			fflush(stdout);	
		   	if (FD_ISSET(netif->sock,&oob)){
		
				ret = recv(netif->sock, 
						netif->rxdata, 
						tlen, 
						MSG_OOB);

				err = socket_oob(netif, ret);
				break;
			}
			
                   	if (FD_ISSET(netif->sock,&ready)){
				api_rx_element_t* api_rx_el = NULL;

				netif->read_enable = 0;
				netif->write_enable = 1;
				api_rx_el = (api_rx_element_t*)&netif->rxdata[0];
				memset(&netif->rxdata[0], 0, rlen_max); 
				ret = recv(netif->sock, 
						netif->rxdata,
						rlen_max,
						0);
				err = socket_rcv_hdlc(netif, ret);
				if (err != SDLATST_OK){
					break;
				}
				if (!(netif->rx_ind % 1000)) aft_log(AFT_LOG_INFO, ".");	
				if (netif->rx_ind >= netif->txcnt){
					break;
				}
			}
		
		   	if (FD_ISSET(netif->sock,&write)){

				netif->read_enable = 1;
				netif->write_enable = 0;
				ret = send(netif->sock, 
					   netif->txdata, 
					   tlen, 
					   0);

				if ((err = socket_send(netif, ret)) != SDLATST_OK){
					aft_log(AFT_LOG_ERR, "Failed to send packet %s\n",
							netif->ifname);
				}
				
				if (netif->tx_ind >= netif->txcnt){
					netif->write_enable = 0;
				}
				
		   	}
			if (!timeout.tv_sec && !timeout.tv_usec){
				socket_error(SDLATST_ERR_RCV_TIMEOUT, 
						netif,
						0, 
						0x00, 
						0x00,
						NULL,
						0);
				err = SDLATST_TIMEOUT;
				break;
			}

		}else{
			socket_error(SDLATST_ERR_RCV_TIMEOUT, 
					netif,
					0, 
					0x00, 
					0x00,
					NULL,
					0);
			err = SDLATST_TIMEOUT;
			break;
		}
	}
	return err;
} 


static int aft_run_test(sdlacard_t* card, int txsize, int txcnt)
{
	netif_t*	netif = &card->netif;

	for(; netif; netif = netif->next){
		MakeConnection(netif, 0);
		netif->txsize = txsize;
		netif->txcnt  = txcnt;

		if (!(netif->pid = fork())){
			int err = SDLATST_OK;
			
			aft_test_log(AFT_LOG_INFO,
					"%s: Start test... (child=%d)!\n",
					netif->ifname, 
					getpid());
			if (handle_socket(netif)){
				err = SDLATST_FAILED;
			}
			aft_test_log(AFT_LOG_INFO, 
					"%s: Test is finished (err=%d, child=%d)!\n",
					netif->ifname, 
					err, 
					getpid());
			CloseConnection(netif);
			exit(err);
		}else if (netif->pid < 0){
			perror("fork");
			return errno;
		}
		CloseConnection(netif);
	}
	return 0;

}

int aft_test(sdlaprod_t* sdlacfg)
{
	sdlacard_t*	card = NULL;
	netif_t*	netif = NULL;
	int		iface_no = 0, status, pid_child, err = 0;

	aft_log(AFT_LOG_INFO, "\nStart AFT production test: ");
	for(card = sdlacfg->head; card; card = card->next){
		if (wanrouter_run(card, WANROUTER_DEVICE | WANROUTER_START)){
			aft_log(AFT_LOG_ERR, "%s: Failed to start device!\n",
					card->devname);
			err = -EINVAL;
			goto aft_test_exit;
		}
		for(netif = &card->netif; netif; netif = netif->next){
			aft_log(AFT_LOG_INFO, "%s ", netif->ifname);
			netif->test = SDLATST_FAILED;
		}
		iface_no += card->iface_no;
	}
	aft_log(AFT_LOG_INFO, "\n");

	for(card = sdlacfg->head; card; card = card->next){
		if (aft_run_test(card, sdlacfg->txsize, sdlacfg->txcnt)){
			aft_log(AFT_LOG_ERR, "Failed to start test for %s\n",
					card->devname);
			iface_no -= card->iface_no;
		}
	}
	while(1){
		status = 0;
		pid_child = wait(&status);
		iface_no --;
		if (WEXITSTATUS(status) != SDLATST_OK){
			/* One of the tests is FAILED.
			 * 1. Terminate all tests that still running 
			 * 2. Return and exit from program
			 */
			for(card = sdlacfg->head; card; card = card->next){
				for(netif = &card->netif; netif; netif = netif->next){
					if (netif->pid == pid_child){
						aft_proc_log(AFT_LOG_INFO,
							"%s: Child process %d exited with error!\n", 
							netif->ifname, 
							pid_child);
						/* This process already finished */
						netif->pid = 0;
					}
				}
			}
			err = SDLATST_FAILED;
		}else{
			for(card = sdlacfg->head; card; card = card->next){
				for(netif = &card->netif; netif; netif = netif->next){
					if (netif->pid == pid_child){
						netif->test = SDLATST_OK;
						aft_proc_log(AFT_LOG_INFO,
							"%s: Child process %d exited!\n", 
							netif->ifname, 
							pid_child);
						break;
					}
				}
			}
		}
		if (!iface_no){
			break;
		}
	}

aft_test_exit:
	for(card = sdlacfg->head; card; card = card->next){
		wanrouter_run(card, WANROUTER_DEVICE | WANROUTER_STOP);
	}
	return err;
}
