
#include <linux/version.h>
#include <linux/config.h>	/* OS configuration options */
#include <linux/stddef.h>	/* offsetof(), etc. */
#include <linux/errno.h>	/* return codes */
#include <linux/string.h>	/* inline memset(), etc. */
#include <linux/slab.h>		/* kmalloc(), kfree() */
#include <linux/kernel.h>	/* printk(), and other useful stuff */
#include <linux/module.h>	/* support for loadable modules */
#include <linux/ioport.h>	/* request_region(), release_region() */

#include <linux/in.h>		/* sockaddr_in */
#include <linux/inet.h>	
#include <linux/if.h>

#include <linux/pci.h>
#include <linux/if_arp.h>	/* ARPHRD_* defines */

#include <asm/uaccess.h>
#include <linux/netdevice.h>

# include <linux/wanpipe_includes.h>
# include <linux/wanpipe_defines.h>
# include <linux/wanpipe_debug.h>
# include <linux/wanpipe_common.h>
# include <linux/wanpipe_cfg.h>
# include <linux/sdladrv.h>
# include <linux/sdlapci.h>

#include "aft_te1_hdlc.h"

MODULE_AUTHOR ("Alex Feldman <al.feldman@sangoma.com>");
MODULE_DESCRIPTION ("Sangoma AFT Production Driver");
#if LINUX_VERSION_CODE > KERNEL_VERSION(2,4,9) 
MODULE_LICENSE("GPL");
#endif

#define MODNAME  "aft_prod"

#define MTAG "aft_te1_hdlc"

static struct sdla_hdlc_priv*	card_list;	
struct pci_dev	g_pci_dev;

//static unsigned int g_base_bar0_addr=0xFEDFFC00;
//static unsigned int g_irq=0x0F05000A;

//#define SDLA_HDLC_T3_PCI_VENDOR_ID 0x11b0
#define SDLA_HDLC_T3_PCI_VENDOR_ID 0x1923
#define SDLA_HDLC_T3_PCI_DEVICE_ID 0x0300

#define	HDLC_DEV_ALLOC 		0x00000001
#define	HDLC_DEV_PRIV_ALLOC 	0x00000002
#define	HDLC_PCI_BAR0_REQ	0x00000004
#define HDLC_PCI_BAR1_REQ	0x00000008
#define HDLC_PCI_BAR0_IOREMAP	0x00000010
#define HDLC_PCI_BAR1_IOREMAP	0x00000020
#define HDLC_PCI_ENABLE		0x00000040
#define HDLC_PCI_IRQ		0x00000080

#define MAX_RX_DMA_BUF_LEN 	5000

void unregister_hw(struct sdla_hdlc_priv* card)
{
	printk(KERN_INFO "%s: Shutdown device!\n", 
				card->devname);

	card->hwiface.down(card->hw);
#if 0
	free_irq(card->irq, card->dev);
#endif
	return;
}

int hdlc_if_open(struct net_device *dev)
{
	if (netif_running(dev))
		return -EBUSY;

	MOD_INC_USE_COUNT;
	return 0;
}

int hdlc_if_close(struct net_device *dev)
{
	MOD_DEC_USE_COUNT;
	return 0;
	
}

int hdlc_if_do_ioctl(struct net_device *dev, struct ifreq *ifr, int cmd)
{
	struct sdla_hdlc_priv*	card= (struct sdla_hdlc_priv *)dev->priv, *tmp;
	struct sdla_hdlc_api	api_cmd;
	char*			data;
	int			err = 0, offset = 0;
		
	if (!card){
		return -ENODEV;
	}

	if (!ifr->ifr_data){
		return -EINVAL;
	}

	if (copy_from_user(&api_cmd,ifr->ifr_data,sizeof(struct sdla_hdlc_api))){
		return -EFAULT;
	}

	switch (cmd){

	case SDLA_HDLC_READ_REG:

		//card->hwiface.peek(card->hw, api_cmd.offset, &api_cmd.data[0], api_cmd.len);
		//printk(KERN_INFO "%s: Reading Bar%i Offset=0x%X Len=%i Data=0x%X\n",
		//		MODNAME,api_cmd.bar,api_cmd.offset,api_cmd.len,
		//		api_cmd.data[0]);
		data = kmalloc(api_cmd.len + 1,GFP_KERNEL);
		if (data == NULL){
			return -EINVAL;
		}
		for(tmp = card_list; tmp; tmp = tmp->next_card){
			if (tmp->cpu != SDLA_CPU_A){
				memset(data, 0xFF, api_cmd.len);
				continue;      
			}
			if (api_cmd.len == 1){
				tmp->hwiface.bus_read_1(
						tmp->hw, 
						api_cmd.offset, 
						(u8*)data);
			}else if (api_cmd.len == 2){
				tmp->hwiface.bus_read_2(
						tmp->hw, 
						api_cmd.offset, 
						(u16*)data);
			}else if (api_cmd.len == 4){
				tmp->hwiface.bus_read_4(
						tmp->hw, 
						api_cmd.offset, 
						(u32*)data);
			}else{
				tmp->hwiface.peek(
						tmp->hw, 
						api_cmd.offset, 
						data, 
						api_cmd.len);
			}
			memcpy(&api_cmd.data[offset], data, api_cmd.len);
			offset += api_cmd.len;
		}
		api_cmd.len = offset;
		if (data){
			kfree(data);
		}

		break;

	case SDLA_HDLC_WRITE_REG:

		//printk(KERN_INFO "%s: Write  Offset=0x%04X Data=0x%08X\n",
		//		card->devname, api_cmd.offset,
		//		*(unsigned int*)&api_cmd.data[0]);
		for(tmp = card_list; tmp; tmp = tmp->next_card){
			if (tmp->cpu != SDLA_CPU_A){
				continue;      
			}
			if (api_cmd.len == 1){
				tmp->hwiface.bus_write_1(
						tmp->hw, 
						api_cmd.offset, 
						*(u8*)&api_cmd.data[0]);
			}else if (api_cmd.len == 2){
				tmp->hwiface.bus_write_2(
						tmp->hw, 
						api_cmd.offset, 
						*(u16*)&api_cmd.data[0]);
			}else if (api_cmd.len == 4){
				tmp->hwiface.bus_write_4(
						tmp->hw, 
						api_cmd.offset, 
						*(u32*)&api_cmd.data[0]);
			}else{
				tmp->hwiface.poke(
						tmp->hw, 
						api_cmd.offset, 
						&api_cmd.data[0], 
						api_cmd.len);
			}
		}
		break;

	case SDLA_HDLC_SET_PCI_BIOS:

		printk(KERN_INFO "%s: Setting PCI 0x10=0x%08X   0x3C=0x%08X\n",
				card->devname, card->pci_base_addr,
				 card->pci_irq);
		for(tmp = card_list; tmp; tmp = tmp->next){

			tmp->hwiface.pci_write_config_dword(tmp->hw, 
					(tmp->cpu == SDLA_CPU_A) ? 
						PCI_IO_BASE_DWORD :
						PCI_MEM_BASE0_DWORD,
					tmp->pci_base_addr);
			tmp->hwiface.pci_write_config_dword(
					tmp->hw,
				       	0x3C, 
					tmp->pci_irq);
			tmp->hwiface.pci_write_config_dword(
					tmp->hw, 
					0x04,
					tmp->pci_status_cmd);
			tmp->hwiface.pci_write_config_dword(
					tmp->hw, 
					0x0C,
					tmp->pci_0x0C);
		}
		break;
		
	default:
		break;
	}

	if (copy_to_user(ifr->ifr_data,&api_cmd,sizeof(struct sdla_hdlc_api))){
		return -EFAULT;
	}

	
	return err;
	
}

void sdla_hdlc_isr (int irq, void* dev_id, struct pt_regs *regs)
{

	struct net_device *dev = (struct net_device*)dev_id;	

	if (!dev->priv){
		if (net_ratelimit()){
			printk(KERN_INFO "%s: ISR: no priv struct\n",MODNAME);
		}
		return;
	}

	if (net_ratelimit()){
		printk(KERN_INFO "%s: ISR Executed for device %s\n",
					MODNAME,dev->name);
	}
	return;
}


int init_dev(struct net_device* dev)
{
	struct sdla_hdlc_priv*	card;	
	int err;

	card = (struct sdla_hdlc_priv*)dev->priv;
	card->hw = sdla_register(&card->hwiface, NULL, card->devname);

	card->hwiface.getcfg(card->hw, SDLA_CARDTYPE, &card->type);
	card->hwiface.getcfg(card->hw, SDLA_CPU, &card->cpu);

	err = card->hwiface.setup(card->hw, NULL);
	if (err){

		printk(KERN_INFO "%s: Hardware setup Failed %i\n",
				card->devname, err);
		sdla_unregister(&card->hw, card->devname);
		return err;
	}

	card->hwiface.getcfg(card->hw, SDLA_IRQ, &card->irq);

	sprintf(dev->name,"wp%daft1", card->cnt);

	dev->open		=&hdlc_if_open;
	dev->stop		=&hdlc_if_close;
//	dev->init		=&hdlc_if_init;
//	dev->hard_start_xmit	=&hdlc_if_send;
//	dev->get_stats          =&hdlc_if_get_stats;
//	dev->tx_timeout		=&hdlc_if_tx_timeout;
//	dev->watchdog_timeo     = (1*HZ);	
	dev->do_ioctl		=&hdlc_if_do_ioctl;

	/* Initialize media-specific parameters */
	dev->flags		|= IFF_POINTOPOINT;
	dev->flags		|= IFF_NOARP;

	dev->type		= ARPHRD_PPP;
	dev->mtu		= 1500;
	dev->hard_header_len	= 0;
        dev->tx_queue_len 	= 100;

#if 0
	if (request_irq(card->irq, sdla_hdlc_isr, 
		       SA_SHIRQ, 
		       MODNAME, dev)){
		printk(KERN_INFO "%s: Can't reserve IRQ %d!\n", 
					MODNAME, card->irq);
		return -EFAULT;
	}
#endif

	err = register_netdev(dev);
	if (err){
		printk(KERN_INFO "%s: Failed register netdev %s!\n",
				card->devname, dev->name);
		return err;
	}
	printk(KERN_INFO "%s:%s: PCI Device loaded OK\n",
			card->devname, dev->name);
	return 0;	
}

int init_module(void)
{
	struct sdla_hdlc_priv*	card, *tmpcard;	
	struct net_device*	dev;
	int i, cnt = 1, err = 0, ncards;

	ncards=0;

	/* Probe for wanpipe cards and return the number found */
	printk(KERN_INFO "wanpipe: Probing for WANPIPE hardware.\n");
	ncards = sdla_hw_probe();
	if (ncards){
		printk(KERN_INFO "wanpipe: Allocating maximum %i devices: wanpipe%i - wanpipe%i.\n",
					ncards,1,ncards);
	}else{
		printk(KERN_INFO "wanpipe: No S514/S508 cards found, unloading modules!\n");
		return -ENODEV;
	}

	for (i=0;i<ncards;i++){
		unsigned int value;
		dev = kmalloc(sizeof(struct net_device),GFP_KERNEL);
		if (!dev){
			printk(KERN_INFO "%s: Failed to allocate dev\n",
					MODNAME);
			err=-ENOMEM;
			goto init_module_err;
		}
		memset(dev,0,sizeof(struct net_device));

		card=kmalloc(sizeof(struct sdla_hdlc_priv),GFP_KERNEL);
		if (!card){
			err=-ENOMEM;
			goto init_module_err;
		}
		memset(card,0,sizeof(struct sdla_hdlc_priv));
		card->cnt = cnt++;
		sprintf(card->devname, "wanpipe%d", card->cnt);
		dev->priv = (void*)card;
		card->dev = dev;
		if (init_dev(dev)){
			cnt--;
			kfree(card);
			kfree(dev);
			continue;
#if 0
			printk(KERN_INFO "%s: Failed to initialize network device!\n",
					MODNAME);
			err = -EINVAL;
			goto init_module_err;
#endif
		}
		card->hwiface.pci_read_config_dword(
					card->hw, 
					0x04,
					&card->pci_status_cmd);
		card->hwiface.pci_read_config_dword(
					card->hw, 
					0x0C,
					&card->pci_0x0C);
		card->hwiface.getcfg(
					card->hw,
				       	SDLA_BASEADDR,
				       	&card->pci_base_addr);
		card->hwiface.pci_read_config_dword(
					card->hw,
					0x3C,
					&card->pci_irq);
		//printk(KERN_INFO "%s:%s: Read global base addr 0x%08X\n",
		//	       	card->devname, dev->name, card->pci_base_addr);
		//printk(KERN_INFO "%s:%s: Read global irq 0x%08X\n",
		//	       	card->devname, dev->name, card->pci_irq);

		if (card_list == NULL){
			card_list = card;
		}else{
			for(tmpcard = card_list;tmpcard->next;tmpcard = tmpcard->next);
			tmpcard->next = card;
			if (card->cpu == SDLA_CPU_A){
				for(tmpcard = card_list;tmpcard->next_card;tmpcard = tmpcard->next_card);
				tmpcard->next_card = card;
			}
		}
	}
	
	return 0;

init_module_err:

	for(card = card_list; card;){
		
		if (card->dev){
			printk(KERN_INFO "%s: Unregister network interface!\n",
					card->dev->name);
			unregister_netdev(card->dev);
		}
		unregister_hw(card);
		tmpcard = card;
		card = card->next;
		if (tmpcard->dev){
			kfree(tmpcard->dev);
			tmpcard->dev=NULL;
		}
		kfree(tmpcard);
	}

	return err;
}


void cleanup_module (void)
{
	struct sdla_hdlc_priv *card, *tmpcard;

	for(card = card_list; card;){
		
		if (card->dev){
			printk(KERN_INFO "%s: Unregister network interface!\n",
					card->dev->name);
			unregister_netdev(card->dev);
		}

		unregister_hw(card);
	
		tmpcard = card;
		card = card->next;
		if (tmpcard->dev){
			kfree(tmpcard->dev);
			tmpcard->dev=NULL;
		}
		kfree(tmpcard);
	}
	return;
}
