#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <sys/time.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <string.h>
#include <errno.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <linux/if.h>
#include <linux/if_packet.h>
#include "wan_aft_cpld.h"
#include "../wan_aften/wan_aften_api.h"

#define WAN_AFTUP_VERSION	"1.1"

struct wan_aftup_api api_cmd;
int sock;
unsigned char if_name[20];
static struct ifreq	req;
struct if_settings 	ifsettings;

int progress_bar(char*);
void main_menu(void);
void read_chip_control_menu(void);
void registers_and_internal_ram_test_menu(void);
void setup_chip_configuration_menu(void);
int exec_command(int cmd);
int exec_reload_pci_cmd(void);
int exec_read_cmd(unsigned int off, unsigned int len, unsigned int *data);
int exec_write_cmd(unsigned int off, unsigned int len, unsigned int data);

void hit_any_key(void);

extern int pmc_initialization (int sock);
//extern int cpld_initialization (int sock);
extern int check_flash_id(int, int, int*);
extern int reload_flash(int);
extern int update_flash(int, int, char*);

//struct cmd_list_struct cmd_list[]={
//	{"READ", SIOC_WAN_READ_REG},
//	{"WRITE",SIOC_WAN_WRITE_REG},
//	{".",0xFF}
//};

int progress_bar(char *msg)
{
	static int index = 0;

	if (index++ == 0){
		printf("\r%s |", msg);
	}else if (index++ == 1){
		printf("\r%s /", msg);
	}else{
		printf("\r%s -", msg);
		index = 0;
	}
	fflush(stdout);
	return 0;
}
int MakeConnection(void) 
{
	struct sockaddr_ll sa;
	struct ifreq ifr;
	
	memset(&sa,0,sizeof(struct sockaddr_ll));
	errno = 0;
   	sock = socket(AF_PACKET, SOCK_DGRAM, 0);
   	if( sock < 0 ) {
      		perror("Socket");
      		return -EIO;
   	} /* if */
  
	strcpy(ifr.ifr_name,if_name);
	if (ioctl(sock,SIOCGIFINDEX,&ifr)){
		perror("Get index: ");
		return -EIO;
	}

	sa.sll_protocol = htons(0x17);
	sa.sll_family=AF_PACKET;
	sa.sll_ifindex = ifr.ifr_ifindex;

        if(bind(sock, (struct sockaddr *)&sa, sizeof(struct sockaddr_ll)) < 0){
                perror("bind");
		printf("Failed to bind a socket to %s interface\n",if_name);
                exit(0);
        }
	return 0;

}

int CloseConnection(void)
{
	close(sock);
	return 0;
}

int san_aftup_ioctl(struct wan_aftup_api *api_cmd, int cmd)
{
	int ret;

	req.ifr_data = (void*)api_cmd;
	strcpy(req.ifr_name, if_name);
	if ((ret = ioctl(sock, cmd, &req)) < 0){
		return -EINVAL;
	}
	return 0;
}

static int san_aftup_getfile(char *filename)
{
	FILE *f;
	
	printf("Specify new flash filename: ");
	scanf("%s",filename);

	f = fopen(filename, "rb");
	if (f == NULL){
		return -EINVAL;
	}
	fclose(f);
	printf("\n");
	return 0;
}

static int san_aftup_program(char flash_rev)
{
	char	filename[100];
	int	flash_id, tmp;
	char	ch;
	
	/* Check Flash ID */
	check_flash_id(MEMORY_TYPE_FLASH, USER_SECTOR_FLASH, &flash_id);
	if (flash_id != 0x014F && flash_id != 0x20E3){
		printf("Failed to read Flash id (%04X)\n",
				flash_id);
		return -EINVAL;
	}
	printf("Sangoma Flash ID is 0x%04X!\n", flash_id);

	if (san_aftup_getfile(filename)){
		printf("Bad file name or file doesn't exists\n");
		return -EINVAL;
	}

	if (update_flash(USER_SECTOR_FLASH, MEMORY_TYPE_FLASH, filename)){
		printf("Failed to re-program flash!\n");
		return -EINVAL;
	}
	
	printf("Sangoma Flash is updated!\n");
	printf("\nYou have to reload flash in order to new flash code will take place\n");
	printf("Do you want to reload now (Y/N)?");
	ch = getchar();
	ch = getchar();
	if (ch != 'y' && ch != 'Y'){
		printf("Please, reboot your computer before using Sangoma AFT Series card\n");
		printf("New flash code will reload while rebooting computer!\n");
		return 0;
	}
	printf("Reloading flash....\n");
	reload_flash(USER_SECTOR_FLASH);

	usleep(1000000);

	/* Read flash revision */
	if (san_aftup_ioctl(&api_cmd, SIOC_WAN_SET_PCI_BIOS)){
		printf("Failed!\n");
		return -EINVAL;
	}

	/* Check Flash ID */
	check_flash_id(MEMORY_TYPE_FLASH, USER_SECTOR_FLASH, &tmp);
	if (tmp != flash_id){
		printf("Failed to read Flash ID (new flash id %04X)\n",
				tmp);
		return -EINVAL;
	}
	
	/* Read new Flash revision */
	if (san_aftup_ioctl(&api_cmd, SIOC_WAN_COREREV)){
		printf("Failed to read new Flash revision!\n");
		return -EINVAL;
	}
	printf("New Sangoma Flash revision %X\n",
				api_cmd.data[0]);
	if (api_cmd.data[0] < flash_rev){
		printf("New flash revision is %d (previous flash revision %d)\n",
				api_cmd.data[0], flash_rev);
		printf("You re-program flash with older version!\n");
		printf("Please, contact Sangoma Technologies (905.474.1990)!\n");
		return -EINVAL;
	}

	printf("Sangoma Flash updated successfully!\n\n");
	return 0;	
}

static int san_aftup_start(void)
{

	/* Print hardware configuration */
	if (san_aftup_ioctl(&api_cmd, SIOC_WAN_HWPROBE)){
		printf("Failed to read hardware configuration!\n");
		return -EINVAL;
	}
	
	/* Read revision */
	if (san_aftup_ioctl(&api_cmd, SIOC_WAN_COREREV)){
		printf("Failed to read flash revision!\n");
		return -EINVAL;
	}
	printf("Sangoma Flash revision %X\n",
				api_cmd.data[0]);

	if (san_aftup_program(api_cmd.data[0])){
		printf("Failed to re-program flash!\n");
		return -EINVAL;
	}

	return 0;
}

static unsigned char title_info[]=
"Sangoma AFT Series card update flash software";

static unsigned char usage_info[]="\n"
"Usage:\n"
"	wan_aftup -i <interface name>\n"
"	wan_aftup -help\n"
"	wan_aftup -v\n";

void usage (void)
{
	printf("%s\n",title_info);
	printf("%s\n",usage_info);
}

void version (void)
{
	printf("%s",title_info);
	printf(" (version %s)\n", WAN_AFTUP_VERSION);
}

void hit_any_key(void)
{
	printf("Hit any key to continue... \n");
	getchar();
	getchar();
}

int main(int argc, char* argv[])
{
	int i=0;
	int err;

	memset(&api_cmd,0,sizeof(struct wan_aftup_api));
	api_cmd.cmd=0xFF;

	if (argc == 1){
		usage();
		return 0;
	}
	for (i = 0; i < argc; i++){

		if (!strcmp(argv[i],"-h")){
			usage();
			return 0;
		}else if (!strcmp(argv[i],"-v")){
			version();
			return 0;
		}else if (!strcmp(argv[i],"-i")){
			strcpy(if_name, argv[i+1]);
			i++;
		}
	}

	printf("\n%s.\n\n",title_info);
	MakeConnection();

	err = san_aftup_start();	
	
	CloseConnection();
	return err;
}

int exec_read_cmd(unsigned int off, unsigned int len, unsigned int *data)
{
	int err;
	struct ifreq ifr;

	strcpy(ifr.ifr_name,if_name);
	ifr.ifr_data = (char*)&api_cmd;

	api_cmd.cmd=SIOC_WAN_READ_REG;
	api_cmd.bar=0;
	api_cmd.len=len;
	api_cmd.offset=off;

	err = ioctl(sock,api_cmd.cmd,&ifr);
	if (err){
		perror("Read Cmd Exec: ");
	}

	if (len==1){
		*(unsigned char*)data = *(unsigned char*)api_cmd.data;
	}else if (len==2){
		*(unsigned short*)data = *(unsigned short*)api_cmd.data;
	}else{
		*(unsigned int*)data = *(unsigned int*)api_cmd.data;
	}

	return err;

}

int exec_write_cmd(unsigned int off, unsigned int len, unsigned int data)
{
	int err;
	struct ifreq ifr;

	strcpy(ifr.ifr_name,if_name);
	ifr.ifr_data = (char*)&api_cmd;

	api_cmd.cmd=SIOC_WAN_WRITE_REG;
	api_cmd.bar=0;
	api_cmd.len=len;
	api_cmd.offset=off;

	if (len==1){
		*(unsigned char*)api_cmd.data=(unsigned char)data;
	}else if (len==2){
		*(unsigned short*)api_cmd.data=(unsigned short)data;
	}else{
		*(unsigned int*)api_cmd.data=(unsigned int)data;
	}

	err = ioctl(sock,api_cmd.cmd,&ifr);
	if (err){
		perror("Write Cmd Exec: ");
	}

	return err;
}


