/*
 *	LAPB release 002
 *
 *	This code REQUIRES 2.1.15 or higher/ NET3.038
 *
 *	This module:
 *		This module is free software; you can redistribute it and/or
 *		modify it under the terms of the GNU General Public License
 *		as published by the Free Software Foundation; either version
 *		2 of the License, or (at your option) any later version.
 */

#include "wanpipe_lapb.h"

/* 
 *  This procedure is passed a buffer descriptor for an iframe. It builds
 *  the rest of the control part of the frame and then writes it out.
 */
static int lapb_send_iframe(lapb_cb *lapb, void *skb, int poll_bit)
{
	unsigned char *frame;

	if (skb == NULL)
		return -1;

	if (wpabs_skb_tailroom(skb) < 4){
		wpabs_skb_free(skb);
		wpabs_debug_event( "BAD ROOM IN SEND FRAME\n");
		return -1;
	}

	if (lapb->mode & LAPB_EXTENDED) {
		frame = wpabs_skb_push(skb, 2);

		frame[0] = LAPB_I;
		frame[0] |= lapb->vs << 1;
		frame[1] = (poll_bit) ? LAPB_EPF : 0;
		frame[1] |= lapb->vr << 1;
	} else {
		frame = wpabs_skb_push(skb, 1);

		*frame = LAPB_I;
		*frame |= (poll_bit) ? LAPB_SPF : 0;
		*frame |= lapb->vr << 5;
		*frame |= lapb->vs << 1;
	}

	return lapb_transmit_buffer(lapb, skb, LAPB_COMMAND,0);	
}


int lapb_transmit_buffer(lapb_cb *lapb, void *skb, int type, char async)
{
	unsigned char *ptr;
	//int err;
	
	FUNC_BEGIN();

	ptr = wpabs_skb_push(skb, 1);

	if (lapb->mode & LAPB_MLP) {
		if (lapb->mode & LAPB_DCE) {
			if (type == LAPB_COMMAND)
				*ptr = LAPB_ADDR_C;
			if (type == LAPB_RESPONSE)
				*ptr = LAPB_ADDR_D;
		} else {
			if (type == LAPB_COMMAND)
				*ptr = LAPB_ADDR_D;
			if (type == LAPB_RESPONSE)
				*ptr = LAPB_ADDR_C;
		}
	} else {
		if (lapb->mode & LAPB_DCE) {
			if (type == LAPB_COMMAND)
				*ptr = LAPB_ADDR_A;
			if (type == LAPB_RESPONSE)
				*ptr = LAPB_ADDR_B;
		} else {
			if (type == LAPB_COMMAND)
				*ptr = LAPB_ADDR_B;
			if (type == LAPB_RESPONSE)
				*ptr = LAPB_ADDR_A;
		}
	}

#if 1
	if (async){
		wpabs_skb_queue_tail(lapb->async_queue,skb);
		return 0;
	}

	return lapb_data_transmit(lapb, skb);
#else

	err = lapb_data_transmit(lapb, skb);
	
	if (err && async){
		wpabs_skb_queue_tail(lapb->async_queue,skb);
	}
	return err;
#endif
}

void lapb_establish_data_link(lapb_cb *lapb)
{
	lapb->condition = 0x00;
	lapb->n2count   = 0;

	/* Nenad Corbic: Initailize all queues before
	 * starting the link */
	lapb_clear_queues(lapb);
	
	if (lapb->mode & LAPB_EXTENDED) {
		lapb_send_control(lapb, LAPB_SABME, LAPB_POLLON, LAPB_COMMAND);
	} else {
		lapb_send_control(lapb, LAPB_SABM, LAPB_POLLON, LAPB_COMMAND);
	}

	lapb_start_t1timer(lapb);
	lapb_stop_t2timer(lapb);
}

void lapb_enquiry_response(lapb_cb *lapb)
{
	if (lapb->condition & LAPB_REJECT_CONDITION){
		lapb_send_control(lapb, LAPB_REJ, LAPB_POLLON, LAPB_RESPONSE);
	}else if (lapb->condition & LAPB_RNR_SENT_CONDITION){
		lapb_send_control(lapb, LAPB_RNR, LAPB_POLLON, LAPB_RESPONSE);
	}else{
		lapb_send_control(lapb, LAPB_RR, LAPB_POLLON, LAPB_RESPONSE);
	}

	lapb->condition &= ~LAPB_ACK_PENDING_CONDITION;
}

void lapb_timeout_response(lapb_cb *lapb)
{

	if (wpabs_skb_queue_len(lapb->rx_queue) > LAPB_RX_Q_H_LIMIT){
		lapb_send_control(lapb, LAPB_RNR, LAPB_POLLOFF, LAPB_RESPONSE);
	}else{
		lapb_send_control(lapb, LAPB_RR, LAPB_POLLOFF, LAPB_RESPONSE);
	}

	lapb->condition &= ~LAPB_ACK_PENDING_CONDITION;
}

void lapb_check_iframes_acked(lapb_cb *lapb, unsigned short nr)
{
	if (lapb->vs == nr) {
		lapb_frames_acked(lapb, nr);
		lapb_stop_t1timer(lapb);
		lapb->n2count = 0;
	} else {
		if (lapb->va != nr) {
			lapb_frames_acked(lapb, nr);
			lapb_start_t1timer(lapb);
		}
	}
}

void lapb_check_need_response(lapb_cb *lapb, int type, int pf)
{
	if (type == LAPB_COMMAND && pf){
		lapb_enquiry_response(lapb);
	}
}


int lapb_bh(lapb_cb *lapb)
{
	void *skb, *skbn;
	volatile unsigned short modulus, start, end;
	int err=0, data_frame_sent_status=0;

	FUNC_BEGIN();
	
	if (wpabs_test_and_set_bit(LAPB_SMP_CRIT,&lapb->critical)){
		wpabs_debug_event( "CRITICAL IN LAPB KICK !!!---!!!\n");
		return 0;
	}

	if (wpabs_test_bit(LAPB_UNREG_CRIT, &lapb->critical)){
		wpabs_clear_bit(LAPB_AWAITING_KICK,&lapb->critical);
		wpabs_clear_bit(LAPB_FRAME_SENT,&lapb->critical);
		wpabs_clear_bit(LAPB_BH_RUNNING,&lapb->critical);
		wpabs_clear_bit(LAPB_SMP_CRIT,&lapb->critical);
		return 0;
	}

	++lapb->kick_cnt;
	
	wpabs_clear_bit(LAPB_FRAME_SENT,&lapb->critical);
	
lapb_bh_send_async:
	while ((skb=wpabs_skb_dequeue(lapb->async_queue)) != NULL){

		unsigned char *data=wpabs_skb_data(skb);

		/* Fix to include all packets that contain
		 * VR numbers */
		switch ((data[1]&0x0F)){
			case LAPB_RR:
			case LAPB_RNR:
			case LAPB_REJ:
				data[1] &= 0x1F;
				data[1] |= (lapb->vr << 5);
				break;
		}
		err = lapb_data_transmit(lapb, skb);

		if (err != 0){
			//wpabs_debug_event( "%s:Failed to send lapb Async Message!\n",
			//		lapb->dev->name);
			wpabs_skb_queue_head(lapb->async_queue,skb);
			wpabs_set_bit(LAPB_AWAITING_KICK,&lapb->critical);
			goto lapb_bh_exit;
		}
	}

	if (lapb->state != LAPB_STATE_CONNECTED)
		goto lapb_bh_exit;

	if (wpabs_test_bit(LAPB_REQUEUING,&lapb->critical)){
		wpabs_debug_event( "(Debug): Lapb requeuing skip kick!\n");
		goto lapb_bh_exit;
	}
	
	modulus = (lapb->mode & LAPB_EXTENDED) ? LAPB_EMODULUS : LAPB_SMODULUS;

        start = lapb->vs;
	end   = (lapb->va + lapb->window) % modulus;
	
	if (!(lapb->condition & LAPB_PEER_RX_BUSY_CONDITION) &&
	    start != end                                &&
	    wpabs_skb_queue_len(lapb->write_queue) != 0){

		/*
		 * Dequeue the frame and copy it.
		 */
		skb  = wpabs_skb_dequeue(lapb->write_queue);

		do {
			if (wpabs_skb_queue_len(lapb->async_queue)){
				//wpabs_debug_event( "Lapb kick re-sending async!\n");
				wpabs_skb_queue_head(lapb->write_queue, skb);
				goto lapb_bh_send_async;
			}

			
			if ((skbn = wpabs_skb_clone(skb)) == NULL) {
				wpabs_skb_queue_head(lapb->write_queue, skb);
				break;
			}
			
			/*
			 * Transmit the frame copy.
			 */
			err = lapb_send_iframe(lapb, skbn, LAPB_POLLOFF);
			
			if (err == 0){

				//printk(KERN_DEBUG "LAPB KICK PACKET OK: Vs=%d\n",
				//		lapb->vs);

				lapb_update_stats (lapb, LAPB_I, wpabs_skb_len(skb), 1);
	
				lapb->vs = (lapb->vs + 1) % modulus;
				/* Requeue the original data frame */
				wpabs_skb_queue_tail(lapb->ack_queue, skb);
	
				wpabs_set_bit(LAPB_FRAME_SENT,&lapb->critical);

			}else if (err == 1){
				/* Failed to Tx data, thus re-queue frame */
#if 0
				wpabs_debug_test("Lapb: Failed to TX Data to Lower layer! Vs=%d\n",
						lapb->vs);
#endif
				wpabs_skb_queue_head(lapb->write_queue, skb);

				wpabs_skb_free(skbn);
				
				wpabs_set_bit(LAPB_AWAITING_KICK,&lapb->critical);
				break;
			}else{
				/* Major Error */ 
				wpabs_debug_event( "Lapb: Failed to TX Data Major Error! Vs=%d\n",
						lapb->vs);
				wpabs_skb_queue_head(lapb->write_queue, skb);
				
				wpabs_skb_free(skbn);

				wpabs_set_bit(LAPB_AWAITING_KICK,&lapb->critical);
				break;
			}
			
		} while (lapb->vs != end && (skb = wpabs_skb_dequeue(lapb->write_queue)) != NULL);

		/* Only stop the RR response if we successfully sent
		 * a packet. Otherwise let the RR clear the window */
		if (wpabs_test_bit(LAPB_FRAME_SENT,&lapb->critical)){

			lapb->condition &= ~LAPB_ACK_PENDING_CONDITION;

			if (!lapb_t1timer_running(lapb))
				lapb_start_t1timer(lapb);
			
			wpabs_clear_bit(LAPB_FRAME_SENT,&lapb->critical);

			data_frame_sent_status=1;
		}
	}

lapb_bh_exit:
	
	wpabs_clear_bit(LAPB_SMP_CRIT,&lapb->critical);
	wpabs_clear_bit(LAPB_BH_RUNNING,&lapb->critical);

	return data_frame_sent_status;
#undef lapb
}

