/*
 *	LAPB release 002
 *
 *	This code REQUIRES 2.1.15 or higher/ NET3.038
 *
 *	This module:
 *		This module is free software; you can redistribute it and/or
 *		modify it under the terms of the GNU General Public License
 *		as published by the Free Software Foundation; either version
 *		2 of the License, or (at your option) any later version.
 *
 *	History
 */

#include "wanpipe_lapb.h"



void lapb_start_t1timer(lapb_cb *lapb)
{
	FUNC_BEGIN();

	if (!lapb->t1){
		return;
	}

	lapb->t1timer = wpabs_get_systemticks();
}

void lapb_start_t2timer(lapb_cb *lapb)
{
	FUNC_BEGIN();

	if (!lapb->t2){
		return;
	}
	
	lapb->t2timer = wpabs_get_systemticks();
}

void lapb_start_t4timer(lapb_cb *lapb, unsigned char reset_t4_cnt)
{
	FUNC_BEGIN();

	if (!lapb->t4){
		return;
	}

	lapb->t4timer = wpabs_get_systemticks();
	
	if (reset_t4_cnt){
		lapb->n4count=0;
	}
}

void lapb_stop_t4timer(lapb_cb *lapb)
{
	FUNC_BEGIN();
	lapb->t4timer=0;
}

void lapb_stop_t1timer(lapb_cb *lapb)
{
	FUNC_BEGIN();
	lapb->t1timer=0;
}

void lapb_stop_t2timer(lapb_cb *lapb)
{
	FUNC_BEGIN();
	lapb->t2timer=0;
}

int lapb_t1timer_running(lapb_cb *lapb)
{
	FUNC_BEGIN();
	return (lapb->t1timer != 0);
}

void lapb_t2timer_expiry(lapb_cb *lapb)
{
	FUNC_BEGIN();

	if (lapb->condition & LAPB_ACK_PENDING_CONDITION) {
		lapb->condition &= ~LAPB_ACK_PENDING_CONDITION;
		lapb_timeout_response(lapb);
	}
}

/*=========================================================
 * lapb_t4timer_expiry
 *
 * 	This timer simulates both T4 and T3 Timers.
 * 	
 * Case1: T3 and T4 defined by user:
 * 
 * 	The T3 value would be calcualted as multiple of T4,
 * 	(ie. T3 =  T3/T4) thus if T4 restarts T3 timers the 
 * 	line will be disconneted.
 *
 * Case2: T3 not set, but T4 set by user:
 *
 * 	Not allowed:
 *
 * Case3: T3 is set by T4 is not:
 *
 *     In this case T4 will default to T3 value give by
 *     the user, and the T3 count will be 1.  
 *     The poll frames will not be sent out but the
 *     if the line is inactive for T3*T4 time, the
 *     line will be disconnected.
 *
 * Case4: T3 is not set and T4 is not set:
 *
 * 	This timer will not be used.
 */

void lapb_t4timer_expiry(lapb_cb *lapb)
{

	FUNC_BEGIN();
	
	if (wpabs_test_bit(LAPB_UNREG_CRIT,&lapb->critical)){
		return; 
	}

	if (lapb->t3 && ++lapb->n4count >= lapb->t3){
		wpabs_debug_event( "%s: T3 Timer Expiry: Line inactive, disconnecting ...\n",
					lapb->name);
		lapb->n4count=0;
		lapb_clear_queues(lapb);
		lapb_state_change(lapb,LAPB_STATE_DISCONNECTED);
		lapb_disconnect_indication(lapb, LAPB_TIMEDOUT);

		lapb_reconnect(lapb);
		return;
	}


	if (!lapb->t4_orig){
		lapb_start_t4timer(lapb,0);
		return;
	}
	
	switch (lapb->state) {
		
		case LAPB_STATE_CONNECTED:
			if (++lapb->n2count > lapb->n2){

				wpabs_debug_event( "%s: LapB N2 (%d) polls exceeded with no response\n",
					lapb->name,lapb->n2);

				lapb->n2count=0;
				lapb_clear_queues(lapb);
				lapb_state_change(lapb,LAPB_STATE_DISCONNECTED);
				lapb_disconnect_indication(lapb, LAPB_TIMEDOUT);

				lapb_reconnect(lapb);
				return;
			}
			
			if (wpabs_skb_queue_len(lapb->rx_queue) > LAPB_RX_Q_H_LIMIT){
				lapb_send_control(lapb, LAPB_RNR, LAPB_POLLON, LAPB_COMMAND);
			}else{
				lapb_send_control(lapb, LAPB_RR, LAPB_POLLON, LAPB_COMMAND);
			}

			break;

		case LAPB_STATE_FRAME_REJECT:
			lapb_transmit_frmr(lapb);
			break;

		default:
			wpabs_debug_event( "%s: T4 Timer Expiry: Invalid State (0x%02X) Disconnecting...\n",
					lapb->name, lapb->state);

			lapb_clear_queues(lapb);
			lapb_state_change(lapb,LAPB_STATE_DISCONNECTED);
			lapb_disconnect_indication(lapb, LAPB_TIMEDOUT);

			lapb_reconnect(lapb);
			return;
	}

	lapb_start_t4timer(lapb,0);
}


void lapb_t1timer_expiry(lapb_cb *lapb)
{

	FUNC_BEGIN();

	if (wpabs_test_bit(LAPB_UNREG_CRIT,&lapb->critical)){
		wpabs_debug_event( "Critical in T1 timer\n");
		return; 
	}

	switch (lapb->state) {

		/*
		 *	If we are a DCE, keep going DM .. DM .. DM
		 */
		case LAPB_STATE_DISCONNECTED:
		
			lapb_clear_queues(lapb);
			lapb_state_change(lapb,LAPB_STATE_CONNECTING);
				
		/*
		 *	Awaiting connection state, send SABM(E), up to N2 times.
		 */
		case LAPB_STATE_CONNECTING: 			

			if (lapb->n2count == lapb->n2) {

				lapb_send_control(lapb, LAPB_DM, LAPB_POLLOFF, LAPB_RESPONSE);

				lapb->n2count=0;
			} else {
				lapb->n2count++;

				if (lapb->mode & LAPB_EXTENDED) {

					lapb_send_control(lapb, LAPB_SABME, LAPB_POLLON, LAPB_COMMAND);
				} else {

					lapb_send_control(lapb, LAPB_SABM, LAPB_POLLON, LAPB_COMMAND);
				}
			}
			break;

		/*
		 *	Awaiting disconnection state, send DISC, up to N2 times.
		 */
		case LAPB_STATE_DISCONNECTING:
			if (lapb->n2count == lapb->n2) {

				lapb->n2count=0;
				lapb_clear_queues(lapb);
				lapb_state_change(lapb,LAPB_STATE_DISCONNECTED);
				lapb_disconnect_indication(lapb, LAPB_TIMEDOUT);



				/* NC: Never turn of T1 timer always
				 * restart the link */
				//return;
			} else {
				lapb->n2count++;

				lapb_send_control(lapb, LAPB_DISC, LAPB_POLLON, LAPB_COMMAND);
			}
			break;

		/*
		 *	Data transfer state, restransmit I frames, up to N2 times.
		 */
		case LAPB_STATE_CONNECTED:
			if (lapb->n2count == lapb->n2) {

				lapb->n2count=0;
				lapb_clear_queues(lapb);
				lapb_state_change(lapb,LAPB_STATE_DISCONNECTED);
				lapb_stop_t2timer(lapb);
				lapb_disconnect_indication(lapb, LAPB_TIMEDOUT);

				/* NC: Never turn of T1 timer always
				 * restart the link */
				//return;
			}else{
#if 0
				wpabs_debug_event("LAPB: T1 Expiry, Requeueing Frames!\n");
#endif
				lapb->n2count++;
				lapb_requeue_frames(lapb);
			}
			break;

		/*
		 *	Frame reject state, restransmit FRMR frames, up to N2 times.
		 */
		case LAPB_STATE_FRAME_REJECT:
			if (lapb->n2count == lapb->n2) {

				lapb->n2count=0;
				lapb_clear_queues(lapb);
				lapb_state_change(lapb,LAPB_STATE_DISCONNECTED);
				lapb_disconnect_indication(lapb, LAPB_TIMEDOUT);


				/* NC: Never turn of T1 timer always
				 * restart the link */
				//return;
			} else {
				lapb->n2count++;
				lapb_transmit_frmr(lapb);
			}
			break;
	}

	lapb_start_t1timer(lapb);
}

