
#if defined(__FreeBSD__) || defined(__OpenBSD__) || defined(__NetBSD__)
# include <net/wanpipe_lip.h>
#else
# include <linux/wanpipe_lip.h>
#endif

/* Function interface between LIP layer and kernel */
extern wan_iface_t wan_iface;

/*=============================================================
 * Funciton Prototypes
 */
extern int wplip_link_prot_change_state(void *wplip_id,int state,unsigned char*,int);
extern int wplip_lipdev_prot_change_state(void *wplip_id,int state,unsigned char*,int);

static int wplip_reg_fr_prot(wplip_link_t *lip_link, wanif_conf_t *conf);
static int wplip_unreg_fr_prot(wplip_link_t *lip_link);
static int wplip_add_fr_dlci(wplip_dev_t *lip_dev, wanif_conf_t *conf);
static int wplip_del_fr_dlci(wplip_dev_t *lip_dev);

static void wplip_prot_timer(wan_timer_arg_t arg);

static int wplip_reg_ppp_chdlc_prot(wplip_dev_t *lip_dev, wanif_conf_t *conf);
static int wplip_unreg_ppp_chdlc_prot(wplip_dev_t *lip_dev);

static int wplip_reg_lapb_prot(wplip_dev_t *lip_dev, wanif_conf_t *conf);
static int wplip_unreg_lapb_prot(wplip_dev_t *lip_dev);

static int wplip_prot_header(wplip_dev_t *lip_dev, netskb_t *skb);

static int wplip_reg_xdlc_prot(wplip_link_t *lip_link, wanif_conf_t *conf);
static int wplip_unreg_xdlc_prot(wplip_link_t *lip_link);
static int wplip_reg_xdlc_addr(wplip_dev_t *lip_dev, wanif_conf_t *conf);
static int wplip_unreg_xdlc_addr(wplip_dev_t *lip_dev);


/*==============================================================
 * Global Functions
 */

int wplip_reg_link_prot(wplip_link_t *lip_link, wanif_conf_t *conf)
{
	int err = -EINVAL;

	DEBUG_TEST("%s: (DEBUG) Registering Protocol 0x%X Xdlc 0x%X\n",
			lip_link->name,conf->protocol,WANCONFIG_XDLC);
	
	lip_link->protocol = conf->protocol;
	
	switch (conf->protocol){

	case WANCONFIG_FR:
		err = wplip_reg_fr_prot(lip_link,conf);
		break;

	case WANCONFIG_XDLC:
		err = wplip_reg_xdlc_prot(lip_link,conf);
		break;

	case WANCONFIG_PPP:
	case WANCONFIG_CHDLC:
	case WANCONFIG_LAPB:
		err = 0;
		break;
		
	}

	return err;
}



int wplip_unreg_link_prot(wplip_link_t *lip_link)
{
	int err = -EINVAL;

	if (!lip_link->prot){
		return 0;
	}
	
	switch (lip_link->protocol){

	case WANCONFIG_FR:
		err = wplip_unreg_fr_prot(lip_link);
		break;

	case WANCONFIG_XDLC:
		err = wplip_unreg_xdlc_prot(lip_link);
		break;

	case WANCONFIG_PPP:
	case WANCONFIG_CHDLC:
	case WANCONFIG_LAPB:
		err = 0;
		break;

	}

	return err;
}



int wplip_reg_lipdev_prot(wplip_dev_t *lip_dev, wanif_conf_t *conf)
{
	int err = 0;
	
	lip_dev->protocol = conf->protocol;

	if (lip_dev->protocol != lip_dev->lip_link->protocol){
		DEBUG_EVENT("%s: Config error: link prot 0x%x != lipdev prot 0x%X\n",
				lip_dev->lip_link->name,
				lip_dev->lip_link->protocol,
				lip_dev->protocol);
		return -EINVAL;
	}
	
	switch (conf->protocol){

	case WANCONFIG_FR:
		err = wplip_add_fr_dlci(lip_dev,conf);
		break;

	case WANCONFIG_PPP:
	case WANCONFIG_CHDLC:
		err = wplip_reg_ppp_chdlc_prot(lip_dev,conf); 
		break;

	case WANCONFIG_LAPB:
		err = wplip_reg_lapb_prot(lip_dev,conf); 
		break;

	case WANCONFIG_XDLC:
		err=  wplip_reg_xdlc_addr(lip_dev,conf);
		break;
	}

	return err;
}

int wplip_unreg_lipdev_prot(wplip_dev_t *lip_dev)
{
	int err=0;

	if (!lip_dev->common.prot_ptr){
		return 0;
	}
	
	switch (lip_dev->protocol){

	case WANCONFIG_FR:
		err = wplip_del_fr_dlci(lip_dev);
		break;

	case WANCONFIG_PPP:
	case WANCONFIG_CHDLC:
		err = wplip_unreg_ppp_chdlc_prot(lip_dev); 
		break;

	case WANCONFIG_LAPB:
		err = wplip_unreg_lapb_prot(lip_dev); 
		break;

	case WANCONFIG_XDLC:
		err=  wplip_unreg_xdlc_addr(lip_dev);
		break;
	}

	return err;
}


int wplip_open_lipdev_prot(wplip_dev_t *lip_dev)
{
	int err=0;
	
	switch (lip_dev->protocol){

	case WANCONFIG_FR:

#if defined(__LINUX__)
		lip_dev->common.dev->hard_header_len = 16;
#endif
		if ((err=wp_fr_open_chan(lip_dev->lip_link->prot, 
					 lip_dev->common.prot_ptr)) != 0){
			return err;
		}
		break;

	case WANCONFIG_PPP:
	case WANCONFIG_CHDLC:
#if defined(__LINUX__)
		wp_sppp_open(lip_dev->common.dev);
#else
		DEBUG_EVENT("%s: Set interface OPEN (not defined)!\n",
				wan_netif_name(lip_dev->common.dev)); 
#endif
		break;

	case WANCONFIG_LAPB:
		err = wp_lapb_open(lip_dev->common.prot_ptr); 
		break;

	case WANCONFIG_XDLC:
		err = wp_xdlc_open(lip_dev->common.prot_ptr); 
		break;

	}

	return err;
}


int wplip_close_lipdev_prot(wplip_dev_t *lip_dev)
{
	int err=0;
	
	switch (lip_dev->protocol){

	case WANCONFIG_FR:
		if ((err=wp_fr_close_chan(lip_dev->lip_link->prot, 
					  lip_dev->common.prot_ptr)) != 0){
			return err;
		}
		break;

	case WANCONFIG_PPP:
	case WANCONFIG_CHDLC:
#if defined(__LINUX__)
		wp_sppp_close(lip_dev->common.dev);
#else
		DEBUG_EVENT("%s: Set interface CLOSE (not defined)!\n",
				wan_netif_name(lip_dev->common.dev)); 
#endif
		break;

#if defined(__LINUX__)
	case WANCONFIG_LAPB:
		err = wp_lapb_close(lip_dev->common.prot_ptr); 
#endif
		break;

	case WANCONFIG_XDLC:
		err = wp_xdlc_close(lip_dev->common.prot_ptr); 
		break;
	
	}

	return err;
}

void wplip_prot_rx_kick(wplip_dev_t *lip_dev)
{
	switch (lip_dev->protocol){

	case WANCONFIG_XDLC:
		wp_xdlc_rx_kick(lip_dev->common.prot_ptr);
		break;
	}
}


void wplip_prot_kick(wplip_link_t *lip_link, wplip_dev_t *lip_dev)
{

	if (!lip_dev){
		DEBUG_EVENT("%s: Asserting error lip_dev=NULL \n",__FUNCTION__);
		return;
	}
	
	switch (lip_link->protocol){

	case WANCONFIG_XDLC:
		wp_xdlc_bh(lip_link->prot,lip_dev->common.prot_ptr);
		break;

	case WANCONFIG_LAPB:
	
		wp_lapb_bh(lip_dev->common.prot_ptr);
		break;
	}
}

void wplip_prot_rx(wplip_link_t *lip_link, netskb_t *skb)
{
	wplip_dev_t *lip_dev;
	int err;

	switch (lip_link->protocol){

	case WANCONFIG_LAPB:
		
		lip_dev=WAN_LIST_FIRST(&lip_link->list_head_ifdev);
		if (!lip_dev){
			wan_skb_free(skb);
			break;
		}

		if (lip_dev->magic != WPLIP_MAGIC_DEV){
			DEBUG_EVENT("%s:%s: Error:  Inv Dev Magic dev=%p Magic=0x%lX\n",
					__FUNCTION__,
					lip_link->name,
					lip_dev,
					lip_dev->magic);
			wan_skb_free(skb);
			break;
		}

		wp_lapb_rx(lip_dev->common.prot_ptr, skb);

		break;
	
	case WANCONFIG_XDLC:

		wp_xdlc_rx(lip_link->prot,skb);
		break;
		
	case WANCONFIG_PPP:
	case WANCONFIG_CHDLC:

		lip_dev=WAN_LIST_FIRST(&lip_link->list_head_ifdev);
		if (!lip_dev){
			wan_skb_free(skb);
			break;
		}
		
		if (lip_dev->magic != WPLIP_MAGIC_DEV){
			DEBUG_EVENT("%s:%s: Error:  Inv Dev Magic dev=%p Magic=0x%lX\n",
					__FUNCTION__,
					lip_link->name,
					lip_dev,
					lip_dev->magic);
			wan_skb_free(skb);
			break;
		}

#if defined(__LINUX__)
		wp_sppp_input(lip_dev->common.dev,skb);
#else
		if (wan_iface.input){
			wan_iface.input(lip_dev->common.dev, skb);
			/*sppp_input (lip_dev->common.dev,skb);*/
		}else{
			DEBUG_EVENT("%s:%d Function Assertion Error!\n",
					__FUNCTION__,__LINE__); 
			wan_skb_free(skb);
		}		
#endif
		break;

	case WANCONFIG_FR:

		if (!lip_link->prot){
			wan_skb_free(skb);
			return;
		}
		
		err=wp_fr_rx (lip_link->prot, skb, SYSTEM_TICKS);
		if (err != 0){
			wan_skb_free(skb);
		}
		break;

	default:
		DEBUG_EVENT("%s:%s: No Protocol defined dropping!\n",
				__FUNCTION__,lip_link->name);
		wan_skb_free(skb);
		break;
	}

	return;
}
	
int wplip_prot_oob(wplip_dev_t *lip_dev, unsigned char *data, int len)
{
	netskb_t *skb;
	wan_api_rx_hdr_t *rx_el;
	
#if defined(__LINUX__)

	if (!data || !len){
		return -1;
	}
	
	switch (lip_dev->protocol){
		
	case WANCONFIG_LAPB:
		skb=wan_skb_alloc(sizeof(wan_api_rx_hdr_t));
		if (!skb){
			return -1;
		}
		rx_el=(wan_api_rx_hdr_t*)wan_skb_push(skb,sizeof(wan_api_rx_hdr_t));

		/* FIXME: */
		rx_el->u.lapb.exception=0;	
		skb->protocol=htons(ETH_P_X25);
		break;

	case WANCONFIG_XDLC:
		{
		wp_xdlc_excep_t *xdlc_excep = (wp_xdlc_excep_t *)data;
		skb=wan_skb_alloc(sizeof(wan_api_rx_hdr_t));
		if (!skb){
			return -1;
		}
		rx_el=(wan_api_rx_hdr_t*)wan_skb_push(skb,sizeof(wan_api_rx_hdr_t));
		rx_el->u.xdlc.state=xdlc_excep->state;
		rx_el->u.xdlc.address=xdlc_excep->address;
		rx_el->u.xdlc.exception=xdlc_excep->exception;
		skb->protocol=htons(ETH_P_X25);
		}
		break;

	default:
		return 0;
	}
	
	skb->pkt_type = WAN_PACKET_ERR;

	if (wan_api_rx(lip_dev,skb)!=0){
		wan_skb_free(skb);
		return -1;
	}
	return 0; 
#else
	return 0;
#endif
}

int wplip_prot_ioctl(wplip_dev_t *lip_dev, int cmd, void *arg)
{	
#if defined(__LINUX__)	
	if (cmd < SIOC_WANPIPE_DEVPRIVATE){
		return -EINVAL;
	}

	cmd-=SIOC_WANPIPE_DEVPRIVATE;
	
	if (lip_dev->protocol == WANCONFIG_XDLC){
		return wp_xdlc_ioctl(lip_dev->common.prot_ptr,cmd,arg);
	}
	return -1;
#else
	return -EINVAL;
#endif
}

int wplip_prot_tx(wplip_dev_t *lip_dev, wan_api_tx_hdr_t *api_tx_hdr, netskb_t *skb)
{
	int err=0;
	
	if (lip_dev->protocol == WANCONFIG_LAPB){

		err=wp_lapb_tx(lip_dev->common.prot_ptr,skb);
		if (err){
			return 1;
		}
		return 0;
	}
	
	if (lip_dev->protocol == WANCONFIG_XDLC){

		err=wp_xdlc_tx(lip_dev->common.prot_ptr,skb);
		if (err){
			return 1;
		}
		return 0;
	}
	
	if (wan_skb_queue_len(&lip_dev->tx_queue) >= MAX_TX_BUF){
		return 1;
	}


	err=wplip_prot_header(lip_dev, skb);
	if (err < 0){
		return -1;
	}
	
	wan_skb_unlink(skb);
	wan_skb_queue_tail(&lip_dev->tx_queue,skb);
	return 0;
}


static int wplip_prot_header(wplip_dev_t *lip_dev, netskb_t *skb)
{
	int err=0;

	if (!lip_dev){
		DEBUG_EVENT("%s: Error: no lip dev!\n",
				__FUNCTION__);
		return -EINVAL;
	}

	if (lip_dev->protocol == WANCONFIG_FR){
#if defined(__LINUX__)
		switch(htons(skb->protocol)){
			case ETH_P_IP:
				err=wp_fr_tx_hdr(lip_dev->common.prot_ptr,skb,FR_MP_IP);
				break;
				
			case ETH_P_IPV6:
				err=wp_fr_tx_hdr(lip_dev->common.prot_ptr,skb,FR_MP_IP);
				break;
				
			default:
				err=wp_fr_tx_hdr(lip_dev->common.prot_ptr,skb,FR_MP_NONE);
				break;
		}
#else
		/* FIXME: INET/INET6 */
		err=wp_fr_tx_hdr(lip_dev->common.prot_ptr,skb,FR_MP_IP);

#endif
	}

	return err;
}


static void wplip_prot_timer(wan_timer_arg_t arg)
{
	wplip_link_t	*lip_link=(wplip_link_t *)arg;
	wplip_dev_t	*lip_dev;
	int		period;
	wan_smp_flag_t	flags;

	DEBUG_TEST("%s: PROT TIMER: %s  devcnt=%d \n",
			__FUNCTION__,lip_link->name,
			lip_link->dev_cnt);

	if (wan_test_bit(WPLIP_LINK_DOWN,&lip_link->tq_working)){
		return;
	}

	wan_spin_lock_irq(&lip_link->bh_lock,&flags);
	
	switch (lip_link->protocol){
		
	case WANCONFIG_LAPB:

		lip_dev=WAN_LIST_FIRST(&lip_link->list_head_ifdev);
		if (!lip_dev){
			break;
		}

		if (lip_dev->magic != WPLIP_MAGIC_DEV){
			DEBUG_EVENT("%s:%s: Error:  Inv Dev Magic dev=%p Magic=0x%lX\n",
					__FUNCTION__,
					lip_link->name,
					lip_dev,
					lip_dev->magic);
			break;
		}

		wp_lapb_timer(lip_dev->common.prot_ptr);

		if (wan_skb_queue_len(&lip_link->tx_queue)){
			wplip_trigger_bh(lip_link);
		}

		wan_add_timer(&lip_link->prot_timer,HZ);
		break;

	case WANCONFIG_XDLC:

		wp_xdlc_timer(lip_link->prot);

		if (wan_skb_queue_len(&lip_link->tx_queue)){
			wplip_trigger_bh(lip_link);
		}

		wan_add_timer(&lip_link->prot_timer,1);
		break;
		
	case WANCONFIG_FR:

		period = wp_fr_timer(lip_link->prot,SYSTEM_TICKS);
	
		if (wan_skb_queue_len(&lip_link->tx_queue)){
			wplip_trigger_bh(lip_link);
		}


		if (period){
			wan_add_timer(&lip_link->prot_timer,period*HZ);
		}

		break;
	}

	wan_spin_unlock_irq(&lip_link->bh_lock,&flags);
}




/*=============================================================
 * FRAME RELAY 
 *============================================================*/ 


static int wplip_fr_rx_up(void *lip_dev_ptr, void *skb_ptr, int type)
{
	wplip_dev_t *lip_dev = lip_dev_ptr;
	struct sk_buff *skb = skb_ptr;

	switch (type){

	case FR_MP_IP:
#if defined(__LINUX__)
		skb->protocol = htons(ETH_P_IP);
		skb->dev = lip_dev->common.dev;
		skb->mac.raw  = skb->data;
#endif
		wplip_data_rx_up(lip_dev,skb);
		break;

	case FR_MP_IPV6:
#if defined(__LINUX__)
		skb->protocol = htons(ETH_P_IPV6);
		skb->dev = lip_dev->common.dev;
		skb->mac.raw  = skb->data;
#endif
		wplip_data_rx_up(lip_dev,skb);
		break;

	case FR_MP_ARP:

		DEBUG_EVENT("%s: FR ARP not supported \n",lip_dev->name);
		wan_skb_free(skb);
		break;
		
	case FR_MP_NONE:

		DEBUG_EVENT("%s: FR RAW not supported \n",lip_dev->name);
		wan_skb_free(skb);
		break;

	default:
		DEBUG_EVENT("%s: FR unknown packet 0x%X\n",
				lip_dev->name,type);
		wan_skb_free(skb);
		break;
	}

	return 0;
}


static int wplip_reg_fr_prot(wplip_link_t *lip_link, wanif_conf_t *conf)
{
	wp_fr_reg_t prot_reg;
	wan_fr_conf_t fr_cfg;
	
	memset(&fr_cfg,0,sizeof(wan_fr_conf_t));
	memset(&prot_reg,0,sizeof(wp_fr_reg_t));
	memcpy(&fr_cfg,&conf->u.fr,sizeof(wan_fr_conf_t));

#if 0
	fr_cfg.signalling=conf->u.fr.signalling;
	fr_cfg.station=conf->station;
	fr_cfg.t391=10;
	fr_cfg.t392=16;
	fr_cfg.n391=6;
	fr_cfg.n392=3;
	fr_cfg.n393=4;
#endif	
	prot_reg.prot_set_state = wplip_link_prot_change_state;
	prot_reg.chan_set_state = wplip_lipdev_prot_change_state;
	prot_reg.mtu=1500;
	prot_reg.tx_down=wplip_link_callback_tx_down;
	prot_reg.rx_up=wplip_fr_rx_up;

	
	DEBUG_EVENT("\n");

	DEBUG_EVENT("%s: Registering Frame Relay protocol\n",
				lip_link->name);
	
	lip_link->prot=wp_register_fr_prot(lip_link, lip_link->name, &fr_cfg, &prot_reg); 
	if (!lip_link->prot){
		DEBUG_EVENT("%s: Failed to register FR protocol %d\n",
			lip_link->name,lip_link->protocol);
		return -EINVAL;
	}

	wan_init_timer(&lip_link->prot_timer,wplip_prot_timer,(wan_timer_arg_t)lip_link);

	wan_add_timer(&lip_link->prot_timer,HZ);

	return 0;
}

static int wplip_unreg_fr_prot(wplip_link_t *lip_link)
{
	if (lip_link->prot){
		WAN_TASKLET_KILL((&lip_link->task));
		wan_del_timer(&lip_link->prot_timer);
		wp_unregister_fr_prot(lip_link->prot);
		lip_link->prot=NULL;
	}

	return 0;
}


static int wplip_add_fr_dlci(wplip_dev_t *lip_dev, wanif_conf_t *conf)
{
	int dlci=0;
	
	if (is_digit(conf->addr[0])) {
		dlci = dec_to_uint(conf->addr, 0);
	}
	
	lip_dev->common.prot_ptr = wp_register_fr_chan(lip_dev, 
						       lip_dev->lip_link->prot, 
						       dlci, 
						       lip_dev->name);
	if (!lip_dev->common.prot_ptr){
		DEBUG_EVENT("%s: Failed to register FR DLCI %d\n",
			lip_dev->name,dlci);
		return -EINVAL;
	}

	return 0;
}

static int wplip_del_fr_dlci(wplip_dev_t *lip_dev)
{
	if (lip_dev->common.prot_ptr){
		wp_unregister_fr_chan(lip_dev, lip_dev->lip_link->prot, lip_dev->common.prot_ptr);
		lip_dev->common.prot_ptr=NULL;
	}

	return 0;
}



/*=============================================================
 * PPP CHDLC 
 *============================================================*/ 


static int wplip_reg_ppp_chdlc_prot(wplip_dev_t *lip_dev, wanif_conf_t *conf)
{
#if defined(__LINUX__)	
	struct ppp_device *pppdev;
	struct sppp *sp;

	pppdev=kmalloc(sizeof(struct ppp_device),GFP_KERNEL);
	if (!pppdev){
		lip_dev->protocol=0;
		return -ENOMEM;
	}

	memset(pppdev,0,sizeof(struct ppp_device));

	lip_dev->common.prot_ptr=(void*)pppdev;

	pppdev->dev=lip_dev->common.dev;

	/* Get authentication info. */
	if(conf->pap == WANOPT_YES){
		pppdev->sppp.myauth.proto = PPP_PAP;
	}else if(conf->chap == WANOPT_YES){
		pppdev->sppp.myauth.proto = PPP_CHAP;
	}else{
		pppdev->sppp.myauth.proto = 0;
	}

	if(pppdev->sppp.myauth.proto){
		memcpy(pppdev->sppp.myauth.name, conf->userid, AUTHNAMELEN);
		memcpy(pppdev->sppp.myauth.secret, conf->passwd, AUTHNAMELEN);
	}

	pppdev->sppp.gateway = conf->gateway;

	if (conf->if_down){
		pppdev->sppp.dynamic_ip = 1;
	}

	sprintf(pppdev->sppp.hwdevname,"%s",lip_dev->lip_link->name);
	
	wp_sppp_attach(pppdev);

	sp = &pppdev->sppp;
	
	lip_dev->common.dev->type = ARPHRD_PPP;

	if (lip_dev->protocol == WANCONFIG_CHDLC){
		DEBUG_EVENT("%s: Starting Kernel CISCO HDLC protocol\n",
				lip_dev->name);
		sp->pp_flags |= PP_CISCO;
	}else{
		DEBUG_EVENT("%s: Starting Kernel Sync PPP protocol\n",
				lip_dev->name);
		sp->pp_flags &= ~PP_CISCO;
	}
#else
	lip_dev->common.is_netdev = 0;
	/*WAN_SPPP_ATTACH(dev);*/
	if (wan_iface.attach){
		wan_iface.attach(lip_dev->common.dev, NULL, lip_dev->common.is_netdev);
	}
#endif
	return 0;
}

static int wplip_unreg_ppp_chdlc_prot(wplip_dev_t *lip_dev)
{

	if (!lip_dev->common.dev){
		return -ENODEV;
	}	
	
#if defined(__LINUX__)
	wp_sppp_detach(lip_dev->common.dev);
	lip_dev->common.dev->do_ioctl = NULL;
	lip_dev->common.dev->hard_header = NULL;
	lip_dev->common.dev->rebuild_header = NULL;
#else
	/*WAN_SPPP_DETACH(dev);*/
	if (wan_iface.detach){
		wan_iface.detach(lip_dev->common.dev, lip_dev->common.is_netdev);
	}
#endif
	

	if (lip_dev->common.prot_ptr){
		wan_free(lip_dev->common.prot_ptr);
		lip_dev->common.prot_ptr= NULL;
	}

	return 0;
}

/*=============================================================
 * LAPB 
 *============================================================*/ 


static int wplip_reg_lapb_prot(wplip_dev_t *lip_dev, wanif_conf_t *conf)
{
	wp_lapb_reg_t lapb_reg;
	wplip_link_t *lip_link = lip_dev->lip_link;

	memset(&lapb_reg,0,sizeof(wp_lapb_reg_t));

	lapb_reg.prot_state_change=wplip_link_prot_change_state;
	lapb_reg.chan_state_change=wplip_lipdev_prot_change_state;
	lapb_reg.tx_down=wplip_callback_tx_down;
	lapb_reg.rx_up=wplip_data_rx_up;
		
	lip_dev->common.prot_ptr = wp_register_lapb_prot(lip_dev, 
						         lip_dev->name,
						    	 &conf->u.lapb,
							 &lapb_reg);

	if (lip_dev->common.prot_ptr == NULL){
		DEBUG_EVENT("%s: Failed to register LAPB protocol %d\n",
			lip_link->name,lip_link->protocol);
		return -EINVAL;
	}

	wan_init_timer(&lip_link->prot_timer,wplip_prot_timer,(wan_timer_arg_t)lip_link);
	wan_add_timer(&lip_link->prot_timer,HZ);
	return 0;
}

static int wplip_unreg_lapb_prot(wplip_dev_t *lip_dev)
{
	wplip_link_t *lip_link = lip_dev->lip_link;

	if (lip_dev->common.prot_ptr){
		WAN_TASKLET_KILL((&lip_link->task));
		wan_del_timer(&lip_link->prot_timer);
		wp_unregister_lapb_prot(lip_dev->common.prot_ptr);
		lip_dev->common.prot_ptr=NULL;
	}

	return 0;
}

/*=============================================================
 * XDLC 
 *============================================================*/ 

static int wplip_reg_xdlc_prot(wplip_link_t *lip_link, wanif_conf_t *conf)
{
	wp_xdlc_reg_t xdlc_reg;

	memset(&xdlc_reg,0,sizeof(wp_xdlc_reg_t));

	xdlc_reg.prot_state_change=wplip_link_prot_change_state;
	xdlc_reg.chan_state_change=wplip_lipdev_prot_change_state;
	xdlc_reg.tx_down=wplip_callback_tx_down;
	xdlc_reg.rx_up=wplip_data_rx_up;
		
	lip_link->prot = wp_reg_xdlc_prot(lip_link, 
					  lip_link->name,
					  &conf->u.xdlc,
					  &xdlc_reg);

	if (lip_link->prot == NULL){
		DEBUG_EVENT("%s: Failed to register LAPB protocol %d\n",
			lip_link->name,lip_link->protocol);
		return -EINVAL;
	}

	wan_init_timer(&lip_link->prot_timer,wplip_prot_timer,(wan_timer_arg_t)lip_link);
	wan_add_timer(&lip_link->prot_timer,HZ);

	return 0;
}

static int wplip_unreg_xdlc_prot(wplip_link_t *lip_link)
{
	if (lip_link->prot){

		WAN_TASKLET_KILL((&lip_link->task));
		wan_del_timer(&lip_link->prot_timer);
		wp_unreg_xdlc_prot(lip_link->prot);
		lip_link->prot=NULL;

	}

	return 0;
}

static int wplip_reg_xdlc_addr(wplip_dev_t *lip_dev, wanif_conf_t *conf)
{
	int address=0;
	
	if (is_digit(conf->addr[0])) {
		address = dec_to_uint(conf->addr, 0);
	}

	conf->u.xdlc.address=address;
	
	lip_dev->common.prot_ptr = wp_reg_xdlc_addr(lip_dev, 
						    lip_dev->lip_link->prot, 
						    lip_dev->name,
						    &conf->u.xdlc);
	if (!lip_dev->common.prot_ptr){
		DEBUG_EVENT("%s: Failed to register XDLC Address %d\n",
			lip_dev->name,address);
		return -EINVAL;
	}

	return 0;

}

static int wplip_unreg_xdlc_addr(wplip_dev_t *lip_dev)
{
	if (lip_dev->common.prot_ptr){
		wp_unreg_xdlc_addr(lip_dev->lip_link->prot, 
				   lip_dev->common.prot_ptr);
		lip_dev->common.prot_ptr=NULL;
	}
	return 0;
}



