/******************************************************************************
 * 
 * sdla_te3.c	Sangoma T3/E3 front end.
 *
 * 		Alex Feldman <al.feldman@sangoma.com>
 * 		
 * 	Copyright Sangoma Technologies Inc. 1999, 2000,2001, 2002, 2003, 2004
 *
 * 	This program is provided subject to the Software License included in 
 * 	this package in the file license.txt. By using this program you agree
 * 	to be bound bythe terms of this license. 
 *
 * 	Should you not have a copy of the file license.txt, or wish to obtain
 * 	a hard copy of the Software License, please contact Sangoma
 * 	technologies Corporation.
 *
 * 	Contact: Sangoma Technologies Inc. 905-474-1990, info@sangoma.com
 *
 *****************************************************************************/


/******************************************************************************
**			   INCLUDE FILES
******************************************************************************/

#if defined(__NetBSD__) || defined(__FreeBSD__) || defined(__OpenBSD__)
# include <net/wanpipe_config.h>
# include <net/wanpipe_includes.h>
# if !defined(CONFIG_PRODUCT_WANPIPE_GENERIC)
#  include <net/wanpipe_snmp.h>
# endif
# include <net/wanpipe.h>	/* WANPIPE common user API definitions */
#elif (defined __LINUX__) || (defined __KERNEL__)
# include <linux/wanpipe_includes.h>
# include <linux/sdla_te3_reg.h>
# include <linux/wanpipe.h>	/* WANPIPE common user API definitions */
#else
# error "No OS Defined"
#endif

/******************************************************************************
**			  DEFINES AND MACROS
******************************************************************************/

/******************************************************************************
**			  DEFINES AND MACROS
******************************************************************************/

#if defined(DEBUG)
#define WRITE_CPLD(reg,val)						\
	DEBUG_EVENT("%s: Write to CPLD reg %d value %X\n", 		\
			fe->name, reg, val);

#define WRITE_FRAMER(reg,val) 						\
	DEBUG_EVENT("%s: Write to Framer off %X value %X\n", 		\
			fe->name, reg, val);
#else

#define WRITE_CPLD(reg,val)						\
	(fe->write_cpld) ? fe->write_cpld(fe->card, reg, val) : -EINVAL;

#define WRITE_FRAMER(reg,val)						\
	(fe->write_framer) ? fe->write_framer(fe->card, reg, val) : -EINVAL;
#endif

/******************************************************************************
**			  FUNCTION DEFINITIONS
******************************************************************************/
int sdla_te3_config(sdla_fe_t *fe, sdla_fe3_iface_t *fe_iface);

static int sdla_te3_polling(sdla_fe_t *fe);
static int sdla_ds3_isr(sdla_fe_t *fe);
static int sdla_e3_isr(sdla_fe_t *fe);
static int sdla_te3_isr(sdla_fe_t *fe);
static int sdla_te3_udp(sdla_fe_t *fe, wan_cmd_t*, unsigned char*);
static int sdla_te3_get_alarm(sdla_fe_t *fe);
static int sdla_te3_pmon(sdla_fe_t *fe);

/******************************************************************************
 *				sdla_te3_polling()	
 *
 * Description:
 * Arguments:	
 * Returns:
 ******************************************************************************
 */
static int sdla_te3_polling(sdla_fe_t *fe)
{
	DEBUG_EVENT("%s: %s: This function is still not supported!\n",
			fe->name, __FUNCTION__);
	return -EINVAL;
}

/******************************************************************************
 *				sdla_te3_isr()	
 *
 * Description:
 * Arguments:	
 * Returns:
 ******************************************************************************
 */
static int sdla_ds3_isr(sdla_fe_t *fe)
{
	DEBUG_EVENT("%s: %s: This function is still not supported!\n",
			fe->name, __FUNCTION__);
	return -EINVAL;
}

/******************************************************************************
 *				sdla_te3_isr()	
 *
 * Description:
 * Arguments:	
 * Returns:
 ******************************************************************************
 */
static int sdla_e3_isr(sdla_fe_t *fe)
{
	DEBUG_EVENT("%s: %s: This function is still not supported!\n",
			fe->name, __FUNCTION__);
	return -EINVAL;
}
 
/******************************************************************************
 *				sdla_te3_isr()	
 *
 * Description:
 * Arguments:	
 * Returns:
 ******************************************************************************
 */
static int sdla_te3_isr(sdla_fe_t *fe)
{
	sdla_fe_cfg_t	*fe_cfg = &fe->fe_cfg;
	int		err = 0;
	
	DEBUG_EVENT("%s: %s: This function is still not supported!\n",
			fe->name, __FUNCTION__);
	err = (fe_cfg->media == WAN_MEDIA_DS3) ?
			sdla_ds3_isr(fe) : sdla_ds3_isr(fe);
	return -EINVAL;
}

/******************************************************************************
 *				sdla_te3_get_alarm()	
 *
 * Description:
 * Arguments:	
 * Returns:
 ******************************************************************************
 */
static int sdla_te3_get_alarm(sdla_fe_t *fe)
{
	DEBUG_EVENT("%s: %s: This function is still not supported!\n",
			fe->name, __FUNCTION__);
	return -EINVAL;
}

/******************************************************************************
 *				sdla_te3_alarm()	
 *
 * Description:
 * Arguments:	
 * Returns:
 ******************************************************************************
 */
static int sdla_te3_alarm(sdla_fe_t *fe, unsigned long alarm)
{
	unsigned char	data = 0x00;
	
	DEBUG_EVENT("%s: %s: This function is still not supported!\n",
			fe->name, __FUNCTION__);
	DEBUG_EVENT("%s: (TEST) Generate Yellow Alarm!\n", 
			fe->name);
	data &= ~BIT_TxDS3_CFG_YELLOW_ALARM;
	WRITE_FRAMER(REG_TxDS3_CFG, data);
	return -EINVAL;
}

/******************************************************************************
 *				sdla_te3_pmon()	
 *
 * Description:
 * Arguments:	
 * Returns:
 ******************************************************************************
 */
static int sdla_te3_pmon(sdla_fe_t *fe)
{
	DEBUG_EVENT("%s: %s: This function is still not supported!\n",
			fe->name, __FUNCTION__);
	return -EINVAL;
}

/******************************************************************************
 *				sdla_te3_udp()	
 *
 * Description:
 * Arguments:	
 * Returns:
 ******************************************************************************
 */
static int sdla_te3_udp(sdla_fe_t *fe, wan_cmd_t* udp_cmd, unsigned char *data)
{
	DEBUG_EVENT("%s: %s: This function is still not supported!\n",
			fe->name, __FUNCTION__);
	return -EINVAL;
}

/******************************************************************************
 *				sdla_te3_liu_config()	
 *
 * Description: Configure Sangoma TE3 board
 * Arguments:	
 * Returns:	WAN_TRUE - TE3 configred successfully, otherwise WAN_FALSE.
 ******************************************************************************
 */
static int sdla_te3_liu_config(sdla_fe_t *fe, sdla_te3_liu_cfg_t *liu, char *name)
{
	sdla_fe_cfg_t	*fe_cfg = &fe->fe_cfg;
	unsigned char	data = 0x00;

	if (fe_cfg->media == WAN_MEDIA_E3){
		data |= BIT_CPLD_CNTRL_E3;
	}
	if (liu->reqen == WAN_TRUE){
		DEBUG_TE3("%s: Enable Receive Equalizer\n",
				name);
		data |= BIT_CPLD_CNTRL_REQEN;
	}else{
		DEBUG_TE3("%s: Disable Receive Equalizer\n",
				name);
		data &= ~BIT_CPLD_CNTRL_REQEN;
	}
	if (liu->tx_lbo == WAN_TRUE){
		DEBUG_TE3("%s: Enable Transmit Build-out\n",
				name);
		data |= BIT_CPLD_CNTRL_TxLEV;
	}else{
		DEBUG_TE3("%s: Disable Transmit Build-out\n",
				name);
		data &= ~BIT_CPLD_CNTRL_TxLEV;
	}
	/* Write value to CPLD Control register */
	WRITE_CPLD(REG_CPLD_CNTRL, data);
	
	data = 0x00;
	if (liu->taos == WAN_TRUE){
		DEBUG_TE3("%s: Enable Transmit All Ones\n",
				name);
		data |= BIT_CPLD_STATUS_TAOS;
	}else{
		DEBUG_TE3("%s: Disable Transmit All Ones\n",
				name);
		data &= ~BIT_CPLD_STATUS_TAOS;
	}
	switch(liu->lb_mode){
	case WAN_TE3_LIU_LB_NONE:
		break;
	case WAN_TE3_LIU_LB_ANALOG:
		DEBUG_TE3("%s: Enable Analog Loopback mode!\n",
				name);
		data |= BIT_CPLD_STATUS_LLB;
		data &= ~BIT_CPLD_STATUS_RLB;
		break;
	case WAN_TE3_LIU_LB_REMOTE:
		DEBUG_TE3("%s: Enable Remote Loopback mode!\n",
				name);
		data &= ~BIT_CPLD_STATUS_LLB;
		data |= BIT_CPLD_STATUS_RLB;
		break;
	case WAN_TE3_LIU_LB_DIGITAL:
		DEBUG_TE3("%s: Enable Digital Loopback mode!\n",
				name);
		data |= BIT_CPLD_STATUS_LLB;
		data |= BIT_CPLD_STATUS_RLB;
		break;
	default :
		DEBUG_EVENT("%s: Unknown loopback mode!\n",
				name);
		break;
	}		
	/* Write value to CPLD Status/Control register */
	WRITE_CPLD(REG_CPLD_STATUS, data);
	return 0;
}

int sdla_te3_config(sdla_fe_t *fe, sdla_fe3_iface_t *fe_iface)
{
	sdla_fe_cfg_t	*fe_cfg = &fe->fe_cfg;
	sdla_te3_cfg_t	*te3_cfg = &fe_cfg->cfg.te3_cfg;
	unsigned char	data = 0x00;
	
	/* configure Line Interface Unit */
	sdla_te3_liu_config(fe, &te3_cfg->liu_cfg, fe->name);

	switch(fe_cfg->media){
	case WAN_MEDIA_DS3:
		DEBUG_TE3("%s: Media type DS3\n",
				fe->name);
		data |= BIT_OPMODE_DS3;
		break;
	case WAN_MEDIA_E3:
		DEBUG_TE3("%s: Media type E3\n",
				fe->name);
		data &= ~BIT_OPMODE_DS3;
		break;
	default:
		return -EINVAL;
	}

	switch(fe_cfg->frame){
	case WAN_FR_E3_G751:
		if (fe_cfg->media != WAN_MEDIA_E3){
			DEBUG_EVENT("%s: Invalid Frame Format Selected!\n",
					fe->name);
			return -EINVAL;
		}
		DEBUG_TE3("%s: Frame type G.751\n",
				fe->name);
		data &= ~BIT_OPMODE_FRAME_FRMT;
		break;
	case WAN_FR_E3_G832:
		if (fe_cfg->media != WAN_MEDIA_E3){
			DEBUG_EVENT("%s: Invalid Frame Format Selected!\n",
					fe->name);
			return -EINVAL;
		}
		DEBUG_TE3("%s: Frame type G.832\n",
				fe->name);
		data |= BIT_OPMODE_FRAME_FRMT;
		break;
	case WAN_FR_DS3_Cbit:
		if (fe_cfg->media != WAN_MEDIA_DS3){
			DEBUG_EVENT("%s: Invalid Frame Format Selected!\n",
					fe->name);
			return -EINVAL;
		}
		DEBUG_TE3("%s: Frame type DS3 C-bit Parity\n",
				fe->name);
		data &= ~BIT_OPMODE_FRAME_FRMT;
		break;
	case WAN_FR_DS3_M13:
		if (fe_cfg->media != WAN_MEDIA_DS3){
			DEBUG_EVENT("%s: Invalid Frame Format Selected!\n",
					fe->name);
			return -EINVAL;
		}
		DEBUG_TE3("%s: Frame type DS3 M13\n",
				fe->name);
		data |= BIT_OPMODE_FRAME_FRMT;
		break;
	default:
		return -EINVAL;
	}
	data |= BIT_OPMODE_INTERNAL_LOS;
	data |= BIT_OPMODE_INT_ENABLE_RESET;
	data |= (BIT_OPMODE_TIMREFSEL1 | BIT_OPMODE_TIMREFSEL0);
	WRITE_FRAMER(REG_OPMODE, data);

	data = 0x00;
	switch(fe_cfg->lcode){
	case WANOPT_LC_AMI: 
		DEBUG_TE3("%s: Line code AMI\n",
				fe->name);
		data |= BIT_IO_CONTROL_AMI;
		break;
	case WANOPT_LC_HDB3:
		DEBUG_TE3("%s: Line code HDB3\n",
				fe->name);
		data &= ~BIT_IO_CONTROL_AMI;
		break;
	case WANOPT_LC_B3ZS:
		DEBUG_TE3("%s: Line code B3ZS\n",
				fe->name);
		data &= ~BIT_IO_CONTROL_AMI;
		break;
	default:
		return -EINVAL;
	}
	data |= BIT_IO_CONTROL_DISABLE_TXCLK;
	data |= BIT_IO_CONTROL_DISABLE_RXLOC;
	data |= BIT_IO_CONTROL_RxLINECLK;
	WRITE_FRAMER(REG_IO_CONTROL, data);

	/* Inialize Front-End interface functions */
	fe_iface->polling	= sdla_te3_polling;
	fe_iface->isr		= sdla_te3_isr;
	fe_iface->process_udp	= sdla_te3_udp;
	fe_iface->get_alarm	= sdla_te3_get_alarm;
	fe_iface->get_pmon	= sdla_te3_pmon;
	fe_iface->gen_alarm	= sdla_te3_alarm;

	return 0;
}

int sdla_te3_unconfig(sdla_fe_t *fe)
{

	DEBUG_EVENT("%s: Unconfiguring T3/E3 interface\n",
			fe->name);
	return 0;
}

