/***************************************************************************
                          conf_file_reader.cpp  -  description
                             -------------------
    begin                : Thu Mar 4 2004
    copyright            : (C) 2004 by David Rokhvarg
    email                : davidr@sangoma.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "list_element_chan_def.h"
#include "conf_file_reader.h"
#include "text_box_help.h"
#include "wancfg.h"

#include "menu_select_and_configure_protocol.h"

#define DBG_CONF_FILE_READER  0
#define DBG_CONF_FILE_READER_AFT 1

//////////////////////////////////////////////////////////////////////////////////////////

/****** Global Data *********************************************************/

char master_lapb_dev[WAN_DRVNAME_SZ+1];
char master_x25_dev[WAN_DRVNAME_SZ+1];
// DSP_20
char master_dsp_dev[WAN_DRVNAME_SZ+1];

char* err_messages[] =				/* Error messages */
{
	"Invalid command line syntax",		/* ERR_SYNTAX */
	"Invalid configuration file syntax",	/* ERR_CONFIG */
	"Wanpipe module not loaded",		/* ERR_MODULE */
	"Unknown error code",			/* ERR_LIMIT */
};

//////////////////////////////////////////////////////////////////////////////////////////
//tables in conf_file_writer.cpp
extern look_up_t frame_relay_station_type_options_table[];
extern look_up_t frame_relay_in_channel_signalling_options_table[];
//////////////////////////////////////////////////////////////////////////////////////////

/*
 * Configuration structure description tables.
 * WARNING:	These tables MUST be kept in sync with corresponding data
 *		structures defined in linux/wanrouter.h
 */
key_word_t common_conftab[] =	/* Common configuration parameters */
{
  { "IOPORT",     offsetof(wandev_conf_t, ioport),     DTYPE_UINT },
  { "MEMADDR",    offsetof(wandev_conf_t, maddr),       DTYPE_UINT },
  { "MEMSIZE",    offsetof(wandev_conf_t, msize),       DTYPE_UINT },
  { "IRQ",        offsetof(wandev_conf_t, irq),         DTYPE_UINT },
  { "DMA",        offsetof(wandev_conf_t, dma),         DTYPE_UINT },
  { "CARD_TYPE",  offsetof(wandev_conf_t, card_type),	DTYPE_UCHAR },
  { "S514CPU",    offsetof(wandev_conf_t, S514_CPU_no), DTYPE_STR },
  { "PCISLOT",    offsetof(wandev_conf_t, PCI_slot_no), DTYPE_UINT },
  { "PCIBUS", 	  offsetof(wandev_conf_t, pci_bus_no),	DTYPE_UINT },
  { "AUTO_PCISLOT",offsetof(wandev_conf_t, auto_pci_cfg), DTYPE_UCHAR },
  { "COMMPORT",   offsetof(wandev_conf_t, comm_port),   DTYPE_UCHAR },

  /* TE1 New hardware parameters for T1/E1 board */
  { "MEDIA",    offsetof(wandev_conf_t, te_cfg)+offsetof(sdla_te_cfg_t, media), DTYPE_UCHAR },
  { "LCODE",    offsetof(wandev_conf_t, te_cfg)+offsetof(sdla_te_cfg_t, lcode), DTYPE_UCHAR },
  { "FRAME",    offsetof(wandev_conf_t, te_cfg)+offsetof(sdla_te_cfg_t, frame), DTYPE_UCHAR },
  { "LBO",      offsetof(wandev_conf_t, te_cfg)+offsetof(sdla_te_cfg_t, lbo), DTYPE_UCHAR },
  { "TE_CLOCK", offsetof(wandev_conf_t, te_cfg)+offsetof(sdla_te_cfg_t, te_clock), DTYPE_UCHAR },
  { "ACTIVE_CH",offsetof(wandev_conf_t, te_cfg)+offsetof(sdla_te_cfg_t, active_ch), DTYPE_ULONG },

  { "BAUDRATE",   offsetof(wandev_conf_t, bps),         DTYPE_UINT },
  { "MTU",        offsetof(wandev_conf_t, mtu),         DTYPE_UINT },
  { "UDPPORT",    offsetof(wandev_conf_t, udp_port),    DTYPE_UINT },
  { "TTL",	  offsetof(wandev_conf_t, ttl),		DTYPE_UCHAR },
  { "INTERFACE",  offsetof(wandev_conf_t, interface),   DTYPE_UCHAR },
  { "CLOCKING",   offsetof(wandev_conf_t, clocking),    DTYPE_UCHAR },
  { "LINECODING", offsetof(wandev_conf_t, line_coding), DTYPE_UCHAR },
  { "CONNECTION", offsetof(wandev_conf_t, connection),  DTYPE_UCHAR },
  { "LINEIDLE",   offsetof(wandev_conf_t, line_idle),   DTYPE_UCHAR },
  { "OPTION1",    offsetof(wandev_conf_t, hw_opt[0]),   DTYPE_UINT },
  { "OPTION2",    offsetof(wandev_conf_t, hw_opt[1]),   DTYPE_UINT },
  { "OPTION3",    offsetof(wandev_conf_t, hw_opt[2]),   DTYPE_UINT },
  { "OPTION4",    offsetof(wandev_conf_t, hw_opt[3]),   DTYPE_UINT },
  { "FIRMWARE",   offsetof(wandev_conf_t, data_size),   DTYPE_FILENAME },
  { "RXMODE",     offsetof(wandev_conf_t, read_mode),   DTYPE_CHAR },
  { "RECEIVE_ONLY", offsetof(wandev_conf_t, receive_only), DTYPE_UCHAR},	
  { "BACKUP",     offsetof(wandev_conf_t, backup), DTYPE_UCHAR},
  { "TTY",        offsetof(wandev_conf_t, tty), DTYPE_UCHAR},
  { "TTY_MAJOR",  offsetof(wandev_conf_t, tty_major), DTYPE_UINT},
  { "TTY_MINOR",  offsetof(wandev_conf_t, tty_minor), DTYPE_UINT},
  { "TTY_MODE",   offsetof(wandev_conf_t, tty_mode), DTYPE_UINT},
  { "IGNORE_FRONT_END",  offsetof(wandev_conf_t, ignore_front_end_status), DTYPE_UCHAR},


  { "MAX_RX_QUEUE", offsetof(wandev_conf_t, max_rx_queue), DTYPE_UINT},
  { "LMI_TRACE_QUEUE", offsetof(wandev_conf_t, max_rx_queue), DTYPE_UINT},
  { NULL, 0, 0 }
 };

key_word_t ppp_conftab[] =	/* PPP-specific configuration */
 {
  { "RESTARTTIMER",   offsetof(wan_ppp_conf_t, restart_tmr),   DTYPE_UINT },
  { "AUTHRESTART",    offsetof(wan_ppp_conf_t, auth_rsrt_tmr), DTYPE_UINT },
  { "AUTHWAIT",       offsetof(wan_ppp_conf_t, auth_wait_tmr), DTYPE_UINT },

  { "DTRDROPTIMER",   offsetof(wan_ppp_conf_t, dtr_drop_tmr),  DTYPE_UINT },
  { "CONNECTTIMEOUT", offsetof(wan_ppp_conf_t, connect_tmout), DTYPE_UINT },
  { "CONFIGURERETRY", offsetof(wan_ppp_conf_t, conf_retry),    DTYPE_UINT },
  { "TERMINATERETRY", offsetof(wan_ppp_conf_t, term_retry),    DTYPE_UINT },
  { "MAXCONFREJECT",  offsetof(wan_ppp_conf_t, fail_retry),    DTYPE_UINT },
  { "AUTHRETRY",      offsetof(wan_ppp_conf_t, auth_retry),    DTYPE_UINT },
  { "AUTHENTICATOR",  offsetof(wan_ppp_conf_t, authenticator), DTYPE_UCHAR},
  { "IP_MODE",        offsetof(wan_ppp_conf_t, ip_mode),       DTYPE_UCHAR},
  { NULL, 0, 0 }
};

key_word_t chdlc_conftab[] =	/* Cisco HDLC-specific configuration */
 {
  { "IGNORE_DCD", offsetof(wan_chdlc_conf_t, ignore_dcd), DTYPE_UCHAR},
  { "IGNORE_CTS", offsetof(wan_chdlc_conf_t, ignore_cts), DTYPE_UCHAR},
  { "IGNORE_KEEPALIVE", offsetof(wan_chdlc_conf_t, ignore_keepalive), DTYPE_UCHAR},
  { "HDLC_STREAMING", offsetof(wan_chdlc_conf_t, hdlc_streaming), DTYPE_UCHAR},
  { "KEEPALIVE_TX_TIMER", offsetof(wan_chdlc_conf_t, keepalive_tx_tmr), DTYPE_UINT },
  { "KEEPALIVE_RX_TIMER", offsetof(wan_chdlc_conf_t, keepalive_rx_tmr), DTYPE_UINT },
  { "KEEPALIVE_ERR_MARGIN", offsetof(wan_chdlc_conf_t, keepalive_err_margin),    DTYPE_UINT },
  { "SLARP_TIMER", offsetof(wan_chdlc_conf_t, slarp_timer),    DTYPE_UINT },
  { "FAST_ISR", offsetof(wan_chdlc_conf_t,fast_isr), DTYPE_UCHAR },
  { NULL, 0, 0 }
};

key_word_t fr_conftab[] =	/* Frame relay-specific configuration */
{
  { "SIGNALLING",    offsetof(wan_fr_conf_t, signalling),     DTYPE_UINT },
  { "T391", 	     offsetof(wan_fr_conf_t, t391),           DTYPE_UINT },
  { "T392",          offsetof(wan_fr_conf_t, t392),           DTYPE_UINT },
  { "N391",	     offsetof(wan_fr_conf_t, n391),           DTYPE_UINT },
  { "N392",          offsetof(wan_fr_conf_t, n392),           DTYPE_UINT },
  { "N393",          offsetof(wan_fr_conf_t, n393),           DTYPE_UINT },
  { "FR_ISSUE_FS",   offsetof(wan_fr_conf_t, issue_fs_on_startup),DTYPE_UCHAR },
  { "NUMBER_OF_DLCI",    offsetof(wan_fr_conf_t, dlci_num),       DTYPE_UINT },
  { "STATION",       offsetof(wan_fr_conf_t, station),     DTYPE_UCHAR },
  { NULL, 0, 0 }
};

key_word_t xilinx_conftab[] =	/* Xilinx specific configuration */
{
  { "MRU_TRANS",     offsetof(wan_xilinx_conf_t, mru_trans),       DTYPE_USHORT },
  { "DMA_PER_CH",    offsetof(wan_xilinx_conf_t, dma_per_ch),      DTYPE_USHORT },
  { "NUMBER_OF_CH",  offsetof(wan_xilinx_conf_t, num_of_ch),       DTYPE_USHORT },
  { NULL, 0, 0 }
};

key_word_t bitstrm_conftab[] =	/* Bitstreaming specific configuration */
{
 /* Bit strm options */
  { "SYNC_OPTIONS",	       offsetof(wan_bitstrm_conf_t, sync_options), DTYPE_UINT },
  { "RX_SYNC_CHAR",	       offsetof(wan_bitstrm_conf_t, rx_sync_char), DTYPE_UCHAR},
  { "MONOSYNC_TX_TIME_FILL_CHAR", offsetof(wan_bitstrm_conf_t, monosync_tx_time_fill_char), DTYPE_UCHAR},
  { "MAX_LENGTH_TX_DATA_BLOCK", offsetof(wan_bitstrm_conf_t,max_length_tx_data_block), DTYPE_UINT},
  { "RX_COMPLETE_LENGTH",       offsetof(wan_bitstrm_conf_t,rx_complete_length), DTYPE_UINT},
  { "RX_COMPLETE_TIMER",        offsetof(wan_bitstrm_conf_t,rx_complete_timer), DTYPE_UINT},	
  { "RBS_CH_MAP",		offsetof(wan_bitstrm_conf_t,rbs_map), DTYPE_ULONG},
  { NULL, 0, 0 }
};

key_word_t atm_conftab[] =
{
  {"ATM_SYNC_MODE",	       offsetof(wan_atm_conf_t,atm_sync_mode), DTYPE_USHORT },
  {"ATM_SYNC_DATA",	       offsetof(wan_atm_conf_t,atm_sync_data), DTYPE_USHORT },
  {"ATM_SYNC_OFFSET", 	       offsetof(wan_atm_conf_t,atm_sync_offset), DTYPE_USHORT },
  {"ATM_HUNT_TIMER", 	       offsetof(wan_atm_conf_t,atm_hunt_timer), DTYPE_USHORT },
  {"ATM_CELL_CFG", 	       offsetof(wan_atm_conf_t,atm_cell_cfg), DTYPE_UCHAR },
  {"ATM_CELL_PT", 	       offsetof(wan_atm_conf_t,atm_cell_pt), DTYPE_UCHAR },
  {"ATM_CELL_CLP", 	       offsetof(wan_atm_conf_t,atm_cell_clp), DTYPE_UCHAR },
  {"ATM_CELL_PAYLOAD", 	       offsetof(wan_atm_conf_t,atm_cell_payload), DTYPE_UCHAR },
  { NULL, 0, 0 }
};

key_word_t atm_if_conftab[] =
{
  {"ENCAPMODE",        offsetof(wan_atm_conf_if_t,encap_mode), DTYPE_UCHAR },
  {"VCI", 	       offsetof(wan_atm_conf_if_t,vci), DTYPE_USHORT },
  {"VPI", 	       offsetof(wan_atm_conf_if_t,vpi), DTYPE_USHORT },

  {"OAM_LOOPBACK",	offsetof(wan_atm_conf_if_t,atm_oam_loopback),  DTYPE_UCHAR },
  {"OAM_LOOPBACK_INT",  offsetof(wan_atm_conf_if_t,atm_oam_loopback_intr),  DTYPE_UCHAR },
  {"OAM_CC_CHECK",	offsetof(wan_atm_conf_if_t,atm_oam_continuity),  DTYPE_UCHAR },
  {"OAM_CC_CHECK_INT",	offsetof(wan_atm_conf_if_t,atm_oam_continuity_intr),  DTYPE_UCHAR },
  {"ATMARP",		offsetof(wan_atm_conf_if_t,atm_arp),  DTYPE_UCHAR },
  {"ATMARP_INT",	offsetof(wan_atm_conf_if_t,atm_arp_intr),  DTYPE_UCHAR },
  { NULL, 0, 0 }
};

key_word_t xilinx_if_conftab[] =
{
  { "SIGNALLING",     offsetof(wan_xilinx_conf_if_t, signalling),  DTYPE_UINT },
  { "STATION",        offsetof(wan_xilinx_conf_if_t, station),     DTYPE_UCHAR },
  { "SEVEN_BIT_HDLC", offsetof(wan_xilinx_conf_if_t, seven_bit_hdlc), DTYPE_CHAR },
  { NULL, 0, 0 }
};

key_word_t bitstrm_if_conftab[] =
{
  {"MAX_TX_QUEUE", offsetof(wan_bitstrm_conf_if_t,max_tx_queue_size), DTYPE_UINT },
  {"MAX_TX_UP_SIZE", offsetof(wan_bitstrm_conf_if_t,max_tx_up_size), DTYPE_UINT },
  {"SEVEN_BIT_HDLC", offsetof(wan_bitstrm_conf_if_t,seven_bit_hdlc), DTYPE_CHAR },
  { NULL, 0, 0 }
};

key_word_t adsl_conftab[] =
{
  {"ENCAPMODE", 	offsetof(wan_adsl_conf_t,EncapMode), DTYPE_UCHAR },
  /*Backward compatibility*/
  {"RFC1483MODE", 	offsetof(wan_adsl_conf_t,EncapMode), DTYPE_UCHAR },
  {"RFC2364MODE", 	offsetof(wan_adsl_conf_t,EncapMode), DTYPE_UCHAR },

  {"VCI", 		offsetof(wan_adsl_conf_t,Vci), DTYPE_USHORT },
  /*Backward compatibility*/
  {"RFC1483VCI",	offsetof(wan_adsl_conf_t,Vci), DTYPE_USHORT },
  {"RFC2364VCI",	offsetof(wan_adsl_conf_t,Vci), DTYPE_USHORT },

  {"VPI", 		offsetof(wan_adsl_conf_t,Vpi), DTYPE_USHORT },
  /*Backward compatibility*/
  {"RFC1483VPI", 	offsetof(wan_adsl_conf_t,Vpi), DTYPE_USHORT },
  {"RFC2364VPI", 	offsetof(wan_adsl_conf_t,Vpi), DTYPE_USHORT },

  {"VERBOSE", 		offsetof(wan_adsl_conf_t,Verbose), DTYPE_UCHAR },
  /*Backward compatibility*/
  {"DSL_INTERFACE", 	offsetof(wan_adsl_conf_t,Verbose), DTYPE_UCHAR },

  {"RXBUFFERCOUNT", 	offsetof(wan_adsl_conf_t,RxBufferCount), DTYPE_USHORT },
  {"TXBUFFERCOUNT", 	offsetof(wan_adsl_conf_t,TxBufferCount), DTYPE_USHORT },

  {"ADSLSTANDARD",      offsetof(wan_adsl_conf_t,Standard), DTYPE_USHORT },
  {"ADSLTRELLIS",       offsetof(wan_adsl_conf_t,Trellis), DTYPE_USHORT },
  {"ADSLTXPOWERATTEN",  offsetof(wan_adsl_conf_t,TxPowerAtten), DTYPE_USHORT },
  {"ADSLCODINGGAIN",    offsetof(wan_adsl_conf_t,CodingGain), DTYPE_USHORT },
  {"ADSLMAXBITSPERBIN", offsetof(wan_adsl_conf_t,MaxBitsPerBin), DTYPE_USHORT },
  {"ADSLTXSTARTBIN",    offsetof(wan_adsl_conf_t,TxStartBin), DTYPE_USHORT },
  {"ADSLTXENDBIN",      offsetof(wan_adsl_conf_t,TxEndBin), DTYPE_USHORT },
  {"ADSLRXSTARTBIN",    offsetof(wan_adsl_conf_t,RxStartBin), DTYPE_USHORT },
  {"ADSLRXENDBIN",      offsetof(wan_adsl_conf_t,RxEndBin), DTYPE_USHORT },
  {"ADSLRXBINADJUST",   offsetof(wan_adsl_conf_t,RxBinAdjust), DTYPE_USHORT },
  {"ADSLFRAMINGSTRUCT", offsetof(wan_adsl_conf_t,FramingStruct), DTYPE_USHORT },
  {"ADSLEXPANDEDEXCHANGE",      offsetof(wan_adsl_conf_t,ExpandedExchange), DTYPE_USHORT },
  {"ADSLCLOCKTYPE",     offsetof(wan_adsl_conf_t,ClockType), DTYPE_USHORT },
  {"ADSLMAXDOWNRATE",   offsetof(wan_adsl_conf_t,MaxDownRate), DTYPE_USHORT },

  /*Backward compatibility*/
  {"GTISTANDARD",      offsetof(wan_adsl_conf_t,Standard), DTYPE_USHORT },
  {"GTITRELLIS",       offsetof(wan_adsl_conf_t,Trellis), DTYPE_USHORT },
  {"GTITXPOWERATTEN",  offsetof(wan_adsl_conf_t,TxPowerAtten), DTYPE_USHORT },
  {"GTICODINGGAIN",    offsetof(wan_adsl_conf_t,CodingGain), DTYPE_USHORT },
  {"GTIMAXBITSPERBIN", offsetof(wan_adsl_conf_t,MaxBitsPerBin), DTYPE_USHORT },
  {"GTITXSTARTBIN",    offsetof(wan_adsl_conf_t,TxStartBin), DTYPE_USHORT },
  {"GTITXENDBIN",      offsetof(wan_adsl_conf_t,TxEndBin), DTYPE_USHORT },
  {"GTIRXSTARTBIN",    offsetof(wan_adsl_conf_t,RxStartBin), DTYPE_USHORT },
  {"GTIRXENDBIN",      offsetof(wan_adsl_conf_t,RxEndBin), DTYPE_USHORT },
  {"GTIRXBINADJUST",   offsetof(wan_adsl_conf_t,RxBinAdjust), DTYPE_USHORT },
  {"GTIFRAMINGSTRUCT", offsetof(wan_adsl_conf_t,FramingStruct), DTYPE_USHORT },
  {"GTIEXPANDEDEXCHANGE",      offsetof(wan_adsl_conf_t,ExpandedExchange), DTYPE_USHORT },
  {"GTICLOCKTYPE",     offsetof(wan_adsl_conf_t,ClockType), DTYPE_USHORT },
  {"GTIMAXDOWNRATE",   offsetof(wan_adsl_conf_t,MaxDownRate), DTYPE_USHORT },

  {"ATM_AUTOCFG", 	offsetof(wan_adsl_conf_t,atm_autocfg), DTYPE_UCHAR },
  {"ADSL_WATCHDOG",	offsetof(wan_adsl_conf_t,atm_watchdog), DTYPE_UCHAR },
  { NULL, 0, 0 }
};

key_word_t bscstrm_conftab[]=
{
  {"BSCSTRM_ADAPTER_FR" , offsetof(wan_bscstrm_conf_t,adapter_frequency),DTYPE_ULONG},
  {"BSCSTRM_MTU", 	offsetof(wan_bscstrm_conf_t,max_data_length),DTYPE_USHORT},
  {"BSCSTRM_EBCDIC" ,  offsetof(wan_bscstrm_conf_t,EBCDIC_encoding),DTYPE_USHORT},
  {"BSCSTRM_RB_BLOCK_TYPE", offsetof(wan_bscstrm_conf_t,Rx_block_type),DTYPE_USHORT},
  {"BSCSTRM_NO_CONSEC_PAD_EOB", offsetof(wan_bscstrm_conf_t,no_consec_PADs_EOB), DTYPE_USHORT},
  {"BSCSTRM_NO_ADD_LEAD_TX_SYN_CHARS", offsetof(wan_bscstrm_conf_t,no_add_lead_Tx_SYN_chars),DTYPE_USHORT},
  {"BSCSTRM_NO_BITS_PER_CHAR", offsetof(wan_bscstrm_conf_t,no_bits_per_char),DTYPE_USHORT},
  {"BSCSTRM_PARITY", offsetof(wan_bscstrm_conf_t,parity),DTYPE_USHORT},
  {"BSCSTRM_MISC_CONFIG_OPTIONS",  offsetof(wan_bscstrm_conf_t,misc_config_options),DTYPE_USHORT},
  {"BSCSTRM_STATISTICS_OPTIONS", offsetof(wan_bscstrm_conf_t,statistics_options),  DTYPE_USHORT},
  {"BSCSTRM_MODEM_CONFIG_OPTIONS", offsetof(wan_bscstrm_conf_t,modem_config_options), DTYPE_USHORT},
  { NULL, 0, 0 }
};


key_word_t ss7_conftab[] =
{
  {"LINE_CONFIG_OPTIONS", offsetof(wan_ss7_conf_t,line_cfg_opt),  DTYPE_UINT },
  {"MODEM_CONFIG_OPTIONS",offsetof(wan_ss7_conf_t,modem_cfg_opt), DTYPE_UINT },
  {"MODEM_STATUS_TIMER",  offsetof(wan_ss7_conf_t,modem_status_timer), DTYPE_UINT },
  {"API_OPTIONS",	  offsetof(wan_ss7_conf_t,api_options),	  DTYPE_UINT },
  {"PROTOCOL_OPTIONS",	  offsetof(wan_ss7_conf_t,protocol_options), DTYPE_UINT },
  {"PROTOCOL_SPECIFICATION", offsetof(wan_ss7_conf_t,protocol_specification), DTYPE_UINT },
  {"STATS_HISTORY_OPTIONS", offsetof(wan_ss7_conf_t,stats_history_options), DTYPE_UINT },
  {"MAX_LENGTH_MSU_SIF", offsetof(wan_ss7_conf_t,max_length_msu_sif), DTYPE_UINT },
  {"MAX_UNACKED_TX_MSUS", offsetof(wan_ss7_conf_t,max_unacked_tx_msus), DTYPE_UINT },
  {"LINK_INACTIVITY_TIMER", offsetof(wan_ss7_conf_t,link_inactivity_timer), DTYPE_UINT },
  {"T1_TIMER",		  offsetof(wan_ss7_conf_t,t1_timer),	  DTYPE_UINT },
  {"T2_TIMER",		  offsetof(wan_ss7_conf_t,t2_timer),	  DTYPE_UINT },
  {"T3_TIMER",		  offsetof(wan_ss7_conf_t,t3_timer),	  DTYPE_UINT },
  {"T4_TIMER_EMERGENCY",  offsetof(wan_ss7_conf_t,t4_timer_emergency), DTYPE_UINT },
  {"T4_TIMER_NORMAL",  offsetof(wan_ss7_conf_t,t4_timer_normal), DTYPE_UINT },
  {"T5_TIMER",		  offsetof(wan_ss7_conf_t,t5_timer),	  DTYPE_UINT },
  {"T6_TIMER",		  offsetof(wan_ss7_conf_t,t6_timer),	  DTYPE_UINT },
  {"T7_TIMER",		  offsetof(wan_ss7_conf_t,t7_timer),	  DTYPE_UINT },
  {"T8_TIMER",		  offsetof(wan_ss7_conf_t,t8_timer),	  DTYPE_UINT },
  {"N1",		  offsetof(wan_ss7_conf_t,n1),	  DTYPE_UINT },
  {"N2",		  offsetof(wan_ss7_conf_t,n2),	  DTYPE_UINT },
  {"TIN",		  offsetof(wan_ss7_conf_t,tin),	  DTYPE_UINT },
  {"TIE",		  offsetof(wan_ss7_conf_t,tie),	  DTYPE_UINT },
  {"SUERM_ERROR_THRESHOLD", offsetof(wan_ss7_conf_t,suerm_error_threshold), DTYPE_UINT},
  {"SUERM_NUMBER_OCTETS", offsetof(wan_ss7_conf_t,suerm_number_octets), DTYPE_UINT},
  {"SUERM_NUMBER_SUS", offsetof(wan_ss7_conf_t,suerm_number_sus), DTYPE_UINT},
  {"SIE_INTERVAL_TIMER", offsetof(wan_ss7_conf_t,sie_interval_timer), DTYPE_UINT},
  {"SIO_INTERVAL_TIMER", offsetof(wan_ss7_conf_t,sio_interval_timer), DTYPE_UINT},
  {"SIOS_INTERVAL_TIMER", offsetof(wan_ss7_conf_t,sios_interval_timer), DTYPE_UINT},
  {"FISU_INTERVAL_TIMER", offsetof(wan_ss7_conf_t,fisu_interval_timer), DTYPE_UINT},
  { NULL, 0, 0 }
};



key_word_t x25_conftab[] =	/* X.25-specific configuration */
{
  { "LOWESTPVC",    offsetof(wan_x25_conf_t, lo_pvc),       DTYPE_UINT },
  { "HIGHESTPVC",   offsetof(wan_x25_conf_t, hi_pvc),       DTYPE_UINT },
  { "LOWESTSVC",    offsetof(wan_x25_conf_t, lo_svc),       DTYPE_UINT },
  { "HIGHESTSVC",   offsetof(wan_x25_conf_t, hi_svc),       DTYPE_UINT },
  { "HDLCWINDOW",   offsetof(wan_x25_conf_t, hdlc_window),  DTYPE_UINT },
  { "PACKETWINDOW", offsetof(wan_x25_conf_t, pkt_window),   DTYPE_UINT },
  { "CCITTCOMPAT",  offsetof(wan_x25_conf_t, ccitt_compat), DTYPE_UINT },
  { "X25CONFIG",    offsetof(wan_x25_conf_t, x25_conf_opt), DTYPE_UINT }, 	
  { "LAPB_HDLC_ONLY", offsetof(wan_x25_conf_t, LAPB_hdlc_only), DTYPE_UCHAR },
  { "CALL_SETUP_LOG", offsetof(wan_x25_conf_t, logging), DTYPE_UCHAR },
  { "OOB_ON_MODEM",   offsetof(wan_x25_conf_t, oob_on_modem), DTYPE_UCHAR},
  { "T1", 	    offsetof(wan_x25_conf_t, t1), DTYPE_UINT },
  { "T2", 	    offsetof(wan_x25_conf_t, t2), DTYPE_UINT },	
  { "T4", 	    offsetof(wan_x25_conf_t, t4), DTYPE_UINT },
  { "N2", 	    offsetof(wan_x25_conf_t, n2), DTYPE_UINT },
  { "T10_T20", 	    offsetof(wan_x25_conf_t, t10_t20), DTYPE_UINT },
  { "T11_T21", 	    offsetof(wan_x25_conf_t, t11_t21), DTYPE_UINT },	
  { "T12_T22", 	    offsetof(wan_x25_conf_t, t12_t22), DTYPE_UINT },
  { "T13_T23", 	    offsetof(wan_x25_conf_t, t13_t23), DTYPE_UINT },
  { "T16_T26", 	    offsetof(wan_x25_conf_t, t16_t26), DTYPE_UINT },
  { "T28", 	    offsetof(wan_x25_conf_t, t28), DTYPE_UINT },
  { "R10_R20", 	    offsetof(wan_x25_conf_t, r10_r20), DTYPE_UINT },
  { "R12_R22", 	    offsetof(wan_x25_conf_t, r12_r22), DTYPE_UINT },
  { "R13_R23", 	    offsetof(wan_x25_conf_t, r13_r23), DTYPE_UINT },
  { "STATION_ADDR", offsetof(wan_x25_conf_t, local_station_address), DTYPE_UCHAR},
  { "DEF_PKT_SIZE", offsetof(wan_x25_conf_t, defPktSize),  DTYPE_UINT },
  { "MAX_PKT_SIZE", offsetof(wan_x25_conf_t, pktMTU),  DTYPE_UINT },
  { NULL, 0, 0 }
};

key_word_t lapb_annexg_conftab[] =
{
  //LAPB STUFF
  { "T1", offsetof(wan_lapb_if_conf_t, t1),    DTYPE_UINT },
  { "T2", offsetof(wan_lapb_if_conf_t, t2),    DTYPE_UINT },
  { "T3", offsetof(wan_lapb_if_conf_t, t3),    DTYPE_UINT },
  { "T4", offsetof(wan_lapb_if_conf_t, t4),    DTYPE_UINT },
  { "N2", offsetof(wan_lapb_if_conf_t, n2),    DTYPE_UINT },
  { "LAPB_MODE", 	offsetof(wan_lapb_if_conf_t, mode),    DTYPE_UINT },
  { "LAPB_WINDOW", 	offsetof(wan_lapb_if_conf_t, window),    DTYPE_UINT },

  { "LABEL",		offsetof(wan_lapb_if_conf_t,label), DTYPE_STR},
  { "VIRTUAL_ADDR",     offsetof(wan_lapb_if_conf_t,virtual_addr), DTYPE_STR},
  { "REAL_ADDR",        offsetof(wan_lapb_if_conf_t,real_addr), DTYPE_STR},

  { "MAX_PKT_SIZE", 	offsetof(wan_lapb_if_conf_t,mtu), DTYPE_UINT},

  { NULL, 0, 0 }
};


key_word_t x25_annexg_conftab[] =
{
 //X25 STUFF
  { "PACKETWINDOW", offsetof(wan_x25_if_conf_t, packet_window),   DTYPE_USHORT },
  { "CCITTCOMPAT",  offsetof(wan_x25_if_conf_t, CCITT_compatibility), DTYPE_USHORT },
  { "T10_T20", 	    offsetof(wan_x25_if_conf_t, T10_T20), DTYPE_USHORT },
  { "T11_T21", 	    offsetof(wan_x25_if_conf_t, T11_T21), DTYPE_USHORT },	
  { "T12_T22", 	    offsetof(wan_x25_if_conf_t, T12_T22), DTYPE_USHORT },
  { "T13_T23", 	    offsetof(wan_x25_if_conf_t, T13_T23), DTYPE_USHORT },
  { "T16_T26", 	    offsetof(wan_x25_if_conf_t, T16_T26), DTYPE_USHORT },
  { "T28", 	    offsetof(wan_x25_if_conf_t, T28),     DTYPE_USHORT },
  { "R10_R20", 	    offsetof(wan_x25_if_conf_t, R10_R20), DTYPE_USHORT },
  { "R12_R22", 	    offsetof(wan_x25_if_conf_t, R12_R22), DTYPE_USHORT },
  { "R13_R23", 	    offsetof(wan_x25_if_conf_t, R13_R23), DTYPE_USHORT },

  { "X25_API_OPTIONS", offsetof(wan_x25_if_conf_t, X25_API_options), DTYPE_USHORT },
  { "X25_PROTOCOL_OPTIONS", offsetof(wan_x25_if_conf_t, X25_protocol_options), DTYPE_USHORT },
  { "X25_RESPONSE_OPTIONS", offsetof(wan_x25_if_conf_t, X25_response_options), DTYPE_USHORT },
  { "X25_STATISTICS_OPTIONS", offsetof(wan_x25_if_conf_t, X25_statistics_options), DTYPE_USHORT },

  { "GEN_FACILITY_1", offsetof(wan_x25_if_conf_t, genl_facilities_supported_1), DTYPE_USHORT },
  { "GEN_FACILITY_2", offsetof(wan_x25_if_conf_t, genl_facilities_supported_2), DTYPE_USHORT },
  { "CCITT_FACILITY", offsetof(wan_x25_if_conf_t, CCITT_facilities_supported), DTYPE_USHORT },
  { "NON_X25_FACILITY",	offsetof(wan_x25_if_conf_t,non_X25_facilities_supported),DTYPE_USHORT },

  { "DFLT_PKT_SIZE", offsetof(wan_x25_if_conf_t,default_packet_size), DTYPE_USHORT },
  { "MAX_PKT_SIZE",  offsetof(wan_x25_if_conf_t,maximum_packet_size), DTYPE_USHORT },

  { "LOWESTSVC",   offsetof(wan_x25_if_conf_t,lowest_two_way_channel), DTYPE_USHORT },
  { "HIGHESTSVC",  offsetof(wan_x25_if_conf_t,highest_two_way_channel), DTYPE_USHORT},

  { "LOWESTPVC",   offsetof(wan_x25_if_conf_t,lowest_PVC), DTYPE_USHORT },
  { "HIGHESTPVC",  offsetof(wan_x25_if_conf_t,highest_PVC), DTYPE_USHORT},

  { "X25_MODE", offsetof(wan_x25_if_conf_t, mode), DTYPE_UCHAR},
  { "CALL_BACKOFF", offsetof(wan_x25_if_conf_t, call_backoff_timeout), DTYPE_UINT },
  { "CALL_LOGGING", offsetof(wan_x25_if_conf_t, call_logging), DTYPE_UCHAR },

  { "X25_CALL_STRING",      offsetof(wan_x25_if_conf_t, call_string),     DTYPE_STR},
  { "X25_ACCEPT_DST_ADDR",  offsetof(wan_x25_if_conf_t, accept_called),   DTYPE_STR},
  { "X25_ACCEPT_SRC_ADDR",  offsetof(wan_x25_if_conf_t, accept_calling),  DTYPE_STR},
  { "X25_ACCEPT_FCL_DATA",  offsetof(wan_x25_if_conf_t, accept_facil),    DTYPE_STR},
  { "X25_ACCEPT_USR_DATA",  offsetof(wan_x25_if_conf_t, accept_udata),    DTYPE_STR},

  { "LABEL",		offsetof(wan_x25_if_conf_t,label),        DTYPE_STR},
  { "VIRTUAL_ADDR",     offsetof(wan_x25_if_conf_t,virtual_addr), DTYPE_STR},
  { "REAL_ADDR",        offsetof(wan_x25_if_conf_t,real_addr),    DTYPE_STR},

  { NULL, 0, 0 }
};

key_word_t dsp_annexg_conftab[] =
{
  //DSP_20 DSP STUFF
  { "PAD",		offsetof(wan_dsp_if_conf_t, pad_type),		DTYPE_UCHAR },
  { "T1_DSP",  		offsetof(wan_dsp_if_conf_t, T1), 		DTYPE_UINT },
  { "T2_DSP",  		offsetof(wan_dsp_if_conf_t, T2), 		DTYPE_UINT },
  { "T3_DSP",  		offsetof(wan_dsp_if_conf_t, T3), 		DTYPE_UINT },
  { "DSP_AUTO_CE",  	offsetof(wan_dsp_if_conf_t, auto_ce), 		DTYPE_UCHAR },
  { "DSP_AUTO_CALL_REQ",offsetof(wan_dsp_if_conf_t, auto_call_req), 	DTYPE_UCHAR },
  { "DSP_AUTO_ACK",  	offsetof(wan_dsp_if_conf_t, auto_ack), 		DTYPE_UCHAR },
  { "DSP_MTU",  	offsetof(wan_dsp_if_conf_t, dsp_mtu), 		DTYPE_USHORT },
  { NULL, 0, 0 }
};

key_word_t chan_conftab[] =	/* Channel configuration parameters */
{
  { "IDLETIMEOUT",   	offsetof(wanif_conf_t, idle_timeout), 	DTYPE_UINT },
  { "HOLDTIMEOUT",   	offsetof(wanif_conf_t, hold_timeout), 	DTYPE_UINT },
  { "X25_SRC_ADDR",   	offsetof(wanif_conf_t, x25_src_addr), 	DTYPE_STR},
  { "X25_ACCEPT_DST_ADDR",  offsetof(wanif_conf_t, accept_dest_addr), DTYPE_STR},
  { "X25_ACCEPT_SRC_ADDR",  offsetof(wanif_conf_t, accept_src_addr),  DTYPE_STR},
  { "X25_ACCEPT_USR_DATA",  offsetof(wanif_conf_t, accept_usr_data),  DTYPE_STR},
  { "CIR",           	offsetof(wanif_conf_t, cir), 	   	DTYPE_UINT },
  { "BC",            	offsetof(wanif_conf_t, bc),		DTYPE_UINT },
  { "BE", 	     	offsetof(wanif_conf_t, be),		DTYPE_UINT },
  { "MULTICAST",     	offsetof(wanif_conf_t, mc),		DTYPE_UCHAR},
  { "IPX",	     	offsetof(wanif_conf_t, enable_IPX),	DTYPE_UCHAR},
  { "NETWORK",       	offsetof(wanif_conf_t, network_number),	DTYPE_ULONG},
  { "PAP",     	     	offsetof(wanif_conf_t, pap),		DTYPE_UCHAR},
  { "CHAP",          	offsetof(wanif_conf_t, chap),		DTYPE_UCHAR},
  { "USERID",        	offsetof(wanif_conf_t, userid),	 	DTYPE_STR},
  { "PASSWD",        	offsetof(wanif_conf_t, passwd),		DTYPE_STR},
  { "SYSNAME",       	offsetof(wanif_conf_t, sysname),		DTYPE_STR},
  { "INARP", 	     	offsetof(wanif_conf_t, inarp),          	DTYPE_UCHAR},
  { "INARPINTERVAL", 	offsetof(wanif_conf_t, inarp_interval), 	DTYPE_UINT },
  { "INARP_RX",      	offsetof(wanif_conf_t, inarp_rx),          	DTYPE_UCHAR},
  { "IGNORE_DCD",  	offsetof(wanif_conf_t, ignore_dcd),        	DTYPE_UCHAR},
  { "IGNORE_CTS",    	offsetof(wanif_conf_t, ignore_cts),        	DTYPE_UCHAR},
  { "IGNORE_KEEPALIVE", offsetof(wanif_conf_t, ignore_keepalive), 	DTYPE_UCHAR},
  { "HDLC_STREAMING", 	offsetof(wanif_conf_t, hdlc_streaming), 	DTYPE_UCHAR},
  { "KEEPALIVE_TX_TIMER",	offsetof(wanif_conf_t, keepalive_tx_tmr), 	DTYPE_UINT },
  { "KEEPALIVE_RX_TIMER",	offsetof(wanif_conf_t, keepalive_rx_tmr), 	DTYPE_UINT },
  { "KEEPALIVE_ERR_MARGIN",	offsetof(wanif_conf_t, keepalive_err_margin),	DTYPE_UINT },
  { "SLARP_TIMER", 	offsetof(wanif_conf_t, slarp_timer),    DTYPE_UINT },
  { "TTL",        	offsetof(wanif_conf_t, ttl),         DTYPE_UCHAR },

 // { "INTERFACE",  	offsetof(wanif_conf_t, interface),   DTYPE_UCHAR },
 // { "CLOCKING",   	offsetof(wanif_conf_t, clocking),    DTYPE_UCHAR },
 // { "BAUDRATE",   	offsetof(wanif_conf_t, bps),         DTYPE_UINT },

  { "STATION" ,          offsetof(wanif_conf_t, station),     DTYPE_UCHAR },
  { "DYN_INTR_CFG",  	offsetof(wanif_conf_t, if_down),     DTYPE_UCHAR },
  { "GATEWAY",  	offsetof(wanif_conf_t, gateway),     DTYPE_UCHAR },
  { "TRUE_ENCODING_TYPE", offsetof(wanif_conf_t,true_if_encoding), DTYPE_UCHAR },

  /* ASYNC Options */
  { "ASYNC_MODE",    	       offsetof(wanif_conf_t, async_mode), DTYPE_UCHAR},	
  { "ASY_DATA_TRANSPARENT",    offsetof(wanif_conf_t, asy_data_trans), DTYPE_UCHAR},
  { "RTS_HS_FOR_RECEIVE",      offsetof(wanif_conf_t, rts_hs_for_receive), DTYPE_UCHAR},
  { "XON_XOFF_HS_FOR_RECEIVE", offsetof(wanif_conf_t, xon_xoff_hs_for_receive), DTYPE_UCHAR},
  { "XON_XOFF_HS_FOR_TRANSMIT",offsetof(wanif_conf_t, xon_xoff_hs_for_transmit), DTYPE_UCHAR},
  { "DCD_HS_FOR_TRANSMIT",     offsetof(wanif_conf_t, dcd_hs_for_transmit), DTYPE_UCHAR},	
  { "CTS_HS_FOR_TRANSMIT",     offsetof(wanif_conf_t, cts_hs_for_transmit), DTYPE_UCHAR},
  { "TX_BITS_PER_CHAR",        offsetof(wanif_conf_t, tx_bits_per_char),    DTYPE_UINT },
  { "RX_BITS_PER_CHAR",        offsetof(wanif_conf_t, rx_bits_per_char),    DTYPE_UINT },
  { "STOP_BITS",               offsetof(wanif_conf_t, stop_bits),    DTYPE_UINT },
  { "PARITY",                  offsetof(wanif_conf_t, parity),    DTYPE_UCHAR },
  { "BREAK_TIMER",             offsetof(wanif_conf_t, break_timer),    DTYPE_UINT },	
  { "INTER_CHAR_TIMER",        offsetof(wanif_conf_t, inter_char_timer),    DTYPE_UINT },
  { "RX_COMPLETE_LENGTH",      offsetof(wanif_conf_t, rx_complete_length),    DTYPE_UINT },
  { "XON_CHAR",                offsetof(wanif_conf_t, xon_char),    DTYPE_UINT },	
  { "XOFF_CHAR",               offsetof(wanif_conf_t, xoff_char),    DTYPE_UINT },	
  { "MPPP_PROT",	       offsetof(wanif_conf_t, protocol),  DTYPE_UCHAR},
  { "PROTOCOL",	       	       offsetof(wanif_conf_t, protocol),  DTYPE_UCHAR},
  { "ACTIVE_CH",	       offsetof(wanif_conf_t, active_ch),  DTYPE_ULONG},
  { "SW_DEV_NAME",	       offsetof(wanif_conf_t, sw_dev_name),  DTYPE_STR},

  { "DLCI_TRACE_QUEUE", offsetof(wanif_conf_t, max_trace_queue), DTYPE_UINT},
  { "MAX_TRACE_QUEUE", offsetof(wanif_conf_t, max_trace_queue), DTYPE_UINT},

  { NULL, 0, 0 }
};

look_up_t conf_def_tables[] =
{
	{ WANCONFIG_PPP,	ppp_conftab	},
	{ WANCONFIG_FR,		fr_conftab	},
	{ WANCONFIG_X25,	x25_conftab	},
	{ WANCONFIG_ADCCP,	x25_conftab	},
	{ WANCONFIG_CHDLC,	chdlc_conftab	},
	{ WANCONFIG_MPPP,	chdlc_conftab	},
	{ WANCONFIG_MFR,	fr_conftab	},
	{ WANCONFIG_SS7,	ss7_conftab 	},
	{ WANCONFIG_ADSL,	adsl_conftab 	},
	{ WANCONFIG_BSCSTRM,	bscstrm_conftab	},
	{ WANCONFIG_ATM,	atm_conftab	},
	{ WANCONFIG_MLINK_PPP,	chdlc_conftab	},
	{ WANCONFIG_AFT,        xilinx_conftab  },
	{ WANCONFIG_AFT_TE3,    xilinx_conftab  },
	{ WANCONFIG_BITSTRM,    bitstrm_conftab },
	{ 0,			NULL		}
};

look_up_t conf_if_def_tables[] =
{
	{ WANCONFIG_ATM,	atm_if_conftab	},
	{ WANCONFIG_BITSTRM,    bitstrm_if_conftab },
	{ WANCONFIG_AFT,	xilinx_if_conftab },
	{ WANCONFIG_AFT_TE3,    xilinx_if_conftab },
	{ 0,			NULL		}
};

look_up_t conf_annexg_def_tables[] =
{
	{ ANNEXG_LAPB,		lapb_annexg_conftab	},
	{ ANNEXG_X25,		x25_annexg_conftab	},
	{ ANNEXG_DSP,		dsp_annexg_conftab	},
	{ 0,			NULL		}
};

look_up_t	config_id_str[] =
{
	{ WANCONFIG_PPP,	(void*)"WAN_PPP"	},
	{ WANCONFIG_FR,		(void*)"WAN_FR"	},
	{ WANCONFIG_X25,	(void*)"WAN_X25"	},
	{ WANCONFIG_CHDLC,	(void*)"WAN_CHDLC"	},
  { WANCONFIG_BSC,        (void*)"WAN_BSC"       },
  { WANCONFIG_HDLC,       (void*)"WAN_HDLC"      },
	{ WANCONFIG_MPPP,       (void*)"WAN_MULTPPP"   },
	{ WANCONFIG_MPROT,      (void*)"WAN_MULTPROT"  },
	{ WANCONFIG_BITSTRM, 	(void*)"WAN_BITSTRM"	},
	{ WANCONFIG_EDUKIT, 	(void*)"WAN_EDU_KIT"	},
	{ WANCONFIG_SS7,        (void*)"WAN_SS7"       },
	{ WANCONFIG_BSCSTRM,    (void*)"WAN_BSCSTRM"   },
	{ WANCONFIG_ADSL,	(void*)"WAN_ADSL"	},
	{ WANCONFIG_ADSL,	(void*)"WAN_ETH"	},
	{ WANCONFIG_SDLC,	(void*)"WAN_SDLC"	},
	{ WANCONFIG_ATM,	(void*)"WAN_ATM"	},
	{ WANCONFIG_POS,	(void*)"WAN_POS"	},
	{ WANCONFIG_AFT,	(void*)"WAN_AFT"	},
	{ WANCONFIG_AFT_TE3,	(void*)"WAN_AFT_TE3"	},
	{ WANCONFIG_AFT,	(void*)"WAN_XILINX"	},
	{ WANCONFIG_MFR,    	(void*)"WAN_MFR"   	},
	{ WANCONFIG_DEBUG,    	(void*)"WAN_DEBUG"   	},
	{ WANCONFIG_ADCCP,    	(void*)"WAN_ADCCP"   	},
	{ WANCONFIG_MLINK_PPP, 	(void*)"WAN_MLINK_PPP" },
	{ 0,			NULL,		}
};

/*
 * Configuration options values and their symbolic names.
 */
look_up_t	sym_table[] =
{
	/*----- General -----------------------*/
	{ WANOPT_OFF,		(void*)"OFF"		},
	{ WANOPT_ON,		(void*)"ON"		},
	{ WANOPT_NO,		(void*)"NO"		},
	{ WANOPT_YES,		(void*)"YES"		},
	/*----- Interface type ----------------*/
	{ WANOPT_RS232,	(void*)"RS232"		},
	{ WANOPT_V35,		(void*)"V35"		},
	/*----- Data encoding -----------------*/
	{ WANOPT_NRZ,		(void*)"NRZ"		},
	{ WANOPT_NRZI,	(void*)"NRZI"		},
	{ WANOPT_FM0,		(void*)"FM0"		},
	{ WANOPT_FM1,		(void*)"FM1"		},

	/*----- Idle Line ----------------------*/
	{ WANOPT_IDLE_FLAG,	(void*)"FLAG"	},
	{ WANOPT_IDLE_MARK,	(void*)"MARK"	},

	/*----- Link type ---------------------*/
	{ WANOPT_POINTTOPOINT,	(void*)"POINTTOPOINT"	},
	{ WANOPT_MULTIDROP,	(void*)"MULTIDROP"	},
	/*----- Clocking ----------------------*/
	{ WANOPT_EXTERNAL,	(void*)"EXTERNAL"	},
	{ WANOPT_INTERNAL,	(void*)"INTERNAL"	},
	/*----- Station -----------------------*/
	{ WANOPT_DTE,		(void*)"DTE"		},
	{ WANOPT_DCE,		(void*)"DCE"		},
	{ WANOPT_CPE,		(void*)"CPE"		},
	{ WANOPT_NODE,		(void*)"NODE"		},
	{ WANOPT_SECONDARY,	(void*)"SECONDARY"	},
	{ WANOPT_PRIMARY,	(void*)"PRIMARY"	},
	/*----- Connection options ------------*/
	{ WANOPT_PERMANENT,	(void*)"PERMANENT"	},
	{ WANOPT_SWITCHED,	(void*)"SWITCHED"	},
	{ WANOPT_ONDEMAND,	(void*)"ONDEMAND"	},
	/*----- Frame relay in-channel signalling */
	{ WANOPT_FR_ANSI,	(void*)"ANSI"		},
	{ WANOPT_FR_Q933,	(void*)"Q933"		},
	{ WANOPT_FR_LMI,	(void*)"LMI"		},
	/*----- PPP IP Mode Options -----------*/
	{ WANOPT_PPP_STATIC,	(void*)"STATIC"	},
	{ WANOPT_PPP_HOST,	(void*)"HOST"		},
	{ WANOPT_PPP_PEER,	(void*)"PEER"		},
	/*----- CHDLC Protocol Options --------*/
/* DF for now	{ WANOPT_CHDLC_NO_DCD,	(void*)"IGNORE_DCD"	},
	{ WANOPT_CHDLC_NO_CTS,	(void*)"IGNORE_CTS"	},
	{ WANOPT_CHDLC_NO_KEEP,	(void*)"IGNORE_KEEPALIVE"},
*/
	{ WANOPT_PRI,           (void*)"PRI"           },
	{ WANOPT_SEC,           (void*)"SEC"           },
	
  /*------Read Mode ---------------------*/
  { WANOPT_INTR,          (void*)"INT"           },
  { WANOPT_POLL,          (void*)"POLL"          },
	/*------- Async Options ---------------*/
	{ WANOPT_ONE,           (void*)"ONE"          	},
	{ WANOPT_TWO,           (void*)"TWO"          	},
	{ WANOPT_ONE_AND_HALF,  (void*)"ONE_AND_HALF" 	},
	{ WANOPT_NONE,          (void*)"NONE"    	},
	{ WANOPT_EVEN,          (void*)"EVEN"    	},
	{ WANOPT_ODD,           (void*)"ODD"    	},

	{ WANOPT_TTY_SYNC,	(void*)"SYNC"		},
	{ WANOPT_TTY_ASYNC,     (void*)"ASYNC"		},


	/* TE1 T1/E1 defines */
  /*------TE Cofngiuration --------------*/
  { WANOPT_MEDIA_T1,      (void*)"T1"            },
  { WANOPT_MEDIA_E1,      (void*)"E1"            },
	{ WANOPT_MEDIA_56K,     (void*)"56K"           },
  { WANOPT_LC_AMI,     	  (void*)"AMI"           },
  { WANOPT_LC_B8ZS,       (void*)"B8ZS"          },
  { WANOPT_LC_HDB3,       (void*)"HDB3"          },
  { WANOPT_FR_D4,         (void*)"D4"            },
  { WANOPT_FR_ESF,        (void*)"ESF"           },
  { WANOPT_FR_NCRC4,      (void*)"NCRC4"         },
  { WANOPT_FR_CRC4,       (void*)"CRC4"          },
  { WANOPT_FR_UNFRAMED,   (void*)"UNFRAMED"      },
  { WANOPT_T1_LBO_0_DB,  	(void*)"0DB"           },
  { WANOPT_T1_LBO_75_DB,  (void*)"7.5DB"         },
  { WANOPT_T1_LBO_15_DB,  (void*)"15DB"          },
  { WANOPT_T1_LBO_225_DB, (void*)"22.5DB"        },
  { WANOPT_T1_0_110,  	  (void*)"0-110FT"       },
  { WANOPT_T1_110_220,  	(void*)"110-220FT"     },
  { WANOPT_T1_220_330,  	(void*)"220-330FT"     },
  { WANOPT_T1_330_440,  	(void*)"330-440FT"     },
  { WANOPT_T1_440_550,  	(void*)"440-550FT"     },
  { WANOPT_T1_550_660,  	(void*)"550-660FT"     },
  { WANOPT_NORMAL_CLK,   	(void*)"NORMAL"        },
  { WANOPT_MASTER_CLK,   	(void*)"MASTER"        },

/*
	{ WANCONFIG_PPP,	  (void*)"MP_PPP"	},
	{ WANCONFIG_CHDLC,	(void*)"MP_CHDLC"	},
	{ WANCONFIG_FR, 	  (void*)"MP_FR"		},
	{ WANOPT_NO,		    (void*)"RAW"		},
	{ WANOPT_NO,		    (void*)"HDLC"		},
	{ WANCONFIG_PPP,	  (void*)"PPP"		},
	{ WANCONFIG_CHDLC,	(void*)"CHDLC"		},
*/
  //David
	{ WANCONFIG_MPPP,	    (void*)"MP_PPP"	},
	{ WANCONFIG_MPCHDLC,	(void*)"MP_CHDLC"	},
	{ WANCONFIG_MFR, 	    (void*)"MP_FR"		},
	{ WANOPT_NO,		      (void*)"RAW"		},
	{ WANCONFIG_HDLC,		    (void*)"HDLC"		},
	{ WANCONFIG_PPP,	    (void*)"PPP"		},
	{ WANCONFIG_CHDLC,	  (void*)"CHDLC"		},
	
	/*-------------SS7 options--------------*/
	{ WANOPT_SS7_ANSI,      (void*)"ANSI" },
	{ WANOPT_SS7_ITU,       (void*)"ITU"  },
	{ WANOPT_SS7_NTT,       (void*)"NTT"  },
	
	{ WANOPT_S50X,		(void*)"S50X"		},
	{ WANOPT_S51X,		(void*)"S51X"		},
	{ WANOPT_ADSL,		(void*)"ADSL"		},
	{ WANOPT_ADSL,		(void*)"S518"		},
	{ WANOPT_AFT,     (void*)"AFT"    },

	/*-------------ADSL options--------------*/
	{ RFC_MODE_BRIDGED_ETH_LLC,	(void*)"ETH_LLC_OA" },
	{ RFC_MODE_BRIDGED_ETH_VC,	(void*)"ETH_VC_OA"  },
	{ RFC_MODE_ROUTED_IP_LLC,	  (void*)"IP_LLC_OA"  },
	{ RFC_MODE_ROUTED_IP_VC,	  (void*)"IP_VC_OA"   },
	{ RFC_MODE_PPP_LLC,		      (void*)"PPP_LLC_OA" },
	{ RFC_MODE_PPP_VC,		      (void*)"PPP_VC_OA"  },
	
	/* Backward compatible */
	{ RFC_MODE_BRIDGED_ETH_LLC,	(void*)"BLLC_OA" },
	{ RFC_MODE_ROUTED_IP_LLC,	  (void*)"CIP_OA" },
	{ LAN_INTERFACE,		        (void*)"LAN" },
	{ WAN_INTERFACE,		        (void*)"WAN" },

  { WANOPT_ADSL_T1_413,                   (void*)"ADSL_T1_413"                   },
  { WANOPT_ADSL_G_LITE,                   (void*)"ADSL_G_LITE"                   },
	{ WANOPT_ADSL_G_DMT,                    (void*)"ADSL_G_DMT"                    },
	{ WANOPT_ADSL_ALCATEL_1_4,              (void*)"ADSL_ALCATEL_1_4"              },
	{ WANOPT_ADSL_ALCATEL,                  (void*)"ADSL_ALCATEL"                  },
	{ WANOPT_ADSL_MULTIMODE,                (void*)"ADSL_MULTIMODE"                },
	{ WANOPT_ADSL_T1_413_AUTO,              (void*)"ADSL_T1_413_AUTO"              },
	{ WANOPT_ADSL_TRELLIS_ENABLE,           (void*)"ADSL_TRELLIS_ENABLE"           },
	{ WANOPT_ADSL_TRELLIS_DISABLE,          (void*)"ADSL_TRELLIS_DISABLE"          },
	{ WANOPT_ADSL_TRELLIS_LITE_ONLY_DISABLE,(void*)"ADSL_TRELLIS_LITE_ONLY_DISABLE"},
	{ WANOPT_ADSL_0DB_CODING_GAIN,          (void*)"ADSL_0DB_CODING_GAIN"          },
	{ WANOPT_ADSL_1DB_CODING_GAIN,          (void*)"ADSL_1DB_CODING_GAIN"          },
	{ WANOPT_ADSL_2DB_CODING_GAIN,          (void*)"ADSL_2DB_CODING_GAIN"          },
	{ WANOPT_ADSL_3DB_CODING_GAIN,          (void*)"ADSL_3DB_CODING_GAIN"          },
	{ WANOPT_ADSL_4DB_CODING_GAIN,          (void*)"ADSL_4DB_CODING_GAIN"          },
	{ WANOPT_ADSL_5DB_CODING_GAIN,          (void*)"ADSL_5DB_CODING_GAIN"          },
	{ WANOPT_ADSL_6DB_CODING_GAIN,          (void*)"ADSL_6DB_CODING_GAIN"          },
	{ WANOPT_ADSL_7DB_CODING_GAIN,          (void*)"ADSL_7DB_CODING_GAIN"          },
	{ WANOPT_ADSL_AUTO_CODING_GAIN,         (void*)"ADSL_AUTO_CODING_GAIN"         },
	{ WANOPT_ADSL_RX_BIN_DISABLE,           (void*)"ADSL_RX_BIN_DISABLE"           },
	{ WANOPT_ADSL_RX_BIN_ENABLE,            (void*)"ADSL_RX_BIN_ENABLE"            },
	{ WANOPT_ADSL_FRAMING_TYPE_0,           (void*)"ADSL_FRAMING_TYPE_0"           },
  { WANOPT_ADSL_FRAMING_TYPE_1,           (void*)"ADSL_FRAMING_TYPE_1"           },
	{ WANOPT_ADSL_FRAMING_TYPE_2,           (void*)"ADSL_FRAMING_TYPE_2"           },
	{ WANOPT_ADSL_FRAMING_TYPE_3,           (void*)"ADSL_FRAMING_TYPE_3"           },
	{ WANOPT_ADSL_EXPANDED_EXCHANGE,        (void*)"ADSL_EXPANDED_EXCHANGE"        },
	{ WANOPT_ADSL_SHORT_EXCHANGE,           (void*)"ADSL_SHORT_EXCHANGE"           },
	{ WANOPT_ADSL_CLOCK_CRYSTAL,            (void*)"ADSL_CLOCK_CRYSTAL"            },
	{ WANOPT_ADSL_CLOCK_OSCILLATOR,         (void*)"ADSL_CLOCK_OSCILLATOR"         },

	{ IBM4680,	(void*)"IBM4680" },
	{ IBM4680,	(void*)"IBM4680" },
  	{ NCR2126,	(void*)"NCR2126" },
	{ NCR2127,	(void*)"NCR2127" },
	{ NCR1255,	(void*)"NCR1255" },
	{ NCR7000,	(void*)"NCR7000" },
	{ ICL,		(void*)"ICL" },

	// DSP_20
	/*------- DSP Options -----------------*/
	{ WANOPT_DSP_HPAD,      (void*)"HPAD"         	},
	{ WANOPT_DSP_TPAD,      (void*)"TPAD"         	},

	{ WANOPT_AUTO,  (void*)"AUTO" },
	{ WANOPT_MANUAL,(void*)"MANUAL" },

	/*----- End ---------------------------*/
	{ 0,			NULL		},
};

//////////////////////////////////////////////////////////////////////////////////////////

/****** Global Function Prototypes *************************************************/

int tokenize (char* str, char **tokens);
char* strstrip (char* str, char* s);
char* strupcase	(char* str);
void* lookup (int val, look_up_t* table);
int name2val (char* name, look_up_t* table);

unsigned int dec_to_uint (unsigned char* str, int len);

void read_adsl_vci_vpi_list(wan_adsl_vcivpi_t* vcivpi_list, unsigned short* vcivpi_num);

extern	int close (int);

//////////////////////////////////////////////////////////////////////////////////////////

conf_file_reader::conf_file_reader(int file_number)
{
  Debug(DBG_CONF_FILE_READER, ("conf_file_reader::conf_file_reader(int file_number)\n"));

  init(file_number);
}


conf_file_reader::conf_file_reader(char* template_file_path)
{
  Debug(DBG_CONF_FILE_READER, ("conf_file_reader::conf_file_reader(char* template_file_path)\n"));

  init(0);

  snprintf(conf_file, MAX_PATH_LENGTH, template_file_path);
}

void conf_file_reader::init(int file_number)
{
  wandev_conf_t *linkconf;

  Debug(DBG_CONF_FILE_READER, ("conf_file_reader::init(int wanpipe_number)\n"));

  obj_list = new objects_list();

  /////////////////////////////////////////////////////////////
	link_defs = (link_def_t*)calloc(1, sizeof(link_def_t));
	if (link_defs == NULL) {
		ERR_DBG_OUT(("Memory allocation failed for 'link_defs'!!\n"));
    exit(1);
	}
	memset(link_defs,0,sizeof(link_def_t));
  /////////////////////////////////////////////////////////////
	link_defs->linkconf = (wandev_conf_t*)malloc(sizeof(wandev_conf_t));
	if (!link_defs->linkconf){
		ERR_DBG_OUT(("Memory allocation failed for 'link_defs->linkconf'!!\n"));
		exit(1);
	}
	memset(link_defs->linkconf, 0, sizeof(wandev_conf_t));
  /////////////////////////////////////////////////////////////
  linkconf = link_defs->linkconf;

  linkconf->card_type = NOT_SET;
  snprintf(link_defs->name, WAN_DRVNAME_SZ, "wanpipe%d", file_number);

  linkconf->config_id = PROTOCOL_NOT_SET;

  //S514CPU
  linkconf->S514_CPU_no[0] = 'A';
  //Comms Port
  //AUTO_PCISLOT
  linkconf->auto_pci_cfg = WANOPT_YES;
  //PCISLOT---------> 0
  //PCIBUS----------> 0
  //Physical Medium
  linkconf->interface = WANOPT_V35;

  //default TE1 configuration values
  sdla_te_cfg_t* te_cfg = &linkconf->te_cfg;
  te_cfg->media = WANOPT_MEDIA_T1;
  te_cfg->lcode = WANOPT_LC_B8ZS;
  te_cfg->frame = WANOPT_FR_ESF;
  te_cfg->lbo = WANOPT_T1_LBO_0_DB;
  te_cfg->te_clock = WANOPT_NORMAL_CLK;
  te_cfg->active_ch = ENABLE_ALL_CHANNELS;
  snprintf(link_defs->active_channels_string, MAX_LEN_OF_ACTIVE_CHANNELS_STRING, "ALL");

  /////////////////////////////////////////////////////////////
  verbose = 1;			/* verbosity level */

  //'file_number' is actually the wanpipe number. store it, as it will be needed
  //by both "New" and "Edit existing"
  wanpipe_number = file_number;

  snprintf(conf_file, MAX_PATH_LENGTH, "%swanpipe%d.conf", wanpipe_cfg_dir, file_number);

  adsl_file = NULL;			/* ADSL VCI/VPI list */
  dev_name = NULL;
  card_name = NULL;
  weanie_flag = 0;
  nodev_flag = 0;
}

conf_file_reader::~conf_file_reader()
{
  Debug(DBG_CONF_FILE_READER, ("conf_file_reader::~conf_file_reader()\n"));

  list_element_chan_def* list_el_chan_def = (list_element_chan_def*)obj_list->get_first();

  //each link may have pointers to other "conf_file_reader" objects. delete them all.
  while(list_el_chan_def != NULL){

    if( list_el_chan_def->conf_file_reader != NULL){
      delete (conf_file_reader*)(list_el_chan_def->conf_file_reader);
      list_el_chan_def->conf_file_reader = NULL;
    }

    list_el_chan_def = (list_element_chan_def*)obj_list->get_next_element(list_el_chan_def);
  }//while()

  if(obj_list != NULL){
    //free_linkdefs();
    delete obj_list;
  }
}


int conf_file_reader::read_conf_file()
{
	int err = 0;
	int res;

  Debug(DBG_CONF_FILE_READER, ("conf_file_reader::read_conf_file()\n"));

	err = parse_conf_file(conf_file);
	if (err){
    return err;
  }

	if (link_defs == NULL) {
		if (verbose){
      printf(" * No link definitions found...\n");
		  return ERR_CONFIG;
    }
	}

	if( card_name != NULL && strcmp(card_name, link_defs->name) != 0 ){
		return ERR_CONFIG;
  }

	if (verbose){
    printf(" * Configuring device %s (%s)\n", link_defs->name, "no description");
  }

	res = configure_link(link_defs,0);
	if (res) {
    Debug(DBG_CONF_FILE_READER, ("conf_file_reader::read_conf_file(): returning: %d\n", res));
    err = res;
	}

  /////////////////////////////////////////////////////
  //if S514X card, get the version (S514-1/2/3...7)
  if(link_defs->linkconf->card_type == WANOPT_S51X){

    if(get_pci_adapter_type_from_wanpipe_conf_file( conf_file,
                                                    &link_defs->card_version) == NO){
      return ERR_CONFIG;
    }else{
      Debug(DBG_CONF_FILE_READER,
        ("conf_file_reader: S514 card_version 0x%04X\n", link_defs->card_version));
    }
  }

	return err;
}


/*============================================================================
 * Configure router.
 * o parse configuration file
 * o configure links
 * o configure logical channels (interfaces)
 */
int conf_file_reader::configure (void)
{
	int err = 0;
	int res;

	err = parse_conf_file(conf_file);
	if (err) return err;

	if (link_defs == NULL) {
		if (verbose) printf(" * No link definitions found...\n");
		return 0;
	}

	if( card_name != NULL && strcmp(card_name, link_defs->name) != 0 )
		return ERR_CONFIG;

	if (verbose) printf(
		" * Configuring device %s (%s)\n",
		link_defs->name,
		"no description");
	res = configure_link(link_defs,0);
	if (res) {
		return res;
	}

	//free_linkdefs();

	return err;
}

/*============================================================================
 * Parse configuration file.
 *	Read configuration file and create lists of link and channel
 *	definitions.
 */
int conf_file_reader::parse_conf_file (char* fname)
{
	int err = 0;
	FILE* file;

  Debug(DBG_CONF_FILE_READER, ("conf_file_reader::parse_conf_file()\n"));

	if (verbose)
		printf(" * Parsing configuration file %s ...\n", conf_file);

	file = fopen(fname, "r");
	if (file == NULL) {
		fprintf(stderr, "\nError: %s not found in %s directory\n",
				fname, wanpipe_cfg_dir);
		show_error(ERR_SYSTEM);
		return ERR_SYSTEM;
	}

	/* Build a list of link and channel definitions */
	err = build_linkdef_list(file);

	if (!err && link_defs){
		err = build_chandef_list(file);
  }

	fclose(file);
  return err;
}

/*============================================================================
 * Build a list of link definitions.
 */
int conf_file_reader::build_linkdef_list (FILE* file)
{
	int err = 0;
	char* conf;		/* -> section buffer */
	char* key;		/* -> current configuration record */
	int len;		  /* record length */
	char *token[MAX_TOKENS];

	int toknum;
	//link_def_t* linkdef;	/* -> link definition */
	int config_id = 0;

  Debug(DBG_CONF_FILE_READER, ("conf_file_reader::build_linkdef_list()\n"));

	/* Read [devices] section */
	conf = read_conf_section(file, "devices");
	if (conf == NULL){
    return ERR_CONFIG;
  }

  //Debug(DBG_CONF_FILE_READER, ("conf:%s\n", conf));

	/* For the record in [devices] section create a link definition
	 * structure and link it to the list of link definitions.
	 */
  //David: current wanpipe#.conf may have only *one* device in the [devices] section.
  key = conf;
	len = strlen(key) + 1;

	toknum = tokenize(key, token);
  Debug(DBG_CONF_FILE_READER, ("toknum:%d\n", toknum));
	if(toknum < 2){
    ERR_DBG_OUT(("Invalid number of tokens (%d) in [devices] section!!\n", toknum));
    err = ERR_CONFIG;
		goto done;
  }

  Debug(DBG_CONF_FILE_READER, ("token[1]:%s\n", token[1]));

	strupcase(token[1]);
	config_id = name2val(token[1], config_id_str);
	if (!config_id) {
		if (verbose) printf(
			" * Media ID %s is invalid!\n", token[1]);
		err = ERR_CONFIG;
		show_error(err);
		goto done;
	}
/*
  //David
	link_defs = (link_def_t*)calloc(1, sizeof(link_def_t));
	if (link_defs == NULL) {
		err = ERR_SYSTEM;
		show_error(err);
		goto done;
	}
	memset(link_defs,0,sizeof(link_def_t));
*/
  Debug(DBG_CONF_FILE_READER, ("token[0]:%s\n", token[0]));

  //store the section name i.g. [wanpipe2]
	strncpy(link_defs->name, token[0], WAN_DRVNAME_SZ);
	link_defs->config_id = config_id;

  //David
  link_defs->start_script = check_wanpipe_start_script_exist(link_defs->name);
  link_defs->stop_script = check_wanpipe_stop_script_exist(link_defs->name);

  Debug(DBG_CONF_FILE_READER, ("link_defs->start_script:%d\n", link_defs->start_script));
  Debug(DBG_CONF_FILE_READER, ("link_defs->stop_script:%d\n", link_defs->stop_script));
	
  //this is for the 'Comment'
  if ((toknum > 2) && token[2]){
	  link_defs->descr = strdup(token[2]);
  }

  Debug(DBG_CONF_FILE_READER_AFT,
    ("calling 'link_defs->conf = read_conf_section(file, link_defs->name)'\n"));
	link_defs->conf = read_conf_section(file, link_defs->name);

done:
  Debug(DBG_CONF_FILE_READER, ("build_linkdef_list() - End\n"));

	free(conf);
	return err;
}

/*============================================================================
 * Read configuration file section.
 *	Return a pointer to memory filled with key entries read from given
 *	section or NULL if section is not found. Each key is a zero-terminated
 *	ASCII string with no leading spaces and comments. The last key is
 *	followed by an empty string.
 *	Note:	memory is allocated dynamically and must be released by the
 *		caller.
 */
char* conf_file_reader::read_conf_section (FILE* file, char* section)
{
	char key[MAX_CFGLINE];		/* key buffer */
	char* buf = NULL;
	int found = 0, offs = 0, len;
  int dbg_len = 0;

  Debug(DBG_CONF_FILE_READER, ("conf_file_reader::read_conf_section ():%s\n", section));
	
	rewind(file);
	while ((len = read_conf_record(file, key)) > 0) {
		char* tmp;

    dbg_len += len;

		if (found) {
			if (*key == '[') break;	/* end of section */

			if (buf){
        tmp = (char*)realloc(buf, offs + len + 1);
      }else{
        tmp = (char*)malloc(len + 1);
      }

			if (tmp) {
				buf = tmp;
				strcpy(&buf[offs], key);
				offs += len;
				buf[offs] = '\0';
			}
			else {	/* allocation failed */
				show_error(ERR_SYSTEM);
 				break;
			}
		}
		else if (*key == '[') {
			tmp = strchr(key, ']');
			if (tmp != NULL) {
				*tmp = '\0';
				if (strcmp(&key[1], section) == 0) {
 					if (verbose) printf(
						" * Reading section [%s]...\n",
						section);
					found = 1;
				}
			}
		}
	}
 	if (verbose && !found){
    printf(" * section [%s] not found!\n", section);
  }
/*
  char* dbg_ptr;
  int counter=0;
  Debug(DBG_CONF_FILE_READER, ("\nprinting [%s] section buffer:\n", section));
  for(dbg_ptr=buf; counter++ < dbg_len; dbg_ptr++){
    //Debug(DBG_CONF_FILE_READER, ("%c", *dbg_ptr));
    if(isprint(*dbg_ptr) || isspace(*dbg_ptr)){
      Debug(DBG_CONF_FILE_READER, ("%c", *dbg_ptr));
    }
  }
  Debug(DBG_CONF_FILE_READER, ("\n\n"));
*/
	return buf;
}

/*============================================================================
 * Read a single record from the configuration file.
 *	Read configuration file stripping comments and leading spaces until
 *	non-empty line is read.  Copy it to the destination buffer.
 *
 * Return string length (incl. terminating zero) or 0 if end of file has been
 * reached.
 */
int conf_file_reader::read_conf_record (FILE* file, char* key)
{
	char buf[MAX_CFGLINE];		/* line buffer */

  Debug(DBG_CONF_FILE_READER, ("conf_file_reader::read_conf_record():key: %s\n", key));

	while (fgets(buf, MAX_CFGLINE, file)) {
		char* str;
		int len;

		/* Strip leading spaces and tabs */
		for (str = buf; *str && strchr(" \t", *str); ++str){
      ;
    }
		/* Skip comments and empty lines */
		len = strcspn(str, "#;\n\r");
		if (len) {
			str[len] = '\0';

      Debug(DBG_CONF_FILE_READER, ("read_conf_record():str: %s\n", str));

			strcpy(key, str);
			return len + 1;
		}
	}
	return 0;
}

/*============================================================================
 * Build a list of channel definitions.
 */
int conf_file_reader::build_chandef_list (FILE* file)
{
	int err = 0;
	char* conf;		/* -> section buffer */
	char* key;		/* -> current configuration record */
	int len;		  /* record length */
	char *token[MAX_TOKENS];
  int aft_channel_counter = 2000; //to avoid conflict with real (DLCI) numbers start from 2000.
                                  //later, function 'reduce_aft_channel_groups_numbers()'
                                  //will reduce to 'normal' numbers like 1,2,3 ... up to 32
                                  //(maximum number of groups of channels on E1).

  Debug(DBG_CONF_FILE_READER, ("conf_file_reader::build_chandef_list()\n"));

	link_def_t* linkdef = link_defs;
  //David
  list_element_chan_def* list_el_chan_def;

	if (linkdef == NULL)
		return ERR_CONFIG;

	/* Read [interfaces] section */
	conf = read_conf_section(file, "interfaces");
	if (conf == NULL)
		return 0;

  Debug(DBG_CONF_FILE_READER_AFT, ("linkdef->config_id: %s\n",
    get_protocol_string(linkdef->config_id)));

	/* For each record in [interfaces] section create channel definition
	 * structure and link it to the list of channel definitions for
	 * appropriate link.
	 */
	for (key = conf; !err && *key; key += len) {
		int toknum;
		char *channel_sect;
		
		chan_def_t* chandef;		/* -> channel definition */

		len = strlen(key) + 1;

		toknum = tokenize(key, token);
		if (toknum < 4) continue;

		/* allocate and initialize channel definition structure */
		//chandef = (chan_def_t*)calloc(1, sizeof(chan_def_t));
    list_el_chan_def = new list_element_chan_def();
    chandef = &list_el_chan_def->data;
		if (chandef == NULL) {
			err = ERR_SYSTEM;
			show_error(err);
			break;
		}

	 	strncpy(chandef->name, token[0], WAN_IFNAME_SZ);
    strcpy(chandef->addr, token[2]);

    Debug(DBG_CONF_FILE_READER_AFT, ("token[2]: %s\n", token[2]));

    //On an AFT channel there is no 'addr' so, 'token[2]' CAN NOT be used for 'addr'.
    //The sorted linked list will not accept duplicates, so create
    //the unique addr:
    if(linkdef->config_id == WANCONFIG_AFT){
      snprintf(chandef->addr, MAX_ADDR_STR_LEN, "%d", aft_channel_counter++);

      //FIXIT:
      //If AFT channel will run 'WANCONFIG_MFR' the address IS REAL.
      //At this point the protocol is NOT KNOWN yet!!
      //It is the number of one and only DLCI supported right now.
      //It can be number or 'auto'.
      if((atoi(token[2]) > 0) || (strcmp(token[2], "auto") == 0)){
        //it is a DLCI number, set again to 'toke[2]'
        strcpy(chandef->aft_dlci_number, token[2]);
        //AFT_DBG
        strcpy(chandef->addr, token[2]);
      }
    }

    ///////////////////////////////////////////////////////
		//chandef->usedby = strdup(token[3]);
    chandef->usedby = (char*)malloc(MAX_USEDBY_STR_LEN);
    if(chandef->usedby == NULL){
      ERR_DBG_OUT(("Memory allocation failed!!\n"));
		  return ERR_CONFIG;
    }
    strcpy(chandef->usedby, token[3]);

    ///////////////////////////////////////////////////////
		chandef->annexg = NO_ANNEXG;

    Debug(DBG_CONF_FILE_READER_AFT, ("chandef->usedby: %s\n", chandef->usedby));
    Debug(DBG_CONF_FILE_READER_AFT, ("toknum: %d\n", toknum));
    Debug(DBG_CONF_FILE_READER_AFT, ("token[4]: %s\n", token[4]));

		/* These are Anexg Connections */
		if (toknum > 5){
      //David - for AFT Frame Relay: token[4] is "fr" and toknum is 6.
			if (!strcmp(token[4],"fr")){
				chandef->LIP_protocol = WANCONFIG_MFR;

        //DBG_CONF_FILE_READER_AFT
        chandef->LIP_root_device = get_last_real_channelized_device();
        if(chandef->LIP_root_device == NULL){
          return ERR_CONFIG;
        }
      }else if(!strcmp(token[4],"ppp")){
				chandef->LIP_protocol = WANCONFIG_MPPP;

        //DBG_CONF_FILE_READER_AFT
        chandef->LIP_root_device = get_last_real_channelized_device();
        if(chandef->LIP_root_device == NULL){
          return ERR_CONFIG;
        }
      }else if(!strcmp(token[4],"chdlc")){
				chandef->LIP_protocol = WANCONFIG_MPCHDLC;

        //DBG_CONF_FILE_READER_AFT
        chandef->LIP_root_device = get_last_real_channelized_device();
        if(chandef->LIP_root_device == NULL){
          return ERR_CONFIG;
        }
      }else{
  			if (!strcmp(token[4],"lapb")){
	  			chandef->annexg = ANNEXG_LAPB;
		  	}else if (!strcmp(token[4],"x25")){
			  	chandef->annexg = ANNEXG_X25;
  			}else if (!strcmp(token[4],"dsp")){
	  			chandef->annexg = ANNEXG_DSP;
		  	}else{
          ERR_DBG_OUT(("Invalid number of tokens!!\n"));
				  return ERR_CONFIG;
  			}
      }
		}
		
		if (!(chandef->usedby) ||
           (( strcmp(chandef->usedby, "WANPIPE")      != 0 ) &&
					  ( strcmp(chandef->usedby, "API")          != 0 ) &&
					  ( strcmp(chandef->usedby, "BRIDGE")       != 0 ) &&
					  ( strcmp(chandef->usedby, "SWITCH")       != 0 ) &&
					  ( strcmp(chandef->usedby, "PPPoE")        != 0 ) &&
					  ( strcmp(chandef->usedby, "STACK")        != 0 ) &&
					  ( strcmp(chandef->usedby, "BRIDGE_NODE")  != 0 ) &&
					  ( strcmp(chandef->usedby, "VoIP") 	      != 0 ) &&
					  ( strcmp(chandef->usedby, "ANNEXG")       != 0 )))
		{
			if (verbose){
        printf(" * %s to be used by an Invalid entity: %s\n",	chandef->name, chandef->usedby);
      }
			return ERR_CONFIG;
		}
	
		if (verbose){
			printf(" * %s to used by %s\n", chandef->name, chandef->usedby);
    }
		
		channel_sect=strdup(token[0]);
		
		if (toknum > 5){
			char *section=strdup(token[5]);
			/* X25_SW */
			if (toknum > 7){
				if (toknum > 6)
					chandef->virtual_addr = strdup(token[6]);
				if (toknum > 7)
					chandef->real_addr = strdup(token[7]);
				if (toknum > 8)
					chandef->label = strdup(token[8]);
			}else{
				if (toknum > 6)
					chandef->label = strdup(token[6]);
			}

      Debug(DBG_CONF_FILE_READER_AFT,
        ("calling 'chandef->conf_profile = read_conf_section(file, section: %s)'\n",
        section));
			chandef->conf_profile = read_conf_section(file, section);
			free(section);

		}else{
			chandef->conf_profile = NULL;
		}

		chandef->conf = read_conf_section(file, channel_sect);	
		free(channel_sect);
		
		/* append channel definition structure to the list */
/*
		if (linkdef->chan) {
			chan_def_t* last;

			for (last = linkdef->chan;
			     last->next;
			     last = last->next)
			;
			last->next = chandef;
		}
		else{
      linkdef->chan = chandef;
    }
*/
    if(obj_list != NULL){
      obj_list->insert(list_el_chan_def);
    }else{
      printf("Error: channels list pointer is NULL!!\n");
			return ERR_CONFIG;
    }
	}
	free(conf);
	return err;
}


/*============================================================================
 * Set configuration parameter.
 *	Look up a keyword in configuration description table to find offset
 *	and data type of the configuration parameter into configuration data
 *	buffer.  Convert parameter to apporriate data type and store it the
 *	configuration data buffer.
 *
 *	Return:	0 - success
 *		1 - error
 *		-1 - not found
 */
int conf_file_reader::set_conf_param (char* key, char* val, key_word_t* dtab, void* conf,
                                      link_def_t* link_def, chan_def_t* chan_def,
                                      list_element_chan_def* list_el_chan_def)
{
	unsigned long tmp = 0;
//  conf_file_reader* cfr_tmp;

  Debug(DBG_CONF_FILE_READER, ("conf_file_reader::set_conf_param()\n"));
  Debug(DBG_CONF_FILE_READER, ("key:%s\n", key));

	/* Search a keyword in configuration definition table */
	for (; dtab->keyword && strcmp(dtab->keyword, key); ++dtab){
    ;
  }
	if(dtab->keyword == NULL){
    //searched to end of the table.
    return -1;	/* keyword not found */
  }

	/* Interpert parameter value according to its data type */
	if (dtab->dtype == DTYPE_FILENAME) {

    if(strcmp(key, "FIRMWARE") == 0){
      //printf("file name: %s\n", val);
      snprintf(link_def->firmware_file_path, MAX_PATH_LENGTH, val);

      //reading the firmware file
      //return read_data_file(
		  //	val, (data_buf_t*)((char*)conf + dtab->offset));
    }
    return 0;
	}

	if (verbose){
    printf(" * Setting %s to %s\n", key, val);
  }

	if (dtab->dtype == DTYPE_STR) {
		strcpy((char*)conf + dtab->offset, val);
		return 0;
	}

	if (!isdigit(*val) ||
	    strcmp(key, "ACTIVE_CH") == 0 ||
	    strcmp(key, "LBO") == 0 ||
	    strcmp(key, "MEDIA") == 0) {
		look_up_t* sym;
		unsigned long tmp_ch;

		strupcase(val);
		for (sym = sym_table;
		     sym->ptr && strcmp((char*)sym->ptr, val);
		     ++sym){
		  ;
    }
		if (sym->ptr == NULL) {
			/* TE1 */
			if (strcmp(key, "ACTIVE_CH")) {		
				if (verbose) printf(" * invalid term %s ...\n", val);
				return 1;
			}
			/* TE1 Convert active channel string to ULONG */
		 	tmp_ch = parse_active_channel(val, link_defs->linkconf->te_cfg.media);	
			if (tmp_ch == 0){
				if (verbose) printf("Illegal active channel range! %s ...\n", val);
				fprintf(stderr, "Illegal active channel range! min=1 max=31\n");
				return -1;
			}
			tmp = (unsigned long)tmp_ch;

      //printf("Active Channels val:\n");
      //printf("%s\n", val);
      //David
      //Active Channels can be passed for both the 'device' and the 'channel'
      //Store the active channels string for display and modification by the user.
      if(link_def != NULL){
        //channels (time slots) for this 'device'
        snprintf(link_def->active_channels_string, MAX_LEN_OF_ACTIVE_CHANNELS_STRING, "%s", val);
      }else if(conf != NULL && chan_def != NULL){
        //channels (time slots) for this 'channel'
        snprintf(chan_def->active_channels_string, MAX_LEN_OF_ACTIVE_CHANNELS_STRING, "%s", val);
      }
		}
		else tmp = sym->val;
	}
	else tmp = strtoul(val, NULL, 0);

	switch (dtab->dtype) {

	case DTYPE_INT:
		*(int*)((char*)conf + dtab->offset) = tmp;
		break;

	case DTYPE_UINT:
		*(uint*)((char*)conf + dtab->offset) = tmp;
		break;

	case DTYPE_LONG:
		*(long*)((char*)conf + dtab->offset) = tmp;
		break;

	case DTYPE_ULONG:
		*(unsigned long*)((char*)conf + dtab->offset) = tmp;
		break;

	case DTYPE_SHORT:
		*(short*)((char*)conf + dtab->offset) = tmp;
		break;

	case DTYPE_USHORT:
		*(ushort*)((char*)conf + dtab->offset) = tmp;
		break;

	case DTYPE_CHAR:
		*(char*)((char*)conf + dtab->offset) = tmp;
		break;

	case DTYPE_UCHAR:
		*(unsigned char*)((char*)conf + dtab->offset) = tmp;
    /*
    //???????????????
	  if (strcmp(key, "HDLC_STREAMING") == 0 ){
      printf("hdlc strm char: %lu\n", tmp);
    }
    //??????????????
    */
		break;

	case DTYPE_PTR:
		*(void**)((char*)conf + dtab->offset) = (void*)tmp;
		break;
	}
	return 0;
}

#if 0
int conf_file_reader::set_conf_param (char* key, char* val, key_word_t* dtab, void* conf,
                                      link_def_t* link_def, chan_def_t* chan_def)
{
	unsigned long tmp = 0;

  Debug(DBG_CONF_FILE_READER, ("conf_file_reader::set_conf_param()\n"));
  Debug(DBG_CONF_FILE_READER, ("key:%s\n", key));

	/* Search a keyword in configuration definition table */
	for (; dtab->keyword && strcmp(dtab->keyword, key); ++dtab){
    ;
  }
	if(dtab->keyword == NULL){
    //searched to end of the table.
    return -1;	/* keyword not found */
  }

	/* Interpert parameter value according to its data type */
	if (dtab->dtype == DTYPE_FILENAME) {

    if(strcmp(key, "FIRMWARE") == 0){
      //printf("file name: %s\n", val);
      snprintf(link_def->firmware_file_path, MAX_PATH_LENGTH, val);

      //reading the firmware file
      //return read_data_file(
		  //	val, (data_buf_t*)((char*)conf + dtab->offset));
    }
    return 0;
	}

	if (verbose){
    printf(" * Setting %s to %s\n", key, val);
  }

	if (dtab->dtype == DTYPE_STR) {
		strcpy((char*)conf + dtab->offset, val);
		return 0;
	}

	if (!isdigit(*val) ||
	    strcmp(key, "ACTIVE_CH") == 0 ||
	    strcmp(key, "LBO") == 0 ||
	    strcmp(key, "MEDIA") == 0) {
		look_up_t* sym;
		unsigned long tmp_ch;

		strupcase(val);
		for (sym = sym_table;
		     sym->ptr && strcmp((char*)sym->ptr, val);
		     ++sym){
		  ;
    }
		if (sym->ptr == NULL) {
			/* TE1 */
			if (strcmp(key, "ACTIVE_CH")) {		
				if (verbose) printf(" * invalid term %s ...\n", val);
				return 1;
			}
			/* TE1 Convert active channel string to ULONG */
		 	tmp_ch = parse_active_channel(val, link_defs->linkconf->te_cfg.media);	
			if (tmp_ch == 0){
				if (verbose) printf("Illegal active channel range! %s ...\n", val);
				fprintf(stderr, "Illegal active channel range! min=1 max=31\n");
				return -1;
			}
			tmp = (unsigned long)tmp_ch;

      //printf("Active Channels val:\n");
      //printf("%s\n", val);
      //David
      //Active Channels can be passed for both the 'device' and the 'channel'
      //Store the active channels string for display and modification by the user.
      if(link_def != NULL){
        //channels (time slots) for this 'device'
        snprintf(link_def->active_channels_string, MAX_LEN_OF_ACTIVE_CHANNELS_STRING, "%s", val);
      }else if(conf != NULL && chan_def != NULL){
        //channels (time slots) for this 'channel'
        snprintf(chan_def->active_channels_string, MAX_LEN_OF_ACTIVE_CHANNELS_STRING, "%s", val);
      }
		}
		else tmp = sym->val;
	}
	else tmp = strtoul(val, NULL, 0);

	switch (dtab->dtype) {

	case DTYPE_INT:
		*(int*)((char*)conf + dtab->offset) = tmp;
		break;

	case DTYPE_UINT:
		*(uint*)((char*)conf + dtab->offset) = tmp;
		break;

	case DTYPE_LONG:
		*(long*)((char*)conf + dtab->offset) = tmp;
		break;

	case DTYPE_ULONG:
		*(unsigned long*)((char*)conf + dtab->offset) = tmp;
		break;

	case DTYPE_SHORT:
		*(short*)((char*)conf + dtab->offset) = tmp;
		break;

	case DTYPE_USHORT:
		*(ushort*)((char*)conf + dtab->offset) = tmp;
		break;

	case DTYPE_CHAR:
		*(char*)((char*)conf + dtab->offset) = tmp;
		break;

	case DTYPE_UCHAR:
		*(unsigned char*)((char*)conf + dtab->offset) = tmp;
    /*
    //???????????????
	  if (strcmp(key, "HDLC_STREAMING") == 0 ){
      printf("hdlc strm char: %lu\n", tmp);
    }
    //??????????????
    */
		break;

	case DTYPE_PTR:
		*(void**)((char*)conf + dtab->offset) = (void*)tmp;
		break;
	}
	return 0;
}
#endif

/*============================================================================
 * Configure WAN link.
 */
int conf_file_reader::configure_link (link_def_t* def, char init)
{
	int err = 0;
	int len = 0;
	int i;
	key_word_t* conf_table = (key_word_t*)lookup(def->config_id, conf_def_tables);
	//char filename[sizeof(router_dir) + WAN_DRVNAME_SZ + 2];
  char filename[strlen(router_dir) + WAN_DRVNAME_SZ + 2];
	chan_def_t* chandef;
	char* conf_rec;
	int dev=-1;
	unsigned short hi_DLCI = 15;
	char *token[MAX_TOKENS];

  //David
  //list_element * list_el;
  list_element_chan_def* list_el_chan_def;

  Debug(DBG_CONF_FILE_READER, ("conf_file_reader::configure_link()\n"));

	if (def->conf == NULL) {
		show_error(ERR_CONFIG);
		return ERR_CONFIG;
	}

	/* Clear configuration structure */
/*
  //David: allocation moved to constructor - clearing not done anymore
	if (def->linkconf){
		if (def->linkconf->data) free(def->linkconf->data);
		free(def->linkconf);
		def->linkconf=NULL;
	}
*/
/*
  //David: allocation moved to constructor
	def->linkconf = (wandev_conf_t*)malloc(sizeof(wandev_conf_t));
	if (!def->linkconf){
		printf("%s: Failed to allocated linkconf!\n",prognamed);
		return -ENOMEM;
	}
	memset(def->linkconf, 0, sizeof(wandev_conf_t));
*/
  def->linkconf->magic = ROUTER_MAGIC;
	def->linkconf->config_id = def->config_id;

	/* Parse link configuration */
	for (conf_rec = def->conf; *conf_rec; conf_rec += len) {
		int toknum;

		len = strlen(conf_rec) + 1;
		toknum = tokenize(conf_rec, token);

    Debug(DBG_CONF_FILE_READER, ("toknum: %d\n", toknum));

		if (toknum < 2) {
			show_error(ERR_CONFIG);
			return ERR_CONFIG;
		}

		/* Look up a keyword first in common, then in media-specific
		 * configuration definition tables.
		 */
		strupcase(token[0]);
		err = set_conf_param(
			  token[0], token[1], common_conftab, def->linkconf, def, NULL, NULL);

		if ((err < 0) && (conf_table != NULL)){
			err = set_conf_param(
				token[0], token[1], conf_table, &def->linkconf->u, def, NULL, NULL);
    }

		if (err < 0) {
			printf(" * Unknown parameter %s\n", token[0]);
			fprintf(stderr, "\n\n\tERROR in %s !!\n",conf_file);
			if(weanie_flag) fprintf(stderr, "\tPlease check %s for errors\n", verbose_log);
			fprintf(stderr, "\n");
			return ERR_CONFIG;
		}

		if (err){
			fprintf(stderr,"\n\tError parsing %s configuration file\n",conf_file);
			if(weanie_flag) fprintf(stderr,"\tPlease check %s for errors\n", verbose_log);
			fprintf(stderr, "\n");
			return err;
		}
	}

	/* Open SDLA device and perform link configuration */
	sprintf(filename, "%s/%s", router_dir, def->name);
	
	/* prepare a list of DLCI(s) and place it in the wandev_conf_t structure
	 * This is done so that we have a list of DLCI(s) available when we
	 * call the wpf_init() routine in sdla_fr.c (triggered by the ROUTER_
	 * SETUP ioctl call) for running global SET_DLCI_CONFIGURATION cmd. This
	 * is only relevant for a station being a NODE, for CPE it polls the
	 * NODE (auto config DLCI)
	 */
	if ((def->linkconf->config_id == WANCONFIG_FR)){
		i = 0;

		if(err){
			return err;
    }
		
    //David
    if(obj_list != NULL){
      list_el_chan_def = (list_element_chan_def*)obj_list->get_first();
    }else{
      printf("Error: configure_link(): obj_list is NULL!\n");
			return ERR_CONFIG;
    }

    while(list_el_chan_def != NULL){

      chandef = &list_el_chan_def->data;
		
			if (strcmp(chandef->addr,"auto")==0){
        list_el_chan_def = (list_element_chan_def*)obj_list->get_next_element(list_el_chan_def);
        //David
        //indication that 'auto' will be used
        strcpy((char*)def->linkconf->u.fr.dlci, "auto");
				continue;
			}

			if (verbose){
				printf(" * Reading DLCI(s) Included : %s\n",
					chandef->addr ? chandef->addr : "not specified");
      }

			if (chandef->annexg != NO_ANNEXG){
        list_el_chan_def = (list_element_chan_def*)obj_list->get_next_element(list_el_chan_def);
				continue;
      }

			def->linkconf->u.fr.dlci[i] = dec_to_uint((unsigned char*)chandef->addr,0);
			if(def->linkconf->u.fr.dlci[i] > hi_DLCI) {
				hi_DLCI = def->linkconf->u.fr.dlci[i];
				i++;
			}else {
				fprintf(stderr,"\n\tDLCI(s) specified in %s are not in order\n",conf_file);
				fprintf(stderr,"\n\tor duplicate DLCI(s) assigned!\n");
				return ERR_SYSTEM;
			}

      list_el_chan_def = (list_element_chan_def*)obj_list->get_next_element(list_el_chan_def);
		}//while()
	}else{
		/* Read standard VCI/VPI list and send it */
		if ((def->linkconf->config_id == WANCONFIG_ADSL)){
			read_adsl_vci_vpi_list(&def->linkconf->u.adsl.vcivpi_list[0],
					       &def->linkconf->u.adsl.vcivpi_num);
		}
	}

#if defined(__LINUX__)
	dev = open(filename, O_RDONLY);
#else
	dev = open(DEV_DIR WANDEV_NAME, O_RDONLY);
#endif

#if defined(__FreeBSD__) || defined(__OpenBSD__)
	strcpy(config.devname, def->name);
    	config.arg = NULL;
#endif

	if (conf_file && strstr(conf_file, "ft1.conf") != NULL){
		def->linkconf->ft1 = 1;
	}else{
		def->linkconf->ft1 = 0;
	}

	if (!init){
		err=0;//exec_link_cmd(dev,def);
		if (def->linkconf->data){
			free(def->linkconf->data);
			def->linkconf->data=NULL;
		}
	}
	
	/* Configure logical channels */
  //David
	if( !err && !nodev_flag ){
    list_el_chan_def = (list_element_chan_def*)obj_list->get_first();

    while(list_el_chan_def != NULL){
		//for (chandef = def->chan; chandef; chandef = chandef->next) {
      chandef = &list_el_chan_def->data;

			switch (chandef->annexg){
				case NO_ANNEXG:
					strncpy(master_lapb_dev,chandef->name, WAN_IFNAME_SZ);
					break;
				case ANNEXG_LAPB:
					strncpy(master_x25_dev,chandef->name, WAN_IFNAME_SZ);
					break;
				case ANNEXG_X25:
					strncpy(master_dsp_dev,chandef->name, WAN_IFNAME_SZ);
					break;
			}
			
			if (!init){
				if(dev_name != NULL && strcmp(dev_name, chandef->name) != 0 ){
          list_el_chan_def = (list_element_chan_def*)obj_list->get_next_element(list_el_chan_def);
					continue;
        }
			}
			
			if (verbose) printf(
				" * Configuring channel %s (%s). Media address: %s\n",
				chandef->name,
				"no description",
				chandef->addr ? chandef->addr : "not specified");

			if (chandef->chanconf){
				free(chandef->chanconf);
				chandef->chanconf=NULL;
			}
			chandef->chanconf=(wanif_conf_t*)malloc(sizeof(wanif_conf_t));
			if (!chandef->chanconf){
				printf("%s: Failed to allocate chandef\n",prognamed);
				close(dev);
				return -ENOMEM;
			}
			memset(chandef->chanconf, 0, sizeof(wanif_conf_t));
			chandef->chanconf->magic = ROUTER_MAGIC;
			chandef->chanconf->config_id = def->config_id;
			err=configure_chan(dev, chandef, init, def->config_id, list_el_chan_def);
			if (err){
				break;
			}

      list_el_chan_def = (list_element_chan_def*)obj_list->get_next_element(list_el_chan_def);
		}//while()
	}//if()
	/* clean up */
	close(dev);
	return err;
}

/*============================================================================
 * Configure WAN logical channel.
 */
int conf_file_reader::configure_chan (int dev, chan_def_t* def, char init, int id,
                                      list_element_chan_def* list_el_chan_def)
{
	int err = 0;
	int len = 0;
	char* conf_rec;
	char *token[MAX_TOKENS];
	key_word_t* conf_annexg_table;
	key_word_t* conf_table = (key_word_t*)lookup(id, conf_if_def_tables);

  Debug(DBG_CONF_FILE_READER, ("conf_file_reader::configure_chan()\n"));	
	
	/* Prepare configuration data */
	strncpy(def->chanconf->name, def->name, WAN_IFNAME_SZ);
	if (def->addr)
		strncpy(def->chanconf->addr, def->addr, WAN_ADDRESS_SZ);

	if (def->usedby)
		strncpy(def->chanconf->usedby, def->usedby, USED_BY_FIELD);

	if (def->label){
    Debug(DBG_CONF_FILE_READER, ("configure_chan(): def->label: %s\n", def->label));

		conf_annexg_table=(key_word_t*)lookup(def->annexg, conf_annexg_def_tables);
		if (conf_annexg_table){
				set_conf_param("LABEL", def->label, conf_annexg_table, &def->chanconf->u, NULL, def, list_el_chan_def);
		}else{
			memcpy(def->chanconf->label,def->label,WAN_IF_LABEL_SZ);
		}
	}else{
    Debug(DBG_CONF_FILE_READER, ("configure_chan(): no 'def->label'\n"));	
  }
		
	if (def->virtual_addr){
		conf_annexg_table=(key_word_t*)lookup(def->annexg, conf_annexg_def_tables);
		if (conf_annexg_table){
			set_conf_param("VIRTUAL_ADDR",def->virtual_addr,
					conf_annexg_table, &def->chanconf->u, NULL, def, list_el_chan_def);
		}
	}	
	if (def->real_addr){
		if (def->annexg){
			conf_annexg_table=(key_word_t*)lookup(def->annexg, conf_annexg_def_tables);
			if (conf_annexg_table){
				set_conf_param("REAL_ADDR",def->real_addr,
						conf_annexg_table, &def->chanconf->u, NULL, def, list_el_chan_def);
			}	
		}
	}
				
	if (def->conf) for (conf_rec = def->conf; *conf_rec; conf_rec += len) {
		int toknum;
		
		len = strlen(conf_rec) + 1;
		toknum = tokenize(conf_rec, token);
		if (toknum < 2) {
			show_error(ERR_CONFIG);
			return -EINVAL;
		}

		/* Look up a keyword first in common, then media-specific
		 * configuration definition tables.
		 */
		strupcase(token[0]);
		if (set_conf_param(token[0], token[1], chan_conftab, def->chanconf, NULL, def, list_el_chan_def)) {
			if (!conf_table || set_conf_param(token[0], token[1], conf_table, &def->chanconf->u, NULL, def, list_el_chan_def)) {
				conf_annexg_table=(key_word_t*)lookup(def->annexg, conf_annexg_def_tables);
				if (def->annexg!=ANNEXG_DSP ||
				    !conf_annexg_table || set_conf_param(token[0], token[1],
							  conf_annexg_table, &def->chanconf->u, NULL, def, list_el_chan_def)) {
				
					printf("Invalid parameter %s\n", token[0]);
					show_error(ERR_CONFIG);
					return ERR_CONFIG;
				}
			}
		}
	}

	len=0;
	
	if (def->annexg){

		conf_annexg_table=(key_word_t*)lookup(def->annexg, conf_annexg_def_tables);

		if (def->conf_profile) for (conf_rec = def->conf_profile; *conf_rec; conf_rec += len) {
			int toknum;

			len = strlen(conf_rec) + 1;
			toknum = tokenize(conf_rec, token);
			if (toknum < 2) {
				show_error(ERR_CONFIG);
				return -EINVAL;
			}

			/* Look up a keyword first in common, then media-specific
			 * configuration definition tables.
			 */
			strupcase(token[0]);

			if (!conf_annexg_table ||
			     set_conf_param(token[0], token[1], conf_annexg_table, &def->chanconf->u, NULL, def, list_el_chan_def)) {
				if (set_conf_param(token[0], token[1], chan_conftab, def->chanconf, NULL, def, list_el_chan_def)) {
					printf("Invalid parameter %s\n", token[0]);
					show_error(ERR_CONFIG);
					return ERR_CONFIG;
				}
			}
		}
	}
  else{
    //DBG_CONF_FILE_READER_AFT
		if (def->conf_profile){
      for (conf_rec = def->conf_profile; *conf_rec; conf_rec += len) {
  			int toknum;

	  		len = strlen(conf_rec) + 1;
		  	toknum = tokenize(conf_rec, token);

        Debug(DBG_CONF_FILE_READER_AFT, ("toknum: %d\n", toknum));	

			  if (toknum != 2) {
  				show_error(ERR_CONFIG);
	  			return -EINVAL;
		  	}

        //FIXIT: only frame relay supported for now
			  //Look up a keyword first in Frame Relay configuration definition tables.
		  	strupcase(token[0]);

        Debug(DBG_CONF_FILE_READER_AFT, ("def->addr: %s\n", def->addr));	
        Debug(DBG_CONF_FILE_READER_AFT, ("def->LIP_protocol: %d\n", def->LIP_protocol));	

				switch(def->LIP_protocol)
        {
        case WANCONFIG_MFR:
          if(set_conf_param(token[0], token[1], fr_conftab, &def->chanconf->u.fr,
                            NULL, def, list_el_chan_def)) {
  				  printf("Invalid parameter %s\n", token[0]);
	  			  show_error(ERR_CONFIG);
	  		    return ERR_CONFIG;
  			  }//if()
          break;

        case WANCONFIG_MPPP:
          //there is no 'chanconf->u.ppp'
/*
          if(set_conf_param(token[0], token[1], fr_conftab, &def->chanconf->u.ppp,
                            NULL, def, list_el_chan_def)) {
  				  printf("Invalid parameter %s\n", token[0]);
	  			  show_error(ERR_CONFIG);
	  		    return ERR_CONFIG;
  			  }//if()
*/
          break;

        case WANCONFIG_MPCHDLC:
          //there is no 'chanconf->u.c'
/*
          if(set_conf_param(token[0], token[1], fr_conftab, &def->chanconf->u.c,
                            NULL, def, list_el_chan_def)) {
  				  printf("Invalid parameter %s\n", token[0]);
	  			  show_error(ERR_CONFIG);
	  		    return ERR_CONFIG;
  			  }//if()
*/
          break;

        default:
          ERR_DBG_OUT(("Unsupported protocol (def->LIP_protocol: %d)!\n", def->LIP_protocol));
   		    return ERR_CONFIG;
        }

#if DBG_CONF_FILE_READER	
        wan_fr_conf_t* fr = &def->chanconf->u.fr;
        Debug(DBG_CONF_FILE_READER_AFT, ("fr->signalling: %d\n", fr->signalling));	
        Debug(DBG_CONF_FILE_READER_AFT, ("fr->station   : %d\n", fr->station));	
#endif



	  	}//for()
    }//if()
  }

	err=0;

	if (init){
		return err;
	}
	
	return err;
}

void conf_file_reader::free_linkdefs(void)
{
  Debug(DBG_CONF_FILE_READER, ("conf_file_reader::free_linkdefs()\n"));	

  //David
  if(obj_list == NULL){
    printf("Error: free_linkdefs(): obj_list is NULL! \n");
    return;
	}

	Debug(DBG_CONF_FILE_READER, ("Freeing Link %s\n", link_defs->name));

	/* Clear (by deleteting) channel definition list */
  delete obj_list;
  obj_list = NULL;

  if(link_defs == NULL){
	  Debug(DBG_CONF_FILE_READER, ("Freeing Link %s\n", link_defs->name));
    return;
  }
	
	if (link_defs->conf) free(link_defs->conf);
	if (link_defs->descr) free(link_defs->descr);
		
	if (link_defs->linkconf){
		if (link_defs->linkconf->data){
			free(link_defs->linkconf->data);
		}
		free(link_defs->linkconf);
	}

	free(link_defs);
  link_defs = NULL;
}


/*============================================================================
 * Show error message.
 */
void conf_file_reader::show_error_dbg (int err)
{
	if (err == ERR_SYSTEM){
		fprintf(stderr, "%s: SYSTEM ERROR %d: %s!\n",
	 		prognamed, errno, strerror(errno));
	}else{
		fprintf(stderr, "%s: ERROR: %s : %s!\n", prognamed,
			err_messages[min(err, ERR_LIMIT) - 2], conf_file);
	}
}

void conf_file_reader::read_adsl_vci_vpi_list(wan_adsl_vcivpi_t* vcivpi_list,
                                              unsigned short* vcivpi_num)
{
	FILE*		file = NULL;
	char		buf[256];
	int		num = 0;
	int 	vci, vpi;

	file = fopen(adsl_file, "r");
	if (file == NULL){
		fprintf( stderr, "%s: cannot open %s\n",
				prognamed, adsl_file);
		show_error(ERR_SYSTEM);
		exit(ERR_SYSTEM);
	}
	
	while(fgets(buf, sizeof(buf) -1, file)){
		if (buf[0] != '#'){
			sscanf(buf, "%d %d", &vci, &vpi);
			vcivpi_list[num].vci = (unsigned short)vci;
			vcivpi_list[num].vpi = (unsigned char)vpi;
			num++;
			if (num >= 100){
				break;
			}
		}
	}	
	*vcivpi_num = num;
	fclose(file);
	return;
}

//this function extracts version of S514 (1/2/...7) card from the "conf" file.
//do not call it for other cards
int conf_file_reader::
  get_pci_adapter_type_from_wanpipe_conf_file(char * conf_file_name, char* card_version)
{
  FILE * conf_file = fopen(conf_file_name, "r+");
  char str_buff[MAX_PATH_LENGTH];

  if(conf_file == NULL){
    ERR_DBG_OUT(("Failed to open configuration file '%s'. File does not exist.\n",
      conf_file_name));
    return NO;
  }

  *card_version = INVALID_CARD_TYPE;
  /////////////////////////////////////////////////////
  Debug(DBG_CONF_FILE_READER, ("Conf file :\n"));
  do{
    fgets(str_buff, MAX_PATH_LENGTH, conf_file);

    if(!feof(conf_file)){

      //Debug(DBG_CONF_FILE_READER, (str_buff));

      if(strstr(str_buff, "CARD_TYPE") != NULL){
        if(get_card_type_from_str(str_buff) == NO){
          //Invalid card type or failed to parse the string.
          return NO;
        }
      }

      *card_version = S5141_ADPTR_1_CPU_SERIAL;

      //if 'MEDIA' exist it can be: 56K or TE1 card
      if(strstr(str_buff, "MEDIA") != NULL){
        if(get_media_type_from_str(str_buff, card_version) == NO){
          //Invalid media type or failed to parse the string.
          return NO;
        }else{
          return YES;
        }
      }//if()
    }//if()

  }while(!feof(conf_file));
  Debug(DBG_CONF_FILE_READER, ("\n"));

  if(*card_version == INVALID_CARD_TYPE){
    ERR_DBG_OUT(("Failed to get S514 card version from 'conf' file!!\n"));
    return NO;
  }else{
    return YES;
  }
}

//call this function
int conf_file_reader::get_card_type_from_str(char * line)
{
  //check it is S514X card
  if(strstr(line, "S51X") != NULL){
    return YES;
  }
  return NO;
}

int conf_file_reader::get_media_type_from_str(char * line, char* card_version)
{
  char * media_str;

  //check if this is a 56K card.
  media_str = strstr(line, "56K");
  if(media_str != NULL){
    printf("S514-5 56K card\n");
    *card_version = S5145_ADPTR_1_CPU_56K;
    return YES;
  }

  printf("Media type is ");

  //is it T1?
  media_str = strstr(line, "T1");
  if(media_str != NULL){
    printf("T1\n");
    *card_version = S5144_ADPTR_1_CPU_T1E1;
    return YES;
  }

  //is it E1?
  media_str = strstr(line, "E1");
  if(media_str != NULL){
    printf("E1\n");
    *card_version = S5144_ADPTR_1_CPU_T1E1;
    return YES;
  }

  return NO;
}

void* conf_file_reader::get_last_real_channelized_device()
{
  if(obj_list != NULL){

  }else{
    ERR_DBG_OUT(("Channels list pointer is NULL!!\n"));
	  return NULL;
  }

  //NOTE: it is assumed that 'real' device is read first than all the interfaces (DLCIs)
  //so we have to search backwards.
  list_element_chan_def* list_el_chan_def = (list_element_chan_def*)obj_list->get_last();

  while(list_el_chan_def != NULL){

    //on real devices 'LIP_protol' is not used, so it must be zero.
    if(list_el_chan_def->data.LIP_protocol == 0){
      Debug(DBG_CONF_FILE_READER_AFT, ("Found 'real channelized' device: %s\n",
        list_el_chan_def->data.name));
      return list_el_chan_def;
    }
    list_el_chan_def = (list_element_chan_def*)obj_list->get_previous_element(list_el_chan_def);
  }//while()

  //if got here 'real' device is NOT found. it is an error - it must be there.
  ERR_DBG_OUT(("Failed to find 'real channelized' device!\n"));
  return NULL;
}

////////////////////////////////////////////////////////////////////////////////////////////////
//Global (non-member) functions.
void init_first_time_tokens(char **token){
	int i;

	for (i=0;i<MAX_TOKENS;i++){
		token[i]=NULL;
	}
}

/*============================================================================
 * Tokenize string.
 *	Parse a string of the following syntax:
 *		<tag>=<arg1>,<arg2>,...
 *	and fill array of tokens with pointers to string elements.
 *
 *	Return number of tokens.
 */
void init_tokens(char **token){
	int i;

	for (i=0;i<MAX_TOKENS;i++){
		token[i]=NULL;
	}
}

/* Bug Fix by Ren Scharfe <l.s.r@web.de>
 * removed strtok
 */
int tokenize (char *str, char **tokens)
{
	int cnt = 0;
 	char *tok;

	init_tokens(tokens);

	if (!str){
		return 0;
	}

 	tok = strchr(str, '=');
  if (!tok)
		return 0;

	*tok='\0';

 	tokens[cnt] = str;
	str=++tok;
	
 	while (tokens[cnt] && (cnt < MAX_TOKENS-1)) {
	
		tokens[cnt] = strstrip(tokens[cnt], " \t");
		
   	if ((tok = strchr(str, ',')) == NULL){
			tokens[++cnt] = strstrip(str, " \t");
			goto end_tokenize;
		}

		*tok='\0';

		tokens[++cnt] = strstrip(str, " \t");
		str=++tok;
 	}

end_tokenize:
 	return ++cnt;
}

/*============================================================================
 * Strip leading and trailing spaces off the string str.
 */
char* strstrip (char* str, char* s)
{
	char* eos = str + strlen(str);		/* -> end of string */

	while (*str && strchr(s, *str))
		++str;				/* strip leading spaces */
	while ((eos > str) && strchr(s, *(eos - 1)))
		--eos;				/* strip trailing spaces */
	*eos = '\0';
	return str;
}

/*============================================================================
 * Uppercase string.
 */
char* strupcase (char* str)
{
	char* s;

	for(s = str; *s; ++s) *s = toupper(*s);
	return str;
}

/*============================================================================
 * Get a pointer from a look-up table.
 */
void* lookup (int val, look_up_t* table)
{
	while(table->val && (table->val != (uint)val)){
    table++;
  }
	return table->ptr;
}

/*============================================================================
 * Look up a symbolic name in name table.
 *	Return a numeric value associated with that name or zero, if name was
 *	not found.
 */
int name2val (char* name, look_up_t* table)
{
	while (table->ptr && strcmp(name, (char*)table->ptr)) table++;
	return table->val;
}

/*============================================================================
 * Convert decimal string to unsigned integer.
 * If len != 0 then only 'len' characters of the string are converted.
 */
unsigned int dec_to_uint (unsigned char* str, int len)
{
	unsigned val;

	if (!len) len = strlen((char*)str);
	for (val = 0; len && is_digit(*str); ++str, --len)
		val = (val * 10) + (*str - (unsigned)'0')
	;
	return val;
}

//****** End *****************************************************************/
