/***************************************************************************
                          conf_file_reader.h  -  description
                             -------------------
    begin                : Thu Mar 4 2004
    copyright            : (C) 2004 by David Rokhvarg
    email                : davidr@sangoma.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef CONF_FILE_READER_H
#define CONF_FILE_READER_H

#include "wancfg.h"

#include <stddef.h>
#include <stdlib.h>
#include <stdio.h>
#include <errno.h>
#include <fcntl.h>
#include <string.h>
#include <ctype.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <dirent.h>
#include <unistd.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <unistd.h>
#include <signal.h>
#include <time.h>
//#include "lib/safe-read.h"
#if defined(__FreeBSD__) || defined(__OpenBSD__)
# include <net/wanpipe_version.h>
# include <net/wanproc.h>
# include <net/wanpipe.h>
#else
# include <linux/wanpipe_version.h>
# include <linux/wanpipe.h>
#endif


/****** Defines *************************************************************/

#ifndef	min
#define	min(a,b)	(((a)<(b))?(a):(b))
#endif

#define	is_digit(ch) (((ch)>=(unsigned)'0'&&(ch)<=(unsigned)'9')?1:0)

enum ErrCodes			/* Error codes */
{
	ERR_SYSTEM = 1,		/* system error */
	ERR_SYNTAX,		/* command line syntax error */
	ERR_CONFIG,		/* configuration file syntax error */
	ERR_MODULE,		/* module not loaded */
	ERR_LIMIT
};

/* Configuration stuff */
#define	MAX_CFGLINE	256
#define	MAX_CMDLINE	256
#define	MAX_TOKENS	32

/* Device directory and WAN device name */
#define DEV_DIR "/dev/"
#define WANDEV_NAME "wanrouter"

/****** Data Types **********************************************************/

typedef struct look_up		/* Look-up table entry */
{
	uint	val;		/* look-up value */
	void*	ptr;		/* pointer */
} look_up_t;

typedef struct key_word		/* Keyword table entry */
{
	char*	keyword;	/* -> keyword */
	uint	offset;		/* offset of the related parameter */
	int	dtype;		  /* data type */
} key_word_t;

typedef struct data_buf		/* General data buffer */
{
	unsigned size;
	void* data;
} data_buf_t;

/*
 * Data types for configuration structure description tables.
 */
#define	DTYPE_INT	1
#define	DTYPE_UINT	2
#define	DTYPE_LONG	3
#define	DTYPE_ULONG	4
#define	DTYPE_SHORT	5
#define	DTYPE_USHORT	6
#define	DTYPE_CHAR	7
#define	DTYPE_UCHAR	8
#define	DTYPE_PTR	9
#define	DTYPE_STR	10
#define	DTYPE_FILENAME	11

#define NO_ANNEXG	0x00
#define ANNEXG_LAPB	0x01
#define ANNEXG_X25	0x02
#define ANNEXG_DSP	0x03

//David
//additinon for Frame Relay on AFT
#define ANNEXG_FR	0x04

#define WANCONFIG_SOCKET "/etc/wanpipe/wanconfig_socket"
#define WANCONFIG_PID    "/var/run/wanconfig.pid"
#define WANCONFIG_PID_FILE "/proc/net/wanrouter/pid"



#define show_error(x) { printf("Error Line %i\n",__LINE__); show_error_dbg(x);}

#include "wancfg.h"

#include "objects_list.h"

/**Reads wanpipe#.conf file. Allocates configuration structures and lists.
  *@author David Rokhvarg
  */

class conf_file_reader {

  int verbose;			/* verbosity level */
  //char* conf_file;
  char conf_file[MAX_PATH_LENGTH];

  //NULL;			/* configuration file */
  char* adsl_file;			/* ADSL VCI/VPI list */
  char *dev_name;
  char *card_name;
  int  weanie_flag;
  int  nodev_flag;

#if defined(__FreeBSD__) || defined(__OpenBSD__)
  wan_conf_t	config;
#endif

public:
  //this member initialized as follows:
  //1. If "New" wanpipe - user selects wanpipe number.
  //2. If "Edit existing" - wanpipe number is given.
  int wanpipe_number;

  //per card (CPU) configuration
  link_def_t * link_defs;
  //list of logical channels
  objects_list * obj_list;

	conf_file_reader(int file_number);
  conf_file_reader(char* template_file_path);
  void init(int wanpipe_number);
	~conf_file_reader();

  int configure (void);
  int parse_conf_file(char* fname);
  int build_linkdef_list (FILE* file);
  char* read_conf_section (FILE* file, char* section);
  int build_chandef_list (FILE* file);

  int set_conf_param (char* key, char* val, key_word_t* dtab, void* conf,
                      link_def_t* link_def, chan_def_t* chan_def,
                      list_element_chan_def* list_el_chan_def);

  int configure_link (link_def_t* def, char init);
  int configure_chan (int dev, chan_def_t* def, char init, int id,
                      list_element_chan_def* list_el_chan_def);

  int read_conf_record (FILE* file, char* key);
  void free_linkdefs(void);

  int read_conf_file();//reads the wanpipe#.conf file, full path should be at 'conf_file'
  void show_error_dbg (int err);
  void read_adsl_vci_vpi_list(wan_adsl_vcivpi_t* vcivpi_list,
                              unsigned short* vcivpi_num);

  ///////////////////
  int get_pci_adapter_type_from_wanpipe_conf_file(char * conf_file_name, char* card_version);
  int get_card_type_from_str(char * line);
  int get_media_type_from_str(char * line, char* card_version);

  void* get_last_real_channelized_device();
};

#endif
