/***************************************************************************
                          conf_file_writer.cpp  -  description
                             -------------------
    begin                : Fri Apr 2 2004
    copyright            : (C) 2004 by David Rokhvarg
    email                : davidr@sangoma.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "conf_file_writer.h"

void print_look_up_t_table(look_up_t* table);
void print_key_word_t_table(key_word_t* table);

extern key_word_t common_conftab[];
extern look_up_t config_id_str[];
extern key_word_t fr_conftab[];
extern key_word_t ppp_conftab[];
extern key_word_t chdlc_conftab[];
extern key_word_t chan_conftab[];

#define DBG_CONF_FILE_WRITER 1


//////////////////////////////////////////////////////////////
//Tables for converting integer values to string values
//NOTE: each integer value in a table MUST BE UNIQUE.
look_up_t	card_type_table[] =
{
	{ WANOPT_S50X,		(void*)"S50X"		},
	{ WANOPT_S51X,		(void*)"S51X"		},
	{ WANOPT_ADSL,		(void*)"ADSL"		},
	{ WANOPT_ADSL,		(void*)"S518"		},
	{ WANOPT_AFT,     (void*)"AFT"    },
	{ 0,			        NULL		        }
};

look_up_t	yes_no_options_table[] =
{
	{ WANOPT_YES,		(void*)"YES"		},
	{ WANOPT_NO,		(void*)"NO" 		},
	{ 0,			      NULL		        }
};

look_up_t	physical_medium_options_table[] =
{
  { WANOPT_MEDIA_NONE,    (void*)"None"   },
  { WANOPT_MEDIA_T1,      (void*)"T1"     },
  { WANOPT_MEDIA_E1,      (void*)"E1"     },
	{ WANOPT_MEDIA_56K,     (void*)"56K"    },
	{ 0,			              NULL		        }
};

look_up_t	te1_line_code_options_table[] =
{
  { WANOPT_LC_AMI,        (void*)"AMI"    },
  { WANOPT_LC_B8ZS,       (void*)"B8ZS"   },
  { WANOPT_LC_HDB3,       (void*)"HDB3"   },
	{ 0,			              NULL		        }
};

look_up_t	te1_clock_mode_options_table[] =
{
  { WANOPT_NORMAL_CLK,   	(void*)"NORMAL" },
  { WANOPT_MASTER_CLK,   	(void*)"MASTER" },
	{ 0,			              NULL		        }
};

look_up_t te1_frame_options_table[] =
{
  { WANOPT_FR_D4,         (void*)"D4"       },
  { WANOPT_FR_ESF,        (void*)"ESF"      },
  { WANOPT_FR_NCRC4,      (void*)"NCRC4"    },
  { WANOPT_FR_CRC4,       (void*)"CRC4"     },
  { WANOPT_FR_UNFRAMED,   (void*)"UNFRAMED" },
	{ 0,			              NULL		          }
};

look_up_t t1_line_build_out_options_table[] =
{
  { WANOPT_T1_LBO_0_DB,  	(void*)"0DB"           },
  { WANOPT_T1_LBO_75_DB,  (void*)"7.5DB"         },
  { WANOPT_T1_LBO_15_DB,  (void*)"15DB"          },
  { WANOPT_T1_LBO_225_DB, (void*)"22.5DB"        },
  { WANOPT_T1_0_110,  	  (void*)"0-110FT"       },
  { WANOPT_T1_110_220,  	(void*)"110-220FT"     },
  { WANOPT_T1_220_330,  	(void*)"220-330FT"     },
  { WANOPT_T1_330_440,  	(void*)"330-440FT"     },
  { WANOPT_T1_440_550,  	(void*)"440-550FT"     },
  { WANOPT_T1_550_660,  	(void*)"550-660FT"     },
	{ 0,			              NULL		               }
};

look_up_t serial_interface_type_options_table[] =
{
	{ WANOPT_RS232,	(void*)"RS232"	},
	{ WANOPT_V35,		(void*)"V35"		},
	{ 0,			              NULL	  }
};

look_up_t serial_clock_type_options_table[] =
{
	{ WANOPT_EXTERNAL,	(void*)"EXTERNAL"	},
	{ WANOPT_INTERNAL,	(void*)"INTERNAL"	},
	{ 0,			              NULL	        }
};

look_up_t frame_relay_in_channel_signalling_options_table[] =
{
	/*----- Frame relay in-channel signalling */
	{ WANOPT_FR_ANSI,	(void*)"ANSI"		},
	{ WANOPT_FR_Q933,	(void*)"Q933"		},
	{ WANOPT_FR_LMI,	(void*)"LMI"		},
	{ 0,			              NULL	    }
};

look_up_t frame_relay_station_type_options_table[] =
{
	{ WANOPT_CPE,		(void*)"CPE"		},
	{ WANOPT_NODE,		(void*)"NODE"		},
	{ 0,			              NULL	    }
};

look_up_t commport_type_options_table[] =
{
	{ WANOPT_PRI,           (void*)"PRI"  },
	{ WANOPT_SEC,           (void*)"SEC"  },
	{ 0,			              NULL	        }
};

look_up_t ppp_ip_mode_options_table[] =
{
	/*----- PPP IP Mode Options -----------*/
	{ WANOPT_PPP_STATIC,	(void*)"STATIC"	},
	{ WANOPT_PPP_HOST,	(void*)"HOST"		},
	{ WANOPT_PPP_PEER,	(void*)"PEER"		},
	{ 0,			              NULL	        }
};

look_up_t connection_type_options_table[] =
{
	/*----- Connection options ------------*/
	{ WANOPT_PERMANENT,	(void*)"PERMANENT"	},
	{ WANOPT_SWITCHED,	(void*)"SWITCHED"	},
	{ WANOPT_ONDEMAND,	(void*)"ONDEMAND"	},
	{ 0,			              NULL	        }
};

look_up_t data_encoding_options_table[] =
{
	/*----- Data encoding -----------------*/
	{ WANOPT_NRZ,		(void*)"NRZ"		},
	{ WANOPT_NRZI,	(void*)"NRZI"		},
	{ WANOPT_FM0,		(void*)"FM0"		},
	{ WANOPT_FM1,		(void*)"FM1"		},
	{ 0,			              NULL	  }
};

look_up_t line_idle_options_table[] =
{
	/*----- Idle Line ----------------------*/
	{ WANOPT_IDLE_FLAG,	(void*)"FLAG"	},
	{ WANOPT_IDLE_MARK,	(void*)"MARK"	},
	{ 0,			              NULL	  }
};

look_up_t protocol_options_table[] =
{
	{ WANCONFIG_MPPP,	    (void*)"MP_PPP"	},
	{ WANCONFIG_MPCHDLC,	(void*)"MP_CHDLC"	},
	{ WANCONFIG_MFR, 	    (void*)"MP_FR"		},
	{ WANOPT_NO,		      (void*)"RAW"		},
	{ WANCONFIG_HDLC,		  (void*)"HDLC"		},
	{ WANCONFIG_PPP,	    (void*)"PPP"		},
	{ WANCONFIG_CHDLC,	  (void*)"CHDLC"		},
	{ 0,			              NULL	  }
};

//////////////////////////////////////////////////////////////

conf_file_writer::conf_file_writer(IN conf_file_reader* cfr)
{
  Debug(DBG_CONF_FILE_WRITER, ("conf_file_writer::conf_file_writer()\n"));
  this->cfr = cfr;
}

conf_file_writer::~conf_file_writer()
{
  Debug(DBG_CONF_FILE_WRITER, ("conf_file_writer::~conf_file_writer()\n"));
}

int conf_file_writer::write()
{
  Debug(DBG_CONF_FILE_WRITER, ("conf_file_writer::write()\n"));
  char tmp_buff[MAX_PATH_LENGTH];
  link_def_t * link_def = cfr->link_defs;
  wandev_conf_t *linkconf = cfr->link_defs->linkconf;
  string main_conf_str = "";
  string top_comment_str;
  string devices_section_str;
  string interfaces_section_str;
  string wanpipe_section_str;
  string wp_interface;
  string full_file_path;

  objects_list * obj_list = cfr->obj_list;
  list_element_chan_def* list_el_chan_def = (list_element_chan_def*)obj_list->get_first();

  //The whole string written to the conf file - assembeled from
  //smaller per-section strings.
  main_conf_str = "";

  //Top comment string. Contains name of the Wan Device, time of creation etc.
  top_comment_str = "";

  //[devices] section string
  devices_section_str = "";

  //[interfaces] section string
  interfaces_section_str = "";

  //[wanpipe??] section string
  wanpipe_section_str = "";

  //per-interface string. call for each interface. on return add to the 'main_conf_str'.
  wp_interface = "";

  //form full path to the conf file we want to write
  full_file_path = wanpipe_cfg_dir;

  full_file_path += cfr->link_defs->name;
  full_file_path += ".conf";

  ///////////////////////////////////////////////////////
  if(form_top_comment_str(top_comment_str) == NO){
    return NO;
  }else{
    main_conf_str += top_comment_str;
  }
  ///////////////////////////////////////////////////////
  if(form_devices_section_str(devices_section_str) == NO){
    return NO;
  }else{
    main_conf_str += devices_section_str;
  }
  ///////////////////////////////////////////////////////
  snprintf(tmp_buff, MAX_PATH_LENGTH,"\n[interfaces]\n");
  main_conf_str += tmp_buff;

  while(list_el_chan_def != NULL){
    Debug(DBG_CONF_FILE_WRITER, ("name: %s\n", list_el_chan_def->data.name));

    interfaces_section_str = "";
    if(form_interface_str(interfaces_section_str, list_el_chan_def) == NO){
      return NO;
    }else{
      main_conf_str += interfaces_section_str;
    }
    list_el_chan_def = (list_element_chan_def*)obj_list->get_next_element(list_el_chan_def);
  }
  ///////////////////////////////////////////////////////
  snprintf(tmp_buff, MAX_PATH_LENGTH,"\n[%s]\n", cfr->link_defs->name);
  main_conf_str += tmp_buff;

  if(form_wanpipe_section_str(wanpipe_section_str) == NO){
    return NO;
  }else{
    main_conf_str += wanpipe_section_str;
  }

  ///////////////////////////////////////////////////////
  //if AFT
  Debug(DBG_CONF_FILE_WRITER, ("link_def->config_id: %d\n", link_def->config_id));
  if(link_def->config_id == WANCONFIG_AFT){

    string profile_section;
    list_el_chan_def = (list_element_chan_def*)obj_list->get_first();

    while(list_el_chan_def != NULL){
      Debug(DBG_CONF_FILE_WRITER, ("name: %s\n", list_el_chan_def->data.name));

      profile_section = "";

      /////////////////////////////////////////////////////////////////
      if(form_profile_str(profile_section, list_el_chan_def) == NO){
        return NO;
      }else{
        main_conf_str += "\n";
        main_conf_str += profile_section;
      }
      /////////////////////////////////////////////////////////////////
      profile_section = "";
      if(form_lip_layer_str(profile_section, list_el_chan_def) == NO){
        return NO;
      }else{
        //main_conf_str += "\n";
        main_conf_str += profile_section;
      }
      /////////////////////////////////////////////////////////////////
      list_el_chan_def = (list_element_chan_def*)obj_list->get_next_element(list_el_chan_def);
    }
  }

  ///////////////////////////////////////////////////////
  //per-interface section of type: [interface name]
  list_el_chan_def = (list_element_chan_def*)obj_list->get_first();

  while(list_el_chan_def != NULL){
    Debug(DBG_CONF_FILE_WRITER, ("name: %s\n", list_el_chan_def->data.name));

    wp_interface = " ";
    if(form_per_interface_str(wp_interface, list_el_chan_def, NO) == NO){
      return NO;
    }else{
      main_conf_str += wp_interface;
    }
    list_el_chan_def = (list_element_chan_def*)obj_list->get_next_element(list_el_chan_def);
  }

  ///////////////////////////////////////////////////////
  Debug(DBG_CONF_FILE_WRITER, ("main_conf_str:\n%s\n", (char*)main_conf_str.c_str()));
//???
//  return YES;

  return write_string_to_file((char*)full_file_path.c_str(),
                              (char*)main_conf_str.c_str());
}

int conf_file_writer::form_top_comment_str(string& top_comment_str)
{
  char tmp_buff[MAX_PATH_LENGTH];

  str_toupper(cfr->link_defs->name);

  snprintf(tmp_buff, MAX_PATH_LENGTH,
"#================================================\n\
# %s Configuration File\n\
#================================================\n\
#\n\
# Date: %s\
#\n\
# Note: This file was generated automatically\n\
#       by /usr/sbin/%s program.\n\
#\n\
#       If you want to edit this file, it is\n\
#       recommended that you use wancfg program\n\
#       to do so.\n\
#================================================\n\
# Sangoma Technologies Inc.\n\
#================================================\n",
cfr->link_defs->name,
get_date_and_time(),
WANCFG_EXECUTABLE_NAME
);

  str_tolower(cfr->link_defs->name);

  top_comment_str = tmp_buff;

  return YES;
}

//[devices]
//wanpipe2 = WAN_FR, Comment
int conf_file_writer::form_devices_section_str(string& devices_section_str)
{
  char tmp_buff[MAX_PATH_LENGTH];

  //check that 'config_id' is in the table.
  if(get_keyword_from_look_up_t_table(config_id_str,
                                      cfr->link_defs->config_id) == NULL){
    ERR_DBG_OUT(("Invalid 'config_id' (%d) passed for saving to file!!\n",
      cfr->link_defs->config_id));
    return NO;
  }

  switch(cfr->link_defs->config_id)
  {
  case WANCONFIG_FR:
  snprintf(tmp_buff, MAX_PATH_LENGTH,
"\n\
[devices]\n\
%s = %s, %s\n",
cfr->link_defs->name,
get_keyword_from_look_up_t_table(config_id_str, cfr->link_defs->config_id),
(cfr->link_defs->descr == NULL ? "Comment" : cfr->link_defs->descr)
);

    break;

  case WANCONFIG_PPP:
  snprintf(tmp_buff, MAX_PATH_LENGTH,
"\n\
[devices]\n\
%s = %s, %s\n",
cfr->link_defs->name,
get_keyword_from_look_up_t_table(config_id_str, cfr->link_defs->config_id),
(cfr->link_defs->descr == NULL ? "Comment" : cfr->link_defs->descr)
);

    break;

  case WANCONFIG_CHDLC:
  snprintf(tmp_buff, MAX_PATH_LENGTH,
"\n\
[devices]\n\
%s = %s, %s\n",
cfr->link_defs->name,
get_keyword_from_look_up_t_table(config_id_str, cfr->link_defs->config_id),
(cfr->link_defs->descr == NULL ? "Comment" : cfr->link_defs->descr)
);

    break;

  case WANCONFIG_HDLC:
    //HDLC streaming - used with CHDLC firmware.
  snprintf(tmp_buff, MAX_PATH_LENGTH,
"\n\
[devices]\n\
%s = %s, %s\n",
cfr->link_defs->name,
get_keyword_from_look_up_t_table(config_id_str, cfr->link_defs->config_id),
(cfr->link_defs->descr == NULL ? "Comment" : cfr->link_defs->descr)
);

    break;

  case WANCONFIG_AFT:
  snprintf(tmp_buff, MAX_PATH_LENGTH,
"\n\
[devices]\n\
%s = %s, %s\n",
cfr->link_defs->name,
get_keyword_from_look_up_t_table(config_id_str, cfr->link_defs->config_id),
(cfr->link_defs->descr == NULL ? "Comment" : cfr->link_defs->descr)
);

    break;

  default:
    ERR_DBG_OUT(("Unsupported 'config_id' (%d) passed for saving to file!!\n",
      cfr->link_defs->config_id));
    return NO;
  }


  Debug(DBG_CONF_FILE_WRITER, ("devices_section_str: %s\n", tmp_buff));
  devices_section_str += tmp_buff;
  return YES;
}

char* conf_file_writer::get_keyword_from_key_word_t_table(IN key_word_t* table,
                                                          IN int offset_in_structure)
{
  char* keyword;
  key_word_t* orig_table_ptr = table;

  Debug(DBG_CONF_FILE_WRITER, ("get_keyword_from_key_word_t_table()\n"));

  keyword = table->keyword;
  while(keyword != NULL && table->offset != (unsigned int)offset_in_structure){
    //Debug(DBG_CONF_FILE_WRITER, ("keyword: %s\n", table->keyword));
    Debug(0, ("keyword: %s\n", table->keyword));
    table++;
    keyword = table->keyword;
  }

  if(keyword == NULL){
    Debug(1, ("get_keyword_from_key_word_t_table(): failed to find value at offset: %d\n",
      offset_in_structure));
    print_key_word_t_table(orig_table_ptr);
    exit(1);
  }

  return keyword;
}

void print_key_word_t_table(key_word_t* table)
{
  char* keyword;

  Debug(1, ("\nPrinting the searched table:\n"));

  keyword = table->keyword;
  while(keyword != NULL){
    Debug(1, ("keyword: %s at offset: %d\n", table->keyword, table->offset));
    table++;
    keyword = table->keyword;
  }
  Debug(1, ("End of table.\n\n"));
}

char* conf_file_writer::get_keyword_from_look_up_t_table(IN look_up_t* table,
                                                         IN int search_value)
{
  char* keyword;
  look_up_t* orig_table_ptr = table;

  Debug(DBG_CONF_FILE_WRITER, ("get_keyword_from_look_up_t_table()\n"));

  keyword = (char*)table->ptr;
  while(keyword != NULL && table->val != (unsigned int)search_value){
    //Debug(DBG_CONF_FILE_WRITER, ("keyword: %s\n", keyword));
    Debug(0, ("keyword: %s\n", keyword));
    table++;
    keyword = (char*)table->ptr;
  }

  if(keyword == NULL){
    Debug(1, ("get_keyword_from_look_up_t_table(): failed to find value: 0x%X\n",
      search_value));
    print_look_up_t_table(orig_table_ptr);
    exit(1);
  }
  return keyword;
}

void print_look_up_t_table(look_up_t* table)
{
  char* keyword;

  Debug(1, ("\nPrinting the searched table:\n"));

  keyword = (char*)table->ptr;
  while(keyword != NULL){
    Debug(1, ("%s\t-->%d\n", keyword, table->val));
    table++;
    keyword = (char*)table->ptr;
  }
  Debug(1, ("End of table.\n\n"));
}

/*
//wp2fr16 = wanpipe2, 16, WANPIPE, Comment
int conf_file_writer::form_interface_str(string& interfaces_section_str,
                                         list_element_chan_def* list_el_chan_def)
{
  char tmp_buff[MAX_PATH_LENGTH];
//  link_def_t * link_def = cfr->link_defs;
//  wandev_conf_t *linkconf = cfr->link_defs->linkconf;

  Debug(DBG_CONF_FILE_WRITER, ("form_interface_str(): name: %s\n", list_el_chan_def->data.name));

  //this stays the same
  Debug(DBG_CONF_FILE_WRITER, ("cfr->link_defs->config_id: %d\n", cfr->link_defs->config_id));
  //this may change on channelized device (WANCONFIG_MFR...)
  Debug(DBG_CONF_FILE_WRITER, ("list_el_chan_def->data.chanconf->protocol: %d (WANCONFIG_MFR: %d)\n",
    list_el_chan_def->data.chanconf->protocol, WANCONFIG_MFR));

  if(cfr->link_defs->config_id == WANCONFIG_AFT){
    if(list_el_chan_def->data.chanconf->protocol == WANCONFIG_MFR){
      Debug(DBG_CONF_FILE_WRITER, ("WANCONFIG_MFR!!\n"));
      Debug(DBG_CONF_FILE_WRITER, ("list_el_chan_def->data.addr: %s\n", list_el_chan_def->data.addr));
    }
  }

  switch(cfr->link_defs->config_id)
  {
  case WANCONFIG_FR:
  case WANCONFIG_PPP:
  case WANCONFIG_CHDLC:
  case WANCONFIG_HDLC:

    snprintf(tmp_buff, MAX_PATH_LENGTH,
      "%s = %s, ",
      list_el_chan_def->data.name,
      cfr->link_defs->name);
    interfaces_section_str += tmp_buff;
    break;

  case WANCONFIG_AFT:
    snprintf(tmp_buff, MAX_PATH_LENGTH,
      "%s = %s, ",
      list_el_chan_def->data.name,
      cfr->link_defs->name);
    interfaces_section_str += tmp_buff;
    break;

  }//switch()

  switch(cfr->link_defs->config_id)
  {
  case WANCONFIG_FR:
    snprintf(tmp_buff, MAX_PATH_LENGTH,
      "%s, %s, %s\n",
      list_el_chan_def->data.addr,
      list_el_chan_def->data.usedby,
      (list_el_chan_def->data.label == NULL ? "Comment" : list_el_chan_def->data.label));
      break;

  case WANCONFIG_PPP:
    snprintf(tmp_buff, MAX_PATH_LENGTH,
      "%s, %s, %s\n",
      list_el_chan_def->data.addr,
      list_el_chan_def->data.usedby,
      (list_el_chan_def->data.label == NULL ? "Comment" : list_el_chan_def->data.label));
      break;

  case WANCONFIG_CHDLC:
  case WANCONFIG_HDLC:
    snprintf(tmp_buff, MAX_PATH_LENGTH,
      "%s, %s, %s\n",
      list_el_chan_def->data.addr,
      list_el_chan_def->data.usedby,
      (list_el_chan_def->data.label == NULL ? "Comment" : list_el_chan_def->data.label));
      break;

  case WANCONFIG_AFT:

    Debug(DBG_CONF_FILE_WRITER, ("form_interface_str(): protocol: %d\n",
      list_el_chan_def->data.chanconf->protocol));

    Debug(DBG_CONF_FILE_WRITER, ("form_interface_str(): aft_dlci_number: %s\n",
      list_el_chan_def->data.aft_dlci_number));

    if( list_el_chan_def->data.chanconf->protocol == WANCONFIG_MFR){

      conf_file_reader* tmp_cfr = (conf_file_reader*)list_el_chan_def->conf_file_reader;
      if(tmp_cfr != NULL){

        list_element_chan_def* local_list_el_chan_def =
          (list_element_chan_def*)tmp_cfr->obj_list->get_first();

        if(local_list_el_chan_def != NULL){

          snprintf(tmp_buff, MAX_PATH_LENGTH,
            "%s, %s, %s\n",
            local_list_el_chan_def->data.addr,
            list_el_chan_def->data.usedby,
            (list_el_chan_def->data.label == NULL ? "Comment" : list_el_chan_def->data.label));
        }else{
          ERR_DBG_OUT(("Uninitialized AFT MP_FR pointer!!\n"));
          return NO;
        }
      }else{
          ERR_DBG_OUT(("Uninitialized AFT MP_FR 'conf_file_reader' pointer!!\n"));
          return NO;
      }
    }else{
      snprintf(tmp_buff, MAX_PATH_LENGTH,
        ", %s, %s\n",
        list_el_chan_def->data.usedby,
        (list_el_chan_def->data.label == NULL ? "Comment" : list_el_chan_def->data.label));
    }
    break;

  default:
    ERR_DBG_OUT(("Unsupported 'config_id' (%d) passed for saving to file!!\n",
      cfr->link_defs->config_id));
    return NO;
  }

  interfaces_section_str += tmp_buff;

  return YES;
}
*/

//wp2fr16 = wanpipe2, 16, WANPIPE, Comment
int conf_file_writer::form_interface_str(string& interfaces_section_str,
                                         list_element_chan_def* list_el_chan_def)
{
  char tmp_buff[MAX_PATH_LENGTH];
//  link_def_t * link_def = cfr->link_defs;
//  wandev_conf_t *linkconf = cfr->link_defs->linkconf;

  Debug(DBG_CONF_FILE_WRITER, ("form_interface_str(): name: %s\n", list_el_chan_def->data.name));

  //this stays the same
  Debug(DBG_CONF_FILE_WRITER, ("cfr->link_defs->config_id: %d\n", cfr->link_defs->config_id));
  //this may change on channelized device (WANCONFIG_MFR...)
  Debug(DBG_CONF_FILE_WRITER, ("list_el_chan_def->data.chanconf->protocol: %d (WANCONFIG_MFR: %d)\n",
    list_el_chan_def->data.chanconf->protocol, WANCONFIG_MFR));

  if(cfr->link_defs->config_id == WANCONFIG_AFT){
    if(list_el_chan_def->data.chanconf->protocol == WANCONFIG_MFR){
      Debug(DBG_CONF_FILE_WRITER, ("WANCONFIG_MFR!!\n"));
      Debug(DBG_CONF_FILE_WRITER, ("list_el_chan_def->data.addr: %s\n", list_el_chan_def->data.addr));
    }
  }

  switch(cfr->link_defs->config_id)
  {
  case WANCONFIG_FR:
  case WANCONFIG_PPP:
  case WANCONFIG_CHDLC:
  case WANCONFIG_HDLC:

    snprintf(tmp_buff, MAX_PATH_LENGTH,
      "%s = %s, ",
      list_el_chan_def->data.name,
      cfr->link_defs->name);
    interfaces_section_str += tmp_buff;
    break;

  case WANCONFIG_AFT:
    snprintf(tmp_buff, MAX_PATH_LENGTH,
      "%s = %s, ",
      list_el_chan_def->data.name,
      cfr->link_defs->name);
    interfaces_section_str += tmp_buff;
    break;

  }//switch()

  switch(cfr->link_defs->config_id)
  {
  case WANCONFIG_FR:
    snprintf(tmp_buff, MAX_PATH_LENGTH,
      "%s, %s, %s\n",
      list_el_chan_def->data.addr,
      list_el_chan_def->data.usedby,
      (list_el_chan_def->data.label == NULL ? "Comment" : list_el_chan_def->data.label));
      break;

  case WANCONFIG_PPP:
    snprintf(tmp_buff, MAX_PATH_LENGTH,
      "%s, %s, %s\n",
      list_el_chan_def->data.addr,
      list_el_chan_def->data.usedby,
      (list_el_chan_def->data.label == NULL ? "Comment" : list_el_chan_def->data.label));
      break;

  case WANCONFIG_CHDLC:
  case WANCONFIG_HDLC:
    snprintf(tmp_buff, MAX_PATH_LENGTH,
      "%s, %s, %s\n",
      list_el_chan_def->data.addr,
      list_el_chan_def->data.usedby,
      (list_el_chan_def->data.label == NULL ? "Comment" : list_el_chan_def->data.label));
      break;

//?????
  case WANCONFIG_AFT:
    {
    Debug(DBG_CONF_FILE_WRITER, ("form_interface_str(): protocol: %d\n",
      list_el_chan_def->data.chanconf->protocol));

    Debug(DBG_CONF_FILE_WRITER, ("form_interface_str(): aft_dlci_number: %s\n",
      list_el_chan_def->data.aft_dlci_number));

    conf_file_reader* tmp_cfr = (conf_file_reader*)list_el_chan_def->conf_file_reader;
    if(tmp_cfr != NULL){

      list_element_chan_def* local_list_el_chan_def =
        (list_element_chan_def*)tmp_cfr->obj_list->get_first();


      Debug(DBG_CONF_FILE_WRITER, ("list_el_chan_def->data.usedby: %s\n",
        list_el_chan_def->data.usedby));
/*
      //must be regular (HDLC streaming) interface, because it has LIP layer above it.
      if(strcmp(list_el_chan_def->data.usedby, "STACK") != 0){
        ERR_DBG_OUT(("Invalid 'usedby' field!!\n"));
        return NO;
      }
*/
      snprintf(tmp_buff, MAX_PATH_LENGTH,
        " , %s, %s\n",
        list_el_chan_def->data.usedby,
        (list_el_chan_def->data.label == NULL ? "Comment" : list_el_chan_def->data.label));

      interfaces_section_str += tmp_buff;

      while(local_list_el_chan_def != NULL){

        if(tmp_cfr->link_defs->config_id == WANCONFIG_HDLC){
          local_list_el_chan_def =
            (list_element_chan_def*)tmp_cfr->obj_list->get_next_element(local_list_el_chan_def);
          continue;
        }

        //it is a STACK
        if(tmp_cfr->link_defs->config_id == WANCONFIG_MFR){
          snprintf(tmp_buff, MAX_PATH_LENGTH,
            "%s = %s, %s, %s, %s, %s.%s%s\n",
            local_list_el_chan_def->data.name,
            cfr->link_defs->name,
            ((atoi(local_list_el_chan_def->data.addr) == 0) ? "auto" : local_list_el_chan_def->data.addr),
            local_list_el_chan_def->data.usedby,
            get_aft_lip_layer_protocol(tmp_cfr->link_defs->config_id),
            get_aft_lip_layer_protocol(tmp_cfr->link_defs->config_id),
            get_aft_lip_layer_protocol(tmp_cfr->link_defs->config_id),
            list_el_chan_def->data.addr);
        }else{
          snprintf(tmp_buff, MAX_PATH_LENGTH,
            //"%s = %s, %s, %s, %s, %s.%s%s\n",
            "%s = %s, , %s, %s, %s.%s%s\n",
            local_list_el_chan_def->data.name,
            cfr->link_defs->name,
            //local_list_el_chan_def->data.addr,
            local_list_el_chan_def->data.usedby,
            get_aft_lip_layer_protocol(tmp_cfr->link_defs->config_id),
            get_aft_lip_layer_protocol(tmp_cfr->link_defs->config_id),
            get_aft_lip_layer_protocol(tmp_cfr->link_defs->config_id),
            list_el_chan_def->data.addr);
        }

        Debug(DBG_CONF_FILE_WRITER, ("local_list_el_chan_def->data.name: %s\n",
          local_list_el_chan_def->data.name));

        interfaces_section_str += tmp_buff;

        local_list_el_chan_def =
          (list_element_chan_def*)tmp_cfr->obj_list->get_next_element(local_list_el_chan_def);
      }//while()

      return YES;
    }else{
      snprintf(tmp_buff, MAX_PATH_LENGTH,
        ", %s, %s\n",
        list_el_chan_def->data.usedby,
        (list_el_chan_def->data.label == NULL ? "Comment" : list_el_chan_def->data.label));

      interfaces_section_str += tmp_buff;
      return YES;
    }
    }
    break;

  default:
    ERR_DBG_OUT(("Unsupported 'config_id' (%d) passed for saving to file!!\n",
      cfr->link_defs->config_id));
    return NO;
  }

  interfaces_section_str += tmp_buff;

  return YES;
}

int conf_file_writer::form_wanpipe_section_str(string& wanpipe_section_str)
{
  string tmp_string;

  tmp_string = "";
  if(form_wanpipe_card_location_str(tmp_string) == NO){
    return NO;
  }else{
    wanpipe_section_str += tmp_string;
  }

  tmp_string = "";
  switch(cfr->link_defs->linkconf->card_type)
  {
  case WANOPT_S51X:

    switch(cfr->link_defs->card_version)
    {
    case S5144_ADPTR_1_CPU_T1E1:

      if(form_te1_card_cfg_str(tmp_string) == NO){
        return NO;
      }else{
        wanpipe_section_str += tmp_string;
      }
      break;
    }
    break;

  case WANOPT_AFT:
    if(form_te1_card_cfg_str(tmp_string) == NO){
      return NO;
    }else{
      wanpipe_section_str += tmp_string;
    }
    break;

  }

  tmp_string = "";
  if(form_wanpipe_card_miscellaneous_options_str(tmp_string) == NO){
    return NO;
  }else{
    wanpipe_section_str += tmp_string;
  }

  tmp_string = "";
  if(form_global_protocol_configuration_string(tmp_string) == NO){
    return NO;
  }else{
    wanpipe_section_str += tmp_string;
  }

  return YES;
}

/*
CARD_TYPE 	= S51X
S514CPU 	= A
AUTO_PCISLOT	= NO
PCISLOT 	= 5
PCIBUS		= 0
*/
int conf_file_writer::form_wanpipe_card_location_str(string& wp_card_location_string)
{
  char tmp_buff[MAX_PATH_LENGTH];
  link_def_t * link_def;
  wandev_conf_t *linkconf;

  link_def = cfr->link_defs;
  linkconf = cfr->link_defs->linkconf;

  Debug(DBG_CONF_FILE_WRITER, ("form_wanpipe_card_location_str()\n"));

  snprintf(tmp_buff, MAX_PATH_LENGTH, "CARD_TYPE 	= %s\n",
    get_keyword_from_look_up_t_table( card_type_table, cfr->link_defs->linkconf->card_type));
  wp_card_location_string = tmp_buff;

  switch(cfr->link_defs->linkconf->card_type)
  {
  case WANOPT_S50X:

    snprintf(tmp_buff, MAX_PATH_LENGTH, "CommPort \t= %s\n",
      get_keyword_from_look_up_t_table( commport_type_options_table, linkconf->comm_port));
    wp_card_location_string += tmp_buff;

    snprintf(tmp_buff, MAX_PATH_LENGTH, "IOPORT\t\t= 0x%X\n", linkconf->ioport);
    wp_card_location_string += tmp_buff;

    snprintf(tmp_buff, MAX_PATH_LENGTH, "IRQ\t\t= %d\n", linkconf->irq);
    wp_card_location_string += tmp_buff;

    if(linkconf->maddr != 0){
      snprintf(tmp_buff, MAX_PATH_LENGTH, "MemAddr \t= 0x%X\n", linkconf->maddr);
      wp_card_location_string += tmp_buff;
    }else{
      //it is auto - do not write anything
    }
    break;

  case WANOPT_S51X:
  case WANOPT_AFT:
  case WANOPT_ADSL:

    snprintf(tmp_buff, MAX_PATH_LENGTH, "S514CPU \t= %s\n",
      cfr->link_defs->linkconf->S514_CPU_no);
    wp_card_location_string += tmp_buff;

/*
    snprintf(tmp_buff, MAX_PATH_LENGTH, "S514CPU \t= %c\n",
      cfr->link_defs->linkconf->S514_CPU_no[0]);
    wp_card_location_string += tmp_buff;
*/
    snprintf(tmp_buff, MAX_PATH_LENGTH, "AUTO_PCISLOT \t= %s\n",
      get_keyword_from_look_up_t_table( yes_no_options_table,
        cfr->link_defs->linkconf->auto_pci_cfg));
    wp_card_location_string += tmp_buff;

    snprintf(tmp_buff, MAX_PATH_LENGTH, "PCISLOT \t= %d\n",
      cfr->link_defs->linkconf->PCI_slot_no);
    wp_card_location_string += tmp_buff;

    snprintf(tmp_buff, MAX_PATH_LENGTH, "PCIBUS  \t= %d\n",
      cfr->link_defs->linkconf->pci_bus_no);
    wp_card_location_string += tmp_buff;

    snprintf(tmp_buff, MAX_PATH_LENGTH, "CommPort \t= %s\n",
      get_keyword_from_look_up_t_table( commport_type_options_table,
        cfr->link_defs->linkconf->comm_port));
    wp_card_location_string += tmp_buff;

    break;

  default:
    ERR_DBG_OUT(("Unsupported 'card_type' (%d) passed for saving to file!!\n",
      cfr->link_defs->linkconf->card_type));
    return NO;
  }

  return YES;
}

/*
//if t1/e1 card
MEDIA 		= T1
LCODE 		= B8ZS
FRAME 		= ESF
TE_CLOCK 	= NORMAL
ACTIVE_CH	= ALL
LBO 		= 0DB
*/
int conf_file_writer::form_te1_card_cfg_str(string& te1_cfg_string)
{
  char tmp_buff[MAX_PATH_LENGTH];

  snprintf(tmp_buff, MAX_PATH_LENGTH, "MEDIA 		= %s\n",
    get_keyword_from_look_up_t_table( physical_medium_options_table,
      cfr->link_defs->linkconf->te_cfg.media));
  te1_cfg_string = tmp_buff;

  snprintf(tmp_buff, MAX_PATH_LENGTH, "LCODE 		= %s\n",
    get_keyword_from_look_up_t_table( te1_line_code_options_table,
      cfr->link_defs->linkconf->te_cfg.lcode));
  te1_cfg_string += tmp_buff;

  snprintf(tmp_buff, MAX_PATH_LENGTH, "FRAME 		= %s\n",
    get_keyword_from_look_up_t_table( te1_frame_options_table,
      cfr->link_defs->linkconf->te_cfg.frame));
  te1_cfg_string += tmp_buff;

  snprintf(tmp_buff, MAX_PATH_LENGTH, "TE_CLOCK 	= %s\n",
    get_keyword_from_look_up_t_table( te1_clock_mode_options_table,
      cfr->link_defs->linkconf->te_cfg.te_clock));
  te1_cfg_string += tmp_buff;

  snprintf(tmp_buff, MAX_PATH_LENGTH, "ACTIVE_CH	= %s\n",
    cfr->link_defs->active_channels_string);
  te1_cfg_string += tmp_buff;

  if(cfr->link_defs->linkconf->te_cfg.media == WANOPT_MEDIA_T1){
    snprintf(tmp_buff, MAX_PATH_LENGTH, "LBO 		= %s\n",
      get_keyword_from_look_up_t_table( t1_line_build_out_options_table,
        cfr->link_defs->linkconf->te_cfg.lbo));
    te1_cfg_string += tmp_buff;
  }
  return YES;
}

/*
Firmware	= /etc/wanpipe/firmware/fr514.sfm
Interface 	= V35
Clocking 	= Internal
BaudRate 	= 1540000
MTU 		= 1500
UDPPORT 	= 9000
TTL 		= 255
IGNORE_FRONT_END  = NO
*/
int conf_file_writer::form_wanpipe_card_miscellaneous_options_str(string& misc_opt_string)
{
  char tmp_buff[MAX_PATH_LENGTH];

  snprintf(tmp_buff, MAX_PATH_LENGTH, "Firmware	= %s\n",
      cfr->link_defs->firmware_file_path);
  misc_opt_string = tmp_buff;

  misc_opt_string += form_keyword_and_value_str(  common_conftab,
                                                  offsetof(wandev_conf_t, interface),
                                                  serial_interface_type_options_table,
                                                  cfr->link_defs->linkconf->interface);

  misc_opt_string += form_keyword_and_value_str(  common_conftab,
                                                  offsetof(wandev_conf_t, clocking),
                                                  serial_clock_type_options_table,
                                                  cfr->link_defs->linkconf->clocking);

  snprintf(tmp_buff, MAX_PATH_LENGTH, "BaudRate 	= %d\n",
      cfr->link_defs->linkconf->bps);
  misc_opt_string += tmp_buff;

  snprintf(tmp_buff, MAX_PATH_LENGTH, "MTU 		= %d\n",
      cfr->link_defs->linkconf->mtu);
  misc_opt_string += tmp_buff;

  snprintf(tmp_buff, MAX_PATH_LENGTH, "UDPPORT 	= %d\n",
      cfr->link_defs->linkconf->udp_port);
  misc_opt_string += tmp_buff;

  snprintf(tmp_buff, MAX_PATH_LENGTH, "TTL\t	= %d\n",
      cfr->link_defs->linkconf->ttl);
  misc_opt_string += tmp_buff;

  snprintf(tmp_buff, MAX_PATH_LENGTH, "IGNORE_FRONT_END = %s\n",
    get_keyword_from_look_up_t_table( yes_no_options_table,
                                      cfr->link_defs->linkconf->ignore_front_end_status));
  misc_opt_string += tmp_buff;

  return YES;
}

char* conf_file_writer::form_keyword_and_value_str( key_word_t* conftab,
                                                    int offset_in_structure,
                                                    look_up_t* table,
                                                    int search_value)
{
  string tmp_str;

  tmp_str = "";

  tmp_str += get_keyword_from_key_word_t_table( conftab, offset_in_structure);
  tmp_str += "\t= ";

  tmp_str +=  get_keyword_from_look_up_t_table( table, search_value);
  tmp_str += "\n";

  return (char*)tmp_str.c_str();
}

int conf_file_writer::form_global_protocol_configuration_string(string& global_protocol_cfg)
{
  char tmp_buff[MAX_PATH_LENGTH];
  wan_fr_conf_t* fr_cfg = &cfr->link_defs->linkconf->u.fr;
  wan_ppp_conf_t* ppp_cfg = &cfr->link_defs->linkconf->u.ppp;
  wan_chdlc_conf_t* chdlc_cfg = &cfr->link_defs->linkconf->u.chdlc;
  wan_chdlc_conf_t* hdlc_strm_cfg = &cfr->link_defs->linkconf->u.chdlc;
  wan_xilinx_conf_t* xilinx_conf = &cfr->link_defs->linkconf->u.xilinx;

  Debug(DBG_CONF_FILE_WRITER, ("form_global_protocol_configuration_string()\n"));

  switch(cfr->link_defs->config_id)
  {
  case WANCONFIG_FR:
    if(form_frame_relay_global_configuration_string(fr_cfg, global_protocol_cfg) == NO){
      return NO;
    }
    break;

  case WANCONFIG_PPP:
    if(form_ppp_global_configuration_string(ppp_cfg, global_protocol_cfg) == NO){
      return NO;
    }
    break;

  case WANCONFIG_CHDLC:
  case WANCONFIG_HDLC:

    if(form_chdlc_global_configuration_string(chdlc_cfg, global_protocol_cfg) == NO){
      return NO;
    }
    break;

  case WANCONFIG_AFT:
    if(form_aft_global_configuration_string(xilinx_conf, global_protocol_cfg) == NO){
      return NO;
    }
    break;

  default:
    ERR_DBG_OUT(("Unsupported 'config_id' (%d) passed for saving to file!!\n",
      cfr->link_defs->config_id));
    return NO;
  }

  return YES;
}

/*
//For Frame Relay - global cfg
NUMBER_OF_DLCI 	= 2
Station 	= CPE
Signalling 	= ANSI
T391 		= 10
T392 		= 16
N391 		= 6
N392 		= 3
N393 		= 4
FR_ISSUE_FS	= YES
*/
int conf_file_writer::form_frame_relay_global_configuration_string( wan_fr_conf_t* fr_cfg,
                                                                    string& fr_global_cfg)
{
  char* keyword;
  char tmp_buff[MAX_PATH_LENGTH];

  Debug(DBG_CONF_FILE_WRITER, ("form_frame_relay_global_configuration_string()\n"));

  fr_global_cfg = "";

  ///////////////////////////////////////////////////////////////////////////////////////
  keyword = get_keyword_from_key_word_t_table(fr_conftab,
                                              offsetof(wan_fr_conf_t, dlci_num));
  if(keyword == NULL){
    ERR_DBG_OUT(("failed to find keyword for 'dlci_num'!\n"));
    return NO;
  }
  Debug(DBG_CONF_FILE_WRITER, ("keyword for 'dlci_num': %s\n", keyword));
  fr_global_cfg += keyword;
  fr_global_cfg += "\t= ";

  snprintf(tmp_buff, MAX_PATH_LENGTH, "%d\n", fr_cfg->dlci_num); //cfr->obj_list->get_size());
  fr_global_cfg += tmp_buff;

  ///////////////////////////////////////////////////////////////////////////////////////
  keyword = get_keyword_from_look_up_t_table( frame_relay_station_type_options_table,
    fr_cfg->station);
//    cfr->link_defs->linkconf->u.fr.station);

  if(keyword == NULL){
    ERR_DBG_OUT(("failed to find value keyword for 'station' (0x%x)!\n",
      cfr->link_defs->linkconf->u.fr.station));
    return NO;
  }
  snprintf(tmp_buff, MAX_PATH_LENGTH, "Station\t\t= %s\n", keyword);
  fr_global_cfg += tmp_buff;

  ///////////////////////////////////////////////////////////////////////////////////////
  keyword = get_keyword_from_key_word_t_table(fr_conftab,
                                              offsetof(wan_fr_conf_t, signalling));
  if(keyword == NULL){
    ERR_DBG_OUT(("failed to find keyword for 'signalling'!\n"));
    return NO;
  }
  Debug(DBG_CONF_FILE_WRITER, ("keyword for 'signalling': %s\n", keyword));

  fr_global_cfg += keyword;
  fr_global_cfg += "\t= ";

  keyword = get_keyword_from_look_up_t_table( frame_relay_in_channel_signalling_options_table,
    fr_cfg->signalling);
  if(keyword == NULL){
    ERR_DBG_OUT(("failed to find value keyword for 'fr_cfg->signalling' (0x%x)!\n",
      fr_cfg->signalling));
    return NO;
  }
  fr_global_cfg += keyword;
  fr_global_cfg += "\n";
  ///////////////////////////////////////////////////////////////////////////////////////
  keyword = get_keyword_from_key_word_t_table(fr_conftab,
                                              offsetof(wan_fr_conf_t, t391));
  if(keyword == NULL){
    ERR_DBG_OUT(("failed to find keyword for 'T391'!\n"));
    return NO;
  }
  Debug(DBG_CONF_FILE_WRITER, ("keyword for 'T391': %s\n", keyword));

  fr_global_cfg += keyword;
  fr_global_cfg += "\t\t= ";

  snprintf(tmp_buff, MAX_PATH_LENGTH, "%d\n", fr_cfg->t391);
  fr_global_cfg += tmp_buff;
  ///////////////////////////////////////////////////////////////////////////////////////
  keyword = get_keyword_from_key_word_t_table(fr_conftab,
                                              offsetof(wan_fr_conf_t, t392));
  if(keyword == NULL){
    ERR_DBG_OUT(("failed to find keyword for 'T392'!\n"));
    return NO;
  }
  Debug(DBG_CONF_FILE_WRITER, ("keyword for 'T392': %s\n", keyword));

  fr_global_cfg += keyword;
  fr_global_cfg += "\t\t= ";

  snprintf(tmp_buff, MAX_PATH_LENGTH, "%d\n", fr_cfg->t392);
  fr_global_cfg += tmp_buff;
  ///////////////////////////////////////////////////////////////////////////////////////
  keyword = get_keyword_from_key_word_t_table(fr_conftab,
                                              offsetof(wan_fr_conf_t, n391));
  if(keyword == NULL){
    ERR_DBG_OUT(("failed to find keyword for 'N391'!\n"));
    return NO;
  }
  Debug(DBG_CONF_FILE_WRITER, ("keyword for 'N391': %s\n", keyword));

  fr_global_cfg += keyword;
  fr_global_cfg += "\t\t= ";

  snprintf(tmp_buff, MAX_PATH_LENGTH, "%d\n", fr_cfg->n391);
  fr_global_cfg += tmp_buff;
  ///////////////////////////////////////////////////////////////////////////////////////
  keyword = get_keyword_from_key_word_t_table(fr_conftab,
                                              offsetof(wan_fr_conf_t, n392));
  if(keyword == NULL){
    ERR_DBG_OUT(("failed to find keyword for 'N392'!\n"));
    return NO;
  }
  Debug(DBG_CONF_FILE_WRITER, ("keyword for 'N392': %s\n", keyword));

  fr_global_cfg += keyword;
  fr_global_cfg += "\t\t= ";

  snprintf(tmp_buff, MAX_PATH_LENGTH, "%d\n", fr_cfg->n392);
  fr_global_cfg += tmp_buff;
  ///////////////////////////////////////////////////////////////////////////////////////
  keyword = get_keyword_from_key_word_t_table(fr_conftab,
                                              offsetof(wan_fr_conf_t, n393));
  if(keyword == NULL){
    ERR_DBG_OUT(("failed to find keyword for 'N393'!\n"));
    return NO;
  }
  Debug(DBG_CONF_FILE_WRITER, ("keyword for 'N393': %s\n", keyword));

  fr_global_cfg += keyword;
  fr_global_cfg += "\t\t= ";

  snprintf(tmp_buff, MAX_PATH_LENGTH, "%d\n", fr_cfg->n393);
  fr_global_cfg += tmp_buff;
  ///////////////////////////////////////////////////////////////////////////////////////
  keyword = get_keyword_from_key_word_t_table(fr_conftab,
                                              offsetof(wan_fr_conf_t, issue_fs_on_startup));
  if(keyword == NULL){
    ERR_DBG_OUT(("failed to find keyword for 'issue_fs_on_startup'!\n"));
    return NO;
  }
  Debug(DBG_CONF_FILE_WRITER, ("keyword for 'issue_fs_on_startup': %s\n", keyword));

  fr_global_cfg += keyword;
  fr_global_cfg += "\t= ";

  keyword = get_keyword_from_look_up_t_table( yes_no_options_table,
                                              fr_cfg->issue_fs_on_startup);
  if(keyword == NULL){
    ERR_DBG_OUT(("failed to find value keyword for 'fr_cfg->issue_fs_on_startup' (0x%x)!\n",
      fr_cfg->issue_fs_on_startup));
    return NO;
  }
  fr_global_cfg += keyword;
  fr_global_cfg += "\n";
  ///////////////////////////////////////////////////////////////////////////////////////
  return YES;
}

int conf_file_writer::form_ppp_global_configuration_string(wan_ppp_conf_t* ppp_cfg,
                                                           string& global_protocol_cfg)
{
  global_protocol_cfg = "";

  global_protocol_cfg += form_keyword_and_value_str(  ppp_conftab,
                                                      offsetof(wan_ppp_conf_t, ip_mode),
                                                      ppp_ip_mode_options_table,
                                                      ppp_cfg->ip_mode);
  return YES;
}

/*
Receive_Only	= NO
Connection	= Permanent
LineCoding	= NRZ
LineIdle	= Flag
*/
int conf_file_writer::form_chdlc_global_configuration_string( wan_chdlc_conf_t* chdlc_cfg,
                                                              string& global_protocol_cfg)
{
  global_protocol_cfg = "";

  global_protocol_cfg += form_keyword_and_value_str(  common_conftab,
                                                      offsetof(wandev_conf_t, receive_only),
                                                      yes_no_options_table,
                                                      cfr->link_defs->linkconf->receive_only);

  global_protocol_cfg += form_keyword_and_value_str(  common_conftab,
                                                      offsetof(wandev_conf_t, connection),
                                                      connection_type_options_table,
                                                      cfr->link_defs->linkconf->connection);

  global_protocol_cfg += form_keyword_and_value_str(  common_conftab,
                                                      offsetof(wandev_conf_t, line_coding),
                                                      data_encoding_options_table,
                                                      cfr->link_defs->linkconf->line_coding);

  global_protocol_cfg += form_keyword_and_value_str(  common_conftab,
                                                      offsetof(wandev_conf_t, line_idle),
                                                      line_idle_options_table,
                                                      cfr->link_defs->linkconf->line_idle);

  return YES;
}

int conf_file_writer::form_aft_global_configuration_string( wan_xilinx_conf_t* xilinx_conf,
                                                            string& global_protocol_cfg)
{
  //nothing has to be done.

  global_protocol_cfg = "";

  return YES;
}

int conf_file_writer::form_per_interface_str( string& wp_interface,
                                              list_element_chan_def* list_el_chan_def,
                                              int lip_layer_flag)
{
  char tmp_buff[MAX_PATH_LENGTH];
  string tmp_str;
  int protocol;

  Debug(DBG_CONF_FILE_WRITER, ("form_per_interface_str()\n"));

  snprintf(tmp_buff, MAX_PATH_LENGTH, "\n[%s]\n", list_el_chan_def->data.name);
  wp_interface += tmp_buff;

  Debug(DBG_CONF_FILE_WRITER, ("list_el_chan_def->data.chanconf->protocol: %d\n",
    list_el_chan_def->data.chanconf->protocol));
  Debug(DBG_CONF_FILE_WRITER, ("cfr->link_defs->config_id: %d\n",
    cfr->link_defs->config_id));

  if(lip_layer_flag == YES){
    protocol = list_el_chan_def->data.chanconf->protocol;
  }else{
    protocol = cfr->link_defs->config_id;
  }

  Debug(DBG_CONF_FILE_WRITER, ("protocol: %d\n", protocol));

  switch(protocol)
  {
  case WANCONFIG_FR:
  case WANCONFIG_MFR:
    if(form_frame_relay_per_interface_str(tmp_str, list_el_chan_def) == NO){
      return NO;
    }
    break;

  case WANCONFIG_PPP:
  case WANCONFIG_MPPP:
    if(form_ppp_per_interface_str(tmp_str, list_el_chan_def) == NO){
      return NO;
    }
    break;

  case WANCONFIG_CHDLC:
  case WANCONFIG_MPCHDLC:
    if(form_chdlc_per_interface_str(tmp_str, list_el_chan_def) == NO){
      return NO;
    }
    break;

  case WANCONFIG_HDLC:
    //the same as CHDLC, only 'HDLC_STREAMING		= YES'
    if(form_chdlc_per_interface_str(tmp_str, list_el_chan_def) == NO){
      return NO;
    }
    break;

  case WANCONFIG_AFT:
    if(form_aft_per_interface_str(tmp_str, list_el_chan_def) == NO){
      return NO;
    }
    break;

  default:
    ERR_DBG_OUT(("Unsupported 'config_id' (%d) passed for saving to file!!\n",
      cfr->link_defs->config_id));
    return NO;
  }

  wp_interface += tmp_str;

  form_common_per_interface_str( wp_interface, list_el_chan_def);

  return YES;
}

/*
[wp2fr16]
CIR 		= 64
BC  		= 64
BE  		= 0
INARP 		= YES
INARPINTERVAL 	= 10
INARP_RX	= YES
*/
int conf_file_writer::form_frame_relay_per_interface_str( string& wp_interface,
                                                          list_element_chan_def* list_el_chan_def)
{
  char tmp_buff[MAX_PATH_LENGTH];

  ///////////////////////////////////////////////////////////////////////////////////////
  wp_interface += get_keyword_from_key_word_t_table(chan_conftab,
                                                    offsetof(wanif_conf_t, cir));
  wp_interface += "\t= ";
  snprintf(tmp_buff, MAX_PATH_LENGTH, "%d\n", list_el_chan_def->data.chanconf->cir);
  wp_interface += tmp_buff;

  ///////////////////////////////////////////////////////////////////////////////////////
  wp_interface += get_keyword_from_key_word_t_table(chan_conftab,
                                                    offsetof(wanif_conf_t, bc));
  wp_interface += "\t= ";
  snprintf(tmp_buff, MAX_PATH_LENGTH, "%d\n", list_el_chan_def->data.chanconf->bc);
  wp_interface += tmp_buff;

  ///////////////////////////////////////////////////////////////////////////////////////
  wp_interface += get_keyword_from_key_word_t_table(chan_conftab,
                                                    offsetof(wanif_conf_t, be));
  wp_interface += "\t= ";
  snprintf(tmp_buff, MAX_PATH_LENGTH, "%d\n", list_el_chan_def->data.chanconf->be);
  wp_interface += tmp_buff;

  ///////////////////////////////////////////////////////////////////////////////////////
  wp_interface += form_keyword_and_value_str( chan_conftab,
                                              offsetof(wanif_conf_t, inarp),
                                              yes_no_options_table,
                                              list_el_chan_def->data.chanconf->inarp);

  ///////////////////////////////////////////////////////////////////////////////////////
  wp_interface += get_keyword_from_key_word_t_table(chan_conftab,
                                                    offsetof(wanif_conf_t, inarp_interval));
  wp_interface += "\t= ";
  snprintf(tmp_buff, MAX_PATH_LENGTH, "%d\n", list_el_chan_def->data.chanconf->inarp_interval);
  wp_interface += tmp_buff;

  ///////////////////////////////////////////////////////////////////////////////////////
  wp_interface += form_keyword_and_value_str( chan_conftab,
                                              offsetof(wanif_conf_t, inarp_rx),
                                              yes_no_options_table,
                                              list_el_chan_def->data.chanconf->inarp_rx);
  return YES;
}

/*
MULTICAST 	= NO
IPX		= YES
NETWORK		= 0xABCDEFAB
TRUE_ENCODING_TYPE	= NO
DYN_INTR_CFG	= NO
*/
int conf_file_writer::form_common_per_interface_str( string& wp_interface,
                                                     list_element_chan_def* list_el_chan_def)
{
  char tmp_buff[MAX_PATH_LENGTH];
  wandev_conf_t *linkconf = cfr->link_defs->linkconf;

  ///////////////////////////////////////////////////////////////////////////////////////
  //multicast
  wp_interface += form_keyword_and_value_str( chan_conftab,
                                              offsetof(wanif_conf_t, mc),
                                              yes_no_options_table,
                                              list_el_chan_def->data.chanconf->mc);

  ///////////////////////////////////////////////////////////////////////////////////////
  //ipx
  if( linkconf->config_id == WANCONFIG_FR ||
      linkconf->config_id == WANCONFIG_PPP){

    wp_interface += form_keyword_and_value_str( chan_conftab,
                                                offsetof(wanif_conf_t, enable_IPX),
                                                yes_no_options_table,
                                                list_el_chan_def->data.chanconf->enable_IPX);

    ///////////////////////////////////////////////////////////////////////////////////////
    if(list_el_chan_def->data.chanconf->enable_IPX == WANOPT_YES){
      wp_interface += get_keyword_from_key_word_t_table(chan_conftab,
                                                        offsetof(wanif_conf_t, network_number));
      wp_interface += "\t= ";
      snprintf(tmp_buff, MAX_PATH_LENGTH, "0x%08X\n", list_el_chan_def->data.chanconf->network_number);
      wp_interface += tmp_buff;
    }
  }

  ///////////////////////////////////////////////////////////////////////////////////////
  wp_interface += form_keyword_and_value_str( chan_conftab,
                                              offsetof(wanif_conf_t, true_if_encoding),
                                              yes_no_options_table,
                                              list_el_chan_def->data.chanconf->true_if_encoding);

  ///////////////////////////////////////////////////////////////////////////////////////
  switch(get_used_by_integer_value(list_el_chan_def->data.chanconf->usedby))
  {
  case WANPIPE:
  case BRIDGE_NODE:
    wp_interface += form_keyword_and_value_str( chan_conftab,
                                                offsetof(wanif_conf_t, if_down),
                                                yes_no_options_table,
                                                list_el_chan_def->data.chanconf->if_down);
    break;
  }
  return YES;
}

/*
PAP       	= YES
CHAP		= NO
USERID 	= my_userid
PASSWD   	= my_password
SYSNAME  	= my_system_name
*/
int conf_file_writer::form_ppp_per_interface_str( string& wp_interface,
                                                  list_element_chan_def* list_el_chan_def)
{
  char authenticate = NO;

  if(list_el_chan_def->data.chanconf->pap == WANOPT_YES){

    authenticate = YES;
  }
  if(list_el_chan_def->data.chanconf->chap == WANOPT_YES){
    authenticate = YES;
  }

  wp_interface += form_keyword_and_value_str( chan_conftab,
                                              offsetof(wanif_conf_t, pap),
                                              yes_no_options_table,
                                              list_el_chan_def->data.chanconf->pap);

  wp_interface += form_keyword_and_value_str( chan_conftab,
                                              offsetof(wanif_conf_t, chap),
                                              yes_no_options_table,
                                              list_el_chan_def->data.chanconf->chap);


  if(authenticate == YES){

    wp_interface += get_keyword_from_key_word_t_table(chan_conftab,
                                                      offsetof(wanif_conf_t, userid));
    wp_interface += "\t= ";
    wp_interface += (char*)list_el_chan_def->data.chanconf->userid;
    wp_interface += "\n";

    wp_interface += get_keyword_from_key_word_t_table(chan_conftab,
                                                      offsetof(wanif_conf_t, passwd));
    wp_interface += "\t= ";
    wp_interface += (char*)list_el_chan_def->data.chanconf->passwd;
    wp_interface += "\n";

    wp_interface += get_keyword_from_key_word_t_table(chan_conftab,
                                                      offsetof(wanif_conf_t, sysname));
    wp_interface += "\t= ";
    wp_interface += (char*)list_el_chan_def->data.chanconf->sysname;
    wp_interface += "\n";
  }

  return YES;
}

/*
IGNORE_DCD		= NO
IGNORE_CTS		= NO
IGNORE_KEEPALIVE	= NO
HDLC_STREAMING		= NO
KEEPALIVE_TX_TIMER	= 10000
KEEPALIVE_RX_TIMER	= 11000
KEEPALIVE_ERR_MARGIN	= 5
SLARP_TIMER		= 0
===========================
IGNORE_DCD		= NO
IGNORE_CTS		= NO
IGNORE_KEEPALIVE	= NO
HDLC_STREAMING		= NO
KEEPALIVE_TX_TIMER	= 10000
KEEPALIVE_RX_TIMER	= 11000
KEEPALIVE_ERR_MARGIN	= 5
SLARP_TIMER		= 1000
*/
int conf_file_writer::form_chdlc_per_interface_str(string& wp_interface,
                                                  list_element_chan_def* list_el_chan_def)
{
  char tmp_buff[MAX_PATH_LENGTH];

  wp_interface += form_keyword_and_value_str( chan_conftab,
                                              offsetof(wanif_conf_t, ignore_dcd),
                                              yes_no_options_table,
                                              list_el_chan_def->data.chanconf->ignore_dcd);

  wp_interface += form_keyword_and_value_str( chan_conftab,
                                              offsetof(wanif_conf_t, ignore_cts),
                                              yes_no_options_table,
                                              list_el_chan_def->data.chanconf->ignore_cts);

  wp_interface += form_keyword_and_value_str( chan_conftab,
                                              offsetof(wanif_conf_t, ignore_keepalive),
                                              yes_no_options_table,
                                              list_el_chan_def->data.chanconf->ignore_keepalive);

  ////////////////////////////////////////////////////////////////////////////////////////////////
  //must be always NO
  wp_interface += form_keyword_and_value_str( chan_conftab,
                                              offsetof(wanif_conf_t, hdlc_streaming),
                                              yes_no_options_table,
                                              list_el_chan_def->data.chanconf->hdlc_streaming);

  ////////////////////////////////////////////////////////////////////////////////////////////////
  wp_interface += get_keyword_from_key_word_t_table(chan_conftab,
                                                    offsetof(wanif_conf_t, keepalive_tx_tmr));
  wp_interface += "\t= ";
  snprintf(tmp_buff, MAX_PATH_LENGTH, "%d\n", list_el_chan_def->data.chanconf->keepalive_tx_tmr);
  wp_interface += tmp_buff;

  ////////////////////////////////////////////////////////////////////////////////////////////////
  wp_interface += get_keyword_from_key_word_t_table(chan_conftab,
                                                    offsetof(wanif_conf_t, keepalive_rx_tmr));
  wp_interface += "\t= ";
  snprintf(tmp_buff, MAX_PATH_LENGTH, "%d\n", list_el_chan_def->data.chanconf->keepalive_rx_tmr);
  wp_interface += tmp_buff;

  ////////////////////////////////////////////////////////////////////////////////////////////////
  wp_interface += get_keyword_from_key_word_t_table(chan_conftab,
                                                    offsetof(wanif_conf_t, keepalive_err_margin));
  wp_interface += "\t= ";
  snprintf(tmp_buff, MAX_PATH_LENGTH, "%d\n", list_el_chan_def->data.chanconf->keepalive_err_margin);
  wp_interface += tmp_buff;

  ////////////////////////////////////////////////////////////////////////////////////////////////
  wp_interface += get_keyword_from_key_word_t_table(chan_conftab,
                                                    offsetof(wanif_conf_t, slarp_timer));
  wp_interface += "\t= ";
  snprintf(tmp_buff, MAX_PATH_LENGTH, "%d\n", list_el_chan_def->data.chanconf->slarp_timer);
  wp_interface += tmp_buff;

  return YES;
}

/*
ACTIVE_CH	= 1-4
PROTOCOL	= MP_FR
IGNORE_DCD	= NO
IGNORE_CTS	= NO
HDLC_STREAMING	= YES
DYN_INTR_CFG	= NO
*/
int conf_file_writer::form_aft_per_interface_str(string& wp_interface,
                                                 list_element_chan_def* list_el_chan_def)
{
  conf_file_reader* tmp_cfr = (conf_file_reader*)list_el_chan_def->conf_file_reader;

  wp_interface += get_keyword_from_key_word_t_table(chan_conftab,
                                                    offsetof(wanif_conf_t, active_ch));
  wp_interface += "\t= ";
  wp_interface += list_el_chan_def->data.active_channels_string;
  wp_interface += "\n";


  wp_interface += "PROTOCOL",
  wp_interface += "\t= ";
  wp_interface += get_keyword_from_look_up_t_table( protocol_options_table,
                                                    list_el_chan_def->data.chanconf->protocol);
  wp_interface += "\n";

/*
  Debug(DBG_CONF_FILE_WRITER, ("list_el_chan_def->data.chanconf->protocol: %d\n",
    list_el_chan_def->data.chanconf->protocol));

  if( list_el_chan_def->data.chanconf->protocol == WANCONFIG_MFR &&
      tmp_cfr != NULL){

    wan_fr_conf_t* fr_cfg = &tmp_cfr->link_defs->linkconf->u.fr;

    Debug(DBG_CONF_FILE_WRITER, ("fr_cfg->station: %d\n", fr_cfg->station));
    Debug(DBG_CONF_FILE_WRITER, ("fr_cfg->signalling: %d\n", fr_cfg->signalling));

    ///////////////////////////////////////////////////////////////////////////////////////
    wp_interface += "STATION\t\t= ";
    wp_interface += get_keyword_from_look_up_t_table( frame_relay_station_type_options_table,
                                                      fr_cfg->station);
    wp_interface += "\n";
    ///////////////////////////////////////////////////////////////////////////////////////
    wp_interface += "SIGNALLING\t= ";
    wp_interface += get_keyword_from_look_up_t_table( frame_relay_in_channel_signalling_options_table,
                                                      fr_cfg->signalling);
    wp_interface += "\n";
    ///////////////////////////////////////////////////////////////////////////////////////
  }
*/
  wp_interface += form_keyword_and_value_str( chan_conftab,
                                              offsetof(wanif_conf_t, hdlc_streaming),
                                              yes_no_options_table,
                                              list_el_chan_def->data.chanconf->hdlc_streaming);

  return YES;
}

char* conf_file_writer::get_aft_lip_layer_protocol(int protocol)
{
  Debug(DBG_CONF_FILE_WRITER, ("get_aft_lip_layer_protocol(): protocol: %d\n",
    protocol));

  switch(protocol)
  {
  case WANCONFIG_MFR:
    return "fr";
    break;

  case WANCONFIG_MPPP:
    return "ppp";
    break;

  case WANCONFIG_MPCHDLC:
    return "chdlc";
    break;

  default:
    return "unknown";
  }
}

int conf_file_writer::form_profile_str(string& profile_str,
                                       list_element_chan_def* list_el_chan_def)
{
  char tmp_buff[MAX_PATH_LENGTH];
  string global_protocol_cfg;
  conf_file_reader* tmp_cfr;
  wan_fr_conf_t* fr_cfg;
  wan_ppp_conf_t* ppp_cfg;
  wan_chdlc_conf_t* chdlc_cfg;

  Debug(DBG_CONF_FILE_WRITER, ("form_profile_str(): name: %s\n",
    list_el_chan_def->data.name));

  Debug(DBG_CONF_FILE_WRITER,
    ("list_el_chan_def: %p\n", list_el_chan_def));

  Debug(DBG_CONF_FILE_WRITER, ("form_interface_str(): protocol: %d\n",
    list_el_chan_def->data.chanconf->protocol));

  Debug(DBG_CONF_FILE_WRITER, ("form_interface_str(): aft_dlci_number: %s\n",
    list_el_chan_def->data.aft_dlci_number));

  tmp_cfr = (conf_file_reader*)list_el_chan_def->conf_file_reader;
  if(tmp_cfr != NULL){

    fr_cfg = &tmp_cfr->link_defs->linkconf->u.fr;
    ppp_cfg = &tmp_cfr->link_defs->linkconf->u.ppp;
    chdlc_cfg = &tmp_cfr->link_defs->linkconf->u.chdlc;


    list_element_chan_def* local_list_el_chan_def =
      (list_element_chan_def*)tmp_cfr->obj_list->get_first();

    //must be regular (HDLC streaming) interface, because it has LIP layer above it.
    if(strcmp(list_el_chan_def->data.usedby, "STACK") != 0){
      Debug(DBG_CONF_FILE_WRITER, ("1. Not a stack - no profile needed\n"));
      return YES;
    }

    //it is a STACK
    snprintf(tmp_buff, MAX_PATH_LENGTH, "[%s.%s%s]\n",
      get_aft_lip_layer_protocol(tmp_cfr->link_defs->config_id),
      get_aft_lip_layer_protocol(tmp_cfr->link_defs->config_id),
      list_el_chan_def->data.addr);

    profile_str = tmp_buff;

    switch(tmp_cfr->link_defs->config_id)
    {
    case WANCONFIG_MFR:

      if(form_frame_relay_global_configuration_string( fr_cfg, global_protocol_cfg) == NO){
        return NO;
      }
      break;

    case WANCONFIG_MPPP:
      if(form_ppp_global_configuration_string(ppp_cfg, global_protocol_cfg) == NO){
        return NO;
      }
      break;

    case WANCONFIG_MPCHDLC:
      if(form_chdlc_global_configuration_string(chdlc_cfg, global_protocol_cfg) == NO){
        return NO;
      }
      break;

    default:
      profile_str += "unsupprted profile!";
      return NO;
    }

    profile_str += global_protocol_cfg;

    while(local_list_el_chan_def != NULL){

      local_list_el_chan_def =
        (list_element_chan_def*)tmp_cfr->obj_list->get_next_element(local_list_el_chan_def);
    }//while()

    Debug(DBG_CONF_FILE_WRITER, ("profile_str: %s\n", profile_str.c_str()));

    return YES;
  }else{

    Debug(DBG_CONF_FILE_WRITER, ("2. Not a stack - no profile needed\n"));
    return YES;
  }
}

/*
Interface name on return: w1grp1
menu_net_interface_setup: chan_def: 0x80a8858


link_def->config_id: 117
name: w1g1
form_profile_str(): name: w1g1
list_el_chan_def: 0x80a2580
form_interface_str(): protocol: 106
form_interface_str(): aft_dlci_number: Not Set
1. Not a stack - no profile needed
form_lip_layer_str(): name: w1g1
list_el_chan_def: 0x80a2580
form_lip_layer_str(): protocol: 106
1. Not a stack - no profile needed
name: w1g1
*/

int conf_file_writer::form_lip_layer_str( string& profile_str,
                                          list_element_chan_def* list_el_chan_def)
{
  string lip_interface_str;
  conf_file_reader* tmp_cfr;

  Debug(DBG_CONF_FILE_WRITER, ("form_lip_layer_str(): name: %s\n",
    list_el_chan_def->data.name));

  Debug(DBG_CONF_FILE_WRITER,
    ("list_el_chan_def: %p\n", list_el_chan_def));

  Debug(DBG_CONF_FILE_WRITER, ("form_lip_layer_str(): protocol: %d\n",
    list_el_chan_def->data.chanconf->protocol));

  tmp_cfr = (conf_file_reader*)list_el_chan_def->conf_file_reader;
  if(tmp_cfr != NULL){

    list_element_chan_def* local_list_el_chan_def =
      (list_element_chan_def*)tmp_cfr->obj_list->get_first();

    //must be regular (HDLC streaming) interface, because it has LIP layer above it.
    if(strcmp(list_el_chan_def->data.usedby, "STACK") != 0){
      Debug(DBG_CONF_FILE_WRITER, ("1. Not a stack - no profile needed\n"));
      return YES;
    }

    while(local_list_el_chan_def != NULL){

      lip_interface_str = " ";

      local_list_el_chan_def->data.chanconf->protocol = tmp_cfr->link_defs->config_id;

      if(form_per_interface_str(lip_interface_str,
                                local_list_el_chan_def,
                                YES) == NO){
        return NO;
      }else{
        profile_str += lip_interface_str;
      }

      local_list_el_chan_def =
        (list_element_chan_def*)tmp_cfr->obj_list->get_next_element(local_list_el_chan_def);
    }//while()

    Debug(DBG_CONF_FILE_WRITER,
      ("profile_str: %s\n", profile_str.c_str()));

    return YES;
  }else{

    Debug(DBG_CONF_FILE_WRITER, ("2. Not a stack - no profile needed\n"));
    return YES;
  }
}
