/***************************************************************************
                          menu_aft_logical_channel_cfg.cpp  -  description
                             -------------------
    begin                : Thu Apr 8 2004
    copyright            : (C) 2004 by David Rokhvarg
    email                : davidr@sangoma.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "menu_aft_logical_channel_cfg.h"
#include "text_box.h"
#include "text_box_yes_no.h"

#include "menu_aft_select_protocol.h"
#include "input_box_active_channels.h"

#include "menu_select_and_configure_protocol.h"
#include "menu_frame_relay_basic_cfg.h"
#include "menu_ppp_basic_cfg.h"
#include "menu_chdlc_basic_cfg.h"

#include "menu_list_existing_wanpipes.h"

#include "menu_net_interfaces_list.h"

char* bound_te1_channels_help_str =
"\n"
"BOUND T1/E1 ACTIVE CHANNELS\n"
"\n"
"The Channels Group can be bound into any single\n"
"or conbination of T1/E1 Active channels.\n"
"\n"
"Using socket interface API user can send and\n"
"receive data on specified T1/E1 channels.\n"
"\n"
"Please specify T1/E1 channles to bind to a\n"
"Channels Group.\n"
"\n"
"You can use the follow format:\n"
"o ALL     - for full T1/E1 links\n"
"o x.a-b.w - where the letters are numbers; to specify\n"
"            the channels x and w and the range of\n"
"            channels a-b, inclusive for fractional\n"
"            links.\n"
" Example:\n"
"o ALL     - All channels will be selcted\n"
"o 1       - A first channel will be selected\n"
"o 1.3-8.12 - Channels 1,3,4,5,6,7,8,12 will be selected.\n"
"\n"
"If more than one Channels Group is configured, make sure\n"
"that \"Bound Channels\" do not overlap between different\n"
"Channel Groups\n";

char* aft_logical_channel_protocol_select_help_str =
"Protocol Selection\n"
"------------------\n"
"\n"
"Please select communications protocol.\n";



enum AFT_LOGICAL_CHANNEL_CONFIGURATION{

  CURRENT_PROTOCOL,
  //Protocol  --------------> MP_PPP
  CONFIGURE_CURRENT_PROTOCOL,
  SELECT_NEW_PROTOCOL,

  //Bound T1/E1 Channels  --> 1-5
  BOUND_TE1_CHANNELS,
/*
  //Ignore DCD Status ------> NO
  IGNORE_DCD,
  //Ignore CTS Status ------> NO
  IGNORE_CTS
*/
  NET_IF_SETUP
};


#define DBG_MENU_AFT_LOGICAL_CHANNEL_CFG 1

menu_aft_logical_channel_cfg::
	menu_aft_logical_channel_cfg(IN char * lxdialog_path,
                               IN conf_file_reader* cfr,
                               IN list_element_chan_def* logical_ch_cfg)
{
  Debug(DBG_MENU_AFT_LOGICAL_CHANNEL_CFG,
    ("menu_aft_logical_channel_cfg::menu_aft_logical_channel_cfg()\n"));

  snprintf(this->lxdialog_path, MAX_PATH_LENGTH, "%s", lxdialog_path);

  this->cfr = cfr;
  this->obj_list = cfr->obj_list;
  this->logical_ch_cfg = logical_ch_cfg;
}

menu_aft_logical_channel_cfg::~menu_aft_logical_channel_cfg()
{
  Debug(DBG_MENU_AFT_LOGICAL_CHANNEL_CFG,
    ("menu_aft_logical_channel_cfg::~menu_aft_logical_channel_cfg()\n"));
}

int menu_aft_logical_channel_cfg::run(IN int wanpipe_number, OUT int * selection_index,
                                      IN int display_protocol_cfg_flag,
                                      IN int number_of_timeslot_groups)
{
  string menu_str;
  int rc;
  char tmp_buff[MAX_PATH_LENGTH];
  int num_of_visible_items;
  //help text box
  text_box tb;
  int old_protocol;
  conf_file_reader* lip_layer_file_reader;

  Debug(DBG_MENU_AFT_LOGICAL_CHANNEL_CFG,
    ("menu_frame_relay_DLCI_configuration::run()\n"));

  input_box_active_channels act_channels_ip;

again:
  rc = YES;
  menu_str = " ";

  if(logical_ch_cfg ==  NULL){
    ERR_DBG_OUT(("%s configuration pointer is NULL (logical_ch_cfg) !!!\n",
      TIME_SLOT_GROUP_STR));
    rc = NO;
    goto cleanup;
  }

  /////////////////////////////////////////////////////////////////////////////////////
  lip_layer_file_reader = (conf_file_reader*)logical_ch_cfg->conf_file_reader;

  if( lip_layer_file_reader != NULL){
    Debug(DBG_MENU_AFT_LOGICAL_CHANNEL_CFG,
      ("1. lip_layer_file_reader->link_defs->config_id: %d\n",
      lip_layer_file_reader->link_defs->config_id));

    Debug(DBG_MENU_AFT_LOGICAL_CHANNEL_CFG, ("logical_ch_cfg->data.usedby: %s\n",
      logical_ch_cfg->data.usedby));
  }

  if(display_protocol_cfg_flag == YES){

    if( lip_layer_file_reader != NULL){

      Debug(DBG_MENU_AFT_LOGICAL_CHANNEL_CFG,
        ("2. lip_layer_file_reader->link_defs->config_id: %d\n",
        lip_layer_file_reader->link_defs->config_id));

      snprintf(tmp_buff, MAX_PATH_LENGTH, " \"%d\" ", CURRENT_PROTOCOL);
      menu_str += tmp_buff;
      snprintf(tmp_buff, MAX_PATH_LENGTH, " \"Selected Protocol : %s\" ",
        get_protocol_string(lip_layer_file_reader->link_defs->config_id));
      menu_str += tmp_buff;

      num_of_visible_items = 4;

      if(lip_layer_file_reader->link_defs->config_id != WANCONFIG_HDLC){
        //on AFT HDLC is in hardware - there is nothing to configure.
        snprintf(tmp_buff, MAX_PATH_LENGTH, " \"%d\" ", CONFIGURE_CURRENT_PROTOCOL);
        menu_str += tmp_buff;
        snprintf(tmp_buff, MAX_PATH_LENGTH, " \"Configure Selected Protocol\" ");
        menu_str += tmp_buff;
      }

      /////////////////////////////////////////////////////////////////////////////////////
      snprintf(tmp_buff, MAX_PATH_LENGTH, " \"%d\" ", SELECT_NEW_PROTOCOL);
      menu_str += tmp_buff;
      snprintf(tmp_buff, MAX_PATH_LENGTH, " \"Change Protocol\" ");
      menu_str += tmp_buff;

      /////////////////////////////////////////////////
      if(number_of_timeslot_groups > 1){
        //if only one timeslot group, NET_IF_SETUP is done in the
        //'menu_new_device_configuration::run()'
        snprintf(tmp_buff, MAX_PATH_LENGTH, " \"%d\" ", NET_IF_SETUP);
        menu_str += tmp_buff;

        //create the menu item
        if(lip_layer_file_reader->obj_list->get_size()){
          snprintf(tmp_buff, MAX_PATH_LENGTH, " \"Interface Setup------------> %d Defined\" ",
            lip_layer_file_reader->obj_list->get_size());
        }else{
          snprintf(tmp_buff, MAX_PATH_LENGTH, " \"Interface Setup------------>  Undefined\" ");
        }
        menu_str += tmp_buff;
      }
      /////////////////////////////////////////////////
    }

  }else{

    num_of_visible_items = 1;
    /////////////////////////////////////////////////////////////////////////////////////
    snprintf(tmp_buff, MAX_PATH_LENGTH, " \"%d\" ", BOUND_TE1_CHANNELS);
    menu_str += tmp_buff;
    snprintf(tmp_buff, MAX_PATH_LENGTH, " \"Timeslots in Group-> %s\" ",
      logical_ch_cfg->data.active_channels_string);
    menu_str += tmp_buff;

    goto label_show_bound_timeslots;
  }

  /////////////////////////////////////////////////////////////////////////////////////
  //create the explanation text for the menu
  snprintf(tmp_buff, MAX_PATH_LENGTH,
"------------------------------------------\
\nSet cofiguration of %s %s.", TIME_SLOT_GROUP_STR, logical_ch_cfg->data.addr);


  if(set_configuration(   YES,//indicates to call V2 of the function
                          MENU_BOX_BACK,//MENU_BOX_SELECT,
                          lxdialog_path,
                          "AFT DS0 CHANNEL CONFIGURATION",
                          WANCFG_PROGRAM_NAME,
                          tmp_buff,
                          MENU_HEIGTH, MENU_WIDTH,
                          num_of_visible_items,
                          (char*)menu_str.c_str()
                          ) == NO){
    rc = NO;
    goto cleanup;
  }

  if(show(selection_index) == NO){
    rc = NO;
    goto cleanup;
  }

  switch(*selection_index)
  {
  case MENU_BOX_BUTTON_SELECT:
    Debug(DBG_MENU_AFT_LOGICAL_CHANNEL_CFG,
      ("option selected for editing: %s\n", get_lxdialog_output_string()));

    switch(atoi(get_lxdialog_output_string()))
    {
    case CURRENT_PROTOCOL:
      //do nothing
      goto again;
      break;

    case CONFIGURE_CURRENT_PROTOCOL:
      if( lip_layer_file_reader != NULL ){
        //display configuration of the protocol.
        if(display_protocol_cfg(lip_layer_file_reader->link_defs->config_id,
                                lip_layer_file_reader) == NO){
          rc = NO;
        }else{

          Debug(DBG_MENU_AFT_LOGICAL_CHANNEL_CFG,
            ("lip_layer_file_reader->link_defs->config_id: %d\n",
            lip_layer_file_reader->link_defs->config_id));

/*
          menu_list_existing_wanpipes::
            update_per_group_of_channels_iformation(cfr);
          exit(0);
*/
          goto again;
        }
      }else{
        //if no LIP layer - this is HDLC streaming. No cfg options, nothing to display.
        goto again;
      }
      break;

    case SELECT_NEW_PROTOCOL:
      {
        /////////////////////////////////////////////////////////////////////////////////
        if(logical_ch_cfg->data.chanconf->protocol != PROTOCOL_NOT_SET){
          //ask if user *really* want to change the protocol.
          if(yes_no_question(   selection_index,
                                lxdialog_path,
                                NO_PROTOCOL_NEEDED,
                                "Are you sure you want to change the Protocol?") == NO){
            //error displaying dialog
            return NO;
          }
          switch(*selection_index)
          {
          case YES_NO_TEXT_BOX_BUTTON_YES:
            break;

          case YES_NO_TEXT_BOX_BUTTON_NO:
            //user does not want to change the protocol.
            goto again;
            break;
          }
        }
        //////////////////////////////////////////////////////////////////////////////////
        if( lip_layer_file_reader != NULL){
          old_protocol = lip_layer_file_reader->link_defs->config_id;
        }else{
          old_protocol = logical_ch_cfg->data.chanconf->protocol;
        }

        Debug(DBG_MENU_AFT_LOGICAL_CHANNEL_CFG,
          ("1. AFT: old_protocol: %d, new protocol: %d\n",
          old_protocol, logical_ch_cfg->data.chanconf->protocol));

        menu_aft_select_protocol aft_select_protocol(lxdialog_path, cfr, logical_ch_cfg);
        if(aft_select_protocol.run(selection_index) == NO){
          rc = NO;
        }else{

          Debug(DBG_MENU_AFT_LOGICAL_CHANNEL_CFG,
            ("2. AFT: old_protocol: %d, new protocol: %d\n",
            old_protocol, logical_ch_cfg->data.chanconf->protocol));

          if(old_protocol != logical_ch_cfg->data.chanconf->protocol){

            Debug(DBG_MENU_AFT_LOGICAL_CHANNEL_CFG,
              ("3. AFT: old_protocol: %d, new protocol: %d\n",
              old_protocol, logical_ch_cfg->data.chanconf->protocol));

              if(handle_protocol_change(logical_ch_cfg->data.chanconf->protocol) == NO){
                ERR_DBG_OUT(("Protocol change failed!!!\n"));
                //exit(0);
                rc = NO;
              }else{
                Debug(DBG_MENU_AFT_LOGICAL_CHANNEL_CFG,
                  ("AFT: logical_ch_cfg->data.chanconf->protocol: %d\n",
                  logical_ch_cfg->data.chanconf->protocol));

                Debug(DBG_MENU_AFT_LOGICAL_CHANNEL_CFG,
                  ("AFT: logical_ch_cfg->conf_file_reader: %p\n",
                  logical_ch_cfg->conf_file_reader));

                Debug(DBG_MENU_AFT_LOGICAL_CHANNEL_CFG,
                  ("AFT: logical_ch_cfg->data.name: %s\n",
                  logical_ch_cfg->data.name));

                //
                if(logical_ch_cfg->data.chanconf->protocol != WANCONFIG_HDLC){
                  strcpy(logical_ch_cfg->data.usedby, "STACK");
                }else{
                  //user switched to HDLC Streaming, set default 'usedby' to WANPIPE.
                  strcpy(logical_ch_cfg->data.usedby, "WANPIPE");
                }
                goto again;
              }
          }else{
            goto again;
          }
        }
      }
      break;

    case BOUND_TE1_CHANNELS:
label_show_bound_timeslots:
      if(act_channels_ip.show(  lxdialog_path,
                                logical_ch_cfg->data.chanconf->protocol,
                                logical_ch_cfg->data.active_channels_string,//initial text
                                selection_index,
                                cfr->link_defs->linkconf->te_cfg.media) == NO){
        rc = NO;
      }else{

        switch(*selection_index)
        {
        case INPUT_BOX_BUTTON_OK:
          snprintf(logical_ch_cfg->data.active_channels_string, MAX_LEN_OF_ACTIVE_CHANNELS_STRING,
                act_channels_ip.get_lxdialog_output_string());

          logical_ch_cfg->data.chanconf->active_ch = act_channels_ip.active_ch;
          break;

        case INPUT_BOX_BUTTON_HELP:
          tb.show_help_message(lxdialog_path, WANCONFIG_AFT, bound_te1_channels_help_str);
          break;
        }

        //goto again;
      }
      break;
#if 0
    case IGNORE_DCD:
      break;

    case IGNORE_CTS:
      break;
#endif

    case NET_IF_SETUP:
      {     //display list of network interfaces.
        menu_net_interfaces_list net_interfaces_list(lxdialog_path, lip_layer_file_reader);
        if(net_interfaces_list.run(selection_index) == NO){
          rc = NO;
        }
      }
      break;

    default:
      ERR_DBG_OUT(("Invalid option selected for editing!! selection: %s\n",
        get_lxdialog_output_string()));
      rc = NO;
    }
    break;

  case MENU_BOX_BUTTON_HELP:
    {
      Debug(DBG_MENU_AFT_LOGICAL_CHANNEL_CFG,
        ("HELP option selected: %s\n", get_lxdialog_output_string()));

      switch(atoi(get_lxdialog_output_string()))
      {
      case CURRENT_PROTOCOL:
        //do nothing
        break;

      case CONFIGURE_CURRENT_PROTOCOL:
        tb.show_help_message(lxdialog_path, WANCONFIG_AFT,
          "Set Configuration of Currently selected protocol.");
        break;

      case SELECT_NEW_PROTOCOL:
        tb.show_help_message(lxdialog_path, WANCONFIG_AFT,
          aft_logical_channel_protocol_select_help_str);
        break;

      case BOUND_TE1_CHANNELS:
        tb.show_help_message(lxdialog_path, WANCONFIG_AFT, bound_te1_channels_help_str);
        break;

      default:
        tb.show_help_message(lxdialog_path, WANCONFIG_AFT, option_not_implemented_yet_help_str);
        break;
      }
      goto again;
    }
    break;

  case MENU_BOX_BUTTON_EXIT:
    //no check is done here because each menu item checks
    //the input separatly.
    rc = YES;
    break;
  }//switch(*selection_index)

cleanup:
  Debug(DBG_MENU_AFT_LOGICAL_CHANNEL_CFG,
    ("menu_frame_relay_DLCI_configuration::run(): rc: %d\n", rc));

  return rc;

}

/*
int menu_aft_logical_channel_cfg::run(IN int wanpipe_number, OUT int * selection_index,
                                      IN int display_protocol_cfg_flag)
{
  string menu_str;
  int rc;
  char tmp_buff[MAX_PATH_LENGTH];
  int num_of_visible_items;
  //help text box
  text_box tb;
  int old_protocol;
  conf_file_reader* lip_layer_file_reader;

  Debug(DBG_MENU_AFT_LOGICAL_CHANNEL_CFG,
    ("menu_frame_relay_DLCI_configuration::run()\n"));

  input_box_active_channels act_channels_ip;

again:
  rc = YES;
  menu_str = " ";

  if(logical_ch_cfg ==  NULL){
    ERR_DBG_OUT(("%s configuration pointer is NULL (logical_ch_cfg) !!!\n",
      TIME_SLOT_GROUP_STR));
    rc = NO;
    goto cleanup;
  }

  /////////////////////////////////////////////////////////////////////////////////////
  lip_layer_file_reader = (conf_file_reader*)logical_ch_cfg->conf_file_reader;

  if( lip_layer_file_reader != NULL){
    Debug(DBG_MENU_AFT_LOGICAL_CHANNEL_CFG,
      ("1. lip_layer_file_reader->link_defs->config_id: %d\n",
      lip_layer_file_reader->link_defs->config_id));

    Debug(DBG_MENU_AFT_LOGICAL_CHANNEL_CFG, ("logical_ch_cfg->data.usedby: %s\n",
      logical_ch_cfg->data.usedby));
  }

  if(display_protocol_cfg_flag == YES){

    if( lip_layer_file_reader != NULL &&
        lip_layer_file_reader->link_defs->config_id != WANCONFIG_HDLC){

      Debug(DBG_MENU_AFT_LOGICAL_CHANNEL_CFG,
        ("2. lip_layer_file_reader->link_defs->config_id: %d\n",
        lip_layer_file_reader->link_defs->config_id));

      snprintf(tmp_buff, MAX_PATH_LENGTH, " \"%d\" ", CURRENT_PROTOCOL);
      menu_str += tmp_buff;
      snprintf(tmp_buff, MAX_PATH_LENGTH, " \"Selected Protocol : %s\" ",
        get_protocol_string(lip_layer_file_reader->link_defs->config_id));
      menu_str += tmp_buff;

      num_of_visible_items = 4;

      snprintf(tmp_buff, MAX_PATH_LENGTH, " \"%d\" ", CONFIGURE_CURRENT_PROTOCOL);
      menu_str += tmp_buff;
      snprintf(tmp_buff, MAX_PATH_LENGTH, " \"Configure Selected Protocol\" ");
      menu_str += tmp_buff;

//  }else if(logical_ch_cfg->data.chanconf->protocol == PROTOCOL_NOT_SET){

    }else{
      snprintf(tmp_buff, MAX_PATH_LENGTH, " \"%d\" ", CURRENT_PROTOCOL);
      menu_str += tmp_buff;
      snprintf(tmp_buff, MAX_PATH_LENGTH, " \"Selected Protocol : %s\" ",
        get_protocol_string(logical_ch_cfg->data.chanconf->protocol));
      menu_str += tmp_buff;

      num_of_visible_items = 3;
    }

    /////////////////////////////////////////////////////////////////////////////////////
    snprintf(tmp_buff, MAX_PATH_LENGTH, " \"%d\" ", SELECT_NEW_PROTOCOL);
    menu_str += tmp_buff;
    snprintf(tmp_buff, MAX_PATH_LENGTH, " \"Change Protocol\" ");
    menu_str += tmp_buff;

  }else{

    num_of_visible_items = 1;
    /////////////////////////////////////////////////////////////////////////////////////
    snprintf(tmp_buff, MAX_PATH_LENGTH, " \"%d\" ", BOUND_TE1_CHANNELS);
    menu_str += tmp_buff;
    snprintf(tmp_buff, MAX_PATH_LENGTH, " \"Timeslots in Group-> %s\" ",
      logical_ch_cfg->data.active_channels_string);
    menu_str += tmp_buff;

    goto label_show_bound_timeslots;
  }

  /////////////////////////////////////////////////////////////////////////////////////
  //create the explanation text for the menu
  snprintf(tmp_buff, MAX_PATH_LENGTH,
"------------------------------------------\
\nSet cofiguration of %s %s.", TIME_SLOT_GROUP_STR, logical_ch_cfg->data.addr);


  if(set_configuration(   YES,//indicates to call V2 of the function
                          MENU_BOX_BACK,//MENU_BOX_SELECT,
                          lxdialog_path,
                          "AFT DS0 CHANNEL CONFIGURATION",
                          WANCFG_PROGRAM_NAME,
                          tmp_buff,
                          MENU_HEIGTH, MENU_WIDTH,
                          num_of_visible_items,
                          (char*)menu_str.c_str()
                          ) == NO){
    rc = NO;
    goto cleanup;
  }

  if(show(selection_index) == NO){
    rc = NO;
    goto cleanup;
  }

  switch(*selection_index)
  {
  case MENU_BOX_BUTTON_SELECT:
    Debug(DBG_MENU_AFT_LOGICAL_CHANNEL_CFG,
      ("option selected for editing: %s\n", get_lxdialog_output_string()));

    switch(atoi(get_lxdialog_output_string()))
    {
    case CURRENT_PROTOCOL:
      //do nothing
      goto again;
      break;

    case CONFIGURE_CURRENT_PROTOCOL:
      if( lip_layer_file_reader != NULL ){
        //display configuration of the protocol.
        if(display_protocol_cfg(lip_layer_file_reader->link_defs->config_id,
                                lip_layer_file_reader) == NO){
          rc = NO;
        }else{
          goto again;
        }
      }else{
        //if no LIP layer - this is HDLC streaming. No cfg options, nothing to display.
        goto again;
      }
      break;

    case SELECT_NEW_PROTOCOL:
      {
        /////////////////////////////////////////////////////////////////////////////////
        if(logical_ch_cfg->data.chanconf->protocol != PROTOCOL_NOT_SET){
          //ask if user *really* want to change the protocol.
          if(yes_no_question(   selection_index,
                                lxdialog_path,
                                NO_PROTOCOL_NEEDED,
                                "Are you sure you want to change the Protocol?") == NO){
            //error displaying dialog
            return NO;
          }
          switch(*selection_index)
          {
          case YES_NO_TEXT_BOX_BUTTON_YES:
            break;

          case YES_NO_TEXT_BOX_BUTTON_NO:
            //user does not want to change the protocol.
            goto again;
            break;
          }
        }
        //////////////////////////////////////////////////////////////////////////////////
        if( lip_layer_file_reader != NULL){
          old_protocol = lip_layer_file_reader->link_defs->config_id;
        }else{
          old_protocol = logical_ch_cfg->data.chanconf->protocol;
        }

        Debug(DBG_MENU_AFT_LOGICAL_CHANNEL_CFG,
          ("1. AFT: old_protocol: %d, new protocol: %d\n",
          old_protocol, logical_ch_cfg->data.chanconf->protocol));

        menu_aft_select_protocol aft_select_protocol(lxdialog_path, cfr, logical_ch_cfg);
        if(aft_select_protocol.run(selection_index) == NO){
          rc = NO;
        }else{

          Debug(DBG_MENU_AFT_LOGICAL_CHANNEL_CFG,
            ("2. AFT: old_protocol: %d, new protocol: %d\n",
            old_protocol, logical_ch_cfg->data.chanconf->protocol));

          if(old_protocol != logical_ch_cfg->data.chanconf->protocol){

            Debug(DBG_MENU_AFT_LOGICAL_CHANNEL_CFG,
              ("3. AFT: old_protocol: %d, new protocol: %d\n",
              old_protocol, logical_ch_cfg->data.chanconf->protocol));

              if(handle_protocol_change(logical_ch_cfg->data.chanconf->protocol) == NO){
                ERR_DBG_OUT(("Protocol change failed!!!\n"));
                //exit(0);
                rc = NO;
              }else{
                Debug(DBG_MENU_AFT_LOGICAL_CHANNEL_CFG,
                  ("AFT: logical_ch_cfg->data.chanconf->protocol: %d\n",
                  logical_ch_cfg->data.chanconf->protocol));

                Debug(DBG_MENU_AFT_LOGICAL_CHANNEL_CFG,
                  ("AFT: logical_ch_cfg->conf_file_reader: %p\n",
                  logical_ch_cfg->conf_file_reader));

                Debug(DBG_MENU_AFT_LOGICAL_CHANNEL_CFG,
                  ("AFT: logical_ch_cfg->data.name: %s\n",
                  logical_ch_cfg->data.name));

                //
                if(logical_ch_cfg->data.chanconf->protocol != WANCONFIG_HDLC){
                  strcpy(logical_ch_cfg->data.usedby, "STACK");
                }else{
                  //user switched to HDLC Streaming, set default 'usedby' to WANPIPE.
                  strcpy(logical_ch_cfg->data.usedby, "WANPIPE");
                }
                goto again;
              }
          }else{
            goto again;
          }
        }
      }
      break;

    case BOUND_TE1_CHANNELS:
label_show_bound_timeslots:
      if(act_channels_ip.show(  lxdialog_path,
                                logical_ch_cfg->data.chanconf->protocol,
                                logical_ch_cfg->data.active_channels_string,//initial text
                                selection_index,
                                cfr->link_defs->linkconf->te_cfg.media) == NO){
        rc = NO;
      }else{

        switch(*selection_index)
        {
        case INPUT_BOX_BUTTON_OK:
          snprintf(logical_ch_cfg->data.active_channels_string, MAX_LEN_OF_ACTIVE_CHANNELS_STRING,
                act_channels_ip.get_lxdialog_output_string());

          logical_ch_cfg->data.chanconf->active_ch = act_channels_ip.active_ch;
          break;

        case INPUT_BOX_BUTTON_HELP:
          tb.show_help_message(lxdialog_path, WANCONFIG_AFT, bound_te1_channels_help_str);
          break;
        }

        //goto again;
      }
      break;
#if 0
    case IGNORE_DCD:
      break;

    case IGNORE_CTS:
      break;
#endif

    default:
      ERR_DBG_OUT(("Invalid option selected for editing!! selection: %s\n",
        get_lxdialog_output_string()));
      rc = NO;
    }
    break;

  case MENU_BOX_BUTTON_HELP:
    {
      Debug(DBG_MENU_AFT_LOGICAL_CHANNEL_CFG,
        ("HELP option selected: %s\n", get_lxdialog_output_string()));

      switch(atoi(get_lxdialog_output_string()))
      {
      case CURRENT_PROTOCOL:
        //do nothing
        break;

      case CONFIGURE_CURRENT_PROTOCOL:
        tb.show_help_message(lxdialog_path, WANCONFIG_AFT,
          "Set Configuration of Currently selected protocol.");
        break;

      case SELECT_NEW_PROTOCOL:
        tb.show_help_message(lxdialog_path, WANCONFIG_AFT,
          aft_logical_channel_protocol_select_help_str);
        break;

      case BOUND_TE1_CHANNELS:
        tb.show_help_message(lxdialog_path, WANCONFIG_AFT, bound_te1_channels_help_str);
        break;

      default:
        tb.show_help_message(lxdialog_path, WANCONFIG_AFT, option_not_implemented_yet_help_str);
        break;
      }
      goto again;
    }
    break;

  case MENU_BOX_BUTTON_EXIT:
    //no check is done here because each menu item checks
    //the input separatly.
    rc = YES;
    break;
  }//switch(*selection_index)

cleanup:
  Debug(DBG_MENU_AFT_LOGICAL_CHANNEL_CFG,
    ("menu_frame_relay_DLCI_configuration::run(): rc: %d\n", rc));

  return rc;

}
*/

int menu_aft_logical_channel_cfg::display_protocol_cfg( IN int protocol,
                                                        IN conf_file_reader* local_cfr)
{
  int rc = YES;
  int selection_index;

  Debug(DBG_MENU_AFT_LOGICAL_CHANNEL_CFG, ("display_protocol_cfg(): %d\n", protocol));

  if(local_cfr == NULL){
    Debug(DBG_MENU_AFT_LOGICAL_CHANNEL_CFG, ("display_protocol_cfg(): local_cfr == NULL!!\n"));
    return NO;
  }

  switch(protocol)
  {
  case WANCONFIG_MFR:
    {
      menu_frame_relay_basic_cfg fr_basic_cfg(lxdialog_path, local_cfr);
      fr_basic_cfg.run(&selection_index, logical_ch_cfg->data.name);
    }
    break;

  case WANCONFIG_MPPP://and WANCONFIG_MPROT too
    {
      menu_ppp_basic_cfg ppp_basic_cfg(lxdialog_path, local_cfr);
      ppp_basic_cfg.run(&selection_index);
    }
    break;

  case WANCONFIG_MPCHDLC:
    {
      menu_chdlc_basic_cfg chdlc_basic_cfg(lxdialog_path, local_cfr);
      chdlc_basic_cfg.run(&selection_index);
    }
    break;

  case WANCONFIG_FR:
  case NO_PROTOCOL_NEEDED:
  case WANCONFIG_X25:
  case WANCONFIG_PPP:
  case WANCONFIG_CHDLC:
  case WANCONFIG_HDLC:
    //for AFT card this protocol is done in hardware - no config options available.
  case WANCONFIG_BSC:
  case WANCONFIG_BITSTRM:
  case WANCONFIG_EDUKIT:
  case WANCONFIG_SS7:
  case WANCONFIG_BSCSTRM:
  case WANCONFIG_ADSL:
  case WANCONFIG_SDLC:
  case WANCONFIG_ATM:
  case WANCONFIG_POS:
  case WANCONFIG_AFT:
  case WANCONFIG_DEBUG:
  case WANCONFIG_ADCCP:
  case WANCONFIG_MLINK_PPP:
  case WANCONFIG_GENERIC:
  default:
    rc = NO;
    break;
  }

  return rc;
}


int menu_aft_logical_channel_cfg::handle_protocol_change( unsigned int protocol_selected)
{
  int rc = YES;
  conf_file_reader* new_cfr;

  //for HDLC Streaming there is no LIP layer, so configuration should be read.
//  if(protocol_selected != WANCONFIG_HDLC){

    new_cfr = new conf_file_reader(
      menu_select_and_configure_protocol::get_template_file_path(protocol_selected));

    //STEP1:
    //read the template file.
    if(new_cfr->read_conf_file() != 0){
      ERR_DBG_OUT(("Failed to parse '%s' file!!",
        menu_select_and_configure_protocol::get_template_file_path(protocol_selected)));
      return NO;
    }

    //STEP2:
    new_cfr->wanpipe_number = cfr->wanpipe_number;
    link_def_t * old_link_def = cfr->link_defs;
    link_def_t * new_link_def = new_cfr->link_defs;

    wandev_conf_t * old_linkconf = cfr->link_defs->linkconf;
    wandev_conf_t * new_linkconf = new_cfr->link_defs->linkconf;

    new_linkconf->card_type = old_linkconf->card_type;
    new_link_def->card_version = old_link_def->card_version;

    //???
    new_link_def->config_id = protocol_selected;

    new_linkconf->auto_pci_cfg = old_linkconf->auto_pci_cfg;
    new_linkconf->PCI_slot_no = old_linkconf->PCI_slot_no;
    new_linkconf->pci_bus_no = old_linkconf->pci_bus_no;
    new_linkconf->S514_CPU_no[0] = old_linkconf->S514_CPU_no[0];

    new_linkconf->interface = old_linkconf->interface;
    new_linkconf->bps = old_linkconf->bps;
    memcpy(&new_linkconf->te_cfg, &old_linkconf->te_cfg, sizeof(sdla_te_cfg_t));

    //STEP3:
    //update wanpipe name. this wanpipe name will get the name of this Timeslot Group.
    snprintf(new_link_def->name, WAN_DRVNAME_SZ, "%s", logical_ch_cfg->data.name);
    Debug(DBG_MENU_AFT_LOGICAL_CHANNEL_CFG, ("new_link_def->name: %s\n",
      logical_ch_cfg->data.name));

    //update default names of each interface
    //
    ///////////////////////////////
    //STEP4:

    objects_list* local_obj_list = new_cfr->obj_list;
    list_element_chan_def* local_list_el_chan_def =
      (list_element_chan_def*)local_obj_list->get_first();

    if(local_list_el_chan_def != NULL){
      strcpy(logical_ch_cfg->data.aft_dlci_number, local_list_el_chan_def->data.addr);
    }else{
      ERR_DBG_OUT(("Failed to initialize 'aft_dlci_number'(1)!!"));
      return NO;
    }

    while(local_list_el_chan_def != NULL){

      Debug(DBG_MENU_AFT_LOGICAL_CHANNEL_CFG,
        ("menu_aft_logical_cannel_cfg:new_link_def->config_id: %d\n",
        new_link_def->config_id));
/*
      if(menu_select_and_configure_protocol::update_interface_name(  local_list_el_chan_def,
                                  new_link_def->config_id,
                                  cfr->wanpipe_number) == NO){
        ERR_DBG_OUT(("Failed to update interface name!!"));
        return NO;
      }
*/
//      strcpy(local_list_el_chan_def->data.name, "hello");

      aft_update_interface_name(local_list_el_chan_def, new_link_def->config_id,
                                logical_ch_cfg->data.name);

      Debug(DBG_MENU_AFT_LOGICAL_CHANNEL_CFG, ("local_list_el_chan_def->data.name: %s\n",
        local_list_el_chan_def->data.name));

      local_list_el_chan_def =
        (list_element_chan_def*)local_obj_list->get_next_element(local_list_el_chan_def);
    }//while()

    //exit(0);

    if(logical_ch_cfg->conf_file_reader != NULL){
      delete (conf_file_reader*)logical_ch_cfg->conf_file_reader;
    }

//    menu_list_existing_wanpipes::update_per_group_of_channels_iformation(underlying_cfr);//new_cfr);

    //exit(0);

    logical_ch_cfg->conf_file_reader = new_cfr;
    ((conf_file_reader*)logical_ch_cfg->conf_file_reader)->link_defs->config_id = protocol_selected;
/*
  }
  else
  {
    //HDLC Streamin on AFT is done in hardware.
    if(logical_ch_cfg->conf_file_reader != NULL){
      delete (conf_file_reader*)logical_ch_cfg->conf_file_reader;
      logical_ch_cfg->conf_file_reader = NULL;
    }

    //set default values for HDLC streaming
    logical_ch_cfg->data.chanconf->hdlc_streaming = WANOPT_YES;
  }
*/
  Debug(DBG_MENU_AFT_LOGICAL_CHANNEL_CFG, ("handle_protocol_change(): returning: rc: %d\n", rc));

  return rc;
}

int menu_aft_logical_channel_cfg::aft_update_interface_name(
                                              list_element_chan_def* list_el_chan_def,
                                              int config_id,
                                              IN char* timeslot_group_name)
{
  Debug(DBG_MENU_AFT_LOGICAL_CHANNEL_CFG,
    ("menu_aft_logical_channel_cfg::aft_update_interface_name()\n"));

  Debug(DBG_MENU_AFT_LOGICAL_CHANNEL_CFG,
    ("cfr->link_defs->name: %s\n", cfr->link_defs->name));

  Debug(DBG_MENU_AFT_LOGICAL_CHANNEL_CFG,
    ("timeslot_group_name: %s\n", timeslot_group_name));

  //exit(0);

  switch(config_id)
  {
  case WANCONFIG_MFR:
    if(atoi(list_el_chan_def->data.addr) == 0){
      //it is auto dlci
      snprintf(list_el_chan_def->data.name, WAN_IFNAME_SZ, "%sf",
               timeslot_group_name
              );
    }else{
      snprintf(list_el_chan_def->data.name, WAN_IFNAME_SZ, "%sf%s",
            timeslot_group_name,
            list_el_chan_def->data.addr);
    }
    break;

  case WANCONFIG_MPPP:
    snprintf(list_el_chan_def->data.name, WAN_IFNAME_SZ, "%sppp",
            timeslot_group_name);
    break;

  case WANCONFIG_MPCHDLC:
    //note: maximum IF len is 8 chars, so using "chdl" not "chdlc"
    snprintf(list_el_chan_def->data.name, WAN_IFNAME_SZ, "%schdl",
            timeslot_group_name);
    break;

  case WANCONFIG_HDLC:
    //note: on AFT HDLC is in hardware so there is NO LIP layer - no postfix needed.
//    snprintf(list_el_chan_def->data.name, WAN_IFNAME_SZ, "%shdlc",
    snprintf(list_el_chan_def->data.name, WAN_IFNAME_SZ, "%s",
            timeslot_group_name);
    break;

  default:
    snprintf(list_el_chan_def->data.name, WAN_IFNAME_SZ, "Unknown!!");
    break;
  }

  return YES;
}
