/***************************************************************************
                          menu_aft_select_protocol.cpp  -  description
                             -------------------
    begin                : Wed Apr 14 2004
    copyright            : (C) 2004 by David Rokhvarg
    email                : davidr@sangoma.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "menu_aft_select_protocol.h"
#include "wancfg.h"
#include "text_box.h"
#include "list_element_chan_def.h"

#define DBG_MENU_AFT_SELECT_PROTOCOL  1

menu_aft_select_protocol::
  menu_aft_select_protocol( IN char * lxdialog_path,
                            IN conf_file_reader* cfr,
                            IN list_element_chan_def* logical_ch_cfg)
{
  Debug(DBG_MENU_AFT_SELECT_PROTOCOL,
    ("menu_aft_select_protocol::menu_aft_select_protocol()\n"));

  snprintf(this->lxdialog_path, MAX_PATH_LENGTH, "%s", lxdialog_path);
  this->cfr = cfr;
  this->logical_ch_cfg = logical_ch_cfg;
}

menu_aft_select_protocol::~menu_aft_select_protocol()
{
  Debug(DBG_MENU_AFT_SELECT_PROTOCOL,
    ("menu_aft_select_protocol::~menu_aft_select_protocol()\n"));
}

int menu_aft_select_protocol::run(OUT int * selection_index)
{
  string menu_str;
  int rc;
  char tmp_buff[MAX_PATH_LENGTH];
  char exit_dialog;
  int num_of_visible_items;
  list_element_chan_def* list_el_chan_def = NULL;

  //help text box
  text_box tb;
  Debug(DBG_MENU_AFT_SELECT_PROTOCOL, ("menu_aft_select_protocol::run()\n"));

again:
  objects_list * obj_list = cfr->obj_list;

  rc = YES;
  exit_dialog = NO;
  menu_str = " ";

  num_of_visible_items = 4;

  //////////////////////////////////////////////////////////////////////////////////////
  snprintf(tmp_buff, MAX_PATH_LENGTH, " \"%d\" ", WANCONFIG_MPPP);
  menu_str += tmp_buff;
  snprintf(tmp_buff, MAX_PATH_LENGTH, " \"%s\" ", get_protocol_string(WANCONFIG_MPPP));
  menu_str += tmp_buff;

  //////////////////////////////////////////////////////////////////////////////////////
  snprintf(tmp_buff, MAX_PATH_LENGTH, " \"%d\" ", WANCONFIG_MPCHDLC);
  menu_str += tmp_buff;
  snprintf(tmp_buff, MAX_PATH_LENGTH, " \"%s\" ", get_protocol_string(WANCONFIG_MPCHDLC));
  menu_str += tmp_buff;

  //////////////////////////////////////////////////////////////////////////////////////
  snprintf(tmp_buff, MAX_PATH_LENGTH, " \"%d\" ", WANCONFIG_MFR);
  menu_str += tmp_buff;
  snprintf(tmp_buff, MAX_PATH_LENGTH, " \"%s\" ", get_protocol_string(WANCONFIG_MFR));
  menu_str += tmp_buff;

  //////////////////////////////////////////////////////////////////////////////////////
  snprintf(tmp_buff, MAX_PATH_LENGTH, " \"%d\" ", WANCONFIG_HDLC);
  menu_str += tmp_buff;
  snprintf(tmp_buff, MAX_PATH_LENGTH, " \"%s\" ", get_protocol_string(WANCONFIG_HDLC));
  menu_str += tmp_buff;

  //////////////////////////////////////////////////////////////////////////////////////
  //create the explanation text for the menu
  snprintf(tmp_buff, MAX_PATH_LENGTH,
"\n------------------------------------------\
\nSelect Protocol for %s %s (Wan Device: %s)",
TIME_SLOT_GROUPS_STR, logical_ch_cfg->data.addr, cfr->link_defs->name);

  if(set_configuration(   YES,//indicates to call V2 of the function
                          MENU_BOX_BACK,//MENU_BOX_SELECT,
                          lxdialog_path,
                          "SELECT (AFT) PROTOCOL",
                          WANCFG_PROGRAM_NAME,
                          tmp_buff,
                          MENU_HEIGTH, MENU_WIDTH,
                          num_of_visible_items,
                          (char*)menu_str.c_str()
                          ) == NO){
    rc = NO;
    goto cleanup;
  }

  if(show(selection_index) == NO){
    rc = NO;
    goto cleanup;
  }

  //////////////////////////////////////////////////////////////////////////////////////

  exit_dialog = NO;
  switch(*selection_index)
  {
  case MENU_BOX_BUTTON_SELECT:
    Debug(DBG_MENU_AFT_SELECT_PROTOCOL,
      ("option selected for editing: %s\n", get_lxdialog_output_string()));

    switch(atoi(get_lxdialog_output_string()))
    {
    case WANCONFIG_MPPP:
	  case WANCONFIG_MPCHDLC:
	  case WANCONFIG_MFR:
	  case WANCONFIG_HDLC:
      logical_ch_cfg->data.chanconf->protocol = atoi(get_lxdialog_output_string());
      rc = YES;
      exit_dialog = YES;
      break;

    default:
#if 1
      ERR_DBG_OUT(("Invalid selection: %s\n", get_lxdialog_output_string()));
      rc = NO;
      exit_dialog = YES;
#endif
    }
    break;

  case MENU_BOX_BUTTON_HELP:
    tb.show_help_message(lxdialog_path, NO_PROTOCOL_NEEDED, "Please select Communicaions Protocol.");
    break;

  case MENU_BOX_BUTTON_EXIT:
    Debug(DBG_MENU_AFT_SELECT_PROTOCOL, ("config_id on exit: %d\n",
      logical_ch_cfg->data.chanconf->protocol));

    if(logical_ch_cfg->data.chanconf->protocol == PROTOCOL_NOT_SET){
      tb.show_error_message(lxdialog_path, NO_PROTOCOL_NEEDED, "Error: No Protocol was selected.");
    }else{
      exit_dialog = YES;
    }
    break;
  }//switch(*selection_index)

  if(exit_dialog == NO){
    goto again;
  }
cleanup:
  return rc;
}
