/***************************************************************************
                          menu_frame_relay_basic_cfg.cpp  -  description
                             -------------------
    begin                : Fri Mar 12 2004
    copyright            : (C) 2004 by David Rokhvarg
    email                : davidr@sangoma.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "menu_frame_relay_basic_cfg.h"
#include "wancfg.h"
#include "text_box.h"
#include "text_box_yes_no.h"
#include "menu_frame_relay_signalling.h"
#include "menu_frame_relay_manual_or_auto_dlci_cfg.h"
#include "menu_frame_relay_dlci_list.h"
#include "menu_frame_relay_advanced_global_configuration.h"
#include "input_box_frame_relay_dlci_number.h"

#define DBG_MENU_AFT_LOGICAL_CHANNELS_LIST  1

char* fr_signalling_help_str =
"Frame Relay: SIGNALLING\n"
"\n"
"This parameter specifies frame relay link management\n"
"type.  Available options are:\n"
"\n"
"ANSI    ANSI T1.617 Annex D (default)\n"
"Q933    ITU Q.933A\n"
"LMI     LMI\n"
"No      Turns off polling, thus DLCI becomes active\n"
"        regardless of the remote end.\n"
"        (This option is for NODE configuration only)\n"
"\n"
"Default: ANSI\n";

char* fr_station_type_help_str =
"Frame Relay: STATION\n"
"\n"
"This parameter specifies whether the  adapter\n"
"should operate as a Customer  Premises Equipment (CPE)\n"
"or emulate a  frame relay switch (Access Node).\n"
"\n"
"Available options are:\n"
"      CPE     CPE mode (default)\n"
"      Node    Access Node (switch emulation mode)\n"
"\n"
"Default: CPE\n";

char* fr_dlci_numbering_help_str =
"DLCI Configuration Mode\n"
"-------------------------\n"
"\n"
"Auto - active DLCIs detected from the Full Status Report.\n"
"       This option valid only for CPE.\n"
"\n"
"Manual - user must provide total number of DLCIs and\n"
"         number for each DLCI.\n";

char* fr_advanced_cfg_options_help_str =
"Advanced Frame Relay Configuration\n"
"----------------------------------\n"
" -Select Station Type (CPE/Node)\n"
" -Set T391 timer\n"
" -Set T392 timer\n"
" -Set N391 timer\n"
" -Set N392 timer\n"
" -Set N393 timer\n"
" -Set Fast Connect Option\n";

char* fr_dlci_list_help_str =
"Dispay list of configured DLCIs.\n"
"Allows set per-DLCI parameters.";


enum FR_BASIC_CFG_OPTIONS{
  FR_SIGNALLING=1,
  DLCI_NUMBERING,
  DLCI_LIST,
  ADVANCED_GLOBAL_FR_CFG,
  //FIXIT: AFT ONLY
  MP_FR_STATION_TYPE,
  MP_FR_DLCI_NUMBERING,
  MP_FR_DLCI_NUMBER,
  EMPTY_LINE
};

char * get_fr_signalling_str(unsigned signalling);

menu_frame_relay_basic_cfg::menu_frame_relay_basic_cfg( IN char * lxdialog_path,
                                                        IN conf_file_reader* cfr)
{
  Debug(DBG_MENU_AFT_LOGICAL_CHANNELS_LIST,
    ("menu_frame_relay_basic_cfg::menu_frame_relay_basic_cfg()\n"));

  snprintf(this->lxdialog_path, MAX_PATH_LENGTH, "%s", lxdialog_path);

  this->cfr = cfr;
  this->fr = &cfr->link_defs->linkconf->u.fr;
}

menu_frame_relay_basic_cfg::~menu_frame_relay_basic_cfg()
{
  Debug(DBG_MENU_AFT_LOGICAL_CHANNELS_LIST,
    ("menu_frame_relay_basic_cfg::~menu_frame_relay_basic_cfg()\n"));
}

char * get_fr_signalling_str(unsigned signalling)
{
  switch(signalling)
  {
  case WANOPT_FR_ANSI:
    return "ANSI";
    break;

  case WANOPT_FR_Q933:
    return "Q933";
    break;

  case WANOPT_FR_LMI:
    return "LMI";
    break;

  default:
    return "Invalid Signalling";
  }
}

int menu_frame_relay_basic_cfg::run(OUT int * selection_index, IN char* name_of_underlying_layer)
{
  string menu_str;
  int rc;
  char tmp_buff[MAX_PATH_LENGTH];
  char exit_dialog;

  //help text box
  text_box tb;

  menu_frame_relay_signalling fr_signalling(lxdialog_path, fr);
  menu_frame_relay_dlci_list dlci_lst(lxdialog_path, cfr);
  menu_frame_relay_advanced_global_configuration fr_global_cfg(lxdialog_path, cfr);

  Debug(DBG_MENU_AFT_LOGICAL_CHANNELS_LIST, ("menu_new_device_configuration::run()\n"));

again:
  rc = YES;
  exit_dialog = NO;
  menu_str = " ";

  //////////////////////////////////////////////////////////////////////////////////////
  snprintf(tmp_buff, MAX_PATH_LENGTH, " \"%d\" ", FR_SIGNALLING);
  menu_str += tmp_buff;
  snprintf(tmp_buff, MAX_PATH_LENGTH, " \"Signalling--------> %s\" ",
    get_fr_signalling_str(fr->signalling));
  menu_str += tmp_buff;

  //////////////////////////////////////////////////////////////////////////////////////
  snprintf(tmp_buff, MAX_PATH_LENGTH, " \"%d\" ", DLCI_NUMBERING);
  menu_str += tmp_buff;
  snprintf(tmp_buff, MAX_PATH_LENGTH, " \"DLCI Numbering----> %s\" ",
    (strcmp((char*)fr->dlci, "auto") == 0 ? "Auto (Single DLCI)" : "Manual"));
  menu_str += tmp_buff;

  //////////////////////////////////////////////////////////////////////////////////////
  snprintf(tmp_buff, MAX_PATH_LENGTH, " \"%d\" ", DLCI_LIST);
  menu_str += tmp_buff;
  snprintf(tmp_buff, MAX_PATH_LENGTH, " \"DLCI Configuration\" ");
  menu_str += tmp_buff;

  //////////////////////////////////////////////////////////////////////////////////////
  snprintf(tmp_buff, MAX_PATH_LENGTH, " \"%d\" ", EMPTY_LINE);
  menu_str += tmp_buff;
  //create the menu item
  snprintf(tmp_buff, MAX_PATH_LENGTH, " \" \" ");
  menu_str += tmp_buff;

  //////////////////////////////////////////////////////////////////////////////////////
  snprintf(tmp_buff, MAX_PATH_LENGTH, " \"%d\" ", ADVANCED_GLOBAL_FR_CFG);
  menu_str += tmp_buff;
  //create the menu item
  snprintf(tmp_buff, MAX_PATH_LENGTH, " \"Advanced Frame Relay Configuration\" ");
  menu_str += tmp_buff;

  //////////////////////////////////////////////////////////////////////////////////////
  //create the explanation text for the menu
  snprintf(tmp_buff, MAX_PATH_LENGTH,
"\n------------------------------------------\
\n      Frame Relay configuration options for\
\nWan Device: %s", cfr->link_defs->name);

  if(set_configuration(   YES,//indicates to call V2 of the function
                          MENU_BOX_BACK,//MENU_BOX_SELECT,
                          lxdialog_path,
                          "FRAME RELAY CONFIGURATION",
                          WANCFG_PROGRAM_NAME,
                          tmp_buff,
                          MENU_HEIGTH, MENU_WIDTH,
                          5,//number of items in the menu, including the empty lines
                          (char*)menu_str.c_str()
                          ) == NO){
    rc = NO;
    goto cleanup;
  }

  if(show(selection_index) == NO){
    rc = NO;
    goto cleanup;
  }

  //////////////////////////////////////////////////////////////////////////////////////

  exit_dialog = NO;
  switch(*selection_index)
  {
  case MENU_BOX_BUTTON_SELECT:
    Debug(DBG_MENU_AFT_LOGICAL_CHANNELS_LIST,
      ("option selected for editing: %s\n", get_lxdialog_output_string()));

    switch(atoi(get_lxdialog_output_string()))
    {
    case FR_SIGNALLING:
        fr_signalling.run(selection_index);
      break;

    case DLCI_NUMBERING:
      {
        menu_frame_relay_manual_or_auto_dlci_cfg dlci_cfg_mode( lxdialog_path,
                                                                cfr->link_defs->linkconf);
        if(dlci_cfg_mode.run(selection_index) == YES){

          //Debug(DBG_MENU_AFT_LOGICAL_CHANNELS_LIST,
          //  ("************ (char*)fr->dlci: %s\n", (char*)fr->dlci));

          if(strcmp((char*)fr->dlci, AUTO_CHAN_CFG) != 0){
            //manual configuration was selected.
            //make sure:  1. number of DLCIs is entered
            //            2. for each DLCI unique number is entered
            if(dlci_lst.run(selection_index, name_of_underlying_layer) == NO){
              rc = NO;
              goto cleanup;
            }
          }else{
            //auto was selected. adjust number of elements in the list
            //by running the dialog. it is better than running
            //'dlci_lst.adjust_number_of_logical_channels_in_list()' directly.
            if(dlci_lst.run(selection_index, name_of_underlying_layer) == NO){
              rc = NO;
              goto cleanup;
            }
          }
          goto again;
        }else{
          rc = NO;
          goto cleanup;
        }
      }
      break;

    case DLCI_LIST://dlci list to get to per-dlci configuration
        dlci_lst.run(selection_index, name_of_underlying_layer);
      break;

    case ADVANCED_GLOBAL_FR_CFG:
      fr_global_cfg.run(selection_index);
      break;

    case EMPTY_LINE:
      //do nothing
      break;

    default:
      ERR_DBG_OUT(("Invalid option selected for editing!! selection: %s\n",
        get_lxdialog_output_string()));
      rc = NO;
    }
    break;

  case MENU_BOX_BUTTON_HELP:
    switch(atoi(get_lxdialog_output_string()))
    {
    case FR_SIGNALLING:
      tb.show_help_message(lxdialog_path, NO_PROTOCOL_NEEDED, fr_signalling_help_str);
      break;

    case DLCI_NUMBERING:
      tb.show_help_message(lxdialog_path, NO_PROTOCOL_NEEDED, fr_dlci_numbering_help_str);
      break;

    case ADVANCED_GLOBAL_FR_CFG:
      tb.show_help_message(lxdialog_path, NO_PROTOCOL_NEEDED, fr_advanced_cfg_options_help_str);
      break;

    case DLCI_LIST:
      tb.show_help_message(lxdialog_path, NO_PROTOCOL_NEEDED, fr_dlci_list_help_str);
      break;

    default:
      tb.show_help_message(lxdialog_path, NO_PROTOCOL_NEEDED, option_not_implemented_yet_help_str);
      break;
    }
    break;

  case MENU_BOX_BUTTON_EXIT:
    exit_dialog = YES;
    break;
  }//switch(*selection_index)

  if(exit_dialog == NO){
    goto again;
  }
cleanup:
  return rc;

}

int menu_frame_relay_basic_cfg::run_aft_mp_fr_cfg(OUT int * selection_index)
{
  string menu_str;
  int rc;
  char tmp_buff[MAX_PATH_LENGTH];
  char exit_dialog;
  int new_dlci;
  int old_dlci;
  char tmp_addr[MAX_ADDR_STR_LEN];

  //help text box
  text_box tb;

  menu_frame_relay_signalling fr_signalling(lxdialog_path, fr);
  menu_frame_relay_dlci_list dlci_lst(lxdialog_path, cfr);
  input_box_frame_relay_DLCI_number dlci_num(&new_dlci);

  Debug(DBG_MENU_AFT_LOGICAL_CHANNELS_LIST,
    ("menu_frame_relay_basic_cfg::run_aft_mp_fr_cfg()\n"));

again:
  rc = YES;
  exit_dialog = NO;
  menu_str = " ";

  wandev_conf_t *linkconf = cfr->link_defs->linkconf;
  if(linkconf == NULL){
    return NO;
  }

  //////////////////////////////////////////////////////////////////////////////////////
  snprintf(tmp_buff, MAX_PATH_LENGTH, " \"%d\" ", FR_SIGNALLING);
  menu_str += tmp_buff;
  snprintf(tmp_buff, MAX_PATH_LENGTH, " \"Signalling-----> %s\" ",
    get_fr_signalling_str(fr->signalling));
  menu_str += tmp_buff;

  //////////////////////////////////////////////////////////////////////////////////////
  snprintf(tmp_buff, MAX_PATH_LENGTH, " \"%d\" ", MP_FR_STATION_TYPE);
  menu_str += tmp_buff;
  //create the menu item
  snprintf(tmp_buff, MAX_PATH_LENGTH, " \"Station--------> %s\" ",
    (linkconf->u.fr.station == WANOPT_NODE ? "Node (switch emulation)" : "CPE"));
  menu_str += tmp_buff;

  //////////////////////////////////////////////////////////////////////////////////////
  snprintf(tmp_buff, MAX_PATH_LENGTH, " \"%d\" ", MP_FR_DLCI_NUMBERING);
  menu_str += tmp_buff;
  snprintf(tmp_buff, MAX_PATH_LENGTH, " \"DLCI Numbering-> %s\" ",
    (strcmp((char*)fr->dlci, "auto") == 0 ? "Auto" : "Manual"));
  menu_str += tmp_buff;

  //////////////////////////////////////////////////////////////////////////////////////
  list_element_chan_def* list_el_chan_def =
    (list_element_chan_def*)cfr->obj_list->get_first();

  if(list_el_chan_def == NULL){
    ERR_DBG_OUT(("Invalid pointer!!\n"));
    return NO;
  }

  if(strcmp((char*)fr->dlci, AUTO_CHAN_CFG) != 0){

    if(strcmp(list_el_chan_def->data.name, CHAN_NOT_CONFIGURED) == 0){

      snprintf(tmp_buff, MAX_PATH_LENGTH, " \"%d\" ", MP_FR_DLCI_NUMBER);
      menu_str += tmp_buff;
      snprintf(tmp_buff, MAX_PATH_LENGTH, " \"DLCI Unconfigured!\" ");
      menu_str += tmp_buff;

    }else{
      if(strcmp(list_el_chan_def->data.addr, AUTO_CHAN_CFG) != 0){

        snprintf(tmp_buff, MAX_PATH_LENGTH, " \"%d\" ", MP_FR_DLCI_NUMBER);
        menu_str += tmp_buff;
        snprintf(tmp_buff, MAX_PATH_LENGTH, " \"DLCI %s\" ", list_el_chan_def->data.addr);
        menu_str += tmp_buff;
      }else{
        ERR_DBG_OUT(("DLCI number set to 'auto' instead of a valid number!!\n"));
      }
    }
  }else{
    //do not display Auto dlci
  }

  //////////////////////////////////////////////////////////////////////////////////////
  //create the explanation text for the menu
  snprintf(tmp_buff, MAX_PATH_LENGTH,
"\n------------------------------------------\
\n      Frame Relay configuration options for\
\nWan Device: %s", cfr->link_defs->name);

  if(set_configuration(   YES,//indicates to call V2 of the function
                          MENU_BOX_BACK,//MENU_BOX_SELECT,
                          lxdialog_path,
                          "FRAME RELAY CONFIGURATION",
                          WANCFG_PROGRAM_NAME,
                          tmp_buff,
                          MENU_HEIGTH, MENU_WIDTH,
                          4,//number of items in the menu, including the empty lines
                          (char*)menu_str.c_str()
                          ) == NO){
    rc = NO;
    goto cleanup;
  }

  if(show(selection_index) == NO){
    rc = NO;
    goto cleanup;
  }

  //////////////////////////////////////////////////////////////////////////////////////

  exit_dialog = NO;
  switch(*selection_index)
  {
  case MENU_BOX_BUTTON_SELECT:
    Debug(DBG_MENU_AFT_LOGICAL_CHANNELS_LIST,
      ("option selected for editing: %s\n", get_lxdialog_output_string()));

    switch(atoi(get_lxdialog_output_string()))
    {
    case FR_SIGNALLING:
      fr_signalling.run(selection_index);
      break;

    case MP_FR_STATION_TYPE:
//////////////////////////////////////////
      if(linkconf->u.fr.station == WANOPT_NODE){
        //current setting is NODE.
        snprintf(tmp_buff, MAX_PATH_LENGTH, "Do you want to change Station type to 'CPE'? ");

        if(yes_no_question(   selection_index,
                              lxdialog_path,
                              WANCONFIG_FR,
                              tmp_buff) == NO){
          return NO;
        }

        switch(*selection_index)
        {
        case YES_NO_TEXT_BOX_BUTTON_YES:
          linkconf->u.fr.station = WANOPT_CPE;
          break;
        }
        goto again;

      }else{
        //current setting is CPE.
        snprintf(tmp_buff, MAX_PATH_LENGTH,
          "Do you want to change Station type to\n'Node (switch emulation)'? ");

        if(strcmp((char*)fr->dlci, AUTO_CHAN_CFG) == 0){
          tb.show_error_message(lxdialog_path, WANCONFIG_FR,
"DLCI List configuration is set to 'Auto'.\n"
"Can not change Station type to 'Node (switch emulation)'.\n"
);
          goto again;
        }

        if(yes_no_question(   selection_index,
                              lxdialog_path,
                              WANCONFIG_FR,
                              tmp_buff) == NO){
          return NO;
        }

        switch(*selection_index)
        {
        case YES_NO_TEXT_BOX_BUTTON_YES:
          linkconf->u.fr.station = WANOPT_NODE;
          break;
        }
        goto again;
      }
//////////////////////////////////////////
      break;

    case MP_FR_DLCI_NUMBERING:
      {
        menu_frame_relay_manual_or_auto_dlci_cfg dlci_cfg_mode( lxdialog_path,
                                                                cfr->link_defs->linkconf);
        if(dlci_cfg_mode.run(selection_index) == YES){

          //Debug(DBG_MENU_AFT_LOGICAL_CHANNELS_LIST,
          //  ("************ (char*)fr->dlci: %s\n", (char*)fr->dlci));

          if(strcmp((char*)fr->dlci, AUTO_CHAN_CFG) != 0){
            //manual configuration was selected.
            //make sure:  1. number of DLCIs is entered
            //            2. for each DLCI unique number is entered
            if(atoi(list_el_chan_def->data.addr) == 0){
              //must be a valid number (16 ...)
              strcpy(list_el_chan_def->data.name, CHAN_NOT_CONFIGURED);
            }
          }else{
            //auto dlci was selected
            strcpy((char*)fr->dlci, AUTO_CHAN_CFG);
            strcpy(list_el_chan_def->data.addr, AUTO_CHAN_CFG);
            strcpy(list_el_chan_def->data.name, AUTO_CHAN_CFG);
          }

          goto again;
        }else{
          rc = NO;
          goto cleanup;
        }
      }
      break;

    case MP_FR_DLCI_NUMBER:
      old_dlci = atoi(remove_spaces_in_int_string(list_el_chan_def->data.addr));
      Debug(DBG_MENU_AFT_LOGICAL_CHANNELS_LIST, ("old_dlci: %d\n", old_dlci));

      //suggest DLCI number.
      if(strcmp(list_el_chan_def->data.name, CHAN_NOT_CONFIGURED) == 0){
        strcpy(list_el_chan_def->data.addr, "16");
      }
      snprintf(tmp_addr, MAX_ADDR_STR_LEN, "%s", list_el_chan_def->data.addr);

      if(dlci_num.show( lxdialog_path,
                        WANCONFIG_FR,
                        tmp_addr,
                        selection_index) == NO){
        goto cleanup;
      }

      //new_dlci = atoi(tmp_addr);
      snprintf(tmp_addr, MAX_ADDR_STR_LEN, "%d", new_dlci);

      Debug(DBG_MENU_AFT_LOGICAL_CHANNELS_LIST,
        ("on return from 'dlci number input box': %s\n", tmp_addr));

      //reset the "unconfigured" flag
      if(strcmp(list_el_chan_def->data.name, CHAN_NOT_CONFIGURED) == 0){
        //create name for the new interface. can be changed later by the user in Net Interface cfg.
        sprintf(list_el_chan_def->data.name, "wp%sfr%d", "0", new_dlci);
      }

      snprintf(list_el_chan_def->data.addr, MAX_ADDR_STR_LEN, "%d", new_dlci);
      break;

    default:
      ERR_DBG_OUT(("Invalid option selected for editing!! selection: %s\n",
        get_lxdialog_output_string()));
      rc = NO;
    }
    break;

  case MENU_BOX_BUTTON_HELP:
    switch(atoi(get_lxdialog_output_string()))
    {
    case FR_SIGNALLING:
      tb.show_help_message(lxdialog_path, NO_PROTOCOL_NEEDED, fr_signalling_help_str);
      break;

    case MP_FR_STATION_TYPE:
      tb.show_help_message(lxdialog_path, NO_PROTOCOL_NEEDED, fr_station_type_help_str);
      break;

    case MP_FR_DLCI_NUMBERING:
      tb.show_help_message(lxdialog_path, NO_PROTOCOL_NEEDED, fr_dlci_numbering_help_str);
      break;

    case MP_FR_DLCI_NUMBER:
      tb.show_help_message(lxdialog_path, NO_PROTOCOL_NEEDED,
        "A unique DLCI number should be provided.");
      break;

    default:
      tb.show_help_message(lxdialog_path, NO_PROTOCOL_NEEDED, option_not_implemented_yet_help_str);
      break;
    }
    break;

  case MENU_BOX_BUTTON_EXIT:

    if(strcmp(list_el_chan_def->data.name, CHAN_NOT_CONFIGURED) == 0){
      tb.show_error_message(lxdialog_path, WANCONFIG_FR,
                            "DLCI configuration is incomplete.\nPlease specify DLCI number.");
    }else{
      exit_dialog = YES;
    }
    break;
  }//switch(*selection_index)

  if(exit_dialog == NO){
    goto again;
  }
cleanup:
  return rc;
}
