/***************************************************************************
                          menu_frame_relay_dlci_list.cpp  -  description
                             -------------------
    begin                : Mon Mar 15 2004
    copyright            : (C) 2004 by David Rokhvarg
    email                : davidr@sangoma.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "menu_frame_relay_dlci_list.h"
#include "text_box.h"

#include "input_box_number_of_dlcis.h"
#include "list_element_frame_relay_dlci.h"
#include "menu_frame_relay_dlci_configuration.h"


enum DLCI_LIST_OPTIONS{
  AUTO_DLCI=2,
  NUMBER_OF_DLCIS
};

char* fr_number_of_dlcis_help_str =
"Number of DLCIs supported\n"
"-------------------------\n"
"\n"
"Number of DLCIs supported by this Frame\n"
"Relay connection.\n"
"\n"
"For each DLCI, a network interface will be\n"
"created.\n"
"\n"
"Each network interface must be configured with\n"
"an IP address.\n";

char* fr_per_dlci_cfg_options_help_str =
"Per-DLCI configuration options\n"
"------------------------------\n"
" -DLCI Number\n"
" -Advanced options: CIR, BC, BE, TX Inv. ARP,\n"
"  RX Inv. ARP\n";


#define DBG_MENU_FRAME_RELAY_DLCI_LIST  1

#define DLCI_NUM_NOT_SET 1

int check_global_frame_relay_configuration(wan_fr_conf_t* fr_global_cfg);

menu_frame_relay_dlci_list::menu_frame_relay_dlci_list( IN char * lxdialog_path,
                                                        IN conf_file_reader* cfr)
{
  Debug(DBG_MENU_FRAME_RELAY_DLCI_LIST,
    ("menu_frame_relay_dlci_list::menu_frame_relay_dlci_list()\n"));

  snprintf(this->lxdialog_path, MAX_PATH_LENGTH, "%s", lxdialog_path);

  this->cfr = cfr;
}

menu_frame_relay_dlci_list::~menu_frame_relay_dlci_list()
{
  Debug(DBG_MENU_FRAME_RELAY_DLCI_LIST,
    ("menu_frame_relay_dlci_list::~menu_frame_relay_dlci_list()\n"));
}

int menu_frame_relay_dlci_list::run(OUT int * selection_index, IN char* name_of_underlying_layer)
{
  string menu_str;
  int rc;
  char tmp_buff[MAX_PATH_LENGTH];
  char exit_dialog;
  int num_of_visible_items;

  //help text box
  text_box tb;

  list_element_frame_relay_dlci* list_el_chan_def = NULL;
  list_element_frame_relay_dlci* chan_def_tmp = NULL;

  objects_list * obj_list = cfr->obj_list;

  wan_fr_conf_t* fr = &cfr->link_defs->linkconf->u.fr;

  Debug(DBG_MENU_FRAME_RELAY_DLCI_LIST, ("menu_frame_relay_dlci_list::run()\n"));

again:
  menu_str = " ";
  rc = YES;
  exit_dialog = NO;

  if(obj_list ==  NULL){
    ERR_DBG_OUT(("Channels list pointer is NULL (obj_list) !!!\n"));
    rc = NO;
    goto cleanup;
  }

  //
  //if AUTO_CHAN_CFG dlci, only one dlci should be in the list, not less, not more.
  //
  if(strcmp((char*)fr->dlci, AUTO_CHAN_CFG) == 0){
    fr->dlci_num = 1;
  }

  if(adjust_number_of_logical_channels_in_list(obj_list, fr->dlci_num) == NO){

    Debug(DBG_MENU_FRAME_RELAY_DLCI_LIST,
      ("adjust_number_of_logical_channels_in_list() - failed!!"));
    rc = NO;
    goto cleanup;
  }

  menu_str = " ";
  if(strcmp((char*)fr->dlci, AUTO_CHAN_CFG) == 0){

    list_el_chan_def = (list_element_frame_relay_dlci*)obj_list->get_first();
    if(list_el_chan_def == NULL){
      ERR_DBG_OUT(("Empty DLCI list when \"auto\" DLCI is used!!\n"));
      rc = NO;
      goto cleanup;
    }

    if(obj_list->get_size() != 1){
      ERR_DBG_OUT(("Invalid number of DLCIs (%d)  when \"auto\" is used!!\n",
        obj_list->get_size()));
      rc = NO;
      goto cleanup;
    }

    //set the AUTO_CHAN_CFG indicator
    snprintf(list_el_chan_def->data.addr, MAX_ADDR_STR_LEN, AUTO_CHAN_CFG);
    //snprintf(list_el_chan_def->data.addr, MAX_ADDR_STR_LEN, CHAN_NOT_CONFIGURED);

    snprintf(list_el_chan_def->data.name, WAN_IFNAME_SZ, "wp%dfr", cfr->wanpipe_number);

    /////////////
    //for the 'auto' dlci go directly the 'dlci configuration page'.
    {
      menu_frame_relay_DLCI_configuration per_dlci_cfg(
                          lxdialog_path,
                          obj_list,
                          (list_element_frame_relay_dlci*)obj_list->get_first());
      rc = per_dlci_cfg.run(cfr->wanpipe_number, selection_index, name_of_underlying_layer);
      if(rc == YES){
        return YES;
      }else{
        return NO;
      }
    }
    ////////////
  }else{

    //non-auto case
    if(create_logical_channels_list_str(menu_str) == NO){
      return NO;
    }
  }//if()

  //////////////////////////////////////////////////////////////////////////////////////
  //create the explanation text for the menu
  snprintf(tmp_buff, MAX_PATH_LENGTH,
"------------------------------------------\
\nSelect DLCI you want to configure.");

  //adjust number of items in the menu depending on number of dlcis
  num_of_visible_items = fr->dlci_num + 1;
  //8 items looks best
  if(num_of_visible_items > 8){
    num_of_visible_items = 8;
  }

  if(set_configuration(   YES,//indicates to call V2 of the function
                          MENU_BOX_BACK,//MENU_BOX_SELECT,
                          lxdialog_path,
                          "FRAME RELAY DLCI LIST",
                          WANCFG_PROGRAM_NAME,
                          tmp_buff,
                          MENU_HEIGTH, MENU_WIDTH,
                          num_of_visible_items,//number of visible items in the menu
                          (char*)menu_str.c_str()
                          ) == NO){
    rc = NO;
    goto cleanup;
  }

  if(show(selection_index) == NO){
    rc = NO;
    goto cleanup;
  }

  switch(*selection_index)
  {
  case MENU_BOX_BUTTON_SELECT:
    Debug(DBG_MENU_FRAME_RELAY_DLCI_LIST,
      ("option selected for editing: %s\n", get_lxdialog_output_string()));

    switch(atoi(get_lxdialog_output_string()))
    {
    case AUTO_DLCI:   //auto config.
                      //config the one and only 'auto' dlci.
      {
        menu_frame_relay_DLCI_configuration per_dlci_cfg(
                            lxdialog_path,
                            obj_list,
                            (list_element_frame_relay_dlci*)obj_list->get_first());
        rc = per_dlci_cfg.run(cfr->wanpipe_number, selection_index, name_of_underlying_layer);
        if(rc == YES){
          goto again;
        }
      }
      break;

    case NUMBER_OF_DLCIS: //number of DLCIs if manual config.
                          //ask for number of DLCIs
      {
        input_box_number_of_dlcis ib_num_of_dlcis((int*)&fr->dlci_num);

        //convert int to string
        if(fr->dlci_num == 0){
          //suggest 1 dlci
          sprintf(tmp_buff, "%d", 1);
        }else{
          sprintf(tmp_buff, "%d", fr->dlci_num);
        }

        ib_num_of_dlcis.show(  lxdialog_path,
                                WANCONFIG_FR,
                                tmp_buff,//IN char * initial_text,
                                selection_index);

        if(fr->dlci_num == 0){
          strcpy((char*)fr->dlci, AUTO_CHAN_CFG);
          fr->dlci_num = 1;
        }
        goto again;
      }
      break;

    default:
      //one of the DLCIs was selected for configuration
      Debug(DBG_MENU_FRAME_RELAY_DLCI_LIST, ("default case selection: %d\n", *selection_index));

      list_el_chan_def = (list_element_frame_relay_dlci*)
                            obj_list->find_element(
                              remove_spaces_in_int_string(get_lxdialog_output_string()));

      if(list_el_chan_def != NULL){

        menu_frame_relay_DLCI_configuration per_dlci_cfg( lxdialog_path,
                                                          obj_list,
                                                          list_el_chan_def);
        //list_el_chan_def = NULL;
        //do NOT use list_el_chan_def on return - it may be invalid!
        rc = per_dlci_cfg.run(cfr->wanpipe_number, selection_index, name_of_underlying_layer);
        if(rc == YES){
          goto again;
        }
      }

#if 0
      ERR_DBG_OUT(("Invalid option selected for editing!! selection: %s\n",
        get_lxdialog_output_string()));
      rc = NO;
#endif

    }
    break;

  case MENU_BOX_BUTTON_HELP:
    switch(atoi(get_lxdialog_output_string()))
    {
    case AUTO_DLCI:

      break;

    case NUMBER_OF_DLCIS:
      tb.show_help_message(lxdialog_path, NO_PROTOCOL_NEEDED, fr_number_of_dlcis_help_str);
      break;

    default:
      tb.show_help_message(lxdialog_path, NO_PROTOCOL_NEEDED, fr_per_dlci_cfg_options_help_str);
    }
    goto again;
    break;

  case MENU_BOX_BUTTON_EXIT:

    //copy DLCI numbers from per-dlci structure in to the global
    //frame relay configuration structure.

    if(strcmp((char*)fr->dlci, AUTO_CHAN_CFG) != 0){
      chan_def_tmp = (list_element_frame_relay_dlci*)obj_list->get_first();
      int i = 0;
      while(chan_def_tmp != NULL){

        if(strcmp(chan_def_tmp->data.name, CHAN_NOT_CONFIGURED)){
          fr->dlci[i++] = atoi(chan_def_tmp->data.addr);
        }else{
          fr->dlci[i++] = DLCI_NUM_NOT_SET;
        }
        chan_def_tmp = (list_element_frame_relay_dlci*)obj_list->get_next_element(chan_def_tmp);
      }
    }

    //check configuration in the temporary structure.
    //if passed the check save it in the real structure.
    if(check_global_frame_relay_configuration(fr) != 0){

      //if failed, prompt user that "Configuration is invalid or incomplete..." and
      //redisplay the dialog.
      text_box tb;
      tb.show_error_message(lxdialog_path, WANCONFIG_FR,
                            "Configuration is invalid or incomplete..");
      goto again;
    }

    exit_dialog = YES;
    break;
  }//switch(*selection_index)

cleanup:
  return rc;
}


int check_global_frame_relay_configuration(wan_fr_conf_t* fr_global_cfg)
{
  if(fr_global_cfg->dlci_num < 1 || fr_global_cfg->dlci_num > MAX_DLCI_NUMBER){
    Debug(DBG_MENU_FRAME_RELAY_DLCI_LIST, ("invalid number of DLCIs: %d\n", fr_global_cfg->dlci_num));
    return 1;
  }

  if(strcmp((char*)fr_global_cfg->dlci, AUTO_CHAN_CFG) != 0){

    //check that there are NO unnmbered DLCIs
    for(unsigned int i = 0; i < fr_global_cfg->dlci_num; i++){

      Debug(DBG_MENU_FRAME_RELAY_DLCI_LIST,
        ("fr_global_cfg->dlci[%d]: %d\n", i, fr_global_cfg->dlci[i]));

      if(fr_global_cfg->dlci[i] == DLCI_NUM_NOT_SET){
        return 2;
      }
    }
  }

  return 0;
}

int menu_frame_relay_dlci_list::create_logical_channels_list_str(string& menu_str)
{
  char tmp_buff[MAX_PATH_LENGTH];

  unsigned int max_valid_number_of_logical_channels;
  sdla_te_cfg_t* te_cfg = &cfr->link_defs->linkconf->te_cfg;

  max_valid_number_of_logical_channels = MAX_DLCI_NUMBER;

  objects_list * obj_list = cfr->obj_list;
  list_element_frame_relay_dlci* list_el_chan_def = NULL;


  snprintf(tmp_buff, MAX_PATH_LENGTH, " \"%d\" ", NUMBER_OF_DLCIS);
  menu_str += tmp_buff;
  if(obj_list->get_size() > 0 && obj_list->get_size() <= max_valid_number_of_logical_channels){

    snprintf(tmp_buff, MAX_PATH_LENGTH, " \"Number of DLCI(s) ---> %u\" ", obj_list->get_size());
    menu_str += tmp_buff;

    list_el_chan_def = (list_element_frame_relay_dlci*)obj_list->get_first();
    //int i = 0;
    while(list_el_chan_def != NULL){

      snprintf(tmp_buff, MAX_PATH_LENGTH, " \"%s\" ", list_el_chan_def->data.addr);
      menu_str += tmp_buff;

      printf("list_el_chan_def->data.name: %s\n", list_el_chan_def->data.name);
      if(strcmp(list_el_chan_def->data.name, CHAN_NOT_CONFIGURED) == 0){
        printf("found CHAN_NOT_CONFIGURED\n");
      }

      if(strcmp(list_el_chan_def->data.name, CHAN_NOT_CONFIGURED)){

        snprintf(tmp_buff, MAX_PATH_LENGTH, " \"DLCI %s\" ", list_el_chan_def->data.addr);
      }else{

        snprintf(tmp_buff, MAX_PATH_LENGTH, " \"DLCI Unconfigured!\" ");
      }

      menu_str += tmp_buff;

      list_el_chan_def =
        (list_element_frame_relay_dlci*)obj_list->get_next_element(list_el_chan_def);
    }

  }else if(obj_list->get_size() == 0){

      snprintf(tmp_buff, MAX_PATH_LENGTH, " \"Number of DLCI(s) ---> Undefined\" ");
      menu_str += tmp_buff;

  }else{

    ERR_DBG_OUT(("Invalid number of DLCIs: %d!!\n", obj_list->get_size()));
    return NO;
  }

  return YES;
}

int menu_frame_relay_dlci_list::
  adjust_number_of_logical_channels_in_list(IN objects_list * obj_list,
                                            IN unsigned int new_number_of_logical_channels)
{
  int rc = YES;
  list_element_chan_def* chan_def_tmp = NULL;
  list_element_chan_def* list_el_chan_def = NULL;

#if 1
  //
  //Adjust number of elements in the list to the number of DLCIs in global config.
  //
  if(new_number_of_logical_channels > obj_list->get_size()){

    //more DLCIs should be added.
    while(obj_list->get_size() != new_number_of_logical_channels &&
          obj_list->get_size() <= MAX_DLCI_NUMBER){

      chan_def_tmp = new list_element_chan_def();

      list_el_chan_def = (list_element_chan_def*)obj_list->get_last();

      if(list_el_chan_def != NULL){
        //list is NOT empty.
        //create the temporary addr str. list does not accept the same addr twice so increment
        //the number for each next one.
        snprintf(chan_def_tmp->data.addr, MAX_ADDR_STR_LEN, "%d",
                                          atoi(list_el_chan_def->data.addr) + 1);
      }else{
        //list is empty. set to default
        snprintf(chan_def_tmp->data.addr, MAX_ADDR_STR_LEN, "%d", 16);
      }

      obj_list->insert(chan_def_tmp);
    }
  }else if(new_number_of_logical_channels < obj_list->get_size()){

    //remove unneeded DLCIs (from the end of the list)
    while(obj_list->get_size() != new_number_of_logical_channels &&
          (chan_def_tmp = (list_element_chan_def*)obj_list->remove_from_head()) != NULL){
      delete chan_def_tmp;
    }
  }else{
    //there is nothing to adjust.
    Debug(DBG_MENU_FRAME_RELAY_DLCI_LIST,
      ("adjust_number_of_logical_channels_in_list(): old == new  == %d.\n",
      obj_list->get_size()));
  }
#endif

  return rc;
}
