/***************************************************************************
                          menu_hdlc_streaming_configuration.cpp  -  description
                             -------------------
    begin                : Mon Apr 5 2004
    copyright            : (C) 2004 by David Rokhvarg
    email                : davidr@sangoma.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "menu_hdlc_streaming_configuration.h"
#include "text_box.h"
#include "text_box_yes_no.h"
#include "menu_hdlc_streaming_connection_type.h"
#include "menu_hdlc_streaming_line_coding.h"
#include "menu_hdlc_streaming_line_idle.h"

extern char* chdlc_ignore_dcd_help_str;
extern char* chdlc_ignore_cts_help_str;

char* hdlc_rx_only_help_str =
"RECEIVE ONLY\n"
"\n"
"This parameter enables Receive only communications.\n"
"In this mode of operation no transmission is possible for\n"
"both primary and secondary CHDLC ports.\n"
"\n"
"When in receive only mode, both ports can run at 2.75 Mbps\n"
"maximum respectively.\n"
"\n"
"Note: Receive Only Mode, applies to S514 PCI cards\n"
"      exclusively.\n"
"\n"
"Default is NO.\n";

char* hdlc_connection_type_help_str =
"HDLC CONNECTION TYPE/OPERATION\n"
"\n"
"The HDLC firmware can operate in two\n"
"modes: Permanent and Switched.\n"
"\n"
" -Permanent:  The RTS and DTS are kept high during\n"
"Tx and Rx of data. For most connections\n"
"this is a default behaviour. It supports\n"
"full duplex operatoin.\n"
"\n"
" -Switched:   The RTS is raised only during Tx of data.\n"
"This mode is usually for half duplex lines,\n"
"for example a terminal that is connected\n"
"to a mainframe or a satelite.\n"
"\n"
"Default: Permanent\n";

char* hdlc_line_coding_help_str =
"HDLC PHYSICAL LINE CODING\n"
"\n"
"The two coding standards are\n"
"NRZ and NRZI.  Please consult you ISP on which\n"
"encoding you require.  By default most todays lines\n"
"use NRZ (default) encoding.  Thus if unsure leave\n"
"as default.\n"
"\n"
"Default: NRZ\n";

char* hdlc_line_idle_help_str =
"HDLC PHYSICAL LINE IDLE\n"
"\n"
"Between each Tx frame, the hardware\n"
"either idles a FLAG or MARK.  By default almost\n"
"all todays physical lines idle FALG, however in\n"
"some instances like satelite links, where\n"
"bandwidth is an issue, MARK might be used during\n"
"line idle.\n"
"\n"
"Default: FLAG\n";


enum HDLC_STREAMING_CFG_OPTIONS{

  //Ignore DCD  --------------> NO          ? ?
  IGNORE_DCD=1,
  //Ignore CTS  --------------> NO          ? ?
  IGNORE_CTS,

  //Receive Only ---> NO
  RX_ONLY,
  //Connection -----> Permanent
  CONNECTION_TYPE,
  //Line Coding ----> NRZ
  LINE_CODING,
  //Line Idle ------> Flag
  LINE_IDLE,
};

#define DBG_MENU_HDLC_STREAMING_CONFIGURATION  1

menu_hdlc_streaming_configuration::menu_hdlc_streaming_configuration( IN char * lxdialog_path,
                                                                      IN conf_file_reader* cfr)
{
  Debug(DBG_MENU_HDLC_STREAMING_CONFIGURATION,
    ("menu_hdlc_streaming_configuration::menu_hdlc_streaming_configuration()\n"));

  snprintf(this->lxdialog_path, MAX_PATH_LENGTH, "%s", lxdialog_path);
  this->cfr = cfr;
}

menu_hdlc_streaming_configuration::~menu_hdlc_streaming_configuration()
{
  Debug(DBG_MENU_HDLC_STREAMING_CONFIGURATION,
    ("menu_hdlc_streaming_configuration::~menu_hdlc_streaming_configuration()\n"));
}

int menu_hdlc_streaming_configuration::run(OUT int * selection_index)
{
  string menu_str;
  int rc;
  char tmp_buff[MAX_PATH_LENGTH];
  char exit_dialog = NO;

  //help text box
  text_box tb;

//  link_def_t * link_def = cfr->link_defs;
  wandev_conf_t *linkconf = cfr->link_defs->linkconf;
//  wan_chdlc_conf_t* chdlc_cfg = &cfr->link_defs->linkconf->u.chdlc;

  list_element_chan_def* list_el_chan_def = NULL;
  objects_list * obj_list = NULL;

  Debug(DBG_MENU_HDLC_STREAMING_CONFIGURATION, ("menu_hdlc_streaming_configuration::run()\n"));

again:
  rc = YES;
  menu_str = " ";

  obj_list = cfr->obj_list;
  if(obj_list == NULL){
    ERR_DBG_OUT(("The 'obj_list' pointer is NULL!!\n"));
    return NO;
  }

  //NOTE: protocol parameters saved in the Interface cofiguration - because
  //of the legacy inherited structure. All of these should be at the commented out
  //'wan_chdlc_conf_t* chdlc_cfg' pointer. For later fix.
  list_el_chan_def = (list_element_chan_def*)obj_list->get_first();
  if(list_el_chan_def == NULL){
    ERR_DBG_OUT(("Interface pointer is NULL!!\n"));
    return NO;
  }

  //////////////////////////////////////////////////////////////////////////////////////
  snprintf(tmp_buff, MAX_PATH_LENGTH, " \"%d\" ", IGNORE_DCD);
  menu_str += tmp_buff;
  snprintf(tmp_buff, MAX_PATH_LENGTH, " \"Ignore DCD------> %s\" ",
    (list_el_chan_def->data.chanconf->ignore_dcd == 0 ? "NO" : "YES"));
  menu_str += tmp_buff;

  //////////////////////////////////////////////////////////////////////////////////////
  snprintf(tmp_buff, MAX_PATH_LENGTH, " \"%d\" ", IGNORE_CTS);
  menu_str += tmp_buff;
  snprintf(tmp_buff, MAX_PATH_LENGTH, " \"Ignore CTS------> %s\" ",
    (list_el_chan_def->data.chanconf->ignore_cts == 0 ? "NO" : "YES"));
  menu_str += tmp_buff;

  //////////////////////////////////////////////////////////////////////////////////////
  snprintf(tmp_buff, MAX_PATH_LENGTH, " \"%d\" ", RX_ONLY);
  menu_str += tmp_buff;
  snprintf(tmp_buff, MAX_PATH_LENGTH, " \"Receive Only----> %s\" ",
    (linkconf->receive_only == 0 ? "NO" : "YES"));
  menu_str += tmp_buff;

  //////////////////////////////////////////////////////////////////////////////////////
  snprintf(tmp_buff, MAX_PATH_LENGTH, " \"%d\" ", CONNECTION_TYPE);
  menu_str += tmp_buff;
  snprintf(tmp_buff, MAX_PATH_LENGTH, " \"Connection Type-> %s\" ",
    (linkconf->connection == 0 ? "Permanent" : "Switched RTS"));
  menu_str += tmp_buff;

  //////////////////////////////////////////////////////////////////////////////////////
  snprintf(tmp_buff, MAX_PATH_LENGTH, " \"%d\" ", LINE_CODING);
  menu_str += tmp_buff;
  snprintf(tmp_buff, MAX_PATH_LENGTH, " \"Line Coding-----> %s\" ",
    (linkconf->line_coding == 0 ? "NRZ" : "NRZI"));
  menu_str += tmp_buff;

  //////////////////////////////////////////////////////////////////////////////////////
  snprintf(tmp_buff, MAX_PATH_LENGTH, " \"%d\" ", LINE_IDLE);
  menu_str += tmp_buff;
  snprintf(tmp_buff, MAX_PATH_LENGTH, " \"Line Idle-------> %s\" ",
    (linkconf->line_idle == 0 ? "Flag" : "Mark"));
  menu_str += tmp_buff;

  //////////////////////////////////////////////////////////////////////////////////////
  //create the explanation text for the menu
  snprintf(tmp_buff, MAX_PATH_LENGTH,
"\n------------------------------------------\
\nHDLC Streaming configuration options for\
\nWan Device: %s", cfr->link_defs->name);

  if(set_configuration(   YES,//indicates to call V2 of the function
                          MENU_BOX_BACK,//MENU_BOX_SELECT,
                          lxdialog_path,
                          "HDLC STREAMING CONFIGURATION",
                          WANCFG_PROGRAM_NAME,
                          tmp_buff,
                          MENU_HEIGTH, MENU_WIDTH,
                          6,//number of items in the menu, including the empty lines
                          (char*)menu_str.c_str()
                          ) == NO){
    rc = NO;
    goto cleanup;
  }

  if(show(selection_index) == NO){
    rc = NO;
    goto cleanup;
  }

  //////////////////////////////////////////////////////////////////////////////////////

  exit_dialog = NO;
  switch(*selection_index)
  {
  case MENU_BOX_BUTTON_SELECT:
    Debug(DBG_MENU_HDLC_STREAMING_CONFIGURATION,
      ("option selected for editing: %s\n", get_lxdialog_output_string()));

    switch(atoi(get_lxdialog_output_string()))
    {
    case IGNORE_DCD:

      snprintf(tmp_buff, MAX_PATH_LENGTH,
"Do you want to %s DCD Monitoring?\n",
(list_el_chan_def->data.chanconf->ignore_dcd == 0 ?
"enable" : "disable"));

      if(yes_no_question(   selection_index,
                            lxdialog_path,
                            WANCONFIG_HDLC,
                            tmp_buff) == NO){
        rc = NO;
      }

      switch(*selection_index)
      {
      case YES_NO_TEXT_BOX_BUTTON_YES:
        if(list_el_chan_def->data.chanconf->ignore_dcd == 1){
          list_el_chan_def->data.chanconf->ignore_dcd = 0;
        }else{
          list_el_chan_def->data.chanconf->ignore_dcd = 1;
        }
        break;

      case YES_NO_TEXT_BOX_BUTTON_NO:
        break;
      }
      break;

    case IGNORE_CTS:

      snprintf(tmp_buff, MAX_PATH_LENGTH,
"Do you want to %s CTS Monitoring?\n",
(list_el_chan_def->data.chanconf->ignore_cts == 0 ?
"enable" : "disable"));

      if(yes_no_question(   selection_index,
                            lxdialog_path,
                            WANCONFIG_HDLC,
                            tmp_buff) == NO){
        rc = NO;
      }

      switch(*selection_index)
      {
      case YES_NO_TEXT_BOX_BUTTON_YES:
        if(list_el_chan_def->data.chanconf->ignore_cts == 1){
          list_el_chan_def->data.chanconf->ignore_cts = 0;
        }else{
          list_el_chan_def->data.chanconf->ignore_cts = 1;
        }
        break;

      case YES_NO_TEXT_BOX_BUTTON_NO:
        break;
      }
      break;

    case RX_ONLY:

      snprintf(tmp_buff, MAX_PATH_LENGTH,
"Do you want to %s Receive Oly Mode?\n",
(linkconf->receive_only == 0 ?
"enable" : "disable"));

      if(yes_no_question(   selection_index,
                            lxdialog_path,
                            WANCONFIG_HDLC,
                            tmp_buff) == NO){
        rc = NO;
      }

      switch(*selection_index)
      {
      case YES_NO_TEXT_BOX_BUTTON_YES:
        if(linkconf->receive_only == 1){
          linkconf->receive_only = 0;
        }else{
          linkconf->receive_only = 1;
        }
        break;

      case YES_NO_TEXT_BOX_BUTTON_NO:
        break;
      }
      break;

    case CONNECTION_TYPE:
      {
        menu_hdlc_streaming_connection_type hdlc_streaming_connection_type(lxdialog_path, cfr);
        if(hdlc_streaming_connection_type.run(selection_index) == NO){
          rc = NO;
          exit_dialog = YES;
        }
      }
      break;

    case LINE_CODING:
      {
        menu_hdlc_streaming_line_coding hdlc_streaming_line_coding(lxdialog_path, cfr);
        if(hdlc_streaming_line_coding.run(selection_index) == NO){
          rc = NO;
          exit_dialog = YES;
        }
      }
      break;

    case LINE_IDLE:
      {
        menu_hdlc_streaming_line_idle hdlc_streaming_line_idle(lxdialog_path, cfr);
        if(hdlc_streaming_line_idle.run(selection_index) == NO){
          rc = NO;
          exit_dialog = YES;
        }
      }
      break;

    default:
      ERR_DBG_OUT(("Invalid option selected for editing!! selection: %s\n",
        get_lxdialog_output_string()));
      rc = NO;
    }
    break;

  case MENU_BOX_BUTTON_HELP:

    switch(atoi(get_lxdialog_output_string()))
    {
    case IGNORE_DCD:
      tb.show_help_message(lxdialog_path, NO_PROTOCOL_NEEDED, chdlc_ignore_dcd_help_str);
      break;

    case IGNORE_CTS:
      tb.show_help_message(lxdialog_path, NO_PROTOCOL_NEEDED, chdlc_ignore_cts_help_str);
      break;

    case RX_ONLY:
      tb.show_help_message(lxdialog_path, NO_PROTOCOL_NEEDED, hdlc_rx_only_help_str);
      break;

    case CONNECTION_TYPE:
      tb.show_help_message(lxdialog_path, NO_PROTOCOL_NEEDED, hdlc_connection_type_help_str);
      break;

    case LINE_CODING:
      tb.show_help_message(lxdialog_path, NO_PROTOCOL_NEEDED, hdlc_line_coding_help_str);
      break;

    case LINE_IDLE:
      tb.show_help_message(lxdialog_path, NO_PROTOCOL_NEEDED, hdlc_line_idle_help_str);
      break;
    }//switch()
    break;

  case MENU_BOX_BUTTON_EXIT:
    exit_dialog = YES;
    break;
  }//switch(*selection_index)

  if(exit_dialog == NO){
    goto again;
  }
cleanup:
  return rc;
}
