/***************************************************************************
                          menu_list_existing_wanpipes.h  -  description
                             -------------------
    begin                : Wed Mar 3 2004
    copyright            : (C) 2004 by David Rokhvarg
    email                : davidr@sangoma.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef MENU_LIST_EXISTING_WANPIPES_H
#define MENU_LIST_EXISTING_WANPIPES_H

#define DBG_MENU_LIST_EXISTING_WANPIPES 1
#define DBG_AFT_READ  1

#include <menu_base.h>

#include "list_element_chan_def.h"
#include "objects_list.h"
#include "conf_file_reader.h"
#include "menu_select_and_configure_protocol.h"

/**List existing wanpipe#.conf files.
  *@author David Rokhvarg
  */

class menu_list_existing_wanpipes : public menu_base  {
  char lxdialog_path[MAX_PATH_LENGTH];
  char conf_file_path[MAX_PATH_LENGTH];

public:
static int check_for_LIP_layer( list_element_chan_def* list_el_chan_def,
                                objects_list* obj_list)
{
  list_element_chan_def* root_list_el_chan_def;
  objects_list* root_obj_list;
  list_element_chan_def* removed_list_el_chan_def;
  conf_file_reader* root_file_reader;
  list_element_chan_def* tmp_list_el_chan_def;
  unsigned int i;

  Debug(DBG_MENU_LIST_EXISTING_WANPIPES, ("list_el_chan_def->data.LIP_root_device: %p\n",
    list_el_chan_def->data.LIP_root_device));

  if(list_el_chan_def->data.LIP_root_device != NULL){

    root_list_el_chan_def = (list_element_chan_def*)list_el_chan_def->data.LIP_root_device;
  }else{
    return NO;
  }

  Debug(DBG_MENU_LIST_EXISTING_WANPIPES, ("root_list_el_chan_def->data.name: %s\n",
    root_list_el_chan_def->data.name));

  Debug(DBG_MENU_LIST_EXISTING_WANPIPES, ("root_list_el_chan_def->conf_file_reader: %p\n",
    root_list_el_chan_def->conf_file_reader));

  if(root_list_el_chan_def->conf_file_reader == NULL){

    Debug(DBG_MENU_LIST_EXISTING_WANPIPES, ("list_el_chan_def->data.LIP_protocol: %d\n",
      list_el_chan_def->data.LIP_protocol));


    root_list_el_chan_def->conf_file_reader =
      new conf_file_reader(
        menu_select_and_configure_protocol::get_template_file_path(
                                              list_el_chan_def->data.LIP_protocol));


    //STEP1:
    //read the template file.
    if(((conf_file_reader*)root_list_el_chan_def->conf_file_reader)->read_conf_file() != 0){
      ERR_DBG_OUT(("Failed to parse '%s' file!!",
        menu_select_and_configure_protocol::get_template_file_path(
                                              list_el_chan_def->data.LIP_protocol)));
      return NO;
    }

    //STEP2:
    //remove ALL interfaces from the new list
    root_obj_list = ((conf_file_reader*)root_list_el_chan_def->conf_file_reader)->obj_list;
    root_obj_list->remove_all_elements();
  }else{
    root_obj_list = ((conf_file_reader*)root_list_el_chan_def->conf_file_reader)->obj_list;
  }

  //STEP3:
  //insert channels into upper layer (LIP) list.
  removed_list_el_chan_def =
    (list_element_chan_def*)obj_list->remove_element(list_el_chan_def->get_comparator());

  Debug(DBG_MENU_LIST_EXISTING_WANPIPES, ("removed_list_el_chan_def->data.name: %s\n",
    removed_list_el_chan_def->data.name));

  root_obj_list->insert(removed_list_el_chan_def);

  root_file_reader = (conf_file_reader*)root_list_el_chan_def->conf_file_reader;

  root_file_reader->link_defs->config_id = removed_list_el_chan_def->data.LIP_protocol;

  //strcpy(root_file_reader->link_defs->name, "hello");
  //strcpy(root_file_reader->link_defs->name, list_el_chan_def->data.name);

  //FIXIT: from this point code assumes it is frame relay, because no other protocol is
  //imlemented in LIP layer right now.

  memcpy( &root_file_reader->link_defs->linkconf->u.fr,
          &removed_list_el_chan_def->data.chanconf->u.fr, sizeof(wan_fr_conf_t));

  root_file_reader->link_defs->linkconf->u.fr.dlci_num = root_obj_list->get_size();
  tmp_list_el_chan_def = (list_element_chan_def*)root_obj_list->get_first();

	if(atoi(removed_list_el_chan_def->data.addr) == 0){

    //indication that 'auto' will be used
    strcpy((char*)root_file_reader->link_defs->linkconf->u.fr.dlci, "auto");

    Debug(DBG_MENU_LIST_EXISTING_WANPIPES, ("removed_list_el_chan_def->data: %s is 'auto'\n",
      removed_list_el_chan_def->data.name));
	}else{

    for(i = 0; i < root_file_reader->link_defs->linkconf->u.fr.dlci_num; i++){

      if(tmp_list_el_chan_def != NULL){

        root_file_reader->link_defs->linkconf->u.fr.dlci[i] =
          atoi(tmp_list_el_chan_def->data.addr);

        tmp_list_el_chan_def =
          (list_element_chan_def*)root_obj_list->get_next_element(tmp_list_el_chan_def);
      }
    }//for()

    for(i = 0; i < root_file_reader->link_defs->linkconf->u.fr.dlci_num; i++){

      Debug(DBG_MENU_LIST_EXISTING_WANPIPES, ("root_file_reader->link_defs->linkconf->u.fr.dlci[%d]: %d\n",
        i, root_file_reader->link_defs->linkconf->u.fr.dlci[i]));
    }
  }//if()

  return YES;
}

//reduce channel numbers to minimum without braking the order in the list.
static int reduce_aft_channel_groups_numbers(conf_file_reader* cfr)
{
  objects_list* obj_list;
  int aft_channel_groups_counter = 1;
  list_element_chan_def* list_el_chan_def;

  obj_list = cfr->obj_list;

  if(obj_list == NULL){
    ERR_DBG_OUT(("Invalid pointer!\n"));
    return NO;
  }

  list_el_chan_def = (list_element_chan_def*)obj_list->get_first();
  while(list_el_chan_def != NULL){

    if(aft_channel_groups_counter > NUM_OF_E1_CHANNELS){
      ERR_DBG_OUT(("Invalid number of Channel Groups!\n"));
      return NO;
    }

    Debug(DBG_AFT_READ, ("old addr: %s\n",  list_el_chan_def->data.addr));

    sprintf(list_el_chan_def->data.addr, "%d", aft_channel_groups_counter);

    Debug(DBG_AFT_READ, ("new addr: %s\n",  list_el_chan_def->data.addr));

    aft_channel_groups_counter++;

    list_el_chan_def = (list_element_chan_def*)obj_list->get_next_element(list_el_chan_def);
  }

  return YES;
}

static int update_per_group_of_channels_iformation(conf_file_reader* wanpipe_cfr)
{
  objects_list* obj_list;
  list_element_chan_def* list_el_chan_def;
  int aft_serial_number = 0;

  Debug(DBG_AFT_READ, ("update_per_group_of_channels_iformation()\n"));
  Debug(DBG_AFT_READ, ("objects in the list: \n"));

  obj_list = wanpipe_cfr->obj_list;
  if(obj_list == NULL){
    ERR_DBG_OUT(("Invalid pointer!\n"));
    return NO;
  }

  list_el_chan_def = (list_element_chan_def*)obj_list->get_first();
  while(list_el_chan_def != NULL){
    Debug(DBG_AFT_READ, ("list_el_chan_def->data.name: %s\n",  list_el_chan_def->data.name));

    Debug(DBG_AFT_READ, ("list_el_chan_def: %p\n", list_el_chan_def));

    Debug(DBG_AFT_READ, ("list_el_chan_def->conf_file_reader: %p\n",
      list_el_chan_def->conf_file_reader));

    //increment counter even if there is NO LIP layer for this AFT interface.
    aft_serial_number++;
/*
    if(list_el_chan_def->conf_file_reader == NULL){
      list_el_chan_def = (list_element_chan_def*)obj_list->get_next_element(list_el_chan_def);
      continue;
    }
*/
    if(list_el_chan_def->conf_file_reader == NULL){
      //on AFT only WANCONFIG_HDLC does NOT have LIP layer.
      //but it is needed to simplify this program, so create it.

      list_el_chan_def->conf_file_reader =
        new conf_file_reader(
          menu_select_and_configure_protocol::get_template_file_path(WANCONFIG_HDLC));

      //STEP1:
      //read the template file.
      if(((conf_file_reader*)list_el_chan_def->conf_file_reader)->read_conf_file() != 0){

        ERR_DBG_OUT(("Failed to parse '%s' file!!",
          menu_select_and_configure_protocol::get_template_file_path(WANCONFIG_HDLC)));
      }

      list_el_chan_def->data.chanconf->protocol = WANCONFIG_HDLC;
    }

    Debug(DBG_AFT_READ,
      ("((conf_file_reader*)list_el_chan_def->conf_file_reader)->link_defs->config_id: %d\n",
      ((conf_file_reader*)list_el_chan_def->conf_file_reader)->link_defs->config_id));

    //setting the name here
    strcpy(((conf_file_reader*)list_el_chan_def->conf_file_reader)->link_defs->name,
      list_el_chan_def->data.name);

    //((conf_file_reader*)list_el_chan_def->conf_file_reader)->link_defs->config_id = WANCONFIG_MFR;
    //((conf_file_reader*)list_el_chan_def->conf_file_reader)->link_defs->config_id = WANCONFIG_FR;

    objects_list* sub_obj_list =
      ((conf_file_reader*)list_el_chan_def->conf_file_reader)->obj_list;

    if(sub_obj_list == NULL){
       Debug(DBG_AFT_READ, ("sub_obj_list == NULL !!!\n"));
       list_el_chan_def = (list_element_chan_def*)obj_list->get_next_element(list_el_chan_def);
       continue;
    }

    list_element_chan_def* sub_list_el_chan_def =
      (list_element_chan_def*)sub_obj_list->get_first();

    Debug(DBG_AFT_READ, ("sublist:\n"));

    while(sub_list_el_chan_def != NULL){

      Debug(DBG_AFT_READ, ("old sub_list_el_chan_def->data.name: %s\n",
        sub_list_el_chan_def->data.name));
      Debug(DBG_AFT_READ, ("old list_el_chan_def->data.name: %s\n",
        list_el_chan_def->data.name));

/*
      if(strcmp(sub_list_el_chan_def->data.name, IF_NAME_NOT_INIT) != 0){
        //name was already changed from the default
        goto next_interface;
      }
*/
      //int lip_interface_serial_number = 0;
/*
      snprintf(sub_list_el_chan_def->data.name, WAN_IFNAME_SZ, "w%daf%sf%s",
        wanpipe_cfr->wanpipe_number, list_el_chan_def->data.addr,
        sub_list_el_chan_def->data.addr);
*/
      switch(((conf_file_reader*)list_el_chan_def->conf_file_reader)->link_defs->config_id)
      {
      case WANCONFIG_MFR:
      //case WANCONFIG_FR:
        if(strcmp(sub_list_el_chan_def->data.name, IF_NAME_NOT_INIT) != 0){
          //name was already changed from the default
          goto next_interface;
        }

        if(atoi(sub_list_el_chan_def->data.addr) == 0){
          //if Frame Relay, it is 'auto'. for other protocols only one PVC is there.
          snprintf(sub_list_el_chan_def->data.name, WAN_IFNAME_SZ, "w%dg%df",
            wanpipe_cfr->wanpipe_number, aft_serial_number);
        }else{
          snprintf(sub_list_el_chan_def->data.name, WAN_IFNAME_SZ, "w%dg%df%s",
            wanpipe_cfr->wanpipe_number, aft_serial_number,
            sub_list_el_chan_def->data.addr);
        }
        break;

      case WANCONFIG_MPPP:
        if(strcmp(sub_list_el_chan_def->data.name, IF_NAME_NOT_INIT) != 0){
          //name was already changed from the default
          goto next_interface;
        }

        snprintf(sub_list_el_chan_def->data.name, WAN_IFNAME_SZ, "w%dg%dppp",
            wanpipe_cfr->wanpipe_number, aft_serial_number);
        break;

      case WANCONFIG_MPCHDLC:
        if(strcmp(sub_list_el_chan_def->data.name, IF_NAME_NOT_INIT) != 0){
          //name was already changed from the default
          goto next_interface;
        }

        //note: maximum IF len is 8 chars, so using "chdl" not "chdlc"
        snprintf(sub_list_el_chan_def->data.name, WAN_IFNAME_SZ, "w%dg%dchdl",
            wanpipe_cfr->wanpipe_number, aft_serial_number);
        break;

      case WANCONFIG_HDLC:
        //note: on AFT HDLC is in hardware so there is NO LIP layer - no postfix needed.
        //snprintf(sub_list_el_chan_def->data.name, WAN_IFNAME_SZ, "w%dg%d",
        //    wanpipe_cfr->wanpipe_number, aft_serial_number);

        if(strcmp(sub_list_el_chan_def->data.name, IF_NAME_NOT_INIT) == 0){
          //'name' has to be the same for Interface and for Timeslot Group
          strcpy(sub_list_el_chan_def->data.name, list_el_chan_def->data.name);
          //'usedby' has to be the same for Interface and for Timeslot Group
          strcpy(sub_list_el_chan_def->data.usedby, list_el_chan_def->data.usedby);
        }

        //if user changed Interface name, copy back to Timeslot Group Name
        //because name has to be the same for Interface and for Timeslot Group.
        strcpy(list_el_chan_def->data.name, sub_list_el_chan_def->data.name);
        //the same logic for 'usedby'
        strcpy(list_el_chan_def->data.usedby, sub_list_el_chan_def->data.usedby);
        break;

      default:
        snprintf(sub_list_el_chan_def->data.name, WAN_IFNAME_SZ, "Unknown!!");
        break;
      }

next_interface:

      Debug(DBG_AFT_READ, ("new sub_list_el_chan_def->data.name: %s\n",
        sub_list_el_chan_def->data.name));
      Debug(DBG_AFT_READ, ("new list_el_chan_def->data.name: %s\n",
        list_el_chan_def->data.name));

      sub_list_el_chan_def =
        (list_element_chan_def*)sub_obj_list->get_next_element(sub_list_el_chan_def);

    }//while(sub_list_el_chan_def != NULL)

    list_el_chan_def = (list_element_chan_def*)obj_list->get_next_element(list_el_chan_def);

  }//while(list_el_chan_def != NULL)

  return YES;
}

	menu_list_existing_wanpipes(IN char * lxdialog_path);
	~menu_list_existing_wanpipes();
  int run(OUT int * selection_index);
};

#endif
