/***************************************************************************
                          menu_net_interface_operation_mode.cpp  -  description
                             -------------------
    begin                : Fri Mar 26 2004
    copyright            : (C) 2004 by David Rokhvarg
    email                : davidr@sangoma.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "menu_net_interface_operation_mode.h"
#include "text_box.h"

char* net_if_operational_mode_help_str =
"Operation Mode\n"
"==============\n"
"\n"
"Wanpipe drivers can work in four modes:\n"
"\n"
"WANPIPE Mode\n"
"------------\n"
"Driver links to the IP stack, and behaves like\n"
"an Ethernet device. The linux routing stack\n"
"determines how to send and receive packets.\n"
"\n"
"API Mode:\n"
"---------\n"
"Wanpipe driver uses a special RAW socket to\n"
"directly couple the user application to the driver.\n"
"Thus, user application can send and receive raw data\n"
"to and from the wanpipe driver.\n"
"\n"
"Note: API option is currently only available on\n"
"CHDLC and Frame Relay protocols.\n"
"\n"
"BRIDGE Mode:\n"
"------------\n"
"For each frame relay network interface\n"
"a virtual ethernet network interface is configured.\n"
"\n"
"Since the virtual network interface is on a BRIDGE\n"
"device it is linked into the kernel bridging code.\n"
"\n"
"Once linked to the kenrel bridge, ethernet\n"
"frames can be transmitted over a WANPIPE frame\n"
"relay connections.\n"
"\n"
"BRIDGED NODE Mode:\n"
"------------------\n"
"\n"
"For each frame relay network interface\n"
"a virtual ethernet netowrk interface is configured.\n"
"\n"
"However, this interface is not on the bridge itself,\n"
"it is connected remotely via frame relay link to the\n"
"bridge, thus it is  a node.\n"
"Since the remote end of the frame link is connected\n"
"to the bridge, this interface will be able to send\n"
"and receive ethernet frames over the frame relay\n"
"line.\n"
"\n"
"Note: Bridging is only supprted under Frame Relay\n"
"\n"
"\n"
"PPPoE Mode:\n"
"-----------\n"
"\n"
"The ADSL card configured for Bridged LLC Ethernet\n"
"over ATM encapsulation, usually uses PPPoE protocol\n"
"to connect to the ISP.\n"
"\n"
"With this option enabled, NO IP setup is necessary\n"
"because PPPoE protocol will obtain IP info from\n"
"the ISP.\n"
"\n"
"\n"
"ANNEXG Mode:\n"
"------------\n"
"\n"
"The ANNEXG mode enables wanpipe kernel\n"
"protocol layers that will run on top of the\n"
"current interface.\n"
"\n"
"Supporter MPAPI protocol are: X25/LAPB\n";

#define DBG_MENU_NET_INTERFACE_OPERATION_MODE 1

menu_net_interface_operation_mode::
  menu_net_interface_operation_mode( IN char * lxdialog_path,
                                     IN conf_file_reader* ptr_cfr,
                                     list_element_chan_def* chan_def)
{
  Debug(DBG_MENU_NET_INTERFACE_OPERATION_MODE,
    ("menu_net_interface_operation_mode::menu_net_interface_operation_mode()\n"));

  snprintf(this->lxdialog_path, MAX_PATH_LENGTH, "%s", lxdialog_path);
  this->ptr_cfr = ptr_cfr;
  this->chan_def = chan_def;
}

menu_net_interface_operation_mode::~menu_net_interface_operation_mode()
{
  Debug(DBG_MENU_NET_INTERFACE_OPERATION_MODE,
    ("menu_net_interface_operation_mode::~menu_net_interface_operation_mode()\n"));
}

int menu_net_interface_operation_mode::run(OUT int * selection_index)
{
  string menu_str;
  int rc;
  char tmp_buff[MAX_PATH_LENGTH];
  unsigned int option_selected;
  char exit_dialog;
  int protocol = ptr_cfr->link_defs->linkconf->config_id;
  int number_of_items_in_menu;

  //help text box
  text_box tb;

  Debug(DBG_MENU_NET_INTERFACE_OPERATION_MODE, ("menu_net_interface_operation_mode::run()\n"));

again:

  rc = YES;
  exit_dialog = NO;

  /////////////////////////////////////////////////
  //passing arguments by reference.
  form_operation_modes_menu_for_protocol(protocol, menu_str, number_of_items_in_menu);

  //////////////////////////////////////////////////////////////////////////////////////
  //create the explanation text for the menu
  snprintf(tmp_buff, MAX_PATH_LENGTH,
"\n------------------------------------------\
\nPlease select Interface Operation Mode.");

  if(set_configuration(   YES,//indicates to call V2 of the function
                          MENU_BOX_BACK,//MENU_BOX_SELECT,
                          lxdialog_path,
                          "INTERFACE OPERATION MODE",
                          WANCFG_PROGRAM_NAME,
                          tmp_buff,
                          MENU_HEIGTH, MENU_WIDTH,
                          4,//number of items in the menu, including the empty lines
                          (char*)menu_str.c_str()
                          ) == NO){
    rc = NO;
    goto cleanup;
  }

  if(show(selection_index) == NO){
    rc = NO;
    goto cleanup;
  }
  //////////////////////////////////////////////////////////////////////////////////////

  Debug(DBG_MENU_NET_INTERFACE_OPERATION_MODE,
    ("net_if_setup: *selection_index: %d\n", *selection_index));

  exit_dialog = NO;
  switch(*selection_index)
  {
  case MENU_BOX_BUTTON_SELECT:
    Debug(DBG_MENU_NET_INTERFACE_OPERATION_MODE,
      ("net_if_setup: option selected for editing: %s\n", get_lxdialog_output_string()));

    switch(atoi(get_lxdialog_output_string()))
    {
    case 1:
      snprintf(chan_def->data.usedby, MAX_USEDBY_STR_LEN, get_used_by_string(WANPIPE));
      exit_dialog = YES;
      break;

    case 2:
      snprintf(chan_def->data.usedby, MAX_USEDBY_STR_LEN, get_used_by_string(API));
      exit_dialog = YES;
      break;

    case 3:
      snprintf(chan_def->data.usedby, MAX_USEDBY_STR_LEN, get_used_by_string(BRIDGE));
      exit_dialog = YES;
      break;

    case 4:
      snprintf(chan_def->data.usedby, MAX_USEDBY_STR_LEN, get_used_by_string(BRIDGE_NODE));
      exit_dialog = YES;
      break;

    case 5:
      snprintf(chan_def->data.usedby, MAX_USEDBY_STR_LEN, get_used_by_string(SWITCH));
      exit_dialog = YES;
      break;

    case 6:
      snprintf(chan_def->data.usedby, MAX_USEDBY_STR_LEN, get_used_by_string(STACK));
      exit_dialog = YES;
      break;

    case 7:
      snprintf(chan_def->data.usedby, MAX_USEDBY_STR_LEN, get_used_by_string(ANNEXG));
      exit_dialog = YES;
      break;

    case 8:
      snprintf(chan_def->data.usedby, MAX_USEDBY_STR_LEN, get_used_by_string(VoIP));
      exit_dialog = YES;
      break;

    case 9:
      snprintf(chan_def->data.usedby, MAX_USEDBY_STR_LEN, get_used_by_string(PPPoE));
      exit_dialog = YES;
      break;

    default:
      ERR_DBG_OUT(("Invalid option selected for editing!! selection: %s\n",
        get_lxdialog_output_string()));
      rc = NO;
      exit_dialog = YES;
    }
    break;

  case MENU_BOX_BUTTON_HELP:

    switch(atoi(get_lxdialog_output_string()))
    {
    case 1:
    case 2:
    case 3:
    case 4:
    case 5:
    case 6:
    case 7:
    case 8:
    case 9:
      tb.show_help_message(lxdialog_path, NO_PROTOCOL_NEEDED, net_if_operational_mode_help_str);
      break;

    default:
      ERR_DBG_OUT(("Invalid option selected for editing!! selection: %s\n",
        get_lxdialog_output_string()));
      rc = NO;
      exit_dialog = YES;
    }
    break;

  case MENU_BOX_BUTTON_EXIT:
    exit_dialog = YES;
    break;
  }//switch(*selection_index)

  if(exit_dialog == NO){
    goto again;
  }

cleanup:
  return rc;
}

int menu_net_interface_operation_mode::form_operation_modes_menu_for_protocol(
                                                                  int protocol,
                                                                  string& operation_modes_str,
                                                                  int& number_of_items_in_menu)
{
  int rc = YES;
  operation_modes_str = "";

  switch(protocol)
  {
  case WANCONFIG_X25:
    operation_modes_str = QUOTED_1;
    operation_modes_str += OP_MODE_WANPIPE;

    operation_modes_str += QUOTED_2;
    operation_modes_str += OP_MODE_API;

    number_of_items_in_menu = 2;
    break;

  case WANCONFIG_FR:
    operation_modes_str = QUOTED_1;
    operation_modes_str += OP_MODE_WANPIPE;

    operation_modes_str += QUOTED_2;
    operation_modes_str += OP_MODE_API;

    operation_modes_str += QUOTED_3;
    operation_modes_str += OP_MODE_BRIDGE;

    operation_modes_str += QUOTED_4;
    operation_modes_str += OP_MODE_BRIDGE_NODE;

    number_of_items_in_menu = 4;
    break;

  case WANCONFIG_PPP:
    operation_modes_str = QUOTED_1;
    operation_modes_str += OP_MODE_WANPIPE;

    operation_modes_str += QUOTED_3;
    operation_modes_str += OP_MODE_BRIDGE;

    operation_modes_str += QUOTED_4;
    operation_modes_str += OP_MODE_BRIDGE_NODE;

    number_of_items_in_menu = 3;
    break;

  case WANCONFIG_CHDLC:
  case WANCONFIG_HDLC:

    operation_modes_str = QUOTED_1;
    operation_modes_str += OP_MODE_WANPIPE;

    operation_modes_str += QUOTED_2;
    operation_modes_str += OP_MODE_API;

    operation_modes_str += QUOTED_3;
    operation_modes_str += OP_MODE_BRIDGE;

    operation_modes_str += QUOTED_4;
    operation_modes_str += OP_MODE_BRIDGE_NODE;

    number_of_items_in_menu = 4;
    break;

  case WANCONFIG_BSC:
    operation_modes_str = QUOTED_2;
    operation_modes_str += OP_MODE_API;
    number_of_items_in_menu = 1;
    break;

  case WANCONFIG_MPPP://and WANCONFIG_MPROT too
    operation_modes_str = QUOTED_1;
    operation_modes_str += OP_MODE_WANPIPE;

    //note: Nenad will change the name soon
    operation_modes_str += QUOTED_5;
    operation_modes_str += OP_MODE_ANNEXG;

    number_of_items_in_menu = 2;
    break;

  case WANCONFIG_BITSTRM:
    operation_modes_str = QUOTED_1;
    operation_modes_str += OP_MODE_WANPIPE;

    operation_modes_str += QUOTED_2;
    operation_modes_str += OP_MODE_API;

    operation_modes_str += QUOTED_5;
    operation_modes_str += OP_MODE_SWITCH;
/*
    operation_modes_str += QUOTED_8;
    operation_modes_str += OP_MODE_VoIP;
*/
    number_of_items_in_menu = 4;
    break;

  case WANCONFIG_EDUKIT:
    //Mode is irrelevant
    rc = NO_OPER_MODE;
    break;

  case WANCONFIG_SS7:
    operation_modes_str = QUOTED_2;
    operation_modes_str += OP_MODE_API;
    number_of_items_in_menu = 1;
    break;

  case WANCONFIG_BSCSTRM:
    operation_modes_str = QUOTED_2;
    operation_modes_str += OP_MODE_API;
    number_of_items_in_menu = 1;
    break;

  case WANCONFIG_MFR:
    //"Mulit-Port Frame Relay over RAW HDLC"
    operation_modes_str = QUOTED_1;
    operation_modes_str += OP_MODE_WANPIPE;

    //note: Nenad will change the name soon
    operation_modes_str += QUOTED_5;
    operation_modes_str += OP_MODE_ANNEXG;

    number_of_items_in_menu = 2;
    break;

  case WANCONFIG_ADSL:
    //"LLC Ethernet Support (ADSL) Protocol"
    operation_modes_str = QUOTED_1;
    operation_modes_str += OP_MODE_WANPIPE;

    operation_modes_str += QUOTED_9;
    operation_modes_str += OP_MODE_PPPoE;

    number_of_items_in_menu = 2;
    break;

  case WANCONFIG_SDLC:
    operation_modes_str = QUOTED_2;
    operation_modes_str += OP_MODE_API;
    number_of_items_in_menu = 1;
    break;

  case WANCONFIG_ATM:
    operation_modes_str = QUOTED_1;
    operation_modes_str += OP_MODE_WANPIPE;

    operation_modes_str += QUOTED_3;
    operation_modes_str += OP_MODE_BRIDGE;

    number_of_items_in_menu = 2;
    break;

  case WANCONFIG_POS:
    //"Point-of-Sale Mode"
    operation_modes_str = QUOTED_2;
    operation_modes_str += OP_MODE_API;

    number_of_items_in_menu = 1;
    break;

  case WANCONFIG_AFT:
    //"AFT Mode"
    operation_modes_str = QUOTED_1;
    operation_modes_str += OP_MODE_WANPIPE;

    operation_modes_str += QUOTED_2;
    operation_modes_str += OP_MODE_API;
/*
    operation_modes_str += QUOTED_8;
    operation_modes_str += OP_MODE_VoIP;
*/
    //FIXIT: for two levels user will not be able to select STACK,
    //because it is set automatically on Protocol selcection
    //in "menu_aft_logical_channel_cfg::run()"
//    operation_modes_str += QUOTED_6;
//    operation_modes_str += OP_MODE_STACK;

    number_of_items_in_menu = 3;
    break;

  case WANCONFIG_ADCCP:
    //Special HDLC LAPB Protocol
    operation_modes_str = QUOTED_2;
    operation_modes_str += OP_MODE_API;
    number_of_items_in_menu = 1;
    break;

  case WANCONFIG_MLINK_PPP:
    //Sync/Async TTY PPP - ("Multi-Link PPP Protocol")
    //Mode is irrelevant
    rc = NO_OPER_MODE;
    break;
/*
  //internal Sangoma use only
  case WANCONFIG_DEBUG:
    break;

  case WANCONFIG_GENERIC:
    break;
*/

  default:
    rc = NO;
    break;
  }

  return rc;
}
