/***************************************************************************
                          menu_net_interface_setup.cpp  -  description
                             -------------------
    begin                : Tue Mar 23 2004
    copyright            : (C) 2004 by David Rokhvarg
    email                : davidr@sangoma.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "menu_net_interface_setup.h"
#include "text_box_help.h"
#include "net_interface_file_reader.h"
#include "menu_net_interface_miscellaneous_options.h"
#include "input_box.h"
#include "menu_net_interface_operation_mode.h"
#include "menu_net_interface_ip_configuration.h"

#include "menu_net_interfaces_list.h"
#include "menu_select_and_configure_protocol.h"

enum NET_IF_OPTIONS{
  IF_NAME,
  OPERATION_MODE,
  IP_SETUP,
  MISC_OPTIONS,
  STACK_NEXT_LEVEL_INTERFACES
};

char* net_if_name_help_str =
"Interface Name\n"
"--------------\n"
"\n"
"Name of the wanpipe interface which will\n"
"be displayed in ifconfig. This interface name will\n"
"also be associated with the local and remote IP of\n"
"device wanpipe2.\n"
"\n"
"The Wanpipe configurator always prepends the\n"
"selected interface name with 'wp#' where #=1,2,3... is\n"
"the device number selected previously. In this case the\n"
"interface name will be prepended with 'wp2'.\n"
"eg:\n"
"The way to decode the interface name: 'wp1ppp'\n"
"is the following: Interface name ppp on device\n"
"wanpipe1.\n";
//"Minimum name length is 1, maximum is 15 characters.\n";

char* operation_mode_help_str =
"Operation Mode\n"
"==============\n"
"\n"
"Wanpipe drivers can work in four modes:\n"
"\n"
"WANPIPE Mode\n"
"------------\n"
"Driver links to the IP stack, and behaves like\n"
"an Ethernet device. The linux routing stack\n"
"determines how to send and receive packets.\n"
"\n"
"API Mode:\n"
"---------\n"
"Wanpipe driver uses a special RAW socket to\n"
"directly couple the user application to the driver.\n"
"Thus, user application can send and receive raw data\n"
"to and from the wanpipe driver.\n"
"\n"
"BRIDGE Mode:\n"
"------------\n"
"For each frame relay network interface\n"
"a virtual ethernet network interface is configured.\n"
"\n"
"Since the virtual network interface is on a BRIDGE\n"
"device it is linked into the kernel bridging code.\n"
"\n"
"Once linked to the kenrel bridge, ethernet\n"
"frames can be transmitted over a WANPIPE frame\n"
"relay connections.\n"
"\n"
"BRIDGED NODE Mode:\n"
"------------------\n"
"For each frame relay network interface\n"
"a virtual ethernet netowrk interface is configured.\n"
"\n"
"However, this interface is not on the bridge itself,\n"
"it is connected remotely via frame relay link to the\n"
"bridge, thus it is  a node.\n"
"\n"
"Since the remote end of the frame link is connected\n"
"to the bridge, this interface will be able to send\n"
"and receive ethernet frames over the frame relay\n"
"line.\n"
"Note: Bridging is only supprted under Frame Relay\n";

char* ip_help_str =
"IP Address Setup\n"
"----------------\n"
"\n"
"IP addresses must be defined for each\n"
"network interface.\n";

char* net_if_miscellaneous_help_str =
"MISCELLANEOUS OPTIONS:\n"
"---------------------\n"
"1. Network Interface Start script\n"
"2. Network Interface Stop script\n"
"3. Multicast option\n"
"4. IPX support\n"
"5. IPX address\n"
"6. True Encoding option\n";



#define DBG_MENU_NET_INTERFACE_SETUP 1

menu_net_interface_setup::menu_net_interface_setup( IN char * lxdialog_path,
                                                    IN conf_file_reader* ptr_cfr,
                                                    list_element_chan_def* chan_def)
{
  Debug(DBG_MENU_NET_INTERFACE_SETUP,
    ("menu_net_interface_setup::menu_net_interface_setup()\n"));

  snprintf(this->lxdialog_path, MAX_PATH_LENGTH, "%s", lxdialog_path);
  this->ptr_cfr = ptr_cfr;
  this->chan_def = chan_def;
}

menu_net_interface_setup::~menu_net_interface_setup()
{
  Debug(DBG_MENU_NET_INTERFACE_SETUP,
    ("menu_net_interface_setup::~menu_net_interface_setup()\n"));
}

int menu_net_interface_setup::run(OUT int * selection_index)
{
  string menu_str;
  int rc;
  char tmp_buff[MAX_PATH_LENGTH];
  unsigned int option_selected;
  char exit_dialog;
  int name_len;
  int protocol;
  net_interface_file_reader* interface_file_reader = NULL;

  //help text box
  text_box tb;

  input_box inb;
  char backtitle[MAX_PATH_LENGTH];
  char explanation_text[MAX_PATH_LENGTH];
  char initial_text[MAX_PATH_LENGTH];

  snprintf(backtitle, MAX_PATH_LENGTH, "WANPIPE Configuration Utility: ");
  snprintf(&backtitle[strlen(backtitle)], MAX_PATH_LENGTH,
          "%s", get_protocol_string(NO_PROTOCOL_NEEDED));

  Debug(DBG_MENU_NET_INTERFACE_SETUP, ("menu_net_interface_setup::run()\n"));

again:

  rc = YES;
  option_selected = 0;
  exit_dialog = NO;

  if(interface_file_reader != NULL){
    delete interface_file_reader;
    interface_file_reader = NULL;
  }

  interface_file_reader = new net_interface_file_reader(chan_def->data.name);

  if(interface_file_reader->parse_net_interface_file() == NO){
    Debug(DBG_MENU_NET_INTERFACE_SETUP, ("parse_net_interface_file() FAILED!!\n"));
    return NO;
  }

  /////////////////////////////////////////////////
  //on user input: check ifname is not longer than WAN_IFNAME_SZ
  menu_str.sprintf(" \"%d\" ", IF_NAME);
  snprintf(tmp_buff, MAX_PATH_LENGTH, " \"Interface Name-----> %s\" ", chan_def->data.name);
  menu_str += tmp_buff;

  /////////////////////////////////////////////////
  if(chan_def->data.usedby == NULL){
    ERR_DBG_OUT(("Uninitialized 'usedby' pointer!!\n"));
    return NO;
  }

  if(chan_def->conf_file_reader == NULL){
    //the 'usedby' selection available only for non-STACK interfaces.
    snprintf(tmp_buff, MAX_PATH_LENGTH, " \"%d\" ", OPERATION_MODE);
    menu_str += tmp_buff;
    snprintf(tmp_buff, MAX_PATH_LENGTH, " \"Operation Mode-----> %s\" ", chan_def->data.usedby);
    menu_str += tmp_buff;
  }

/*
  if( chan_def->conf_file_reader != NULL &&
      get_used_by_integer_value(chan_def->data.usedby) != STACK){

      Debug(DBG_MENU_NET_INTERFACE_SETUP,
        ("chan_def->conf_file_reader: %p, chan_def->data.usedby: %s\n",
        chan_def->conf_file_reader, chan_def->data.usedby));
      //exit(0);
  }
*/
  /////////////////////////////////////////////////
//  Debug(DBG_MENU_NET_INTERFACE_SETUP, ("strlen(interface_file_reader.if_config.ipaddr): %d\n",
//    strlen(interface_file_reader.if_config.ipaddr)));

  switch(get_used_by_integer_value(chan_def->data.usedby))
  {
  case WANPIPE:
  case BRIDGE_NODE:

    snprintf(tmp_buff, MAX_PATH_LENGTH, " \"%d\" ", IP_SETUP);
    menu_str += tmp_buff;

    if( strlen(interface_file_reader->if_config.ipaddr) <  MIN_IP_ADDR_STR_LEN ||
        strlen(interface_file_reader->if_config.point_to_point_ipaddr) < MIN_IP_ADDR_STR_LEN){

      snprintf(tmp_buff, MAX_PATH_LENGTH, " \"IP Address Setup---> Incomplete\" ");
    }else{
      snprintf(tmp_buff, MAX_PATH_LENGTH, " \"IP Address Setup---> Complete\" ");
    }
    menu_str += tmp_buff;
    break;

  case STACK:
    //////////////////////////////////////////////////////////////////////////////////////
    //If STACK show option to see the list of next level interfaces
    if(chan_def->conf_file_reader != NULL){
      snprintf(tmp_buff, MAX_PATH_LENGTH, " \"%d\" ", STACK_NEXT_LEVEL_INTERFACES);
      menu_str += tmp_buff;
      snprintf(tmp_buff, MAX_PATH_LENGTH, " \"List STACK interfaces\" ");
      menu_str += tmp_buff;
    }else{
      //if STACK - the next level protocol was already configured in
      //"menu_aft_logical_channel_cfg.cpp".
/*
      //FIXIT: in the next release there should be no limit on number of Levels in STACK,
      //so here the complete Protocol selection/configuration should be available.
      //If STACK and no next level protocol, set Frame Relay as new protocol protocol.
//////////////////////////////////////////////
      if(create_default_protocol_cfg(  WANCONFIG_FR,
                                    (conf_file_reader**)&chan_def->conf_file_reader,
                                    ptr_cfr) == NO){
        ERR_DBG_OUT(("Failed to read default protocol!!\n"));
        return NO;
      }

      list_element_chan_def* next_level_list_el_chan_def =
        (list_element_chan_def*)((conf_file_reader*)chan_def->conf_file_reader)->obj_list->get_first();

      //interface counter starting from 0. usage dependes on protocol.
      int if_counter = 0;
      while(next_level_list_el_chan_def != NULL){

        if(update_interface_name( chan_def,
                             next_level_list_el_chan_def,
                             ((conf_file_reader*)chan_def->conf_file_reader)->link_defs->config_id,
                             if_counter) == NO){
          rc = NO;
          break;
        }

        next_level_list_el_chan_def =
          (list_element_chan_def*)((conf_file_reader*)chan_def->conf_file_reader)->
                        obj_list->get_next_element(next_level_list_el_chan_def);

        if_counter++;
      }//while()
      goto again;
*/
//////////////////////////////////////////////
    }
    break;

  default:
    break;
  }

  /////////////////////////////////////////////////
  snprintf(tmp_buff, MAX_PATH_LENGTH, " \"%d\" ", MISC_OPTIONS);
  menu_str += tmp_buff;
  snprintf(tmp_buff, MAX_PATH_LENGTH, " \"Miscellaneous options\" ");
  menu_str += tmp_buff;


  //////////////////////////////////////////////////////////////////////////////////////
  //create the explanation text for the menu
  protocol = ptr_cfr->link_defs->linkconf->config_id;

  Debug(DBG_MENU_NET_INTERFACE_SETUP, ("creating explanation string\n"));

  if(protocol == WANCONFIG_FR){
    snprintf(tmp_buff, MAX_PATH_LENGTH,
"\n------------------------------------------\
\nInterface: %s (bound to DLCI: %s)",
    chan_def->data.name, chan_def->data.addr);
  }else{
    snprintf(tmp_buff, MAX_PATH_LENGTH,
"\n------------------------------------------\
\nParameters for interface: %s",
    chan_def->data.name);
  }

  Debug(DBG_MENU_NET_INTERFACE_SETUP, ("explanation string: %s\n", tmp_buff));

  if(set_configuration(   YES,//indicates to call V2 of the function
                          MENU_BOX_BACK,//MENU_BOX_SELECT,
                          lxdialog_path,
                          "INTERFACE CONFIGURATION",
                          WANCFG_PROGRAM_NAME,
                          tmp_buff,
                          MENU_HEIGTH, MENU_WIDTH,
                          4,//number of items in the menu, including the empty lines
                          (char*)menu_str.c_str()
                          ) == NO){
    rc = NO;
    goto cleanup;
  }

  if(show(selection_index) == NO){
    rc = NO;
    goto cleanup;
  }
  //////////////////////////////////////////////////////////////////////////////////////

  Debug(1, ("net_if_setup: *selection_index: %d\n", *selection_index));

  exit_dialog = NO;
  switch(*selection_index)
  {
  case MENU_BOX_BUTTON_SELECT:
    Debug(DBG_MENU_NET_INTERFACE_SETUP,
      ("net_if_setup: option selected for editing: %s\n", get_lxdialog_output_string()));

    switch(atoi(get_lxdialog_output_string()))
    {
    case IF_NAME://interface name
      /////////////////////////////////////////////////////////////
show_if_name_input_box:
      snprintf(explanation_text, MAX_PATH_LENGTH, "Please specify a Network Interface Name");
      snprintf(initial_text, MAX_PATH_LENGTH, "%s", chan_def->data.name);

      inb.set_configuration(  lxdialog_path,
                              backtitle,
                              explanation_text,
                              INPUT_BOX_HIGTH,
                              INPUT_BOX_WIDTH,
                              initial_text);

      inb.show(selection_index);

      switch(*selection_index)
      {
      case INPUT_BOX_BUTTON_OK:
        Debug(DBG_MENU_NET_INTERFACE_SETUP,
          ("Interface name on return: %s\n", inb.get_lxdialog_output_string()));

        name_len = strlen(inb.get_lxdialog_output_string());

        if( name_len < 1 || name_len > WAN_IFNAME_SZ){

          tb.show_error_message(lxdialog_path, NO_PROTOCOL_NEEDED,
"Invalid Interface name.\n\
%s\n\
Minimum name length is %d, maximum is %d characters.\n",
net_if_name_help_str,
1,
WAN_IFNAME_SZ
);
          goto show_if_name_input_box;
        }else{
          snprintf(chan_def->data.name, WAN_IFNAME_SZ, inb.get_lxdialog_output_string());
          //list_el_chan_def->data.name
          Debug(DBG_MENU_NET_INTERFACE_SETUP,
            ("menu_net_interface_setup: chan_def: %p\n", chan_def));
        }
        break;

      case INPUT_BOX_BUTTON_HELP:

        tb.show_help_message(lxdialog_path, NO_PROTOCOL_NEEDED, net_if_name_help_str);
        goto show_if_name_input_box;
        break;
      }//switch(*selection_index)
      /////////////////////////////////////////////////////////////
      break;

    case OPERATION_MODE://Operational Mode
      {
        menu_net_interface_operation_mode
          net_interface_operation_mode(lxdialog_path, ptr_cfr, chan_def);
        if(net_interface_operation_mode.run(selection_index) == NO){
          rc = NO;
          exit_dialog = YES;
        }
      }
      break;

    case IP_SETUP://IP address
      {
        menu_net_interface_ip_configuration
          net_interface_ip_configuration(lxdialog_path, ptr_cfr, chan_def);
        if(net_interface_ip_configuration.run(selection_index, *interface_file_reader) == NO){
          rc = NO;
          exit_dialog = YES;
        }else{
          if(interface_file_reader->create_net_interface_file(
                              &interface_file_reader->if_config) == NO){
            Debug(DBG_MENU_NET_INTERFACE_SETUP, ("create_net_interface_file() FAILED!!\n"));
            return NO;
          }
        }
      }
      break;

    case STACK_NEXT_LEVEL_INTERFACES:
      //instead of recurcive call
      { //display list of next level network interfaces.

        Debug(DBG_MENU_NET_INTERFACE_SETUP, ("chan_def %p\n", chan_def));

        menu_net_interfaces_list net_interfaces_list( lxdialog_path,
                               (conf_file_reader*)chan_def->conf_file_reader);
        if(net_interfaces_list.run(selection_index) == NO){
           rc = NO;
           exit_dialog = YES;
        }
      }
      break;

    case MISC_OPTIONS:
      {
        menu_net_interface_miscellaneous_options
          net_interface_miscellaneous_options(lxdialog_path, ptr_cfr, chan_def);
        if(net_interface_miscellaneous_options.run(selection_index) == NO){
          rc = NO;
          exit_dialog = YES;
        }
      }
      break;

    default:
      ERR_DBG_OUT(("Invalid option selected for editing!! selection: %s\n",
        get_lxdialog_output_string()));
      rc = NO;
      exit_dialog = YES;
    }
    break;

  case MENU_BOX_BUTTON_HELP:

    switch(atoi(get_lxdialog_output_string()))
    {
    case IF_NAME:
      tb.show_help_message(lxdialog_path, NO_PROTOCOL_NEEDED, net_if_name_help_str);
      break;

    case OPERATION_MODE:
      tb.show_help_message(lxdialog_path, NO_PROTOCOL_NEEDED, operation_mode_help_str);
      break;

    case IP_SETUP:
      tb.show_help_message(lxdialog_path, NO_PROTOCOL_NEEDED, ip_help_str);
      break;

    case MISC_OPTIONS:
      tb.show_help_message(lxdialog_path, NO_PROTOCOL_NEEDED, net_if_miscellaneous_help_str);
      break;

    default:
      ERR_DBG_OUT(("Invalid option selected for editing!! selection: %s\n",
        get_lxdialog_output_string()));
      rc = NO;
      exit_dialog = YES;
    }
    break;

  case MENU_BOX_BUTTON_EXIT:
    exit_dialog = YES;
    break;
  }//switch(*selection_index)

  if(exit_dialog == NO){
    goto again;
  }

cleanup:
  return rc;
}

int menu_net_interface_setup::create_default_protocol_cfg(
                                    unsigned int protocol_selected,
                                    conf_file_reader** next_level_file_reader,
                                    conf_file_reader* root_dev_file_reader)
{
  int rc = YES;
  //help text box
  text_box tb;

  Debug(DBG_MENU_NET_INTERFACE_SETUP, ("create_default_protocol_cfg()\n"));
  Debug(DBG_MENU_NET_INTERFACE_SETUP, ("root_dev_file_reader->link_defs->name: %s\n",
    root_dev_file_reader->link_defs->name));

  //1.  read the template file
  //2.  copy everything what has to do with card type and location into
  //    the new 'file_reader' object
  //3.  update wanpipe name to the real one
  //4.  update each interface name to the real one
  conf_file_reader* cfr_tmp =
    new conf_file_reader(menu_select_and_configure_protocol::
                                    get_template_file_path(protocol_selected));

  Debug(DBG_MENU_NET_INTERFACE_SETUP,
    ("handle_protocol_change(): %p\n", cfr_tmp));

  //STEP1:
  //read the template file.
  if(cfr_tmp->read_conf_file() != 0){
    tb.show_error_message(lxdialog_path, NO_PROTOCOL_NEEDED,
      "Failed to parse '%s' file!!",
      menu_select_and_configure_protocol::get_template_file_path(protocol_selected));
    rc = NO;
  }

  //STEP2:
  cfr_tmp->wanpipe_number = root_dev_file_reader->wanpipe_number;
  link_def_t * old_link_def = root_dev_file_reader->link_defs;
  link_def_t * new_link_def = cfr_tmp->link_defs;

  wandev_conf_t * old_linkconf = root_dev_file_reader->link_defs->linkconf;
  wandev_conf_t * new_linkconf = cfr_tmp->link_defs->linkconf;

  new_linkconf->card_type = old_linkconf->card_type;
  new_link_def->card_version = old_link_def->card_version;

  new_linkconf->auto_pci_cfg = old_linkconf->auto_pci_cfg;
  new_linkconf->PCI_slot_no = old_linkconf->PCI_slot_no;
  new_linkconf->pci_bus_no = old_linkconf->pci_bus_no;
  new_linkconf->S514_CPU_no[0] = old_linkconf->S514_CPU_no[0];

  new_linkconf->interface = old_linkconf->interface;
  new_linkconf->bps = old_linkconf->bps;
  memcpy(&new_linkconf->te_cfg, &old_linkconf->te_cfg, sizeof(sdla_te_cfg_t));

  //STEP3:
  //update wanpipe name
  snprintf(new_link_def->name, WAN_DRVNAME_SZ, "%s", old_link_def->name);
  //update default names of each interface
  list_element_chan_def* list_el_chan_def;

  ///////////////////////////////
  //STEP4:
  Debug(DBG_MENU_NET_INTERFACE_SETUP,
    ("cfr_tmp->wanpipe_number: %d\n", cfr_tmp->wanpipe_number));

  objects_list* obj_list = cfr_tmp->obj_list;
  list_el_chan_def = (list_element_chan_def*)obj_list->get_first();

  /////////////////////////////////////////////////////////////////////////////////
  //FIXIT: AFT only code!!
  Debug(DBG_MENU_NET_INTERFACE_SETUP,
    ("AFT: protocol : %d\n", list_el_chan_def->data.chanconf->protocol));

  //FIXIT:
//  if(list_el_chan_def->data.chanconf->protocol == WANCONFIG_MFR){

//???
//  if(list_el_chan_def->data.chanconf->protocol != WANCONFIG_HDLC){

    conf_file_reader* local_cfr = (conf_file_reader*)list_el_chan_def->conf_file_reader;
    if(local_cfr != NULL){

      list_element_chan_def* local_list_el_chan_def =
        (list_element_chan_def*)local_cfr->obj_list->get_first();

      if(list_el_chan_def != NULL){
        strcpy(list_el_chan_def->data.aft_dlci_number, local_list_el_chan_def->data.addr);
      }else{
        ERR_DBG_OUT(("Failed to initialize 'aft_dlci_number'(2)!!"));
        return NO;
      }
    }else{
      ERR_DBG_OUT(("Uninitialized per-AFT-logical channel 'conf_file_reader' pointer!!"));
      return NO;
    }
//???
//  }//if(list_el_chan_def->data.chanconf->protocol == WANCONFIG_MFR)
  /////////////////////////////////////////////////////////////////////////////////
/*
  //interface counter starting from 0. usage dependes on protocol.
  int if_counter = 0;
  while(list_el_chan_def != NULL){

    if(update_interface_name( list_el_chan_def,
                              new_link_def->config_id,
                              cfr_tmp->wanpipe_number) == NO){
      rc = NO;
      break;
    }

    list_el_chan_def = (list_element_chan_def*)obj_list->get_next_element(list_el_chan_def);

    if_counter++;
  }//while()
  //////////////////////////////
*/

  //replace with the new pointer
  *next_level_file_reader = cfr_tmp;

  return rc;
}

int menu_net_interface_setup::update_interface_name(
                            list_element_chan_def* root_list_el_chan_def,
                            list_element_chan_def* next_level_list_el_chan_def,
                            int protocol,
                            int if_counter)
{
    int rc = YES;
  	chan_def_t* chandef = &next_level_list_el_chan_def->data;
  	chan_def_t* root_chandef = &root_list_el_chan_def->data;

    Debug(DBG_MENU_NET_INTERFACE_SETUP, ("menu_net_interface_setup::update_interface_name()\n"));
    Debug(DBG_MENU_NET_INTERFACE_SETUP,
      ("protocol: %d, if_counter: %d\n",
      protocol, if_counter));

    Debug(DBG_MENU_NET_INTERFACE_SETUP, ("root_chandef->name: %s\n", root_chandef->name));

    switch(protocol)
    {
    case WANCONFIG_FR:
      if (strcmp(chandef->addr,"auto")==0){
        snprintf(chandef->name, WAN_IFNAME_SZ, "%sf", root_chandef->name);
      }else{
        snprintf(chandef->name, WAN_IFNAME_SZ, "%sf%s",
          root_chandef->name, chandef->addr);
      }
      break;

    case WANCONFIG_PPP:
      snprintf(chandef->name, WAN_IFNAME_SZ, "%sppp", root_chandef->name);
      break;

    case WANCONFIG_CHDLC:
      snprintf(chandef->name, WAN_IFNAME_SZ, "%schdlc", root_chandef->name);
      break;

    default:
      ERR_DBG_OUT(("Invalid new protocol! (0x%X)", protocol));
      rc = NO;
    }

    return rc;
}
