/***************************************************************************
                          menu_net_interfaces_list.cpp  -  description
                             -------------------
    begin                : Wed Mar 24 2004
    copyright            : (C) 2004 by David Rokhvarg
    email                : davidr@sangoma.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "menu_net_interfaces_list.h"
#include "text_box_help.h"
#include "menu_net_interface_setup.h"

char* net_if_list_no_interface_error_message =
"Network Interfaces List\n"
"-----------------------\n"
"No Network Interfaces defined.\n"
"Each Network Interface correspons to a Logical Channel.\n"
"Add at least one Logical Channel on the Protocol Level.\n";

char* net_if_list_help =
"NETWORK INTERFACE CONFIGURATION\n"
"\n"
"Each network interface has the following\n"
"options: Interface Name, Operation Mode,\n"
"IP Address Setup and Interface Scripts.\n"
"\n"
"Select and configure each interface\n"
"displayed in the menu.  When the configuration of ALL\n"
"interfaces is complete, use the <Back> option to exit\n"
"the dialog\n";


#define DBG_MENU_NET_INTERFACES_LIST 1

menu_net_interfaces_list::menu_net_interfaces_list( IN char * lxdialog_path,
                                                    IN conf_file_reader* cfr)
{
  Debug(DBG_MENU_NET_INTERFACES_LIST,
    ("menu_net_interfaces_list::menu_net_interfaces_list()\n"));

  snprintf(this->lxdialog_path, MAX_PATH_LENGTH, "%s", lxdialog_path);
  this->cfr = cfr;
}

menu_net_interfaces_list::~menu_net_interfaces_list()
{
  Debug(DBG_MENU_NET_INTERFACES_LIST,
    ("menu_net_interfaces_list::~menu_net_interfaces_list()\n"));
}

int menu_net_interfaces_list::run(OUT int * selection_index)
{
  string menu_str;
  int rc = YES;
  char tmp_buff[MAX_PATH_LENGTH];
  char exit_dialog = NO;
  int number_of_items;
  int protocol;
  list_element_chan_def* list_el_chan_def = NULL;
  //help/error text box
  text_box tb;

  Debug(DBG_MENU_NET_INTERFACES_LIST, ("menu_net_interfaces_list::run()\n"));

  if(cfr == NULL){
    ERR_DBG_OUT(("Conf. file reader pointer is NULL!!\n"));
    return NO;
  }
  objects_list * obj_list = cfr->obj_list;

  if(obj_list == NULL){
    ERR_DBG_OUT(("'obj_list' pointer is NULL!!\n"));
    return NO;
  }

again:
  list_el_chan_def = NULL;
  menu_str = "";
  protocol = cfr->link_defs->linkconf->config_id;

  /////////////////////////////////////////////////
  if(obj_list->get_size() > MAX_NUMBER_OF_NET_INTERFACES_PER_WANPIPE){
    ERR_DBG_OUT(("Too many interfaces in list! (number of interface: %u)\n",
      obj_list->get_size()));
    return NO;
  }

  if(obj_list->get_size() == 0){
    menu_str = "0";//zero is rezerved for this case.
    snprintf(tmp_buff, MAX_PATH_LENGTH, " \"No interfaces defined.\" ");
    menu_str += tmp_buff;
  }else{
//chan_def 0x809f638

    list_el_chan_def = (list_element_chan_def*)obj_list->get_first();
    //numbers starting from 1.
    int i = 1;
    while(list_el_chan_def != NULL){

      //create tag
      snprintf(tmp_buff, MAX_PATH_LENGTH, " \"%d\" ", i);
      menu_str += tmp_buff;

      if(protocol == WANCONFIG_FR || protocol == WANCONFIG_MFR){
        snprintf(tmp_buff, MAX_PATH_LENGTH, " \"Interface%d (DLCI:%s)--> %s\" ",
          i,
          list_el_chan_def->data.addr,
          list_el_chan_def->data.name);
/*
      }else if(protocol == WANCONFIG_AFT){

        snprintf(tmp_buff, MAX_PATH_LENGTH, " \"Interface%d (%s:%s)--> %s\" ",
          i,
          TIME_SLOT_GROUP_STR,
          list_el_chan_def->data.addr,
          list_el_chan_def->data.name);
*/
      }else{
        snprintf(tmp_buff, MAX_PATH_LENGTH, " \"Interface%d--> %s\" ",
          i,
          list_el_chan_def->data.name);
      }

      menu_str += tmp_buff;
      list_el_chan_def = (list_element_chan_def*)obj_list->get_next_element(list_el_chan_def);

      i++;
    }//while()
  }//if()

  number_of_items = obj_list->get_size();
  if(number_of_items > MAX_NUM_OF_VISIBLE_ITEMS_IN_MENU){
    number_of_items = MAX_NUM_OF_VISIBLE_ITEMS_IN_MENU - 4;
  }

  //////////////////////////////////////////////////////////////////////////////////////
  //create the explanation text for the menu
  snprintf(tmp_buff, MAX_PATH_LENGTH,
"\n------------------------------------------\
\nList of Network Interfaces for Wan Device: %s.\
\n-------------------------------------------\
\nPlease select Network Interface you want to configure.", cfr->link_defs->name);

  if(set_configuration(   YES,//indicates to call V2 of the function
                          MENU_BOX_BACK,//MENU_BOX_SELECT,
                          lxdialog_path,
                          "NETWORK INTERFACE(S) LIST",
                          WANCFG_PROGRAM_NAME,
                          tmp_buff,
                          MENU_HEIGTH, MENU_WIDTH,
                          number_of_items,
                          (char*)menu_str.c_str()
                          ) == NO){
    rc = NO;
    goto cleanup;
  }

  if(show(selection_index) == NO){
    rc = NO;
    goto cleanup;
  }

  //////////////////////////////////////////////////////////////////////////////////////

  exit_dialog = NO;
  switch(*selection_index)
  {
  case MENU_BOX_BUTTON_SELECT:
    Debug(DBG_MENU_NET_INTERFACES_LIST,
      ("option selected for editing: %s\n", get_lxdialog_output_string()));

    switch(atoi(get_lxdialog_output_string()))
    {
    case 0:
      //put message saying: "Add channels on Protocol level."
      tb.show_error_message(lxdialog_path, WANCONFIG_FR, net_if_list_no_interface_error_message);
      break;

    default:

      int order_number_in_list;
      int element_counter;
      list_el_chan_def = NULL;

      Debug(DBG_MENU_NET_INTERFACES_LIST,
        ("option selected for editing: %s\n", get_lxdialog_output_string()));

      order_number_in_list = atoi(get_lxdialog_output_string()) - 1;
      element_counter = 0;

      //start from the begining of the list until, 'element_counter'
      //reaches 'order_number_in_list'

      list_el_chan_def = (list_element_chan_def*)obj_list->get_first();
      do{

        if(element_counter == order_number_in_list){
          break;
        }else{
          element_counter++;
          list_el_chan_def = (list_element_chan_def*)obj_list->get_next_element(list_el_chan_def);
        }

      }while(list_el_chan_def != NULL);

      if(list_el_chan_def == NULL){
        ERR_DBG_OUT(("Failed to fined selected element '%s' in the list\n",
          get_lxdialog_output_string()));
        rc = NO;
        exit_dialog = YES;
      }else{
        //found interface in the list.
        Debug(DBG_MENU_NET_INTERFACES_LIST,
          ("Interface selected for configuration %s\n", list_el_chan_def->data.name));
/*
        if(protocol == WANCONFIG_AFT){
          //instead of recurcive call
          {     //display list of network interfaces.

            //FIXIT: next level protocol must be set elsewere AUTOMATICALLY!!
//            ((conf_file_reader*)list_el_chan_def->conf_file_reader)->link_defs->
//            linkconf->config_id = list_el_chan_def->data.LIP_protocol;//WANCONFIG_FR;

            Debug(DBG_MENU_NET_INTERFACES_LIST,
              ("linkconf->config_id %d\n", ((conf_file_reader*)list_el_chan_def->conf_file_reader)->link_defs->
              linkconf->config_id));

            Debug(DBG_MENU_NET_INTERFACES_LIST,
              ("list_el_chan_def->conf_file_reader %p\n", list_el_chan_def->conf_file_reader));

            Debug(DBG_MENU_NET_INTERFACES_LIST,
              ("list_el_chan_def %p\n", list_el_chan_def));

            menu_net_interfaces_list net_interfaces_list( lxdialog_path,
                                        (conf_file_reader*)list_el_chan_def->conf_file_reader);
            if(net_interfaces_list.run(selection_index) == NO){
              rc = NO;
              exit_dialog = YES;
            }
          }

        }else{
*/
          menu_net_interface_setup net_interface_setup(lxdialog_path, cfr, list_el_chan_def);
          net_interface_setup.run(selection_index);
//        }

        rc = YES;
        exit_dialog = NO;
      }
    }
    break;

  case MENU_BOX_BUTTON_HELP:
    tb.show_help_message(lxdialog_path, NO_PROTOCOL_NEEDED, net_if_list_help);
    break;

  case MENU_BOX_BUTTON_EXIT:
    exit_dialog = YES;
    break;
  }//switch(*selection_index)

  if(exit_dialog == NO){
    goto again;
  }

cleanup:
  return rc;
}

