/***************************************************************************
                          menu_new_device_configuration.cpp  -  description
                             -------------------
    begin                : Tue Mar 9 2004
    copyright            : (C) 2004 by David Rokhvarg
    email                : davidr@sangoma.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <string>

#include "menu_new_device_configuration.h"
#include "text_box_help.h"
#include "text_box_yes_no.h"
#include "menu_select_and_configure_protocol.h"
#include "menu_net_interfaces_list.h"
#include "menu_hardware_setup.h"
#include "menu_device_miscellaneous_options.h"
#include "menu_aft_logical_channels_list.h"
#include "net_interface_file_reader.h"

#include "menu_list_existing_wanpipes.h"

#include "menu_aft_logical_channel_cfg.h"

char* new_dev_cfg_help_str =
"Hardware Setup\n"
"--------------\n"
"Select type of Sangoma card and\n"
"resources used by it.\n"
"\n"
"Protocol\n"
"--------\n"
"Select Communications Protocol.\n"
"Following protocols available for\n"
"S514 and S508 cards:\n"
"  Frame Relay, PPP, CHDLC, HDLC Streaming\n"
"A101/102 cards:\n"
"  MP Frame Relay, MP PPP, MP CHDLC, MP HDLC Streaming\n"
"\n"
"Interface Setup\n"
"---------------\n"
"  -set Interface Name\n"
"  -select Interface Operation Mode: WANPIPE, API, BRIDGE...\n"
"  -set Interface IP information.\n"
"  -configure per-Interface start/stop scripts.\n"
"\n"
"Miscellaneous WANPIPE options\n"
"-----------------------------\n"
"  -configure WANPIPE UDP debugging.\n"
"  -configure per-WANPIPE start/stop scripts.\n";

char* logical_channel_setup_help_str =
"Timeslot Groups Configuration\n"
"-----------------------------\n"
" -Set Number of Timeslot Groups\n"
" -Set Configuration for each Timeslot Group\n";

enum NEW_DEV_CFG_OPTIONS{
  HW_SETUP=100,
  PROTOCOL_SETUP,
  NET_IF_SETUP,
  MISCELLANEOUS_OPTIONS
//  LOGICAL_CHANNELS_SETUP
};

#define DBG_MENU_NEW_DEVICE_CONFIG  1

menu_new_device_configuration::menu_new_device_configuration( IN char * lxdialog_path,
                                                              IN conf_file_reader** ptr_cfr)
{
  Debug(DBG_MENU_NEW_DEVICE_CONFIG,
    ("menu_new_device_configuration::menu_new_device_configuration()\n"));

  snprintf(this->lxdialog_path, MAX_PATH_LENGTH, "%s", lxdialog_path);
  this->ptr_cfr = ptr_cfr;
  cfr = *ptr_cfr;
}

menu_new_device_configuration::~menu_new_device_configuration()
{
  Debug(DBG_MENU_NEW_DEVICE_CONFIG,
    ("menu_new_device_configuration::~menu_new_device_configuration()\n"));
}

int menu_new_device_configuration::run(OUT int * selection_index)
{
  string menu_str;
  int rc, number_of_items;
  int exit_dialog;
  char tmp_buff[MAX_PATH_LENGTH];
  char* ip_setup_completion_check;

  //help text box
  text_box tb;

  link_def_t * link_def;
  wandev_conf_t *linkconf;
  objects_list* obj_list;
  list_element_chan_def* list_el_chan_def;
  conf_file_reader* lip_layer_file_reader;

again:
  rc = YES;
  exit_dialog = NO;
  number_of_items = 0;

  link_def = cfr->link_defs;
  linkconf = cfr->link_defs->linkconf;

  Debug(DBG_MENU_NEW_DEVICE_CONFIG, ("menu_new_device_configuration::run()\n"));
  Debug(DBG_MENU_NEW_DEVICE_CONFIG, ("cfr->link_defs->name: %s\n", link_def->name));
  Debug(DBG_MENU_NEW_DEVICE_CONFIG, ("linkconf->card_type: %d\n", linkconf->card_type));
  Debug(DBG_MENU_NEW_DEVICE_CONFIG, ("linkconf->config_id: %d\n", linkconf->config_id));

  /////////////////////////////////////////////////
  menu_str = " ";

  snprintf(tmp_buff, MAX_PATH_LENGTH, " \"%d\" ", HW_SETUP);
  menu_str += tmp_buff;
  //create the menu item
  snprintf(tmp_buff, MAX_PATH_LENGTH, " \"Hardware Setup--> %s\" ",
    get_card_type_string(linkconf->card_type));
  menu_str += tmp_buff;

  if(linkconf->card_type == NOT_SET){

    number_of_items = 1;
    goto display_hw_setup_dialog_label;

  }else if(linkconf->card_type != WANOPT_AFT){
//  }else if(linkconf->card_type == WANOPT_S51X || linkconf->card_type == WANOPT_AFT){

    number_of_items = 4;
    /////////////////////////////////////////////////
    snprintf(tmp_buff, MAX_PATH_LENGTH, " \"%d\" ", PROTOCOL_SETUP);
    menu_str += tmp_buff;

    snprintf(tmp_buff, MAX_PATH_LENGTH, " \"Protocol--------> %s\" ",
      get_protocol_string(linkconf->config_id));
    menu_str += tmp_buff;

  }else{

    //for AFT card display list of Timeslot Groups.
    number_of_items = 4;

    menu_list_existing_wanpipes::update_per_group_of_channels_iformation(cfr);


    obj_list = cfr->obj_list;
    if(obj_list == NULL){
      ERR_DBG_OUT(("Invalid pointer!\n"));
      return NO;
    }

    Debug(DBG_MENU_NEW_DEVICE_CONFIG, ("number of objects in the list: %d\n", obj_list->get_size()));
    number_of_items += obj_list->get_size();

    create_logical_channels_list_str(menu_str);
  }

  if(linkconf->config_id != PROTOCOL_NOT_SET ){

    if(linkconf->card_type != WANOPT_AFT){
      /////////////////////////////////////////////////
      snprintf(tmp_buff, MAX_PATH_LENGTH, " \"%d\" ", NET_IF_SETUP);
      menu_str += tmp_buff;

      //create the menu item
      if(cfr->obj_list->get_size()){
        snprintf(tmp_buff, MAX_PATH_LENGTH, " \"Interface Setup-> %d Defined\" ",
          cfr->obj_list->get_size());
      }else{
        snprintf(tmp_buff, MAX_PATH_LENGTH, " \"Interface Setup->  Undefined\" ");
      }
      menu_str += tmp_buff;
    }

    /////////////////////////////////////////////////
    snprintf(tmp_buff, MAX_PATH_LENGTH, " \"%d\" ", MISCELLANEOUS_OPTIONS);
    menu_str += tmp_buff;
    snprintf(tmp_buff, MAX_PATH_LENGTH, " \"Miscellaneous WANPIPE options\" " );
    menu_str += tmp_buff;
  }

  Debug(DBG_MENU_NEW_DEVICE_CONFIG, ("\nmenu_str:%s\n", menu_str.c_str()));

  //create the explanation text for the menu
  snprintf(tmp_buff, MAX_PATH_LENGTH,
    "Configuration for Wan Device: %s\n\n", cfr->link_defs->name);

  //snprintf(&tmp_buff[strlen(tmp_buff)], MAX_PATH_LENGTH - strlen(tmp_buff), "%s", MENUINSTR_EXIT);

  if(number_of_items > 8){
    number_of_items = 8;
  }

  if(set_configuration(   YES,//indicates to call V2 of the function
                          MENU_BOX_BACK,//MENU_BOX_SELECT,
                          lxdialog_path,
                          "WANPIPE CONFIGURATION",
                          WANCFG_PROGRAM_NAME,
                          tmp_buff,
                          MENU_HEIGTH, MENU_WIDTH,
                          number_of_items,
                          (char*)menu_str.c_str()
                          ) == NO){
    rc = NO;
    goto cleanup;
  }

  if(show(selection_index) == NO){
    Debug(DBG_MENU_NEW_DEVICE_CONFIG, ("menu_new_device_configuration.show() - failed!!\n"));
    rc = NO;
    goto cleanup;
  }

  exit_dialog = NO;
  switch(*selection_index)
  {
  case MENU_BOX_BUTTON_SELECT:
    Debug(DBG_MENU_NEW_DEVICE_CONFIG,
      ("option selected for editing: %s\n", get_lxdialog_output_string()));

    switch(atoi(get_lxdialog_output_string()))
    {
    case HW_SETUP:
display_hw_setup_dialog_label:
      {
        //menu_hardware_setup hardware_setup(lxdialog_path, cfr);
        menu_hardware_setup hardware_setup(lxdialog_path, ptr_cfr);
        if(hardware_setup.run(selection_index) == NO){
          rc = NO;
          exit_dialog = YES;
        }else{
          //VERY IMPORTANT: use the (potentially) new pointer to the 'file reader'
          cfr = *ptr_cfr;
        }
      }
      break;

    case PROTOCOL_SETUP:
      {
        menu_select_and_configure_protocol select_config_protocol(lxdialog_path, ptr_cfr);
        if(select_config_protocol.run(selection_index) == NO){
          rc = NO;
          exit_dialog = YES;
        }else{
          //VERY IMPORTANT: use the (potentially) new pointer to the 'file reader'
          cfr = *ptr_cfr;
        }
      }
      break;

    case NET_IF_SETUP:

      //display list of network interfaces.

      if(linkconf->card_type == WANOPT_AFT){
        //for AFT we can get here if there is ONLY ONE  Timeslot Group!!!
        list_el_chan_def = (list_element_chan_def*)obj_list->get_first();

        lip_layer_file_reader = (conf_file_reader*)list_el_chan_def->conf_file_reader;

        if(lip_layer_file_reader == NULL){
          ERR_DBG_OUT(("Invalid 'lip_layer_file_reader' pointer!!\n"));
          rc = NO;
          exit_dialog = YES;
          break;
        }

        menu_net_interfaces_list net_interfaces_list(lxdialog_path, lip_layer_file_reader);
        if(net_interfaces_list.run(selection_index) == NO){
          rc = NO;
          exit_dialog = YES;
        }

      }else{
        menu_net_interfaces_list net_interfaces_list(lxdialog_path, cfr);
        if(net_interfaces_list.run(selection_index) == NO){
          rc = NO;
          exit_dialog = YES;
        }
      }
      break;

    case MISCELLANEOUS_OPTIONS:
      {
        menu_device_miscellaneous_options device_miscellaneous_options(lxdialog_path, cfr);
        if(device_miscellaneous_options.run(selection_index) == NO){
          rc = NO;
          exit_dialog = YES;
        }
      }
      break;
/*
    case LOGICAL_CHANNELS_SETUP:
      //for the AFT card:
      //1. get number of logical channels
      //2. for each log. cha-l get the timeslots it will use
      //3. for each log. cha-l get the protocol
      {
        menu_aft_logical_channels_list aft_logical_channels_list(lxdialog_path, ptr_cfr);
        if(aft_logical_channels_list.run(selection_index) == NO){
          rc = NO;
          exit_dialog = YES;
        }
      }
*/
      break;

    default:

      Debug(DBG_MENU_NEW_DEVICE_CONFIG, ("default case selection: %d\n", *selection_index));

      if(linkconf->card_type == WANOPT_AFT){
        ////////////////////////////////////////////////////////////
        //general case for AFT: more than one Timeslot Group exist.
        //one of the Timeslot Groups was selected for configuration

        list_el_chan_def = (list_element_chan_def*)
                            obj_list->find_element(
                            remove_spaces_in_int_string(get_lxdialog_output_string()));

        if(list_el_chan_def != NULL){

          menu_aft_logical_channel_cfg per_aft_logical_channel_cfg( lxdialog_path,
                                                                  cfr,
                                                                  list_el_chan_def);

          //do NOT use list_el_chan_def on return - it may be invalid!
          rc = per_aft_logical_channel_cfg.run( cfr->wanpipe_number, selection_index, YES,
                                                obj_list->get_size());
          if(rc == YES){
            goto again;
          }
        }else{
          //failed to find the element
#if 1
          ERR_DBG_OUT(("Failed to find the element!! selection: %s\n",
            get_lxdialog_output_string()));
          rc = NO;
          exit_dialog = YES;
#endif
        }//if(list_el_chan_def != NULL)
      }else{
        ERR_DBG_OUT(("Card type is invalid! linkconf->card_type: %d\n", linkconf->card_type));
        rc = NO;
        exit_dialog = YES;
      }//if(linkconf->card_type == WANOPT_AFT)
      ///////////////////////////////////////////////////////
    }//switch()
    break;

  case MENU_BOX_BUTTON_HELP:
    switch(atoi(get_lxdialog_output_string()))
    {
    case HW_SETUP:
    case PROTOCOL_SETUP:
    case NET_IF_SETUP:
    case MISCELLANEOUS_OPTIONS:
      tb.show_help_message(lxdialog_path, NO_PROTOCOL_NEEDED, new_dev_cfg_help_str);
      break;
/*
    case LOGICAL_CHANNELS_SETUP:
      tb.show_help_message(lxdialog_path, NO_PROTOCOL_NEEDED, logical_channel_setup_help_str);
      break;
*/
    }
    break;

  case MENU_BOX_BUTTON_EXIT:

    //run checks on the configuration here. notify user about errors.
    if(linkconf->config_id == PROTOCOL_NOT_SET){
      tb.show_error_message(lxdialog_path, NO_PROTOCOL_NEEDED,
        "Protocol is NOT selected!!");
      break;
    }
/*
    ip_setup_completion_check = ip_address_configuration_complete(cfr->obj_list);
    if(ip_setup_completion_check != NULL){

      replace_new_line_with_space(ip_setup_completion_check);

      tb.show_error_message(lxdialog_path, NO_PROTOCOL_NEEDED,
        ip_setup_completion_check);
      break;
    }
*/
    if(linkconf->card_type == WANOPT_AFT){

      if(check_aft_timeslot_groups_cfg() == NO){
        break;
      }

    }else{
      ip_setup_completion_check = ip_address_configuration_complete(cfr->obj_list);
      if(ip_setup_completion_check != NULL){

        replace_new_line_with_space(ip_setup_completion_check);

        tb.show_error_message(lxdialog_path, NO_PROTOCOL_NEEDED,
          ip_setup_completion_check);
        break;
      }
    }

    ///////////////////////////////////////////////////////////
    //if no errors found ask confirmation to save to file.
    ///////////////////////////////////////////////////////////
    snprintf(tmp_buff, MAX_PATH_LENGTH, "Do you want save '%s.conf' configuration file?",
      cfr->link_defs->name);

    if(yes_no_question(   selection_index,
                          lxdialog_path,
                          NO_PROTOCOL_NEEDED,
                          tmp_buff) == NO){
      //error displaying dialog
      rc = NO;
      goto cleanup;
    }

    switch(*selection_index)
    {
    case YES_NO_TEXT_BOX_BUTTON_YES:
      {
        conf_file_writer cf_writer(cfr);
        cf_writer.write();
      }
      break;

    case YES_NO_TEXT_BOX_BUTTON_NO:
      break;
    }
    ///////////////////////////////////////////////////////////
    rc = YES;
    exit_dialog = YES;
    break;
  }//switch(*selection_index)

  if(exit_dialog == NO){
    goto again;
  }

cleanup:
  return rc;
  ////////////////////////////////////////////////////
}

char* menu_new_device_configuration::ip_address_configuration_complete(objects_list * obj_list)
{
  static char err_buf[MAX_PATH_LENGTH];

  Debug(DBG_MENU_NEW_DEVICE_CONFIG, ("ip_address_configuration_complete()\n"));

  if(obj_list == NULL){
    ERR_DBG_OUT(("ip_address_configuration_complete(): obj_list == NULL!!\n"));
    snprintf(err_buf, MAX_PATH_LENGTH,
      "ip_address_configuration_complete(): obj_list == NULL!!\n");
    return err_buf;
  }

  list_element_chan_def* list_el_chan_def = (list_element_chan_def*)obj_list->get_first();

  while(list_el_chan_def != NULL){

    Debug(DBG_MENU_NEW_DEVICE_CONFIG,
      ("list_el_chan_def->data.addr: %s\n", list_el_chan_def->data.addr));

    switch(get_used_by_integer_value(list_el_chan_def->data.usedby))
    {
    case WANPIPE:
    case BRIDGE_NODE:
      {
        Debug(DBG_MENU_NEW_DEVICE_CONFIG,
          ("attempting to read 'net interface file' for %s\n", list_el_chan_def->data.addr));

        net_interface_file_reader interface_file_reader(list_el_chan_def->data.name);

        if(interface_file_reader.parse_net_interface_file() == NO){
          Debug(DBG_MENU_NEW_DEVICE_CONFIG, ("parse_net_interface_file() FAILED!!\n"));

          snprintf(err_buf, MAX_PATH_LENGTH,
            "Failed to parse 'Net Interface' file for %s!!\n", list_el_chan_def->data.name);
          return err_buf;
        }
        /////////////////////////////////////////////////////////////////
        //file was parsed, check the contents is complete
        if( strlen(interface_file_reader.if_config.ipaddr) <  MIN_IP_ADDR_STR_LEN ||
          strlen(interface_file_reader.if_config.point_to_point_ipaddr) < MIN_IP_ADDR_STR_LEN){
                                                         //  v NO SPACE HERE!!
          snprintf(err_buf, MAX_PATH_LENGTH, "IP Setup for %sis Incomplete!",
            interface_file_reader.if_config.device);
          return err_buf;
        }else{
          //ip cfg is complete
          //snprintf(tmp_buff, MAX_PATH_LENGTH, " \"IP Address Setup---> Complete\" ");
        }
      }
      break;

    default:
      Debug(DBG_MENU_NEW_DEVICE_CONFIG,
        ("NO NEED to read 'net interface file' for %s\n", list_el_chan_def->data.addr));
      break;
    }

    list_el_chan_def =
      (list_element_chan_def*)obj_list->get_next_element(list_el_chan_def);

  }//while()

  return NULL;
}

int menu_new_device_configuration::create_logical_channels_list_str(string& menu_str)
{
  char tmp_buff[MAX_PATH_LENGTH];

  unsigned int max_valid_number_of_logical_channels;
  sdla_te_cfg_t* te_cfg = &cfr->link_defs->linkconf->te_cfg;

  conf_file_reader* lip_layer_file_reader;

  if(te_cfg->media == WANOPT_MEDIA_T1){

    max_valid_number_of_logical_channels = NUM_OF_T1_CHANNELS;
  }else if(te_cfg->media == WANOPT_MEDIA_T1){

    max_valid_number_of_logical_channels = NUM_OF_E1_CHANNELS;
  }else{
    ERR_DBG_OUT(("Invalid Media Type!!\n"));
    return NO;
  }

  objects_list * obj_list = cfr->obj_list;
  list_element_chan_def* list_el_chan_def = NULL;

  //special case - only one UNCHANNELIZED device.
  if(obj_list->get_size() == 1){

    list_el_chan_def = (list_element_chan_def*)obj_list->get_first();

    snprintf(tmp_buff, MAX_PATH_LENGTH, " \"%s\" ", list_el_chan_def->data.addr);
    menu_str += tmp_buff;

    printf("list_el_chan_def->data.name: %s\n", list_el_chan_def->data.name);
    if(strcmp(list_el_chan_def->data.name, CHAN_NOT_CONFIGURED) == 0){
      printf("found CHAN_NOT_CONFIGURED\n");
    }

    if(strcmp(list_el_chan_def->data.name, CHAN_NOT_CONFIGURED)){

      lip_layer_file_reader = (conf_file_reader*)list_el_chan_def->conf_file_reader;

      if( lip_layer_file_reader != NULL){

        snprintf(tmp_buff, MAX_PATH_LENGTH, " \"%s %s\" ",
          "Protocol-------->",
        get_protocol_string(lip_layer_file_reader->link_defs->config_id));
        menu_str += tmp_buff;

        /////////////////////////////////////////////////
        snprintf(tmp_buff, MAX_PATH_LENGTH, " \"%d\" ", NET_IF_SETUP);
        menu_str += tmp_buff;

        //create the menu item
        if(lip_layer_file_reader->obj_list->get_size()){
          snprintf(tmp_buff, MAX_PATH_LENGTH, " \"Interface Setup-> %d Defined\" ",
            lip_layer_file_reader->obj_list->get_size());
        }else{
          snprintf(tmp_buff, MAX_PATH_LENGTH, " \"Interface Setup->  Undefined\" ");
        }
        menu_str += tmp_buff;
        /////////////////////////////////////////////////
      }else{

        ERR_DBG_OUT(("Invalid 'lip_layer_file_reader' pointer!!\n"));
        return NO;
      }

    }else{

      snprintf(tmp_buff, MAX_PATH_LENGTH, " \"%s: %s\" ",
        TIME_SLOT_GROUP_STR, "Unconfigured!");
      menu_str += tmp_buff;
    }

    return YES;
  }

  if(obj_list->get_size() > 0 && obj_list->get_size() <= max_valid_number_of_logical_channels){

    list_el_chan_def = (list_element_chan_def*)obj_list->get_first();
    //int i = 0;
    while(list_el_chan_def != NULL){

      snprintf(tmp_buff, MAX_PATH_LENGTH, " \"%s\" ", list_el_chan_def->data.addr);
      menu_str += tmp_buff;

      printf("list_el_chan_def->data.name: %s\n", list_el_chan_def->data.name);
      if(strcmp(list_el_chan_def->data.name, CHAN_NOT_CONFIGURED) == 0){
        printf("found CHAN_NOT_CONFIGURED\n");
      }

      if(strcmp(list_el_chan_def->data.name, CHAN_NOT_CONFIGURED)){

        lip_layer_file_reader = (conf_file_reader*)list_el_chan_def->conf_file_reader;

        if( lip_layer_file_reader != NULL){

          snprintf(tmp_buff, MAX_PATH_LENGTH, " \"%s %s: %s\" ",
            TIME_SLOT_GROUP_STR, list_el_chan_def->data.addr,
            get_protocol_string(lip_layer_file_reader->link_defs->config_id));

        }else{
          ERR_DBG_OUT(("Invalid 'lip_layer_file_reader' pointer!!\n"));
          return NO;
        }

      }else{
        snprintf(tmp_buff, MAX_PATH_LENGTH, " \"%s: %s\" ",
          TIME_SLOT_GROUP_STR, "Unconfigured!");
      }

      menu_str += tmp_buff;

      list_el_chan_def =
        (list_element_chan_def*)obj_list->get_next_element(list_el_chan_def);
    }
  }else{
    ERR_DBG_OUT(("Invalid number of %s: %d!!\n", TIME_SLOT_GROUPS_STR, obj_list->get_size()));
    return NO;
  }

  return YES;
}

//all Timeslot Groups check
int menu_new_device_configuration::check_aft_timeslot_groups_cfg()
{
  objects_list * obj_list = cfr->obj_list;
  list_element_chan_def* list_el_chan_def = NULL;

  Debug(DBG_MENU_NEW_DEVICE_CONFIG,
      ("int menu_new_device_configuration::check_aft_timeslot_groups_cfg()\n"));

  list_el_chan_def = (list_element_chan_def*)obj_list->get_first();
  //int i = 0;
  while(list_el_chan_def != NULL){

    Debug(DBG_MENU_NEW_DEVICE_CONFIG,
      ("list_el_chan_def->data.addr: %s\n", list_el_chan_def->data.addr));

    if(check_aft_timeslot_group_cfg(list_el_chan_def) == NO){
      return NO;
    }

    list_el_chan_def =
      (list_element_chan_def*)obj_list->get_next_element(list_el_chan_def);
  }

  return YES;
}

//per-Timeslot Group check
int menu_new_device_configuration::
  check_aft_timeslot_group_cfg(list_element_chan_def* list_el_chan)
{
  conf_file_reader* lip_layer_file_reader;
  objects_list * obj_list = NULL;
//  list_element_chan_def* list_el_chan_def = NULL;
  char* ip_setup_completion_check;
  //help text box
  text_box tb;

  Debug(DBG_MENU_NEW_DEVICE_CONFIG,
      ("int menu_new_device_configuration::check_aft_timeslot_group_cfg()\n"));

  lip_layer_file_reader = (conf_file_reader*)list_el_chan->conf_file_reader;
  obj_list = lip_layer_file_reader->obj_list;

/*
  list_el_chan_def = (list_element_chan_def*)obj_list->get_first();
  //int i = 0;
  while(list_el_chan_def != NULL){

    Debug(DBG_MENU_NEW_DEVICE_CONFIG,
      ("list_el_chan_def->data.addr: %s\n", list_el_chan_def->data.addr));

    list_el_chan_def =
      (list_element_chan_def*)obj_list->get_next_element(list_el_chan_def);
  }
*/

  ip_setup_completion_check = ip_address_configuration_complete(obj_list);
  if(ip_setup_completion_check != NULL){

    replace_new_line_with_space(ip_setup_completion_check);

    tb.show_error_message(lxdialog_path, NO_PROTOCOL_NEEDED,
                          ip_setup_completion_check);
    return NO;
  }

  return YES;
}
