/***************************************************************************
                          menu_protocol_mode_select.cpp  -  description
                             -------------------
    begin                : Fri Mar 12 2004
    copyright            : (C) 2004 by David Rokhvarg
    email                : davidr@sangoma.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "menu_protocol_mode_select.h"

#include "wancfg.h"
#include "text_box.h"


menu_protocol_mode_select::menu_protocol_mode_select(IN char * lxdialog_path,
                                                     IN conf_file_reader* cfr)
{
  Debug(DBG_MENU_PROTOCOL_MODE_SELECT,
    ("menu_protocol_mode_select::menu_protocol_mode_select()\n"));
  snprintf(this->lxdialog_path, MAX_PATH_LENGTH, "%s", lxdialog_path);
  this->cfr = cfr;
}

menu_protocol_mode_select::~menu_protocol_mode_select()
{
  Debug(DBG_MENU_PROTOCOL_MODE_SELECT,
    ("menu_protocol_mode_select::~menu_protocol_mode_select()\n"));
}

int menu_protocol_mode_select::run(OUT int * selected_protocol)
{
  string menu_str;
  int rc;
  char tmp_buff[MAX_PATH_LENGTH];
  //help text box
  text_box tb;

  link_def_t * link_def;
  wandev_conf_t *linkconf;

  Debug(DBG_MENU_PROTOCOL_MODE_SELECT, ("menu_new_device_configuration::run()\n"));

again:
  rc = YES;
  link_def = cfr->link_defs;
  linkconf = cfr->link_defs->linkconf;

  menu_str = "";
  for(int protocol = WANCONFIG_X25; protocol < WANCONFIG_X25 + NUMBER_OF_PROTOCOLS; protocol++){

    if(is_protocol_valid_for_card_type( protocol, linkconf->card_type,
                                        link_def->card_version,
                                        linkconf->comm_port) == YES){
      ///////////////////////////////////////////////////
      //create and insert tag string
      snprintf(tmp_buff, MAX_PATH_LENGTH, "\"%d\" ", protocol);
      menu_str += tmp_buff;
      ///////////////////////////////////////////////////
      //crete and insert item string
      snprintf(tmp_buff, MAX_PATH_LENGTH, "\"%s\" ", get_protocol_string(protocol));
      menu_str += tmp_buff;
      /////////////////////////////////////////////////
    }//if()
  }//for()

  snprintf(tmp_buff, MAX_PATH_LENGTH,
"Please select a WAN Protocol or an Operational Mode.\
 This list displays protocols valid for current Card Type and Port.");

  //snprintf(&tmp_buff[strlen(tmp_buff)], MAX_PATH_LENGTH, "%s", MENUINSTR_EXIT);

  if(set_configuration(   YES,//indicates to call V2 of the function
                          MENU_BOX_BACK,
                          lxdialog_path,
                          "PROTOCOL SELECTION",
                          WANCFG_PROGRAM_NAME,
                          tmp_buff,
                          MENU_HEIGTH, MENU_WIDTH,
                          8,//MAX_NUM_OF_VISIBLE_ITEMS_IN_MENU,
                          (char*)menu_str.c_str()
                          ) == NO){
    rc = NO;
    goto cleanup;
  }

  int selection_index;
  if(show(&selection_index) == NO){
    rc = NO;
    goto cleanup;
  }

  switch(selection_index)
  {
  case MENU_BOX_BUTTON_SELECT:
    Debug(DBG_MENU_PROTOCOL_MODE_SELECT,
      ("option selected: %s\n", get_lxdialog_output_string()));

    *selected_protocol = atoi(get_lxdialog_output_string());
    break;

  case MENU_BOX_BUTTON_HELP:
    tb.show_help_message(lxdialog_path, NO_PROTOCOL_NEEDED, option_not_implemented_yet_help_str);
    goto again;
    break;

  case MENU_BOX_BUTTON_EXIT:
    //exit the dialog
    break;
  }

cleanup:
  return rc;
}

/*
//FIXIT:
//this is for final version of this program, when configuration
//of all protocols is implemented
//
int is_protocol_valid_for_S508(int protocol)
{
  int rc = NO;

  switch(protocol)
  {
  case WANCONFIG_X25:
  case WANCONFIG_FR:
  case WANCONFIG_PPP:
  case WANCONFIG_CHDLC:
  case WANCONFIG_HDLC:
  case WANCONFIG_BSC:
  case WANCONFIG_MPPP:
//  case WANCONFIG_MPROT WANCONFIG_MPPP:
  case WANCONFIG_MFR:
  case WANCONFIG_SDLC:
  case WANCONFIG_MLINK_PPP:
    rc = YES;
    break;
  }
  return rc;
}

int is_protocol_valid_for_S514(int protocol, char card_version)
{
  int rc = NO;

  if( card_version == S5141_ADPTR_1_CPU_SERIAL ||
      card_version == S5143_ADPTR_1_CPU_FT1)
  {
    switch(protocol)
    {
    case WANCONFIG_X25:
    case WANCONFIG_FR:
    case WANCONFIG_PPP:
    case WANCONFIG_CHDLC:
    case WANCONFIG_HDLC:
    case WANCONFIG_BSC:
    case WANCONFIG_MPPP:
    case WANCONFIG_MLINK_PPP:
//  case WANCONFIG_MPROT WANCONFIG_MPPP:
    case WANCONFIG_BITSTRM:
    case WANCONFIG_EDUKIT:
    case WANCONFIG_SS7:
    case WANCONFIG_BSCSTRM:
    case WANCONFIG_MFR:
    case WANCONFIG_SDLC:
    case WANCONFIG_ATM:
    case WANCONFIG_POS:
    case WANCONFIG_ADCCP:
//  case WANCONFIG_DEBUG:
//  case WANCONFIG_GENERIC:
      rc = YES;
      break;
    }//switch()

  }else if( card_version == S5144_ADPTR_1_CPU_T1E1){

    switch(protocol)
    {
    case WANCONFIG_FR:
    case WANCONFIG_PPP:
    case WANCONFIG_CHDLC:
    case WANCONFIG_HDLC:
    case WANCONFIG_MPPP:
    case WANCONFIG_MLINK_PPP:
//  case WANCONFIG_MPROT WANCONFIG_MPPP:
    case WANCONFIG_BITSTRM:
    case WANCONFIG_SS7:
    case WANCONFIG_MFR:
    case WANCONFIG_ATM:
//  case WANCONFIG_DEBUG:
//  case WANCONFIG_GENERIC:
      rc = YES;
      break;
    }//switch()

  }else if( card_version == S5145_ADPTR_1_CPU_56K){
    switch(protocol)
    {
    case WANCONFIG_FR:
    case WANCONFIG_PPP:
    case WANCONFIG_CHDLC:
    case WANCONFIG_HDLC:
    case WANCONFIG_MPPP:
    case WANCONFIG_MLINK_PPP:
//  case WANCONFIG_MPROT WANCONFIG_MPPP:
    case WANCONFIG_MFR:
//  case WANCONFIG_DEBUG:
//  case WANCONFIG_GENERIC:
      rc = YES;
      break;
    }
  }
  return rc;
}
*/

int is_protocol_valid_for_S508(int protocol, int comm_port)
{
  int rc = NO;

  if(comm_port == WANOPT_PRI){
    switch(protocol)
    {
    case WANCONFIG_FR:
    case WANCONFIG_PPP:
    case WANCONFIG_CHDLC:
    case WANCONFIG_HDLC:
      rc = YES;
      break;
    }//switch()
  }else{
    switch(protocol)
    {
    case WANCONFIG_CHDLC:
    case WANCONFIG_HDLC:
      rc = YES;
      break;
    }//switch()
  }
  return rc;
}

int is_protocol_valid_for_S514(int protocol, char card_version, int comm_port)
{
  int rc = NO;

  if( card_version == S5141_ADPTR_1_CPU_SERIAL ||
      card_version == S5143_ADPTR_1_CPU_FT1)
  {
    if(comm_port == WANOPT_PRI){
      switch(protocol)
      {
      case WANCONFIG_FR:
      case WANCONFIG_PPP:
      case WANCONFIG_CHDLC:
      case WANCONFIG_HDLC:
        rc = YES;
        break;
      }//switch()
    }else{
      switch(protocol)
      {
      case WANCONFIG_CHDLC:
      case WANCONFIG_HDLC:
        rc = YES;
        break;
      }//switch()
    }

  }else if( card_version == S5144_ADPTR_1_CPU_T1E1){

    //there is no Secodary port on this card.
    switch(protocol)
    {
    case WANCONFIG_FR:
    case WANCONFIG_PPP:
    case WANCONFIG_CHDLC:
    case WANCONFIG_HDLC:
      rc = YES;
      break;
    }//switch()

  }else if( card_version == S5145_ADPTR_1_CPU_56K){

    //there is no Secodary port on this card.
    switch(protocol)
    {
    case WANCONFIG_FR:
    case WANCONFIG_PPP:
    case WANCONFIG_CHDLC:
    case WANCONFIG_HDLC:
      rc = YES;
      break;
    }
  }
  return rc;
}

int is_protocol_valid_for_S518_ADSL(int protocol)
{
  int rc = NO;

  switch(protocol)
  {
  case WANCONFIG_ADSL:
//  case WANCONFIG_GENERIC:
    rc = YES;
    break;
  }
  return rc;
}

/*
 Frame Relay Protocol (Default)
 PPP Protocol
 Cisco HDLC Protocol
 RAW HDLC Protocol
*/

/*
[wp8aft1]
ACTIVE_CH       = ALL
PROTOCOL        = MP_FR
IGNORE_DCD      = NO
IGNORE_CTS      = NO
HDLC_STREAMING  = YES
DYN_INTR_CFG    = NO
*/

/*
[wp8aft1]
ACTIVE_CH	= ALL
PROTOCOL	= MP_PPP
IGNORE_DCD	= NO
IGNORE_CTS	= NO
HDLC_STREAMING	= YES
DYN_INTR_CFG	= NO
*/

/*
[wp8aft1]
ACTIVE_CH	= ALL
PROTOCOL	= MP_CHDLC
IGNORE_DCD	= NO
IGNORE_CTS	= NO
HDLC_STREAMING	= YES
DYN_INTR_CFG	= NO
*/

/*
[wp8aft1]
ACTIVE_CH	= ALL
PROTOCOL	= HDLC
IGNORE_DCD	= NO
IGNORE_CTS	= NO
HDLC_STREAMING	= YES
DYN_INTR_CFG	= NO
*/

//
//for AFT card protocol is selected on per interface basis,
//so here only "AFT Mode" is valid.
//

int is_protocol_valid_for_AFT(int protocol)
{
  int rc = NO;

  switch(protocol)
  {
//  case WANCONFIG_MFR:
//  case WANCONFIG_MPPP:
//  case WANCONFIG_MPROT://CHDLC
  case WANCONFIG_AFT:
    rc = YES;
    break;
  }
  return rc;
}

/*
    case WANCONFIG_MPPP:
	  case WANCONFIG_MPCHDLC:
	  case WANCONFIG_MFR:
	  case WANCONFIG_HDLC:

int is_protocol_valid_for_AFT(int protocol)
{
  int rc = NO;

  switch(protocol)
  {
  case WANCONFIG_MFR:
  case WANCONFIG_MPPP:
  case WANCONFIG_MPCHDLC:
  case WANCONFIG_HDLC:
    rc = YES;
    break;
  }
  return rc;
}
*/
