/***************************************************************************
                          menu_protocol_mode_select.h  -  description
                             -------------------
    begin                : Fri Mar 12 2004
    copyright            : (C) 2004 by David Rokhvarg
    email                : davidr@sangoma.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef MENU_PROTOCOL_MODE_SELECT_H
#define MENU_PROTOCOL_MODE_SELECT_H

#include <menu_base.h>
#include "conf_file_reader.h"

#define DBG_MENU_PROTOCOL_MODE_SELECT  1

int is_protocol_valid_for_S508(int protocol, int comm_port);
int is_protocol_valid_for_S514(int protocol, char card_version, int comm_port);
int is_protocol_valid_for_S518_ADSL(int protocol);
int is_protocol_valid_for_AFT(int protocol);

/**Presents list of valid Protocols and Modes. Returns user selection to the caller.
  *@author David Rokhvarg
  */

class menu_protocol_mode_select : public menu_base  {

  char lxdialog_path[MAX_PATH_LENGTH];
  conf_file_reader* cfr;

public:
	menu_protocol_mode_select(IN char * lxdialog_path, IN conf_file_reader* cfr);
	~menu_protocol_mode_select();

  int run(OUT int * selected_protocol);
//  int is_protocol_valid_for_card_type(int protocol, char card_type,
//                                      char card_version, int comm_port);
static int is_protocol_valid_for_card_type( int protocol, char card_type,
                                            char card_version, int comm_port)
{
  int rc;

  Debug(DBG_MENU_PROTOCOL_MODE_SELECT,
    ("is_protocol_valid_for_card_type():\n"));

  Debug(DBG_MENU_PROTOCOL_MODE_SELECT,
    ("protocol (%d): %s\n", protocol, get_protocol_string(protocol)));

  Debug(DBG_MENU_PROTOCOL_MODE_SELECT,
    ("card_type: %d, card_version: %d, comm_port: %d\n",
      card_type, card_version, comm_port));

  if(protocol == PROTOCOL_NOT_SET){
    Debug(DBG_MENU_PROTOCOL_MODE_SELECT, ("protocol == PROTOCOL_NOT_SET\n"));
    //protocol is not set yet any hardware is OK
    return YES;
  }

  switch(card_type)
  {
  case WANOPT_S50X:
    rc = is_protocol_valid_for_S508(protocol, comm_port);
    break;
  case WANOPT_S51X:
    rc = is_protocol_valid_for_S514(protocol, card_version, comm_port);
    break;
  case WANOPT_ADSL:
    rc = is_protocol_valid_for_S518_ADSL(protocol);
    break;
  case WANOPT_AFT:
    rc = is_protocol_valid_for_AFT(protocol);
    break;
  default:
    //card type is not set. NO protocol is valid in this case.
    rc = NO;
  }
  return rc;
}

};

#endif
