/***************************************************************************
                          menu_select_and_configure_protocol.h  -  description
                             -------------------
    begin                : Tue Mar 16 2004
    copyright            : (C) 2004 by David Rokhvarg
    email                : davidr@sangoma.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef MENU_SELECT_AND_CONFIGURE_PROTOCOL_H
#define MENU_SELECT_AND_CONFIGURE_PROTOCOL_H

#include <menu_base.h>
#include "conf_file_reader.h"

#define DBG_MENU_SELECT_AND_CONFIGURE_PROTOCOL  1

/**
  *@author David Rokhvarg
  */

class menu_select_and_configure_protocol : public menu_base  {
  char lxdialog_path[MAX_PATH_LENGTH];
  conf_file_reader* cfr;
  conf_file_reader** ptr_cfr;

  int handle_protocol_mode_selection();
  int display_protocol_cfg(IN int protocol);
  int handle_protocol_change(unsigned int protocol_selected);

  char tmp_buff[MAX_PATH_LENGTH];

public:

///////////////////////////////////////////////////////////////////////
  static int update_interface_name(  list_element_chan_def* list_el_chan_def,
                                     int protocol,
                                     int wanpipe_number)
  {
    int rc = YES;
  	chan_def_t* chandef = &list_el_chan_def->data;

    Debug(DBG_MENU_SELECT_AND_CONFIGURE_PROTOCOL, ("update_interface_name()\n"));
    Debug(DBG_MENU_SELECT_AND_CONFIGURE_PROTOCOL,
      ("protocol: %d, wanpipe_number: %d\n", protocol, wanpipe_number));

    Debug(DBG_MENU_SELECT_AND_CONFIGURE_PROTOCOL,
      ("chandef->name: %s\n", chandef->name));

    switch(protocol)
    {
    case WANCONFIG_FR:
      if (strcmp(chandef->addr,"auto")==0){
        snprintf(chandef->name, WAN_IFNAME_SZ, "wp%dfr", wanpipe_number);
      }else{
        snprintf(chandef->name, WAN_IFNAME_SZ, "wp%dfr%s",
          wanpipe_number, chandef->addr);
      }
      break;

    case WANCONFIG_PPP:
      snprintf(chandef->name, WAN_IFNAME_SZ, "wp%dppp", wanpipe_number);
      break;

    case WANCONFIG_CHDLC:
      snprintf(chandef->name, WAN_IFNAME_SZ, "wp%dchdlc", wanpipe_number);
      break;

    case WANCONFIG_HDLC:
      snprintf(chandef->name, WAN_IFNAME_SZ, "wp%dhdlc", wanpipe_number);
      break;

    case WANCONFIG_AFT:

      if(strcmp(chandef->name, IF_NAME_NOT_INIT) == 0){

        snprintf(chandef->name, WAN_IFNAME_SZ, "w%dg%s", wanpipe_number, chandef->addr);

        Debug(DBG_MENU_SELECT_AND_CONFIGURE_PROTOCOL,
          ("for AFT name is not initialized!! initializing to: %s\n", chandef->name));
      }else{
        Debug(DBG_MENU_SELECT_AND_CONFIGURE_PROTOCOL,
          ("AFT name is initialized to: %s. NOT changing\n", chandef->name));
      }

      break;

    default:
      ERR_DBG_OUT(("Invalid new protocol! (0x%X)", protocol));
      rc = NO;
    }

    return rc;
  }//update_interface_name()

///////////////////////////////////////////////////////////////////////

#define AFT_TEMPLATE_FILE         "aft.template"
#define CHDLC_TEMPLATE_FILE       "chdlc.template"
#define HDLC_STRM_TEMPLATE_FILE   "hdlc_strm.template"
#define FRAME_RELAY_TEMPLATE_FILE "frame_relay.template"
#define PPP_TEMPLATE_FILE         "ppp.template"

  static char* get_template_file_path(unsigned int protocol_selected)
  {
    static char tmp_buff[MAX_PATH_LENGTH];

    switch(protocol_selected)
    {
    case WANCONFIG_FR:
    case WANCONFIG_MFR:
      snprintf(tmp_buff, MAX_PATH_LENGTH, "%s%s", wancfg_cfg_dir, FRAME_RELAY_TEMPLATE_FILE);
      break;

    case WANCONFIG_PPP:
    case WANCONFIG_MPPP:
      snprintf(tmp_buff, MAX_PATH_LENGTH, "%s%s", wancfg_cfg_dir, PPP_TEMPLATE_FILE);
      break;

    case WANCONFIG_CHDLC:
    case WANCONFIG_MPCHDLC:
      snprintf(tmp_buff, MAX_PATH_LENGTH, "%s%s", wancfg_cfg_dir, CHDLC_TEMPLATE_FILE);
      break;

    case WANCONFIG_HDLC:
      //Getting here for all cards (S508/S514/AFT), but actual file is not read if AFT.
      snprintf(tmp_buff, MAX_PATH_LENGTH, "%s%s", wancfg_cfg_dir, HDLC_STRM_TEMPLATE_FILE);
      break;

    case WANCONFIG_AFT:
      //This is HDLC Streaming. done in hardware, no configuration options available.
      //Getting here ONLY on "new card's configuration", not when new protocol selected
      //for "group of channels".
      snprintf(tmp_buff, MAX_PATH_LENGTH, "%s%s", wancfg_cfg_dir, AFT_TEMPLATE_FILE);
      break;

    default:
      ERR_DBG_OUT(("Invalid new protocol selected! (0x%X)", protocol_selected));
      return NULL;
    }

    Debug(DBG_MENU_SELECT_AND_CONFIGURE_PROTOCOL, ("template file path: %s\n", tmp_buff));

    return tmp_buff;
  }
///////////////////////////////////////////////////////////////////////

  conf_file_reader* cfr_tmp;

	menu_select_and_configure_protocol(IN char * lxdialog_path, IN conf_file_reader** ptr_cfr);
	~menu_select_and_configure_protocol();
  int run(OUT int * selected_option);
};

#endif
