#ifndef _WANPIPE_XDLC_H
#define _WANPIPE_XDLC_H 

#include "stddef.h"
#include "wanpipe_abstr.h"
#include "wanpipe_cfg.h"
#include "wanpipe_xdlc_iface.h"

#define	XDLC_HEADER_LEN	20		/* XDLC over Ethernet + a bit more */

#define	XDLC_ACK_PENDING_CONDITION	0x01
#define	XDLC_REJECT_CONDITION		0x02
#define	XDLC_PEER_RX_BUSY_CONDITION	0x04
#define XDLC_RNR_SENT_CONDITION		0x08
#define XDLC_HALF_DUPLEX_THROTTLE	0x10

#define XDLC_RX_Q_H_LIMIT 4
#define XDLC_RX_Q_L_LIMIT 3

	/* Control field templates */
#define	XDLC_I		0x00	/* Information frames */
#define	XDLC_S		0x01	/* Supervisory frames */
#define	XDLC_U		0x03	/* Unnumbered frames */

#define	XDLC_RR		0x01	/* Receiver ready */
#define	XDLC_RNR	0x05	/* Receiver not ready */
#define	XDLC_REJ	0x09	/* Reject */

#if 0
#define	XDLC_SABM	0x2F	/* Set Asynchronous Balanced Mode */
#define	XDLC_SABME	0x6F	/* Set Asynchronous Balanced Mode Extended */
#define	XDLC_DISC	0x43	/* Disconnect */
#define	XDLC_DM		0x0F	/* Disconnected mode */
#define	XDLC_UA		0x63	/* Unnumbered acknowledge */
#define	XDLC_FRMR	0x87	/* Frame reject */
#endif

#define XDLC_SNRM	0x83	/* Set normal response mode: 
				   tx on request by secondary only */	
#define XDLC_SNRME	0xCF
#define XDLC_UA		0x63	/* Acnk for unnumbered cmd (SNRM, SNRME, DISC or SIM)*/
#define XDLC_DM         0x0F	/* This station is in disconnect mode */

#define XDLC_DISC	0x43	/* Do not transmit or rx info */
#define XDLC_RD		0x43	/* This station wants to disconnect */

#define XDLC_FRMR	0x87	/* Invalid frame received: 
				   must rx SNRM, SNRME, DISC or SIM */

#define XDLC_ILLEGAL	0x100	/* Impossible to be a real frame type */
#define XDLC_CORRUPT    0x101   /* Corrupted address field */

#define	XDLC_SPF	0x10	/* Poll/final bit for standard XDLC */
#define	XDLC_EPF	0x01	/* Poll/final bit for extended XDLC */

#define	XDLC_FRMR_W	0x01	/* Control field invalid	*/
#define	XDLC_FRMR_X	0x02	/* I field invalid		*/
#define	XDLC_FRMR_Y	0x04	/* I field too long		*/
#define	XDLC_FRMR_Z	0x08	/* Invalid N(R)			*/

#define	XDLC_POLLOFF	0
#define	XDLC_POLLON	1

	/* XDLC C-bit */
#define XDLC_COMMAND	1
#define XDLC_RESPONSE	2

#if 0
#define	XDLC_ADDR_A	0x03
#define	XDLC_ADDR_B	0x01
#define	XDLC_ADDR_C	0x0F
#define	XDLC_ADDR_D	0x07
#endif

#define TX_TIMEOUT  5
	/* Define Link State constants. */
	enum {
		XDLC_STATE_DISCONNECTED,		/* Disconnected State		*/
		XDLC_STATE_CONNECTING,		/* Awaiting Connection State	*/
		XDLC_STATE_DISCONNECTING, 	/* Awaiting Disconnection State	*/
		XDLC_STATE_CONNECTED,		/* Data Transfer State		*/
		XDLC_STATE_FRAME_REJECT		/* Frame Reject State		*/
	};

#define	XDLC_DEFAULT_MODE		(XDLC_STANDARD | XDLC_SLP | XDLC_DTE)
#define	XDLC_DEFAULT_WINDOW		7		/* Window=7 */
#define	XDLC_DEFAULT_T1			(5 * wpabs_get_hz())	/* T1=5s    */
#define	XDLC_DEFAULT_T2			(1 * wpabs_get_hz())	/* T2=1s    */
#define	XDLC_DEFAULT_N2			20		/* N2=20    */

#define	XDLC_SMODULUS	8
#define	XDLC_EMODULUS	128

#define XDLC_UNREG_CRIT		0
#define XDLC_SMP_CRIT		1
#define XDLC_AWAITING_KICK 	2
#define XDLC_FRAME_SENT		3
#define XDLC_RX_SMP		4
#define XDLC_BH_RUNNING		5
#define XDLC_TIMER_CRIT		6
#define XDLC_REQUEUING		7
#define XDLC_DOWN		8

#define	XDLC_OK			0
#define	XDLC_BADTOKEN		1
#define	XDLC_INVALUE		2
#define	XDLC_CONNECTED		3
#define	XDLC_NOTCONNECTED	4
#define	XDLC_REFUSED		5
#define	XDLC_TIMEDOUT		6
#define	XDLC_NOMEM		7

#define	XDLC_STANDARD		0x00
#define	XDLC_EXTENDED		0x01

#define	XDLC_SLP		0x00
#define	XDLC_MLP		0x02

#define	XDLC_DTE		0x00
#define	XDLC_DCE		0x04

#define	XDLC_PROT		0x00
#define	SDLC_PROT		0x01

#define XDLC_SEC		0x00
#define XDLC_PRI		0x01

/*
 *	Information about the current frame.
 */
struct xdlc_frame {
	unsigned short		type;		/* Parsed type		*/
	unsigned short		nr, ns;		/* N(R), N(S)		*/
	unsigned char		cr;		/* Command/Response	*/
	unsigned char		pf;		/* Poll/Final		*/
	unsigned short		address;	/* address */
	unsigned char		control[2];	/* Original control data*/
};

struct xdlc_protocol_stats {
	unsigned int rr_tx;
	unsigned int rnr_tx;
	unsigned int rej_tx;
	unsigned int sabm_tx;
	unsigned int ua_tx;
	unsigned int disc_tx;
	unsigned int dm_tx;
	unsigned int ifrm_tx;
	unsigned int frmr_tx;
	unsigned int ill_tx;
	
	unsigned int rr_rx;
	unsigned int rnr_rx;
	unsigned int rej_rx;
	unsigned int sabm_rx;
	unsigned int ua_rx;
	unsigned int disc_rx;
	unsigned int dm_rx;
	unsigned int ifrm_rx;
	unsigned int frmr_rx;
	unsigned int ill_rx;
};

#define MAX_XDLC_ADDRS 	0xFF

typedef struct {

	void	 		*dev_map[MAX_XDLC_ADDRS+1];	
	void			*dev;
	unsigned int		state;
	unsigned int 		protocol;
	unsigned long		critical;
	int			devcnt;
	wp_xdlc_reg_t		reg;
	int			refcnt;
}xdlc_link_t;

/*
 *	The per XDLC connection control structure.
 */
typedef struct {
	
	unsigned long		critical;
	
	/* Link status fields */
	wan_xdlc_conf_t		cfg;

	unsigned char		state;
	unsigned char		user_state;
	unsigned int		mode;
	unsigned char		protocol;
	
	volatile unsigned short	vs, vr, va;
	unsigned short 		old_nr,old_nr_type,nr_type;

	unsigned short		condition;
	unsigned short		max_no_response_cnt;
	unsigned short		max_frame_retransmit_cnt;
	unsigned short		max_rnr_cnt;

	unsigned short		sec_nrm_timeout;
	unsigned short		pri_slow, pri_norm, pri_frm_resp;

	unsigned short		tws_max_ack_count;

	unsigned long		pri_poll_timer, pri_frm_timer, sec_nrm_timer;

	/* Internal control information */
	void 			*write_queue;
	void			*ack_queue;
	void			*async_queue;
	void			*rx_queue;
	unsigned char		window;
	unsigned int	 	mtu;
	
	/* FRMR control information */
	struct xdlc_frame	frmr_data;
	unsigned char		frmr_type;

	/* Transmit device */
	void 			*dev;
	void 			*tx_dev;
	unsigned char 		*hw_name;

	unsigned char		name[50];

	/* X25 layer interface */
	xdlc_link_t* xdlc_link;
	void * sk_id;
	
	struct xdlc_protocol_stats xdlcstats;

	unsigned int 		kick_cnt;
	int			refcnt;

}xdlc_dev_t;

/* xdlc_iface.c */
void xdlc_connect_indication(xdlc_dev_t *, int);
void xdlc_disconnect_indication(xdlc_dev_t *, int);
void xdlc_connecting_indication(xdlc_dev_t *, int);
void xdlc_disconnecting_indication(xdlc_dev_t *, int);
int  xdlc_data_indication(xdlc_dev_t * xdlc, void *);
int  xdlc_data_transmit(xdlc_dev_t *, void  *);

/* xdlc_in.c */
void xdlc_data_input(xdlc_dev_t *, void  *);

/* xdlc_out.c */
int xdlc_bh(xdlc_dev_t*);
int xdlc_transmit_buffer(xdlc_dev_t *, void  *, int,char);
void xdlc_establish_data_link(xdlc_dev_t *);
void xdlc_enquiry_response(xdlc_dev_t *);
void xdlc_timeout_response(xdlc_dev_t *);
inline void xdlc_check_iframes_acked(xdlc_dev_t *, unsigned short);
void xdlc_check_need_response(xdlc_dev_t *, int, int);

/* xdlc_subr.c */
xdlc_dev_t* xdlc_create_cb(void);
int 	xdlc_free_cb(xdlc_dev_t*);
void 	xdlc_requeue_frames(xdlc_dev_t *);
void 	xdlc_decode(xdlc_dev_t *, void  *, struct xdlc_frame *);
void 	xdlc_send_control(xdlc_dev_t *, int, int, int);
void 	xdlc_transmit_frmr(xdlc_dev_t *);
void 	xdlc_update_stats (xdlc_dev_t *xdlc, int type, int len, char direction);
void 	xdlc_state_change(xdlc_dev_t *xdlc, int state);
void 	xdlc_link_up (xdlc_dev_t *xdlc);
void 	xdlc_link_down (xdlc_dev_t *xdlc);
void    xdlc_rx_recover(xdlc_dev_t *xdlc);


/* xdlc_timer.c */
void xdlc_timer (void *);

void xdlc_start_pri_frm_resp_timer(xdlc_dev_t *);
void xdlc_start_pri_poll_timer(xdlc_dev_t *);

void xdlc_stop_pri_poll_timer(xdlc_dev_t *);
void xdlc_stop_pri_frm_resp_timer(xdlc_dev_t *);

void xdlc_pri_poll_expiry(xdlc_dev_t *xdlc);
void xdlc_pri_frm_response_expiry(xdlc_dev_t *xdlc);

void xdlc_stop_sec_nrm_timer(xdlc_dev_t *);
void xdlc_start_sec_nrm_timer(xdlc_dev_t *);
void xdlc_sec_nrm_expiry(xdlc_dev_t *xdlc);

#define wp_dev_hold(dev)    do{\
				wpabs_debug_test("%s:%d (dev_hold) Refnt %i!\n",\
							__FUNCTION__,__LINE__,dev->refcnt );\
				dev->refcnt++; \
			    }while(0)

#define wp_dev_put(dev)	    do{\
				wpabs_debug_test("%s:%d (wp_dev_put): Refnt %i!\n",\
							__FUNCTION__,__LINE__,dev->refcnt );\
				if (--dev->refcnt == 0){\
					wpabs_debug_event("%s:%d (wp_dev_put): Deallocating %s!\n",\
							__FUNCTION__,__LINE__,#dev);\
					wpabs_free(dev);\
					dev=NULL;\
				}\
		            }while(0)

#define FUNC_BEGIN() wpabs_debug_test("%s:%d ---Begin---\n",__FUNCTION__,__LINE__); 
#define FUNC_END() wpabs_debug_test("%s:%d ---End---\n\n",__FUNCTION__,__LINE__); 



static __inline void xdlc_condition_reset(xdlc_dev_t *xdlc)
{
	if (xdlc->cfg.station == WANOPT_PRIMARY){
		xdlc->condition=0;
	}else{
		xdlc->condition=0;
		xdlc->condition|=XDLC_HALF_DUPLEX_THROTTLE;
	}
}

/*
 *	This routine purges all the queues of frames.
 */

static __inline void xdlc_clear_queues(xdlc_dev_t *xdlc)
{
	FUNC_BEGIN();
	wpabs_skb_queue_purge(xdlc->write_queue);
	wpabs_skb_queue_purge(xdlc->ack_queue);
	wpabs_skb_queue_purge(xdlc->async_queue);
	wpabs_skb_queue_purge(xdlc->rx_queue);

	xdlc_condition_reset(xdlc);

	xdlc->vs        = 0;
	xdlc->vr        = 0;
	xdlc->va        = 0;
}

/*
 *	Validate that the value of nr is between va and vs. Return true or
 *	false for testing.
 */
static __inline int xdlc_validate_nr(xdlc_dev_t *xdlc, volatile unsigned short nr)
{
	volatile unsigned short vc = xdlc->va;
	volatile int modulus;

	wpabs_debug_test("%s: %s()\n",
			xdlc->name, __FUNCTION__);

	modulus = (xdlc->mode & XDLC_EXTENDED) ? XDLC_EMODULUS : XDLC_SMODULUS;

	while (vc != xdlc->vs) {
		if (nr == vc) {
			xdlc->old_nr=nr;
			return 1;
		}
		vc = (vc + 1) % modulus;
	}
	
	if (nr == xdlc->vs) {
		xdlc->old_nr=nr;
		return 1;
	}
	wpabs_debug_event( "%s: Error : Rx Frame=0x%X Invalid NR=%i, PrevNR=%i PrevType=%i Vs=%i, Va=%i\n",
			xdlc->name, xdlc->nr_type, nr, xdlc->old_nr,
			xdlc->old_nr_type,xdlc->vs,xdlc->va);
	
	return 0;
}


/*
 * This routine purges the input queue of those frames that have been
 * acknowledged. This replaces the boxes labelled "V(a) <- N(r)" on the
 * SDL diagram.
 */
static __inline void xdlc_frames_acked(xdlc_dev_t *xdlc, unsigned short nr)
{
	void *skb;
	volatile int modulus;

	modulus = (xdlc->mode & XDLC_EXTENDED) ? XDLC_EMODULUS : XDLC_SMODULUS;

	/*
	 * Remove all the ack-ed frames from the ack queue.
	 */

	wpabs_debug_test("%s: %s() nr=%i  vs=%i va=%i\n",
			xdlc->name, 
			__FUNCTION__,
			nr,
			xdlc->vs,
			xdlc->va);
	
	if (xdlc->va != nr) {
		while (wpabs_skb_queue_len(xdlc->ack_queue) != 0 && xdlc->va != nr) {
		        skb = wpabs_skb_dequeue(xdlc->ack_queue);
			xdlc->va = (xdlc->va + 1) % modulus;
			wpabs_skb_free(skb);
		}
	}
}


#endif
