/*****************************************************************************
* wanpipe_xdlc_iface.c	WANPIPE XDLC Protocol Module. Main code.
*
* Author:	Nenad Corbic
*
* Copyright:	(c)2004 Sangoma Technologies Inc.
*
*		This program is free software; you can redistribute it and/or
*		modify it under the terms of the GNU General Public License
*		as published by the Free Software Foundation; either version
*		2 of the License, or (at your option) any later version.
* ============================================================================
*	This module:
*		This module is free software; you can redistribute it and/or
*		modify it under the terms of the GNU General Public License
*		as published by the Free Software Foundation; either version
*		2 of the License, or (at your option) any later version.
*/
#include "wanpipe_xdlc.h"

static int xdlc_setparms(xdlc_dev_t *xdlc, wan_xdlc_conf_t *parms);

void *wp_reg_xdlc_prot(void *link_ptr, 
		       char *devname,
		       wan_xdlc_conf_t *conf, 
		       wp_xdlc_reg_t *prot_reg)
{
	xdlc_link_t *xdlc_link;

	xdlc_link = wpabs_malloc(sizeof(xdlc_link_t));
	if (!xdlc_link){
		wpabs_debug_event("%s: Failed to allocate xdlc link %s!\n",
				__FUNCTION__,devname);
		return NULL;
	}
	wpabs_memset(xdlc_link,0,sizeof(xdlc_link_t));

	wp_dev_hold(xdlc_link);
	xdlc_link->dev=link_ptr;

	wpabs_memcpy(&xdlc_link->reg,prot_reg,sizeof(wp_xdlc_reg_t));

	/* There is not Link state in XDCL therefore, just set it
	 * to connected on registration */
	if (xdlc_link->reg.prot_state_change){
		xdlc_link->reg.prot_state_change(xdlc_link->dev,WAN_CONNECTED,NULL,0);
	}
	
	return xdlc_link;
}

int wp_unreg_xdlc_prot(void *xdlc_link_ptr)
{
	xdlc_link_t *xdlc_link = (xdlc_link_t *)xdlc_link_ptr;

	if (xdlc_link->devcnt){
		wpabs_debug_event("%s: Error: Protocol devcnt (%d) is not zero!\n",
				__FUNCTION__,xdlc_link->devcnt);
		return -1;
	}

	if (xdlc_link->reg.prot_state_change){
		xdlc_link->reg.prot_state_change(xdlc_link->dev,WAN_DISCONNECTED,NULL,0);
	}

	wpabs_set_bit(XDLC_UNREG_CRIT, &xdlc_link->critical);

	xdlc_link->dev=NULL;

	wp_dev_put(xdlc_link);

	return 0;
}



void *wp_reg_xdlc_addr(void *callback_dev_ptr, 
		       void *xdlc_link_ptr,
		       char *devname, 
		       wan_xdlc_conf_t *cfg)
{
	xdlc_dev_t  *xdlc;
	xdlc_link_t *xdlc_link=(xdlc_link_t *)xdlc_link_ptr;

	if (cfg->address <= 0 || cfg->address >= MAX_XDLC_ADDRS){
		wpabs_debug_event( "%s: Error: Invalid address (0x%02X) value for dev %s\n",
				__FUNCTION__,cfg->address,devname);
		return NULL;
	}

	if (xdlc_link->dev_map[cfg->address]){
		wpabs_debug_event( "%s: Error: Address (0x%02X) already exists on dev %s\n",
				__FUNCTION__,cfg->address,devname);
		return NULL;
	}
	
	if ((xdlc = xdlc_create_cb()) == NULL){
		wpabs_debug_event( "XDLC: Failed to create cb %s\n",
				devname);
		return NULL;
	}

	wpabs_strncpy(xdlc->name,devname,sizeof(xdlc->name)-1);

	if (xdlc_setparms(xdlc,cfg)){
		xdlc_free_cb(xdlc);
		return NULL;
	}
	
	xdlc->dev   = callback_dev_ptr;
	xdlc->state = XDLC_STATE_DISCONNECTED;
	xdlc->user_state=XDLC_STATE_DISCONNECTED;

	xdlc->condition |= XDLC_HALF_DUPLEX_THROTTLE;
	
	xdlc->xdlc_link=xdlc_link;
	wp_dev_hold(xdlc_link);

	wpabs_debug_event("%s: Registering Address 0x%X\n",
			xdlc->name,xdlc->cfg.address);

	xdlc_link->dev_map[cfg->address] = xdlc;
	xdlc_link->devcnt++;
	wp_dev_hold(xdlc);
	
	return xdlc;
}


int wp_unreg_xdlc_addr(void *xdlc_link_ptr,void *xdlc_dev_ptr)
{	
	xdlc_link_t *xdlc_link	=(xdlc_link_t *)xdlc_link_ptr;
	xdlc_dev_t  *xdlc 	=(xdlc_dev_t*)xdlc_dev_ptr;

	if (xdlc->cfg.address <= 0 || xdlc->cfg.address >= MAX_XDLC_ADDRS){
		wpabs_debug_event( "%s: Error: Invalid address (0x%02X) value for dev %s\n",
				__FUNCTION__,xdlc->cfg.address,xdlc->name);
		return -1;
	}

	if (xdlc_link->dev_map[xdlc->cfg.address] != xdlc){
		wpabs_debug_event("%s: Error: Protocol doesn't exist on link dev map!\n",
				__FUNCTION__);
		return -1;
	}
	
	wpabs_set_bit(XDLC_UNREG_CRIT, &xdlc->critical);


	xdlc_link->dev_map[xdlc->cfg.address]=NULL;
	xdlc_link->devcnt--;

	if (xdlc->xdlc_link){
		wp_dev_put(xdlc->xdlc_link);
		xdlc->xdlc_link=NULL;
	}

	wp_dev_put(xdlc);

	xdlc->dev=NULL;
	xdlc_free_cb(xdlc);

	return 0;
}

int wp_xdlc_open(void *xdlc_ptr)
{
	/* Do not start protocol here. The user
	 * application must start us manually */
	return 0;
}

int wp_xdlc_close(void *xdlc_ptr)
{
	xdlc_dev_t *xdlc =(xdlc_dev_t*)xdlc_ptr;

	wpabs_debug_event("%s: CLOSE state 0x%X\n",
			xdlc->name,xdlc->state);

	xdlc->user_state=XDLC_STATE_DISCONNECTED;

	if (xdlc->state == XDLC_STATE_DISCONNECTED ||
	    xdlc->state == XDLC_STATE_DISCONNECTING ){
		return 0;
	}
	
	xdlc_link_down(xdlc);

	return 0;
}

int wp_xdlc_rx_kick(void *xdlc_dev_ptr)
{
	xdlc_dev_t *xdlc=(xdlc_dev_t*)xdlc_dev_ptr;
	void *newskb;

	if (!wpabs_skb_queue_len(xdlc->rx_queue)){
		return 0;
	}
	
	while ((newskb=wpabs_skb_dequeue(xdlc->rx_queue)) != NULL){
		if (xdlc_data_indication(xdlc, newskb) != 0){
			wpabs_debug_test( "%s: Failed to pass data up : top layer busy!\n",
					xdlc->name);
			wpabs_skb_queue_head(xdlc->rx_queue,newskb);
			break;
		}
	}

	if (wpabs_skb_queue_len(xdlc->rx_queue) < xdlc->window){
		xdlc->condition &= ~XDLC_RNR_SENT_CONDITION;
		xdlc->condition &= ~XDLC_REJECT_CONDITION;
		xdlc->condition |= XDLC_ACK_PENDING_CONDITION;
	}
	
	return 0;
}


int wp_xdlc_rx(void *xdlc_link_ptr, void *skb)
{
	xdlc_link_t *xdlc_link 	= (xdlc_link_t *)xdlc_link_ptr;
	unsigned char *data	= wpabs_skb_data(skb);
	xdlc_dev_t *xdlc_dev;
	unsigned int address;

	FUNC_BEGIN();

	if (wpabs_skb_len(skb) < 2){
		wpabs_debug_event("%s: Error invalid rx pkt Len=%d\n",
				__FUNCTION__,wpabs_skb_len(skb));
		wpabs_skb_free(skb);
		return 0;
	}

	if ((address=data[0]) > MAX_XDLC_ADDRS){
		wpabs_debug_event("%s: Error invalid rx pkt Data=%d\n",
				__FUNCTION__,address);
		wpabs_skb_free(skb);
		return 0;
	}

	if (!(xdlc_dev=xdlc_link->dev_map[address])){
		wpabs_debug_event("%s: Error: Rx Data on unconfigured address 0x%X\n",
				__FUNCTION__,address);
		wpabs_skb_free(skb);
		return 0;
	}
	wp_dev_hold(xdlc_dev);

	xdlc_data_input(xdlc_dev, skb);

	xdlc_bh(xdlc_dev);
	
	wp_dev_put(xdlc_dev);
	
	FUNC_END();
	return 0;
}

int wp_xdlc_bh(void *xdlc_link_ptr, void *xdlc_dev_ptr)
{
	xdlc_link_t *xdlc_link = (xdlc_link_t *)xdlc_link_ptr;
	xdlc_dev_t *xdlc = (xdlc_dev_t *)xdlc_dev_ptr;
	int devcnt=0;
	int i;
	
	FUNC_BEGIN();

	if (wpabs_test_bit(XDLC_UNREG_CRIT, &xdlc_link->critical)){
		return -1;
	}


	if (xdlc){
		if (wpabs_test_bit(XDLC_UNREG_CRIT, &xdlc->critical)){
			return -1;
		}

		xdlc_bh(xdlc);
		return 0;
	}
	
	for (i=0;i<MAX_XDLC_ADDRS;i++){

		xdlc=xdlc_link->dev_map[i];
		if (!xdlc){
			continue;
		}

		if (wpabs_test_bit(XDLC_UNREG_CRIT, &xdlc->critical)){
			continue;
		}

		wp_dev_hold(xdlc);

		xdlc_bh(xdlc);

		wp_dev_put(xdlc);

		if (++devcnt > xdlc_link->devcnt){
			break;
		}
	}

	FUNC_BEGIN();

	return 0;
}


int wp_xdlc_tx(void *xdlc_ptr, void *skb)
{
	xdlc_dev_t *xdlc = (xdlc_dev_t*)xdlc_ptr;

	if (wpabs_skb_tailroom(skb) < 4 || wpabs_skb_headroom(skb) < 6){
		wpabs_debug_event("xdlc: No skb room for header H %d T %d!!!\n",
				wpabs_skb_headroom(skb), wpabs_skb_tailroom(skb));
		wpabs_skb_free(skb);
		return 0;
	}

	if (xdlc->state != XDLC_STATE_CONNECTED){ 
		return 1;
	}

#if 0
	if (wpabs_skb_queue_len(xdlc->write_queue) >= xdlc->window ||
	    (wpabs_skb_queue_len(xdlc->write_queue)+
	     wpabs_skb_queue_len(xdlc->ack_queue)) >= xdlc->window){
#else
	if (wpabs_skb_queue_len(xdlc->write_queue) >= xdlc->window){
#endif
		wpabs_debug_test("%s: Interface busy queue > Max W=%d A=%d\n",
				xdlc->name,
				wpabs_skb_queue_len(xdlc->write_queue),
				wpabs_skb_queue_len(xdlc->ack_queue));
		return 1;
	}

#ifdef __DEBUG_DROP_X25_PACKET_
	if (wpabs_skb_len(skb) >= 3){
		unsigned int data = wpabs_skb_data(skb);
		unsigned short lcn=((data[0]&0x0F)<<8) | data[1];
		
		if (data[2] == 0x13 && lcn == 10){
			wpabs_debug_event("%s: Dropping Clear Call :)\n",xdlc->name);
			wpabs_skb_free(skb);
			return 0;
		}
	}	
#endif

	wpabs_skb_unlink(skb);
	
	wpabs_skb_queue_tail(xdlc->write_queue, skb);

	return 0;
}

void wp_xdlc_timer(void *xdlc_link_ptr)
{
	xdlc_link_t *xdlc_link = (xdlc_link_t *)xdlc_link_ptr;
	xdlc_dev_t *xdlc;
	unsigned long period = wpabs_get_systemticks();
	int i,devcnt=0;
	int kick_dev=1;

	FUNC_BEGIN();
	
	if (wpabs_test_bit(XDLC_UNREG_CRIT, &xdlc_link->critical)){
		return;
	}

	for (i=1;i<MAX_XDLC_ADDRS;i++){

		xdlc=xdlc_link->dev_map[i];
		if (!xdlc){
			continue;
		}

		if (wpabs_test_bit(XDLC_UNREG_CRIT, &xdlc->critical)){
			continue;
		}

		wp_dev_hold(xdlc);
		
		if (xdlc->pri_poll_timer){
			unsigned long timeout;	
			if (xdlc->state == XDLC_STATE_CONNECTED){
				timeout = xdlc->pri_norm;
			}else{	
				timeout = xdlc->pri_slow;
			}
			
			if ((period-xdlc->pri_poll_timer) > timeout){
				xdlc->pri_poll_timer=0;
				xdlc_pri_poll_expiry(xdlc);		
				kick_dev=1;
			}
		}

		if (xdlc->pri_frm_timer > 0){
			if ((period-xdlc->pri_frm_timer) > xdlc->pri_frm_resp){
				xdlc->pri_frm_timer=0;
				xdlc_pri_frm_response_expiry(xdlc);		
				kick_dev=1;
			}
		}

		if (xdlc->sec_nrm_timer > 0){
			if ((period-xdlc->sec_nrm_timer) > xdlc->sec_nrm_timeout){
				xdlc->sec_nrm_timer=0;
				xdlc_sec_nrm_expiry(xdlc);		
				kick_dev=1;
			}
		}

		if (kick_dev){
			xdlc_bh(xdlc);
		}

		wp_dev_put(xdlc);

		if (++devcnt > xdlc_link->devcnt){
			break;
		}

	}

	FUNC_END();
}


int wp_xdlc_ioctl(void *xdlc_dev_ptr, int cmd, void *arg)
{
	xdlc_dev_t *xdlc = (xdlc_dev_t*)xdlc_dev_ptr;
	
	switch (cmd){
	
	case SIOCS_XDLC_START:
		xdlc->user_state=XDLC_STATE_CONNECTED;
		if (xdlc->state!=XDLC_STATE_CONNECTED){
			wpabs_debug_event("%s: XDLC Protocol Start by user (state=0x%X)!\n",
					xdlc->name,xdlc->state);
			xdlc_link_up(xdlc);
		}
		break;

	case SIOCS_XDLC_STOP:

		xdlc->user_state=XDLC_STATE_DISCONNECTED;
		
		if (xdlc->state == XDLC_STATE_DISCONNECTED ||
	    	    xdlc->state == XDLC_STATE_DISCONNECTING ){
			return 0;
		}
		
		wpabs_debug_event("%s: XDLC Protocol Stopped by user (state=0x%X)!\n",
				xdlc->name,xdlc->state);
		xdlc_link_down(xdlc);
		break;

	case SIOCS_XDLC_GET_ADDRESS:
		{		
			unsigned int *addr=(unsigned int*)arg;
			if (!arg){
				wpabs_debug_event("%s: Ioctl Error: Get Address no argument!\n",
						xdlc->name);
				return -1;
			}
			*addr=xdlc->cfg.address;
		}
		break;

	default:
		return -1;
	}

	return 0;
}


/*======================================================
 * xdlc_data_transmit
 *
 * Transmitt the xdlc packet to the lower layer,
 * vi its master (tx_dev) device.
 * 
 */
int xdlc_data_transmit(xdlc_dev_t *xdlc, void *skb)
{
	if (xdlc->dev){
		return xdlc->xdlc_link->reg.tx_down(xdlc->dev,skb);
	}
	
	return 1;
}

/*======================================================
 * xdlc_data_indication
 *
 * Transmitt the xdlc packet to the lower layer,
 * vi its master (tx_dev) device.
 * 
 */
int xdlc_data_indication(xdlc_dev_t *xdlc, void *skb)
{
	wpabs_debug_test("%s: %s() Len=%d\n",
			xdlc->name,__FUNCTION__,wpabs_skb_len(skb));

	if (xdlc->dev){
		return xdlc->xdlc_link->reg.rx_up(xdlc->dev,skb);
	}

	return 1;
}



/**********************************************************
 * Private Functions
 **********************************************************/


/*======================================================
 * xdlc_connect_indication
 *
 * Indicate to the upper layer (x25) that the
 * xdlc is layer is connected and that it should
 * start its connection process.
 */
void xdlc_connect_indication(xdlc_dev_t *xdlc, int reason)
{
	wp_xdlc_excep_t xdlc_excep;
	
	wpabs_debug_test("%s: %s() Reason=%d\n",
			xdlc->name,__FUNCTION__,reason);

	xdlc_excep.state=EXCEP_LINK_CONNECTED;
	xdlc_excep.address=xdlc->cfg.address;
	xdlc_excep.exception=reason;

	if (xdlc->dev && xdlc->xdlc_link->reg.chan_state_change){
		xdlc->xdlc_link->reg.chan_state_change(xdlc->dev,
						       WAN_CONNECTED,
						       (unsigned char*)&xdlc_excep,
						       sizeof(wp_xdlc_excep_t));
	}
}
void xdlc_connecting_indication(xdlc_dev_t *xdlc, int reason)
{
	wp_xdlc_excep_t xdlc_excep;

	wpabs_debug_test("%s: %s() Reason=%d\n",
			xdlc->name,__FUNCTION__,reason);

	xdlc_excep.state=EXCEP_LINK_CONNECTING;
	xdlc_excep.address=xdlc->cfg.address;
	xdlc_excep.exception=reason;

	if (xdlc->dev && xdlc->xdlc_link->reg.chan_state_change){
		xdlc->xdlc_link->reg.chan_state_change(xdlc->dev,
						       WAN_CONNECTING,
						       (unsigned char*)&xdlc_excep,
						       sizeof(wp_xdlc_excep_t));
	}
}

void xdlc_disconnecting_indication(xdlc_dev_t *xdlc, int reason)
{
	wp_xdlc_excep_t xdlc_excep;

	wpabs_debug_test("%s: %s() Reason=%d\n",
			xdlc->name,__FUNCTION__,reason);

	xdlc_excep.state=EXCEP_LINK_DISCONNECTING;
	xdlc_excep.address=xdlc->cfg.address;
	xdlc_excep.exception=reason;

	if (xdlc->dev && xdlc->xdlc_link->reg.chan_state_change){
		xdlc->xdlc_link->reg.chan_state_change(xdlc->dev,
						       WAN_CONNECTING,
						       (unsigned char*)&xdlc_excep,
						       sizeof(wp_xdlc_excep_t));
	}
}


/*======================================================
 * xdlc_disconnect_indication
 *
 * Indicate to the upper layer (x25) that the
 * xdlc is layer is disconnected and that it should
 * drop the connection.
 */
void xdlc_disconnect_indication(xdlc_dev_t *xdlc, int reason)
{
	wp_xdlc_excep_t xdlc_excep;

	wpabs_debug_test("%s: %s() Reason=%d\n",
			xdlc->name,__FUNCTION__,reason);

	xdlc_excep.state=EXCEP_LINK_DISCONNECTED;
	xdlc_excep.address=xdlc->cfg.address;
	xdlc_excep.exception=reason;

	if (xdlc->dev && xdlc->xdlc_link->reg.chan_state_change){
		xdlc->xdlc_link->reg.chan_state_change(xdlc->dev,
						       WAN_CONNECTING,
						       (unsigned char*)&xdlc_excep,
						       sizeof(wp_xdlc_excep_t));
	}
}


static int xdlc_setparms(xdlc_dev_t *xdlc, wan_xdlc_conf_t *parms)
{

	if (!parms){
		return 0;
	}

	wpabs_debug_event("%s: XDLC Configuration: \n",xdlc->name);

	wpabs_memcpy(&xdlc->cfg,parms,sizeof(wan_xdlc_conf_t));


	wpabs_debug_event("%s:      Station                : %s\n",
			xdlc->name,
			xdlc->cfg.station==WANOPT_PRIMARY?"Primary":"Secondary");

	wpabs_debug_event("%s:      Address                : 0x%X\n",
			xdlc->name,
			xdlc->cfg.address);

	if (parms->window < 1 || parms->window > 7){
		wpabs_debug_event(
			"%s: Cfg Error: Invalid Window Specified (%d) (Min=1 Max=7)\n",
				xdlc->name,parms->window);
		return -1;
	}

	xdlc->window = parms->window;
	wpabs_debug_event("%s:      Window                 : %d\n",
			xdlc->name,
			parms->window);

	xdlc->mtu   = parms->max_I_field_length;
	wpabs_debug_event("%s:      MAX I Frame            : %d\n",
			xdlc->name,
			parms->max_I_field_length);

	if (xdlc->cfg.station == WANOPT_PRIMARY){

		if (parms->pri_normal_poll_timer < 10){
			wpabs_debug_event(
			"%s: Cfg Error: Invalid Normal Poll Timer (%d) (Min=10) \n",
					xdlc->name,parms->pri_normal_poll_timer);
			return -1;
		}

		xdlc->pri_norm = wpabs_get_hz()*parms->pri_normal_poll_timer/1000;
		wpabs_debug_event("%s:      Pri Normal Poll        : %d (ms %d)\n",
			xdlc->name,
			parms->pri_normal_poll_timer,xdlc->pri_norm);


		if (parms->pri_slow_poll_timer < 10){
			wpabs_debug_event(
			"%s: Cfg Error: Invalid Slow Poll Timer (%d) (Min=10) \n",
					xdlc->name,parms->pri_slow_poll_timer);
			return -1;
		}

		xdlc->pri_slow = wpabs_get_hz()*parms->pri_slow_poll_timer/1000;

		wpabs_debug_event("%s:      Pri Slow Poll          : %d (ms %d)\n",
			xdlc->name,
			parms->pri_slow_poll_timer,xdlc->pri_slow);

	
				
		if (parms->pri_frame_response_timer < 10){
			wpabs_debug_event(
			"%s: Cfg Error: Invalid Frm Response Timer (%d) (Min=10) \n",
					xdlc->name,parms->pri_frame_response_timer);
			return -1;
		}

		xdlc->pri_frm_resp = wpabs_get_hz()*parms->pri_frame_response_timer/1000;

		wpabs_debug_event("%s:      Pri Frame Response Poll: %d (ms)\n",
			xdlc->name,
			parms->pri_frame_response_timer);

	}else{
		if (parms->sec_nrm_timer > 0 && parms->sec_nrm_timer < 10){
			wpabs_debug_event(
				"%s: Cfg Error: Invalid Sec NRM Timer (%d) (Min=10) \n",
					xdlc->name,parms->sec_nrm_timer);
			return -1;
		}

		xdlc->sec_nrm_timeout = wpabs_get_hz()*parms->sec_nrm_timer/1000;

		wpabs_debug_event("%s:      Sec NRM Timeout        : %d (ms)\n",
			xdlc->name,
			parms->sec_nrm_timer);
	}

	wpabs_debug_event("%s:      Max No Response Counter: %d\n",
			xdlc->name,
			parms->max_no_response_cnt);
	xdlc->max_no_response_cnt = 0;


	wpabs_debug_event("%s:      Max Frm Retransmit Cnt : %d\n",
			xdlc->name,
			parms->max_frame_retransmit_cnt);
	xdlc->max_frame_retransmit_cnt=0;

	wpabs_debug_event("%s:      Max RNR Cnt            : %d\n",
			xdlc->name,
			parms->max_rnr_cnt);
	xdlc->max_rnr_cnt=0;

	if (xdlc->cfg.station == WANOPT_PRIMARY){
		wpabs_debug_event("%s:      Protocol Config        : %s \n",
				xdlc->name,
				parms->protocol_config==WANOPT_TWO_WAY_ALTERNATE?
					"Two Way Alt":"Two Way Sim");

		
		if (parms->protocol_config==WANOPT_TWO_WAY_SIMULTANEOUS){

			if (parms->TWS_max_ack_count <= xdlc->window){
				wpabs_debug_event("%s: Invalid TWS ack count %d < Windown(%d)\n",
							xdlc->name,parms->TWS_max_ack_count,
							xdlc->window);
				return -1;
			}
			
			wpabs_debug_event("%s:      TWS Max Ack Count      : %d \n",
					xdlc->name,
					parms->TWS_max_ack_count);

			xdlc->tws_max_ack_count=parms->TWS_max_ack_count-xdlc->window;
		}
		
		wpabs_debug_event("%s:      Error Response Config  : %s \n",
				xdlc->name,
				parms->error_response_config==WANOPT_PRI_DISC_ON_NO_RESP?
					"DISC":"SNRM");

	}
	return 0;
}

