/*
 *	XDLC release 002
 *
 *	This code REQUIRES 2.1.15 or higher/ NET3.038
 *
 *	This module:
 *		This module is free software; you can redistribute it and/or
 *		modify it under the terms of the GNU General Public License
 *		as published by the Free Software Foundation; either version
 *		2 of the License, or (at your option) any later version.
 *
 */
#include "wanpipe_xdlc.h"

/*
 *	State machine for state 0, Disconnected State.
 *	The handling of the timer(s) is in file xdlc_timer.c.
 */
static void xdlc_state0_machine(xdlc_dev_t *xdlc, void *skb, struct xdlc_frame *frame)
{
	switch (frame->type) {
		case XDLC_SNRM:

			if (wpabs_test_bit(XDLC_DOWN,&xdlc->critical)){
				return;
			}
			
			if (xdlc->mode & XDLC_EXTENDED) {
				xdlc_send_control(xdlc, XDLC_DM, frame->pf, XDLC_RESPONSE);
			} else {

				if (xdlc->user_state != XDLC_STATE_CONNECTED){
					xdlc_enquiry_response(xdlc);
					return;
				}
			
				xdlc_clear_queues(xdlc);
				xdlc_send_control(xdlc, XDLC_UA, frame->pf, XDLC_RESPONSE);
				xdlc_state_change(xdlc,XDLC_STATE_CONNECTED);
				xdlc_connect_indication(xdlc, XDLC_OK);
			}
			break;
			
		case XDLC_SNRME:

			if (wpabs_test_bit(XDLC_DOWN,&xdlc->critical)){
				return;
			}

			if (xdlc->mode & XDLC_EXTENDED) {

				xdlc_clear_queues(xdlc);
				xdlc_send_control(xdlc, XDLC_UA, frame->pf, XDLC_RESPONSE);
				xdlc_state_change(xdlc,XDLC_STATE_CONNECTED);
				xdlc_connect_indication(xdlc, XDLC_OK);
			} else {
				xdlc_send_control(xdlc, XDLC_DM, frame->pf, XDLC_RESPONSE);
			}
			break;
		
		case XDLC_DISC:
			xdlc_send_control(xdlc, XDLC_DM, frame->pf, XDLC_RESPONSE);
			break;

		default:
			if (xdlc->cfg.station == WANOPT_PRIMARY){
				xdlc_send_control(xdlc, XDLC_DISC, frame->pf, XDLC_RESPONSE);
			}else{
				if (frame->pf){
					xdlc_send_control(xdlc, XDLC_DM, frame->pf, XDLC_RESPONSE);
				}
			}
			break;
	}

	wpabs_skb_free(skb);
}

/*
 *	State machine for state 1, Awaiting Connection State.
 *	The handling of the timer(s) is in file xdlc_timer.c.
 */
static void xdlc_state1_machine(xdlc_dev_t *xdlc, void *skb, struct xdlc_frame *frame)
{
	switch (frame->type) {
		case XDLC_SNRM:

			if (xdlc->cfg.station == WANOPT_SECONDARY){

				if (xdlc->user_state != XDLC_STATE_CONNECTED){
					xdlc_enquiry_response(xdlc);
					return;
				}

				xdlc_clear_queues(xdlc);
				xdlc_state_change(xdlc,XDLC_STATE_CONNECTED);
				xdlc_connect_indication(xdlc, 0);
			}

			if (xdlc->mode & XDLC_EXTENDED) {
				xdlc_send_control(xdlc, XDLC_DM, frame->pf, XDLC_RESPONSE);
			} else {
				xdlc_send_control(xdlc, XDLC_UA, frame->pf, XDLC_RESPONSE);
			}

			break;

		case XDLC_SNRME:
			if (xdlc->mode & XDLC_EXTENDED) {
				xdlc_send_control(xdlc, XDLC_UA, frame->pf, XDLC_RESPONSE);
			} else {
				xdlc_send_control(xdlc, XDLC_DM, frame->pf, XDLC_RESPONSE);
			}
			break;

		case XDLC_DISC:
			xdlc_send_control(xdlc, XDLC_DM, frame->pf, XDLC_RESPONSE);
			break;

		case XDLC_UA:
			if (frame->pf) {

				if (xdlc->user_state != XDLC_STATE_CONNECTED){
					xdlc_enquiry_response(xdlc);
					return;
				}

				xdlc_clear_queues(xdlc);
				xdlc_state_change(xdlc,XDLC_STATE_CONNECTED);
				xdlc_connect_indication(xdlc, XDLC_OK);
			}
			break;

		case XDLC_DM:
			if (frame->pf) {
				xdlc_clear_queues(xdlc);
				xdlc_state_change(xdlc,XDLC_STATE_DISCONNECTED);
				xdlc_disconnect_indication(xdlc, EXCEP_FRM_DM_FRAME_RECEIVED);
			}
			break;

		default:
			break;
	}

	wpabs_skb_free(skb);
}

/*
 *	State machine for state 2, Awaiting Release State.
 *	The handling of the timer(s) is in file xdlc_timer.c
 */
static void xdlc_state2_machine(xdlc_dev_t *xdlc, void *skb, struct xdlc_frame *frame)
{
	switch (frame->type) {
		case XDLC_SNRM:
		case XDLC_SNRME:
			xdlc_send_control(xdlc, XDLC_RD, frame->pf, XDLC_RESPONSE);
			break;

		case XDLC_DISC:
			xdlc_send_control(xdlc, XDLC_UA, frame->pf, XDLC_RESPONSE);
			xdlc_state_change(xdlc,XDLC_STATE_DISCONNECTED);
			xdlc_disconnect_indication(xdlc, 0);
			break;

		case XDLC_UA:
			if (frame->pf) {
				xdlc_state_change(xdlc,XDLC_STATE_DISCONNECTED);
				xdlc_disconnect_indication(xdlc, 0);
			}
			break;

		case XDLC_DM:
			if (frame->pf) {
				xdlc_state_change(xdlc,XDLC_STATE_DISCONNECTED);
				xdlc_disconnect_indication(xdlc, 0);
			}
			break;

		case XDLC_I:
		case XDLC_REJ:
		case XDLC_RNR:
		case XDLC_RR:
			if (frame->pf){
				if (xdlc->cfg.station == WANOPT_PRIMARY){
					xdlc_send_control(xdlc, XDLC_DISC, frame->pf, XDLC_RESPONSE);
				}else{
					xdlc_send_control(xdlc, XDLC_RD, frame->pf, XDLC_RESPONSE);
				}
			}
			break;
				
		default:
			break;
	}

	wpabs_skb_free(skb);
}

/*
 *	State machine for state 3, Connected State.
 *	The handling of the timer(s) is in file xdlc_timer.c
 */
static void xdlc_state3_machine(xdlc_dev_t *xdlc, void *skb, struct xdlc_frame *frame)
{
	int queued = 0;
	int modulus;
	
	modulus = (xdlc->mode & XDLC_EXTENDED) ? XDLC_EMODULUS : XDLC_SMODULUS;

	xdlc->nr_type=frame->type;

	switch (frame->type) {
		case XDLC_SNRM:
			
			wpabs_debug_event( "%s: XDLC Event : State Connected : Rx SABM !\n",xdlc->name);
			
			if (xdlc->mode & XDLC_EXTENDED) {
				xdlc_send_control(xdlc, XDLC_DM, frame->pf, XDLC_RESPONSE);
			} else {
				xdlc_clear_queues(xdlc);
				xdlc_send_control(xdlc, XDLC_UA, frame->pf, XDLC_RESPONSE);
				xdlc_requeue_frames(xdlc);
			}

			xdlc_disconnect_indication(xdlc, EXCEP_SEC_SNRM_FRAME_RECEIVED);
			xdlc_connect_indication(xdlc, XDLC_OK);
			break;

		case XDLC_SNRME:
			
			wpabs_debug_event( "%s: XDLC Event : State Connected : Rx SABME !\n",xdlc->name);

			if (xdlc->mode & XDLC_EXTENDED) {
				xdlc_clear_queues(xdlc);
				xdlc_send_control(xdlc, XDLC_UA, frame->pf, XDLC_RESPONSE);
				xdlc_requeue_frames(xdlc);
			} else {
				xdlc_send_control(xdlc, XDLC_DM, frame->pf, XDLC_RESPONSE);
			}
			
			xdlc_disconnect_indication(xdlc, EXCEP_SEC_SNRM_FRAME_RECEIVED);
			xdlc_connect_indication(xdlc, XDLC_OK);
			break;

		case XDLC_DISC:
//		case XDLC_RD: 

			/* XDLC_RD == XDLC_DISC */
			wpabs_debug_event( "%s: XDLC Event : State connected : Rx DISC\n",xdlc->name);	
			xdlc_clear_queues(xdlc);

			if (xdlc->cfg.station == WANOPT_PRIMARY){
				xdlc_send_control(xdlc, XDLC_DISC, frame->pf, XDLC_RESPONSE);
				xdlc_state_change(xdlc,XDLC_STATE_DISCONNECTING);
				xdlc_disconnecting_indication(xdlc, EXCEP_PRI_RD_FRAME_RECEIVED);
			}else{
				xdlc_send_control(xdlc, XDLC_UA, frame->pf, XDLC_RESPONSE);
				xdlc_state_change(xdlc,XDLC_STATE_DISCONNECTED);
				xdlc_disconnect_indication(xdlc, EXCEP_SEC_DISC_FRAME_RECEIVED);
			}
			break;

		case XDLC_DM:
			wpabs_debug_event( "%s: XDLC Event : State connected : Rx DM\n",xdlc->name);
			xdlc_clear_queues(xdlc);
			xdlc_state_change(xdlc,XDLC_STATE_DISCONNECTED);
			xdlc_disconnect_indication(xdlc, EXCEP_FRM_DM_FRAME_RECEIVED);
			break;

		case XDLC_RNR:

			wpabs_debug_test( "%s: XDLC Event : State connected : Rx RNR\n",xdlc->name);

			
			xdlc->condition |= XDLC_PEER_RX_BUSY_CONDITION;
			xdlc_check_need_response(xdlc, frame->cr, frame->pf);

			if (xdlc_validate_nr(xdlc, frame->nr)) {
			 	xdlc_check_iframes_acked(xdlc, frame->nr);

			} else {
				xdlc->old_nr_type=frame->type;

				xdlc->frmr_data = *frame;
				xdlc->frmr_type = XDLC_FRMR_Z;
				xdlc_transmit_frmr(xdlc);
				xdlc_state_change(xdlc,XDLC_STATE_FRAME_REJECT);
			}
			break;

		case XDLC_RR:

			wpabs_debug_test("%s: Rx: RR nr=%u pf=%u\n",
					xdlc->name,frame->nr,frame->pf);

			xdlc->condition &= ~XDLC_PEER_RX_BUSY_CONDITION;

			xdlc_check_need_response(xdlc, frame->cr, frame->pf);
			if (xdlc_validate_nr(xdlc, frame->nr)) {
				xdlc_check_iframes_acked(xdlc, frame->nr);
			} else {
				xdlc->old_nr_type=frame->type;

				xdlc->frmr_data = *frame;
				xdlc->frmr_type = XDLC_FRMR_Z;
				xdlc_transmit_frmr(xdlc);
				xdlc_state_change(xdlc,XDLC_STATE_FRAME_REJECT);
			}
			break;

		case XDLC_REJ:
			
			wpabs_debug_test( "%s: XDLC Event : State Connected : Rx REJ\n",xdlc->name);
		
			xdlc->condition &= ~XDLC_PEER_RX_BUSY_CONDITION;
			xdlc_check_need_response(xdlc, frame->cr, frame->pf);

			if (xdlc_validate_nr(xdlc, frame->nr)) {

				xdlc_frames_acked(xdlc, frame->nr);
				xdlc_requeue_frames(xdlc);

			} else {
				xdlc->old_nr_type=frame->type;

				xdlc->frmr_data = *frame;
				xdlc->frmr_type = XDLC_FRMR_Z;
				xdlc_transmit_frmr(xdlc);
				xdlc_state_change(xdlc,XDLC_STATE_FRAME_REJECT);
			}
			break;

		case XDLC_I:

			if (wpabs_skb_queue_len(xdlc->rx_queue) >= xdlc->window){

				xdlc->condition |= XDLC_REJECT_CONDITION;
				xdlc->condition |= XDLC_ACK_PENDING_CONDITION;

				wpabs_debug_event("%s: Rx I: REJ Condition: pf=%d Nr=%d Ns=%d Vs=%d Vr=%d\n",
									xdlc->name,
									frame->pf,
									frame->nr,
									frame->ns,
									xdlc->vs,
									xdlc->vr);
				if (frame->pf){
					xdlc_enquiry_response(xdlc);
				}
				break;
			}
			

			if (!xdlc_validate_nr(xdlc, frame->nr)) {

				xdlc->old_nr_type=frame->type;

				wpabs_debug_event( "%s: xdlc Error: I-Frame Rx invalid NR count %d !!!\n",
						xdlc->name,frame->nr);
				
				xdlc->frmr_data = *frame;
				xdlc->frmr_type = XDLC_FRMR_Z;
				xdlc_transmit_frmr(xdlc);
				xdlc_state_change(xdlc,XDLC_STATE_FRAME_REJECT);
				break;
			}

			if (xdlc->condition & XDLC_PEER_RX_BUSY_CONDITION){
				xdlc_frames_acked(xdlc, frame->nr);
			}else{
				xdlc_check_iframes_acked(xdlc, frame->nr);
			}
			
			if (frame->ns == xdlc->vr){

				void *newskb;
				
				/*
				 * If upper layer has dropped the frame, we
				 * basically ignore any further protocol
				 * processing. This will cause the peer
				 * to re-transmit the frame later like
				 * a frame lost on the wire.
				 */
#if 0
				if (wpabs_skb_queue_len(xdlc->rx_queue) >= xdlc->window){
					wpabs_debug_event("%s: Rx window closed dropping pkt ns=%d xdlc->vr=%d!\n",
							xdlc->name,frame->ns,xdlc->vr);
					break;
				}
#endif				
				queued = 1;	
			
				wpabs_skb_queue_tail(xdlc->rx_queue,skb);
				xdlc->vr = (xdlc->vr + 1) % modulus;

				while ((newskb=wpabs_skb_dequeue(xdlc->rx_queue)) != NULL){
					if (xdlc_data_indication(xdlc, newskb) != 0){
						wpabs_debug_event( "%s: Failed to pass data up : top layer busy!\n",
								xdlc->name);
						wpabs_skb_queue_head(xdlc->rx_queue,newskb);
						break;
					}
				}
				
				xdlc->condition &= ~XDLC_REJECT_CONDITION;
				
				if (frame->pf){

					if (wpabs_skb_queue_len(xdlc->rx_queue) >= xdlc->window){
						xdlc->condition |= XDLC_ACK_PENDING_CONDITION;
						xdlc->condition |= XDLC_RNR_SENT_CONDITION;

						wpabs_debug_event("%s: XDLC receive queue limit on PF 1 ! Sending RNR!\n",
									xdlc->name);

					}
					
					xdlc_enquiry_response(xdlc);
				}else{
#if 1
					xdlc->condition |= XDLC_ACK_PENDING_CONDITION;

					/* If the rx queue has reached maximum, send
					 * out an RNR to flow control the incoming
					 * traffic, until the queue is offloaded by
					 * the above protocol */
					if (wpabs_skb_queue_len(xdlc->rx_queue) >= xdlc->window){
						xdlc->condition |= XDLC_RNR_SENT_CONDITION; 
#if 1
						wpabs_debug_event("%s: XDLC receive queue limit PF 0 ! Sending RNR!\n",
								xdlc->name);
#endif

#if 0
						xdlc_send_control(xdlc, XDLC_RNR, 
							XDLC_POLLOFF, XDLC_RESPONSE);
#endif
						break;
					}else{
						if (xdlc->cfg.station == WANOPT_PRIMARY &&
						    xdlc->cfg.protocol_config == WANOPT_TWO_WAY_SIMULTANEOUS){
							
							if (xdlc->tws_max_ack_count){
								xdlc_send_control(xdlc, XDLC_RR, XDLC_POLLOFF, XDLC_RESPONSE);
								xdlc->tws_max_ack_count--;
							}
							
							
						}
					}
#endif
				}
			} else {
				xdlc->condition |= XDLC_REJECT_CONDITION;
				xdlc->condition |= XDLC_ACK_PENDING_CONDITION;

				wpabs_debug_event("%s: XDLC in reject condition PF %d: ns(%d) != vr(%d)\n",
									xdlc->name,
									frame->pf,
									frame->ns,
									xdlc->vr);
				if (frame->pf){
					xdlc_enquiry_response(xdlc);
				}
			}
			break;

		case XDLC_FRMR:

			wpabs_debug_event( "%s: XDLC Event : State Connected : Rx FRMR\n",xdlc->name);
			xdlc_establish_data_link(xdlc);
	
			xdlc_requeue_frames(xdlc);
			xdlc_state_change(xdlc,XDLC_STATE_CONNECTING);
			xdlc_connecting_indication(xdlc, EXCEP_FRMR_FRAME_RECEIVED);
			break;

		case XDLC_ILLEGAL:

			xdlc->frmr_data = *frame;
			xdlc->frmr_type = XDLC_FRMR_W;
			xdlc_transmit_frmr(xdlc);
			xdlc_state_change(xdlc,XDLC_STATE_FRAME_REJECT);
			break;

		default:
			break;
	}

	if (!queued){
		wpabs_skb_free(skb);
	}
}

/*
 *	State machine for state 4, Frame Reject State.
 *	The handling of the timer(s) is in file xdlc_timer.c.
 */
static void xdlc_state4_machine(xdlc_dev_t *xdlc, void *skb, struct xdlc_frame *frame)
{
	switch (frame->type) {
		case XDLC_SNRM:
			if (xdlc->mode & XDLC_EXTENDED) {

				xdlc_send_control(xdlc, XDLC_DM, frame->pf, XDLC_RESPONSE);
			} else {

				if (xdlc->user_state != XDLC_STATE_CONNECTED){
					xdlc_enquiry_response(xdlc);
					return;
				}


				xdlc_clear_queues(xdlc);
				xdlc_send_control(xdlc, XDLC_UA, frame->pf, XDLC_RESPONSE);
				xdlc_state_change(xdlc,XDLC_STATE_CONNECTED);
				xdlc_connect_indication(xdlc, XDLC_OK);
			}
			break;

		case XDLC_SNRME:

			if (xdlc->mode & XDLC_EXTENDED) {

				xdlc_clear_queues(xdlc);
				xdlc_send_control(xdlc, XDLC_UA, frame->pf, XDLC_RESPONSE);
				xdlc_state_change(xdlc,XDLC_STATE_CONNECTED);
				xdlc_connect_indication(xdlc, XDLC_OK);
			} else {

				xdlc_send_control(xdlc, XDLC_DM, frame->pf, XDLC_RESPONSE);
			}
			break;

		default:
			break;
	}

	wpabs_skb_free(skb);
}

void xdlc_exceptions(xdlc_dev_t *xdlc, struct xdlc_frame *frame)
{

	if (frame->type == XDLC_RNR){
		/* Test to see if the counter is
		 * enabled */
		if (xdlc->cfg.max_rnr_cnt){
			xdlc->max_rnr_cnt++;
		}
	}else{
		xdlc->max_rnr_cnt=0;
	}

	if (frame->type == XDLC_REJ){
		/* Test to see if the counter is
		 * enabled */
		if (xdlc->cfg.max_frame_retransmit_cnt){
			xdlc->max_frame_retransmit_cnt++;
		}
	}else{
		xdlc->max_frame_retransmit_cnt=0;
	}


	if (xdlc->cfg.station == WANOPT_SECONDARY){

		if (xdlc->state == XDLC_STATE_DISCONNECTING){
			return;
		}
		
		if (xdlc->max_rnr_cnt > xdlc->cfg.max_rnr_cnt){
			xdlc->max_rnr_cnt=0;
			wpabs_debug_event("%s: Error: Sec Max consec RNR rx count (%d) reached!\n",
					xdlc->name,xdlc->cfg.max_rnr_cnt);
			
			xdlc_clear_queues(xdlc);
			xdlc_state_change(xdlc,XDLC_STATE_DISCONNECTING);
			xdlc_disconnecting_indication(xdlc, EXCEP_RNR_COUNTER);
			return;
		}

		if (xdlc->max_frame_retransmit_cnt > xdlc->cfg.max_frame_retransmit_cnt){
			xdlc->max_frame_retransmit_cnt=0;
			wpabs_debug_event("%s: Error: Sec Max consec REJ rx count (%d) reached!\n",
					xdlc->name,xdlc->cfg.max_frame_retransmit_cnt);
			
			xdlc_clear_queues(xdlc);
			xdlc_state_change(xdlc,XDLC_STATE_DISCONNECTING);
			xdlc_disconnecting_indication(xdlc, EXCEP_FRM_RETRANS_COUNTER);
			return;
		}
	}
}


/*
 *	Process an incoming XDLC frame
 */
void xdlc_data_input(xdlc_dev_t *xdlc, void *skb)
{
	struct xdlc_frame frame;
	int len = wpabs_skb_len(skb);

	FUNC_BEGIN();

	wpabs_debug_test("%s: RX:wq=%d ack=%d asy=%d cond=0x%X!!\n",
			xdlc->name,
			wpabs_skb_queue_len(xdlc->write_queue),
			wpabs_skb_queue_len(xdlc->ack_queue),
			wpabs_skb_queue_len(xdlc->async_queue),
			xdlc->condition);

	if (wpabs_test_bit(XDLC_UNREG_CRIT,&xdlc->critical)){
		wpabs_skb_free(skb);
		return;
	}

	if (wpabs_test_and_set_bit(XDLC_RX_SMP,&xdlc->critical)){
		wpabs_debug_event( "---!!!__ Critical in xdlc Rx!\n");
	}
	
	xdlc_decode(xdlc, skb, &frame);

	if (frame.type == XDLC_CORRUPT){
		wpabs_skb_free(skb);
		wpabs_debug_event("%s: XDLC RX: incoming corrupt frames %d\n",
				xdlc->name,wpabs_skb_len(skb));
		goto xdlc_data_input_error;
	}

	if (wpabs_skb_len(skb) > xdlc->mtu){
		wpabs_debug_event("%s: XDLC RX: incoming packet size %d exceed maximum %d\n",
				xdlc->name,wpabs_skb_len(skb), xdlc->mtu);
		frame.type = XDLC_ILLEGAL;
	}
	xdlc_update_stats(xdlc,frame.type,len,0);

	if (frame.pf){
		if (xdlc->cfg.station != WANOPT_PRIMARY){
			xdlc_start_sec_nrm_timer(xdlc);
		}
		xdlc->condition &= ~XDLC_HALF_DUPLEX_THROTTLE;

		xdlc_stop_pri_frm_resp_timer(xdlc);
	}

	xdlc_exceptions(xdlc,&frame);
	
	switch (xdlc->state) {
		case XDLC_STATE_DISCONNECTED:
			xdlc_state0_machine(xdlc, skb, &frame);
			break;
		case XDLC_STATE_CONNECTING:
			xdlc_state1_machine(xdlc, skb, &frame);
			break;
		case XDLC_STATE_DISCONNECTING:
			xdlc_state2_machine(xdlc, skb, &frame);
			break;
		case XDLC_STATE_CONNECTED:
			xdlc_state3_machine(xdlc, skb, &frame);
			xdlc_rx_recover(xdlc);	
			break;
		case XDLC_STATE_FRAME_REJECT:
			xdlc_state4_machine(xdlc, skb, &frame);
			break;
		default:
			wpabs_debug_event( "%s: xdlc Rx in invalid state: droping packet\n",
					xdlc->name);
			wpabs_skb_free(skb);
			break;
	}

xdlc_data_input_error:
	wpabs_clear_bit(XDLC_RX_SMP,&xdlc->critical);

	FUNC_END();
}



