/*
 *	XDLC release 002
 *
 *	This code REQUIRES 2.1.15 or higher/ NET3.038
 *
 *	This module:
 *		This module is free software; you can redistribute it and/or
 *		modify it under the terms of the GNU General Public License
 *		as published by the Free Software Foundation; either version
 *		2 of the License, or (at your option) any later version.
 */

#include "wanpipe_xdlc.h"

/* 
 *  This procedure is passed a buffer descriptor for an iframe. It builds
 *  the rest of the control part of the frame and then writes it out.
 */
static int xdlc_send_iframe(xdlc_dev_t *xdlc, void *skb, int poll_bit)
{
	unsigned char *frame;

	if (skb == NULL)
		return -1;

	if (wpabs_skb_tailroom(skb) < 4){
		wpabs_skb_free(skb);
		wpabs_debug_event( "BAD ROOM IN SEND FRAME\n");
		return -1;
	}

	if (xdlc->mode & XDLC_EXTENDED) {
		frame = wpabs_skb_push(skb, 2);

		frame[0] = XDLC_I;
		frame[0] |= xdlc->vs << 1;
		frame[1] = (poll_bit) ? XDLC_EPF : 0;
		frame[1] |= xdlc->vr << 1;
	} else {
		frame = wpabs_skb_push(skb, 1);

		*frame = XDLC_I;
		*frame |= (poll_bit) ? XDLC_SPF : 0;
		*frame |= xdlc->vr << 5;
		*frame |= xdlc->vs << 1;
	}


	if (xdlc->cfg.station==WANOPT_PRIMARY && poll_bit){
		xdlc_start_pri_frm_resp_timer(xdlc);	
	}

	return xdlc_transmit_buffer(xdlc, skb, XDLC_COMMAND,0);	
}


int xdlc_transmit_buffer(xdlc_dev_t *xdlc, void *skb, int type, char async)
{
	unsigned char *ptr;
	//int err;
	
	FUNC_BEGIN();

	ptr = wpabs_skb_push(skb, 1);

	*ptr = xdlc->cfg.address;
	
#if 0
	}else if (xdlc->mode & XDLC_MLP) {
		if (xdlc->mode & XDLC_DCE) {
			if (type == XDLC_COMMAND)
				*ptr = XDLC_ADDR_C;
			if (type == XDLC_RESPONSE)
				*ptr = XDLC_ADDR_D;
		} else {
			if (type == XDLC_COMMAND)
				*ptr = XDLC_ADDR_D;
			if (type == XDLC_RESPONSE)
				*ptr = XDLC_ADDR_C;
		}
	} else {
		if (xdlc->mode & XDLC_DCE) {
			if (type == XDLC_COMMAND)
				*ptr = XDLC_ADDR_A;
			if (type == XDLC_RESPONSE)
				*ptr = XDLC_ADDR_B;
		} else {
			if (type == XDLC_COMMAND)
				*ptr = XDLC_ADDR_B;
			if (type == XDLC_RESPONSE)
				*ptr = XDLC_ADDR_A;
		}
	}
#endif

#if 1
	if (async){
		wpabs_skb_queue_tail(xdlc->async_queue,skb);
		return 0;
	}

	return xdlc_data_transmit(xdlc, skb);
#else

	err = xdlc_data_transmit(xdlc, skb);
	
	if (err && async){
		wpabs_skb_queue_tail(xdlc->async_queue,skb);
	}
	return err;
#endif
}

void xdlc_establish_data_link(xdlc_dev_t *xdlc)
{
	/* Nenad Corbic: Initailize all queues before
	 * starting the link */
	xdlc_clear_queues(xdlc);

	if (xdlc->cfg.station == WANOPT_PRIMARY){
		xdlc_send_control(xdlc, XDLC_SNRM, XDLC_POLLON, XDLC_COMMAND);
		xdlc_start_pri_poll_timer(xdlc);
	}
}

void xdlc_enquiry_response(xdlc_dev_t *xdlc)
{
	wpabs_debug_tx("%s: %s() user state=0x%X cond=0x%X Qlen=%d\n",
			xdlc->name, __FUNCTION__,
			xdlc->user_state,
			xdlc->condition,
			wpabs_skb_queue_len(xdlc->write_queue));
			
	if (xdlc->user_state != XDLC_STATE_CONNECTED){
		if (xdlc->cfg.station==WANOPT_PRIMARY){
			xdlc_send_control(xdlc, XDLC_DISC, XDLC_POLLON, XDLC_RESPONSE);
		}else{
			xdlc_send_control(xdlc, XDLC_DM, XDLC_POLLON, XDLC_RESPONSE);
		}
	}else if (xdlc->condition & XDLC_REJECT_CONDITION){
		xdlc_send_control(xdlc, XDLC_REJ, XDLC_POLLON, XDLC_RESPONSE);
	}else if (xdlc->condition & XDLC_RNR_SENT_CONDITION){
		xdlc_send_control(xdlc, XDLC_RNR, XDLC_POLLON, XDLC_RESPONSE);
	}else{
		xdlc_send_control(xdlc, XDLC_RR, XDLC_POLLON, XDLC_RESPONSE);
	}

	xdlc->condition |= XDLC_HALF_DUPLEX_THROTTLE;

	xdlc_start_pri_frm_resp_timer(xdlc);
}

void xdlc_timeout_response(xdlc_dev_t *xdlc)
{

	if (wpabs_skb_queue_len(xdlc->rx_queue) > XDLC_RX_Q_H_LIMIT){
		xdlc_send_control(xdlc, XDLC_RNR, XDLC_POLLOFF, XDLC_RESPONSE);
	}else{
		xdlc_send_control(xdlc, XDLC_RR, XDLC_POLLOFF, XDLC_RESPONSE);
	}

	xdlc->condition &= ~XDLC_ACK_PENDING_CONDITION;
	xdlc_start_pri_frm_resp_timer(xdlc);
}

void xdlc_check_iframes_acked(xdlc_dev_t *xdlc, unsigned short nr)
{
	wpabs_debug_test("%s: %s() nr=%d  vs=%d va=%d\n",
			xdlc->name, __FUNCTION__,nr,
			xdlc->vs,xdlc->va);

	if (xdlc->vs == nr){
		xdlc_frames_acked(xdlc, nr);
	}else{
		if (xdlc->va != nr){
			xdlc_frames_acked(xdlc, nr);
		}
	}
	xdlc_requeue_frames(xdlc);
}

void xdlc_check_need_response(xdlc_dev_t *xdlc, int type, int pf)
{

	if (xdlc->cfg.station == WANOPT_PRIMARY){
		return;
	}

	if (pf){

		int modulus = (xdlc->mode & XDLC_EXTENDED) ? 
				XDLC_EMODULUS : XDLC_SMODULUS;
		int start = xdlc->vs;
		int end   = (xdlc->va + xdlc->window) % modulus;
		
		if (wpabs_skb_queue_len(xdlc->write_queue) &&
		    xdlc->user_state == XDLC_STATE_CONNECTED &&
		    start != end && 
		    !(xdlc->condition & XDLC_PEER_RX_BUSY_CONDITION) &&
		    !(xdlc->condition & XDLC_HALF_DUPLEX_THROTTLE)){
			return;
		}

		xdlc_enquiry_response(xdlc);
	}
}


int xdlc_bh(xdlc_dev_t *xdlc)
{
	void *skb, *skbn;
	volatile unsigned short modulus, start, end;
	int err=0, data_frame_sent_status=0;

	FUNC_BEGIN();
	
	if (wpabs_test_and_set_bit(XDLC_SMP_CRIT,&xdlc->critical)){
		wpabs_debug_event( "CRITICAL IN XDLC KICK !!!---!!!\n");
		return 0;
	}

	if (wpabs_test_bit(XDLC_UNREG_CRIT, &xdlc->critical)){
		wpabs_clear_bit(XDLC_AWAITING_KICK,&xdlc->critical);
		wpabs_clear_bit(XDLC_FRAME_SENT,&xdlc->critical);
		wpabs_clear_bit(XDLC_BH_RUNNING,&xdlc->critical);
		wpabs_clear_bit(XDLC_SMP_CRIT,&xdlc->critical);
		return 0;
	}

	++xdlc->kick_cnt;
	
	wpabs_clear_bit(XDLC_FRAME_SENT,&xdlc->critical);
	
xdlc_bh_send_async:
	while ((skb=wpabs_skb_dequeue(xdlc->async_queue)) != NULL){

		unsigned char *data=wpabs_skb_data(skb);
#if 0
		unsigned char tmpdata=data[1];
#endif
		unsigned char tmpvr=0;

		/* Fix to include all packets that contain
		 * VR numbers */
		switch ((data[1]&0x0F)){
			case XDLC_RR:
			case XDLC_RNR:
			case XDLC_REJ:
				data[1] &= 0x1F;
				data[1] |= (xdlc->vr << 5);
				tmpvr=xdlc->vr;
				break;
		}
		err = xdlc_data_transmit(xdlc, skb);

		if (err != 0){
			//wpabs_debug_event( "%s:Failed to send xdlc Async Message!\n",
			//		xdlc->dev->name);
			wpabs_skb_queue_head(xdlc->async_queue,skb);
			wpabs_set_bit(XDLC_AWAITING_KICK,&xdlc->critical);
			goto xdlc_bh_exit;
		}

#if 0
		if ((tmpdata&0x0F) == XDLC_REJ){
			wpabs_debug_event("%s: Tx Rej Vr=%d\n",xdlc->name,tmpvr); 
		}
#endif
	}

	if (xdlc->state != XDLC_STATE_CONNECTED)
		goto xdlc_bh_exit;

	if (wpabs_test_bit(XDLC_REQUEUING,&xdlc->critical)){
		wpabs_debug_event( "(Debug): xdlc requeuing skip kick!\n");
		goto xdlc_bh_exit;
	}
	
	modulus = (xdlc->mode & XDLC_EXTENDED) ? XDLC_EMODULUS : XDLC_SMODULUS;

        start = xdlc->vs;
	end   = (xdlc->va + xdlc->window) % modulus;
	
	if (!(xdlc->condition & XDLC_HALF_DUPLEX_THROTTLE) &&
	    !(xdlc->condition & XDLC_PEER_RX_BUSY_CONDITION) &&
	    start != end                                &&
	    wpabs_skb_queue_len(xdlc->write_queue) != 0){

		/*
		 * Dequeue the frame and copy it.
		 */
		skb  = wpabs_skb_dequeue(xdlc->write_queue);

		do {
			if (wpabs_skb_queue_len(xdlc->async_queue)){
				//wpabs_debug_event( "xdlc kick re-sending async!\n");
				wpabs_skb_queue_head(xdlc->write_queue, skb);
				goto xdlc_bh_send_async;
			}

			
			if ((skbn = wpabs_skb_clone(skb)) == NULL) {
				wpabs_skb_queue_head(xdlc->write_queue, skb);
				break;
			}
			
			/*
			 * Transmit the frame copy.
			 */

			err = xdlc_send_iframe(xdlc, skbn, XDLC_POLLOFF);
			if (err == 0){

				//printk(KERN_DEBUG "XDLC KICK PACKET OK: Vs=%d\n",
				//		xdlc->vs);

				xdlc_update_stats (xdlc, XDLC_I, wpabs_skb_len(skb), 1);
	
				xdlc->vs = (xdlc->vs + 1) % modulus;
				/* Requeue the original data frame */
				wpabs_skb_queue_tail(xdlc->ack_queue, skb);
	
				wpabs_set_bit(XDLC_FRAME_SENT,&xdlc->critical);

			}else if (err == 1){
				/* Failed to Tx data, thus re-queue frame */
#if 0
				wpabs_debug_test("xdlc: Failed to TX Data to Lower layer! Vs=%d\n",
						xdlc->vs);
#endif
				wpabs_skb_queue_head(xdlc->write_queue, skb);

				wpabs_skb_free(skbn);
				
				wpabs_set_bit(XDLC_AWAITING_KICK,&xdlc->critical);
				break;
			}else{
				/* Major Error */ 
				wpabs_debug_event( "xdlc: Failed to TX Data Major Error! Vs=%d\n",
						xdlc->vs);
				wpabs_skb_queue_head(xdlc->write_queue, skb);
				
				wpabs_skb_free(skbn);

				wpabs_set_bit(XDLC_AWAITING_KICK,&xdlc->critical);
				break;
				
			}
			
		} while (xdlc->vs != end && (skb = wpabs_skb_dequeue(xdlc->write_queue)) != NULL);
	
		xdlc_enquiry_response(xdlc);

		/* Only stop the RR response if we successfully sent
		 * a packet. Otherwise let the RR clear the window */
		if (wpabs_test_bit(XDLC_FRAME_SENT,&xdlc->critical)){

			xdlc->condition &= ~XDLC_ACK_PENDING_CONDITION;

			wpabs_clear_bit(XDLC_FRAME_SENT,&xdlc->critical);

			data_frame_sent_status=1;
		}
	}else{
#if 0
		wpabs_debug_test("%s: %s() Tx data not enabled: S=%d E=%d Cond=0x%X Len=%d\n",
				xdlc->name,
				__FUNCTION__,
				start,end,
				xdlc->condition,
				wpabs_skb_queue_len(xdlc->write_queue));
#endif
	}


xdlc_bh_exit:

	
	wpabs_clear_bit(XDLC_SMP_CRIT,&xdlc->critical);
	wpabs_clear_bit(XDLC_BH_RUNNING,&xdlc->critical);

	return data_frame_sent_status;
#undef xdlc
}

