/*
 *	XDLC release 002
 *
 *	This code REQUIRES 2.1.15 or higher/ NET3.038
 *
 *	This module:
 *		This module is free software; you can redistribute it and/or
 *		modify it under the terms of the GNU General Public License
 *		as published by the Free Software Foundation; either version
 *		2 of the License, or (at your option) any later version.
 */

#include "wanpipe_xdlc.h"


/*
 *	Create an empty XDLC control block.
 */
xdlc_dev_t *xdlc_create_cb(void)
{
	xdlc_dev_t *xdlc;
	
	xdlc=wpabs_malloc(sizeof(*xdlc));
	if (xdlc == NULL)
		return NULL;

	wpabs_memset(xdlc, 0x00, sizeof(xdlc_dev_t));

	wp_dev_hold(xdlc);

	xdlc->write_queue = wpabs_skb_alloc_queue();
	if (!xdlc->write_queue){
		goto xdlc_create_cb_failed;
	}
	wpabs_skb_queue_init(xdlc->write_queue);

	xdlc->ack_queue   = wpabs_skb_alloc_queue();
	if (!xdlc->ack_queue){
		goto xdlc_create_cb_failed;
	}
	wpabs_skb_queue_init(xdlc->ack_queue);

	xdlc->async_queue = wpabs_skb_alloc_queue();
	if (!xdlc->async_queue){
		goto xdlc_create_cb_failed;
	}
	wpabs_skb_queue_init(xdlc->async_queue);
	
	xdlc->rx_queue   = wpabs_skb_alloc_queue();
	if (!xdlc->rx_queue){
		goto xdlc_create_cb_failed;
	}
	wpabs_skb_queue_init(xdlc->rx_queue);
	
	xdlc->window  = XDLC_DEFAULT_WINDOW;
	xdlc->state   = XDLC_STATE_DISCONNECTED;

	return xdlc;
	
xdlc_create_cb_failed:

	xdlc_free_cb(xdlc);

	return NULL;
}

int xdlc_free_cb(xdlc_dev_t *xdlc)
{
	if (!xdlc){
		return -1;
	}
	
	if (xdlc->write_queue){
		wpabs_skb_free_queue(xdlc->write_queue);
		xdlc->write_queue=NULL;
	}

	if (xdlc->ack_queue){
		wpabs_skb_free_queue(xdlc->ack_queue);
		xdlc->ack_queue=NULL;
	}

	if (xdlc->async_queue){
		wpabs_skb_free_queue(xdlc->async_queue);
		xdlc->async_queue=NULL;
	}
	
	if (xdlc->rx_queue){
		wpabs_skb_free_queue(xdlc->rx_queue);
		xdlc->rx_queue=NULL;
	}

	xdlc_stop_pri_poll_timer(xdlc);
	xdlc_stop_pri_frm_resp_timer(xdlc);

	wp_dev_put(xdlc);

	return 0;
}


void xdlc_state_change(xdlc_dev_t *xdlc, int state)
{
	FUNC_BEGIN();

	switch (state){
	
	case XDLC_STATE_CONNECTING:
		wpabs_debug_event( "%s: Xdlc Link connecting...\n",xdlc->name);
		break;

	case XDLC_STATE_CONNECTED:
		wpabs_debug_event( "%s: Xdlc Link connected\n",xdlc->name);
		break;

	case XDLC_STATE_DISCONNECTED:
		wpabs_debug_event( "%s: Xdlc Link disconnected!\n",xdlc->name);
		break;
		
	case XDLC_STATE_FRAME_REJECT:
		wpabs_debug_event( "%s: Xdlc Link frame reject!\n",xdlc->name);
		break;

	case XDLC_STATE_DISCONNECTING:
		wpabs_debug_event( "%s: Xdlc Link disconnecting...\n",xdlc->name);
		break;
	default:
		wpabs_debug_event( "%s: Xdlc Invalid link state %d!",
				xdlc->name,state);
		break;

	}

	xdlc->state = state;
}


void xdlc_requeue_frames(xdlc_dev_t *xdlc)
{
        void *skb, *skb_prev = NULL;

	/*
	 * Requeue all the un-ack-ed frames on the output queue to be picked
	 * up by xdlc_bh called from the timer. This arrangement handles the
	 * possibility of an empty output queue.
	 */

	wpabs_set_bit(XDLC_REQUEUING,&xdlc->critical);

#if 0
	if (wpabs_skb_queue_len(xdlc->ack_queue)){
		wpabs_debug_event( "%s: Requeing %d non acked frames for Tx, new Vs=%d !\n",
			xdlc->name,skb_queue_len(xdlc->ack_queue),xdlc->va);
	}
#endif
	
	//FIXME: LOCK NO LOCKS HERE, PLEASE CONFIRM!!!!
	
	while ((skb = wpabs_skb_dequeue(xdlc->ack_queue)) != NULL) {
		if (skb_prev == NULL){
			wpabs_skb_queue_head(xdlc->write_queue, skb);
		}else{
			wpabs_skb_append(skb_prev, skb);
		}
		skb_prev = skb;
	}

#if 0
	wpabs_debug_event( "%s: Requeue setting vs=%d back to va=%d\n",
			xdlc->name,xdlc->vs,xdlc->va);
#endif

	xdlc->vs=xdlc->va;
	
	wpabs_clear_bit(XDLC_REQUEUING,&xdlc->critical);
}


/*
 *	This routine is the centralised routine for parsing the control
 *	information for the different frame formats.
 */
void xdlc_decode(xdlc_dev_t *xdlc, void* skb, struct xdlc_frame *frame)
{
	unsigned char *data = wpabs_skb_data(skb);
	frame->type = XDLC_ILLEGAL;

	frame->address = data[0];

	wpabs_skb_pull(skb, 1);
	data = wpabs_skb_data(skb);

	if (xdlc->mode & XDLC_EXTENDED) {

		if ((data[0] & XDLC_S) == 0) {
			frame->type       = XDLC_I;			/* I frame - carries NR/NS/PF */
			frame->ns         = (data[0] >> 1) & 0x7F;
			frame->nr         = (data[1] >> 1) & 0x7F;
			frame->pf         = data[1] & XDLC_EPF;
			frame->control[0] = data[0];
			frame->control[1] = data[1];
			wpabs_skb_pull(skb, 2);
		} else if ((data[0] & XDLC_U) == 1) { 	/* S frame - take out PF/NR */
			frame->type       = data[0] & 0x0F;
			frame->nr         = (data[1] >> 1) & 0x7F;
			frame->pf         = data[1] & XDLC_EPF;
			frame->control[0] = data[0];
			frame->control[1] = data[1];
			wpabs_skb_pull(skb, 2);
		} else if ((data[0] & XDLC_U) == 3) { 	/* U frame - take out PF */
			frame->type       = data[0] & ~XDLC_SPF;
			frame->pf         = data[0] & XDLC_SPF;
			frame->control[0] = data[0];
			frame->control[1] = 0x00;
			wpabs_skb_pull(skb, 1);
		}
	} else {
		if ((data[0] & XDLC_S) == 0) {
			frame->type = XDLC_I;			/* I frame - carries NR/NS/PF */
			frame->ns   = (data[0] >> 1) & 0x07;
			frame->nr   = (data[0] >> 5) & 0x07;
			frame->pf   = data[0] & XDLC_SPF;
		} else if ((data[0] & XDLC_U) == 1) { 	/* S frame - take out PF/NR */
			frame->type = data[0] & 0x0F;
			frame->nr   = (data[0] >> 5) & 0x07;
			frame->pf   = data[0] & XDLC_SPF;
		} else if ((data[0] & XDLC_U) == 3) { 	/* U frame - take out PF */
			frame->type = data[0] & ~XDLC_SPF;
			frame->pf   = data[0] & XDLC_SPF;
		}

		frame->control[0] = data[0];

		wpabs_skb_pull(skb, 1);
	}
}

/* 
 *	This routine is called when the HDLC layer internally  generates a
 *	command or  response  for  the remote machine ( eg. RR, UA etc. ). 
 *	Only supervisory or unnumbered frames are processed, FRMRs are handled
 *	by xdlc_transmit_frmr below.
 */
void xdlc_send_control(xdlc_dev_t *xdlc, int frametype, int poll_bit, int type)
{
	void *skb;
	unsigned char  *dptr;

	FUNC_BEGIN();

	if (xdlc->cfg.station==WANOPT_PRIMARY && poll_bit){
		xdlc_start_pri_frm_resp_timer(xdlc);	
	}

	skb=wpabs_skb_alloc(XDLC_HEADER_LEN + 3);
	if (skb == NULL)
		return;
	
	wpabs_skb_reserve(skb, XDLC_HEADER_LEN + 1);

	if (xdlc->mode & XDLC_EXTENDED) {
		if ((frametype & XDLC_U) == XDLC_U) {
			dptr   = wpabs_skb_put(skb, 1);
			*dptr  = frametype;
			*dptr |= (poll_bit) ? XDLC_SPF : 0;
		} else {
			dptr     = wpabs_skb_put(skb, 2);
			dptr[0]  = frametype;
			dptr[1]  = (xdlc->vr << 1);
			dptr[1] |= (poll_bit) ? XDLC_EPF : 0;
		}
	} else {
		dptr   = wpabs_skb_put(skb, 1);
		*dptr  = frametype;
		*dptr |= (poll_bit) ? XDLC_SPF : 0;
		if ((frametype & XDLC_U) == XDLC_S)		/* S frames carry NR */
			*dptr |= (xdlc->vr << 5);
	}

	xdlc_update_stats (xdlc, frametype, wpabs_skb_len(skb), 1);

	xdlc_transmit_buffer(xdlc, skb, type,1);

	FUNC_END();
}

/* 
 *	This routine generates FRMRs based on information previously stored in
 *	the XDLC control block.
 */
void xdlc_transmit_frmr(xdlc_dev_t *xdlc)
{
	struct sk_buff *skb;
	unsigned char  *dptr;

	skb=wpabs_skb_alloc(XDLC_HEADER_LEN + 7);
	if (skb == NULL)
		return;

	wpabs_skb_reserve(skb, XDLC_HEADER_LEN + 1);

	if (xdlc->mode & XDLC_EXTENDED) {
		dptr    = wpabs_skb_put(skb, 6);
		*dptr++ = XDLC_FRMR;
		*dptr++ = xdlc->frmr_data.control[0];
		*dptr++ = xdlc->frmr_data.control[1];
		*dptr++ = (xdlc->vs << 1) & 0xFE;
		*dptr   = (xdlc->vr << 1) & 0xFE;
		if (xdlc->frmr_data.cr == XDLC_RESPONSE)
			*dptr |= 0x01;
		dptr++;
		*dptr++ = xdlc->frmr_type;

	}else{
		dptr    = wpabs_skb_put(skb, 4);
		*dptr++ = XDLC_FRMR;
		*dptr++ = xdlc->frmr_data.control[0];
		*dptr   = (xdlc->vs << 1) & 0x0E;
		*dptr  |= (xdlc->vr << 5) & 0xE0;
		if (xdlc->frmr_data.cr == XDLC_RESPONSE)
			*dptr |= 0x10;
		dptr++;
		*dptr++ = xdlc->frmr_type;

	}
	
	xdlc_transmit_buffer(xdlc, skb, XDLC_RESPONSE,1);

	xdlc_connecting_indication(xdlc, EXCEP_FRMR_FRAME_TRANSMITTED);
}

void xdlc_update_stats (xdlc_dev_t *xdlc, int type, int len, char direction)
{
	FUNC_BEGIN();

	switch (type){

		case XDLC_I:
			if (!direction)
				++xdlc->xdlcstats.ifrm_rx;
			else
				++xdlc->xdlcstats.ifrm_tx;	
			break;
		
		case XDLC_RR:
			if (!direction)
				++xdlc->xdlcstats.rr_rx;
			else
				++xdlc->xdlcstats.rr_tx;	
			break;
		
		case XDLC_RNR:
			if (!direction)
				++xdlc->xdlcstats.rnr_rx;
			else
				++xdlc->xdlcstats.rnr_tx;	
			break;

		case XDLC_REJ:
			if (!direction)
				++xdlc->xdlcstats.rej_rx;
			else
				++xdlc->xdlcstats.rej_tx;	
			break;
#if 0	
		case XDLC_SABME:
		case XDLC_SABM:
			if (!direction)
				++xdlc->xdlcstats.sabm_rx;
			else
				++xdlc->xdlcstats.sabm_tx;	
			break;
#endif

		case XDLC_SNRME:
		case XDLC_SNRM:
			if (!direction)
				++xdlc->xdlcstats.sabm_rx;
			else
				++xdlc->xdlcstats.sabm_tx;	
			break;
	
		case XDLC_DISC:
			if (!direction)
				++xdlc->xdlcstats.disc_rx;
			else
				++xdlc->xdlcstats.disc_tx;	
			break;
		
		case XDLC_DM:
			if (!direction)
				++xdlc->xdlcstats.dm_rx;
			else
				++xdlc->xdlcstats.dm_tx;	
			break;
		
		case XDLC_UA:
			if (!direction)
				++xdlc->xdlcstats.ua_rx;
			else
				++xdlc->xdlcstats.ua_tx;	
			break;

		case XDLC_FRMR:
			if (!direction)
				++xdlc->xdlcstats.frmr_rx;
			else
				++xdlc->xdlcstats.frmr_tx;	
			break;
	
		default:
			wpabs_debug_event( "%s: %s illegal packet type 0x%02X !\n",
					xdlc->name,direction ? "Tx" : "Rx" ,type);
			if (!direction)
				++xdlc->xdlcstats.ill_rx;
			else
				++xdlc->xdlcstats.ill_tx;	
			break;
	}

#if 0
	if (!direction){
		++xdlc->ifstats.rx_packets;
		xdlc->ifstats.rx_bytes += len;
	}else{
		++xdlc->ifstats.tx_packets;
		xdlc->ifstats.tx_bytes += len;
	}
#endif
}

#if 0
//NC: Not used any more
void xdlc_reconnect(xdlc_dev_t *xdlc)
{
	xdlc_start_pri_frm_resp_timer(xdlc);
	xdlc_stop_pri_frm_resp_timer(xdlc);
}
#endif

void xdlc_link_up (xdlc_dev_t *xdlc)
{
	if (!xdlc)
		return;

	if (wpabs_test_bit(XDLC_DOWN,&xdlc->critical)){
		return;
	}

	xdlc_clear_queues(xdlc);

	/* Do not actually try to establish connection 
	 * here.  Since frame relay needs time to start up
	 * let the Pri Poll timer expire and retry the 
	 * startup sequence */
	if (xdlc->cfg.station == WANOPT_PRIMARY){
		xdlc_state_change(xdlc,XDLC_STATE_CONNECTING);
	}else{
		xdlc_state_change(xdlc,XDLC_STATE_DISCONNECTED);
	}

	xdlc_start_pri_poll_timer(xdlc);
	xdlc_stop_pri_frm_resp_timer(xdlc);

	return;
}

void xdlc_link_down (xdlc_dev_t *xdlc)
{
	/* Note: Do not call the *_indication()
	 * function here, since this function can be
	 * called by the critical code.  And we
	 * don't want any callback functions going
	 * back and potentially creating a deadlock */
	
	if (!xdlc)
		return;

	if (xdlc->state == XDLC_STATE_CONNECTED){
		xdlc_state_change(xdlc,XDLC_STATE_DISCONNECTING);

	}else if (xdlc->state == XDLC_STATE_DISCONNECTING){
		return;

	}else{
		xdlc_state_change(xdlc,XDLC_STATE_DISCONNECTING);
	}

	return;
}

void xdlc_rx_recover(xdlc_dev_t *xdlc)
{
	void *newskb;
	int rx_ok=0;

	if (!wpabs_skb_queue_len(xdlc->rx_queue)){
		return;
	}
	
	while ((newskb=wpabs_skb_dequeue(xdlc->rx_queue)) != NULL){
		if (xdlc_data_indication(xdlc, newskb) != 0){
			wpabs_debug_test( "%s: Failed to pass data up : top layer busy!\n",
					xdlc->name);
			wpabs_skb_queue_head(xdlc->rx_queue,newskb);
			break;
		}
		rx_ok=1;
	}

	if (!rx_ok){
		return;
	}
	
	if (wpabs_skb_queue_len(xdlc->rx_queue) < xdlc->window){
		xdlc->condition &= ~XDLC_RNR_SENT_CONDITION;
		xdlc->condition &= ~XDLC_REJECT_CONDITION;
		xdlc->condition |= XDLC_ACK_PENDING_CONDITION;
	}
	
	return;
}

