/*
 *	XDLC release 002
 *
 *	This code REQUIRES 2.1.15 or higher/ NET3.038
 *
 *	This module:
 *		This module is free software; you can redistribute it and/or
 *		modify it under the terms of the GNU General Public License
 *		as published by the Free Software Foundation; either version
 *		2 of the License, or (at your option) any later version.
 *
 *	History
 */

#include "wanpipe_xdlc.h"


void xdlc_start_pri_frm_resp_timer(xdlc_dev_t *xdlc)
{
	FUNC_BEGIN();

	if (xdlc->cfg.station != WANOPT_PRIMARY){
		return;
	}
	
	xdlc->pri_frm_timer = wpabs_get_systemticks();
}

void xdlc_start_pri_poll_timer(xdlc_dev_t *xdlc)
{
	FUNC_BEGIN();

	if (xdlc->cfg.station != WANOPT_PRIMARY){
		return;
	}

	xdlc->pri_poll_timer = wpabs_get_systemticks();
	
}

void xdlc_stop_pri_frm_resp_timer(xdlc_dev_t *xdlc)
{
	FUNC_BEGIN();

	if (xdlc->cfg.station != WANOPT_PRIMARY){
		return;
	}

	xdlc->max_no_response_cnt=0;
	xdlc->pri_frm_timer=0;
}

void xdlc_stop_pri_poll_timer(xdlc_dev_t *xdlc)
{
	FUNC_BEGIN();

	if (xdlc->cfg.station != WANOPT_PRIMARY){
		return;
	}

	xdlc->pri_poll_timer=0;
}


/*=========================================================
 * xdlc_pri_poll_expiry
 *
 */

void xdlc_pri_poll_expiry(xdlc_dev_t *xdlc)
{

	FUNC_BEGIN();
	
	if (wpabs_test_bit(XDLC_UNREG_CRIT,&xdlc->critical)){
		return; 
	}

	if (xdlc->user_state == XDLC_STATE_DISCONNECTED &&
	    xdlc->state == XDLC_STATE_DISCONNECTED){
		return;
	}

	wpabs_debug_test("%s: %s() State = 0x%X\n",
			xdlc->name,__FUNCTION__,
			xdlc->state);
	
	switch (xdlc->state) {
		
		case XDLC_STATE_CONNECTED:
	
			if (xdlc->condition & XDLC_HALF_DUPLEX_THROTTLE){
				wpabs_debug_test("%s: Error: Sec Peer busy on poll!!\n",xdlc->name);
				break;
			}

			if (xdlc->cfg.protocol_config == WANOPT_TWO_WAY_SIMULTANEOUS){
				xdlc->tws_max_ack_count=xdlc->cfg.TWS_max_ack_count-xdlc->window;
			}

			if (xdlc->max_rnr_cnt > xdlc->cfg.max_rnr_cnt){
				xdlc->max_rnr_cnt=0;
				wpabs_debug_event("%s: Error: Pri Max consec RNR rx count (%d) reached!\n",
						xdlc->name,xdlc->cfg.max_rnr_cnt);
				
				xdlc_clear_queues(xdlc);
				xdlc_state_change(xdlc,XDLC_STATE_CONNECTING);
				xdlc_connecting_indication(xdlc, EXCEP_RNR_COUNTER);
				break;
			}

			if (xdlc->max_frame_retransmit_cnt > xdlc->cfg.max_frame_retransmit_cnt){
				xdlc->max_frame_retransmit_cnt =0;
				wpabs_debug_event("%s: Error: Pri Max consec REJ rx count (%d) reached!\n",
						xdlc->name,xdlc->cfg.max_frame_retransmit_cnt);
				
				xdlc_clear_queues(xdlc);
				xdlc_state_change(xdlc,XDLC_STATE_CONNECTING);
				xdlc_connecting_indication(xdlc, EXCEP_FRM_RETRANS_COUNTER);
				break;
			}
				
			xdlc->condition &= ~XDLC_HALF_DUPLEX_THROTTLE;
		
			if (wpabs_skb_queue_len(xdlc->write_queue) &&
			    !(xdlc->condition & XDLC_PEER_RX_BUSY_CONDITION)){
				wpabs_debug_test("%s: TIMER: Pri has tx data wq=%d ack=%d asy=%d cond=0x%X!!\n",
						xdlc->name,
						wpabs_skb_queue_len(xdlc->write_queue),
						wpabs_skb_queue_len(xdlc->ack_queue),
						wpabs_skb_queue_len(xdlc->async_queue),
						xdlc->condition);
				break;
			}

			xdlc_enquiry_response(xdlc);

			break;

		case XDLC_STATE_CONNECTING: 			
			xdlc_send_control(xdlc, XDLC_SNRM, XDLC_POLLON, XDLC_COMMAND);
			break;
	
		case XDLC_STATE_FRAME_REJECT:
			xdlc_transmit_frmr(xdlc);
			break;

		case XDLC_STATE_DISCONNECTED:
			xdlc_clear_queues(xdlc);
			xdlc_state_change(xdlc,XDLC_STATE_CONNECTING);
			break;

		case XDLC_STATE_DISCONNECTING:
			xdlc_send_control(xdlc, XDLC_DISC, XDLC_POLLON, XDLC_COMMAND);
			break;
			
		default:
			wpabs_debug_event( "%s: Pri Poll Timer Expiry: Invalid State 0x%02X\n",
					xdlc->name, xdlc->state);

			xdlc_clear_queues(xdlc);
			xdlc_state_change(xdlc,XDLC_STATE_CONNECTING);
			break;
	}

	xdlc_start_pri_poll_timer(xdlc);
}

void xdlc_pri_frm_response_expiry(xdlc_dev_t *xdlc)
{

	FUNC_BEGIN();

	wpabs_debug_test("%s: %s() State = 0x%X Cnt=%d\n",
			xdlc->name,__FUNCTION__,
			xdlc->state,
			xdlc->max_no_response_cnt);

	if (wpabs_test_bit(XDLC_UNREG_CRIT,&xdlc->critical)){
		wpabs_debug_event( "Critical in T1 timer\n");
		return; 
	}

	switch (xdlc->state) {
		
		case XDLC_STATE_CONNECTED:

			if (!xdlc->cfg.max_no_response_cnt){
				break;
			}
		
			wpabs_debug_event("%s: %s() Warning: RESETING DUPLEX \n",
					xdlc->name,__FUNCTION__);

			xdlc->condition &= ~XDLC_HALF_DUPLEX_THROTTLE;

			if (xdlc->max_no_response_cnt >= xdlc->cfg.max_no_response_cnt-1){

				xdlc->max_no_response_cnt=0;

				xdlc_clear_queues(xdlc);
				xdlc_disconnect_indication(xdlc, EXCEP_NO_RESPONSE_COUNTER);

				wpabs_debug_event("%s: Error: Max no resonse counter(%d) exceeded\n",
					xdlc->name,xdlc->cfg.max_no_response_cnt);
	
				if (xdlc->cfg.error_response_config == WANOPT_PRI_DISC_ON_NO_RESP){
					xdlc_state_change(xdlc,XDLC_STATE_DISCONNECTED);	
					xdlc_send_control(xdlc, XDLC_DISC, XDLC_POLLON, XDLC_COMMAND);
				}else{
					xdlc_state_change(xdlc,XDLC_STATE_CONNECTING);
					xdlc_send_control(xdlc, XDLC_SNRM, XDLC_POLLON, XDLC_COMMAND);	
				}

			}else{
				xdlc->max_no_response_cnt++;
			}
			break;

		default:
			break;
	}
}



void xdlc_start_sec_nrm_timer(xdlc_dev_t *xdlc)
{
	FUNC_BEGIN();

	if (xdlc->cfg.station != WANOPT_SECONDARY){
		return;
	}

	if (xdlc->cfg.sec_nrm_timer){
		xdlc->sec_nrm_timer = wpabs_get_systemticks();
	}
	
}

void xdlc_stop_sec_nrm_timer(xdlc_dev_t *xdlc)
{
	FUNC_BEGIN();

	if (xdlc->cfg.station != WANOPT_SECONDARY){
		return;
	}

	xdlc->sec_nrm_timer=0;
}



void xdlc_sec_nrm_expiry(xdlc_dev_t *xdlc)
{
	FUNC_BEGIN();

	wpabs_debug_test("%s: %s() State = 0x%X Cnt=%d\n",
			xdlc->name,__FUNCTION__,
			xdlc->state,
			xdlc->max_no_response_cnt);

	if (wpabs_test_bit(XDLC_UNREG_CRIT,&xdlc->critical)){
		wpabs_debug_event("%s: Device down exiting %s()\n",
				xdlc->name);
		return; 
	}

	switch (xdlc->state) {
		
		case XDLC_STATE_CONNECTED:
			wpabs_debug_event("%s: Sec NRM timer expired!\n",
					xdlc->name);
			xdlc_clear_queues(xdlc);
			xdlc_state_change(xdlc,XDLC_STATE_DISCONNECTING);
			xdlc_disconnecting_indication(xdlc, EXCEP_SEC_NRM_TIMEOUT);
			break;
	}
}

