#include <linux/wanpipe_includes.h>
#include <linux/wanpipe_cfg.h>
#include "wanpipe_dsp.h"


#ifdef DSP_DEBUG
extern unsigned short dsp_debug;
#endif

/*
 * ******************************************************************
 * 			FUNCTION DEFINITION			    *
 * ******************************************************************
 */
/*
 * ******************************************************************
 * Common WANPIPE API calls.					    *
 * ******************************************************************
 */
int dsp_api_cmd_setup(dsp_link_t* dsp_link, struct ifreq* ifr)
{
	dspapi_t*	dspapi = NULL;
	int err = 0;

	DSP_ASSERT(dsp_link == NULL);
	DSP_ASSERT(ifr == NULL);
	DSP_ASSERT(ifr->ifr_data == NULL);

	memset(&dsp_link->dsp_api,0,sizeof(dspapi_t));
	memset(dsp_link->call_string,0,sizeof(dsp_link->call_string));
		
	dspapi = (dspapi_t*)ifr->ifr_data;
			
	err = copy_from_user(&dsp_link->dsp_api,
			     &dspapi->hdr,
			     sizeof(dspapi_hdr_t));
	if (err)
		return err;

	if (dsp_link->dsp_api.hdr.dsp_length > 0 && 
	    dsp_link->dsp_api.hdr.dsp_length <= DSP_CALL_STR_SZ){
		err = copy_from_user(dsp_link->call_string,
				     dspapi->data,
				     dspapi->hdr.dsp_length);
		if (err)
			return err;
	}

	return 0;
}


int dsp_api_place_call (dsp_link_t *dsp_link, struct ifreq* ifr)
{
	struct ifreq	  ifr_x25;	
	x25api_t	  x25_api;
	int 		  err = 0;
	
	DSP_ASSERT(dsp_link == NULL);
	/* 
	 */
	if (ifr && ifr->ifr_data) 
		dsp_api_cmd_setup(dsp_link, ifr);
	
	memset(&x25_api, 0x00, sizeof(x25api_t));
	/* Create CALL REQUEST DSP message */
	if (strlen(dsp_link->call_string))
		strcpy(x25_api.data, dsp_link->call_string);
	sprintf(&x25_api.data[strlen(dsp_link->call_string)], 
		" -u%02X%02X%02X%02X%02X%02X%02X%02X%02X",
		(dsp_link->cfg.pad_type == DSP_HPAD) ?
			DSP_CALL_REQ_HOST_MSG : DSP_CALL_REQ_TERM_MSG,
		dsp_link->dsp_api.hdr.dsp_src_cu_addr, 
		dsp_link->dsp_api.hdr.dsp_src_dev_addr,
		dsp_link->dsp_api.hdr.dsp_ctrl_byte, 
		dsp_link->dsp_api.hdr.dsp_dev_info,
		dsp_link->dsp_api.hdr.dsp_crm_type, 
		0x00, 
		dsp_link->dsp_api.hdr.dsp_dst_cu_addr,
		dsp_link->dsp_api.hdr.dsp_dst_dev_addr);
	printk(KERN_INFO "%s: DSP Place Call %s.\n", 
			dsp_link->name, x25_api.data); 
	/* Update X25 API header */
	x25_api.hdr.length = strlen(x25_api.data);
	ifr_x25.ifr_data = (char*)&x25_api;
	
	if (dsp_link->state != DSP_CONNECTING_CALL_REQ_AGAIN)
		dsp_set_state(dsp_link, DSP_CONNECTING);
	if (dsp_link->master && dsp_link->master->do_ioctl){
		err = dsp_link->master->do_ioctl(dsp_link->master, 
						 &ifr_x25, 
				   		 SIOC_X25_PLACE_CALL);
		/* Start timer only if AUTO CALL REQUEST is ON. */
		if (err == DSP_OK && dsp_link->cfg.auto_call_req)
			dsp_start_t1_timer(dsp_link);
		dsp_link->dsp_api_event.hdr.dsp_pad_type = dsp_link->cfg.pad_type;
		if (x25_get_call_info(dsp_link->master, &dsp_link->lcn, NULL, NULL)){
			printk(KERN_INFO "%s: No master device\n", 
					dsp_link->name);
			return -ENODEV;
		}
		DSP_INC_TX_STAT(tx_call_req);	
	}else{
		err = -ENODEV;
	}
	return err;
}


int dsp_api_clear_call (dsp_link_t *dsp_link, struct ifreq* ifr)
{
	int 		err = 0;
	dspapi_t*	dspapi = NULL;
	x25api_t	x25api;
	struct ifreq	ifr_x25;

	DSP_ASSERT(dsp_link == NULL);

	if (dsp_link->state == DSP_READY || 
	    dsp_link->state == DSP_DISCONNECTED || 
	    dsp_link->state & DSP_MJ_DISCONNECTING)
		return 0;
	
	DEBUG1((KERN_INFO "%s: DSP Clear Call.\n", dsp_link->name));
	
	dsp_set_state(dsp_link, DSP_DISCONNECTING_CL_REQ);

	if (ifr && ifr->ifr_data){
		dspapi = (dspapi_t*)ifr->ifr_data;
		err=copy_from_user(&dsp_link->dsp_api,dspapi,sizeof(dspapi_hdr_t));
		if (err)
			return err;

		if (dspapi->hdr.dsp_length > 0 && 
		    dspapi->hdr.dsp_length <= DSP_CALL_STR_SZ){
			err = copy_from_user(x25api.data,
					     dspapi->data,
					     dspapi->hdr.dsp_length);
			if (err)
				return err;
			ifr_x25.ifr_data = (char*)&x25api;	
		}
	}else{
		ifr_x25.ifr_data = NULL;
	}
	
	if (dsp_link->master && dsp_link->master->do_ioctl){
		DEBUG1((KERN_INFO "%s: DSP Clear Call Sent...\n", dsp_link->name));
		err = dsp_link->master->do_ioctl(
					dsp_link->master, 
					(ifr_x25.ifr_data)?&ifr_x25:NULL, 
				   	SIOC_X25_CLEAR_CALL);
		if (!err && dsp_link->cfg.auto_call_req)
			dsp_stop_t1_timer(dsp_link);
	}else{
		err = -ENODEV;
	}

	return err;
}

int dsp_api_accept_call(dsp_link_t *dsp_link, struct ifreq *ifr)
{
	struct ifreq	ifr_x25;
	x25api_t	x25api;
	dspapi_t*	dspapi = NULL;
	int 		err = 0;
	
	DSP_ASSERT(dsp_link == NULL);
	memset(&dsp_link->dsp_api,0,sizeof(dspapi_t));

	/* Read LCN from X25 */
	memset(&x25api,0,sizeof(x25api_t));
	ifr_x25.ifr_data = NULL;
	x25_get_call_info(dsp_link->master, &dsp_link->lcn,NULL,NULL);

	printk(KERN_INFO "%s: DSP Accepted Call\n", dsp_link->name);
	
	if (ifr && ifr->ifr_data){
		dspapi = (dspapi_t*)ifr->ifr_data;
		err=copy_from_user(&dsp_link->dsp_api,
				   dspapi,
				   sizeof(dspapi_hdr_t));
		if (err)
			return err;

		if (dspapi->hdr.dsp_length > 0 && 
		    dspapi->hdr.dsp_length <= DSP_CALL_STR_SZ){
			err = copy_from_user(x25api.data,
					     dspapi->data,
					     dspapi->hdr.dsp_length);
			if (err)
				return err;
			ifr_x25.ifr_data = (char*)&x25api;	
		}
	}else{
		ifr_x25.ifr_data = NULL;
	}
	if (dsp_link->master && dsp_link->master->do_ioctl)
		err = dsp_link->master->do_ioctl(
					dsp_link->master, 
					(ifr_x25.ifr_data)?&ifr_x25:NULL, 
				   	SIOC_X25_ACCEPT_CALL);
	else
		return -ENODEV;

	if (err)
		return err;

	DSP_INC_TX_STAT(tx_call_acc);	
	/* If AUTO_CE is TRUE, send CIRCUIT ENABLED now */
	if (dsp_link->cfg.auto_ce)
		return dsp_api_circuit_enabled(dsp_link, NULL);
	return 0;
}

int dsp_api_reset_call(dsp_link_t *dsp_link, struct ifreq *ifr)
{
	struct ifreq	ifr_x25;
	x25api_t 	x25api;
	dspapi_t*	dspapi = NULL;
	int 		err = 0;;

	DSP_ASSERT(dsp_link == NULL);
	memset(&dsp_link->dsp_api,0,sizeof(dspapi_t));

	printk(KERN_INFO "%s: DSP Reset Call\n", dsp_link->name);
	if (ifr && ifr->ifr_data){
		dspapi = (dspapi_t*)ifr->ifr_data;
		err=copy_from_user(x25api.data,
				   dspapi,
				   sizeof(dspapi_hdr_t));
		if (err) return err;
		ifr_x25.ifr_data = (char*)&x25api;
	}else{
		ifr_x25.ifr_data = NULL;
	}
	
	if (dsp_link->master && dsp_link->master->do_ioctl)
		err = dsp_link->master->do_ioctl(
					dsp_link->master, 
					(ifr_x25.ifr_data)?&ifr_x25:NULL, 
				   	SIOC_X25_RESET_CALL);
	else
		err = -ENODEV;

	return err;
}


int dsp_api_get_cmd_result(dsp_link_t *dsp_link, struct ifreq *ifr)
{
	dspapi_t*	dspapi = (dspapi_t*)ifr->ifr_data;
	int		err = 0;
	
	DSP_ASSERT(dsp_link == NULL);
	DSP_ASSERT(ifr == NULL);
	DSP_ASSERT(ifr->ifr_data == NULL);

	dsp_link->dsp_api_event.hdr.dsp_lcn = dsp_link->lcn;
	err = copy_to_user(dspapi, 
			   &dsp_link->dsp_api_event, 
			   sizeof(dspapi_t));
		
	return (!err) ? DSP_OK : err;
}

/*
 * ******************************************************************
 * Private DSP API calls.					    *
 * ******************************************************************
 */
int dsp_api_circuit_enabled(dsp_link_t* dsp_link, struct ifreq* ifr)
{
	struct sk_buff*	call_skb = NULL;
	dspapi_hdr_t	dspapi_hdr;
	char*		data = NULL;
	int 		err = 0;

	DSP_ASSERT(dsp_link == NULL);

	DEBUG5((KERN_INFO "%s: TX M_CE.\n", dsp_link->name)); 
	if (ifr && ifr->ifr_data){
		/* Circuit ENABLED sent from application */
		err = copy_from_user(&dspapi_hdr,
				     ifr->ifr_data,
				     sizeof(dspapi_hdr_t));
		if (err)
			return err;
	}else{
		memcpy(&dspapi_hdr, 
		       &dsp_link->dspapi_hdr,
		       sizeof(dspapi_hdr_t));
		       
	}

	ALLOC_SKB(call_skb,DSP_CIRCUIT_ENABLED_MSG_LEN,dsp_link);
	if (call_skb == NULL){
		printk(KERN_INFO "%s: Critical Error: No Memory!\n",
			dsp_link->name);
		return -ENOMEM;
	}
	data = skb_put(call_skb, DSP_CIRCUIT_ENABLED_MSG_LEN);
	IS_NULL(data, -ENOMEM);
	DSP_SKB_INC(call_skb->truesize, dsp_link);
	/* Create CALL REQUEST DSP message */
	data[0] = dspapi_hdr.dsp_user_cn;
	data[1] = DSP_CIRCUIT_ENABLED_MSG;
	data[2] = dspapi_hdr.dsp_src_cu_addr;
	data[3] = dspapi_hdr.dsp_src_dev_addr;
	data[4] = dspapi_hdr.dsp_ctrl_byte;
	data[5] = dspapi_hdr.dsp_dev_info;
	data[6] = dspapi_hdr.dsp_crm_type;
	data[7] = 0x00;
	data[8] = dspapi_hdr.dsp_dst_cu_addr;
	data[9] = dspapi_hdr.dsp_dst_dev_addr;
	err = dsp_send_control(dsp_link, DSP_CIRCUIT_ENABLED_MSG, 
			       call_skb->data, call_skb->len);
	if (err == DSP_OK){
		dsp_set_state(dsp_link, DSP_CONNECTED);
		DSP_INC_TX_STAT(tx_ce);	
	}
	KFREE_SKB(call_skb,dsp_link);
	return err;
}

int dsp_api_circuit_reset(dsp_link_t* dsp_link, struct ifreq* ifr, int is_resp)
{
	struct sk_buff*	call_skb = NULL;
	unsigned char	reason_code = 0x00,
			user_cn = 0x00;
	unsigned char*	data = NULL;
	int 		err = 0;

	DSP_ASSERT(dsp_link == NULL);

	if (dsp_link->state != DSP_CONNECTED && 
	    dsp_link->state != DSP_CONNECTED_CR){
		printk(KERN_INFO "%s: DSP not in the CONNECTED state (%i)\n",
				dsp_link->name, dsp_link->state);
		return -EINVAL;
	}
	DEBUG5((KERN_INFO "%s: TX M_CR %d %d\n",
			dsp_link->name, dsp_link->rx_sn, dsp_link->tx_sn)); 
	if (ifr && ifr->ifr_data){
		/* Circuit REST sent from application */
		dspapi_hdr_t	dspapi_hdr;

		err = copy_from_user(&dspapi_hdr,
				     ifr->ifr_data,
				     sizeof(dspapi_hdr_t));
		if (err)
			return err;
		reason_code 	= dspapi_hdr.dsp_status;
		user_cn		= dspapi_hdr.dsp_user_cn;
		is_resp 	= dspapi_hdr.dsp_resp;
	}else{
		reason_code = dsp_link->reason_code;
	}

	ALLOC_SKB(call_skb,DSP_CIRCUIT_RESET_MSG_LEN,dsp_link);
	if (call_skb == NULL){
		printk(KERN_INFO "%s: Critical Error: No Memory!\n", 
			dsp_link->name);
		return -ENOMEM;
	}
	data = skb_put(call_skb, DSP_CIRCUIT_RESET_MSG_LEN);
	IS_NULL(data, -ENOMEM);
	DSP_SKB_INC(call_skb->truesize, dsp_link);
	/* Create CIRCUIT RESET message */
	call_skb->data[0] = user_cn;
	call_skb->data[1] = DSP_CIRCUIT_RESET_MSG;
	if (dsp_link->cfg.pad_type == DSP_HPAD){
		call_skb->data[2] = dsp_link->rx_sn;
		call_skb->data[3] = dsp_link->tx_sn;
	}else{
		call_skb->data[2] = dsp_link->tx_sn;
		call_skb->data[3] = dsp_link->rx_sn;
	}
	call_skb->data[4] = reason_code;
	err = dsp_send_control(dsp_link, DSP_CIRCUIT_RESET_MSG, 
			       call_skb->data, call_skb->len);
	if (err == DSP_OK){
		DSP_INC_TX_STAT(tx_cr);	
	}
	/* Start timer if send was success, AUTO_ITC and this is not
	 * response.
	 * */
	if (err == DSP_OK && is_resp == DSP_FALSE)
		dsp_start_t2_timer(dsp_link);
	/* If DSP in DSP_CONNECTED state, change state to DSP_CONNECTED_CR */
	if (dsp_link->state == DSP_CONNECTED && is_resp == DSP_FALSE){
		dsp_set_state(dsp_link, DSP_CONNECTED_CR);	
	}
	KFREE_SKB(call_skb,dsp_link);
	return 0;
}

int dsp_api_inv_to_clear(dsp_link_t* dsp_link, struct ifreq* ifr)
{
	struct sk_buff*	call_skb = NULL;
	unsigned char*	data = NULL;
	unsigned char	reason_code = 0x00;
	int 		err = 0;

	DSP_ASSERT(dsp_link == NULL);

	DEBUG5((KERN_INFO "%s: TX M_ITC\n", dsp_link->name)); 
	if (ifr && ifr->ifr_data){
		dspapi_hdr_t	dspapi_hdr;

		err = copy_from_user(&dspapi_hdr,
				     ifr->ifr_data,
				     sizeof(dspapi_hdr_t));
		if (err)
			return err;
		reason_code = dspapi_hdr.dsp_status;
	}else{
		reason_code = dsp_link->reason_code;
	}

	ALLOC_SKB(call_skb,DSP_INV_TO_CLEAR_MSG_LEN,dsp_link);
	if (call_skb == NULL){
		printk(KERN_INFO "%s: Critical Error: No Memory!\n", 
			dsp_link->name);
		return -ENOMEM;
	}
	data = skb_put(call_skb, DSP_INV_TO_CLEAR_MSG_LEN);
	IS_NULL(data, -ENOMEM);
	DSP_SKB_INC(call_skb->truesize, dsp_link);
	/* Create CALL REQUEST DSP message */
	call_skb->data[0] = 0x00;
	call_skb->data[1] = DSP_INV_TO_CLEAR_MSG;
	call_skb->data[2] = reason_code;
	err = dsp_send_control(dsp_link, DSP_INV_TO_CLEAR_MSG, 
			       call_skb->data, call_skb->len);
	if (err == DSP_OK){
		dsp_set_state(dsp_link, DSP_DISCONNECTING);
		DSP_INC_TX_STAT(tx_inv_to_clear);	
	}
	KFREE_SKB(call_skb,dsp_link);
	return 0;
}


int dsp_api_eeack(dsp_link_t* dsp_link, struct ifreq* ifr)
{
	struct sk_buff*	call_skb = NULL;
	unsigned char*	data = NULL;
	unsigned char	seq_num = 0x00,
			next_ack_sn = 0x00,
			user_cn = 0x00;
	int 		err = 0;

	DSP_ASSERT(dsp_link == NULL);
	if (ifr && ifr->ifr_data){
		dspapi_hdr_t	dspapi_hdr;

		err = copy_from_user(&dspapi_hdr,
				     ifr->ifr_data,
				     sizeof(dspapi_hdr_t));
		if (err)
			return err;
		seq_num = dspapi_hdr.dsp_sn;
		user_cn = dspapi_hdr.dsp_user_cn;
	}else{
		seq_num = dsp_link->rx_sn;
	}

	DEBUG5((KERN_INFO "%s: TX M_EEACK (%i)\n", dsp_link->name, seq_num)); 
	dsp_stop_t3_timer(dsp_link);
	if (dsp_chk_ack_sn(dsp_link, seq_num, &next_ack_sn, DSP_TRUE) == DSP_FALSE){
		dspapi_hdr_t hdr;
		printk(KERN_INFO "%s: M_EEACK (%i) out of order (next=%d)\n",
				dsp_link->name, seq_num, next_ack_sn);
		/* Send OOB message to the socket with the extected sequence number */
		hdr.dsp_pkt_type	= DSP_ACK_MSG;
		hdr.dsp_sn		= next_ack_sn;
		dsp_snd_api_oob_msg(dsp_link, &hdr, NULL, 0);
		dsp_start_t3_timer(dsp_link);
		return -EINVAL;
	}

	ALLOC_SKB(call_skb,DSP_ACK_MSG_LEN,dsp_link);
	if (call_skb == NULL){
		printk(KERN_INFO "%s: Critical Error: No Memory!\n",
			dsp_link->name);
		return -ENOMEM;
	}
	data = skb_put(call_skb, DSP_ACK_MSG_LEN);
	IS_NULL(data, -ENOMEM);
	DSP_SKB_INC(call_skb->truesize, dsp_link);
	/* Create DSP End-to-End Acknoledgment message */
	call_skb->data[0] = user_cn;
	call_skb->data[1] = DSP_ACK_MSG;
	call_skb->data[2] = seq_num;
	err = dsp_send_control(dsp_link, DSP_ACK_MSG, 
			       call_skb->data, call_skb->len);
	if (err == DSP_OK){
		DSP_INC_TX_STAT(tx_ack);	
	}
	/* If there is another Data Message waiting for ACK, start
	 * T3 again now.
	 */
	if (dsp_wait4ack(dsp_link) == DSP_TRUE){
		dsp_start_t3_timer(dsp_link);
	}
	KFREE_SKB(call_skb,dsp_link);
		
	return 0;
}

int dsp_api_circuit_disconnect(dsp_link_t* dsp_link, struct ifreq* ifr)
{
	struct sk_buff*	call_skb = NULL;
	unsigned char*	data = NULL;
	unsigned char	reason_code = 0x00,
			user_cn = 0x00;
	int 		err = 0;

	DSP_ASSERT(dsp_link == NULL);

	DEBUG5((KERN_INFO "%s: TX M_CD\n", dsp_link->name)); 
	if (ifr && ifr->ifr_data){
		/* Circuit REST sent from application */
		dspapi_hdr_t	dspapi_hdr;

		err = copy_from_user(&dspapi_hdr,
				     ifr->ifr_data,
				     sizeof(dspapi_hdr_t));
		if (err)
			return err;
		/* Reason code */
		reason_code 	= dspapi_hdr.dsp_status; 
		/* User circuit number */
		user_cn 	= dspapi_hdr.dsp_user_cn;

	}else{
		reason_code 	= dsp_link->reason_code;
	}

	ALLOC_SKB(call_skb,DSP_CIRCUIT_DISCONNECT_MSG_LEN,dsp_link);
	if (call_skb == NULL){
		printk(KERN_INFO "%s: Critical Error: No Memory!\n",
			dsp_link->name);
		return -ENOMEM;
	}
	data = skb_put(call_skb, DSP_CIRCUIT_DISCONNECT_MSG_LEN);
	IS_NULL(data, -ENOMEM);
	DSP_SKB_INC(call_skb->truesize, dsp_link);
	/* Create CIRCUIT RESET message */
	call_skb->data[0] = user_cn;
	call_skb->data[1] = DSP_CIRCUIT_DISCONNECT_MSG;
	call_skb->data[2] = reason_code;
	err = dsp_send_control(dsp_link, DSP_CIRCUIT_DISCONNECT_MSG, 
			       call_skb->data, call_skb->len);
	if (err == DSP_OK){
		DSP_INC_TX_STAT(tx_cd);	
	}
	KFREE_SKB(call_skb,dsp_link);
	return 0;
}

int dsp_api_status(dsp_link_t* dsp_link, struct ifreq* ifr)
{
	struct sk_buff*	call_skb = NULL;
	unsigned char*	data = NULL;
	dspapi_hdr_t	dspapi_hdr;
	unsigned char	user_cn = 0x00;
	int 		err = 0;

	DSP_ASSERT(dsp_link == NULL);

	DEBUG5((KERN_INFO "%s: TX M_STATUS\n", dsp_link->name)); 
	if (ifr && ifr->ifr_data){
		/* Circuit REST sent from application */
		err = copy_from_user(&dspapi_hdr,
				     ifr->ifr_data,
				     sizeof(dspapi_hdr_t));
		if (err)
			return err;
		user_cn 	= dspapi_hdr.dsp_user_cn;
	}else
		return -EINVAL;

	ALLOC_SKB(call_skb,DSP_STATUS_MSG_LEN,dsp_link);
	if (call_skb == NULL){
		printk(KERN_INFO "%s: Critical Error: No Memory!\n", 
			dsp_link->name);
		return -ENOMEM;
	}
	data = skb_put(call_skb, DSP_STATUS_MSG_LEN);
	IS_NULL(data, -ENOMEM);
	DSP_SKB_INC(call_skb->truesize, dsp_link);
	/* Create CIRCUIT RESET message */
	call_skb->data[0] = user_cn;
	call_skb->data[1] = DSP_STATUS_MSG;
	call_skb->data[2] = dspapi_hdr.dsp_status;
	call_skb->data[3] = dspapi_hdr.dsp_sense;
	err = dsp_send_control(dsp_link, DSP_STATUS_MSG, 
			       call_skb->data, call_skb->len);
	if (err == DSP_OK){
		DSP_INC_TX_STAT(tx_status);	
	}
	KFREE_SKB(call_skb,dsp_link);
	return 0;
}

int dsp_api_circuit_request(dsp_link_t* dsp_link, struct ifreq* ifr)
{
	struct sk_buff*	call_skb = NULL;
	unsigned char*	data = NULL;
	dspapi_hdr_t	dspapi_hdr;
	int 		err = 0;

	DSP_ASSERT(dsp_link == NULL);

	DEBUG5((KERN_INFO "%s: TX M_CIRCUIT_REQUEST.\n", dsp_link->name)); 
	if (ifr && ifr->ifr_data){
		/* Circuit REQUEST sent from application */
		err = copy_from_user(&dspapi_hdr,
				     ifr->ifr_data,
				     sizeof(dspapi_hdr_t));
		if (err)
			return err;
	}else
		return -EINVAL;
		

	ALLOC_SKB(call_skb,DSP_CIRCUIT_REQUEST_MSG_LEN,dsp_link);
	if (call_skb == NULL){
		printk(KERN_INFO "%s: Critical Error: No Memory!\n",
			dsp_link->name);
		return -ENOMEM;
	}
	data = skb_put(call_skb, DSP_CIRCUIT_REQUEST_MSG_LEN);
	IS_NULL(data, -ENOMEM);
	DSP_SKB_INC(call_skb->truesize, dsp_link);
	/* Create CALL REQUEST DSP message */
	call_skb->data[0] = dspapi_hdr.dsp_user_cn;
	call_skb->data[1] = DSP_CIRCUIT_REQUEST_MSG;
	call_skb->data[2] = dspapi_hdr.dsp_src_cu_addr;
	call_skb->data[3] = dspapi_hdr.dsp_src_dev_addr;
	call_skb->data[4] = dspapi_hdr.dsp_ctrl_byte;
	call_skb->data[5] = dspapi_hdr.dsp_dev_info;
	call_skb->data[6] = dspapi_hdr.dsp_crm_type;
	call_skb->data[7] = dspapi_hdr.dsp_dst_cu_addr;
	call_skb->data[8] = dspapi_hdr.dsp_dst_dev_addr;
	err = dsp_send_control(dsp_link, DSP_CIRCUIT_REQUEST_MSG, 
			       call_skb->data, call_skb->len);
	if (err == DSP_OK){
		DSP_INC_TX_STAT(tx_creq);	
	}
	KFREE_SKB(call_skb,dsp_link);
	return err;
}

int dsp_api_cmd_res_undelivered(dsp_link_t* dsp_link, struct ifreq* ifr)
{
	struct sk_buff*	call_skb = NULL;
	unsigned char*	data = NULL;
	dspapi_hdr_t	dspapi_hdr;
	int		msg_len = 0;
	int 		err = 0;

	DSP_ASSERT(dsp_link == NULL);

	DEBUG5((KERN_INFO "%s: TX CMD/RESP Undelivered.\n", 
			dsp_link->name)); 
	if (ifr && ifr->ifr_data){
		/* CMD/RES undelivered sent from application */
		err = copy_from_user(&dspapi_hdr,
				     ifr->ifr_data,
				     sizeof(dspapi_hdr_t));
		if (err)
			return err;
	}else
		return -EINVAL;
		
	msg_len = (dspapi_hdr.dsp_resp == DSP_TRUE) ? 
				DSP_RES_UNDELIVERED_MSG_LEN :
				DSP_CMD_UNDELIVERED_MSG_LEN;

	ALLOC_SKB(call_skb,msg_len,dsp_link);
	if (call_skb == NULL){
		printk(KERN_INFO "%s: Critical Error: No Memory!\n",
			dsp_link->name);
		return -ENOMEM;
	}
	data = skb_put(call_skb, msg_len);
	IS_NULL(data, -ENOMEM);
	DSP_SKB_INC(call_skb->truesize, dsp_link);
	/* Create CALL REQUEST DSP message */
	call_skb->data[0] = dspapi_hdr.dsp_user_cn;
	call_skb->data[1] = DSP_CMD_RES_UNDELIVERED_MSG;
	call_skb->data[2] = dspapi_hdr.dsp_sn;
	call_skb->data[3] = dspapi_hdr.dsp_error_code;
	if (dspapi_hdr.dsp_resp == DSP_TRUE){
		call_skb->data[4] = dspapi_hdr.dsp_status;
		call_skb->data[5] = dspapi_hdr.dsp_sense;
	}
	err = dsp_send_control(dsp_link, DSP_CMD_RES_UNDELIVERED_MSG, 
			       call_skb->data, call_skb->len);
	if (err == DSP_OK){
		DSP_INC_TX_STAT(tx_cmd_res_undelivered);	
	}
	KFREE_SKB(call_skb,dsp_link);
	return err;
}

int dsp_api_cmd_res_aborted(dsp_link_t* dsp_link, struct ifreq* ifr)
{
	struct sk_buff*	call_skb = NULL;
	unsigned char*	data = NULL;
	dspapi_hdr_t	dspapi_hdr;
	int		msg_len = 0;
	int 		err = 0;

	DSP_ASSERT(dsp_link == NULL);

	DEBUG5((KERN_INFO "%s: TX CMD/RESP Aborted.\n", 
			dsp_link->name)); 
	if (ifr && ifr->ifr_data){
		/* CMD/RES aborted sent from application */
		err = copy_from_user(&dspapi_hdr,
				     ifr->ifr_data,
				     sizeof(dspapi_hdr_t));
		if (err)
			return err;
	}else
		return -EINVAL;
		
	msg_len = (dspapi_hdr.dsp_resp == DSP_TRUE) ? 
				DSP_RES_ABORTED_MSG_LEN :
				DSP_CMD_ABORTED_MSG_LEN;

	ALLOC_SKB(call_skb,msg_len,dsp_link);
	if (call_skb == NULL){
		printk(KERN_INFO "%s: Critical Error: No Memory!\n",
			dsp_link->name);
		return -ENOMEM;
	}
	data = skb_put(call_skb, msg_len);
	IS_NULL(data, -ENOMEM);
	DSP_SKB_INC(call_skb->truesize, dsp_link);
	/* Create CALL REQUEST DSP message */
	call_skb->data[0] = dspapi_hdr.dsp_user_cn;
	call_skb->data[1] = DSP_CMD_RES_ABORTED_MSG;
	call_skb->data[2] = dspapi_hdr.dsp_sn;
	call_skb->data[3] = dspapi_hdr.dsp_error_code;
	if (dspapi_hdr.dsp_resp == DSP_TRUE){
		call_skb->data[4] = dspapi_hdr.dsp_status;
		call_skb->data[5] = dspapi_hdr.dsp_sense;
	}
	err = dsp_send_control(dsp_link, DSP_CMD_RES_ABORTED_MSG, 
			       call_skb->data, call_skb->len);
	if (err == DSP_OK){
		DSP_INC_TX_STAT(tx_cmd_res_aborted);	
	}
	KFREE_SKB(call_skb,dsp_link);
	return err;
}

/*
 * ******************************************************************
 * *
 * ******************************************************************
 */
int dsp_snd_api_call_request(dsp_link_t* dsp_link, 
			     void* sk_id, 
			     struct sk_buff* skb)
{
	struct net_device* 	dsp_dev = NULL;
	x25api_hdr_t		x25api_hdr;
	dspapi_hdr_t*		dspapi_hdr = NULL;
	unsigned char* 		buf = NULL;
	
	DSP_ASSERT(dsp_link == NULL);
	dsp_dev = (struct net_device*)dsp_link->dev;

	/* Copy/Remove X25 API header */
	memcpy(&x25api_hdr, skb->data, sizeof(x25api_hdr_t));
	skb_pull(skb, sizeof(x25api_hdr_t));

	/* Add DSP API header */
	if (skb_headroom(skb) < sizeof(dspapi_hdr_t)){
		DSP_SKB_HEAD(skb);
		return -ENOMEM;
	}
	DEBUG6((KERN_INFO "%s: Sending Call Request up to the socket.\n",
				dsp_link->name));
	buf = skb_push(skb, sizeof(dspapi_hdr_t));
	IS_NULL(buf, -ENOMEM);

	dspapi_hdr = (dspapi_hdr_t*)buf;
	memset(dspapi_hdr, 0, sizeof(dspapi_hdr_t));
	
	buf = strstr(&skb->data[sizeof(dspapi_hdr_t)], "-u");
	if (buf == NULL){
		printk(KERN_INFO "%s: There is no User Data in Incomi Call message.\n",
				dsp_link->name);
		return -EINVAL;
	}
	
	dspapi_hdr->dsp_pkt_type	= DSP_GET_CR_BYTE(buf, 2);
	dspapi_hdr->dsp_length 		= x25api_hdr.length;
	dsp_link->dspapi_hdr.dsp_src_cu_addr  = DSP_GET_CR_BYTE(buf, 16);  
	dsp_link->dspapi_hdr.dsp_src_dev_addr = DSP_GET_CR_BYTE(buf, 18);
	dsp_link->dspapi_hdr.dsp_ctrl_byte    = DSP_GET_CR_BYTE(buf, 8);
	dsp_link->dspapi_hdr.dsp_dev_info     = DSP_GET_CR_BYTE(buf, 10);
	dsp_link->dspapi_hdr.dsp_crm_type     = DSP_GET_CR_BYTE(buf, 12);
	dsp_link->dspapi_hdr.dsp_dst_cu_addr  = DSP_GET_CR_BYTE(buf, 4);
	dsp_link->dspapi_hdr.dsp_dst_dev_addr = DSP_GET_CR_BYTE(buf, 6);

	if (dspapi_hdr->dsp_pkt_type == DSP_CALL_REQ_HOST_MSG || 
	    dspapi_hdr->dsp_pkt_type == DSP_CALL_REQ_TERM_MSG){
		if (dspapi_hdr->dsp_pkt_type == DSP_CALL_REQ_HOST_MSG){
			if (dsp_link->cfg.pad_type == DSP_HPAD)
				printk(KERN_INFO "%s: Hmmm... I'm the HOST PAD too!\n",
						dsp_link->name);
		}else{
			if (dsp_link->cfg.pad_type == DSP_TPAD)
				printk(KERN_INFO "%s: Hmmm... I'm the TERM PAD too!\n",
						dsp_link->name);
		}
	}
/* Update skb type */
	skb->pkt_type 	= WAN_PACKET_ERR;
	dev_hold(dsp_dev);
	skb->dev	= (struct net_device*)dsp_dev;
	skb->destructor = wan_skb_destructor;
	skb->protocol	= htons(DSP_PROT);

	return wanpipe_api_listen_rx(skb, sk_id);	
}

int dsp_snd_api_oob_msg (dsp_link_t* dsp_link, 
		 	 dspapi_hdr_t* hdr, 
			 unsigned char *data_ptr, 
			 unsigned short data_len)
{
	unsigned char*	buf = NULL;
	int		len = sizeof(dspapi_hdr_t)+data_len;
	dspapi_hdr_t*	dspapi_hdr = NULL;
	struct sk_buff*	skb = NULL;
	unsigned long 	flags;
	int 		err=0;
	
	DSP_ASSERT(dsp_link == NULL);
	DSP_ASSERT(hdr == NULL);
	
	if (dsp_link->sk_id == NULL)
		return -ENODEV;
		
	ALLOC_SKB(skb,len,dsp_link);
	if (skb == NULL){
		printk(KERN_INFO "%s: Critical Error: No Memory!\n",
			dsp_link->name);
		return -ENOMEM;
	}
	
	DEBUG6((KERN_INFO "%s: Sending OOB MSG %X up to the socket\n",
				dsp_link->name, hdr->dsp_pkt_type));
	buf = skb_put(skb,sizeof(dspapi_hdr_t));
	IS_NULL(buf, -ENOMEM);
	DSP_SKB_INC(skb->truesize, dsp_link);

	dspapi_hdr = (dspapi_hdr_t *)buf;
	memcpy(dspapi_hdr, hdr, sizeof(dspapi_hdr_t));
	 
	if (data_ptr && data_len){
		buf = skb_put(skb,data_len);
	 	IS_NULL(buf, -ENOMEM);
		memcpy(buf, data_ptr, data_len);
	}
	
	skb->pkt_type 	= WAN_PACKET_ERR;
	skb->dev	= dsp_link->dev;
	skb->protocol	= htons(DSP_PROT);

	DSP_SKB_DEC(skb->truesize, dsp_link); 
	err=wanpipe_api_sock_rx(skb, dsp_link->dev, dsp_link->sk_id);
	if (err != DSP_OK){
		if (err == -EINVAL){
			printk(KERN_INFO "%s: Major Error in Socket Above: OOB!!!\n",
							dsp_link->name);
			spin_lock_irqsave(&dsp_link->list_lock,flags);
			__sock_put(dsp_link->sk_id);
			dsp_link->sk_id = NULL;
			spin_unlock_irqrestore(&dsp_link->list_lock,flags);
		}
		KFREE_SKB(skb,dsp_link);
		err=0;
	}
	return err;
}


