/*
 *	LAPB release 002
 *
 *	This code REQUIRES 2.1.15 or higher/ NET3.038
 *
 *	This module:
 *		This module is free software; you can redistribute it and/or
 *		modify it under the terms of the GNU General Public License
 *		as published by the Free Software Foundation; either version
 *		2 of the License, or (at your option) any later version.
 *
 *	History
 */

#include <linux/wanpipe_includes.h>
#include <linux/wanpipe_cfg.h>
#include "wanpipe_lapb.h"

extern struct lapb_x25_register_struct x25_protocol;
 
extern lapb_cb* dev_to_lapb(struct net_device *dev);
int lapb_enqueue_data(struct net_device *lapb_dev, struct sk_buff *skb);


int lapb_x25_register(struct net_device *lapb_dev,
		      char *dev_name,
		      struct net_device **new_dev)
{
	lapb_cb *lapb;

	if ((lapb = dev_to_lapb(lapb_dev)) == NULL){
		printk(KERN_INFO "Error: Lapb X25 Reg: No Lapb struct on %s\n",lapb_dev->name);
		return -ENODEV;
	}

	if (!IS_FUNC_CALL(x25_protocol,x25_register)){
		printk(KERN_INFO "%s: X25 Protocol has is not supported!\n",
				lapb_dev->name);
		return -EPROTONOSUPPORT;
	}
	
	return x25_protocol.x25_register(&lapb->x25_link, lapb_dev, dev_name, new_dev);
}

int lapb_x25_unregister(struct net_device *x25_dev)
{
	/* We don't know what lapb device is the master
	 * for the x25_dev being supplied.  
	 *
	 * The x25_unregister function will 
	 * provide the lapb_dev.  Furthermore, if its return
	 * code is 0 then the x25_link has been unregistered.
	 * Therefore break the link to the x25.
	 */

	struct net_device *lapb_dev=NULL;
	int x25_link_status=0;

	if (!IS_FUNC_CALL(x25_protocol,x25_unregister)){
		printk(KERN_INFO "%s: X25 Protocol has is not supported!\n",
				lapb_dev->name);
		return  -EPROTONOSUPPORT;
	}
	
	if ((x25_link_status = x25_protocol.x25_unregister(x25_dev)) < 0)
		return -EINVAL;

	return 0;
}


int lapb_x25_setparms (struct net_device *lapb_dev, 
			struct net_device *x25_dev, 
			struct x25_parms_struct *x25_parms)
{
	lapb_cb *lapb;
	
	if ((lapb=(lapb_cb*)lapb_dev->priv) == NULL)
		return -ENODEV;

	/* X25_SW */
	strncpy(x25_parms->virtual_addr, lapb->virtual_addr, WAN_ADDRESS_SZ);
	strncpy(x25_parms->real_addr, lapb->real_addr, WAN_ADDRESS_SZ);
	
	if (!IS_FUNC_CALL(x25_protocol,x25_setparms))
		return -EPROTONOSUPPORT;

	return x25_protocol.x25_setparms(x25_dev, x25_parms);
}


