#include <linux/wanpipe_includes.h>
#include <linux/wanpipe_cfg.h>
#include "wanpipe_x25.h"


extern struct x25_dsp_register_struct x25_dsp_protocol;
extern spinlock_t x25_link_lock;

int send_api_oob_msg (x25_lcn_t *x25_lcn, struct x25_frame *frame, 
		      unsigned char *data_ptr, unsigned short data_len)
{
	unsigned char *buf;
	int len = sizeof(x25api_hdr_t)+data_len;
	x25api_hdr_t *x25_api_hdr;
	struct sk_buff *skb;
	int err=0;
	int truesize=0;
	
	if (x25_lcn->usedby != API)
		return -ENODEV;
		
	if (!x25_lcn->sk_id && !x25_lcn->dsp_id){
		return -ENODEV;
	}
	
	ALLOC_SKB(skb,len);
	if (!skb)
		return -ENOMEM;
	
	truesize=skb->truesize;
	
	buf = skb_put(skb,sizeof(x25api_hdr_t));
	if (!buf){
		KFREE_SKB(skb);
		return -ENOMEM;
	}

	x25_api_hdr = (x25api_hdr_t *)buf;

	x25_api_hdr->qdm = (frame->gfi >> 1);
	if (frame->mbit){
		set_bit(0,(unsigned long*)&x25_api_hdr->qdm);
	}else{
		clear_bit(0,(unsigned long*)&x25_api_hdr->qdm);	
	}
	x25_api_hdr->cause = frame->cause;
	x25_api_hdr->diagn = frame->diag;
	x25_api_hdr->pktType = frame->type;
	x25_api_hdr->length  = len;
	x25_api_hdr->result = 0;
	x25_api_hdr->lcn = frame->lcn;
	x25_api_hdr->mtu = x25_lcn->tx_pkt_size;
	x25_api_hdr->mru = x25_lcn->rx_pkt_size;

	
	if (data_ptr && data_len){
		buf = skb_put(skb,data_len);
		if (!buf){
			KFREE_SKB(skb);
			return -ENOMEM;
		}
		memcpy(buf,data_ptr,data_len);
	}
	
	skb->pkt_type = WAN_PACKET_ERR;
	skb->protocol=htons(ETH_P_X25);

	truesize = skb->truesize;
	
	if (x25_lcn->sk_id){
		err=wanpipe_api_sock_rx(skb,x25_lcn->dev,x25_lcn->sk_id);
		if (err != 0 ){
			if (err == -EINVAL){
				printk(KERN_INFO "%s:Major Error in Socket Above: OOB!!!\n",
								x25_lcn->dev->name);
				__sock_put(x25_lcn->sk_id);
				clear_bit(LCN_SK_ID,&x25_lcn->used);
				x25_lcn->sk_id = NULL;
			}
			KFREE_SKB(skb);
			
			err=0;
		}else{
			X25_SKB_DEC(truesize);
		}
	}else if (x25_lcn->dsp_id){

		if (!IS_FUNC_CALL(x25_dsp_protocol, dsp_oob_msg)){
			printk(KERN_INFO "%s: DSP Protocol has is not supported!\n",
				x25_lcn->dev->name);
			KFREE_SKB(skb);
		}else{
		
			err=x25_dsp_protocol.dsp_oob_msg(x25_lcn->dsp_id,skb);
			if (err != 0 ){
				printk(KERN_INFO "%s:Major Error: Failed to send OOB to dsp!\n",
								x25_lcn->dev->name);
				KFREE_SKB(skb);
			}else{
				X25_SKB_DEC(truesize);
			}
		}

		err=0;

	}else{
		printk(KERN_INFO "%s: Error: No dev for OOB\n",x25_lcn->dev->name);
		err=0;
		KFREE_SKB(skb);
	}

	return err;
}



int send_global_api_oob_msg (x25_link_t *x25_link, struct x25_frame *frame, 
		      unsigned char *data_ptr, unsigned short data_len)
{

	struct net_device *dev;
	int trigger=0;
	x25_lcn_t *x25_lcn;
	unsigned long flags;

	WP_READ_LOCK(&x25_link->dev_list_lock,flags);
	
	for (dev=x25_link->dev_list; dev; dev = *(struct net_device **)dev->priv){
		x25_lcn = (x25_lcn_t *)dev->priv;

		/* Send the message only once */
		if (x25_lcn->usedby == API){
			//FIXME: what about this tigger
			if (x25_lcn->sk_id && !trigger){
				trigger=1;
				if (send_api_oob_msg (x25_lcn,frame,data_ptr,data_len) != X25_OK){
					printk(KERN_INFO "X25: Failed to pass global OOB msg to API!\n");
				}
			}
		}
	}

	WP_READ_UNLOCK(&x25_link->dev_list_lock,flags);

	return 0;
}




int send_api_call_request (x25_lcn_t *x25_lcn, struct x25_frame *frame,
			   struct sk_buff *skb)
{
	unsigned char *buf;
	int len = skb->len;
	x25api_hdr_t *x25_api_hdr;
		
	if (x25_lcn->usedby != API || !x25_lcn->x25_link->api_sk_id){
		printk(KERN_INFO "X25 Send API call request, no Sk ID\n");
		return -ENODEV;
	}

	if (skb_headroom(skb) < sizeof(x25api_hdr_t)){
		printk(KERN_INFO "X25 Sending API listend no skb room !\n");
		return -EINVAL;
	}
	
	buf = skb_push(skb,sizeof(x25api_hdr_t));
	if (!buf)
		return -ENOMEM;

	x25_api_hdr = (x25api_hdr_t *)buf;

	x25_api_hdr->qdm = (frame->gfi >> 1);
	if (frame->mbit){
		set_bit(0,(unsigned long*)&x25_api_hdr->qdm);
	}else{
		clear_bit(0,(unsigned long*)&x25_api_hdr->qdm);	
	}
	x25_api_hdr->cause = frame->cause;
	x25_api_hdr->diagn = frame->diag;
	x25_api_hdr->pktType = frame->type;
	x25_api_hdr->length  = len;
	x25_api_hdr->result = 0;
	x25_api_hdr->lcn = frame->lcn;
	x25_api_hdr->mtu = x25_lcn->tx_pkt_size;
	x25_api_hdr->mru = x25_lcn->rx_pkt_size;

	
	skb->pkt_type = WAN_PACKET_ERR;
	X25_HOLD(x25_lcn->dev);
	skb->dev=x25_lcn->dev;
	skb->destructor=wan_skb_destructor;
	skb->protocol=htons(ETH_P_X25);

	return wanpipe_api_listen_rx(skb,x25_lcn->x25_link->api_sk_id);
}

int x25_api_cmd_setup(x25_lcn_t *x25_lcn, struct ifreq *ifr)
{
	x25api_t *x25api;
	int err;
	
	memset(&x25_lcn->x25_api_cmd_hdr,0,sizeof(x25api_hdr_t));
	memset(x25_lcn->call_string,0,sizeof(x25_lcn->call_string));
		
	if (!ifr)
		return -EINVAL;
	
	x25api = (x25api_t*)ifr->ifr_data;
	if (!x25api)
		return -EINVAL;
		
	err = copy_from_user(&x25_lcn->x25_api_cmd_hdr,x25api,sizeof(x25api_hdr_t));
	if (err)
		return err;

	if (x25api->hdr.length > 0 && x25api->hdr.length <= X25_CALL_STR_SZ){
		return copy_from_user(x25_lcn->call_string,x25api->data,x25api->hdr.length);
	}else{
		if (IS_PVC(x25_lcn)){
			return 0;
		}
		
		printk(KERN_INFO "%s: Invalid Place Call string: %s!\n",
				x25_lcn->dev->name,
				x25api->hdr.length ? "Call string too big" : 
						     "Call string size is zero");
		return -EINVAL;
	}
	return 0;
}


int x25_api_get_cmd_result(x25_lcn_t *x25_lcn, struct ifreq *ifr)
{
	x25api_t *x25api;
	int err=0;

	if (!ifr)
		return -EINVAL;
	
	x25api = (x25api_t*)ifr->ifr_data;

	if (!x25api)
		return -EINVAL;

	if (x25_lcn->dsp_id){
		memcpy(x25api,&x25_lcn->x25_api_event,sizeof(x25api_t));
	}else{
		err=copy_to_user(x25api,&x25_lcn->x25_api_event,sizeof(x25api_t));
	}
	return err;
}


int x25_api_place_call(x25_lcn_t *x25_lcn, struct ifreq *ifr)
{
	x25api_t *x25api = NULL;
	int err=0;
	unsigned long flags;

	if (IS_PVC(x25_lcn)){


		switch (x25_lcn->state){

			case X25_ST_PKT_LVL_RDY_D1:
			case X25_ST_RESET_REQ_SNT_D2:
				break;

			default:
				printk(KERN_INFO "%s: PVC connection failed: PVC not ready!\n",
					x25_lcn->dev->name);
				return -ENOTCONN;
		}
		

		/* We must indicate a state change in order
		 * to indicate to the above socket, that
		 * it's connected.  
		 *
		 * Note we must reset api_state
		 * bit, to simulate that the link has just come up,
		 * otherwise the, state change will not trigger
		 * socket state change. */
		clear_bit(0,&x25_lcn->api_state);

		if (x25_lcn->x25_api_cmd_hdr.pktType == RESET_REQUEST_PKT){

			if (x25_lcn->state == X25_ST_RESET_REQ_SNT_D2){
				/* If the pvc was left in D2 state, we must
				 * get it out of this state, because, reset
				 * function will not send a reset when PVC
				 * is in D2 state. */
				lcn_state_change(x25_lcn,X25_ST_RESET_REQ_REC_D3);
			}
			spin_lock_irqsave(&x25_lcn->x25_link->bh_lock,flags);
			err=x25_reset_call(x25_lcn);
			spin_unlock_irqrestore(&x25_lcn->x25_link->bh_lock,flags);
		}else{
			lcn_state_change(x25_lcn,X25_ST_PKT_LVL_RDY_D1);

			spin_lock_irqsave(&x25_lcn->x25_link->bh_lock,flags);
			x25_lcn->x25_api_event.hdr.pktType = RESET_REQUEST_PKT;
			x25_lcn->x25_api_event.hdr.lcn = atomic_read(&x25_lcn->lcn);
			x25_lcn->x25_api_event.hdr.mtu = x25_lcn->tx_pkt_size;
			x25_lcn->x25_api_event.hdr.mru = x25_lcn->rx_pkt_size;
			spin_unlock_irqrestore(&x25_lcn->x25_link->bh_lock,flags);
		}

		return err;
	}
	
	if (ifr && ifr->ifr_data){
		
		x25api = (x25api_t*)ifr->ifr_data;
		
		if (x25_lcn->dsp_id){
			memcpy(&x25_lcn->x25_api_cmd_hdr,x25api,sizeof(x25api_hdr_t));
		}else{
			err = copy_from_user(&x25_lcn->x25_api_cmd_hdr,x25api,sizeof(x25api_hdr_t));
			if (err) return err;
		}	
		
		if (x25api->hdr.length > 0 && x25api->hdr.length <= X25_CALL_STR_SZ){
		 	if (x25_lcn->dsp_id){
				memcpy(x25_lcn->call_string,x25api->data,x25api->hdr.length);
			}else{
				err = copy_from_user(x25_lcn->call_string,x25api->data,x25api->hdr.length);
				if (err) return err;
			}
		}
	}

	if (atomic_read(&x25_lcn->lcn) != 0 && atomic_read(&x25_lcn->lcn) != 0xFFFF){
		printk(KERN_INFO "%s: Error: Tx Call Request failed, dev already bound to Lcn=%i\n",
				x25_lcn->dev->name,atomic_read(&x25_lcn->lcn));
		return -EBUSY;
	}

	if (x25_lcn->state != X25_ST_PKT_LVL_RDY_P1){
		return -EINVAL;
	}

	spin_lock_irqsave(&x25_lcn->x25_link->bh_lock,flags);
	err=x25_place_call(x25_lcn);
	spin_unlock_irqrestore(&x25_lcn->x25_link->bh_lock,flags);

	return err;
}



int x25_api_clear_call(x25_lcn_t *x25_lcn, struct ifreq *ifr)
{
	x25api_t *x25api;
	int err;
	unsigned long flags;

        if (IS_PVC(x25_lcn)){
                DEBUG_EVENT("%s: Error: Clear Call not supported on PVC \n",
                                x25_lcn->dev->name);
                return -EOPNOTSUPP;
        }

	memset(&x25_lcn->x25_api_cmd_hdr,0,sizeof(x25api_hdr_t));
	memset(x25_lcn->call_string,0,sizeof(x25_lcn->call_string));

	if (ifr && ifr->ifr_data){
		x25api = (x25api_t*)ifr->ifr_data;
		if (x25_lcn->dsp_id){
			memcpy(&x25_lcn->x25_api_cmd_hdr,x25api,sizeof(x25api_hdr_t));
		}else{
			err=copy_from_user(&x25_lcn->x25_api_cmd_hdr,
					   x25api,
					   sizeof(x25api_hdr_t));
			if (err) return err;
		}

		if (x25api->hdr.length > 0 && x25api->hdr.length <= X25_CALL_STR_SZ){
		 	if (x25_lcn->dsp_id){
				memcpy(x25_lcn->call_string,
				       x25api->data,
				       x25api->hdr.length);
			}else{
				err = copy_from_user(x25_lcn->call_string,
						     x25api->data,
						     x25api->hdr.length);
				if (err) return err;
			}
		}
	}

	if (atomic_read(&x25_lcn->lcn) == 0 || atomic_read(&x25_lcn->lcn)==0xFFFF){
		return 0;
	}

	if (x25_lcn->state == X25_ST_CLEAR_REQ_SNT_P6){
		return 0;
	}

	spin_lock_irqsave(&x25_lcn->x25_link->bh_lock,flags);
	err=x25_clear_call(x25_lcn,0);
	spin_unlock_irqrestore(&x25_lcn->x25_link->bh_lock,flags);

	return err;
}

int x25_api_accept_call(x25_lcn_t *x25_lcn, struct ifreq *ifr)
{
	x25api_t *x25api;
	int err;
	unsigned long flags;

        if (IS_PVC(x25_lcn)){
                DEBUG_EVENT("%s: Error: Accept Call not supported on PVC \n",
                                x25_lcn->dev->name);
                return -EOPNOTSUPP;
        }
	
	memset(&x25_lcn->x25_api_cmd_hdr,0,sizeof(x25api_hdr_t));
	memset(x25_lcn->call_string,0,sizeof(x25_lcn->call_string));

	if (ifr && ifr->ifr_data){
		x25api = (x25api_t*)ifr->ifr_data;
		if (x25_lcn->dsp_id){
			memcpy(&x25_lcn->x25_api_cmd_hdr,x25api,sizeof(x25api_hdr_t));
		}else{
			err=copy_from_user(&x25_lcn->x25_api_cmd_hdr,
					   x25api,
					   sizeof(x25api_hdr_t));
			if (err) return err;
		}

		if (x25api->hdr.length > 0 && x25api->hdr.length <= X25_CALL_STR_SZ){
		 	if (x25_lcn->dsp_id){
				memcpy(x25_lcn->call_string,
				       x25api->data,
				       x25api->hdr.length);
			}else{
				err = copy_from_user(x25_lcn->call_string,
						     x25api->data,
						     x25api->hdr.length);
				if (err) return err;
			}
		}
	}

	spin_lock_irqsave(&x25_lcn->x25_link->bh_lock,flags);
	err=x25_accept_call(x25_lcn);
	spin_unlock_irqrestore(&x25_lcn->x25_link->bh_lock,flags);

	return err;
}


int x25_api_reset_call(x25_lcn_t *x25_lcn, struct ifreq *ifr)
{
	x25api_t *x25api;
	int err;
	unsigned long flags;

	memset(&x25_lcn->x25_api_cmd_hdr,0,sizeof(x25api_hdr_t));
	memset(x25_lcn->call_string,0,sizeof(x25_lcn->call_string));

	if (ifr && ifr->ifr_data){
		x25api = (x25api_t*)ifr->ifr_data;
		if (x25_lcn->dsp_id){
			memcpy(&x25_lcn->x25_api_cmd_hdr,x25api,sizeof(x25api_hdr_t));
		}else{
			err=copy_from_user(&x25_lcn->x25_api_cmd_hdr,
					   x25api,
					   sizeof(x25api_hdr_t));
			if (err) return err;
		}
	}

	spin_lock_irqsave(&x25_lcn->x25_link->bh_lock,flags);
	err=x25_reset_call(x25_lcn);
	spin_unlock_irqrestore(&x25_lcn->x25_link->bh_lock,flags);

	return err;
}

void x25_update_api_state(x25_lcn_t *x25_lcn)
{
	int err=0;

	/* If the LCN state changes from Connected to Disconnected, and
	 * we are in the API mode, then notify the socket that the 
	 * connection has been lost */

	if (x25_lcn->usedby == API && x25_lcn->sk_id){

		if (x25_lcn->state != X25_ST_DATA_TRANSFER_P4 && test_bit(0,&x25_lcn->api_state)){

			clear_bit(0,&x25_lcn->api_state);

			if (IS_PVC(x25_lcn) && x25_lcn->state != X25_ST_HDLC_NOT_IN_ABM){
				err=protocol_connecting(x25_lcn->dev,x25_lcn->sk_id);
			}else if (x25_lcn->state == X25_ST_RESET_REQ_SNT_D2 ||
				  x25_lcn->state == X25_ST_RESET_REQ_REC_D3){
				err=protocol_connecting(x25_lcn->dev,x25_lcn->sk_id);
			}else{
				err=protocol_disconnected (x25_lcn->sk_id);
				__sock_put(x25_lcn->sk_id);
				clear_bit(LCN_SK_ID,&x25_lcn->used);
				x25_lcn->sk_id = NULL;
			}
			return;
		}

			
		if (x25_lcn->state == X25_ST_DATA_TRANSFER_P4 && !test_bit(0,&x25_lcn->api_state)){
			set_bit(0,&x25_lcn->api_state);
			err=protocol_connected (x25_lcn->dev,x25_lcn->sk_id);
			if (err == -EINVAL){
				printk(KERN_INFO "%s:Major Error in Socket Above: CONN!!!\n",
						x25_lcn->dev->name);
				__sock_put(x25_lcn->sk_id);
				clear_bit(LCN_SK_ID,&x25_lcn->used);
				x25_lcn->sk_id = NULL;
				x25_clear_call(x25_lcn,1);
				err=0;
			}
			return;
		}

		if (x25_lcn->state == X25_ST_PKT_LVL_RDY_P1){
			if (IS_PVC(x25_lcn)){
				err=protocol_connecting(x25_lcn->dev,x25_lcn->sk_id);
			}else{
				err = protocol_disconnected (x25_lcn->sk_id);
				__sock_put(x25_lcn->sk_id);
				clear_bit(LCN_SK_ID,&x25_lcn->used);
				x25_lcn->sk_id = NULL;
			}
		}
	}
}


