#include <linux/wanpipe_includes.h>
#include <linux/wanpipe_cfg.h>
#include "wanpipe_x25.h"

extern struct x25_dsp_register_struct x25_dsp_protocol;


static inline void x25_skb_queue_purge(struct sk_buff_head *list)
{
	struct sk_buff *skb;
	while ((skb=skb_dequeue(list))!=NULL){
		KFREE_SKB(skb);
	}
}


inline int x25_bind_lcn_to_dev(x25_link_t *x25_link,x25_lcn_t *x25_lcn)
{
	unsigned long flags;
	WP_WRITE_LOCK(&x25_link->map_lock,flags);
	if (x25_link->lcn_to_dev_map[atomic_read(&x25_lcn->lcn)%MAX_LCN]){
		WP_WRITE_UNLOCK(&x25_link->map_lock,flags);
		return -EBUSY;
	}else{
		X25_HOLD(x25_lcn->dev);
		x25_link->lcn_to_dev_map[atomic_read(&x25_lcn->lcn)%MAX_LCN] = x25_lcn->dev;
	}
	WP_WRITE_UNLOCK(&x25_link->map_lock,flags);
	return 0;
}

inline x25_lcn_t* x25_get_free_tcpip_dev (x25_link_t *x25_link,unsigned short lcn)
{
	struct net_device *dev;
	unsigned long flags;
	x25_lcn_t *x25_lcn=NULL;

	WP_READ_LOCK(&x25_link->dev_list_lock,flags);
	for (dev=x25_link->dev_list; dev; dev = *(struct net_device **)dev->priv){
		x25_lcn = dev_to_x25(dev);

		if (!x25_lcn || 
		    !(dev->flags&IFF_UP) || 
		    test_bit(LCN_DOWN,&x25_lcn->critical)){
			continue;
		}
		
		if (x25_lcn->usedby == TCPIP && 
		    !x25_lcn->dsp_id &&
		    !x25_lcn->sk_id &&
		    !x25_lcn->x25_sw_lcn &&
		    x25_lcn->state == X25_ST_PKT_LVL_RDY_P1 &&
		    atomic_read(&x25_lcn->lcn) == 0 &&
		    atomic_read(&x25_lcn->refcnt) == 1 &&
		    !IS_PVC(x25_lcn) &&
		    !test_and_set_bit(LCN_USED,&x25_lcn->used)){
			atomic_set(&x25_lcn->lcn,lcn);
			break;
		}
	}
	WP_READ_UNLOCK(&x25_link->dev_list_lock,flags);

	
	if (!dev){
		return NULL;
	}

	return x25_lcn;
}


inline x25_lcn_t *x25_get_free_dsp_dev (x25_link_t *x25_link, unsigned short lcn)
{
	struct net_device *dev;
	x25_lcn_t *x25_lcn=NULL;
	unsigned long flags;

	WP_READ_LOCK(&x25_link->dev_list_lock,flags);
	for (dev=x25_link->dev_list; dev; dev = *(struct net_device **)dev->priv){
		x25_lcn = dev_to_x25(dev);
		
		if (!x25_lcn || 
		    !(dev->flags&IFF_UP) || 
		    test_bit(LCN_DOWN,&x25_lcn->critical)){
			continue;
		}


		if (x25_lcn->usedby == API && 
		    x25_lcn->dsp_id &&
		    !x25_lcn->sk_id &&
		    !x25_lcn->x25_sw_lcn &&
		    x25_lcn->state == X25_ST_PKT_LVL_RDY_P1 &&
		    atomic_read(&x25_lcn->lcn) == 0 &&
		    atomic_read(&x25_lcn->refcnt) == 1 &&
		    !IS_PVC(x25_lcn) &&
		    !test_and_set_bit(LCN_USED,&x25_lcn->used)){
			atomic_set(&x25_lcn->lcn,lcn);
			break;
		}else{
			clear_bit(LCN_USED,&x25_lcn->used);
		}
	}
	WP_READ_UNLOCK(&x25_link->dev_list_lock,flags);
	
	if (!dev){
		return NULL;
	}

	return x25_lcn;
}


inline x25_lcn_t * x25_get_free_api_dev (x25_link_t *x25_link, unsigned short lcn)
{
	struct net_device *dev;
	x25_lcn_t *x25_lcn=NULL;
	unsigned long flags;

	WP_READ_LOCK(&x25_link->dev_list_lock,flags);
	for (dev=x25_link->dev_list; dev; dev = *(struct net_device **)dev->priv){
		x25_lcn = dev_to_x25(dev);

		if (!x25_lcn || 
		    !(dev->flags&IFF_UP) || 
		    test_bit(LCN_DOWN,&x25_lcn->critical)){
			continue;
		}

		if (x25_lcn->usedby == API && 
		    !x25_lcn->dsp_id &&
		    !x25_lcn->sk_id &&
		    !x25_lcn->x25_sw_lcn &&
		    x25_lcn->state == X25_ST_PKT_LVL_RDY_P1 &&
		    atomic_read(&x25_lcn->lcn) == 0 &&
		    atomic_read(&x25_lcn->refcnt) == 1 &&
		    !IS_PVC(x25_lcn) &&
		    !test_and_set_bit(LCN_USED,&x25_lcn->used)){
			atomic_set(&x25_lcn->lcn,lcn);
			break;
		}
	}
	WP_READ_UNLOCK(&x25_link->dev_list_lock,flags);

	if (!dev){
		return NULL;
	}

	return x25_lcn;
}

void lcn_state_change(x25_lcn_t *x25_lcn, unsigned int state)
{

	DEBUG_TEST("%s: %s: STATE =%i -> %i\n",
			__FUNCTION__,x25_lcn->dev->name,x25_lcn->state,state);
	
	if (IS_PVC(x25_lcn) && state == X25_ST_PKT_LVL_RDY_P1){
		/* For PVC's convert the svc ready state to
		 * PVC ready state */
		state = X25_ST_PKT_LVL_RDY_D1;
	}
	
	x25_lcn->state = state;
	
	if (x25_lcn->state == X25_ST_DATA_TRANSFER_P4){
		if (x25_lcn->usedby == API){
			x25_lcn->dev->mtu = x25_lcn->tx_pkt_size+sizeof(x25api_hdr_t);
		}else{
			x25_lcn->dev->mtu = x25_lcn->tx_pkt_size;
		}
	}
	
	if (x25_lcn->sk_id){
		x25_update_api_state(x25_lcn);
	
	}else if (x25_lcn->dsp_id){
		x25_update_dsp_state(x25_lcn);
	
	}
		
	if (x25_lcn->state != X25_ST_DATA_TRANSFER_P4){
		clear_bit(0,&x25_lcn->api_state);
	}
}

inline x25_lcn_t *get_lcn_device (x25_link_t *x25_link, unsigned short lcn)
{
	unsigned long flags;
	x25_lcn_t *x25_lcn=NULL;

	WP_READ_LOCK(&x25_link->map_lock,flags);
	if (!x25_link->lcn_to_dev_map[lcn%MAX_LCN]){
		WP_READ_UNLOCK(&x25_link->map_lock,flags);
		return NULL;
	}

	x25_lcn = (x25_lcn_t*)x25_link->lcn_to_dev_map[lcn%MAX_LCN]->priv;
	if (!atomic_read(&x25_lcn->lcn)){
		WP_READ_UNLOCK(&x25_link->map_lock,flags);
		return NULL;
	}
	
	WP_READ_UNLOCK(&x25_link->map_lock,flags);
	return x25_lcn;
}

inline void x25_release_lcn_dev (x25_link_t *x25_link, x25_lcn_t *x25_lcn)
{
	unsigned long flags;
	if (atomic_read(&x25_lcn->lcn) == 0 || 
	    atomic_read(&x25_lcn->lcn) == 0xFFFF){
		atomic_set(&x25_lcn->lcn,0);
		if (test_bit(LCN_USED,&x25_lcn->used)){
			clear_bit(LCN_USED,&x25_lcn->used);
		}
		return;
	}
	
	WP_WRITE_LOCK(&x25_link->map_lock,flags);
	if (x25_link->lcn_to_dev_map[atomic_read(&x25_lcn->lcn)%MAX_LCN]){
		X25_PUT(x25_link->lcn_to_dev_map[atomic_read(&x25_lcn->lcn)%MAX_LCN]);
		x25_link->lcn_to_dev_map[atomic_read(&x25_lcn->lcn)%MAX_LCN]=NULL;
	}
	
	
	if (x25_lcn->x25_sw_lcn){
		X25_PUT((x25_lcn_t*)x25_lcn->x25_sw_lcn);
		x25_lcn->x25_sw_lcn=NULL;
	}
	WP_WRITE_UNLOCK(&x25_link->map_lock,flags);

	atomic_set(&x25_lcn->lcn,0);

	clear_bit(LCN_USED,&x25_lcn->used);
}

struct net_device * x25_get_lcn_dev(x25_link_t *x25_link, unsigned short lcn)
{
	unsigned long flags;
	struct net_device *dev;

	WP_READ_LOCK(&x25_link->map_lock,flags);
	dev = x25_link->lcn_to_dev_map[lcn % MAX_LCN];
	if (dev){
		X25_HOLD(dev);
	}
	WP_READ_UNLOCK(&x25_link->map_lock,flags);

	return dev;
}

void x25_set_lcn_states(x25_link_t *x25, int x25_lcn_state)
{
	struct net_device *dev;
	unsigned long flags;
	
	WP_READ_LOCK(&x25->dev_list_lock,flags);
	for (dev=x25->dev_list; dev; dev = *(struct net_device **)dev->priv){
		x25_lcn_t *x25_lcn = dev_to_x25(dev);
		if (!x25_lcn){
			continue;
		}
		lcn_state_change(x25_lcn,x25_lcn_state);
	}
	WP_READ_UNLOCK(&x25->dev_list_lock,flags);

}

/*=================================================
 * x25_initialize_lcn
 *
 * 
 * Synchronization: 
 *  This function must be called with bh_lock
 *  if its called outside the x25_link bh.
 */ 

void x25_initialize_lcn (x25_lcn_t *x25_lcn, int x25_lcn_state, int hard)
{
	//FIXME LOCK
	x25_skb_queue_purge(&x25_lcn->write_queue);
	x25_skb_queue_purge(&x25_lcn->async_queue);
	
	x25_lcn->Pr_count = 0;
	/* X25_SW X.25 switching */
	x25_lcn->sw_Pr_count = 0;
	x25_lcn->Pr_ack = 0;
	x25_lcn->Ps_count = 0;
	x25_lcn->Ps_ack = 0;

	x25_stop_t12_t22_timer(x25_lcn);

	if (x25_lcn->dev->flags & IFF_UP){
		netif_wake_queue(x25_lcn->dev);
	}
	
	clear_bit (IF_SEND_AWAITING_KICK,&x25_lcn->critical);
	if (x25_lcn->sk_id){
		wanpipe_api_poll_wake(x25_lcn->sk_id);
	}
	
	if (hard){

		/* The hard reset will never happen for PVCs */
		memset(&x25_lcn->dlabel[0],0,WAN_IF_LABEL_SZ);
		x25_lcn->pid=0;
		
		x25_lcn->rx_window_size = x25_lcn->x25_link->cfg.packet_window;
		x25_lcn->tx_window_size = x25_lcn->x25_link->cfg.packet_window;

		x25_lcn->rx_pkt_size = x25_lcn->x25_link->cfg.default_packet_size;
		x25_lcn->tx_pkt_size = x25_lcn->x25_link->cfg.default_packet_size;

		x25_stop_t11_t21_timer(x25_lcn);
		x25_stop_t13_t23_timer(x25_lcn);

		clear_bit(TX_WINDOW_CLOSED,&x25_lcn->critical);
		clear_bit(IF_SEND_AWAITING_KICK,&x25_lcn->critical);
		clear_bit(LCN_TIMER_EXPIRED,&x25_lcn->critical);

		x25_release_lcn_dev (x25_lcn->x25_link, x25_lcn);
	}

	lcn_state_change(x25_lcn,x25_lcn_state);
}


void x25_initialize_all_lcn(x25_link_t *x25, int x25_lcn_state)
{
	struct net_device *dev;
//	int i;
	unsigned long flags;

	WP_READ_LOCK(&x25->dev_list_lock,flags);
	for (dev=x25->dev_list; dev; dev = *(struct net_device **)dev->priv){
		x25_lcn_t *x25_lcn = dev_to_x25(dev);
		
		if (!x25_lcn){
			continue;
		}	

		//FIXME LOCK
		x25_skb_queue_purge(&x25_lcn->write_queue);
		x25_skb_queue_purge(&x25_lcn->async_queue);

		x25_lcn->Pr_count = 0;
		/* X25_SW X.25 switching */
		x25_lcn->sw_Pr_count = 0;
		x25_lcn->Pr_ack = 0;
		x25_lcn->Ps_count = 0;
		x25_lcn->Ps_ack = 0;

		x25_lcn->rx_window_size = x25_lcn->x25_link->cfg.packet_window;
		x25_lcn->tx_window_size = x25_lcn->x25_link->cfg.packet_window;

		x25_lcn->rx_pkt_size = x25_lcn->x25_link->cfg.default_packet_size;
		x25_lcn->tx_pkt_size = x25_lcn->x25_link->cfg.default_packet_size;

		x25_stop_t11_t21_timer(x25_lcn);
		x25_stop_t12_t22_timer(x25_lcn);
		x25_stop_t13_t23_timer(x25_lcn);

		clear_bit(TX_WINDOW_CLOSED,&x25_lcn->critical);
		clear_bit(IF_SEND_AWAITING_KICK,&x25_lcn->critical);
		clear_bit(LCN_TIMER_EXPIRED,&x25_lcn->critical);

		lcn_state_change(x25_lcn,x25_lcn_state);

		if (x25_lcn->x25_sw_lcn && x25_lcn_state==X25_ST_HDLC_NOT_IN_ABM){
			x25_sw_disconnect(x25_lcn->x25_sw_lcn);	
		}

		if (!test_bit(LCN_CONFIG_PVC,&x25_lcn->lcn_config)){
			x25_release_lcn_dev (x25_lcn->x25_link, x25_lcn);
		}
	
		if (x25_lcn->dev){
			netif_start_queue(x25_lcn->dev);
		}
	}
	WP_READ_UNLOCK(&x25->dev_list_lock,flags);

#if 0	
	/* Cannot do this because, we could have
	 * PVCs and PVCs are bound until unregisered */
	WP_WRITE_LOCK(&x25->map_lock,flags);
	for (i=0;i<MAX_LCN;i++){
		if (x25->lcn_to_dev_map[i]){
			X25_PUT(x25->lcn_to_dev_map[i]);
		}
		x25->lcn_to_dev_map[i]=NULL;
	}
	WP_WRITE_UNLOCK(&x25->map_lock,flags);
#endif
}

/*
 *	This routine purges all the queues of frames.
 */
void x25_clear_link_queues(x25_link_t *x25, int x25_lcn_state)
{
	x25->state = x25_lcn_state;

	//FIXME LOCK
	x25_skb_queue_purge(&x25->async_queue);
	x25_skb_queue_purge(&x25->rx_queue);
	
	x25_stop_t10_t20_timer(x25);
	x25_stop_t28_timer(x25);

	x25_initialize_all_lcn(x25,x25_lcn_state);
}


void set_LCN_misc_status_bits (x25_lcn_t *x25_lcn, unsigned char misc_status_bits)
{

	/* Note that if the value passed has the high bit set, 
	 * then we must use the '&' logic.*/

	if(misc_status_bits & 0x80){
		x25_lcn->misc_status_bits = x25_lcn->misc_status_bits & misc_status_bits;
	}else{
		x25_lcn->misc_status_bits = x25_lcn->misc_status_bits | misc_status_bits;
	}
}

int x25_trigger_link_bh(x25_link_t *x25_link,char rx_kick)
{
	if (test_bit(LINK_DOWN,&x25_link->tq_working))
			return 1;
	
	if (x25_link->state == X25_ST_HDLC_NOT_IN_ABM)
		return 1;

	if (test_bit(BH_AWAITING_KICK,&x25_link->tq_working)){
		if (x25_link->master && !netif_queue_stopped(x25_link->master)){
		//	printk(KERN_INFO "%s: X25 trigger waiting for kick with master open %s!\n",
		//			x25_link->name,x25_link->master->name);
			clear_bit(BH_AWAITING_KICK,&x25_link->tq_working);
		}
	}
	
	if ((!test_bit(BH_AWAITING_KICK,&x25_link->tq_working)||rx_kick)){
	
		/* The BH_RUNNING flag is not necessary because
		 * tasklet_schedule has its own flag */
	        set_bit(BH_RUNNING,&x25_link->tq_working);
		tasklet_schedule(&x25_link->x25_task);
		return 0;
	}

	return 1;
}

struct net_device *get_first_tx_dev(x25_link_t *x25_link)
{
	int i;
	unsigned long flags;
	int first_channel,last_channel;
	struct net_device *dev=NULL;
	
	WP_READ_LOCK(&x25_link->map_lock,flags);
	
	/* PVC */
	if (x25_link->cfg.lowest_PVC){
		first_channel=x25_link->cfg.lowest_PVC;
	}else{
		first_channel=x25_link->cfg.lowest_two_way_channel;
	}
	
	if (x25_link->cfg.highest_two_way_channel){
		last_channel=x25_link->cfg.highest_two_way_channel;
	}else{
		last_channel=x25_link->cfg.highest_PVC;
	}
	
	
	for (i=first_channel;
	     i<=last_channel;
	     i++){
		
		if (x25_link->lcn_to_dev_map[i%MAX_LCN]){
			
			X25_HOLD(x25_link->lcn_to_dev_map[i%MAX_LCN]);
			dev=x25_link->lcn_to_dev_map[i%MAX_LCN];
			
			WP_READ_UNLOCK(&x25_link->map_lock,flags);
			return dev;
		}
	}
	WP_READ_UNLOCK(&x25_link->map_lock,flags);

	return NULL;
}

struct net_device *get_next_tx_dev(x25_link_t *x25_link, int lcn)
{
	int orig_lcn = lcn;
	unsigned long flags;
	struct net_device *dev=NULL;
	int first_channel,last_channel;

	/* PVC */
	if (x25_link->cfg.lowest_PVC){
		first_channel=x25_link->cfg.lowest_PVC;
	}else{
		first_channel=x25_link->cfg.lowest_two_way_channel;
	}
	
	if (x25_link->cfg.highest_two_way_channel){
		last_channel=x25_link->cfg.highest_two_way_channel;
	}else{
		last_channel=x25_link->cfg.highest_PVC;
	}

	
	WP_READ_LOCK(&x25_link->map_lock,flags);
	for (;;){
	
		if (++lcn > last_channel){
			lcn = first_channel;
		}
					
		if (x25_link->lcn_to_dev_map[lcn%MAX_LCN]){
			
			X25_HOLD(x25_link->lcn_to_dev_map[lcn%MAX_LCN]);
			dev = x25_link->lcn_to_dev_map[lcn%MAX_LCN];
			
			WP_READ_UNLOCK(&x25_link->map_lock,flags);
			
			return dev;
		}

		/* All LCN's are deactivated */
		if (lcn == orig_lcn){
			WP_READ_UNLOCK(&x25_link->map_lock,flags);
			return NULL;
		}
	}
	WP_READ_UNLOCK(&x25_link->map_lock,flags);
}



/**************** Async Packet Handlers ****************************/


/*=======================================================================
 * handle_Rx_Call_Request_pkt
 *
 * Description:  
 *
 * 	Handle incoming CALL requests. 
 *
 * 	TCP/IP LCN Mode:
 *              Accept or Clear the call based on user defined 
 *              criteria.
 *
 *      API LCN Mode:
 *      	Pass the decoded packet up the stack, and let the
 *      	user respond with an accpet or clear. 
 *
 * 	In case of a packet/any error, the call will be cleared.
 */ 

int handle_Rx_Call_Request_pkt(x25_link_t *x25_link, struct sk_buff *skb, struct x25_frame *frame)
{

	struct sk_buff *newskb=NULL;	
	x25_lcn_t *x25_lcn=NULL;
	int err;
	unsigned int off_facils=0, facils_lgth=0, user_data_lgth=0;

	/* check the length of this received packet - 
	 * if the length is bad, then we are done  */

	if ((err=packet_length_error(x25_link, frame)) != X25_OK){
		printk(KERN_INFO "%s: Rx Call Request :LCN=%i: Invalid Packet Length !!!\n",
				x25_link->name, frame->lcn);
		return err;		
	}
	
	if (!skb->len){
		return -EINVAL;
	}

	if (frame->lcn > x25_link->cfg.highest_two_way_channel ||
	    frame->lcn < x25_link->cfg.lowest_two_way_channel){
		printk(KERN_INFO "%s: Rx Call Request :LCN=%i: Invalid logical channel !!!\n",
				x25_link->name, frame->lcn);
		return PKT_NOT_ALLOWED_UNASSIGNED_LCN;
	}
		
	ALLOC_SKB(newskb,MAX_CALL_REQ_ASYNC_PKT);
	if (!newskb){
		/* Local procedure error */
		return -EINVAL;
	}

	err = x25_decode_asy_pkt(x25_link, newskb, skb, frame,
			         &off_facils, &facils_lgth, &user_data_lgth);

	if (err != 0){
		KFREE_SKB(newskb);
		return err;
	}	

	x25_log_proc_event(x25_link,NULL,"Rx Call Request",
			   frame->lcn,frame->cause,frame->diag,
			   newskb->data);

	err = 64;

	/* Look for -uCC in place call data, if it exists this call
	 * is for the TCP/IP interfaces.  Otherwise, it's intended
	 * for the API */
	if (str_index(newskb->data, "-uCC", strlen(newskb->data))){
		
		int toknum;
		char* token[MAX_TOKENS];

		CALL_LOG(x25_link,"%s: X25 Event: Rx Call Request :TCP/IP:Lcn=%i: %s \n",
				x25_link->name,frame->lcn,newskb->data);
		
		toknum = tokenize(newskb->data, token, "-", " \t\n", "-");
		if (toknum < 1){
			printk(KERN_INFO "%s: X25 Rx Call Req Error: Invalid Call String!\n",
					x25_link->name);
			err =  67;
			goto rx_call_handle_err;
		}
		
		err=64;
		x25_lcn = x25_get_free_tcpip_dev(x25_link,frame->lcn);
		if (x25_lcn){
			if ((err=x25_match_accept_criteria(x25_lcn,token,toknum)) != X25_OK){
				x25_release_lcn_dev (x25_link, x25_lcn);
				goto rx_call_handle_err;
			}
		}else{
			printk(KERN_INFO "%s: X25 Failed to accept incoming call no free TCP/IP svc's!\n",
					x25_link->name);
			goto rx_call_handle_err;
		}

	
		if (x25_bind_lcn_to_dev(x25_link,x25_lcn) != 0){
			x25_release_lcn_dev (x25_link, x25_lcn);
			err = 71;
			printk(KERN_INFO "%s: X25 Critical Error: free device already bouded!\n",
					x25_lcn->dev->name);
			goto rx_call_handle_err;
		}
		
		/* check for an error when decoding the facilities field */
		if (facils_lgth){
			if ((err = error_in_decoding_facils(x25_lcn,
							    frame->type,
				   		            skb->data,
							    facils_lgth,
							    off_facils))) {
				printk(KERN_INFO 
					"%s: X25 Failed to accept incoming call, Invalid Facilites!\n",
						x25_link->name);
				goto rx_call_handle_err;
			}
		}
		
		if (asy_pkt_Data_lgth_excessive(x25_lcn->config,user_data_lgth)){
			printk(KERN_INFO "%s: X25 Failed to accept incoming call, Invalid Data Length!\n",
					x25_link->name);
			err = PKT_TOO_LONG; 
			goto rx_call_handle_err;
		}	

		err = 0;
		lcn_state_change(x25_lcn,X25_ST_CALL_REQ_REC_P3);
		printk(KERN_INFO "%s: Incoming call bounded to TCP/IP device %s\n",
				x25_link->name,x25_lcn->dev->name);

	}else if (str_index(newskb->data, "-u57", strlen(newskb->data)) || 
		  str_index(newskb->data, "-u56", strlen(newskb->data))){

		CALL_LOG(x25_link,"%s: X25 Event: Rx Call Request :DSP:Lcn=%i: %s \n",
			x25_link->name,frame->lcn,newskb->data);
		
		if (!x25_link->dsp_sk_id){
			err = 64;
			printk(KERN_INFO "%s: Error: No listening DSP socket. Clearing call... \n",
					x25_link->name);
			goto rx_call_handle_err;
		}
	
		x25_lcn = x25_get_free_dsp_dev(x25_link,frame->lcn);
		if (!x25_lcn){
			err = 71;
			printk(KERN_INFO "%s: X25 Failed to accept incoming call no free DSP devices!\n",
					x25_link->name);
			goto rx_call_handle_err;
		}

		if (x25_bind_lcn_to_dev(x25_link,x25_lcn) != 0){
			x25_release_lcn_dev (x25_link, x25_lcn);
			err = 71;
			printk(KERN_INFO "%s: X25 Critical Error: free device already bouded!\n",
					x25_lcn->dev->name);
			goto rx_call_handle_err;
		}

		if (facils_lgth){
			if ((err = error_in_decoding_facils(x25_lcn,
							    frame->type,
							    skb->data,
							    facils_lgth,
							    off_facils))) {
				printk(KERN_INFO "%s: Failed to accept incoming call, Invalid Facilites !\n",
						x25_link->name);
				goto rx_call_handle_err;
			}
		}
		
		if (asy_pkt_Data_lgth_excessive(x25_lcn->config,user_data_lgth)){
			printk(KERN_INFO "%s: Failed to accept incoming call, Invalid Data Length\n",
					x25_link->name);
			err = PKT_TOO_LONG; 
			goto rx_call_handle_err;
		}	

		x25_lcn->x25_api_event.hdr.qdm = (frame->gfi >> 1);
		if (frame->mbit){
			set_bit(0,(unsigned long*)&x25_lcn->x25_api_event.hdr.qdm);
		}else{
			clear_bit(0,(unsigned long*)&x25_lcn->x25_api_event.hdr.qdm);
		}
		x25_lcn->x25_api_event.hdr.cause = frame->cause;
		x25_lcn->x25_api_event.hdr.diagn = frame->diag;
		x25_lcn->x25_api_event.hdr.pktType = frame->type;
		x25_lcn->x25_api_event.hdr.length  = 0;
		x25_lcn->x25_api_event.hdr.result = 0;
		x25_lcn->x25_api_event.hdr.lcn = frame->lcn;
		x25_lcn->x25_api_event.hdr.mtu = x25_lcn->tx_pkt_size;
		x25_lcn->x25_api_event.hdr.mru = x25_lcn->rx_pkt_size;

		
		if (newskb->len > 0  && newskb->len <= X25_CALL_STR_SZ){
			x25_lcn->x25_api_event.hdr.length=newskb->len;
			memcpy(x25_lcn->x25_api_event.data, newskb->data, newskb->len);
		}

		X25_SKB_DEC(newskb->truesize);
		if (send_dsp_call_request(x25_lcn,frame,newskb) != X25_OK){
			printk(KERN_INFO "%s: Failed to pass call request to the dsp listen socket!\n",
					x25_link->name);
			err=64;
			X25_SKB_INC(newskb->truesize);
			goto rx_call_handle_err;
		}
		

		 /* Do not free the newskb, since it has been passed 
		  * up to the dsp layer */
                lcn_state_change(x25_lcn,X25_ST_CALL_REQ_REC_P3);
                return 0;

#ifdef X25_SW		
	}else if (str_index(newskb->data, "-uC9", strlen(newskb->data))){

 #ifdef X25_SWITCH_DEFAULT_API
pass_call_to_api:
 #endif
		
#else
	}else{
#endif
	
		CALL_LOG(x25_link,"\n%s: X25 Event: Rx Call Request :API:Lcn=%i: %s\n",
			x25_link->name,frame->lcn,newskb->data);
	
		if (!x25_link->api_sk_id){
			err = 64;
			printk(KERN_INFO "%s: X25 Failed to accept incoming call: No listening API socket!\n",
					x25_link->name);
			goto rx_call_handle_err;
		}
	
		x25_lcn = x25_get_free_api_dev(x25_link,frame->lcn);
		if (!x25_lcn){
			err = 64;
			printk(KERN_INFO "%s: X25 Failed to accept incoming call no free API devices!\n",
					x25_link->name);
			goto rx_call_handle_err;
		}

		if (x25_bind_lcn_to_dev(x25_link,x25_lcn) != 0){
			x25_release_lcn_dev (x25_link, x25_lcn);
			err = 71;
			printk(KERN_INFO "%s: X25 Critical Error: free device already bouded!\n",
					x25_lcn->dev->name);
			goto rx_call_handle_err;
		}

		if (facils_lgth){
			if ((err = error_in_decoding_facils(x25_lcn,
							    frame->type,
							    skb->data,
							    facils_lgth,
							    off_facils))) {
				printk(KERN_INFO "%s: Failed to accept incoming call, Invalid Facilites!\n",
						x25_link->name);
				goto rx_call_handle_err;
			}
		}
	
		if (asy_pkt_Data_lgth_excessive(x25_lcn->config,user_data_lgth)){
			printk(KERN_INFO "%s: Failed to accept incoming call, Invalid Data Length\n",
					x25_link->name);
			err = PKT_TOO_LONG; 
			goto rx_call_handle_err;
		}	

		
		x25_lcn->x25_api_event.hdr.qdm = (frame->gfi >> 1);
		if (frame->mbit){
			set_bit(0,(unsigned long*)&x25_lcn->x25_api_event.hdr.qdm);
		}else{
			clear_bit(0,(unsigned long*)&x25_lcn->x25_api_event.hdr.qdm);
		}
		x25_lcn->x25_api_event.hdr.cause = frame->cause;
		x25_lcn->x25_api_event.hdr.diagn = frame->diag;
		x25_lcn->x25_api_event.hdr.pktType = frame->type;
		x25_lcn->x25_api_event.hdr.length  = 0;
		x25_lcn->x25_api_event.hdr.result = 0;
		x25_lcn->x25_api_event.hdr.lcn = frame->lcn;
		x25_lcn->x25_api_event.hdr.mtu = x25_lcn->tx_pkt_size;
		x25_lcn->x25_api_event.hdr.mru = x25_lcn->rx_pkt_size;
	
		if (newskb->len > 0  && newskb->len <= X25_CALL_STR_SZ){
			x25_lcn->x25_api_event.hdr.length=newskb->len;
			memcpy(x25_lcn->x25_api_event.data, newskb->data, newskb->len);
		}

		X25_SKB_DEC(newskb->truesize);
		if (send_api_call_request(x25_lcn,frame,newskb) != X25_OK){
			printk(KERN_INFO "%s: Failed to pass call request to the API listen socket!\n",
					x25_link->name);
			err=64;
			X25_SKB_INC(newskb->truesize);
			goto rx_call_handle_err;
		}
	
		lcn_state_change(x25_lcn,X25_ST_CALL_REQ_REC_P3);
		return 0;
		
#ifdef X25_SW
	}else{

		/* X25_SW
		 * Received Call Request. 
		 * Try to switchi this Call Request.
		 */
		CALL_LOG(x25_link,"\n%s: X25 Event: Rx Call Request :SWITCH:Lcn=%i: %s\n",
			x25_link->name,frame->lcn,newskb->data);

		x25_lcn = x25_get_free_api_dev(x25_link,frame->lcn);
		if (!x25_lcn){
			err = 64;
			printk(KERN_INFO "%s: X25 Failed to accept incoming call no free SWITCH devices!\n",
					x25_link->name);
			goto rx_call_handle_err;
		}

		if (x25_bind_lcn_to_dev(x25_link,x25_lcn) != 0){
			x25_release_lcn_dev (x25_link, x25_lcn);
			err = 71;
			printk(KERN_INFO "%s: X25 Critical Error: free device already bouded!\n",
					x25_lcn->dev->name);
			goto rx_call_handle_err;
		}

		if (x25_sw_call_request(x25_link, x25_lcn->dev, newskb, frame) != X25_OK){

			x25_release_lcn_dev (x25_link, x25_lcn);
 #ifdef X25_SWITCH_DEFAULT_API
			CALL_LOG(x25_link,"%s: X25 failed to switch incoming call: Passing up the API stack!\n",x25_link->name);
			goto pass_call_to_api;
 #else
			CALL_LOG(x25_link,"%s: X25 failed to switch incoming call: clearing...\n",x25_link->name);
			err=71;

			goto rx_call_handle_err;
 #endif
		}
		
		/* We must free skb if the switching was successful, because
		 * the switching layer doesn't clear the skb buffer */
		lcn_state_change(x25_lcn,X25_ST_CALL_REQ_REC_P3);
		KFREE_SKB(newskb);
		return 0;
#endif
	}

rx_call_handle_err:
	KFREE_SKB(newskb);
	if (err != 0){
		return err;
	}


	if (!x25_lcn){
		printk(KERN_INFO "%s: Error, Rx call req, but no LCN dev\n",x25_link->name);
		return 64;
	}
		
	/* for a Call Request or a Call Accept packet, set the D-bit 
	 * pragmatics flag for the LCN according to the received GFI */
	
	if ( (frame->gfi & GFI_WITH_D_BIT_SET) && 
	     !(x25_link->cfg.X25_protocol_options & D_BIT_PRAGMATICS_NOT_SUPPORTED)){

		x25_lcn->config = x25_lcn->config | D_BIT_PRAGMATICS_FOR_SVC;
	}else{
		x25_lcn->config = x25_lcn->config & ~D_BIT_PRAGMATICS_FOR_SVC;
	}


	
	switch(x25_lcn->x25_link->incoming_call_handling) {	
		
		/* handle the Call Request according to the current 
		 * 'incoming_call_handling' configuration */

		case ACCEPT_ALL_INCOMING_CALLS:				
			
			/* automatically accept all incoming Call Requests 
			 * if the Call Request includes the Fast select 
			 * with restriction on response facilities, then 
			 * pass the call to the application as the Call Request 
			 * may only be cleared */
			
			if (!(x25_lcn->config & FAST_SEL_RESTR_ON_RESP_BIT_SET)) {
			
				/* If the incoming call contained Packet and Window
				 * faciliteis, we must respond with the negotiated
				 * values. */
				
				struct sk_buff *accept_skb;
				unsigned char *tmp_ptr;
				
				if (x25_lcn->config & INCOMING_PKT_WIN_FACILS){
					ALLOC_SKB(accept_skb,7);
					if (!accept_skb){
						goto accept_call_error;
					}
					tmp_ptr=skb_put(accept_skb,1);
					*tmp_ptr=0;
					insert_flow_control_facils(x25_lcn, CALL_ACCEPT_PKT, accept_skb);
					x25_lcn->config &= ~INCOMING_PKT_WIN_FACILS;
				}else{
					ALLOC_SKB(accept_skb,2);
					if (!accept_skb){
						goto accept_call_error;
					}
					tmp_ptr = skb_put(accept_skb,2);
					tmp_ptr[0]=0;
					tmp_ptr[1]=0;
				}
				
				/* issue the Call Accept packet */
				lcn_state_change(x25_lcn,X25_ST_DATA_TRANSFER_P4);
				x25_send_control(x25_lcn->x25_link,
						 gfi_decode(x25_lcn->config),
						 frame->lcn,
						 CALL_ACCEPT_PKT,
						 0, 0,
						 accept_skb->data,
						 accept_skb->len);

				if (accept_skb)
					KFREE_SKB(accept_skb);
			}
			break;

		case CLEAR_ALL_INCOMING_CALLS:				
accept_call_error:			
			/* automatically clear all incoming Call Requests
			 * set the flag which indicates to the 'handle_auto_resp_prot_vio_TO()' 
			 * procedure that the Clear  Request is being automatically 
			 * issued in response to an incoming Call Request packet */
			
			/* issue the Clear Request packet */
			return CALL_SETUP_CLEAR_OR_REGSTR_PROB;

		default:
			/* the default option is to pass all incoming Call Requests to 
			 * the application for processing */
			return CALL_SETUP_CLEAR_OR_REGSTR_PROB;
	}

	return 0; 
}


int handle_Rx_Call_Accept_pkt(x25_lcn_t *x25_lcn, struct sk_buff *skb, struct x25_frame *frame)
{
	struct sk_buff *newskb=NULL;	
	x25_link_t *x25_link=x25_lcn->x25_link;
	int err;
	unsigned int off_facils=0, facils_lgth=0, user_data_lgth=0;


	/* check the length of this received packet - 
	 * if the length is bad, then we are done  */
	if ((err=packet_length_error(x25_link, frame)) != X25_OK){
		//ALEX
		printk(KERN_INFO "%s: X25 Event: Rx Accept Pkt: Invalid packet length %i (err=%02X)\n",
				x25_lcn->dev->name, frame->len, err);
		return err;		
	}

#if 0
ALEX SHORT_ACCEPT_CALL: 
	The Accept Call packet cannot includes Address/Facilities/User Data
	fields if bit 6 in X25_protocol_options is set to 1. So, do not check
	the length for skb (it can be zero).
	       
	if (!skb->len){
		return -EINVAL;
	}
#endif		

	ALLOC_SKB(newskb,MAX_CALL_REQ_ASYNC_PKT);
	if (!newskb){
		/* Local procedure error */
		return -EINVAL;
	}
	err = x25_decode_asy_pkt(x25_link, newskb, skb, frame,
			         &off_facils, &facils_lgth, &user_data_lgth);

	if (err != 0){
		//ALEX
		printk(KERN_INFO "%s: Failed to decode ASY pkt (err=%02X)\n",
				x25_lcn->dev->name, err);
		KFREE_SKB(newskb);
		return err;
	}	

	x25_log_proc_event(x25_link,x25_lcn->dev->name,"Rx Call Accept",
			   frame->lcn,frame->cause,frame->diag,
			   newskb->len?newskb->data:NULL);

	CALL_LOG(x25_link,"%s: X25 Event: Rx Call Accept   :Lcn=%i\n",
			x25_lcn->dev->name, frame->lcn);

	if (newskb->len && newskb->len < X25_CALL_STR_SZ){
		x25_lcn->x25_api_event.hdr.length=newskb->len;
		memcpy(x25_lcn->x25_api_event.data,newskb->data,newskb->len);
	}

	KFREE_SKB(newskb);
	
	/* check for an error when decoding the facilities field */
	if (facils_lgth){
		if ((err = error_in_decoding_facils(x25_lcn,
						    frame->type,
						    skb->data,
						    facils_lgth,
						    off_facils))) {
			printk(KERN_INFO 
				"%s: Failed to accept incoming call ! Invalid Facilites %i\n",x25_lcn->dev->name,err);
			return err;
		}
	}
	
	if (asy_pkt_Data_lgth_excessive(x25_lcn->config,user_data_lgth)){
		printk(KERN_INFO "%s: Failed to accept incoming call! Invalid Data Length\n",x25_lcn->dev->name);
		return PKT_TOO_LONG; 
	}	

	x25_lcn->x25_api_event.hdr.qdm = (frame->gfi >> 1);
	if (frame->mbit){
		set_bit(0,(unsigned long*)&x25_lcn->x25_api_event.hdr.qdm);
	}else{
		clear_bit(0,(unsigned long*)&x25_lcn->x25_api_event.hdr.qdm);
	}
	x25_lcn->x25_api_event.hdr.cause = frame->cause;
	x25_lcn->x25_api_event.hdr.diagn = frame->diag;
	x25_lcn->x25_api_event.hdr.pktType = frame->type;
	x25_lcn->x25_api_event.hdr.result = 0;
	x25_lcn->x25_api_event.hdr.lcn = frame->lcn;
	x25_lcn->x25_api_event.hdr.mtu = x25_lcn->tx_pkt_size;
	x25_lcn->x25_api_event.hdr.mru = x25_lcn->rx_pkt_size;

	
	if (x25_lcn->x25_sw_lcn){
		/* X25_SW
		 * This LCN is used for swtiching. Send Accept Call to remote switching device.
		 */
		x25_sw_accept_call(x25_lcn, skb, frame);	
	}
	
	return X25_OK;
}


/*=======================================================================
 * handle_Rx_Clear_Request_pkt
 *
 * Description:  
 *
 * 	Handle incoming CLEAR requests. 
 *
 *	If the packet is error free, then initialize the LCN:
 *		1. Clear Pr,Ps, numbers,
 *		2. Set the binded LCN number to 0
 *		3. Flush all data queues.
 *
 * 	In case of a packet/any error, the clear request will
 * 	be sent.
 */ 

void handle_Rx_Clear_Request_pkt (x25_lcn_t *x25_lcn, struct sk_buff *skb, struct x25_frame *frame)
{
	struct sk_buff *newskb=NULL;	
	x25_link_t *x25_link=x25_lcn->x25_link;
	int err;
	unsigned int off_facils=0, facils_lgth=0, user_data_lgth=0;
	x25_lcn_t *x25_sw_lcn;

	CALL_LOG(x25_link,"%s: X25 Event: Rx Clear Request :Lcn=%i:Cause=0x%02X:Diag=0x%02X\n",
			x25_lcn->dev->name,frame->lcn,frame->cause,frame->diag);

	/* check the length of this received packet - 
	 * if the length is bad, then we are done  */

	if ((err=packet_length_error(x25_lcn->x25_link, frame))){
		
		printk(KERN_INFO "%s: Error Rx Clear Req: Invalid Packet Length !\n",
				x25_lcn->dev->name);
		
		if (x25_lcn->state != X25_ST_CLEAR_REQ_SNT_P6){
			lcn_state_change(x25_lcn,X25_ST_CLEAR_REQ_SNT_P6);
			x25_send_control(x25_lcn->x25_link,0,frame->lcn,
				 	CLEAR_REQUEST_PKT, 
				 	0,err,NULL,0);
		}
		return;		
	}
	

	/* if we receive a Call Accept with the D-bit set and our 
	 * outgoing Call Request was not issued with the D-bit, then we 
	 * issue a Clear Request with a diagnostic code #166 */
	
	if ((frame->gfi & GFI_WITH_D_BIT_SET) && !(x25_lcn->config & D_BIT_PRAGMATICS_FOR_SVC)){

		if (x25_lcn->state != X25_ST_CLEAR_REQ_SNT_P6){
			lcn_state_change(x25_lcn,X25_ST_CLEAR_REQ_SNT_P6);
			x25_send_control(x25_lcn->x25_link,0,frame->lcn,
					CLEAR_REQUEST_PKT, 
					0,D_BIT_PROCEDURE_NOT_SUPPORTED,NULL,0);
		}

		return;								
	}

	
	/* for a Call Request or a Call Accept packet, set the D-bit 
	 * pragmatics flag for the LCN according to the received GFI */
	
	if ( (frame->gfi & GFI_WITH_D_BIT_SET) && 
	     !(x25_lcn->x25_link->cfg.X25_protocol_options & D_BIT_PRAGMATICS_NOT_SUPPORTED)){

		x25_lcn->config = x25_lcn->config | D_BIT_PRAGMATICS_FOR_SVC;
	}else{
		x25_lcn->config = x25_lcn->config & ~D_BIT_PRAGMATICS_FOR_SVC;
	}

	if (skb->len){
		ALLOC_SKB(newskb,MAX_CALL_REQ_ASYNC_PKT);
		if (!newskb)
			return;

		err = x25_decode_asy_pkt(x25_link, newskb, skb, frame,
			         &off_facils, &facils_lgth, &user_data_lgth);

		if (err != 0){

			if (err < 0)
				err = 0;
			
			if(x25_lcn->state != X25_ST_CLEAR_REQ_SNT_P6){
				printk(KERN_INFO "%s: Error Rx Clear Req: Invalid Packet Format !\n",
						x25_lcn->dev->name);

				lcn_state_change(x25_lcn,X25_ST_CLEAR_REQ_SNT_P6);
				x25_send_control(x25_lcn->x25_link,0,frame->lcn,
						 CLEAR_REQUEST_PKT, 0, err,NULL,0);
			}
			KFREE_SKB(newskb);
			return;
		}	

		if (facils_lgth){
			if ((err = error_in_decoding_facils(x25_lcn,
							    frame->type,
				   		            skb->data,
							    facils_lgth,
							    off_facils))) {
				printk(KERN_INFO "%s: Error Rx Clear Req: Invalid Facilites %i\n",
						x25_lcn->dev->name,err);
				
				KFREE_SKB(newskb);
				
				if(x25_lcn->state != X25_ST_CLEAR_REQ_SNT_P6){
					lcn_state_change(x25_lcn,X25_ST_CLEAR_REQ_SNT_P6);
					x25_send_control(x25_lcn->x25_link,0,frame->lcn,
						 CLEAR_REQUEST_PKT, 0, err,NULL,0);
				}
				return;
			}
		}
		
		if (asy_pkt_Data_lgth_excessive(x25_lcn->config,user_data_lgth)){
			
			printk(KERN_INFO "%s: Error Rx Clear Req: Invalid Data Length %i\n",
					x25_lcn->dev->name,user_data_lgth);
			
			KFREE_SKB(newskb);

			if(x25_lcn->state != X25_ST_CLEAR_REQ_SNT_P6){
				lcn_state_change(x25_lcn,X25_ST_CLEAR_REQ_SNT_P6);
				x25_send_control(x25_lcn->x25_link,0,frame->lcn,
					 CLEAR_REQUEST_PKT, 0, PKT_TOO_LONG,NULL,0);
			}
			return; 
		}	

		x25_log_proc_event(x25_link,x25_lcn->dev->name,"Rx Clear Request",
			   frame->lcn,frame->cause,frame->diag,
			   newskb->data);
	}else{
		x25_log_proc_event(x25_link,x25_lcn->dev->name,"Rx Clear Request",
			   frame->lcn,frame->cause,frame->diag,
			   NULL);
	}

	x25_lcn->x25_api_event.hdr.qdm = (frame->gfi >> 1);
	if (frame->mbit){
		set_bit(0,(unsigned long*)&x25_lcn->x25_api_event.hdr.qdm);
	}else{
		clear_bit(0,(unsigned long*)&x25_lcn->x25_api_event.hdr.qdm);
	}

	x25_lcn->x25_api_event.hdr.cause = frame->cause;
	x25_lcn->x25_api_event.hdr.diagn = frame->diag;
	x25_lcn->x25_api_event.hdr.pktType = frame->type;
	x25_lcn->x25_api_event.hdr.length  = 0;
	x25_lcn->x25_api_event.hdr.result = 0;
	x25_lcn->x25_api_event.hdr.lcn = frame->lcn;

	if (newskb && newskb->len && newskb->len < X25_CALL_STR_SZ){
		x25_lcn->x25_api_event.hdr.length = newskb->len;
		memcpy(x25_lcn->x25_api_event.data,newskb->data,newskb->len);
	}

	
	/* X.25 switching */
	if ((x25_sw_lcn=x25_lcn->x25_sw_lcn)!=NULL){
		//unsigned long flags;
		/* X25_SW
		 * This LCN is used for switching.
		 * Send Clear Request to remote switching device.
		 */
		lcn_state_change(x25_lcn,X25_ST_CLEAR_REQ_REC_P7);
		lcn_state_change(x25_sw_lcn,X25_ST_CLEAR_REQ_SNT_P6);
		//spin_lock_irqsave(&x25_sw_lcn->x25_link->bh_lock,flags);
		x25_sw_send_control(x25_sw_lcn, newskb?newskb:NULL, frame);	
		//spin_unlock_irqrestore(&x25_sw_lcn->x25_link->bh_lock,flags);
	}else{
		if (x25_lcn->usedby == API){
			send_api_oob_msg(x25_lcn,
	 		         frame,
				 newskb ? newskb->data : NULL,
				 newskb ? newskb->len : 0);
		}
	}

	if (newskb){
		KFREE_SKB(newskb);
	}
	
	/* set the window/packet sizes to their original startup values */
	x25_lcn->rx_window_size = x25_lcn->tx_window_size = x25_lcn->x25_link->cfg.packet_window;
	x25_lcn->rx_pkt_size = x25_lcn->tx_pkt_size = x25_lcn->x25_link->cfg.default_packet_size;
		

	
	if (x25_lcn->x25_sw_lcn == NULL){

	       /* if we are configured as a DTE and we have already issued 
	 	* a Clear Request on this LCN, we consider the clearing 
	 	* procedures to be completed and we do not issue a 
	 	* Clear Confirmation packet (C3-F6) */
	
		if(x25_lcn->x25_link->cfg.dte && (x25_lcn->state == X25_ST_CLEAR_REQ_SNT_P6)){
			printk(KERN_INFO "%s: DTE: Initializing due to Rx clear on P6\n",
					x25_lcn->dev->name);

			x25_initialize_lcn(x25_lcn,X25_ST_PKT_LVL_RDY_P1,HARD_INIT);
		}else{
			//FIXME: CHECK IF WE SHOULD RESPOND TO THE CLEAR REQUEST 
			//       AUTOMATICALLY. IF NOT: STATE IS P7 : STATE IS P1
			/* X25_SW
			 * Send Clear Confirmation only for non switching LCN.
			 */
			x25_initialize_lcn(x25_lcn, X25_ST_PKT_LVL_RDY_P1,HARD_INIT);

			CALL_LOG(x25_lcn->x25_link, "%s: X25 Event: Tx Clear Conf    :Lcn=%i\n",
					x25_lcn->dev->name, frame->lcn);
			/* issue a Clear Confirmation packet */
			x25_send_control(x25_lcn->x25_link, 
					 gfi_decode(x25_lcn->config),
					 frame->lcn,
					 CLEAR_CONF_PKT,
					 0,0,NULL,0);
		}
	}
	
	/* reset the miscellaneous status byte for this LCN */
	set_LCN_misc_status_bits(x25_lcn, (unsigned char)~(NETWORK_DOWN | 
					  	           RNR_RX_ON_LCN | 
						           D_BIT_DATA_PKT_RX | 
						           D_BIT_DATA_PKT_TX));

	/* this LCN is not configured for Fast Select usage */
	x25_lcn->config = x25_lcn->config & ~FAST_SEL_WITH_RESTR_ON_RESP;
	
	DEBUG_RX("%s:---------- End of Rx Clear request--------------\n",
			x25_lcn->dev->name);
}


/*=======================================================================
 * handle_Rx_Reset_Request_pkt
 *
 * Description:  
 *
 * 	Handle incoming RESET requests. 
 *
 *	If the packet is error free, then initialize the LCN:
 *		1. Clear Pr,Ps, numbers, 
 *		2. Flush all data queues.
 *
 *  	NOTE: we leave the LCN connected, by NOT clearing the
 *  	      LCN number.
 *  	      
 * 	In case of a packet/any error, the reset request will
 * 	be sent.
 */ 

void handle_Rx_Reset_Request_pkt(x25_lcn_t *x25_lcn, struct sk_buff *skb, struct x25_frame *frame)
{
	int err;
	int network_down=0;
	
	printk(KERN_INFO "%s: X25 Event: Rx Reset Req :Lcn=%i:Cause=0x%02X:Diag=0x%02X\n",
			x25_lcn->dev->name,frame->lcn,frame->cause,frame->diag);

	x25_log_proc_event(x25_lcn->x25_link,x25_lcn->dev->name,"Rx Reset Req",
			   frame->lcn,frame->cause,frame->diag,
			   NULL);

	if ((err=packet_length_error(x25_lcn->x25_link, frame))){
		
		printk(KERN_INFO "%s: Error Rx Reset Req: Invalid Packet Length %i!\n",
				x25_lcn->dev->name, err);


		if (IS_PVC(x25_lcn)){
			lcn_state_change(x25_lcn,X25_ST_RESET_REQ_SNT_D2);
			x25_send_control(x25_lcn->x25_link,0,
				 frame->lcn,
				 RESET_REQUEST_PKT,0,
				 PKT_TOO_SHORT,NULL,0);
	
		}else{
			if (x25_lcn->state != X25_ST_CLEAR_REQ_SNT_P6){
				lcn_state_change(x25_lcn,X25_ST_CLEAR_REQ_SNT_P6);
				x25_send_control(x25_lcn->x25_link,0,frame->lcn,
						CLEAR_REQUEST_PKT, 
						0,err,NULL,0);
			}
		}
		return;		
	}

	if (!(IS_PVC(x25_lcn))){

		/* if we receive a Call Accept with the D-bit set and our 
		 * outgoing Call Request was not issued with the D-bit, then we 
		 * issue a Clear Request with a diagnostic code #166 */

		if ((frame->gfi & GFI_WITH_D_BIT_SET) && 
		    !(x25_lcn->config & D_BIT_PRAGMATICS_FOR_SVC)){

			if (x25_lcn->state != X25_ST_CLEAR_REQ_SNT_P6){
				lcn_state_change(x25_lcn,X25_ST_CLEAR_REQ_SNT_P6);
				x25_send_control(x25_lcn->x25_link,0,frame->lcn,
						CLEAR_REQUEST_PKT, 
						0,D_BIT_PROCEDURE_NOT_SUPPORTED,NULL,0);
			}

			return;								
		}

		
		/* for a Call Request or a Call Accept packet, set the D-bit 
		 * pragmatics flag for the LCN according to the received GFI */
		
		if ( (frame->gfi & GFI_WITH_D_BIT_SET) && 
		     !(x25_lcn->x25_link->cfg.X25_protocol_options & D_BIT_PRAGMATICS_NOT_SUPPORTED)){

			x25_lcn->config = x25_lcn->config | D_BIT_PRAGMATICS_FOR_SVC;
		}else{
			x25_lcn->config = x25_lcn->config & ~D_BIT_PRAGMATICS_FOR_SVC;
		}

	}
	
	x25_lcn->x25_api_event.hdr.qdm = (frame->gfi >> 1);
	if (frame->mbit){
		set_bit(0,(unsigned long*)&x25_lcn->x25_api_event.hdr.qdm);
	}else{
		clear_bit(0,(unsigned long*)&x25_lcn->x25_api_event.hdr.qdm);
	}
	x25_lcn->x25_api_event.hdr.cause = frame->cause;
	x25_lcn->x25_api_event.hdr.diagn = frame->diag;
	x25_lcn->x25_api_event.hdr.pktType = frame->type;
	x25_lcn->x25_api_event.hdr.length  = 0;
	x25_lcn->x25_api_event.hdr.result = 0;
	x25_lcn->x25_api_event.hdr.lcn = frame->lcn;

	
	/* X.25 switching */
	if (x25_lcn->x25_sw_lcn){
		x25_lcn_t *x25_sw_lcn = (x25_lcn_t*)x25_lcn->x25_sw_lcn;
		/* X25_SW
		 * This LCN is used for switching.
		 * Send Reset Request to remote switching device.
		 */
		printk(KERN_INFO "%s: Lcn=%i SWITCH, Tx Reset Request to %s: Lcn=%i\n",
				x25_lcn->dev->name,
				atomic_read(&x25_lcn->lcn),
				x25_sw_lcn->dev->name,
				atomic_read(&x25_sw_lcn->lcn));

		lcn_state_change(x25_sw_lcn,X25_ST_PKT_LVL_RDY_D1);
		x25_sw_send_control(x25_lcn->x25_sw_lcn, NULL, frame);	
	}else{
		if (x25_lcn->usedby == API){
			if (test_bit(LCN_SK_ID,&x25_lcn->used) && x25_lcn->sk_id){
				send_api_oob_msg(x25_lcn,frame,NULL,0);
			}	
		}
	}

	/* set the LCN state to record the reception of the 
	 * Reset Request packet */
	if (IS_PVC(x25_lcn)){
		if (test_bit(LCN_SK_ID,&x25_lcn->used) && x25_lcn->sk_id){
			x25_initialize_lcn(x25_lcn,X25_ST_RESET_REQ_REC_D3,SOFT_INIT);
		}else{
			/* Ignore reser request if the user application
			 * is still connected to us */
			return;
		}
	}else{
		x25_initialize_lcn(x25_lcn,X25_ST_RESET_REQ_REC_D3,SOFT_INIT);
	}
	
	/* reset the miscellaneous status byte for this LCN */
	set_LCN_misc_status_bits(x25_lcn, (UCHAR)~(NETWORK_DOWN | 
						   RNR_RX_ON_LCN | 
						   D_BIT_DATA_PKT_RX | 
						   D_BIT_DATA_PKT_TX));

	/* we mask out the high bit (public or private data network bit) 
	 * of the Resetting cause field and check for an 
	 * an out-of-order cause code */
	switch(frame->cause & MASK_PRIVATE_NW_CAUSE_BIT) {

		/* the remote node is out of order */
		case RESET_OUT_OF_ORDER:						
		
		/* the network is out of order */
		case RESET_NETWORK_OUT_OF_ORDER:		
			/* set our internal LCN status to indicate that the 
			 * LCN is not operational */
			set_LCN_misc_status_bits(x25_lcn,(UCHAR)NETWORK_DOWN);

			printk(KERN_INFO "%s: X25 Event: Network Down!  Cause=0x%X\n",
					x25_lcn->dev->name,frame->cause);

			x25_initialize_lcn(x25_lcn,X25_ST_PVC_OUT_OF_ORDER,SOFT_INIT);
			network_down=1;
			break;

		default:	
			/* ignore all other cause codes */
			break;
	}


	//FIXME No support for user response, it is always on Auto Answer !!!	
 	if (x25_lcn->state != X25_ST_RESET_REQ_SNT_D2){	

		printk(KERN_INFO "%s: X25 Event: Tx Reset Conf :Lcn=%i\n",
			x25_lcn->dev->name,frame->lcn);
		
		if (!network_down){
			/* If the state indicates a correctly received
			 * Reset packet, then we can update the state
			 * to ready, because we will automatically
			 * send reset confirmation.
			 *
			 * However, if the network is down, the
			 * lcn state will not be changed to ready
			 * by reset conf will still be sent out.*/
			lcn_state_change(x25_lcn,X25_ST_PKT_LVL_RDY_D1);
		}

		x25_send_control(x25_lcn->x25_link,0,
				 frame->lcn,
				 RESET_CONF_PKT,
				 0,0,NULL,0);
	}

	if (x25_lcn->dsp_id){
		if (IS_FUNC_CALL(x25_dsp_protocol, dsp_svc_reset)){
			x25_dsp_protocol.dsp_svc_reset(x25_lcn->dsp_id);	
		}
	}
	/* the Reset Request was handled */
	return;			
}

/*=======================================================================
 * x25_validate_Pr
 *
 * Description:  
 *
 * 	Validate the Pr value of the incoming DATA, RR or RNR packet.
 *  	
 *  	The Pr value indicates that the all packets up to the sequence
 *  	number Pr-1 have been acknowledged, and that sequence Pr is
 *  	expected next.
 *  	
 * 	If the Pr value checks out ok, then update the Ps ack and
 * 	return 0. The Ps_ack is the bottom window edge of the
 * 	outgoing data packets. 
 *
 * 	If the Pr value is invalid return -EINVAL.
 */ 

inline int x25_validate_Pr(x25_lcn_t *x25_lcn, struct x25_frame *frame)
{
	volatile unsigned short ps_ack = x25_lcn->Ps_ack;
	volatile unsigned char modulus;
	
	/* Determine the Window size, MOD8 = 8 or MOD128 = 128 */
	modulus = (x25_lcn->x25_link->cfg.mode & GFI_MOD_8_BIT_SET) ? 
			MOD8_MODULUS : MOD128_MODULUS;
	
	while (ps_ack != x25_lcn->Ps_count) {
		if (frame->Pr_count == ps_ack){ 
			x25_lcn->Ps_ack = ps_ack; 
			return X25_OK;
		}
		ps_ack = (ps_ack + 1) % modulus;
	}
	
	if (frame->Pr_count == x25_lcn->Ps_count){ 
		x25_lcn->Ps_ack = ps_ack;
		return X25_OK;
	}

	printk(KERN_INFO "%s: ***!!!*** Before Invalid Ps %i, Ps_ack %i, Pr %i\n",
			 x25_lcn->dev->name, x25_lcn->Ps_count,x25_lcn->Ps_ack,frame->Pr_count);
	
	return -EINVAL;
}

