#include <linux/wanpipe_includes.h>
#include <linux/wanpipe_cfg.h>
#include "wanpipe_x25.h"


/* X25's own task queue, used for all API's.
 * All protocol specific tasks will be instered
 * into "x25_tq_custom" task_queue. 

 * On each rx_interrupt, the whole task queue
 * (x25_tq_custom) will be queued into 
 * IMMEDIATE_BH via x25_mark_bh() call. 
 
 * The IMMEDIATE_BH will execute run_x25_tq() 
 * function, which will execute all pending,
 * tasks in x25_tq_custom queue */

#if 0
static void run_x25_tq (unsigned long data)
{
	task_queue *tq_queue = (task_queue *)data;
	run_task_queue (tq_queue);
}

DECLARE_TASK_QUEUE(x25_tq_custom);
static struct tq_struct x25_tq_task = 
{
	routine: (void (*)(void *)) run_x25_tq,
	data: &x25_tq_custom
};


__inline__ int x25_queue_tq (struct tq_struct *bh_pointer)
{
	if (!queue_task(bh_pointer,&x25_tq_custom)){
		return 1;
	}
	return 0;
}

__inline__ int x25_mark_bh (void)
{
	if (!queue_task(&x25_tq_task,&tq_immediate)){
		return 1;
	}
	mark_bh(IMMEDIATE_BH);
	return 0;
} 
#endif
/*============================================================
 * RESTART TIMER
 * ==========================================================*/

static void x25_t10_t20_timer_expiry(unsigned long x25_id)
{
	x25_link_t *x25_link = (x25_link_t*)x25_id;
	unsigned long flags;

	printk(KERN_INFO "%s: X25 T10-T20 Restart Timer Expired Cnt %i\n",
			x25_link->name,x25_link->T10_T20_timer_cnt);

	if (test_bit(LINK_DOWN,&x25_link->tq_working))
		return;

	
	x25_link->state = X25_ST_RESTART_REQ_SNT_R2;
	
	if (++x25_link->T10_T20_timer_cnt > x25_link->cfg.R10_R20){
		printk(KERN_INFO "%s: X25 T10-T20 Exceeded the %i Restart Re-tries\n",
				x25_link->name,x25_link->cfg.R10_R20);
		return;
	}

	spin_lock_irqsave(&x25_link->bh_lock,flags);
	printk(KERN_INFO "%s: T10-T20 Timer: Tx Restart Request\n",
			x25_link->name);
	x25_link->state = X25_ST_RESTART_REQ_SNT_R2;
	x25_set_lcn_states(x25_link,X25_ST_RESTART_REQ_SNT_R2);
		
	x25_send_control(x25_link,0,0,RESTART_REQUEST_PKT,0,TIMER_EXPIRED_RESTART_REQ,NULL,0);
	spin_unlock_irqrestore(&x25_link->bh_lock,flags);
}

void x25_start_t10_t20_timer(x25_link_t *x25_link)
{
	WAN_ASSERT1((!x25_link));
	
	if (test_bit(LINK_DOWN,&x25_link->tq_working))
		return;
	
	x25_link->T10_T20_timer.data     = (unsigned long)x25_link;
	x25_link->T10_T20_timer.function = &x25_t10_t20_timer_expiry;
	x25_link->T10_T20_timer.expires  = jiffies + x25_link->cfg.T10_T20;

	del_timer(&x25_link->T10_T20_timer);
	add_timer(&x25_link->T10_T20_timer);
}

void x25_stop_t10_t20_timer(x25_link_t *x25_link)
{
	WAN_ASSERT1((!x25_link));

	del_timer(&x25_link->T10_T20_timer);
	x25_link->T10_T20_timer_cnt = 0;
}

/*==============================================================
 * REGISTRATION Timer 
 * ============================================================*/

static void x25_t28_timer_expiry(unsigned long x25_id)
{
	//x25_link_t *x25_link = (x25_link_t*)x25_id;

	printk(KERN_INFO "X25 T28 Registration Timer Expired\n");
#if 0
	lcn_state_change(x25_link,X25_ST_RESTART_REQ_SNT_R2);

	if (++x25_link->T28_timer_cnt > x25_link->R28){
		printk(KERN_INFO "X25 T28 Exceeded the R28 Restart Re-tries\n");
		return;
	}

	x25_initialize_all_lcn(x25_link, X25_ST_RESTART_REQ_SNT_R2);
	x25_send_control(x25_link,0,0,RESTART_REQUEST_PKT,0x0,0,NULL,0);
#endif
}



void x25_start_t28_timer(x25_link_t *x25_link)
{
	WAN_ASSERT1((!x25_link));
	
	if (test_bit(LINK_DOWN,&x25_link->tq_working))
		return;

	x25_link->T28_timer.data     = (unsigned long)x25_link;
	x25_link->T28_timer.function = &x25_t28_timer_expiry;
	x25_link->T28_timer.expires  = jiffies + x25_link->cfg.T28;

	del_timer(&x25_link->T28_timer);
	add_timer(&x25_link->T28_timer);
}

void x25_stop_t28_timer(x25_link_t *x25_link)
{
	WAN_ASSERT1((!x25_link));

	del_timer(&x25_link->T28_timer);
	x25_link->T28_timer_cnt = 0;
}

/*==============================================================
 * CALL REQUEST TIMER
 *=============================================================*/


static void x25_t11_t21_timer_expiry(unsigned long x25_id)
{
	x25_lcn_t *x25_lcn = (x25_lcn_t*)x25_id;
	int lcn = atomic_read(&x25_lcn->lcn);

	if (test_bit(LINK_DOWN,&x25_lcn->x25_link->tq_working))
		return;

	if (test_bit(LCN_DOWN,&x25_lcn->critical))
		return;

	printk(KERN_INFO "%s: X25 T11_T21 Call Request Timer Expired Lcn %i\n",
			x25_lcn->dev->name,lcn);

	lcn_state_change(x25_lcn,X25_ST_CLEAR_REQ_SNT_P6);
	x25_send_control(x25_lcn->x25_link,0,lcn,CLEAR_REQUEST_PKT,0x0,TIMER_EXPIRED_CALL_REQ,NULL,0);
}

void x25_start_t11_t21_timer(x25_lcn_t *x25_lcn)
{
	WAN_ASSERT1((!x25_lcn));

	if (test_bit(LINK_DOWN,&x25_lcn->x25_link->tq_working))
		return;

	if (test_bit(LCN_DOWN,&x25_lcn->critical))
		return;
	
	x25_lcn->T11_T21_timer.data     = (unsigned long)x25_lcn;
	x25_lcn->T11_T21_timer.function = &x25_t11_t21_timer_expiry;
	x25_lcn->T11_T21_timer.expires  = jiffies + x25_lcn->x25_link->cfg.T11_T21;

	del_timer(&x25_lcn->T11_T21_timer);
	add_timer(&x25_lcn->T11_T21_timer);
}

void x25_stop_t11_t21_timer(x25_lcn_t *x25_lcn)
{
	WAN_ASSERT1((!x25_lcn));

	del_timer(&x25_lcn->T11_T21_timer);
	x25_lcn->T11_T21_timer_cnt = 0;
}

/*====================================================================
 * RESET REQUEST TIMER
 *===================================================================*/


static void x25_t12_t22_timer_expiry(unsigned long x25_id)
{
	x25_lcn_t *x25_lcn = (x25_lcn_t*)x25_id;
	int lcn = atomic_read(&x25_lcn->lcn);
	

	printk(KERN_INFO "%s: X25 T12_T22 Reset Request Timer Expired\n",
			x25_lcn->dev->name);

	if (test_bit(LINK_DOWN,&x25_lcn->x25_link->tq_working))
		return;
	
	if (test_bit(LCN_DOWN,&x25_lcn->critical))
		return;

	
	if (++x25_lcn->T12_T22_timer_cnt > x25_lcn->R12_R22){
		printk(KERN_INFO "%s: X25 T12-T22 Exceeded the %i Reset Re-tries\n",
				x25_lcn->dev->name, x25_lcn->R12_R22);
		return;
	}
	
	
	lcn_state_change(x25_lcn,X25_ST_RESET_REQ_SNT_D2);
	x25_send_control(x25_lcn->x25_link,0,lcn,RESET_REQUEST_PKT,0x0,TIMER_EXPIRED_RESET_REQ,NULL,0);

}

void x25_start_t12_t22_timer(x25_lcn_t *x25_lcn)
{
	WAN_ASSERT1((!x25_lcn));

	if (test_bit(LINK_DOWN,&x25_lcn->x25_link->tq_working))
		return;

	if (test_bit(LCN_DOWN,&x25_lcn->critical))
		return;
	
	x25_lcn->T12_T22_timer.data     = (unsigned long)x25_lcn;
	x25_lcn->T12_T22_timer.function = &x25_t12_t22_timer_expiry;
	x25_lcn->T12_T22_timer.expires  = jiffies + x25_lcn->x25_link->cfg.T12_T22;

	del_timer(&x25_lcn->T12_T22_timer);
	add_timer(&x25_lcn->T12_T22_timer);
}

void x25_stop_t12_t22_timer(x25_lcn_t *x25_lcn)
{
	WAN_ASSERT1((!x25_lcn));

	del_timer(&x25_lcn->T12_T22_timer);
	x25_lcn->T12_T22_timer_cnt = 0;
}

/*======================================================================
 * CLEAR REQUEST TIMER
 *====================================================================*/

 
static void x25_t13_t23_timer_expiry(unsigned long x25_id)
{
	x25_lcn_t *x25_lcn = (x25_lcn_t*)x25_id;
	int lcn = atomic_read(&x25_lcn->lcn);	
	
	if (test_bit(LINK_DOWN,&x25_lcn->x25_link->tq_working))
		return;
	
	if (test_bit(LCN_DOWN,&x25_lcn->critical))
		return;

	printk(KERN_INFO "%s: X25 T13_T23 Clear Request Timer Expired\n",
			x25_lcn->dev->name);

	if (++x25_lcn->T13_T23_timer_cnt > x25_lcn->R13_R23){
		printk(KERN_INFO "%s: X25 T13-T23 Exceeded the %i Clear Re-tries\n",
				x25_lcn->dev->name,x25_lcn->R13_R23);
		return;
	}

	lcn_state_change(x25_lcn,X25_ST_CLEAR_REQ_SNT_P6);
	x25_send_control(x25_lcn->x25_link,0,lcn,CLEAR_REQUEST_PKT,0x0,TIMER_EXPIRED_CLEAR_REQ,NULL,0);

	x25_start_t13_t23_timer(x25_lcn);
}

void x25_start_t13_t23_timer(x25_lcn_t *x25_lcn)
{
	WAN_ASSERT1((!x25_lcn));

	if (test_bit(LINK_DOWN,&x25_lcn->x25_link->tq_working))
		return;
		
	if (test_bit(LCN_DOWN,&x25_lcn->critical))
		return;
	
	x25_lcn->T13_T23_timer.data     = (unsigned long)x25_lcn;
	x25_lcn->T13_T23_timer.function = &x25_t13_t23_timer_expiry;
	x25_lcn->T13_T23_timer.expires  = jiffies + x25_lcn->x25_link->cfg.T13_T23;

	del_timer(&x25_lcn->T13_T23_timer);
	add_timer(&x25_lcn->T13_T23_timer);
}

void x25_stop_t13_t23_timer(x25_lcn_t *x25_lcn)
{
	del_timer(&x25_lcn->T13_T23_timer);
	x25_lcn->T13_T23_timer_cnt = 0;
}

