#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <sys/time.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <string.h>
#include <errno.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <linux/if.h>
#include <linux/if_packet.h>
#include "../sdla_t3_hdlc.h"

struct sdla_hdlc_api api_cmd;
int sock;
unsigned char i_name[20];

struct cmd_list_struct cmd_list[]={
	{"READ",SDLA_HDLC_READ_REG},
	{"WRITE",SDLA_HDLC_WRITE_REG},
	{"GET_TXDMA",SDLA_HDLC_GET_TXDMA},
	{"GET_RXDMA",SDLA_HDLC_GET_RXDMA},
	{"FREE_TXDMA",SDLA_HDLC_FREE_TXDMA},
	{"FREE_RXDMA",SDLA_HDLC_FREE_RXDMA},
	{"SET_TXDMA",SDLA_HDLC_SET_TXDMA},
	{"READ_RXDMA",SDLA_HDLC_READ_RXDMA},
	{".",0xFF}
};

int MakeConnection(void) 
{
	struct sockaddr_ll sa;
	struct ifreq ifr;
	
	memset(&sa,0,sizeof(struct sockaddr_ll));
	errno = 0;
   	sock = socket(AF_PACKET, SOCK_DGRAM, 0);
   	if( sock < 0 ) {
      		perror("Socket");
      		return -EIO;
   	} /* if */
  
	strcpy(ifr.ifr_name,i_name);
	if (ioctl(sock,SIOCGIFINDEX,&ifr)){
		perror("Get index: ");
		return -EIO;
	}

	sa.sll_protocol = htons(0x16);
	sa.sll_family=AF_PACKET;
	sa.sll_ifindex = ifr.ifr_ifindex;

        if(bind(sock, (struct sockaddr *)&sa, sizeof(struct sockaddr_ll)) < 0){
                perror("bind");
		printf("Failed to bind a socket to %s interface\n",i_name);
                exit(0);
        }
	printf("Socket bound to %s\n\n",i_name);

	return 0;

}

int arg_error_check(int i, int argc)
{
	if (i+1 > argc-1){
		printf("Argument syntax error!\n");
		printf("Type sdla_t3_hdlc_api <cr> for help\n");
		return -EINVAL;
	}
	return 0;
}

void usage (void)
{
	printf("
Usage:
	args: -i <ifname> 
	      -cmd [READ|WRITE|RXDMA|TXDMA] 
	      -bar [0|1] 
	      -off <number>
	      -l [1|2|4|X] 
	      -d <number>

	./sdla_t3_hdlc_api -i whdlc0 -cmd READ -bar 0 -off 10 -l 1
	./sdla_t3_hdlc_api -i whdlc0 -cmd WRITE -bar 0 -off 10 -l 4 -d 0x80000000
\n");
}

int main(int argc, char* argv[])
{
	int i,j=0;
	int err;
	struct ifreq ifr;


	if (argc == 1){
		usage();
		return 0;
	}
	
	memset(&api_cmd,0,sizeof(struct sdla_hdlc_api));
	api_cmd.cmd=0xFF;

	for (i = 0; i < argc; i++){

		if (!strcmp(argv[i],"-h")){
		
			usage();
			return 0;
					
		}else if (!strcmp(argv[i],"-bar")){
			
			if (arg_error_check(i,argc)){
				return -EINVAL;
			}	
			
			api_cmd.bar=atoi(argv[i+1]);
			
		}else if (!strcmp(argv[i],"-c")){
			
			if (arg_error_check(i,argc)){
				return -EINVAL;
			}	

			api_cmd.cmd=0xFF;
			
			for (j=0;;j++){
				if (!strcmp(cmd_list[j].cmd_str,".")){
					break;
				}
	
				if (!strcmp(cmd_list[j].cmd_str,argv[i+1])){
					api_cmd.cmd=cmd_list[j].cmd;
					break;
				}
			}

			if (api_cmd.cmd==0xFF){
				printf("Error: invalid cmd %s\n",argv[i+1]);
				return -EINVAL;
			}
		
		}else if (!strcmp(argv[i],"-l")){
			
			if (arg_error_check(i,argc)){
				return -EINVAL;
			}	
			
			api_cmd.len=atoi(argv[i+1]);
		
		}else if (!strcmp(argv[i],"-d")){
			
			if (arg_error_check(i,argc)){
				return -EINVAL;
			}
		
			if (api_cmd.len == 0){
				printf("\nError Invalid data len = 0\n");
				printf("Run -h option for usage\n");
				return -EINVAL;
			}
			
			if (api_cmd.len == 1){
				*(unsigned char*)api_cmd.data = (unsigned char)strtoul(argv[i+1], NULL, 0);
			}else if (api_cmd.len == 2){
				*(unsigned short*)api_cmd.data = (unsigned short)strtoul(argv[i+1], NULL, 0);
			}else if (api_cmd.len == 4){
				*(unsigned int*)api_cmd.data = (unsigned int)strtoul(argv[i+1], NULL, 0);
			}else{
				strcpy(api_cmd.data,argv[i+1]);
			}

			printf("Data = 0x%x len = %i Str=%s\n",
					*(unsigned int*)api_cmd.data,api_cmd.len,argv[i+1]);
			
		}else if (!strcmp(argv[i],"-i")){

			if (arg_error_check(i,argc)){
				return -EINVAL;
			}
			
			strcpy(i_name,argv[i+1]);
			
		}else if (!strcmp(argv[i],"-off")){
		
			api_cmd.offset=atoi(argv[i+1]);
		}
	}

	if (api_cmd.cmd==0xFF){
		printf("Error: invalid cmd\n");
		return -EINVAL;
	}
		

	if ((err=MakeConnection()) != 0){
		return err;
	}

	strcpy(ifr.ifr_name,i_name);
	ifr.ifr_data = (char*)&api_cmd;

	printf("Executing cmd %s Len %i Bar %i Off %i Data 0x%x\n",
			cmd_list[j].cmd_str,
			api_cmd.len,
			api_cmd.bar,
			api_cmd.offset,
			*(unsigned int*)api_cmd.data);

	err = ioctl(sock,api_cmd.cmd,&ifr);
	if (err){
		perror("Cmd Exec: ");
		return err;
	}

	printf("Cmd Ok\n");
	if (api_cmd.len == 1){
		printf("Result Data u8  = 0x%02x\n",*(unsigned char*)api_cmd.data);
	}else if (api_cmd.len == 2){
		printf("Result Data u16 = 0x%04x\n",*(unsigned short*)api_cmd.data);
	}else if (api_cmd.len == 4){
		printf("Result Data u32 = 0x%08x\n",*(unsigned int*)api_cmd.data);
	}else{
		int i;
		printf("Result Data Len=%i : ",api_cmd.len);
		for (i=0;i<api_cmd.len;i++){
			printf("0x%x ",api_cmd.data[i]);
		}
		printf("\n");
	}

		
	close (sock);
	return err;
};

