#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <sys/time.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <string.h>
#include <errno.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <linux/if.h>
#include <linux/if_packet.h>
#include "../sdla_t3_hdlc.h"
#include "wanpipe_cfg.h"
#include "sdla_te1_def.h"

#define JP8_VALUE               0x02
#define JP7_VALUE               0x01
#define SW0_VALUE               0x04
#define SW1_VALUE               0x08   

#define WRITE_DEF_SECTOR_DSBL   0x01
#define FRONT_END_TYPE_MASK     0x38

#define BIT_DEV_ADDR_CLEAR	0x600
#define BIT_DEV_ADDR_CPLD	0x200

#define MEMORY_TYPE_SRAM	0x00
#define MEMORY_TYPE_FLASH	0x01
#define MASK_MEMORY_TYPE_SRAM	0x10
#define MASK_MEMORY_TYPE_FLASH	0x20

#define DEF_SECTOR_FLASH	0x00
#define USER_SECTOR_FLASH	0x01
#define MASK_DEF_SECTOR_FLASH	0x00
#define MASK_USER_SECTOR_FLASH	0x04

#define BIT_A18_SECTOR_SA4_SA7	0x20
#define USER_SECTOR_START_ADDR	0x40000
/*
 ******************************************************************************
			  FUNCTION PROTOTYPES
 ******************************************************************************
*/
		
extern int exec_read_cmd(unsigned int off, unsigned int len, unsigned int *data);
extern int exec_write_cmd(unsigned int off, unsigned int len, unsigned int data);
extern void hit_any_key(void);

unsigned int write_cpld(unsigned short cpld_off,unsigned short cpld_data)
{
	cpld_off &= ~BIT_DEV_ADDR_CLEAR; 
	cpld_off |= BIT_DEV_ADDR_CPLD; 
	exec_write_cmd(0x46, 2, cpld_off);
	exec_write_cmd(0x44, 2, cpld_data);
	return 0;
}

unsigned int read_cpld(unsigned short cpld_off)
{
	unsigned int cpld_data;
	cpld_off &= ~BIT_DEV_ADDR_CLEAR; 
	cpld_off |= BIT_DEV_ADDR_CPLD; 
	exec_write_cmd(0x46, 2, cpld_off);
	if (exec_read_cmd(0x44, 4, &cpld_data) == 0){
		return cpld_data;
	}else{
		return 0;
	}
}

unsigned int write_flash(int sector_type, int memory_type, unsigned long flash_off,unsigned char flash_data)
{
	unsigned char	offset;

	//Writing flash address to cpld
	offset = flash_off & 0xFF;
	write_cpld(0x05, offset);
	offset = (flash_off >> 8) & 0xFF;
	write_cpld(0x06, offset);
	offset = (flash_off >> 16) & 0x3;
	if (memory_type == MEMORY_TYPE_SRAM){
		offset |= MASK_MEMORY_TYPE_SRAM;
	}else if (memory_type == MEMORY_TYPE_FLASH){
		offset |= MASK_MEMORY_TYPE_FLASH;
	}else{
		return -EINVAL;
	}
	if (sector_type == USER_SECTOR_FLASH){
		offset |= MASK_USER_SECTOR_FLASH;
	}else if (sector_type == DEF_SECTOR_FLASH){
		;
	}else{
		return -EINVAL;
	}
	write_cpld(0x07, offset);
	write_cpld(0x04, flash_data);
        write_cpld(0x07, 0x00);  // disable CS signal for the Boot FLASH/SRAM

	return 0;
}

unsigned char read_flash(int sector_type, int memory_type, unsigned long flash_off)
{
	unsigned char offset;
        unsigned char data;

	//Writing flash address to cpld
	offset = flash_off & 0xFF;
	write_cpld(0x05, offset);
	offset = (flash_off >> 8) & 0xFF;
	write_cpld(0x06, offset);
	offset = (flash_off >> 16) & 0x3;
        offset |= MASK_MEMORY_TYPE_FLASH;
//           
//	if (memory_type == MEMORY_TYPE_SRAM){
//		offset |= MASK_MEMORY_TYPE_SRAM;
//	}else if (memory_type == MEMORY_TYPE_FLASH){
//		offset |= MASK_MEMORY_TYPE_FLASH;
//	}else{
//		return -EINVAL;
//	}
	if (sector_type == USER_SECTOR_FLASH){
		offset |= MASK_USER_SECTOR_FLASH;
	}else if (sector_type == DEF_SECTOR_FLASH){
		;
	}else{
		return -EINVAL;
	}

	write_cpld(0x07, offset);
        data = read_cpld(0x04);
        write_cpld(0x07, 0x00); // Disable CS for the Boot FLASH/SRAM
	return data;
}

int cpld_set_clock(unsigned char media)
{
	int i = 0;
	write_cpld(0x00, 0x00);
	for(i=0;i<400;i++);
	switch(media){
	case WANOPT_MEDIA_T1:
		write_cpld(0x00, 0x03);
		break;

	case WANOPT_MEDIA_E1:
		write_cpld(0x00, 0x01);
		break;
	}
	for(i=0;i<400;i++);
	return 0;
}

static int led_test(void)
{
	int i = 0;
	unsigned int j = 0, jj = 0;

	// Configuration LED
	write_cpld(0x01, 0xc);

	for(i=0;i<20;i++){
		// Turn ON RED
		write_cpld(0x01, 0xd);
		for(j=0;j<100000;j++)	
			for(jj=0;jj<100;jj++);	

		// Turn ON GREEN
		write_cpld(0x01, 0xe);
		for(j=0;j<100000;j++)
			for(jj=0;jj<100;jj++);	
	}	
	// Turn OFF RED/GREEN
	write_cpld(0x01, 0xf);

	return 0;
}

static int erase_sector_flash(int sector_type)
{
	unsigned long offset = 0x00;
	unsigned char data = 0x00;
	int sector_no = 0;
	int cnt=0;
	unsigned char	val;

	// Checking write enable to the Default Boot Flash Sector 
        if(sector_type == DEF_SECTOR_FLASH){
		val = read_cpld(0x09);
		if (val & WRITE_DEF_SECTOR_DSBL){
			printf("erase_sector: Default sector protected!\n");
			return -EINVAL;
		}
	}    

	if (sector_type == USER_SECTOR_FLASH){
                offset  = USER_SECTOR_START_ADDR;
          	}
	for(sector_no = 0; sector_no < 4; sector_no++){
                offset += (sector_no << 16); 
		write_flash(sector_type, MEMORY_TYPE_FLASH, 0x5555, 0xAA);
		write_flash(sector_type, MEMORY_TYPE_FLASH, 0x2AAA, 0x55);
		write_flash(sector_type, MEMORY_TYPE_FLASH, 0x5555, 0x80);
		write_flash(sector_type, MEMORY_TYPE_FLASH, 0x5555, 0xAA);
		write_flash(sector_type, MEMORY_TYPE_FLASH, 0x2AAA, 0x55);
		write_flash(sector_type, MEMORY_TYPE_FLASH, offset, 0x30);
		do{
//MF			for(i=0;i<100000;i++);
			data = read_flash(sector_type, MEMORY_TYPE_FLASH,
                                                  offset);
			if (data & 0x80){
				break;
			}else if (data & 0x20){
				data = read_flash(sector_type, 
						  MEMORY_TYPE_FLASH, 
						  offset);
				if (data & 0x80){
					break;
				}else{
					printf("erase_sector_flash: Failed!\n");
					printf("erase_sector_flash: Sector=%d!\n",
					   (sector_type == USER_SECTOR_FLASH) ? 
							sector_no+4:sector_no);
					return -EINVAL;
				}
			}
		} while(1);

		printf("erase_sector_flash: Sector %d PASSED!\n", 
			(sector_type == USER_SECTOR_FLASH) ? 
					sector_no+4:sector_no);
	}

	// Verify that flash is 0xFF
	offset = 0x00;
	if (sector_type == USER_SECTOR_FLASH){
		offset = USER_SECTOR_START_ADDR;
	}
//        sector_type = DEF_SECTOR_FLASH;    // M.F debug
//        offset = 0;  

	printf("erase_sector_flash: Verification ");
	fflush(stdout);
	for(cnt = 0; cnt < 0x40000; cnt++){
//MF		for(i=0;i<10000;i++);
		data = read_flash(sector_type, MEMORY_TYPE_FLASH, offset+cnt);	
		if (data != 0xFF){
			printf(" Failed to compare! %05lx -> %02x \n",offset+cnt,data);
			return -EINVAL;
		}
		if ((cnt & 0x1FFF) == 0x1000){
			printf(".");
			fflush(stdout);
		}
	}
	printf("PASSED.\n");
	return 0;
}

static int prg_flash_byte(int sector_type, unsigned long off32, unsigned char data)
{
	unsigned char data1 = 0x00;
	unsigned char	val;

	// Checking write enable to the Default Boot Flash Sector 
        if(sector_type == DEF_SECTOR_FLASH){
		val = read_cpld(0x09);
		if (val & WRITE_DEF_SECTOR_DSBL){
			printf("prg_flash_byte: Default sector protected!\n");
			return -EINVAL;
			}
		}    
	write_flash(sector_type, MEMORY_TYPE_FLASH, 0x5555, 0xAA);
	write_flash(sector_type, MEMORY_TYPE_FLASH, 0x2AAA, 0x55);
	write_flash(sector_type, MEMORY_TYPE_FLASH, 0x5555, 0xA0);
	write_flash(sector_type, MEMORY_TYPE_FLASH, off32, data);
	do{
//MF		for(i=0;i<1000;i++);
		data1 = read_flash(sector_type, MEMORY_TYPE_FLASH, off32);
		if ((data1 & 0x80) == (data & 0x80)){
			break;
		}else if (data1 & 0x20){
			data1 = read_flash(sector_type, 
					  MEMORY_TYPE_FLASH, 
					  off32);
			if ((data1 & 0x80) == (data & 0x80)){
				break;
			}else{
				printf("prg_flash_byte: Failed!\n");
				return -EINVAL;
			}
		}
	} while(1);
	return 0;
}

static int reset_flash(void)
{
	write_flash(DEF_SECTOR_FLASH, MEMORY_TYPE_FLASH, 0x00, 0xF0);
	return 0;
}

static unsigned long filesize(FILE* f)
{
	unsigned long size = 0;
	unsigned long cur_pos = 0;

	cur_pos = ftell(f);
	if ((cur_pos != -1l) && !fseek(f, 0, SEEK_END)){
		size = ftell(f);
		fseek(f, cur_pos, SEEK_SET);
	}
	return size;
}

static long read_data_file(char* filename, char** data)
{
	FILE* f = NULL;
	char* buf = NULL;
	unsigned long fsize = 0;

	f = fopen(filename, "rb");
	if (f == NULL){
		printf("read_data_file: Can't open data file %s!\n",
					filename);
		return -EINVAL;
	}
	fsize = filesize(f);
	if (!fsize){
		printf("read_data_file: Data file %s is empty!\n",
					filename);
		return -EINVAL;
	}
	buf = malloc(fsize);
	if (buf == NULL){
		printf("read_data_file: Can't allocate memory for data!\n");
		return -ENOMEM;
	}
	if (fread(buf, 1, fsize, f) < fsize){
		printf("read_data_file: Can't copy all data to local buffer!\n");
		free(buf);
		return -EINVAL;
	}
	*data = buf;
	fclose(f);
	return fsize;
	
}

static int update_man(int memory_type, char* filename)
{
	char* data = NULL;
	unsigned long offset = 0, findex = 0;
	long fsize = 0;
	char val = 0x00;
        int sector_type =   DEF_SECTOR_FLASH; 

        
	// Checking write enable to the Default Boot Flash Sector 
        
	fsize = read_data_file(filename, &data);
	if (fsize < 0){
		return -EINVAL;
	}
	if (fsize > 0x80000){
		printf("update: Data file is too big!\n");
		free(data);
		return -EINVAL;
	}

        sector_type = DEF_SECTOR_FLASH;
	erase_sector_flash(sector_type);

	sector_type = USER_SECTOR_FLASH;
	erase_sector_flash(sector_type);


        sector_type = DEF_SECTOR_FLASH;
	offset = 0;
	printf("update: Programming      BOOT    ");
	fflush(stdout);
	while(findex < fsize/2){
		prg_flash_byte(sector_type, offset+findex, data[findex]);
		findex++;
		if ((findex & 0x1FFF) == 0x1000){
			printf(".");
			fflush(stdout);
		}
	}
	printf("PASSED.\n");

	sector_type = USER_SECTOR_FLASH;
	offset = USER_SECTOR_START_ADDR;

	printf("update: Programming      USER    ");
	while(findex < fsize){
		prg_flash_byte(sector_type, offset+findex, data[findex]);
		findex++;
		if ((findex & 0x1FFF) == 0x1000){
			printf(".");
			fflush(stdout);
		}
	}
	printf("PASSED.\n");
	reset_flash();

        sector_type = DEF_SECTOR_FLASH;
	offset = 0;
	findex = 0;
	printf("update: Verification     BOOT    ");
	fflush(stdout);
	while(findex < fsize/2){
		val = read_flash(sector_type, MEMORY_TYPE_FLASH, offset+findex);
		if (val != data[findex]){
			printf("update: Failed to compare data (%lx)!\n",
					findex);
			free(data);
			return -EINVAL;
		}
		if ((findex & 0x1FFF) == 0x1000){
			printf(".");
			fflush(stdout);
		}
		findex++;
	}
	printf("PASSED.\n");

	sector_type = USER_SECTOR_FLASH;
	offset = USER_SECTOR_START_ADDR;
	printf("update: Verification     USER    ");

	while(findex < fsize){
		val = read_flash(sector_type, MEMORY_TYPE_FLASH, offset+findex);
		if (val != data[findex]){
			printf("update: Failed to compare data (%lx)!\n",
					findex);
			free(data);
			return -EINVAL;
		}
		if ((findex & 0x1FFF) == 0x1000){
			printf(".");
			fflush(stdout);
		}
		findex++;
	}
	printf("PASSED.\n");
	fflush(stdout);
	free(data);
	return 0;
}

static int update(int sector_type, int memory_type, char* filename)
{
	char* data = NULL;
	unsigned long offset = 0, findex = 0;
	long fsize = 0;
	char val = 0x00;


	// Checking write enable to the Default Boot Flash Sector 
        if(sector_type == DEF_SECTOR_FLASH){
		val = read_cpld(0x09);
		if (val & WRITE_DEF_SECTOR_DSBL){
			printf("update: Default sector protected!\n");
			return -EINVAL;
			}
		}    
	fsize = read_data_file(filename, &data);
	if (fsize < 0){
		return -EINVAL;
	}
	if (fsize > 0x40000){
		printf("update: Data file is too big!\n");
		free(data);
		return -EINVAL;
	}

	if (sector_type == USER_SECTOR_FLASH){
		offset = USER_SECTOR_START_ADDR;
	}
	erase_sector_flash(sector_type);
	printf("update: Programming                 ");
	fflush(stdout);
	while(findex < fsize){
		prg_flash_byte(sector_type, offset+findex, data[findex]);
		findex++;
		if ((findex & 0x1FFF) == 0x1000){
			printf(".");
			fflush(stdout);
		}
	}
	printf("PASSED.\n");
	reset_flash();
	findex = 0;
	printf("update: Verification                ");
	fflush(stdout);
	while(findex < fsize){
		val = read_flash(sector_type, MEMORY_TYPE_FLASH, offset+findex);
		if (val != data[findex]){
			printf("update: Failed to compare data (%lx)!\n",
					findex);
			free(data);
			return -EINVAL;
		}
		if ((findex & 0x1FFF) == 0x1000){
			printf(".");
			fflush(stdout);
		}
		findex++;
	}
	printf("PASSED.\n");
	fflush(stdout);

	free(data);
	return 0;
}

static int verify(int sector_type, int memory_type, char* filename)
{
	char* data = NULL;
	unsigned long offset = 0, findex = 0;
	long fsize = 0;
	char val = 0x00;

	fsize = read_data_file(filename, &data);
	if (fsize < 0){
		return -EINVAL;
	}
	if (fsize > 0x40000){
		printf("update: Data file is too big!\n");
		free(data);
		return -EINVAL;
	}

	if (sector_type == USER_SECTOR_FLASH){
		offset = USER_SECTOR_START_ADDR;
	}
	findex = 0;
	printf("update: Verification                ");
	fflush(stdout);
	while(findex < fsize){
		val = read_flash(sector_type, MEMORY_TYPE_FLASH, offset+findex);
		if (val != data[findex]){
			printf("update: Failed to compare data (%lx)!\n",
					findex);
			free(data);
			return -EINVAL;
		}
		if ((findex & 0x1FFF) == 0x1000){
			printf(".");
			fflush(stdout);
		}
		findex++;
	}
	printf("PASSED.\n");
	fflush(stdout);

	free(data);
	return 0;
}

static unsigned char cpld_config[]="\n"
"	Xilinx: CPLD Configuration\n"
"	------------------------------\n"
"\n"
" 1. Read CPLD Access.\n"
" 2. Write CPLD Access.\n"
" 3. LED Test.\n"
" 4. Read Front End Type.\n"
" 5. Boot Flash read.\n"
" 6. Boot Flash write.\n"
" 7. Check Flash ID.\n"
" 8. Update Flash code (Default sector)\n"
" 9. Update Flash code (User sector)\n"
"10. Reset Flash.\n"
"11. Default Flash sector Erase !\n"
"12. User Flash sector Erase !\n"
"13. Chip Erase !\n"
"14. Program Flash (byte).\n"
"15. Default sector verification.\n"
"16. Read Board Status.\n"
"17. Reboot Xilinx from the Flash Default sector.\n"
"18. Reboot Xilinx from the Flash User sector.\n"
"19. Program Flash Full.\n"
" q. Quit.\n"
"\n"
"Please Select (Opt|q): ";



int cpld_initialization (int sock)
{
	int err;
	int val;
	int off;
	int data;
	int memory_type = MEMORY_TYPE_FLASH, sector_type = USER_SECTOR_FLASH;
	unsigned long off32;
	char filename[15];

	for (;;){
	
		val=-1;
		
		system("clear");
	
	printf("%s",cpld_config);

		scanf("%d",&val);
		if (val==-1){
			getchar();
		}
		switch (val){

		case 1:

try_read_cpld_again:
			system("clear");
			printf("\n");
	
			printf("CPLD READ: \n");

			off=-1;
			printf("Specify Offset (hex): ");
			scanf("%x",&off);
			if (off==-1){
				getchar();
				goto try_read_cpld_again;
			}

			err=read_cpld(off);
			printf("CPLD Read Offset=0x%X Data=0x%X\n",	
					off,err);
			hit_any_key();
			break;

		case 2:

try_write_cpld_again:
                        system("clear");
                        printf("\n");

                        printf("CPLD WRITE: \n");

                        off=-1;
                        printf("Specify Offset (hex): ");
                        scanf("%x",&off);
                        if (off==-1){
                                getchar();
                                goto try_write_cpld_again;
                        }

			data=-1;
                        printf("Specify Data (hex): ");
                        scanf("%x",&data);
                        if (data==-1){
                                getchar();
                                goto try_write_cpld_again;
                        }

			write_cpld(off,data);

			printf("CPLD Write Offset=0x%X Data=0x%X\n",
				off,data);
			hit_any_key();
			break;

		case 3:
			led_test();
			hit_any_key();
			break;

		case 4:
			err=read_cpld(0x00);
			printf("CPLD Status Slot 0 Data=0x%X\n",	
					err);
			if ((err & FRONT_END_TYPE_MASK) == 0x00){
				printf("T1/E1/J1 Interface module (PMC based)\n");
			}

			hit_any_key();
			break;

                case 5:         // Boot Flash read
try_read_flash_again:
			system("clear");
			printf("\n");
	
			printf("FLASH READ: \n");

			off32=-1;
			printf("Specify Offset (hex): ");
			scanf("%lx",&off32);
			if (off32==-1){
				getchar();
				goto try_read_flash_again;
			}
			sector_type = DEF_SECTOR_FLASH;
                        if((off32 & 0x40000) != 0) {
                                sector_type = USER_SECTOR_FLASH;
                        }
			err=read_flash(sector_type, memory_type, off32);
			printf("FLASH Read Offset=0x%lX Data=0x%X\n",	
					off32,err);
			hit_any_key();
			break;

                case 6:         // Boot Flash write   
try_write_flash_again:
                        system("clear");
                        printf("\n");

                        printf("FLASH WRITE: \n");

                        off32=-1;
                        printf("Specify Offset (hex): ");
                        scanf("%lx",&off32);
                        if (off32==-1){
                                getchar();
                                goto try_write_flash_again;
                        }

			data=-1;
                        printf("Specify Data (hex): ");
                        scanf("%x",&data);
                        if (data==-1){
                                getchar();
                                goto try_write_flash_again;
			}
			write_flash(sector_type, memory_type, off32,data);

			printf("FLASH Write Offset=0x%lX Data=0x%X\n",
				off32,data);
			hit_any_key();
			break;

                case 7:         // Check Flash ID 
			write_flash(sector_type, memory_type, 0x5555, 0xAA);
			write_flash(sector_type, memory_type, 0x2AAA, 0x55);
			write_flash(sector_type, memory_type, 0x5555, 0x90);
			data = read_flash(sector_type, memory_type, 0x00);
			printf("Manufacture ID (0x01): %02X\n",	data);
			data = read_flash(sector_type, memory_type, 0x01);
			printf("Device ID (0x4F)     : %02X\n", data);
			reset_flash();
			hit_any_key();
			break;

	        case 8:         // Update Flash Code (Deafult sector) 
			printf("Specify filename: ");
			scanf("%s",filename);
                   //   strcpy(filename, "SANGOMA.BIN");
			reset_flash();
			update(DEF_SECTOR_FLASH, MEMORY_TYPE_FLASH, filename);
			hit_any_key();
			break;

		case 9:         // Update Flash Code (User sector) 
			printf("Specify filename: ");
			scanf("%s",filename);
			//strcpy(filename, "SANGOMA.BIN");
			reset_flash();
			update(USER_SECTOR_FLASH, MEMORY_TYPE_FLASH, filename);
			hit_any_key();
			break;

		case 10:        // Reset Flash
			reset_flash();
			hit_any_key();
			break;

		case 11:        // Erase DEF Flash sector   
			reset_flash();
			erase_sector_flash(DEF_SECTOR_FLASH);
			hit_any_key();
			break; 	

		case 12:        // Erase user Flash sector   
			reset_flash();
			erase_sector_flash(USER_SECTOR_FLASH);
			hit_any_key();
			break; 	

		case 13:        // Chip Erase
			reset_flash();
			write_flash(sector_type, memory_type, 0x5555, 0xAA);
			write_flash(sector_type, memory_type, 0x2AAA, 0x55);
			write_flash(sector_type, memory_type, 0x5555, 0x80);
			write_flash(sector_type, memory_type, 0x5555, 0xAA);
			write_flash(sector_type, memory_type, 0x2AAA, 0x55);
			write_flash(sector_type, memory_type, 0x5555, 0x10);
			hit_any_key();
			break;

		case 14:
try_prg_flash_again:
                        system("clear");
                        printf("\n");

                        printf("Program FLASH: \n");

                        off32=-1;
                        printf("Specify Offset (hex): ");
                        scanf("%lx",&off32);
                        if (off32==-1){
                                getchar();
                                goto try_prg_flash_again;
                        }

			data=-1;
                        printf("Specify Data (hex): ");
                        scanf("%x",&data);
                        if (data==-1){
                                getchar();
                                goto try_prg_flash_again;
			}
                        sector_type = DEF_SECTOR_FLASH; 
			if((off32 & 0x40000) != 0) {
                                sector_type = USER_SECTOR_FLASH;
                        } 
			reset_flash();
			prg_flash_byte(sector_type, off32, data);

			printf("Program FLASH Offset=0x%lX Data=0x%X\n",
				off32,data);
			hit_any_key();
			break;
			
		case 15:
			printf("Specify filename: ");
			scanf("%s",filename);
//			strcpy(filename, "SANGOMA.BIN");
			reset_flash();
			verify(DEF_SECTOR_FLASH, MEMORY_TYPE_FLASH, filename);
			hit_any_key();
			break;
		case 16:
			err=read_cpld(0x09);
			printf("Board Status =0x%X\n",err);

			if ((err & JP7_VALUE) == 0x00){
				printf("Boot Flash default sector write protecte				d\n");
			}

			if ((err & JP8_VALUE) == 0x00){
				printf("Force to Boot from default sector active\n");
			}

			if ((err & SW0_VALUE) == 0x00){
				printf("Dip switch 0  - ON\n");
			} else {
				printf("Dip switch 0  - OFF\n");
			}

			if ((err & SW1_VALUE) == 0x00){
				printf("Dip switch 1  - ON\n");
			} else {
				printf("Dip switch 1  - OFF\n");
			}
			hit_any_key();
			break;

		case 17:
                        /* Reload new code in to Xilinx from
                           Flash Default sector */   
                        write_cpld(0x07,0x80);   
			break;		
                case 18:
                        /* Reload new code in to Xilinx from
                           Flash User sector */  
                        write_cpld(0x07,0xC0); 
                        break;

	        case 19:   // Update Flash Code (Deafult sector) 
			printf("Specify filename: ");
			scanf("%s",filename);
			reset_flash();
			update_man(MEMORY_TYPE_FLASH, filename);
			hit_any_key();
			break;

		case -1:
			return 0;
		}
	}
	return 0;
}

