#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <sys/time.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <string.h>
#include <errno.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <linux/if.h>
#include <linux/if_packet.h>
#include "../sdla_t3_hdlc.h"
#include "xilinx_cpld.h"

struct sdla_hdlc_api api_cmd;
int sock;
unsigned char i_name[20];

void main_menu(void);
void read_chip_control_menu(void);
void registers_and_internal_ram_test_menu(void);
void setup_chip_configuration_menu(void);
int exec_command(int cmd);
int exec_reload_pci_cmd(void);
int exec_read_cmd(unsigned int off, unsigned int len, unsigned int *data);
int exec_write_cmd(unsigned int off, unsigned int len, unsigned int data);

void hit_any_key(void);

extern int pmc_initialization (int sock);
extern int cpld_initialization (int sock);

struct cmd_list_struct cmd_list[]={
	{"READ",SDLA_HDLC_READ_REG},
	{"WRITE",SDLA_HDLC_WRITE_REG},
	{".",0xFF}
};

int MakeConnection(void) 
{
	struct sockaddr_ll sa;
	struct ifreq ifr;
	
	memset(&sa,0,sizeof(struct sockaddr_ll));
	errno = 0;
   	sock = socket(AF_PACKET, SOCK_DGRAM, 0);
   	if( sock < 0 ) {
      		perror("Socket");
      		return -EIO;
   	} /* if */
  
	strcpy(ifr.ifr_name,i_name);
	if (ioctl(sock,SIOCGIFINDEX,&ifr)){
		perror("Get index: ");
		return -EIO;
	}

	sa.sll_protocol = htons(PVC_PROT);
	sa.sll_family=AF_PACKET;
	sa.sll_ifindex = ifr.ifr_ifindex;

        if(bind(sock, (struct sockaddr *)&sa, sizeof(struct sockaddr_ll)) < 0){
                perror("bind");
		printf("Failed to bind a socket to %s interface\n",i_name);
                exit(0);
        }
	printf("Socket bound to %s\n\n",i_name);

	hit_any_key();
	return 0;

}

int arg_error_check(int i, int argc)
{
	if (i+1 > argc-1){
		printf("Argument syntax error!\n");
		printf("Type sdla_t3_hdlc_api <cr> for help\n");
		return -EINVAL;
	}
	return 0;
}

static unsigned char usage_info[]="\n"
"Usage:\n"
"	args: -i <ifname>\n" 
"	      -c [READ|WRITE|RXDMA|TXDMA]\n" 
"	      -bar [0|1]\n" 
"	      -off <number>\n"
"	      -l [1|2|4|X] \n"
"	      -d <number>\n"
"\n"
"	./sdla_t3_hdlc_api -i whdlc0 -cmd READ -bar 0 -off 10 -l 1\n"
"	./sdla_t3_hdlc_api -i whdlc0 -cmd WRITE -bar 0 -off 10 -l 4 -d 0x80000000\n";

void usage (void)
{
	printf("%s",usage_info);
}

void hit_any_key(void)
{
	printf("Hit any key to continue... \n");
	getchar();
	getchar();
}

int main(int argc, char* argv[])
{
	int i,j=0;
	int err;
	struct ifreq ifr;
	char menu_driven=0;

	if (argc == 1){
		sprintf(i_name,"whdlc0");
		menu_driven=1;
		goto skip_args;
	}
	
	memset(&api_cmd,0,sizeof(struct sdla_hdlc_api));
	api_cmd.cmd=0xFF;

	for (i = 0; i < argc; i++){

		if (!strcmp(argv[i],"-h")){
		
			usage();
			return 0;

		}else if (!strcmp(argv[i],"-menu")){
			api_cmd.cmd=1;
			menu_driven=1;
					
		}else if (!strcmp(argv[i],"-bar")){
			
			if (arg_error_check(i,argc)){
				return -EINVAL;
			}	
			
			api_cmd.bar=atoi(argv[i+1]);
			
		}else if (!strcmp(argv[i],"-c")){
			
			if (arg_error_check(i,argc)){
				return -EINVAL;
			}	

			api_cmd.cmd=0xFF;
			
			for (j=0;;j++){
				if (!strcmp(cmd_list[j].cmd_str,".")){
					break;
				}
	
				if (!strcmp(cmd_list[j].cmd_str,argv[i+1])){
					api_cmd.cmd=cmd_list[j].cmd;
					break;
				}
			}

			if (api_cmd.cmd==0xFF){
				printf("Error: invalid cmd %s\n",argv[i+1]);
				return -EINVAL;
			}
		
		}else if (!strcmp(argv[i],"-l")){
			
			if (arg_error_check(i,argc)){
				return -EINVAL;
			}	
			
			api_cmd.len=atoi(argv[i+1]);
		
		}else if (!strcmp(argv[i],"-d")){
			
			if (arg_error_check(i,argc)){
				return -EINVAL;
			}
		
			if (api_cmd.len == 0){
				printf("\nError Invalid data len = 0\n");
				printf("Run -h option for usage\n");
				return -EINVAL;
			}
			
			if (api_cmd.len == 1){
				*(unsigned char*)api_cmd.data = (unsigned char)strtoul(argv[i+1], NULL, 0);
			}else if (api_cmd.len == 2){
				*(unsigned short*)api_cmd.data = (unsigned short)strtoul(argv[i+1], NULL, 0);
			}else if (api_cmd.len == 4){
				*(unsigned int*)api_cmd.data = (unsigned int)strtoul(argv[i+1], NULL, 0);
			}else{
				strcpy(api_cmd.data,argv[i+1]);
			}

			printf("Data = 0x%x len = %i Str=%s\n",
					*(unsigned int*)api_cmd.data,api_cmd.len,argv[i+1]);
			
		}else if (!strcmp(argv[i],"-i")){

			if (arg_error_check(i,argc)){
				return -EINVAL;
			}
			
			strcpy(i_name,argv[i+1]);
			if (argc == 3){
				menu_driven=1;
				goto skip_args;
			}
		
		}else if (!strcmp(argv[i],"-off")){
		
			api_cmd.offset=atoi(argv[i+1]);
		}
	}

	if (api_cmd.cmd==0xFF){
		printf("Error: invalid cmd\n");
		return -EINVAL;
	}
		
skip_args:
	if ((err=MakeConnection()) != 0){
		return err;
	}

	strcpy(ifr.ifr_name,i_name);
	ifr.ifr_data = (char*)&api_cmd;


	if (menu_driven){
		main_menu();
		close(sock);
		return 0;
	}
	
	printf("Executing cmd %s Len %i Bar %i Off %i Data 0x%x\n",
			cmd_list[j].cmd_str,
			api_cmd.len,
			api_cmd.bar,
			api_cmd.offset,
			*(unsigned int*)api_cmd.data);

	err = ioctl(sock,api_cmd.cmd,&ifr);
	if (err){
		perror("Cmd Exec: ");
		return err;
	}

	printf("Cmd Ok\n");
	if (api_cmd.len == 1){
		printf("Result Data u8  = 0x%02x\n",*(unsigned char*)api_cmd.data);
	}else if (api_cmd.len == 2){
		printf("Result Data u16 = 0x%04x\n",*(unsigned short*)api_cmd.data);
	}else if (api_cmd.len == 4){
		printf("Result Data u32 = 0x%08x\n",*(unsigned int*)api_cmd.data);
	}else{
		int i;
		printf("Result Data Len=%i : ",api_cmd.len);
		for (i=0;i<api_cmd.len;i++){
			printf("0x%x ",api_cmd.data[i]);
		}
		printf("\n");
	}

		
	close (sock);
	return err;
};

static unsigned char main_menu_list[]="\n"
"\n"
"Xilinx: Main Menu\n" 
"-----------------\n"
"\n"
"1. Read Chip Control and Status\n"
"\n"
"2. Registers and Internal RAM Tests\n"
"\n"
"3. Setup chip configuration\n"
"\n"
"4. Reload PCI Bios Info\n"
"\n"
"5. PMC Initialization\n"
"\n"
"6. CPLD Initialization\n"
"\n"
"q. Quit\n"
"\n"
"Please Select (<opt>|q): ";

void main_menu(void)
{
	int val;
//	unsigned int data;
//	int err;

#if 0	
	err=exec_read_cmd(0x40,4,&data);
	if (err){
		return;
	}

	data&=~(0x60);

	err=exec_write_cmd(0x40,4,data);
	if (err){
		return;
	}
#endif	
	
	for (;;){

		val=-1;
		
		system("clear");
	
		printf("%s",main_menu_list);

		scanf("%d",&val);
		if (val==-1){
			getchar();
		}

		printf("Val=%d\n",val);
		
		switch (val){

		case 1:
			printf ("Read Chip Control\n");
			read_chip_control_menu();
			break;

		case 2:
			printf ("Registers and Internal RAM Tests\n");
			registers_and_internal_ram_test_menu();
			break;

		case 3:
			printf ("Setup chip configuration\n");
			break;

		case 4:
			printf ("Reloading pci bios info\n");
			exec_reload_pci_cmd();	
			break;

		case 5: 
			printf ("PMC Initialization\n");
			if (pmc_initialization(sock)){
				return;
			}
			break;

		case 6: 
			printf ("CPLD Initialization\n");
			if (cpld_initialization(sock)){
				return;
			}
			break;
			
		default:
			printf("Exiting ...\n");
			return;
		}
	}
}


enum {
	READ_PCI_REGISTERS,
	READ_CHIP_CONFIG,
	READ_HDLC_INTERRUPT_PENDING_REG,
	READ_DMA_INTERRUPT_PENDING_REG,
	READ_CONTROL_RAM,
	READ_DMA_CONTROL_REG,
	READ_DMA_STATUS_REG,
	READ_DMA_DESCRIPTORS,
	RESET_HDLC,
	RESET_CHIP,
	READ_8_16_32,
	WRITE_8_16_32,

	GPIO_PORT_CFG_TEST,
	CONTROL_RAM_ADDR_TEST,
	CONTROL_RAM_DATA_TEST,
	HDLC_DATA_TEST,
	HDLC_ADDR_TEST,
	DMA_DESCR_RAM_ADDR_TEST,
	DMA_DESCR_RAM_DATA_TEST,
	ACCESS_TEST_CHIP_REG,
	RELOAD_BIOS_SETTINGS
		
};

static unsigned char read_chip_control[]="\n"
"	Xilinx: Read Chip Control\n" 
"	-------------------------\n"
"\n"
" 1. Read PCI registers (00-10,3C)\n"
"\n"
" 2. Read Chip configuration (40)\n"
"\n"
" 3. Read HDLC Interrupt pending register \n"
"\n"
" 4. Read DMA Interrupt pending register\n"
"\n"
" 5. Read Control RAM\n"
"\n"
" 6. Read DMA control register\n"
"\n"
" 7. Read DMA status register (0x74,0x78,0x80)\n"
"\n"
" 8. Read DMA descriptrs, (from 0x100-0x2FF)\n"
"\n"
" 9. Reset HDLC\n"
"\n"
"10. Reset Chip\n"
"\n"
"11. Read (8/16/32) from offset\n"
"\n"
"12. Write (8/16/32) from offset\n"
"\n"
"q. Quit\n"
"\n"
"Please Select (<opt>|q): \n";


void read_chip_control_menu(void)
{

	int val;

	
	for (;;){		

		val=-1;
		
		system("clear");
	
		printf("%s", read_chip_control);

		scanf("%d",&val);
		if (val==-1){
			getchar();
		}

		switch (val){

		case 1:
			printf ("Read PCI registers (00-10,3C)\n");
			exec_command(READ_PCI_REGISTERS);
			hit_any_key();
			break;
		case 2:
			printf ("Read Chip configuration\n");
			exec_command(READ_CHIP_CONFIG);
			hit_any_key();
			break;
		case 3:
			printf ("Read HDLC Interrupt pending register\n");
			exec_command(READ_HDLC_INTERRUPT_PENDING_REG);
			hit_any_key();
			break;
		case 4:
			printf ("Read DMA Interrupt pending register\n");
			exec_command(READ_DMA_INTERRUPT_PENDING_REG);
			hit_any_key();
			break;
		case 5:
			printf ("Read Control RAM\n");
			exec_command(READ_CONTROL_RAM);
			hit_any_key();
			break;
		case 6:
			printf ("Read DMA control register\n");
			exec_command(READ_DMA_CONTROL_REG);
			hit_any_key();
			break;
		case 7:
			printf ("Read DMA status register (0x74,0x78,0x80)\n");
			exec_command(READ_DMA_STATUS_REG);
			hit_any_key();
			break;	
		case 8:
			printf ("Read DMA descriptrs, (from 0x100-0x2FF)\n");
			exec_command(READ_DMA_DESCRIPTORS);
			hit_any_key();
			break;
		case 9:
			printf ("Reset HDLC\n");
			exec_command(RESET_HDLC);
			hit_any_key();
			break;
		case 10:
			printf ("Reset Chip\n");
			exec_command(RESET_CHIP);
			hit_any_key();
			break;
		case 11:
			printf ("Read (8/16/32) from offset\n");
			exec_command(READ_8_16_32);
			hit_any_key();
			break;	
		case 12:
			printf ("Write (8/16/32) from offset\n");
			exec_command(WRITE_8_16_32);
			hit_any_key();
			break;

		default:
			return;
		}

	}
}


static unsigned char reg_internal_ram_tests[]="\n"
"	Xilinx: Registers and Internal Ram Tests\n" 
"	----------------------------------------\n"
"\n"
"1. GPIO port configuration register test (0x44)\n"
"\n"
"2. Control RAM address test\n"
"\n"
"3. Control RAM data test\n"
"\n"
"4. HDLC address test (for all 32 channels)\n"
"\n"
"5. HDLC data test (for all 32 channels)\n"
"\n"
"6. DMA descr RAM address test\n"
"\n"
"7. DMA descr RAM data test\n"
"\n"
"8. Access test on Reg 0x40\n"
"\n"
"q. Quit\n"
"\n"
"Please Select (<opt>|q): ";


void registers_and_internal_ram_test_menu(void)
{
	int val;
	
	for (;;){
	
		val=-1;
		
		system("clear");
	
		printf("%s",reg_internal_ram_tests);

		scanf("%d",&val);
		if (val==-1){
			getchar();
		}
		
		switch (val){

		case 1:
			printf ("GPIO port configuration register test\n");
			exec_command(GPIO_PORT_CFG_TEST);
			hit_any_key();
			break;
		case 2:
			printf ("\n");
			exec_command(CONTROL_RAM_ADDR_TEST);
			hit_any_key();	
			break;
		case 3:
			printf ("\n");
			exec_command(CONTROL_RAM_DATA_TEST);
			hit_any_key();
			break;
		case 4:
			printf ("\n");
			exec_command(HDLC_ADDR_TEST);
			hit_any_key();
			break;
		case 5:
			printf ("\n");
			exec_command(HDLC_DATA_TEST);
			hit_any_key();
			break;
		case 6:
			printf ("\n");
			exec_command(DMA_DESCR_RAM_ADDR_TEST);
			hit_any_key();
			break;
		case 7:
			printf ("\n");
			exec_command(DMA_DESCR_RAM_DATA_TEST);
			hit_any_key();
			break;
		case 8:
			printf("\n");
			exec_command(ACCESS_TEST_CHIP_REG);
			hit_any_key();
			break;
		default:
			return;
		}
	}
}

static unsigned char set_chip_config[]="\n"
"	Xilinx: Setup Chip Configuration\n" 
"	--------------------------------\n"
"\n"
"1. Read and display configuration file\n"
"\n"
"2. Change parameters\n"
"\n"
"3. Chip setup\n"
"\n"
"q. Quit\n"
"\n"
"Please Select (<opt>|q): ";


void setup_chip_configuration_menu(void)
{
	int val;

	for (;;){

		val=-1;
		
		system("clear");
	
		printf("%s",set_chip_config);

		scanf("%d",&val);
		if (val==-1){
			getchar();
		}

		switch (val){

		case 1:
			printf ("Read Chip Control\n");
			hit_any_key();
			break;
		case 2:
			printf ("Registers and Internal RAM Tests\n");
			hit_any_key();	
			break;
		case 3:
			printf ("Setup chip configuration\n");
			hit_any_key();
			break;
		default:
			return;
		}
	}
}

static unsigned char cpld_config[]="\n"
"	Xilinx: CPLD Configuration\n"
"	------------------------------\n"
"\n"
" 1. Read CPLD Access.\n"
" 2. Write CPLD Access.\n"
" 3. LED Test.\n"
" 4. Read Front End Type.\n"
" 5. Boot Flash read.\n"
" 6. Boot Flash write.\n"
" 7. Check Flash ID.\n"
" 8. Update Flash code (Default sector)\n"
" 9. Update Flash code (User sector)\n"
"10. Reset Flash.\n"
"11. Default Flash sector Erase !\n"
"12. User Flash sector Erase !\n"
"13. Chip Erase !\n"
"14. Program Flash (byte).\n"
"15. Default sector verification.\n"
"16. Read Board Status.\n"
"17. Reboot Xilinx from the Flash Default sector.\n"
"18. Reboot Xilinx from the Flash User sector.\n"
"19. Program Flash Full.\n"
" q. Quit.\n"
"\n"
"Please Select (Opt|q): ";



int cpld_initialization (int sock)
{
	int err;
	int val;
	int off;
	int data;
	int memory_type = MEMORY_TYPE_FLASH, sector_type = USER_SECTOR_FLASH;
	unsigned long off32;
	char filename[15];
	int flash_id;

	for (;;){
	
		val=-1;
		
		system("clear");
	
		printf("%s",cpld_config);

		scanf("%d",&val);
		if (val==-1){
			getchar();
		}
		switch (val){

		case 1:

try_read_cpld_again:
			system("clear");
			printf("\n");
	
			printf("CPLD READ: \n");

			off=-1;
			printf("Specify Offset (hex): ");
			scanf("%x",&off);
			if (off==-1){
				getchar();
				goto try_read_cpld_again;
			}

			err=read_cpld(off);
			printf("CPLD Read Offset=0x%X Data=0x%X\n",	
					off,err);
			hit_any_key();
			break;

		case 2:

try_write_cpld_again:
                        system("clear");
                        printf("\n");

                        printf("CPLD WRITE: \n");

                        off=-1;
                        printf("Specify Offset (hex): ");
                        scanf("%x",&off);
                        if (off==-1){
                                getchar();
                                goto try_write_cpld_again;
                        }

			data=-1;
                        printf("Specify Data (hex): ");
                        scanf("%x",&data);
                        if (data==-1){
                                getchar();
                                goto try_write_cpld_again;
                        }

			write_cpld(off,data);

			printf("CPLD Write Offset=0x%X Data=0x%X\n",
				off,data);
			hit_any_key();
			break;

		case 3:
			led_test();
			hit_any_key();
			break;

		case 4:
			err=read_cpld(0x00);
			printf("CPLD Status Slot 0 Data=0x%X\n",	
					err);
			if ((err & FRONT_END_TYPE_MASK) == 0x00){
				printf("T1/E1/J1 Interface module (PMC based)\n");
			}

			hit_any_key();
			break;

                case 5:         // Boot Flash read
try_read_flash_again:
			system("clear");
			printf("\n");
	
			printf("FLASH READ: \n");

			off32=-1;
			printf("Specify Offset (hex): ");
			scanf("%lx",&off32);
			if (off32==-1){
				getchar();
				goto try_read_flash_again;
			}
			sector_type = DEF_SECTOR_FLASH;
                        if((off32 & 0x40000) != 0) {
                                sector_type = USER_SECTOR_FLASH;
                        }
			err=read_flash(sector_type, memory_type, off32);
			printf("FLASH Read Offset=0x%lX Data=0x%X\n",	
					off32,err);
			hit_any_key();
			break;

                case 6:         // Boot Flash write   
try_write_flash_again:
                        system("clear");
                        printf("\n");

                        printf("FLASH WRITE: \n");

                        off32=-1;
                        printf("Specify Offset (hex): ");
                        scanf("%lx",&off32);
                        if (off32==-1){
                                getchar();
                                goto try_write_flash_again;
                        }

			data=-1;
                        printf("Specify Data (hex): ");
                        scanf("%x",&data);
                        if (data==-1){
                                getchar();
                                goto try_write_flash_again;
			}
			write_flash(sector_type, memory_type, off32,data);

			printf("FLASH Write Offset=0x%lX Data=0x%X\n",
				off32,data);
			hit_any_key();
			break;

                case 7:         // Check Flash ID 
			check_flash_id(memory_type, sector_type, &flash_id);
			printf("Manufacture ID (0x01): %02X\n",	(flash_id & 0xFF00) >> 8);
			printf("Device ID (0x4F)     : %02X\n", (flash_id & 0xFF));
			hit_any_key();
			break;

	        case 8:         // Update Flash Code (Deafult sector) 
			printf("Specify filename: ");
			scanf("%s",filename);
                   //   strcpy(filename, "SANGOMA.BIN");
			reset_flash();
			update(DEF_SECTOR_FLASH, MEMORY_TYPE_FLASH, filename);
			hit_any_key();
			break;

		case 9:         // Update Flash Code (User sector) 
			printf("Specify filename: ");
			scanf("%s",filename);
			//strcpy(filename, "SANGOMA.BIN");
			reset_flash();
			update(USER_SECTOR_FLASH, MEMORY_TYPE_FLASH, filename);
			hit_any_key();
			break;

		case 10:        // Reset Flash
			reset_flash();
			hit_any_key();
			break;

		case 11:        // Erase DEF Flash sector   
			reset_flash();
			erase_sector_flash(DEF_SECTOR_FLASH, 1);
			hit_any_key();
			break; 	

		case 12:        // Erase user Flash sector   
			reset_flash();
			erase_sector_flash(USER_SECTOR_FLASH, 1);
			hit_any_key();
			break; 	

		case 13:        // Chip Erase
			chip_erase(memory_type, sector_type);
			hit_any_key();
			break;

		case 14:
try_prg_flash_again:
                        system("clear");
                        printf("\n");

                        printf("Program FLASH: \n");

                        off32=-1;
                        printf("Specify Offset (hex): ");
                        scanf("%lx",&off32);
                        if (off32==-1){
                                getchar();
                                goto try_prg_flash_again;
                        }

			data=-1;
                        printf("Specify Data (hex): ");
                        scanf("%x",&data);
                        if (data==-1){
                                getchar();
                                goto try_prg_flash_again;
			}
                        sector_type = DEF_SECTOR_FLASH; 
			if((off32 & 0x40000) != 0) {
                                sector_type = USER_SECTOR_FLASH;
                        } 
			reset_flash();
			prg_flash_byte(sector_type, off32, data);

			printf("Program FLASH Offset=0x%lX Data=0x%X\n",
				off32,data);
			hit_any_key();
			break;
			
		case 15:
			printf("Specify filename: ");
			scanf("%s",filename);
//			strcpy(filename, "SANGOMA.BIN");
			reset_flash();
			verify(DEF_SECTOR_FLASH, MEMORY_TYPE_FLASH, filename);
			hit_any_key();
			break;
		case 16:
			err=read_cpld(0x09);
			printf("Board Status =0x%X\n",err);

			if ((err & JP7_VALUE) == 0x00){
				printf("Boot Flash default sector write protecte				d\n");
			}

			if ((err & JP8_VALUE) == 0x00){
				printf("Force to Boot from default sector active\n");
			}

			if ((err & SW0_VALUE) == 0x00){
				printf("Dip switch 0  - ON\n");
			} else {
				printf("Dip switch 0  - OFF\n");
			}

			if ((err & SW1_VALUE) == 0x00){
				printf("Dip switch 1  - ON\n");
			} else {
				printf("Dip switch 1  - OFF\n");
			}
			hit_any_key();
			break;

		case 17:
                        /* Reload new code in to Xilinx from
                           Flash Default sector */   
                        write_cpld(0x07,0x80);   
			break;		
                case 18:
                        /* Reload new code in to Xilinx from
                           Flash User sector */  
                        write_cpld(0x07,0xC0); 
                        break;

	        case 19:   // Update Flash Code (Deafult sector) 
			printf("Specify filename: ");
			scanf("%s",filename);
			reset_flash();
			update_man(MEMORY_TYPE_FLASH, filename);
			hit_any_key();
			break;

		case -1:
			return 0;
		}
	}
	return 0;
}

int exec_command(int cmd)
{
	struct ifreq ifr;
	int i,x,err,range;
	unsigned int data;
	unsigned int off,len;

	strcpy(ifr.ifr_name,i_name);
	ifr.ifr_data = (char*)&api_cmd;
	
	switch (cmd){

	case READ_PCI_REGISTERS:

		printf("READ_PCI_REGISTERS: \n");
	
		for (i=0;i<6;i++){

			api_cmd.cmd=SDLA_HDLC_READ_REG;
			api_cmd.bar=0;
			api_cmd.len=4;

			if (i<5){
				api_cmd.offset=i<<2;
			}else{
				api_cmd.offset=0x3C;
			}
			
			err = ioctl(sock,api_cmd.cmd,&ifr);
			if (err){
				perror("Cmd Exec: ");
				return err;
			}

			printf("\tOff=0x%X  Val=0x%08X\n",api_cmd.offset,*(unsigned int*)api_cmd.data);
		}

		printf("\n");
		
		break;
		
	case READ_CHIP_CONFIG:
	
		printf("READ_PCI_REGISTERS: \n");
		
		api_cmd.cmd=SDLA_HDLC_READ_REG;
		api_cmd.bar=0;
		api_cmd.len=4;
		api_cmd.offset=0x40;
			
		err = ioctl(sock,api_cmd.cmd,&ifr);
		if (err){
			perror("Cmd Exec: ");
			return err;
		}

		printf("\tOff=0x%X  Val=0x%08X\n",api_cmd.offset,*(unsigned int*)api_cmd.data);
		
		break;	
		
	case READ_HDLC_INTERRUPT_PENDING_REG:
	
		printf("READ_HDLC_INTERRUPT_PENDING_REG: \n");
		
		api_cmd.cmd=SDLA_HDLC_READ_REG;
		api_cmd.bar=0;
		api_cmd.len=4;
		api_cmd.offset=0x50;
			
		err = ioctl(sock,api_cmd.cmd,&ifr);
		if (err){
			perror("Cmd Exec: ");
			return err;
		}

		printf("\tTX: Off=0x%X  Val=0x%08X\n",api_cmd.offset,*(unsigned int*)api_cmd.data);

		api_cmd.cmd=SDLA_HDLC_READ_REG;
		api_cmd.bar=0;
		api_cmd.len=4;
		api_cmd.offset=0x54;
			
		err = ioctl(sock,api_cmd.cmd,&ifr);
		if (err){
			perror("Cmd Exec: ");
			return err;
		}

		printf("\tRX: Off=0x%X  Val=0x%08X\n",api_cmd.offset,*(unsigned int*)api_cmd.data);
		
		break;	
		
	case READ_DMA_INTERRUPT_PENDING_REG:
		
		printf("READ_DMA_INTERRUPT_PENDING_REG: \n");
		
		api_cmd.cmd=SDLA_HDLC_READ_REG;
		api_cmd.bar=0;
		api_cmd.len=4;
		api_cmd.offset=0x58;
			
		err = ioctl(sock,api_cmd.cmd,&ifr);
		if (err){
			perror("Cmd Exec: ");
			return err;
		}

		printf("\tTX: Off=0x%X  Val=0x%08X\n",api_cmd.offset,*(unsigned int*)api_cmd.data);

		api_cmd.cmd=SDLA_HDLC_READ_REG;
		api_cmd.bar=0;
		api_cmd.len=4;
		api_cmd.offset=0x5C;
			
		err = ioctl(sock,api_cmd.cmd,&ifr);
		if (err){
			perror("Cmd Exec: ");
			return err;
		}

		printf("\tRX: Off=0x%X  Val=0x%08X\n",api_cmd.offset,*(unsigned int*)api_cmd.data);
		
		break;
		
	case READ_CONTROL_RAM:
	
		printf("READ_CONTROL_RAM: \n");
	
		api_cmd.cmd=SDLA_HDLC_READ_REG;
		api_cmd.bar=0;
		api_cmd.len=4;
		api_cmd.offset=0x40;
			
		err = ioctl(sock,api_cmd.cmd,&ifr);
		if (err){
			perror("Cmd Exec: ");
			return err;
		}

		if (!(api_cmd.data[0] & 0x01)){
			range=24;
		}else{
			range=32;
		}
	
		api_cmd.cmd=SDLA_HDLC_WRITE_REG;
		api_cmd.bar=0;
		api_cmd.len=4;
		api_cmd.offset=0x40;
		api_cmd.data[0] &= ~(0x60); 

		printf("Range=%i, Setting Off=0x40 to Data=0x%X\n",
			range, api_cmd.data[0]);

		err = ioctl(sock,api_cmd.cmd,&ifr);
		if (err){
			perror("Cmd Exec: ");
			return err;
		}

		for (i=0;i<range;i++){

			api_cmd.cmd=SDLA_HDLC_WRITE_REG;
			api_cmd.bar=0;
			api_cmd.len=1;
			api_cmd.offset=0x62;
			api_cmd.data[0]=i;
			
			err = ioctl(sock,api_cmd.cmd,&ifr);
			if (err){
				perror("Cmd Exec: ");
				return err;
			}

			api_cmd.cmd=SDLA_HDLC_READ_REG;
			api_cmd.bar=0;
			api_cmd.len=4;
			api_cmd.offset=0x6C;
			
			err = ioctl(sock,api_cmd.cmd,&ifr);
			if (err){
				perror("Cmd Exec: ");
				return err;
			}
		
			printf("\tTimeSlot=%i  Val=0x%08lX",i,*(unsigned long*)api_cmd.data);
			if (i!=0 && i%2){	
				printf("\n");
			}
		}
		printf("\n");
		break;		
		
	case READ_DMA_CONTROL_REG:

		printf("READ_DMA_CONTROL_REG: \n");
		
		api_cmd.cmd=SDLA_HDLC_READ_REG;
		api_cmd.bar=0;
		api_cmd.len=4;
		api_cmd.offset=0x70;
		
		err = ioctl(sock,api_cmd.cmd,&ifr);
		if (err){
			perror("Cmd Exec: ");
			return err;
		}
		
		printf("\tOff=0x%X  Val=0x%08X\n",api_cmd.offset,*(unsigned int*)api_cmd.data);
		break;
		
	case READ_DMA_STATUS_REG:

		printf("READ_DMA_STATUS_REG: \n");

		api_cmd.cmd=SDLA_HDLC_READ_REG;
		api_cmd.bar=0;
		api_cmd.len=4;
		api_cmd.offset=0x74;
			
		err = ioctl(sock,api_cmd.cmd,&ifr);
		if (err){
			perror("Cmd Exec: ");
			return err;
		}

		printf("\tTX: Off=0x%X  Val=0x%08X\n",api_cmd.offset,*(unsigned int*)api_cmd.data);

		api_cmd.cmd=SDLA_HDLC_READ_REG;
		api_cmd.bar=0;
		api_cmd.len=4;
		api_cmd.offset=0x78;
			
		err = ioctl(sock,api_cmd.cmd,&ifr);
		if (err){
			perror("Cmd Exec: ");
			return err;
		}

		printf("\tRX: Off=0x%X  Val=0x%08X\n",api_cmd.offset,*(unsigned int*)api_cmd.data);

		api_cmd.cmd=SDLA_HDLC_READ_REG;
		api_cmd.bar=0;
		api_cmd.len=4;
		api_cmd.offset=0x80;
			
		err = ioctl(sock,api_cmd.cmd,&ifr);
		if (err){
			perror("Cmd Exec: ");
			return err;
		}

		printf("\tRX Empty: Off=0x%X  Val=0x%08X\n",api_cmd.offset,*(unsigned int*)api_cmd.data);

		break;
		
	case READ_DMA_DESCRIPTORS:
	
		printf("READ_DMA_DESCRIPTORS: \n");

		printf("          Tx                         Rx\n");
		
		for (i=0;i<128;i++){

			api_cmd.cmd=SDLA_HDLC_READ_REG;
			api_cmd.bar=0;
			api_cmd.len=4;
			api_cmd.offset=0x100+(i<<2);
			
			err = ioctl(sock,api_cmd.cmd,&ifr);
			if (err){
				perror("Cmd Exec: ");
				return err;
			}

			if (i==0 || !(i%4)){
				printf("\n");
				printf("Disc %03i: ",i);	
			}

			printf("0x%08X ",*(unsigned int*)api_cmd.data);
		}

		printf("\n");

		
		break;
		
	case RESET_HDLC:

		printf("RESET_HDLC: \n");
		
		api_cmd.cmd=SDLA_HDLC_READ_REG;
		api_cmd.bar=0;
		api_cmd.len=4;
		api_cmd.offset=0x40;
		
		err = ioctl(sock,api_cmd.cmd,&ifr);
		if (err){
			perror("Cmd Exec: ");
			return err;
		}
	
		printf("\tRead: Off=0x%X  Val=0x%08X\n",api_cmd.offset,*(unsigned int*)api_cmd.data);

		
		api_cmd.cmd=SDLA_HDLC_WRITE_REG;
		api_cmd.bar=0;
		api_cmd.len=4;
		api_cmd.offset=0x40;
		api_cmd.data[0]|=0x40;
		
		err = ioctl(sock,api_cmd.cmd,&ifr);
		if (err){
			perror("Cmd Exec: ");
			return err;
		}

		api_cmd.cmd=SDLA_HDLC_WRITE_REG;
		api_cmd.bar=0;
		api_cmd.len=4;
		api_cmd.offset=0x40;
		api_cmd.data[0]&=~(0x40);
		
		err = ioctl(sock,api_cmd.cmd,&ifr);
		if (err){
			perror("Cmd Exec: ");
			return err;
		}

		printf("Reset Done\n");

		break;
		
	case RESET_CHIP:

		printf("RESET_CHIP: \n");
		
		api_cmd.cmd=SDLA_HDLC_READ_REG;
		api_cmd.bar=0;
		api_cmd.len=4;
		api_cmd.offset=0x40;
		
		err = ioctl(sock,api_cmd.cmd,&ifr);
		if (err){
			perror("Cmd Exec: ");
			return err;
		}
	
		printf("\tRead: Off=0x%X  Val=0x%08X\n",api_cmd.offset,*(unsigned int*)api_cmd.data);

		
		api_cmd.cmd=SDLA_HDLC_WRITE_REG;
		api_cmd.bar=0;
		api_cmd.len=4;
		api_cmd.offset=0x40;
		api_cmd.data[0]|=0x20;
		
		err = ioctl(sock,api_cmd.cmd,&ifr);
		if (err){
			perror("Cmd Exec: ");
			return err;
		}

		api_cmd.cmd=SDLA_HDLC_WRITE_REG;
		api_cmd.bar=0;
		api_cmd.len=4;
		api_cmd.offset=0x40;
		api_cmd.data[0]&=~(0x20);
		
		err = ioctl(sock,api_cmd.cmd,&ifr);
		if (err){
			perror("Cmd Exec: ");
			return err;
		}

		printf("Reset Done\n");

		break;
		
	case READ_8_16_32:

		{
		int off,len;

try_read_again:
		system("clear");
		printf("\n");
	
		printf("READ_8_16_32: \n");

		off=-1;
		printf("Specify Offset (hex): ");
		scanf("%x",&off);
		if (off==-1){
			getchar();
			goto try_read_again;
		}

		len=-1;
		printf("Specify Len (1|2|4): ");
		scanf("%d",&len);
		if (len==-1){
			getchar();
			goto try_read_again;
		}

		err=exec_read_cmd(off,len,&data);
		if (err){
			return err;
		}	
#if 0
		api_cmd.cmd=SDLA_HDLC_READ_REG;
		api_cmd.bar=0;
		api_cmd.len=len;
		api_cmd.offset=off;

		err = ioctl(sock,api_cmd.cmd,&ifr);
		if (err){
			perror("Cmd Exec: ");
			return err;
		}
#endif	
		printf("\tRead: Off=0x%X Len=%i Val=0x%08X\n",
				api_cmd.offset,len,data);

		}
		break;
		
	case WRITE_8_16_32:


try_write_again:
		system("clear");
	
		printf("WRITE_8_16_32: \n");

		off=-1;
		printf("Specify Offset (hex): ");
		scanf("%x",&off);
		if (off==-1){
			getchar();
			goto try_write_again;
		}
		
		len=-1;
		printf("Specify Len (1|2|4): ");
		scanf("%d",&len);
		if (len==-1){
			getchar();
			goto try_write_again;
		}

		data=-1;
		printf("Specify Data: ");
		scanf("%x",&data);
		if (len==-1){
			getchar();
			goto try_write_again;
		}

		err=exec_write_cmd(off,len,data);
		if (err){
			return err;
		}
		
#if 0
		api_cmd.cmd=SDLA_HDLC_WRITE_REG;
		api_cmd.bar=0;
		api_cmd.len=len;
		api_cmd.offset=off;
		api_cmd.data[0]=data;
		if (len==1){
			*(unsigned char*)api_cmd.data=(unsigned char)data;
		}else if (len==2){
			*(unsigned short*)api_cmd.data=(unsigned short)data;
		}else{
			*(unsigned int*)api_cmd.data=(unsigned int)data;
		}

		err = ioctl(sock,api_cmd.cmd,&ifr);
		if (err){
			perror("Cmd Exec: ");
			return err;
		}
#endif	
		printf("\tWrite: Off=0x%X  Val=0x%08X\n",
				api_cmd.offset,data);

		break;


	case GPIO_PORT_CFG_TEST:


		for (i=0;i<10;i++){

			err=exec_write_cmd(0x44,4,0x00000000);
			if (err){
				return err;
			}	

			err=exec_read_cmd(0x44,4,&data);
			if (err){
				return err;
			}

			if (data != 0x00000000){
				printf("Zero Write test failed Data=0x%x Iter=%i!\n",data,i);
				return 1;
			}

			err=exec_write_cmd(0x44,4,0xFFFFFFFF);
			if (err){
				return err;
			}	

			err=exec_read_cmd(0x44,4,&data);
			if (err){
				return err;
			}

			if (data != 0xFFFFFFFF){
				printf("0xFF Write test failed Data=0x%X Iter=%i !\n",data,i);
				return 1;
			}

			err=exec_write_cmd(0x44,4,0xAAAAAAAA);
			if (err){
				return err;
			}	

			err=exec_read_cmd(0x44,4,&data);
			if (err){
				return err;
			}

			if (data != 0xAAAAAAAA){
				printf("0xAA Write test failed!\n");
				return 1;
			}
			
			err=exec_write_cmd(0x44,4,0x55555555);
			if (err){
				return err;
			}	

			err=exec_read_cmd(0x44,4,&data);
			if (err){
				return err;
			}

			if (data != 0x55555555){
				printf("0x55 Write test failed!\n");
				return 1;
			}
		}

		printf ("GPIO Port config test: 0,F,A,5 : PASSED!\n");


		for (i=0;i<10;i++){

			range=0x00000001;
			
			for (x=0;x<32;x++){
				err=exec_write_cmd(0x44,4,(range<<x));
				if (err){
					return err;
				}	
				err=exec_read_cmd(0x44,4,&data);
				if (err){
					return err;
				}

				if (data != (range<<x)){
					printf("Shift 1 test failed\n");
					return 1;
				}
			}
		}

		printf ("GPIO Port config test: Shift 1 : PASSED!\n");


		for (i=0;i<10;i++){

			range=0x00000001;
			
			for (x=0;x<32;x++){
				err=exec_write_cmd(0x44,4,~(range<<x));
				if (err){
					return err;
				}	
				err=exec_read_cmd(0x44,4,&data);
				if (err){
					return err;
				}

				if (data != ~(range<<x)){
					printf("Shift 0 test failed\n");
					return 1;
				}
			}
		}

		printf ("GPIO Port config test: Shift 0 : PASSED!\n");
		
		break;
	
	case CONTROL_RAM_ADDR_TEST:
		
		err=exec_read_cmd(0x40,4,&data);
		if (err){
			return err;
		}
		err=exec_write_cmd(0x40,4,(data|0x01));
		if (err){
			return err;
		}
	

		for (i=0;i<32;i++){

			err=exec_write_cmd(0x60,4,i<<16);
			if (err){
				return err;
			}

			err=exec_write_cmd(0x6C,4,i);
			if (err){
				return err;
			}
		
		}


		for (i=0;i<32;i++){
		
			err=exec_write_cmd(0x60,4,i<<16);
			if (err){
				return err;
			}
			
			err=exec_read_cmd(0x6C,4,&data);
			if (err){
				return err;
			}

		
			if ((data&0x1F) != i){
				printf ("Control RAM Addr Test Failed!\n");
				return 1;
			}


		}

		printf ("Control RAM Addr Test PASSED!\n");
		break;

	case CONTROL_RAM_DATA_TEST:

		/* Enable 32 channels */
		err=exec_read_cmd(0x40,4,&data);
		if (err){
			return err;
		}
		err=exec_write_cmd(0x40,4,(data|0x01));
		if (err){
			return err;
		}
	

		for (i=0;i<8;i++){
			err=exec_write_cmd(0x60,4,(i<<18));
			if (err){
				return err;
			}

			err=exec_write_cmd(0x6C,4,0x00000000);
			if (err){
				return err;
			}

			err=exec_write_cmd(0x60,4,((i<<18)+0x10000));
			if (err){
				return err;
			}

			err=exec_write_cmd(0x6C,4,0xFFFFFFFF);
			if (err){
				return err;
			}	

			err=exec_write_cmd(0x60,4,((i<<18)+0x20000));
			if (err){
				return err;
			}

			err=exec_write_cmd(0x6C,4,0xAAAAAAAA);
			if (err){
				return err;
			}

			err=exec_write_cmd(0x60,4,((i<<18)+0x30000));
			if (err){
				return err;
			}

			err=exec_write_cmd(0x6C,4,0x55555555);
			if (err){
				return err;
			}
			
		}


		for (i=0;i<8;i++){
	
			err=exec_write_cmd(0x60,4,(i<<18));
			if (err){
				return err;
			}
			
			err=exec_read_cmd(0x6C,4,&data);
			if (err){
				return err;
			}

		
			if ((data&0x1F) != 0x00000000){
				printf ("Control RAM Data: 0 Test Failed Data=0x%x!\n",data);
				return 1;
			}

			err=exec_write_cmd(0x60,4,((i<<18)+0x10000));
			if (err){
				return err;
			}
			
			err=exec_read_cmd(0x6C,4,&data);
			if (err){
				return err;
			}

		
			if ((data&0x1F) != 0x1F){
				printf ("Control RAM Data: F Test Failed!\n");
				return 1;
			}
			
			err=exec_write_cmd(0x60,4,((i<<18)+0x20000));
			if (err){
				return err;
			}
			
			err=exec_read_cmd(0x6C,4,&data);
			if (err){
				return err;
			}

		
			if ((data&0x1F) != 0x0A){
				printf ("Control RAM Data: A Test Failed!\n");
				return 1;
			}

			err=exec_write_cmd(0x60,4,((i<<18)+0x30000));
			if (err){
				return err;
			}
			
			err=exec_read_cmd(0x6C,4,&data);
			if (err){
				return err;
			}

		
			if ((data&0x1F) != 0x15){
				printf ("Control RAM Data: 5 Test Failed!\n");
				return 1;
			}

		}

		printf ("Control RAM Data: 0,F,A,5 Test PASSED!\n");
		break;
	
		
	case HDLC_DATA_TEST:
	
		printf("HDLC_DATA_TEST: \n");
		
		/* Enable 32 timeslots */
		err=exec_read_cmd(0x40,4,&data);
		if (err){
			return err;
		}
		err=exec_write_cmd(0x40,4,(data|0x01));
		if (err){
			return err;
		}

		/* Enable 32 logical channels */
		for (i=0;i<32;i++){
			err=exec_write_cmd(0x60,4,i<<16);
			if (err){
				return err;
			}

			err=exec_write_cmd(0x6C,4,i);
			if (err){
				return err;
			}
		}

		for (i=0;i<32;i++){
	
			/* Select channel i */
			err=exec_write_cmd(0x60,4,i);
			if (err){
				return err;
			}
			
			err=exec_write_cmd(0x68,4,0x00000000);
			if (err){
				return err;
			}	

			err=exec_read_cmd(0x68,4,&data);
			if (err){
				return err;
			}

			if (data != 0x00000000){
				printf("Zero Write test failed!\n");
				return 1;
			}

			err=exec_write_cmd(0x68,4,0xFFFFFFFF);
			if (err){
				return err;
			}	

			err=exec_read_cmd(0x68,4,&data);
			if (err){
				return err;
			}

			if (data != 0xFFFFFFFF){
				printf("0xFF Write test failed!\n");
				return 1;
			}

			err=exec_write_cmd(0x68,4,0xAAAAAAAA);
			if (err){
				return err;
			}	

			err=exec_read_cmd(0x68,4,&data);
			if (err){
				return err;
			}

			if (data != 0xAAAAAAAA){
				printf("0xAA Write test failed!\n");
				return 1;
			}
			
			err=exec_write_cmd(0x68,4,0x55555555);
			if (err){
				return err;
			}	

			err=exec_read_cmd(0x68,4,&data);
			if (err){
				return err;
			}

			if (data != 0x55555555){
				printf("0x55 Write test failed!\n");
				return 1;
			}
		}

		printf("HDLC DATA Test: 0,F,A,5 PASSED!\n");

	
		for (i=0;i<32;i++){
	
			/* Select channel i */
			err=exec_write_cmd(0x60,4,i);
			if (err){
				return err;
			}

			range=0x00000001;
			
			for (x=0;x<32;x++){
				err=exec_write_cmd(0x68,4,(range<<x));
				if (err){
					return err;
				}	
				err=exec_read_cmd(0x68,4,&data);
				if (err){
					return err;
				}

				if (data != (range<<x)){
					printf("Shift 1 test failed\n");
					return err;
				}
			}

			range=0x00000001;
			
			for (x=0;x<32;x++){
				err=exec_write_cmd(0x68,4,~(range<<x));
				if (err){
					return err;
				}	
				err=exec_read_cmd(0x68,4,&data);
				if (err){
					return err;
				}

				if (data != ~(range<<x)){
					printf("Shift 0 test failed\n");
					return err;
				}
			}
		}
		
		printf("HDLC DATA Test: Shift 0 & 1 PASSED!\n");

		
		break;

	case HDLC_ADDR_TEST:

		printf("HDLC_ADDR_TEST: \n");
		
		/* Enable 32 timeslots */
		err=exec_read_cmd(0x40,4,&data);
		if (err){
			return err;
		}
		err=exec_write_cmd(0x40,4,(data|0x01));
		if (err){
			return err;
		}

		/* Enable 32 logical channels */
		for (i=0;i<32;i++){
			err=exec_write_cmd(0x60,4,i<<16);
			if (err){
				return err;
			}

			err=exec_write_cmd(0x6C,4,i);
			if (err){
				return err;
			}
		}
	

		for (i=0;i<32;i++){

			err=exec_write_cmd(0x60,4,i);
			if (err){
				return err;
			}

			range=0;
			
			err=exec_write_cmd(0x68,4,range);
			if (err){
				return err;
			} 

			range+=0x00010001;
		}


		for (i=0;i<32;i++){
		
			err=exec_write_cmd(0x60,4,i);
			if (err){
				return err;
			}
			
			err=exec_read_cmd(0x68,4,&data);
			if (err){
				return err;
			}

			range=0;
			
			if ((data&0x001F001F) != range){
				printf ("HDLC Addr Test Failed!\n");
				return 1;
			}

			range+=0x00010001;

		}

		printf ("HDLC Addr Test PASSED!\n");

		break;

	case DMA_DESCR_RAM_ADDR_TEST:
	
		for (i=0;i<10;i++){

			for (x=0;x<128;x++){
				
				err=exec_write_cmd((0x100+(x<<2)),4,x);
				if (err){
					return err;
				}
			}
			
			for (x=0;x<128;x++){
				
				err=exec_read_cmd((0x100+(x<<2)),4,&data);
				if (err){
					return err;
				}

				if (data != x){
					printf("DMA Descr RAM Addr test Faild!\n");
					return 1;
				}
			}
		}

		printf("DMA Descr RAM Addr test PASSED!\n");
		
		break;

	case DMA_DESCR_RAM_DATA_TEST:

		for (i=0;i<10;i++){

			for (x=0;x<32;x++){
			
				err=exec_write_cmd((0x100+(x<<4)),4,0x00000000);
				if (err){
					return err;
				}
				
				err=exec_write_cmd((0x100+(x<<4)+4),4,0xFFFFFFFF);
				if (err){
					return err;
				}
				
				err=exec_write_cmd((0x100+(x<<4)+8),4,0xAAAAAAAA);
				if (err){
					return err;
				}
				
				err=exec_write_cmd((0x100+(x<<4)+0x0C),4,0x55555555);
				if (err){
					return err;
				}

			}

			
			for (x=0;x<32;x++){
			
				err=exec_read_cmd((0x100+(x<<4)),4,&data);
				if (err){
					return err;
				}
				
				if (data != 0x00000000){
					printf("DMA RAM DATA Test: 0 Failed!\n");
					return err;
				}
				
				err=exec_read_cmd((0x100+(x<<4)+4),4,&data);
				if (err){
					return err;
				}

				if (data != 0xFFFFFFFF){
					printf("DMA RAM DATA Test: F Failed!\n");
					return err;
				}

			
				err=exec_read_cmd((0x100+(x<<4)+8),4,&data);
				if (err){
					return err;
				}
				if (data != 0xAAAAAAAA){
					printf("DMA RAM DATA Test: A Failed!\n");
					return err;
				}
				
				err=exec_read_cmd((0x100+(x<<4)+0x0C),4,&data);
				if (err){
					return err;
				}
				if (data != 0x55555555){
					printf("DMA RAM DATA Test: 5 Failed!\n");
					return err;
				}

			}
		}	

		printf("DMA Descr RAM Addr test PASSED!\n");

		break;

	case ACCESS_TEST_CHIP_REG:

		printf("Starting CHIP Accesss Test...\n");
		
		for (i=0;i<1000000;i++){
		
			err=exec_read_cmd(0x40,4,&data);
			if (err){
				return err;
			}

			data=0x0060;
			err=exec_write_cmd(0x40,4,data);
			if (err){
				return err;
			}
		
			data=0x0001;
			err=exec_write_cmd(0x40,4,data);
			if (err){
				return err;
			}
try_ready_again:
			err=exec_read_cmd(0x40,4,&data);
			if (err){
				return err;
			}

			if (data&0x00000080){
				printf("HDLC Ready OK: 0x%X\n",data);
			}else{
				goto try_ready_again;
			}

			data=0x800103CA;
			err=exec_write_cmd(0x70,4,data);
			if (err){
				return err;
			}
			
			err=exec_read_cmd(0x40,4,&data);
			if (err){
				return err;
			}
	
			data=0x0181;
			err=exec_write_cmd(0x40,4,data);
			if (err){
				return err;
			}


			data=0x00030000;
			err=exec_write_cmd(0x60,4,data);
                        if (err){
                                return err;
                        }
			data=0x00000000;
			err=exec_write_cmd(0x6C,4,data);
                        if (err){
                                return err;
                        }

			for (i=0;i<32;i++){

				if (i==3)
					continue;

				data=0;
				data=i<<16;
	                        err=exec_write_cmd(0x60,4,data);
        	                if (err){
                	                return err;
                	        }
                        	data=0x00000001;
                        	err=exec_write_cmd(0x6C,4,data);
                        	if (err){
                                	return err;
                        	}
			}

//???????????????????????/
			data=0x00000000;
			err=exec_write_cmd(0x60,4,data);
                        if (err){
                                return err;
                        }
		
			data=0x03030000;
                        err=exec_write_cmd(0x64,4,data);
                        if (err){
                                return err;
                        }

			err=exec_read_cmd(0x70,4,&data);
                        if (err){
                                return err;
                        }
			

			data=0x800103CA;
			err=exec_write_cmd(0x70,4,data);
                        if (err){
				return err;
                        }

			data=0x00000000;
			err=exec_write_cmd(0x10C,4,data);
                        if (err){
                                return err;
                        }
	
			data=0x00000000;
                        err=exec_write_cmd(0x104,4,data);
                        if (err){
                                return err;
                        }

			data=0x00000000;
                        err=exec_write_cmd(0x60,4,data);
                        if (err){
                                return err;
                        }

			data=0x00000000;
                        err=exec_write_cmd(0x64,4,data);
                        if (err){
                                return err;
                        }

			err=exec_read_cmd(0x40,4,&data);
                        if (err){
                                return err;
                        }

			data=0x00000060;
                        err=exec_write_cmd(0x40,4,data);
                        if (err){
                                return err;
                        }
		}

		printf("Chip Access Teset PASSED!\n");
		
		break;
		
	default:
		return -EINVAL;

	}

	return 0;
}


int exec_reload_pci_cmd(void)
{
        int err;
        struct ifreq ifr;

        strcpy(ifr.ifr_name,i_name);
        ifr.ifr_data = (char*)&api_cmd;

        api_cmd.cmd=SDLA_HDLC_SET_PCI_BIOS;
        api_cmd.bar=0;
        api_cmd.len=4;

        err = ioctl(sock,api_cmd.cmd,&ifr);
        if (err){
                perror("Relad PCI Cmd Exec: ");
        }

	return 0;
}


int exec_read_cmd(unsigned int off, unsigned int len, unsigned int *data)
{
	int err;
	struct ifreq ifr;

	strcpy(ifr.ifr_name,i_name);
	ifr.ifr_data = (char*)&api_cmd;

	api_cmd.cmd=SDLA_HDLC_READ_REG;
	api_cmd.bar=0;
	api_cmd.len=len;
	api_cmd.offset=off;

	err = ioctl(sock,api_cmd.cmd,&ifr);
	if (err){
		perror("Read Cmd Exec: ");
	}

	if (len==1){
		*(unsigned char*)data = *(unsigned char*)api_cmd.data;
	}else if (len==2){
		*(unsigned short*)data = *(unsigned short*)api_cmd.data;
	}else{
		*(unsigned int*)data = *(unsigned int*)api_cmd.data;
	}

	return err;

}

int exec_write_cmd(unsigned int off, unsigned int len, unsigned int data)
{
	int err;
	struct ifreq ifr;

	strcpy(ifr.ifr_name,i_name);
	ifr.ifr_data = (char*)&api_cmd;

	api_cmd.cmd=SDLA_HDLC_WRITE_REG;
	api_cmd.bar=0;
	api_cmd.len=len;
	api_cmd.offset=off;

	if (len==1){
		*(unsigned char*)api_cmd.data=(unsigned char)data;
	}else if (len==2){
		*(unsigned short*)api_cmd.data=(unsigned short)data;
	}else{
		*(unsigned int*)api_cmd.data=(unsigned int)data;
	}

	err = ioctl(sock,api_cmd.cmd,&ifr);
	if (err){
		perror("Write Cmd Exec: ");
	}

	return err;
}


