
#ifndef _WANPIPE_DSP_PROT_H_
#define _WANPIPE_DSP_PROT_H_

#undef DSP_DEBUG
/*
 ********************************************************************
 *			Include files                               *
 ********************************************************************
 */
#include <linux/wanpipe_cfg.h>
#include <linux/wanpipe_debug.h>
#include <linux/wanpipe_defines.h>
#include <linux/wanpipe_common.h>
#include <linux/wanpipe_dsp_kernel.h>
/*
 ********************************************************************
 * 			Defines/Macros				    *
 ********************************************************************
 */
/* DSP message size */
#define DSP_CALL_REQ_MSG_LEN		8
#define DSP_INV_TO_CLEAR_MSG_LEN	3
#define DSP_STATUS_MSG_LEN		4
#define DSP_ACK_MSG_LEN			3
#define DSP_CIRCUIT_ENABLED_MSG_LEN	10
#define DSP_CIRCUIT_RESET_MSG_LEN	5
#define DSP_CIRCUIT_REQUEST_MSG_LEN	9
#define DSP_CIRCUIT_DISCONNECT_MSG_LEN	3
#define DSP_CMD_UNDELIVERED_MSG_LEN	6
#define DSP_RES_UNDELIVERED_MSG_LEN	4
#define DSP_CMD_ABORTED_MSG_LEN		6
#define DSP_RES_ABORTED_MSG_LEN		4

#define DSP_HDR_LEN			3

#define TCPIP				0x00
#define API				0x01

/* DSP major states */
enum {
	DSP_MJ_NONE		= 0x0000,
	DSP_MJ_DISCONNECTED	= 0x0001,	/* Initial state */
	DSP_MJ_READY		= 0x0002,	/* Ready to receive Inc. Call */
	DSP_MJ_CONNECTING	= 0x0004,	/* Connecting */
	DSP_MJ_CONNECTED	= 0x0008,	/* Connected */
	DSP_MJ_DISCONNECTING	= 0x0010	/* Disconnecting */
};
/* DSP minor state */
enum {
	DSP_MN_NONE		= 0x0000,
	DSP_MN_CALL_REQ		= 0x0100,
	DSP_MN_ACC_CALL		= 0x0200,
	DSP_MN_CALL_REQ_AGAIN	= 0x0400,
	DSP_MN_CIRUIT_RESET	= 0x0800,
	DSP_MN_ITC		= 0x1000,
	DSP_MN_CLEAR_REQ	= 0x2000
};

#define DSP_MJ_MASK		0x00FF
#define DSP_MN_MASK		0xFF00
// DSP state machine 
#define DSP_DISCONNECTED		(DSP_MJ_DISCONNECTED)	/* Initial state */
#define DSP_READY			(DSP_MJ_READY)		/* Ready state   */
#define DSP_CONNECTING			(DSP_MJ_CONNECTING)	/* Rx/Tx CALL REQUEST */
#define DSP_CONNECTING_CALL_REQ_AGAIN	(DSP_MJ_CONNECTING | DSP_MN_CALL_REQ_AGAIN)	/* Rx/Tx C.REQ again */
#define DSP_CONNECTING_ACC_CALL		(DSP_MJ_CONNECTING | DSP_MN_ACC_CALL) 	/* Rx/Tx C. ACC */
#define DSP_CONNECTED			(DSP_MJ_CONNECTED)			/* Rx/Tx M_CE */
#define DSP_CONNECTED_CR		(DSP_MJ_CONNECTED | DSP_MN_CIRUIT_RESET)/* Rx/Tx M_CR */
#define DSP_DISCONNECTING		(DSP_MJ_DISCONNECTING | DSP_MN_ITC)	/* Rx/Tx M_ITC */
#define DSP_DISCONNECTING_CL_REQ	(DSP_MJ_DISCONNECTING | DSP_MN_CLEAR_REQ) /* Rx/Tx CLEAR REQ */
#if 0
enum {
	DSP_DISCONNECTED	= 0x00,	/* Initial state */
	DSP_READY,
	DSP_CONNECTING,			/* received/sent CALL REQUEST */
	DSP_CONNECTING_CALL_REQ_AGAIN,	/* received/sent CALL REQUEST again */
	DSP_CONNECTING_ACC_CALL,	/* received/sent ACCEPTED CALL */
	DSP_CONNECTED,			/* received/sent M_CE */
	DSP_CONNECTED_CR,		/* received/sent M_CR */
	DSP_DISCONNECTING,		/* received/sent M_ITC message */
	DSP_DISCONNECTING_CL_REQ	/* received/sent CLEAR REQUEST */
};
#endif


/* BH flags */
#define BH_RUNNING		0
#define BH_AWAITING_KICK	1
#define MORE_TX			2
#define DSP_KICKED		3

/* Critical flag */
#define T1_RUNNING		0
#define T2_RUNNING		1
#define T3_RUNNING		2
#define RX_SKB_FREE		3
#define BH_SMP			4
#define CRIT_LINK_DOWN		5

#define DSP_SN_MODULES		256
#define DSP_SN_NOT_USED		0x00
#define DSP_SN_WAIT_ACK_R	0x01
#define DSP_SN_WAIT_ACK_L	0x02

#define DSP_ESC			0x27
#define DSP_SOH			0x01
#define DSP_STX			0x02
#define DSP_ETX			0x03
#define DSP_ETB			0x26

#define DSP_ASYNC_QUEUE_SIZE	7

#define DSP_MTU_DEFAULT		5000

/* Bit map in `aqdm` member of DSP API header */
#define DSP_L_M_BIT	0x01
#define DSP_L_D_BIT	0x02
#define DSP_L_Q_BIT	0x04
#define DSP_L_ACK_BIT	0x08
/* Bit map in DSP header */
#define DSP_R_ACK_BIT	0x40
#define DSP_R_XPR_BIT	0x20

#define IS_L_M_BIT(val)	(val & DSP_L_M_BIT) ? DSP_TRUE : DSP_FALSE
#define SET_L_M_BIT(val)(val) ? DSP_L_M_BIT : 0
#define IS_L_Q_BIT(val)	(val & DSP_L_Q_BIT) ? DSP_TRUE : DSP_FALSE
#define SET_L_Q_BIT(val)(val) ? DSP_L_Q_BIT : 0
#define IS_L_ACK(val)	(val & DSP_L_ACK_BIT) ? DSP_TRUE : DSP_FALSE
#define SET_L_ACK(val)	(val) ? DSP_L_ACK_BIT : 0

#define IS_R_ACK(val)	(val & DSP_R_ACK_BIT) ? DSP_TRUE : DSP_FALSE
#define SET_R_ACK(val)	(val) ? DSP_R_ACK_BIT : 0
#define IS_R_XPR(val)	(val & DSP_R_XPR_BIT) ? DSP_TRUE : DSP_FALSE
#define SET_R_XPR(val)	(val) ? DSP_R_XPR_BIT : 0

#if 0
ALEX 10.4.2001
ETX_ETB_ESC
/*******************************************************/
/* The ESC byte in data from HOST is part of the frame */
/*******************************************************/

#define SET_ET_BYTE(val)	(val == DSP_TRUE) ? DSP_ETX: DSP_ETB
#define GET_ET_BYTE(val)	(val == DSP_ETX) ? DSP_TRUE: DSP_FALSE

#define DSP_TX_HDR_LEN(dsp_link) ((dsp_link->cfg.pad_type == DSP_HPAD)?(DSP_HDR_LEN + 1):DSP_HDR_LEN)
#define DSP_RX_HDR_LEN(dsp_link) ((dsp_link->cfg.pad_type == DSP_TPAD)?(DSP_HDR_LEN + 1):DSP_HDR_LEN)
#endif

/* Convert character to integer */
#define GET_CH(val)	(isdigit(val) ? (val - '0') : (val - 'A' + 10))

/* Get byte from CALL REQUEST user data */
#define DSP_GET_CR_BYTE(str, i) (GET_CH(toupper(str[i])) << 4 | GET_CH(toupper(str[i+1])))

/* Increment TX/RX statistics */
#define DSP_INC_TX_STAT(x)	{ dsp_link->dspstats.x++; dsp_link->dspstats.tx_total++;dsp_link->ifstats.tx_packets++; }
#define DSP_INC_RX_STAT(x)	{ dsp_link->dspstats.x++; dsp_link->dspstats.rx_total++;dsp_link->ifstats.rx_packets++; }

#ifdef DSP_DEBUG
# define DSP_DEBUG_NONE		0x0000
# define DSP_DEBUG_INT		0x0001
# define DSP_DEBUG_RX		0x0002
# define DSP_DEBUG_TX		0x0004
# define DSP_DEBUG_TIMER	0x0008
# define DSP_DEBUG_MSG		0x0010
# define DSP_DEBUG_SOCK		0x0020
# define DSP_DEBUG_BH		0x0040
# define DSP_DEBUG_MEM		0x0080
# define DSP_DEBUG_OTHER	0x8000
# define DSP_DEBUG_ALL		0xFFFF
# define DEBUG1(x)	if (dsp_debug & DSP_DEBUG_INT)		printk x
# define DEBUG2(x)	if (dsp_debug & DSP_DEBUG_RX)		printk x
# define DEBUG3(x)	if (dsp_debug & DSP_DEBUG_TX)		printk x
# define DEBUG4(x)	if (dsp_debug & DSP_DEBUG_TIMER)	printk x
# define DEBUG5(x)	if (dsp_debug & DSP_DEBUG_MSG)		printk x
# define DEBUG6(x)	if (dsp_debug & DSP_DEBUG_SOCK)		printk x
# define DEBUG7(x)	if (dsp_debug & DSP_DEBUG_BH)		printk x
# define DEBUG20(x)	if (dsp_debug & DSP_DEBUG_OTHER)	printk x
#else
# define DEBUG1(x)
# define DEBUG2(x)
# define DEBUG3(x)
# define DEBUG4(x)
# define DEBUG5(x)
# define DEBUG6(x)
# define DEBUG7(x)
# define DEBUG20(x)
#endif

#define IS_NULL(ptr, ret_value)	if (ptr == NULL){		\
			printk(KERN_INFO "%s:%d - No memory\n",	\
					__FILE__, __LINE__);	\
					return ret_value;	\
				}

#define DSP_ASSERT(val) if (val){					\
			printk(KERN_INFO "**************** ASSERT FAILED ****************\n");	\
			printk(KERN_INFO "%s:%d - Critical error\n",	\
					__FILE__, __LINE__);		\
			return -EINVAL;					\
			}

#define DSP_ASSERT1(val) if (val){					\
			printk(KERN_INFO "**************** ASSERT FAILED ****************\n");	\
			printk(KERN_INFO "%s:%d - Critical error\n",	\
					__FILE__, __LINE__);		\
			return;					\
			}

#define DSP_ASSERT2(val, ret) if (val){					\
			printk(KERN_INFO "**************** ASSERT FAILED ****************\n");	\
			printk(KERN_INFO "%s:%d - Critical error\n",	\
					__FILE__, __LINE__);		\
			return ret;					\
			}

#define DSP_MEM_INC(size)	atomic_add(size, &dsp_mem_in_used)

#define DSP_MEM_DEC(size)	atomic_sub(size, &dsp_mem_in_used)

#define DSP_SKB_INC(size,dsp)	{atomic_add(size,&dsp->mem_in_used);	\
				 DSP_MEM_INC(size);}

#define DSP_SKB_DEC(size,dsp)	{atomic_sub(size,&dsp->mem_in_used);	\
				 DSP_MEM_DEC(size);}

#define DSP_SKB_HEAD(skb)	printk(KERN_INFO "%s:%d: HEAD ROOM H %i\n",	\
						__FILE__, __LINE__, skb_headroom(skb))

#define DSP_SKB_TAIL(skb)	printk(KERN_INFO "%s:%d: BAD TAIL ROOM T %i\n",	\
						__FILE__, __LINE__, skb_tailroom(skb))

#define DSP_DEBUG_MEM
#ifdef DSP_DEBUG_MEM
#define ALLOC_SKB(new_skb,len, dsp) { new_skb = dev_alloc_skb(len);			\
				     if (new_skb != NULL && dsp) { DSP_SKB_INC(new_skb->truesize,dsp); }else{ WAN_MEM_ASSERT("LAPB");} }
#define KFREE_SKB(skb,dsp)	    { if (dsp) DSP_SKB_DEC(skb->truesize, dsp); \
	                              dev_kfree_skb_any(skb); }

 #define KMALLOC(ptr,len,flag)	{ ptr=kmalloc(len, flag); \
	  			  if (ptr != NULL){ DSP_MEM_INC(len);} else { WAN_MEM_ASSERT("DSP");} }
 #define KFREE(ptr)		{DSP_MEM_DEC(sizeof(*ptr)); kfree(ptr);}


#else
 #define KMALLOC(ptr,len,flag)	ptr=kmalloc(len, flag)
 #define KFREE(ptr)		kfree(ptr)	
				  
#define ALLOC_SKB(new_skb,len, dsp) new_skb = alloc_skb(len)
#define KFREE_SKB(skb,dsp)	    dev_kfree_skb_any(skb)
#endif
						
/*
 ********************************************************************
 * 		Typedef/Structures				    *
 ********************************************************************
 */
/*
 * DSP Driver Statistics
 */
typedef struct dspstats {
	unsigned int rx_ack_data_msg;
	unsigned int rx_data_msg;
	unsigned int rx_data_pkt;
	unsigned int rx_call_req;
	unsigned int rx_call_acc;
	unsigned int rx_ce;
	unsigned int rx_cr;
	unsigned int rx_cd;
	unsigned int rx_creq;
	unsigned int rx_ack;
	unsigned int rx_status;
	unsigned int rx_cmd_res_undelivered;
	unsigned int rx_cmd_res_aborted;
	unsigned int rx_inv_to_clear;
	unsigned int rx_dropped;
	unsigned int rx_total;

	unsigned int tx_ack_data_msg;
	unsigned int tx_data_msg;
	unsigned int tx_data_pkt;
	unsigned int tx_call_req;
	unsigned int tx_call_acc;
	unsigned int tx_ce;
	unsigned int tx_cr;
	unsigned int tx_cd;
	unsigned int tx_creq;
	unsigned int tx_ack;
	unsigned int tx_status;
	unsigned int tx_cmd_res_undelivered;
	unsigned int tx_cmd_res_aborted;
	unsigned int tx_inv_to_clear;
	unsigned int tx_dropped;
	unsigned int tx_total;
}dspstats_t;

/*
 * DSP device per X.25 logical channel
 */
typedef struct dsp_link{
	
	struct dsp_link*	next;
	
	struct net_device*	master;
	struct net_device*	dev;
	unsigned short		lcn;
	
	char			name[IFNAMSIZ];
	unsigned char 		usedby;
	unsigned long 		critical;

	unsigned char		appl_id;

	/* Sequence numbers */
	unsigned char 		rx_sn;
	unsigned char 		tx_sn;
	unsigned char 		array_sn[DSP_SN_MODULES];
	unsigned char 		last_ack_sn_l;
	unsigned char 		last_ack_sn_r;

	unsigned char 		is_rx_more_data;
	unsigned char 		is_tx_more_data;
	//dspapi_hdr_t		rx_hdr;
	//dspapi_hdr_t		tx_hdr;
	struct sk_buff*		rx_skb;
	unsigned short		tx_pkt_len;
	unsigned short		rx_pkt_len;

	struct	dsp_parms	cfg;
	int	 		state;
	unsigned char 		incoming_call_handling;

	/* Internal control information */
	struct sk_buff_head 	async_queue;
	//struct sk_buff_head rx_queue;
	struct sk_buff_head 	write_queue;
	
	/* reason code for DSP command */
	unsigned char 		reason_code;

	/* Service Timeout period */
	struct timer_list 	T1_timer;
	
	/* PAD protocol timeout period */
	struct timer_list 	T2_timer;
	
	/* Timeout between two packets of the same M_DATA */
	struct timer_list 	T3_timer;
	
	//struct tq_struct dsp_rx_task;
	struct tasklet_struct 	dsp_tx_task;
	unsigned long 		tq_working;
	
	spinlock_t 		map_lock;	
	spinlock_t 		list_lock; 

	dspapi_hdr_t		dspapi_hdr;
	dspapi_t		dsp_api;
	unsigned char		call_string[DSP_CALL_STR_SZ+1];

	dspapi_t		dsp_api_event;

	void* 			sk_id;

	dspstats_t		dspstats;
	unsigned int		tx_kick_cnt;

	/* Proc file enteries for each x25 link */
	struct proc_dir_entry *proc_dir;
	struct proc_dir_entry *dent;

	atomic_t		mem_in_used;

	struct net_device_stats ifstats;
	pid_t pid;

} dsp_link_t;


/*
 ********************************************************************
 * 			Global variables			    *
 ********************************************************************
 */
extern atomic_t dsp_mem_in_used;

/*
 ********************************************************************
 * 			Function prototypes			    *
 ********************************************************************
 */

extern void dsp_link_send (unsigned long dsp_link_data);
extern int dsp_data_tx_up(dsp_link_t*, struct sk_buff*);
extern int dsp_data_tx_down(dsp_link_t*, struct sk_buff*);

/* dsp_subr.c */
extern void dsp_clear_link_queues(dsp_link_t *dsp, int dsp_state);
extern int dsp_trigger_link_bh(dsp_link_t *dsp_link);
extern void dsp_set_state(dsp_link_t*, int);
extern char* dsp_get_mj_state(int);
extern char* dsp_get_mn_state(int);
extern unsigned char dsp_get_sn(unsigned char);
extern int dsp_chk_ack_sn(dsp_link_t*, unsigned char, unsigned char*, int);
extern int dsp_wait4ack(dsp_link_t*);
extern void dsp_free_rx_skb(dsp_link_t*);
extern int dsp_split_tx_data(dsp_link_t*, struct sk_buff*);
extern struct sk_buff* dsp_concat_rx_data(dsp_link_t*, struct sk_buff*);

/* dsp_out.c */
extern void dsp_data_input(dsp_link_t *dsp_link, struct sk_buff *skb);
extern int dsp_send_data_frame(dsp_link_t*, struct sk_buff*);
extern int dsp_send_control(dsp_link_t*, unsigned char, char*, int);

/* dsp_timer.c */
extern void dsp_queue_tq (struct tq_struct *bh_pointer);
extern void dsp_mark_bh (void);
extern void dsp_start_t1_timer(dsp_link_t *dsp_link);
extern void dsp_stop_t1_timer(dsp_link_t *dsp_link);
extern void dsp_start_t2_timer(dsp_link_t *dsp_link);
extern void dsp_stop_t2_timer(dsp_link_t *dsp_link);
extern void dsp_start_t3_timer(dsp_link_t *dsp_link);
extern void dsp_stop_t3_timer(dsp_link_t *dsp_link);

/* dsp_api_iface.c */
extern int dsp_api_cmd_setup(dsp_link_t*, struct ifreq *);
extern int dsp_api_get_cmd_result(dsp_link_t*, struct ifreq *);
extern int dsp_api_place_call(dsp_link_t*, struct ifreq *);
extern int dsp_api_clear_call(dsp_link_t*, struct ifreq *);
extern int dsp_api_accept_call(dsp_link_t*, struct ifreq *);
extern int dsp_api_reset_call(dsp_link_t*, struct ifreq *);
extern int dsp_api_circuit_enabled(dsp_link_t*, struct ifreq*);
extern int dsp_api_circuit_reset(dsp_link_t*, struct ifreq*, int);
extern int dsp_api_circuit_request(dsp_link_t*, struct ifreq*);
extern int dsp_api_circuit_disconnect(dsp_link_t*, struct ifreq*);
extern int dsp_api_inv_to_clear(dsp_link_t*, struct ifreq*);
extern int dsp_api_eeack(dsp_link_t*, struct ifreq*);
extern int dsp_api_status(dsp_link_t*, struct ifreq*);
extern int dsp_api_cmd_res_undelivered(dsp_link_t*, struct ifreq*);
extern int dsp_api_cmd_res_aborted(dsp_link_t*, struct ifreq*);
extern int dsp_snd_api_call_request(dsp_link_t*, void*, struct sk_buff*);
extern int dsp_snd_api_oob_msg (dsp_link_t*, dspapi_hdr_t*,unsigned char*, unsigned short);

/* dsp_prot.c */
int dsp_proc_init (void);
void dsp_proc_cleanup (void);
int dsp_proc_link_init (dsp_link_t *dsp_link);
int dsp_proc_link_cleanup(dsp_link_t *dsp_link);

/* FIXME: Add other defines here */

#endif /* _WANPIPE_DSP_PROT_H_*/
