# ============================================================================
# Makefile	Multiprotocol WAN Router for Linux.  Make Script.
#
# Copyright	(c) 1995-1997 Sangoma Technologies Inc.  All Rights Reserved.
# ----------------------------------------------------------------------------
# Mar 27  2000  Nenad Corbic	Version 2.0.5 to 2.1.2
# Jan 07, 1999	Jaspreet Singh	Version 2.0.4
# Aug 25, 1998	Jaspreet Singh  Version 2.0.3
# Nov 06, 1997	Jaspreet Singh	Version 2.0.0
# Jul 28, 1997	Jaspreet Singh  Version 1.0.5
# Jul 10, 1997  Jaspreet Singh	Version 1.0.4
# June 3, 1997	Jaspreet Singh	Version 1.0.3	
# Jan 15, 1997	Gene Kozin	Version 1.0.1.
# Dec 31, 1996	Gene Kozin	Initial version.
# ============================================================================

####### DEFINES ##############################################################

# Build Options.
PROD	= wanrouter
VERSION	= 2.2.3
DEBUG	= 2

# Project File Paths.
TMPDIR	= tmp
OUTDIR	= mod
LOCINC	= /usr/src/linux/include
VPATH	= $(LOCINC):$(TMPDIR)
ARCH    = $(shell uname -m)

KERN := $(shell uname -r | grep 2.4)

ifneq "${KERN}" ""

CFLAGS=-D__KERNEL__ -DCONFIG_ALL_WANPIPE_PROT -I/usr/src/linux/include -Wall -Wstrict-prototypes -O2 -fomit-frame-pointer -fno-strict-aliasing -pipe -mpreferred-stack-boundary=2 -march=$(ARCH) -DMODULE -DEXPORT_SYMTAB

else

CFLAGS=-D__KERNEL__ -DCONFIG_STANDARD_WANPIPE_DRIVERS -DCONFIG_ALL_WANPIPE_PROT -I$(LOCINC) -Wall -Wstrict-prototypes -O2 -fomit-frame-pointer -fno-strict-aliasing -pipe -fno-strength-reduce -m486 -malign-loops=2 -malign-jumps=2 -malign-functions=2 -DMODULE -DEXPORT_SYMTAB

endif

TARGETS	= $(OUTDIR)/wanrouter.o \
	  $(OUTDIR)/sdladrv.o \
	  $(OUTDIR)/wanpipe.o \
	  $(OUTDIR)/syncppp.o 

# Miscellaneous.
TAGFILE	= net/wanrouter/patchlevel

####### RULES ################################################################

all:	$(TARGETS)
	@echo "Ok."

update:
#	@if [ ! -z $(shell uname -r | grep 2.4) ]; then \
#		echo "Updating Modules For 2.4.X Kernel"; \
#		cp $(OUTDIR)/wanrouter.o  /lib/modules/$(shell uname -r)/kernel/net/wanrouter; \
#		cp $(OUTDIR)/sdladrv.o /lib/modules/$(shell uname -r)/kernel/drivers/net/wan; \
#		cp $(OUTDIR)/wanpipe.o /lib/modules/$(shell uname -r)/kernel/drivers/net/wan; \
#	else \
#	 	echo "Updating Modules for kernels 2.2.X or lower"; \
#		cp $(OUTDIR)/wanrouter.o  /lib/modules/$(shell uname -r)/misc; \
#		cp $(OUTDIR)/sdladrv.o /lib/modules/$(shell uname -r)/net; \
#		cp $(OUTDIR)/wanpipe.o /lib/modules/$(shell uname -r)/net; \
#	fi
	../../scripts/Mkupdate $(PROD) $(VERSION) $(TAGFILE)

patch:
	../../scripts/Mkpatch $(PROD) $(VERSION)

# ----- Multiprotocol WAN router module --------------------------------------

$(OUTDIR)/wanrouter.o:	wanmain.o wanproc.o
	ld -r -o $@ $^
	chmod 664 $@

$(TMPDIR)/wanmain.o:	wanmain.c 
	$(CC) $(CFLAGS) -c -o $@ $<

$(TMPDIR)/wanproc.o:	wanproc.c 
	$(CC) $(CFLAGS) -c -o $@ $<

# ----- SDLA Support Module --------------------------------------------------

$(OUTDIR)/sdladrv.o:	sdladrv.c 
	$(CC) $(CFLAGS) -c -o $@ $<
	chmod 664 $@

$(OUTDIR)/syncppp.o:	syncppp.c 
	$(CC) $(CFLAGS) -c -o $@ $<
	chmod 664 $@


# ----- WANPIPE(tm) Driver Module --------------------------------------------

$(OUTDIR)/wanpipe.o:	sdlamain.o sdla_fr.o sdla_ppp.o sdla_chdlc.o sdla_x25.o sdla_ft1.o wanpipe_multppp.o sdla_te1.o sdla_56k.o sdla_bitstrm.o sdla_edu.o
	ld -r -o $@ $^
	chmod 664 $@

$(TMPDIR)/sdlamain.o:	sdlamain.c
	$(CC) $(CFLAGS) -c -o $@ $<
	chmod 664 $@

$(TMPDIR)/sdla_x25.o:	sdla_x25.c
	$(CC) $(CFLAGS) -c -o $@ $<
	chmod 664 $@

$(TMPDIR)/sdla_fr.o:	sdla_fr.c
	$(CC) $(CFLAGS) -c -o $@ $<
	chmod 664 $@

$(TMPDIR)/sdla_ppp.o:	sdla_ppp.c 
	$(CC) $(CFLAGS) -c -o $@ $<
	chmod 664 $@

$(TMPDIR)/sdla_chdlc.o:	sdla_chdlc.c 
	$(CC) $(CFLAGS) -c -o $@ $<
	chmod 664 $@

$(TMPDIR)/sdla_ft1.o:	sdla_ft1.c 
	$(CC) $(CFLAGS) -c -o $@ $<
	chmod 664 $@

$(TMPDIR)/wanpipe_multppp.o:	wanpipe_multppp.c
	$(CC) $(CFLAGS) -c -o $@ $<
	chmod 664 $@

$(TMPDIR)/sdla_te1.o:	sdla_te1.c
	$(CC) $(CFLAGS) -c -o $@ $<
	chmod 664 $@

$(TMPDIR)/sdla_56k.o:	sdla_56k.c
	$(CC) $(CFLAGS) -c -o $@ $<
	chmod 664 $@

$(TMPDIR)/sdla_bitstrm.o:	sdla_bitstrm.c
	$(CC) $(CFLAGS) -c -o $@ $<
	chmod 664 $@

$(TMPDIR)/sdla_edu.o:	sdla_edu.c
	$(CC) $(CFLAGS) -c -o $@ $<
	chmod 664 $@

###### END ###################################################################
