#!/bin/sh



# ----------------------------------------------------------------------------
# Prompt user for input.
# ----------------------------------------------------------------------------
prompt()
{
	echo -ne "$*" >&2
	read CMD rest
	return 0
}

# ----------------------------------------------------------------------------
# Get Yes/No
# ----------------------------------------------------------------------------
getyn()
{
	while prompt "$* (y/n) "
	do	case $CMD in
			[yY])	return 0
				;;
			[nN])	return 1
				;;
			*)	echo -e "\nPlease answer y or n" >&2
				;;
		esac
	done
}
make_patches () 
{
	echo "PATCHES OK";
	
	for kern in $KERN_LIST; do
		
		cd $SRC_DIR
		eval "./$PATCH_SETUP $kern"
		if [ $? -ne 0 ]; then
			echo "Failed to setup $kern kernel directories for patching !!!\n"
			return 1;
		fi

		cd $ROUTER_DIR
		make update
		make patch
	done
	
	cd $PATCH_DIR
	./xupdate

	for kern in $KERN_LIST; do
		cd $SRC_DIR
		eval "./$PATCH_SETUP $kern"
		break
	done

	echo "Patches created and Updated OK"
	echo
	echo
	return 0;
}

delete_old_utils()
{
	local tool
	local tool_exe

	for tool in $WAN_TOOLS; do
		tool_exe="$WANTOOLS_DIR/bin/$tool"
		[ -f "$tool_exe" ] && rm -f $tool_exe	
	done
}


compile_tools()
{

	if [ -f "$WANTOOLS_DIR/config/ft1/source/ft1_exec" ]; then
		rm -f $WANTOOLS_DIR/config/ft1/source/ft1_exec
	fi

	if [ -f "$WANTOOLS_DIR/config/lxdialog/lxdialog" ]; then
		rm -f $WANTOOLS_DIR/config/lxdialog/lxdialog
		rm -f $WANTOOLS_DIR/config/lxdialog/*.o
	fi

	#Compiling CFGFT1 Libraries

	cd $WANTOOLS_DIR/config/ft1

	exec make | grep "Ok\." > /dev/null	
	if [ $? -eq 0 ]; then
		echo -e "\tWANPIPE CFGFT1 Libraries compiled OK\n"
	else
		echo -e "\tERROR: Failed to compile WANPIPE CFGFT1 Libraries !!!"
		exit 1
	fi

	#Compiling LXDIALOG

	cd $WANTOOLS_DIR/config/lxdialog

	exec make | grep "Ok\." > /dev/null	
	if [ $? -eq 0 ]; then
		echo -e "\tWANPIPE Lxdialog compiled OK\n"
	else
		echo -e "\tERROR: Failed to compile Lxdialog !!!"
		exit 1
	fi

	cd $WANTOOLS_DIR/util

	delete_old_utils

	exec make | grep "Ok\." > /dev/null	
	if [ $? -eq 0 ]; then
		echo -e "\tWANPIPE Utilities compiled OK\n"
	else
		echo -e "\tERROR: Failed to compile WANPIPE utilities !!!"
		exit 1
	fi
}


ROUTER_DIR=/wanpipe/code/router
SRC_DIR=/usr/src
PATCH_SETUP=patch_setup.sh
PATCH_DIR=/wanpipe/patches
WANTOOLS_DIR=/wanpipe/code/wantools
WAN_TOOLS="wanconfig fpipemon cpipemon ppipemon xpipemon sdladump"
KERN_LIST="2.4.9 2.4.4 2.4.0 2.2.19 2.2.18 2.2.15 2.2.14 2.2.13"

# Setup drivers
getyn "Would you like to make patches?"
if [ $? -eq 0 ]; then
	make_patches || exit 1;
fi

#Compile all tools
compile_tools;

#Update the file list
cd $WANTOOLS_DIR/scripts
./make_filelist.pl
echo -e "\tFile List Updated"


echo -e "\tSETUP COMPLETE\n"
exit 0
