/*****************************************************************************
* chdlc_api.c	CHDLC API: Receive Module
*
* Author(s):	Gideon Hack & Nenad Corbic <ncorbic@sangoma.com>
*
* Copyright:	(c) 1995-2001 Sangoma Technologies Inc.
*
*		This program is free software; you can redistribute it and/or
*		modify it under the terms of the GNU General Public License
*		as published by the Free Software Foundation; either version
*		2 of the License, or (at your option) any later version.
* ============================================================================
* Description:
*
* 	The chdlc_api.c utility will bind to a socket to a chdlc network
* 	interface, and continously tx and rx packets to an from the sockets.
*
*	This example has been written for a single interface in mind, 
*	where the same process handles tx and rx data.
*
*	A real world example, should use different processes to handle
*	tx and rx spearately.  
*/



#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <sys/time.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <linux/if_wanpipe.h>
#include <string.h>
#include <signal.h>
#include <wait.h>
#include <errno.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <linux/wanpipe.h>
#include <linux/sdla_chdlc.h>

#define FALSE	0
#define TRUE	1

#define MAX_NUM_OF_TIMESLOTS  31

#define LGTH_CRC_BYTES	2
#define MAX_TX_DATA     1000 //MAX_NUM_OF_TIMESLOTS*10	/* Size of tx data */
#define MAX_TX_FRAMES 	1000000     /* Number of frames to transmit */  

#define WRITE 1
#define MAX_IF_NAME 20
typedef struct {

	int sock;
	int rx_cnt;
	int tx_cnt;
	int data;
	char if_name[MAX_IF_NAME+1];
	
} timeslot_t;


timeslot_t tslot_array[MAX_NUM_OF_TIMESLOTS];

int tx_change_data=0, tx_change_data_cnt=0;


/*===================================================
 * MakeConnection
 *
 *   o Create a Socket
 *   o Bind a socket to a wanpipe network interface
 *       (Interface name is supplied by the user)
 *==================================================*/         

int MakeConnection(timeslot_t *slot, char *router_name ) 
{
	struct wan_sockaddr_ll 	sa;

	memset(&sa,0,sizeof(struct wan_sockaddr_ll));
	errno = 0;
   	slot->sock = socket(AF_WANPIPE, SOCK_RAW, 0);
   	if( slot->sock < 0 ) {
      		perror("Socket");
      		return( FALSE );
   	} /* if */
  
	printf("\nConnecting to router %s, interface %s\n", router_name, slot->if_name);
	
	strcpy( sa.sll_device, slot->if_name);
	strcpy( sa.sll_card, router_name);
	sa.sll_protocol = htons(PVC_PROT);
	sa.sll_family=AF_WANPIPE;

        if(bind(slot->sock, (struct sockaddr *)&sa, sizeof(struct wan_sockaddr_ll)) < 0){
                perror("bind");
		printf("Failed to bind a socket to %s interface\n",slot->if_name);
                exit(0);
        }
	printf("Socket bound to %s\n\n",slot->if_name);

	return(TRUE);

}

#define ALL_OK  	0
#define	BOARD_UNDERRUN 	1
#define TX_ISR_UNDERRUN 2
#define UNKNOWN_ERROR 	4

void process_con_rx(timeslot_t *slot) 
{
	unsigned int Rx_count;
	fd_set 	ready;
	int err,serr,i;
	int stream_sync=0;
	unsigned char Rx_data[1600];
	int error_bit=0;
	int ok=0,brd=0,txisr=0,unknown=0;
	struct timeval tv;
	int idle_before_sync=0;
	int b4_change=0;
	
	tv.tv_usec = 0;
	tv.tv_sec = 10;
	
        Rx_count = 0;

	printf("%s: Rx Sock %i looking for 0x%x\n",slot->if_name,slot->sock,slot->data);

	for(;;) {	
		FD_ZERO(&ready);
		FD_SET(slot->sock,&ready);

		tv.tv_usec = 0;
		tv.tv_sec = 10;

		/* The select function must be used to implement flow control.
	         * WANPIPE socket will block the user if the socket cannot send
		 * or there is nothing to receive.
		 *
		 * By using the last socket file descriptor +1 select will wait
		 * for all active sockets.
		 */ 

		error_bit=0;
  		if((serr=select(slot->sock + 1,&ready, NULL, NULL, &tv))){

			/* Check for rx packets */
                   	if (FD_ISSET(slot->sock,&ready)){

				err = recv(slot->sock, Rx_data, sizeof(Rx_data), 0);

				/* err indicates bytes received */
				if(err > 0) {
	
					/* Rx packet recevied OK
					 * Each rx packet will contain 16 bytes of
					 * rx header, that must be removed.  The
					 * first byte of the 16 byte header will
					 * indicate an error condition.
					 */
					
	
					for (i=sizeof(api_tx_hdr_t);i<err;i++){
	
						idle_before_sync++;
						
						if (Rx_data[i] >= slot->data){

							if (tx_change_data){
								tx_change_data_cnt++;
								if (Rx_data[i]==slot->data){
									printf("Tx Change Data = %i\n",
											tx_change_data_cnt);
									tx_change_data=0;
									tx_change_data_cnt=0;
								}
							}
							
							if (!stream_sync){
								printf("\n\nGot Sync Offset=%i 0x%x IdleB4=%i\n\n",
									i-16,slot->data,idle_before_sync);
								stream_sync=1;
								error_bit=0;
								brd=0;
								txisr=0;
								unknown=0;
							}
							ok++;
#if 0
						}else if (Rx_data[i] == 0x02){

							printf("OUT OF SYNC Cnt=%i, Data=0x%x\n",
                                                                i-16,Rx_data[i]);

							error_bit |= BOARD_UNDERRUN;
							brd++;
						}else if (Rx_data[i] == 0x04){
							txisr++;
							error_bit |= TX_ISR_UNDERRUN;
#endif
						}else{
							if (stream_sync){
								printf("OUT OF SYNC Cnt=%i, Data=0x%x\n",
                                                                	i-sizeof(api_tx_hdr_t),Rx_data[i]);
								break;
							}else{
								//putchar('\r');
								//printf("OUT OF SYNC: 0x%x",Rx_data[i]);
							}

							unknown++;
							error_bit |= UNKNOWN_ERROR;
						}
					}

					if (stream_sync && error_bit){
						printf("\n%s: CRITICAL OUT OF SYNC RECEIVED :%s !\n\n",
								slot->if_name,
								((error_bit&TX_ISR_UNDERRUN) ? "TX UNDR" :
								 (error_bit&BOARD_UNDERRUN) ? "BRD UNDR" :
								 "UNK UNDR"
								 ));
						break;
					}

					if (stream_sync){
						putchar('\r');
						printf("ok=%i brd=%i tx=%i unk=%i len=%i data=0x%x",
								ok,brd,txisr,unknown,err,slot->data);
					}	
					fflush(stdout);
#if 0
					printf("%s: Received packet %i, length = %i : %s\n", 
							slot->if_name,++Rx_count, err,
							((error_bit == 0)? "USR DATA OK" :
							 (error_bit&BOARD_UNDERRUN)? "BRD_UNDR" : 
							 (error_bit&TX_ISR_UNDERRUN) ? "TX_UNDR" :
							 "UNKNOWN ERROR"));
#endif

				} else {
					//printf("\n%s: Error receiving data\n",slot->if_name);
				}

			 } 
		  
		} else {
			printf("\n%s: Error selecting rx socket rc=0x%x errno=0x%x\n",
					slot->if_name,serr,errno);
			perror("Select: ");
			//break;
		}
	}
} 

 
void process_con_tx(timeslot_t *slot) 
{
	unsigned int Tx_count,Tx_length;
	fd_set 	write;
	int err,i;
	unsigned char Tx_data[MAX_TX_DATA + sizeof(api_tx_hdr_t)];

	Tx_count = 0;
	Tx_length = MAX_TX_DATA;
	
	/* If running HDLC_STREAMING then the received CRC bytes
         * will be passed to the application as part of the 
         * received data.  The CRC bytes will be appended as the 
	 * last two bytes in the rx packet.
	 */

	memset(&Tx_data[0],0,MAX_TX_DATA + sizeof(api_tx_hdr_t));


	for (i=0;i<Tx_length;i++){
		Tx_data[i+sizeof(api_tx_hdr_t)] = slot->data;
	}

	/* If drivers has been configured in HDLC_STREAMING
	 * mode, the CRC bytes will be included into the 
	 * rx packet.  Thus, the application should remove
	 * the last two bytes of the frame. 
	 * 
	 * The no_CRC_bytes_Rx will indicate to the application
	 * how many bytes to cut from the end of the rx frame.
	 *
	 */

	printf("%s: Tx Starting to write on sock %i data (0x%X)  %x %x %x \n",
			slot->if_name,slot->sock,slot->data,Tx_data[16],Tx_data[17],Tx_data[18]);
	
	//pause();
	
	for(;;) {	
		FD_ZERO(&write);
		FD_SET(slot->sock,&write);

		/* The select function must be used to implement flow control.
	         * WANPIPE socket will block the user if the socket cannot send
		 * or there is nothing to receive.
		 *
		 * By using the last socket file descriptor +1 select will wait
		 * for all active sockets.
		 */ 

  		if(select(slot->sock + 1,NULL, &write, NULL, NULL)){

			/* Check if the socket is ready to tx data */
		   	if (FD_ISSET(slot->sock,&write)){

				err = send(slot->sock,Tx_data, Tx_length+sizeof(api_tx_hdr_t), 0);

				if (err > 0){
					/* Packet sent ok */
					//printf("\t\t%s:Packet sent: Len=%i Data=0x%x : %i\n",
					//		slot->if_name,err,slot->data,++Tx_count);
					//putchar('T');
					Tx_count++;
				}else{
					if (errno != EBUSY){
						printf("Errno = %i EBUSY=%i\n",errno,-EBUSY);
						perror("Send Failed: ");
						break;
					}
					/* The driver is busy,  we must requeue this
					 * tx packet, and try resending it again 
					 * later. */
				}
				
		   	}else{
				printf("Error Tx nothing IFFSET\n");
			}


			if (Tx_count > MAX_TX_FRAMES){
				break;
			}

			if (Tx_count % 500){
				slot->data=slot->data+1;
				for (i=0;i<Tx_length;i++){
					Tx_data[i+sizeof(api_tx_hdr_t)] = slot->data;
				}
				tx_change_data=1;
				tx_change_data_cnt=0;
			}
		} else {
                	printf("\nError selecting socket\n");
			break;
		}
	}
} 



/***************************************************************
 * Main:
 *
 *    o Make a socket connection to the driver.
 *    o Call process_con() to read/write the socket 
 *
 **************************************************************/


int main(int argc, char* argv[])
{
	int proceed;
	char router_name[20];	
	pid_t pid;
	int x,i;
	
	if (argc < 4  || argc%2){
		printf("Usage: rec_wan_sock <router name> <interface name> <data> ...\n");
		exit(0);
	}	

	memset(&tslot_array,0,sizeof(tslot_array));

	strncpy(router_name,argv[1],(sizeof(router_name)-1));
		
	for (x=0,i=2;i<argc;x++,i+=2){	

		strncpy(tslot_array[x].if_name, argv[i], MAX_IF_NAME);		
		sscanf(argv[i+1],"%x",&tslot_array[x].data);
		//tslot_array[x].data = atox(argv[i+1]);

#if 1		
		proceed = MakeConnection(&tslot_array[x],router_name);
		if( proceed == TRUE ){

			printf("Creating %s with tx data 0x%x : Sock=%i : x=%i\n",
				tslot_array[x].if_name,
				tslot_array[x].data,
				tslot_array[x].sock,
				x);
			
			if (!clone()){
				process_con_rx(&tslot_array[x]);
				exit(0);
			}
			if (!clone()){
				process_con_tx(&tslot_array[x]);
				exit(0);
			}
		}
#endif
	}

	for (x=0;x<MAX_NUM_OF_TIMESLOTS;x++){
		if (tslot_array[x].sock){
			close(tslot_array[x].sock);
			tslot_array[x].sock=0;
		}
	}
	
	/* Wait for the clear call children */
	pause();
	return 0;
};
