/*****************************************************************************
* chdlc_rec.c	CHDLC API: Receive Module
*
* Author(s):	Gideon Hack & Nenad Corbic <ncorbic@sangoma.com>
*
* Copyright:	(c) 1995-2000 Sangoma Technologies Inc.
*
*		This program is free software; you can redistribute it and/or
*		modify it under the terms of the GNU General Public License
*		as published by the Free Software Foundation; either version
*		2 of the License, or (at your option) any later version.
* ============================================================================
*/


#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
#include <unistd.h>
#include <sys/time.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <linux/if_wanpipe.h>
#include <string.h>
#include <errno.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <signal.h>
#include <linux/if.h>
#include <linux/wanpipe.h>
#include <linux/sdla_ss7.h>
#include <linux/ss7_linux.h>

#define FALSE	0
#define TRUE	1

#define MAX_TX_DATA     10	/* Size of tx data */  
#define MAX_FRAMES 	5000     /* Number of frames to transmit */  

unsigned short Rx_lgth;

unsigned char Rx_data[16000];
unsigned char Tx_data[MAX_TX_DATA + sizeof(api_tx_hdr_t)];


/* Prototypes */
int MakeConnection(void);
void handle_socket( void);
int init_args(int argc, char *argv[]);
void usage(void);
void prot_disconnect(void);
void sig_end(int sigid);

int 	sock;
char	read_enable=0;
char 	write_enable=0;
int 	tx_cnt=1;
int	tx_size=10;
int	tx_delay=0;
char	verbose=0;

unsigned char card_name[WAN_IFNAME_SZ];
unsigned char if_name[WAN_IFNAME_SZ];
/***************************************************
* MakeConnection
*
*   o Create a Socket
*   o Bind a socket to a wanpipe network interface
*       (Interface name is supplied by the user)
*/         

int MakeConnection(void) 
{
	struct wan_sockaddr_ll 	sa;

	memset(&sa,0,sizeof(struct wan_sockaddr_ll));
	errno = 0;
   	sock = socket(AF_WANPIPE, SOCK_RAW, 0);
   	if( sock < 0 ) {
      		perror("Socket");
      		return( FALSE );
   	} /* if */
  
	printf("\nConnecting to router %s, interface %s prot %x\n", card_name, if_name,htons(SS7_PROT));
	
	strcpy( sa.sll_device, if_name);
	strcpy( sa.sll_card, card_name);
	sa.sll_protocol = htons(SS7_PROT);
	sa.sll_family=AF_WANPIPE;
	
        if(bind(sock, (struct sockaddr *)&sa, sizeof(struct wan_sockaddr_ll)) < 0){
                perror("bind");
		printf("Failed to bind a socket to %s interface\n",if_name);
                exit(0);
        }

	printf("Socket bound to %s\n\n",if_name);
	{
		int err=0;
		wan_mbox_t mb;		
		
		memset (&mb,0,sizeof(mb));

		mb.wan_command = READ_SS7_CODE_VERSION;
		mb.wan_data_len = 0;

		err = ioctl(sock,SIOCS_GENERAL_CMD,&mb);

		if (err == 0){
			printf("Command Successful Len %i Data %s!!!\n",
					mb.wan_data_len,mb.wan_data);
		}else{
			printf("Command Failed %i !!!\n",err);
		}


		printf("Front end state is %s\n",
			(ioctl(sock,SIOCS_CHECK_FRONT_STATE,0) != 0) ? 
				"DISCONNECTED" : "CONNECTED");
	
		if (connect(sock,NULL,0) < 0){
			/* errno= EINVAL = protcol not SS7_PROT
			 *       EISCONN = socket already connected
			 *       0  = OK */
			printf( "Connect failed %i\n",errno);
			perror("Connect: ");
			exit(0);
		}
		
		memset (&mb,0,sizeof(mb));
			
		mb.wan_command = L2_START;
		mb.wan_data_len = 0;

		err = ioctl(sock,SIOCS_GENERAL_CMD,&mb);
	
		if (err == 0){
			printf("Command Successful Len %i Data %s Rc=%i\n",
					mb.wan_data_len,mb.wan_data,mb.wan_return_code);
		}else{
			if (err == L2_INVALID_STATE_FOR_CMND){
				printf("Command Successful Len %i Rc=Already Running!\n",
					mb.wan_data_len);

			}else{
				printf("Command 0x%02X Failed Rc=%x !!!\n",
					mb.wan_command,err);
			}
		}
	}
	
	return( TRUE );

}

void prot_disconnect(void)
{
	wan_mbox_t mb;

	memset (&mb,0,sizeof(mb));
		
	/* L2 STOP - Only use this to disconnect 
	 *           the SS7 Link ! */
#if 0
	printf("Disconnecting Protocol\n");

	mb.command = L2_STOP;
	mb.buffer_length = 0;

	ioctl(sock,SIOCS_GENERAL_CMD,&mb);
#endif
}

/***************************************************
* HANDLE SOCKET 
*
*   o Read a socket 
*   o Cast data received to api_rx_element_t data type 
*   o The received packet contains 16 bytes header 
*
*	------------------------------------------
*      |  16 bytes      |        X bytes        ...
*	------------------------------------------
* 	   Header              Data
*
*   o Data structures:
*
*	typedef struct {
*        	unsigned char   SIO      	PACKED;
*        	unsigned short  time_stamp      PACKED;
*        	unsigned char   reserved[13]    PACKED;
*	} api_rx_hdr_t; 
*
*	typedef struct {
*        	api_rx_hdr_t    api_rx_hdr      PACKED;
*        	void *          data            PACKED;
*	} api_rx_element_t;
*
*
*/

void handle_socket(void) 
{
	unsigned int Rx_count,Tx_count,Tx_length;
	api_rx_element_t* api_rx_el;
	api_tx_element_t * api_tx_el;
	fd_set 	ready,write,oob;
	int err,i;
	unsigned char *data;

        Rx_count = 0;
	Tx_count = 0;
	Tx_length = tx_size;

	printf("\n\nSocket Handler: Rx=%d Tx=%i TxCnt=%i TxLen=%i TxDelay=%i\n",
			read_enable,write_enable,tx_cnt,tx_size,tx_delay);	
	
	/* If running HDLC_STREAMING then the received CRC bytes
         * will be passed to the application as part of the 
         * received data.  
	 */

	memset(&Tx_data[0],0,MAX_TX_DATA + sizeof(api_tx_hdr_t));

	api_tx_el = (api_tx_element_t*)&Tx_data[0];
	data = (unsigned char *)&api_tx_el->data;

	for (i=0;i<Tx_length;i++){
		data[i] = (unsigned char)i;
	}
	api_tx_el->api_tx_hdr.SIO = 0x2A;

	for(;;) {	
		FD_ZERO(&ready);
		FD_ZERO(&write);
		FD_ZERO(&oob);
		FD_SET(sock,&oob);
	     	FD_SET(sock,&ready);

		if (write_enable){
		     FD_SET(sock,&write);
		}

		fflush(stdout);
  		if(select(sock + 1,&ready, &write, &oob, NULL)){
	
		   	if (FD_ISSET(sock,&oob)){
		
				wan_mbox_t *mb;	
				err = recv(sock, Rx_data, sizeof(Rx_data), MSG_OOB);

				if(err < 0 ) {
					printf("Failed to receive OOB %i , %i\n", Rx_count, err);
					err = ioctl(sock,SIOC_WANPIPE_SOCK_STATE,0);
					printf("Sock state is %s\n",
							(err == 0) ? "CONNECTED" : 
							(err == 1) ? "DISCONNECTED" :
							 "CONNECTING");
					break;
				}
					
				mb = (wan_mbox_t *)Rx_data;

				printf("GOT OOB EXCEPTION CMD 0x%x, Return Code 0x%x, Data Len %i\n",
						mb->wan_command, mb->wan_return_code, mb->wan_data_len);

				if (mb->wan_data_len){
					int i;
					printf("OOB DATA: ");
					for (i=0;i<mb->wan_data_len;i++){
						printf("0x%02X ",mb->wan_data[i]);
					}
					printf("\n");
				}
				
				/* There could be other exception conditions thus,
				 * to not try to send or receive until they are all
				 * handled */
				continue;
			}
		  
			
                   	if (FD_ISSET(sock,&ready)){

				err = recv(sock, Rx_data, sizeof(Rx_data), 0);

				if (!read_enable){
					goto ss7_skip_read;
				}
					
				/* err indicates bytes received */
				if(err > 0) {

					api_rx_el = (api_rx_element_t*)&Rx_data[0];

		                	/* Check the packet length */
                                	Rx_lgth = err - sizeof(api_rx_hdr_t);
                                	if(!Rx_lgth) {
                                        	printf("\nShort frame received (%d)\n",
                                                	Rx_lgth);
                                        	return;
                                	}

					if(verbose){	
						printf("Received MSU %i, length = %i, SIO = 0x%02X\n", 
							++Rx_count, Rx_lgth, 
							api_rx_el->api_rx_hdr.SIO);
						data=(unsigned char *)&api_rx_el->data;
						printf("Data: ");
						for(i=0;(i<Rx_lgth)&&(i<10); i ++) {
							printf("0x%02X ", *(data+i));
						} 
						printf("\n");
					}else{
						putchar('R');
					}

					send(sock,Rx_data, err, 0);

				} else {
					printf("\nError receiving data\n");
					break;
				}

		   	}
ss7_skip_read:	
		   	if (FD_ISSET(sock,&write)){

				err = send(sock,Tx_data, Tx_length + sizeof(api_tx_hdr_t), 0);

				if (err <= 0){
					if (errno == EBUSY){
						printf("Sock busy try again!\n");
						/* Socket busy try sending again !*/
					}else{
				 		/* Check socket state */
						err = ioctl(sock,SIOC_WANPIPE_SOCK_STATE,0);
						printf("Sock state is %s\n",
							(err == 0) ? "CONNECTED" : 
							(err == 1) ? "DISCONNECTED" :
							 "CONNECTING");

						printf("Faild to send %i \n",errno);
						perror("Send: ");
								
						break;
					}
				}else{
					printf("Packet sent: Sent %i : %i\n",
						err,++Tx_count);
				}
		   	}

			if (tx_delay){
				sleep(tx_delay);
			}

			if (tx_size && Tx_count == tx_cnt){
				break;
			}
		}
	}

	printf("Checking for TX\n");
	while (ioctl(sock,SIOC_WANPIPE_CHECK_TX,0) > 0);

	prot_disconnect();

	close (sock);
} 



/***************************************************************
 * Main:
 *
 *    o Make a socket connection to the driver.
 *    o Call handle_socket() to read the socket 
 *
 **************************************************************/


int main(int argc, char* argv[])
{
	int proceed;

	proceed=init_args(argc,argv);
	if (proceed != WAN_TRUE){
		usage();
		return -1;
	}

	signal(SIGINT,&sig_end);
	
	proceed = MakeConnection();
	if( proceed == WAN_TRUE){
		handle_socket();
		return 0;
	}

	return 0;
};


int init_args(int argc, char *argv[])
{
	int i;
	int i_cnt=0,c_cnt=0,card_cnt=0;


	for (i = 0; i < argc; i++){
		
		if (!strcmp(argv[i],"-i")){

			if (i+1 > argc-1){
				printf("ERROR: Invalid Interface Name!\n");
				return WAN_FALSE;
			}

			strncpy(if_name, argv[i+1],WAN_IFNAME_SZ);
			i_cnt=1;
			
		}else if (!strcmp(argv[i],"-c")){
			if (i+1 > argc-1){
				printf("ERROR: Invalid Card Name!\n");
				return WAN_FALSE;
			}
			
			strncpy(card_name, argv[i+1], WAN_IFNAME_SZ);
			card_cnt=1;
			
		}else if (!strcmp(argv[i],"-r")){
			read_enable=1;
			c_cnt=1;
	
		}else if (!strcmp(argv[i],"-w")){
			write_enable=1;
			c_cnt=1;

		}else if (!strcmp(argv[i],"-verbose")){
			verbose=1;
		
		}else if (!strcmp(argv[i],"-txcnt")){
			if (i+1 > argc-1){
				printf("ERROR: Invalid tx cnt!\n");
				return WAN_FALSE;
			}		

			if(isdigit(argv[i+1][0])){
				tx_cnt = atoi(argv[i+1]);
			}else{
				printf("ERROR: Invalid tx cnt Port!\n");
				return WAN_FALSE;
			}

		}else if (!strcmp(argv[i],"-txsize")){
			if (i+1 > argc-1){
				printf("ERROR: Invalid tx size!\n");
				return WAN_FALSE;
			}		

			if(isdigit(argv[i+1][0])){
				tx_size = atoi(argv[i+1]);
			}else{
				printf("ERROR: Invalid tx size, must be a digit!\n");
				return WAN_FALSE;
			}
		}else if (!strcmp(argv[i],"-txdelay")){
			if (i+1 > argc-1){
				printf("ERROR: Invalid tx delay!\n");
				return WAN_FALSE;
			}		

			if(isdigit(argv[i+1][0])){
				tx_delay = atoi(argv[i+1]);
			}else{
				printf("ERROR: Invalid tx delay, must be a digit!\n");
				return WAN_FALSE;
			}
		}
	}

	if (!i_cnt){
		printf("ERROR: No Interface Name!\n");
		return WAN_FALSE;
	}
	if (!card_cnt){
		printf("ERROR: No Card name!\n");
		return WAN_FALSE;
	}
	if (!c_cnt){
		printf("ERROR: No Read or Write Command!\n");
		return WAN_FALSE;
	}

	return WAN_TRUE;
}

void sig_end(int sigid)
{
	prot_disconnect();
}

void usage(void)
{
printf ("

SS7 API USAGE


	ss7_api <options> <extra options>

<options>:
	-i  <ifname>     #interface name
	-c  <card name>  #card name
	-r               #read enable
	-w               #write eable

<extra options>
	-txcnt   <digit>  #number of tx packets  (Dflt: 1)
	-txsize  <digit>  #tx packet size        (Dflt: 10)
	-txdealy <digit>  #delay in sec after each tx packet (Dflt: 0)

");

}
