/*****************************************************************************
* chdlc_rec.c	CHDLC API: Receive Module
*
* Author(s):	Gideon Hack & Nenad Corbic <ncorbic@sangoma.com>
*
* Copyright:	(c) 1995-2000 Sangoma Technologies Inc.
*
*		This program is free software; you can redistribute it and/or
*		modify it under the terms of the GNU General Public License
*		as published by the Free Software Foundation; either version
*		2 of the License, or (at your option) any later version.
* ============================================================================
*/


#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <sys/time.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <linux/if_wanpipe.h>
#include <string.h>
#include <errno.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <signal.h>
#include <linux/if.h>
#include <linux/wanpipe.h>
#include <linux/if_packet.h>
#include <linux/if_wanpipe.h>
#include <linux/if_ether.h>
#include <linux/sdla_ss7.h>
#include <linux/ss7_linux.h>

#define FALSE	0
#define TRUE	1

#define MAX_TX_DATA     10	/* Size of tx data */  
#define MAX_FRAMES 	100     /* Number of frames to transmit */  

#define WRITE 1
unsigned short Rx_lgth;

unsigned char Rx_data[16000];
unsigned char Tx_data[MAX_TX_DATA + sizeof(api_tx_hdr_t)];


/* Prototypes */
int MakeConnection(char *, char *);

char i_name[20];

int 	sock;
/***************************************************
* MakeConnection
*
*   o Create a Socket
*   o Bind a socket to a wanpipe network interface
*       (Interface name is supplied by the user)
*/         

int MakeConnection(char *r_name, char *i_name ) 
{
	struct sockaddr_ll soin;
	struct ifreq ifr;

	memset(&soin,0,sizeof(struct sockaddr_ll));
	errno = 0;
   	sock = socket(PF_PACKET, SOCK_RAW, 0);
   	if( sock < 0 ) {
      		perror("Socket");
      		return( FALSE );
   	} /* if */
  
	printf("\nConnecting to router %s, interface %s prot %x\n", r_name, i_name,htons(SS7_PROT));

	soin.sll_family = AF_PACKET;
	soin.sll_protocol = htons(ETH_P_IP);
	strcpy (ifr.ifr_name,i_name);
   	if (ioctl(sock,SIOCGIFINDEX,&ifr) <0){
		perror("Ioctl: ");
		return (FALSE);
	}
	soin.sll_ifindex = ifr.ifr_ifindex;

	if (bind(sock, (struct sockaddr *)&soin, sizeof(soin)) < 0) {
		perror("bind");
		printf("Failed to bind a socket to %s interface\n",i_name);
                exit(0);
   	} /* if */

	printf("Socket bound to %s\n\n",i_name);

	return TRUE;
}


int ss7_l2_connect (char *i_name){
	
	int err=0;
	struct ifreq ifr;
	SS7_MAILBOX_STRUCT mb;		
		
	memset (&mb,0,sizeof(mb));
	memset (&ifr,0,sizeof(ifr));

	mb.command = READ_SS7_CODE_VERSION;
	mb.buffer_length = 0;

	strcpy(ifr.ifr_name, i_name);
	ifr.ifr_data = (char*)&mb;

	err = ioctl(sock,SIOCS_GENERAL_CMD,&ifr);

	if (err == 0){
		printf("Command Successful Len %i Data %s!!!\n",
				mb.buffer_length,mb.data);
	}else{
		printf("Command Failed %i !!!\n",err);
	}


	printf("Front end state is %s\n",
		(ioctl(sock,SIOCS_CHECK_FRONT_STATE,0) != 0) ? 
			"DISCONNECTED" : "CONNECTED");

	memset (&mb,0,sizeof(mb));

stop_layer2:
	printf("Stopping Layer 2!\n");
	mb.command = L2_STOP;
	mb.buffer_length = 0;
	ioctl(sock,SIOCS_GENERAL_CMD,&ifr);
	
	
start_layer2:
	printf("Starting Layer 2!\n");
	mb.command = L2_START;
	mb.buffer_length = 0;

	err = ioctl(sock,SIOCS_GENERAL_CMD,&ifr);
	if (err == 0){
		printf("L2 Start Successful Len %i Data %s Rc=%i!!!\n",
				mb.buffer_length,mb.data,mb.return_code);
	}else{
		printf("L2 Start 0x%02X Failed Rc=%i !!!\n",
				mb.command,err);
		goto stop_layer2;
	}

	
	mb.command = L2_READ_LINK_STATUS;
	mb.buffer_length = 0;
	
	for (;;){

		mb.command = L2_READ_LINK_STATUS;
		mb.buffer_length = 0;
		err = ioctl(sock,SIOCS_GENERAL_CMD,&ifr);

		if (err==0){
			L2_LINK_STATUS_STRUCT *l2_status_str = (L2_LINK_STATUS_STRUCT *)mb.data;	
			
			switch (l2_status_str->L2_link_status){

				case L2_LINK_STAT_OUT_OF_SERVICE:
					printf("Link Out of Service!\n");
					goto start_layer2;

				case L2_LINK_STAT_IN_SERVICE:
					goto link_in_service_ok;

				default:
					break;
			}
		}else{
			perror("L2 Link Status Ioctl");
			return FALSE;
		}	
		sleep(5);
	}

link_in_service_ok:
	printf("L2 link status in service!\n");

	return( TRUE );
}

/***************************************************
* RECV_SOCKET 
*
*   o Read a socket 
*   o Cast data received to api_rx_element_t data type 
*   o The received packet contains 16 bytes header 
*
*	------------------------------------------
*      |  16 bytes      |        X bytes        ...
*	------------------------------------------
* 	   Header              Data
*
*   o Data structures:
*
*	typedef struct {
*        	unsigned char   SIO      	PACKED;
*        	unsigned short  time_stamp      PACKED;
*        	unsigned char   reserved[13]    PACKED;
*	} api_rx_hdr_t; 
*
*	typedef struct {
*        	api_rx_hdr_t    api_rx_hdr      PACKED;
*        	void *          data            PACKED;
*	} api_rx_element_t;
*
*
*/

void send_socket(char *i_name) 
{
	unsigned int Tx_count,Tx_length;
	api_tx_element_t * api_tx_el;
	int err,i;
	unsigned char *data;
	struct ifreq ifr;
	SS7_MAILBOX_STRUCT mb;		
		
	memset (&mb,0,sizeof(mb));
	memset (&ifr,0,sizeof(ifr));

	strcpy(ifr.ifr_name, i_name);
	ifr.ifr_data = (char*)&mb;
	
	Tx_count = 0;
	Tx_length = MAX_TX_DATA;
	
	for(;;) {	

		//--------------CHECK LINK STATUS----------------
		
		mb.command = L2_READ_LINK_STATUS;
		mb.buffer_length = 0;
		
		err = ioctl(sock,SIOCS_GENERAL_CMD,&ifr);	
		if (err==0){
			L2_LINK_STATUS_STRUCT *l2_status_str = (L2_LINK_STATUS_STRUCT *)mb.data;	
			
			switch (l2_status_str->L2_link_status){

				case L2_LINK_STAT_IN_SERVICE:
					break;

				default:
					printf("Link out of service\n");
					return;
			}
		}else{
			perror("L2 Link Status Ioctl");
			return;
		}	

		//-----------SEND-------------------------

		api_tx_el = (api_tx_element_t*)mb.data;
		data = (unsigned char *)&api_tx_el->data;
		for (i=0;i<Tx_length;i++){
			data[i] = (unsigned char)i;
		}
		api_tx_el->api_tx_hdr.SIO = 0x2A;
		mb.buffer_length=Tx_length+sizeof(api_tx_hdr_t);
	
		err = ioctl(sock,SIOC_SEND,&ifr);
		if (err == 0){
			printf("Tx Packet Sent OK %i\n",++Tx_count);
		}else{
			perror("TX: Ioctl");
		}	

		usleep(100);
	}
} 


void recv_socket(char *i_name) 
{
	unsigned int Rx_count;
	api_rx_element_t* api_rx_el;
	int err,i;
	char data_avail=0;
	unsigned char *data;
	struct ifreq ifr;
	SS7_MAILBOX_STRUCT mb;		
		
	memset (&mb,0,sizeof(mb));
	memset (&ifr,0,sizeof(ifr));

	strcpy(ifr.ifr_name, i_name);
	ifr.ifr_data = (char*)&mb;
	
        Rx_count = 0;
	
	for(;;) {	

		err = ioctl(sock,SIOC_RECEIVE_WAIT,&ifr);
		if (err > 0){

			Rx_lgth = mb.buffer_length;		
			api_rx_el = (api_rx_element_t*)mb.data;

			printf("Received MSU %i, length = %i, SIO = 0x%02X\n", 
						++Rx_count, Rx_lgth, 
						api_rx_el->api_rx_hdr.SIO);
	
			data=(unsigned char *)&api_rx_el->data;
			printf("Data: ");
			for(i=0;(i<Rx_lgth)&&(i<10); i ++) {
				printf("0x%02X ", *(data+i));
			} 
			printf("\n");
			data_avail=1;
		}else{
			data_avail=0;
			perror("RX: Ioctl");
		}
	}
} 


void sig_handler(int sigint)
{
	struct ifreq ifr;
	SS7_MAILBOX_STRUCT mb;		
		
	memset (&mb,0,sizeof(mb));
	memset (&ifr,0,sizeof(ifr));

	strcpy(ifr.ifr_name, i_name);
	ifr.ifr_data = (char*)&mb;
	
	mb.command = L2_STOP;
	mb.buffer_length = 0;

	printf("Singal Closing Connecting!\n");
	ioctl(sock,SIOCS_GENERAL_CMD,&ifr);

	close(sock);
	exit(1);
}

/***************************************************************
 * Main:
 *
 *    o Make a socket connection to the driver.
 *    o Call recv_socket() to read the socket 
 *
 **************************************************************/


int main(int argc, char* argv[])
{
	int proceed;
	pid_t pid;
	
	if (argc != 3){
		printf("Usage: rec_wan_sock <router name> <interface name>\n");
		exit(0);
	}	

	signal(SIGINT,&sig_handler);
	signal(SIGTERM,&sig_handler);
	
	strncpy(i_name,argv[argc - 1],20);
	
	proceed = MakeConnection(argv[argc - 2], i_name);
	if( proceed == TRUE ){
try_again:
		while (ss7_l2_connect(i_name) != TRUE){
			sleep(5);
		}

		if (!(pid=fork())){
			recv_socket(i_name);
			exit(0);
		}
		
		send_socket(i_name);
		kill(pid,SIGTERM);
		goto try_again;
		return 0;
	}

	return 0;
};
