/*****************************************************************************
* wpbackupd.c   Wanpipe Dailup-Backup Daemon
*
* Author:       Nenad Corbic <ncorbic@sangoma.com>	
*
* Copyright:	(c) 2000-2001 Sangoma Technologies Inc.
*
*		This program is free software; you can redistribute it and/or
*		modify it under the terms of the GNU General Public License
*		as published by the Free Software Foundation; either version
*		2 of the License, or (at your option) any later version.
* ----------------------------------------------------------------------------
* Jan 11, 2000	Nenad Corbic	Initial version
*****************************************************************************/

#include <stdio.h>                        //printf()
#include <stdlib.h>
#include <sys/types.h>                    //open()
#include <sys/stat.h>                     //open()
#include <fcntl.h>                        //open()
#include <sys/ioctl.h>                    //ioctl()
#include <errno.h>                        //errno()
#include <string.h>
#include <signal.h>
#include <sys/wait.h>
#include <unistd.h>
#include <signal.h>
#include <errno.h>
#include <syslog.h>
#include <sys/time.h>

#include <sys/socket.h>
#include <netinet/in.h>
#include <net/if.h>
#include <sys/ioctl.h>
#include <arpa/inet.h>
#include <linux/if_packet.h>
#include <linux/if_ether.h>
#include <linux/wanpipe.h>

#if defined (LINUX_2_4) || defined (LINUX_2_1)

#define MAX_TOKENS 32
#define KBD_NUM 'n'
#define KBD_CAP 'c'
#define KBD_SCR 's'

#define KERNEL_SYNCPPP_PROTOCOL "MPPP"
#define STATUS_TOKEN_NUM 8
#define NAME_OFF 0
#define PROT_OFF 1


static void cleanup(void);
static char child_exit=0;
char device[15];
char pri_if[15];
char state[50];
void (*get_wanpipe_state)(char *device_name, char *state);

static pid_t pid=0;

void sig_handler(int signum)
{
	unsigned int stat;
	if (pid){
		if (!kill(pid,SIGUSR1)){
			while ((pid=waitpid(pid,&stat,WUNTRACED)) > 0){
				//printf("Child %d terminated\n",pid);
			}
		}
	}
	cleanup();
	exit(0);
}

void child_handler(int signum)
{
	int stat;
	pid_t lpid=0;

	child_exit=1;
	
	if (!kill(lpid,SIGTERM)){
		while ((lpid=waitpid(lpid,&stat,WUNTRACED)) > 0){
			//printf("Child CHILD %d terminated\n",lpid);
		}
	}
}

void sig_fake(int signum)
{
	return;
}

int remove_default_route(char *ifname)
{
	char cmd[50];
	FILE* pipe_fd;

	sprintf(cmd,"route del -net 0 %s 2> /dev/null",ifname);
	
	pipe_fd = popen(cmd, "r");
	if (pipe_fd == NULL)
		return -1;

	pclose(pipe_fd);
	return 0;
}

int add_default_route(char *ifname)
{
	char cmd[50];
	FILE *pipe_fd;
	
	sprintf(cmd,"route add -net 0 %s 2> /dev/null",ifname);
	
	pipe_fd = popen(cmd, "r");
	if (pipe_fd == NULL)
		return -1;

	pclose(pipe_fd);
	return 0;
}

/* NENAD
 * I used select instead of sleep() because 
 * alarm() and sleep() are not compatible.
 * The select is a much better way of 
 * introducing a delay. */

void delay_time (int sdelay, int udelay)
{
	struct timeval to;
	
	to.tv_sec = sdelay;
	to.tv_usec = udelay;
	
	select(0,NULL,NULL,NULL,&to);
}

int sock;
/*============================================================================
 * Strip leading and trailing spaces off the string str.
 */
char* strstrip (char* str, char* s)
{
	char* eos = str + strlen(str);		/* -> end of string */

	while (*str && strchr(s, *str))
		++str;				/* strip leading spaces */
	while ((eos > str) && strchr(s, *(eos - 1)))
		--eos;				/* strip trailing spaces */
	*eos = '\0';
	return str;
}


/*============================================================================
 * Tokenize string.
 *	Parse a string of the following syntax:
 *		<tag>=<arg1>,<arg2>,...
 *	and fill array of tokens with pointers to string elements.
 *
 *	Return number of tokens.
 */
int tokenize (char* str, char **tokens, char *arg1, char *arg2, char *arg3)
{
	int cnt = 0;

	tokens[0] = strtok(str, arg1);
	while (tokens[cnt] && (cnt < MAX_TOKENS - 1)) {
		tokens[cnt] = strstrip(tokens[cnt], arg2);
		tokens[++cnt] = strtok(NULL, arg3);
	}
	return cnt;
}
/*============================================================================
 * get_wanpipe_state
 *
 *	Obtain and return the state of the device_name
 *		
 * 	The state is obtained from /proc/net/wanrouter/status file.
 */

void get_mppp_wanpipe_state (char *device_name, char *state)
{
	FILE *pipe_fd;
	char line[100];
	int i,found=0;
	
	strcpy(state,"unconfigured");
	
	pipe_fd = popen("cat /proc/net/syncppp", "r");
	if (pipe_fd == NULL)
		return;

	strcpy(state,"down");
	
	i=0;
	while (fgets(line,sizeof(line)-1,pipe_fd)){
		if (strstr(line,device_name) != NULL){
			found = 1;
			break;
		}
	}

	pclose(pipe_fd);

	if (!found){
		return;
	}

	if (strstr(line,"up") != NULL){
		strcpy(state,"connected");
	}
	
	return;
}


void get_gen_wanpipe_state (char *device_name, char *state)
{
	char router_dir[] = "/proc/net/wanrouter";	
	FILE *pipe_fd;
	char line[100];
	int i,len,found=0;
	int toknum;
	char* token[MAX_TOKENS];
	struct stat file_stat;
	
	strcpy(state,"unconfigured");
	
	if (stat(router_dir, &file_stat) < 0)
		return;
	
	pipe_fd = popen("cat /proc/net/wanrouter/status", "r");
	if (pipe_fd == NULL)
		return;

	i=0;
	while (fgets(line,sizeof(line)-1,pipe_fd)){
		if (strstr(line,device_name) != NULL){
			found = 1;
			break;
		}
	}

	pclose(pipe_fd);

	if (!found){
		return;
	}

	len = strlen(line) + 1;
	toknum = tokenize(line, token, "|", " \t\n", "|");

	if (toknum <= 8)
		return;

	if (strcmp(token[0],device_name))
		return;

	strcpy(state,token[8]);
	
	return;
}

int check_wan_protocol (char *device)
{
	
	char router_dir[] = "/proc/net/wanrouter";	
	FILE *pipe_fd;
	char line[100];
	int i,len,found=0;
	int toknum;
	char* token[MAX_TOKENS];
	struct stat file_stat;
	
	if (stat(router_dir, &file_stat) < 0)
		return -EINVAL;
	
	pipe_fd = popen("cat /proc/net/wanrouter/status", "r");
	if (pipe_fd == NULL)
		return -EINVAL;

	i=0;
	while (fgets(line,sizeof(line)-1,pipe_fd)){
		if (strstr(line,device) != NULL){
			found = 1;
			break;
		}
	}

	pclose(pipe_fd);

	if (!found){
		return -EINVAL;
	}

	len = strlen(line) + 1;
	toknum = tokenize(line, token, "|", " \t\n", "|");

	if (toknum <= STATUS_TOKEN_NUM)
		return -EINVAL;

	if (strcmp(token[NAME_OFF],device))
		return -EINVAL;

	if (!strcmp(token[PROT_OFF],KERNEL_SYNCPPP_PROTOCOL)){
		printf("CHECKING MPPP PROTOCOL\n");
		get_wanpipe_state = &get_mppp_wanpipe_state;
	}else{
		printf("CHECKING GENERAL PPP PROTOCOL\n");
		get_wanpipe_state = &get_gen_wanpipe_state;
	}
	
	return 0;
}

/*============================================================================
 * check_and_set_lock_file
 *
 * 	Initialize the lock file, so we don't have multiple
 * 	daemons running.  
 * 	If a lock exists exit the daemon with an error message.
 */

int check_and_set_lock_file(void)
{
	char wanpipe_dir[] = "/usr/local/wanrouter";
	char lock_file[] = "/usr/local/wanrouter/wpkbd_lock";
	struct stat file_stat;
	
	if (stat(wanpipe_dir, &file_stat) < 0){
		printf("wpbackupd: Error, WANPIPE utilites not installed\n");
		printf("wpbackupd: Directory not found: %s\n",wanpipe_dir);
		return 1;
	}
	
	if (stat(lock_file, &file_stat) < 0){
		system ("touch /usr/local/wanrouter/wpkbd_lock");
	}else{
		printf("\nwpbackupd: Error, Wanpipe backup daemon already running!\n");
 		printf("                 The /usr/local/wanrouter/wpkbd_lock exists.\n\n");
		return 1;
	}

	return 0;
}

/*============================================================================
 * poll_primary_link
 *
 * 	Establish a socket connection to the primary link and
 * 	monitor the input data.  If we detect PPP requests, it
 * 	means that the PPP line is back up and we should restart
 * 	the link and turn off dailup connection.
 *
 * 	NOTE: this funciton is called when the primary link goes
 * 	      down. 
 */


int poll_primary_link(void)
{
	int stat;
	
	for(;;) {	
		get_wanpipe_state(pri_if, state);

		if (!strcmp(state,"connected")){
			break;
		}
		delay_time(1,0);
	}

	//Remove Dialup Route !!! 
	remove_default_route("ppp0");
	add_default_route(pri_if);
	
	syslog(LOG_INFO, "Shutting down backup PPP dailup link!\n");
	if (!kill(pid,SIGUSR1)){
		while ((waitpid(pid,&stat,WUNTRACED)) > 0){
			//printf("Child %d terminated\n",pid);
		}
	}
	pid=0;
	return 0;
}


/*============================================================================
 * cleanup
 *
 * 	Remove the lock file and exit
 */


void cleanup (void)
{
	system("rm -f /usr/local/wanrouter/wpkbd_lock");
	syslog(LOG_INFO, "Exit.\n");
}

/*============================================================================
 * main
 *
 * 	o Monitor the Primary PPP high speed connection.
 * 	o If the connecting goes down, shutdown PPP protocol and
 * 	  restart the SREAMING HDLC protocol on the Primary port
 * 	o and ASYNC TTY protocol on the secondary port.
 * 	o The primary port is used to monitor the sycn line for
 * 	  signs of life.
 * 	o Start the PPPD using the WANPIPE TTY driver.
 * 	o If the Primary link comes back to life, shutdown PPPD and
 * 	  restart the Primary link.
 */


int main (int argc, char **argv)
{
	int update_interval=5;
	char pppd_cmd[50];
	char filename[100];
	int errno;
	FILE *fp;
	
	struct sigaction sa;
	
	//General Error Message
	const char *help =
	      "\n  wpbackupd v1.0 - (c)2000 Nenad Corbic <ncorbic@sangoma.com>\n\n"
                "  This program is distributed under the terms of GPL.\n\n"
		"  Description:\n\n"
		"    WANPIPE Backup Daemon utility is used to monitor the PPP Prim. HIGH\n"
		"    speed link.  If the Primary link goes down, the wpbackupd will\n"
		"    establish a dialup connection using the PPPD daemon.  At the \n"
		"    same time the primary link will be monitored for signs of life.\n"
		"    If the PPP protocol is detected on the primary link, the dialup\n"
		"    connection will be terminated and the Primary High speed connection\n"
		"    will be re-established.\n\n"
		"    IMPORTANT:\n"
		"       Run /usr/local/wanrouter/backupd/configure script before\n"
		"       starting wpbackupd.  The configure script will setup the\n"
		"       wpbackupd working environment.\n"
		"       Please read the /usr/local/wanrouter/backupd/README file.\n\n"
		
                "  Usage:\n"
                "  wpbackupd <wanpipe device> <interface name> <pppd call script> [update_interval]\n\n\n"
                "            <wanpipe device>  = wanpipe device to mointor\n"
		"                                (ex wanpipe1).\n"
		"            <interface name>  = wanpipe network interface to monitor\n"
		"                                (ex. wp1_fr16)\n"
		"            <pppd call script>= name of the pppd call script in /etc/ppp/peers\n"
		"                                directory. ex: isp_wanpipe4\n"
		"            [update interval] = 1-60 seconds , optional (default 5 sec)\n\n"
		"  ex: wpbakcupd wanpipe1 wp1_ppp isp_wanpipe2 5\n\n"
		"      Monitor interface wp1_ppp on device wanpipe1 every 5 seconds.\n"
		"      If the connection is lost, run 'pppd call isp_wanpipe2' command\n"
		"      where isp_wanpipe2 is a pppd call script in /etc/ppp/peers directory.\n\n";

	openlog(argv[0], LOG_PID | LOG_NDELAY, LOG_DAEMON);
	
	if (argc < 4){
		printf("\nError in usage: Arguments missing\n");
		printf("%s",help);
		exit(1);
	}

	if (argc == 5){
		update_interval = atoi(argv[4]);
		if (update_interval < 1 || update_interval > 60){
			printf("Error: Update interval out of range!\n");	
			printf("%s",help);
			exit(1);
		}
	}

	/* Setup a signal handler for SIGINT
	 * SIGINT (Ctrl C) will be used to terminate the
	 * wpbackupd program */
	memset(&sa,0,sizeof(sa));
	sa.sa_handler=sig_handler;
	
	if (sigaction(SIGINT,&sa,NULL)){
		perror("sigaction");
	}

	/* The SIGTERM signal will not be allowed to 
	 * terminate the main wpbackupd process, however
	 * all the children will DIE. 
	 *
	 * This is a nice way of cleaning up all the 
	 * childern that have been spwaned during, the cross-over
	 * process */
	signal(SIGTERM,sig_fake);
	

	/* Obtain the primary device name and interface name */
	strncpy(device,argv[1],sizeof(device));
	strncpy(pri_if,argv[2],sizeof(pri_if));
	
	/* The third argument is the pppd call script,
	 * we must check for its size and whether it is
	 * located in /etc/ppp directory */
	if (strlen(argv[3]) > 20){
		printf("Error: PPPD Call Script name too long! Max=20char\n");
		exit(1);
	}

	/* Initialize PPPD configuration file name */
	sprintf(filename,"/etc/ppp/peers/%s",argv[3]);
	
	/* Check that pppd configuration files exist */
	fp=fopen(filename, "r");
	if (fp==NULL){
		printf("Error: PPPD Call script /etc/ppp/peers/%s not found!\n",
				argv[3]);
		exit(1);
	}else{
		fclose(fp);
	}
		
	/* Initialize the pppd call command */
	sprintf(pppd_cmd,"exec pppd call %s > /dev/null",argv[3]);
	
	if (check_wan_protocol(device) != 0){
		printf("Error: Failed to determine WAN Protocol from: /proc/net/wanrouter/status !\n");
		exit(1);
	}

	if (check_and_set_lock_file()){
		exit(1);
	}

	/* Just in case the route is not there */
	add_default_route(pri_if);

	syslog(LOG_INFO, 
		"Starting Wanpipe Backup Daemon!\n");
	syslog(LOG_INFO,
		"Device: %s Interface: %s\n",
		  argv[1],argv[2]);

	syslog(LOG_INFO,
		"Update Time Interval: %i sec\n",update_interval);
	
	for (;;){
	
		get_wanpipe_state(pri_if, state);

		if (!strcmp(state,"unconfigured")){
			syslog(LOG_WARNING,
			 "Device %s is down, exiting debugger!\n",
			   device);
			goto wpkbd_exit;
			
		}else if (strcmp(state, "connected")){

			syslog(LOG_WARNING, "Primary PPP: DOWN!");
			syslog(LOG_WARNING, "Switching to backup dialup PPP!");
		
			//Remove Primary Link Route
			remove_default_route(pri_if);
			
			if ((pid=fork())==0){
				signal(SIGUSR1,child_handler);
				for (;;){
					syslog(LOG_INFO, "Starting Async Dialup PPP...\n");
					system(pppd_cmd);
					if (child_exit)
						break;
					delay_time(1,0);
					syslog(LOG_INFO, "Re-Starting Async Dialup PPP\n");
				}
				syslog(LOG_INFO,"Backup Dialup PPP Link is down!\n");
				exit(0);
			}
			
			poll_primary_link();
			
			syslog(LOG_INFO, "Primary Link re-starting!\n");
		}
		delay_time(update_interval,0);
	}

wpkbd_exit:
	
	cleanup();
	return 0;
}
























#else  /* LINUX_2_4 LINUX_2_1 */ 

void main()
{
	printf("\nERROR: This program is not supported on 2.0.X kernels\n");
	exit(0);
}

#endif





