#include <string.h>
#include <vector.h>
#include <stdio.h>
#include <iostream.h>
#include <errno.h>

extern "C" {
	#include <sys/stat.h>                     //open()
}

#define UPDATE_INTERVAL 5
#define MAX_PRIMARY_DEVICES 10
#define IP_ADDR_SIZE 10
#define MAX_TOKENS 32

//**************************************************
//GOBAL DEFINES
//**************************************************
enum state_t{ DISCONNECTED, CONNECTED };
enum connection_type_t { SINGLE, BALANCED, BOD };

//**************************************************
//FUNCTION PROTOTYPES
//**************************************************
int check_wan_protocol (char *device);
void get_gen_wanpipe_state (char *device_name, char *state);
void get_mppp_wanpipe_state (char *device_name, char *state);

//**************************************************
//DEVICE CLASS
//
//**************************************************
class Device {

public:
	//Constructors, Destructors
//	Device (char *device, char *intr_name);
	Device ();
	~Device();

	//Modifiers
	int up();
	int down();

	virtual int link_chk();
	int ping_chk(char* ptp_ip);
	int bod_chk();
	virtual void print();
	
	void set_state(state_t state);
	state_t get_state(void);
	
	char* strstrip (char* str, char* s);
	int tokenize (char* str, char **tokens, char *arg1, char *arg2, char *arg3);

protected:
	//Variables
	char *dev_name;
	char *if_name;
	
	unsigned char local_ip[IP_ADDR_SIZE];
	unsigned char remote_ip[IP_ADDR_SIZE];
	
	char default_route;
	char gw_ipaddr[IP_ADDR_SIZE];
	
	state_t state;
	
	char *up_exec_script;
	char *down_exec_script;
};

//**************************************************
//Device Class, Constructors, Desctructors and
//              Modifiers
//**************************************************

#if 0
Device::Device(char *device, char*intr_name)
{
	int ssize;
	ssize = strlen(device)+1;
	dev_name = new char [ssize];
	strcpy (dev_name,device);

	ssize = strlen(intr_name)+1;
	if_name = new char [ssize];
	strcpy (if_name,intr_name);

	state = DISCONNECTED;
	up_exec_script=NULL;
	down_exec_script=NULL;
}
#endif

Device::Device()
{
	dev_name=NULL;
	if_name=NULL;
	state = DISCONNECTED;
	up_exec_script=NULL;
	down_exec_script=NULL;
}

Device::~Device()
{
	if (dev_name)
		delete dev_name;
	if (if_name)
		delete if_name;
}

void Device::print(void)
{
	//cout << "Primary Device: " << dev_name 	<< " Secondary Device: " << if_name << "\n"; 
	printf("Primary Device %s, Secondary Device %s\n",dev_name,if_name);
}

void Device::set_state(state_t state_tmp)
{
	state = state_tmp;
}

state_t Device::get_state(void)
{
	return state;
}



//********************************************************
// Strip leading and trailing spaces off the string str.
//********************************************************
char* Device::strstrip (char* str, char* s)
{
	char* eos = str + strlen(str);		/* -> end of string */

	while (*str && strchr(s, *str))
		++str;				/* strip leading spaces */
	while ((eos > str) && strchr(s, *(eos - 1)))
		--eos;				/* strip trailing spaces */
	*eos = '\0';
	return str;
}


//********************************************************
// Tokenize string.
//	Parse a string of the following syntax:
// 		<tag>=<arg1>,<arg2>,...
// 	and fill array of tokens with pointers to string elements.
// 
// 	Return number of tokens.
//********************************************************

int Device::tokenize (char* str, char **tokens, char *arg1, char *arg2, char *arg3)
{
	int cnt = 0;

	tokens[0] = strtok(str, arg1);
	while (tokens[cnt] && (cnt < MAX_TOKENS - 1)) {
		tokens[cnt] = strstrip(tokens[cnt], arg2);
		tokens[++cnt] = strtok(NULL, arg3);
	}
	return cnt;
}


#if 0

//**************************************************
//UTILITY CLASS
//**************************************************

class Utility{

public:
	Utility() {}
	~Utility() {}
			
	char* strstrip (char* str, char* s);
	int tokenize (char* str, char **tokens, char *arg1, char *arg2, char *arg3);
};

//********************************************************
// Strip leading and trailing spaces off the string str.
//********************************************************
char* Utility::strstrip (char* str, char* s)
{
	char* eos = str + strlen(str);		/* -> end of string */

	while (*str && strchr(s, *str))
		++str;				/* strip leading spaces */
	while ((eos > str) && strchr(s, *(eos - 1)))
		--eos;				/* strip trailing spaces */
	*eos = '\0';
	return str;
}


//********************************************************
// Tokenize string.
//	Parse a string of the following syntax:
// 		<tag>=<arg1>,<arg2>,...
// 	and fill array of tokens with pointers to string elements.
// 
// 	Return number of tokens.
//********************************************************

int Utility::tokenize (char* str, char **tokens, char *arg1, char *arg2, char *arg3)
{
	int cnt = 0;

	tokens[0] = strtok(str, arg1);
	while (tokens[cnt] && (cnt < MAX_TOKENS - 1)) {
		tokens[cnt] = strstrip(tokens[cnt], arg2);
		tokens[++cnt] = strtok(NULL, arg3);
	}
	return cnt;
}

#endif

//**************************************************
//SYNCPPP PROTOCOL CLASS
//**************************************************

class SyncPPP_Prot : public Device {

public:
	SyncPPP_Prot(char *device, char*intr_name);
	~SyncPPP_Prot();
	
	int link_chk();
};

SyncPPP_Prot::SyncPPP_Prot(char *device, char*intr_name)
{
}
SyncPPP_Prot::~SyncPPP_Prot()
{
}

int SyncPPP_Prot::link_chk()
{
	FILE *pipe_fd;
	char line[100];
	int i,found=0;
	
	pipe_fd = popen("cat /proc/net/syncppp", "r");
	if (pipe_fd == NULL)
		return -EINVAL;

	i=0;
	while (fgets(line,sizeof(line)-1,pipe_fd)){
		if (strstr(line,if_name) != NULL){
			found = 1;
			break;
		}
	}

	pclose(pipe_fd);

	if (!found){
		return -EINVAL;
	}

	if (strstr(line,"up") != NULL){
		set_state(CONNECTED);
	}else{
		set_state(DISCONNECTED);
	}
	
	return get_state();
}

//**************************************************
//WANPIPE PROTOCOL CLASS
//
//**************************************************

class Wanpipe_Prot : public Device {

public:
	Wanpipe_Prot() {}
	~Wanpipe_Prot() {}
	
	int link_chk();
};

int Wanpipe_Prot::link_chk()
{
	char router_dir[] = "/proc/net/wanrouter";	
	FILE *pipe_fd;
	char line[100];
	int i,len,found=0;
	int toknum;
	char* token[MAX_TOKENS];
	struct stat file_stat;
	
	if (stat(router_dir, &file_stat) < 0)
		return -EINVAL;
	
	pipe_fd = popen("cat /proc/net/wanrouter/status", "r");
	if (pipe_fd == NULL)
		return -EINVAL;

	i=0;
	while (fgets(line,sizeof(line)-1,pipe_fd)){
		if (strstr(line,dev_name) != NULL){
			found = 1;
			break;
		}
	}

	pclose(pipe_fd);

	if (!found){
		return -EINVAL;
	}

	len = strlen(line) + 1;
	toknum = tokenize(line, token, "|", " \t\n", "|");

	if (toknum <= 8)
		return -EINVAL;

	if (strcmp(token[0],dev_name))
		return -EINVAL;

	if (!strcmp(token[8],"connected")){
		set_state(CONNECTED);
	}else{
		set_state(DISCONNECTED);
	}
	
	return get_state();
}

//**************************************************
//CONNECTION CLASS
//
//**************************************************
class Connection {

public:
	//Constructor, Destructor
	Connection (connection_type_t c_type, int devnum);
	~Connection();

	//Modifiers
	int attach_device(Device* dev);
	int update_state(void);
	void print(void);

	//Variables
	vector<Device*>dev_list;
	
	state_t state;
	connection_type_t type;

	unsigned int link_chk_frq;
	unsigned int ping_chk_frq;
	char *ping_chk_ip[IP_ADDR_SIZE];	
};

Connection::Connection(connection_type_t c_type,int devnum)
{
	type=c_type;
	dev_list.reserve(devnum);
}

Connection::~Connection()
{
	dev_list.clear();
}

int Connection::attach_device(Device* dev_ptr)
{
	dev_list.push_back(dev_ptr);
	return 0;
}

void Connection::print(void)
{
	vector<Device*>::iterator p;
	
	for (p = dev_list.begin(); p!=dev_list.end(); ++p)
		(*p)->print();
}


//*****************************************************************
// MAIN
//*****************************************************************

int main (void)
{
	Wanpipe_Prot *dev1;
	dev1 = new Wanpipe_Prot();
	SyncPPP_Prot *dev2;
	dev2 = new SyncPPP_Prot("wanpipe2", "wp2_ppp");
	
	Connection *primary;
	primary = new Connection(SINGLE, 2);

	printf ("Primary Link State %i, Secondary Link State %i\n",
		dev1->link_chk(), dev2->link_chk());

	primary->attach_device(dev1);
	primary->attach_device(dev2);
	primary->print();

	delete dev1;
	delete dev2;
	delete primary;
}
