/*****************************************************************************
* wpbackupd.c   Wanpipe Dailup-Backup Daemon
*
* Author:       Nenad Corbic <ncorbic@sangoma.com>	
*
* Copyright:	(c) 2000-2001 Sangoma Technologies Inc.
*
*		This program is free software; you can redistribute it and/or
*		modify it under the terms of the GNU General Public License
*		as published by the Free Software Foundation; either version
*		2 of the License, or (at your option) any later version.
* ----------------------------------------------------------------------------
* Jan 11, 2000	Nenad Corbic	Initial version
*****************************************************************************/

#include <stdio.h>                        //printf()
#include <stdlib.h>
#include <sys/types.h>                    //open()
#include <sys/stat.h>                     //open()
#include <fcntl.h>                        //open()
#include <sys/ioctl.h>                    //ioctl()
#include <errno.h>                        //errno()
#include <string.h>
#include <signal.h>
#include <sys/wait.h>
#include <unistd.h>
#include <signal.h>
#include <errno.h>
#include <syslog.h>
#include <sys/time.h>

#include <sys/socket.h>
#include <netinet/in.h>
#include <net/if.h>
#include <sys/ioctl.h>
#include <arpa/inet.h>
#include <linux/if_packet.h>
#include <linux/if_ether.h>
#include <linux/wanpipe.h>

#if defined (LINUX_2_4) || defined (LINUX_2_1)

#define MAX_TOKENS 32
#define KBD_NUM 'n'
#define KBD_CAP 'c'
#define KBD_SCR 's'

#define KERNEL_SYNCPPP_PROTOCOL "MPPP"
#define STATUS_TOKEN_NUM 8
#define NAME_OFF 0
#define PROT_OFF 1

#define CONN_OFF 7
#define DEV_OFF 0

static void cleanup(void);
char device[15];
char state[50];

void sig_handler(int signum)
{
	cleanup();
	exit(0);
}

/* NENAD
 * I used select instead of sleep() because 
 * alarm() and sleep() are not compatible.
 * The select is a much better way of 
 * introducing a delay. */

void delay_time (int sdelay, int udelay)
{
	struct timeval to;
	
	to.tv_sec = sdelay;
	to.tv_usec = udelay;
	
	select(0,NULL,NULL,NULL,&to);
}

int sock;
/*============================================================================
 * Strip leading and trailing spaces off the string str.
 */
char* strstrip (char* str, char* s)
{
	char* eos = str + strlen(str);		/* -> end of string */

	while (*str && strchr(s, *str))
		++str;				/* strip leading spaces */
	while ((eos > str) && strchr(s, *(eos - 1)))
		--eos;				/* strip trailing spaces */
	*eos = '\0';
	return str;
}


/*============================================================================
 * Tokenize string.
 *	Parse a string of the following syntax:
 *		<tag>=<arg1>,<arg2>,...
 *	and fill array of tokens with pointers to string elements.
 *
 *	Return number of tokens.
 */
int tokenize (char* str, char **tokens, char *arg1, char *arg2, char *arg3)
{
	int cnt = 0;

	tokens[0] = strtok(str, arg1);
	while (tokens[cnt] && (cnt < MAX_TOKENS - 1)) {
		tokens[cnt] = strstrip(tokens[cnt], arg2);
		tokens[++cnt] = strtok(NULL, arg3);
	}
	return cnt;
}


void get_ping_state (char *cmd)
{
	system("exec cmd 2> /dev/null > /dev/null");
}

void get_wanpipe_state (char *device_name, char *state)
{
	char router_dir[] = "/proc/net/wanrouter";	
	FILE *pipe_fd;
	char line[100];
	int i,len,found=0;
	int toknum;
	char* token[MAX_TOKENS];
	struct stat file_stat;
	
	strcpy(state,"unconfigured");
	
	if (stat(router_dir, &file_stat) < 0)
		return;
	
	pipe_fd = popen("cat /proc/net/wanrouter/status", "r");
	if (pipe_fd == NULL)
		return;

	i=0;
	while (fgets(line,sizeof(line)-1,pipe_fd)){
		if (strstr(line,device_name) != NULL){
			found = 1;
			break;
		}
	}

	pclose(pipe_fd);

	if (!found){
		return;
	}

	len = strlen(line) + 1;
	toknum = tokenize(line, token, "|", " \t\n", "|");

	if (toknum <= CONN_OFF)
		return;

	if (strcmp(token[DEV_OFF],device_name))
		return;

	strcpy(state,token[CONN_OFF]);
	
	return;
}

/*============================================================================
 * check_and_set_lock_file
 *
 * 	Initialize the lock file, so we don't have multiple
 * 	daemons running.  
 * 	If a lock exists exit the daemon with an error message.
 */

int check_and_set_lock_file(void)
{
	char wanpipe_dir[] = "/usr/local/wanrouter";
	char lock_file[] = "/usr/local/wanrouter/wprestart_lock";
	struct stat file_stat;
	
	if (stat(wanpipe_dir, &file_stat) < 0){
		printf("wpbackupd: Error, WANPIPE utilites not installed\n");
		printf("wpbackupd: Directory not found: %s\n",wanpipe_dir);
		return 1;
	}
	
	if (stat(lock_file, &file_stat) < 0){
		system ("exec touch /usr/local/wanrouter/wprestart_lock");
	}else{
		printf("\nwpbackupd: Error, Wanpipe backup daemon already running!\n");
 		printf("                 The /usr/local/wanrouter/wprestart_lock exists.\n\n");
		return 1;
	}

	return 0;
}

/*============================================================================
 * cleanup
 *
 * 	Remove the lock file and exit
 */


void cleanup (void)
{
	unlink("/usr/local/wanrouter/wprestart_lock");
	syslog(LOG_INFO, "Exit.\n");
}

/*============================================================================
 * main
 *
 * 	o Monitor the Primary PPP high speed connection.
 * 	o If the connecting goes down, shutdown PPP protocol and
 * 	  restart the SREAMING HDLC protocol on the Primary port
 * 	o and ASYNC TTY protocol on the secondary port.
 * 	o The primary port is used to monitor the sycn line for
 * 	  signs of life.
 * 	o Start the PPPD using the WANPIPE TTY driver.
 * 	o If the Primary link comes back to life, shutdown PPPD and
 * 	  restart the Primary link.
 */


int main (int argc, char **argv)
{
	int update_interval=5;
	int restart_interval=60;
	int errno;
	char cmd[100];
	
	//General Error Message
	const char *help =
	      "\n  wprestartd v1.0 - (c)2000 Nenad Corbic <ncorbic@sangoma.com>\n\n"
                "  This program is distributed under the terms of GPL.\n\n"
		"  Description:\n\n"
		"    WANPIPE Restart Daemon utility is used to monitor the Primary HIGH\n"
		"    speed link.  If the Primary link goes down, link will be restarted. \n"
		
                "  Usage:\n"
                "  wpbackupd <wanpipe device> <update frq> <restart wait>\n\n\n"
                "            <wanpipe device>  = wanpipe device to mointor\n"
		"                                (ex wanpipe1).\n"
		"            <update frq>  = 1-60  seconds \n"
		"            <restart wait> = 1-1000 seconds (wait time once the link is restared) \n\n"
		"  ex: wpbakcupd wanpipe1 wp1_ppp 5 60\n\n"
		"      Monitor device wanpipe1 every 5 seconds.\n"
		"      If the connection is lost, restart wanpipe1. \n\n";

	openlog(argv[0], LOG_PID | LOG_NDELAY, LOG_DAEMON);

	signal(SIGTERM,sig_handler);
	signal(SIGINT,sig_handler);

	
	if (argc < 4){
		printf("\nError in usage: Arguments missing\n");
		printf("%s",help);
		exit(1);
	}

	
	
	update_interval = atoi(argv[2]);
	if (update_interval < 1 || update_interval > 60){
		printf("Error: Update interval out of range!\n");	
		printf("%s",help);
		exit(1);
	}

	restart_interval = atoi(argv[3]);
	if (restart_interval < 1 || restart_interval > 1000){
		printf("Error: Restart ainterval out of range!\n");	
		printf("%s",help);
		exit(1);
	}

	
	/* Obtain the primary device name and interface name */
	strncpy(device,argv[1],sizeof(device));
	
	if (check_and_set_lock_file()){
		exit(1);
	}
	
	syslog(LOG_INFO, 
		"Starting Wanpipe Restart Daemon!\n");
	syslog(LOG_INFO,
		"Device: %s \n",
		  argv[1]);

	syslog(LOG_INFO,
		"Update Time Interval: %i sec\n",update_interval);

	syslog(LOG_INFO,
		"Restart Wait Interval: %i sec\n",restart_interval);

	
	for (;;){
	
		get_wanpipe_state(device, state);

		if (!strcmp(state,"unconfigured")){
			syslog(LOG_WARNING,
			 "Device %s is unconfigured, exiting debugger!\n",
			   device);
			goto wpkbd_exit;
			
		}else if (strcmp(state, "connected")){

			syslog(LOG_WARNING, "Primary link is Down!");
			syslog(LOG_WARNING, "Restarting...");
			
			sprintf(cmd,"exec /usr/sbin/wanrouter stop %s > /dev/null",device);
			system(cmd);
			sprintf(cmd,"exec /usr/sbin/wanrouter start %s > /dev/null",device);
			system(cmd);
			
			syslog(LOG_INFO, "Primary link re-started!\n");
			delay_time(restart_interval,0);
		}
		delay_time(update_interval,0);
	}

wpkbd_exit:
	
	cleanup();
	return 0;
}
























#else  /* LINUX_2_4 LINUX_2_1 */ 

void main()
{
	printf("\nERROR: This program is not supported on 2.0.X kernels\n");
	exit(0);
}

#endif





