# ============================================================================
# Makefile	Make script for building Linux WAN router utilities.
# ----------------------------------------------------------------------------
# Copyright (c) 1995-2000 Sangoma Technologies Inc.  All Rights Reserved.
# ============================================================================

####### MACROS ###############################################################

# Build options.
OS_TYPE	= _LINUX_
DEBUG	= 2

# Project file paths.
OUTDIR	= bin
SYSINC	= /usr/src/linux/include
WAN_VIRTUAL=
LXDIR=../config/lxdialog
ENABLE_GUI=YES


HOSTCFLAGS=-I$(SYSINC)
ifeq (/usr/include/ncurses/ncurses.h, $(wildcard /usr/include/ncurses/ncurses.h))
        HOSTCFLAGS += -I/usr/include/ncurses
else
ifeq (/usr/include/ncurses/curses.h, $(wildcard /usr/include/ncurses/curses.h))
        HOSTCFLAGS += -I/usr/include/ncurses
else
endif
endif

# Tools options.
CFLAGS  = -Wall -O2 -D$(OS_TYPE) -D_DEBUG_=$(DEBUG) $(HOSTCFLAGS)

ifeq "${ENABLE_GUI}" ""
LX_OBJ =
else
LX_OBJ = wangui.c $(LXDIR)/checklist.o $(LXDIR)/menubox.o $(LXDIR)/inputbox.o $(LXDIR)/util.o -lncurses
CFLAGS += -DWANPIPEMON_GUI -DLOCALE  -DCURSES_LOC="<ncurses.h>"
endif

####### RULES ################################################################

all: $(OUTDIR)/wanconfig $(OUTDIR)/sdladump $(OUTDIR)/wanpipemon $(OUTDIR)/bpipemon $(OUTDIR)/wpkbdmon
	@echo "Ok."

$(OUTDIR)/wanconfig:	wanconfig.c 
	$(CC) $(CFLAGS) -o $@ $<

$(OUTDIR)/sdladump:	sdladump.c 
	$(CC) $(CFLAGS) -o $@ $<

$(OUTDIR)/wanpipemon:   wanpipemon.c fpipemon.c cpipemon.c ppipemon.c xpipemon.c dslpipemon.c ft1_lib.c xml_lib.c ../config/ft1/source/unixio.c $(LX_OBJ)
	$(CC) -D_GNUC_ $(CFLAGS) -o $@ $^

$(OUTDIR)/bpipemon:	bpipemon.c
	$(CC) -D_GNUC_ $(CFLAGS) -o $@ $<	
	
#$(OUTDIR)/mppipemon:	mppipemon.c
#	$(CC) -D_GNUC_ $(CFLAGS) -o $@ $<

$(OUTDIR)/wpkbdmon:	wpkbdmon.c
	$(CC) -D_GNUC_ $(CFLAGS) -o $@ $<


install:
	@echo "Installing Wanpipe utilites in $(WAN_VIRTUAL)/usr/sbin"
	install -D bin/wanconfig  $(WAN_VIRTUAL)/usr/sbin/wanconfig
	install -D bin/sdladump	  $(WAN_VIRTUAL)/usr/sbin/sdladump
	install -D bin/wanpipemon	  $(WAN_VIRTUAL)/usr/sbin/wanpipemon
	install -D bin/wpkbdmon   $(WAN_VIRTUAL)/usr/sbin/wpkbdmon
	install -D bin/bpipemon   $(WAN_VIRTUAL)/usr/sbin/bpipemon

uninstall:
	@echo "Un-installing Wanpipe utilites from $(WAN_VIRTUAL)/usr/sbin"
	rm -f $(WAN_VIRTUAL)/usr/sbin/wanconfig
	rm -f $(WAN_VIRTUAL)/usr/sbin/sdladump
	rm -f $(WAN_VIRTUAL)/usr/sbin/mppipemon
	rm -f $(WAN_VIRTUAL)/usr/sbin/wpkbdmon
	rm -f $(WAN_VIRTUAL)/usr/sbin/bpipemon
	rm -f $(WAN_VIRTUAL)/usr/sbin/wanpipemon

clean:

	rm -f bin/*
